=== modified file 'drizzled/Makefile.am'
--- drizzled/Makefile.am	2009-12-05 23:29:59 +0000
+++ drizzled/Makefile.am	2009-12-08 00:22:46 +0000
@@ -257,8 +257,13 @@
 		optimizer/key_field.h \
 		optimizer/key_use.h \
 		optimizer/position.h \
+		optimizer/quick_index_merge_select.h \
+		optimizer/quick_range.h \
+		optimizer/quick_range_select.h \
 		optimizer/range.h \
+		optimizer/range_param.h \
 		optimizer/sargable_param.h \
+		optimizer/sel_arg.h \
 		optimizer/sum.h \
 		order.h \
 		plugin.h \
@@ -606,7 +611,11 @@
 		name_resolution_context_state.cc \
 		natural_join_column.cc \
 		optimizer/key_field.cc \
+		optimizer/quick_index_merge_select.cc \
+		optimizer/quick_range.cc \
+		optimizer/quick_range_select.cc \
 		optimizer/range.cc \
+		optimizer/sel_arg.cc \
 		optimizer/sum.cc \
 		plugin/authentication.cc \
 		plugin/client.cc \

=== modified file 'drizzled/db.cc'
--- drizzled/db.cc	2009-12-03 23:50:07 +0000
+++ drizzled/db.cc	2009-12-08 21:44:46 +0000
@@ -49,9 +49,6 @@
                                  const char *db, const char *path,
                                  TableList **dropped_tables);
 
-static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name);
-            
-
 /**
   Return default database collation.
 
@@ -92,19 +89,12 @@
 }
 
 /* path is path to database, not schema file */
-static int write_schema_file(Session *session,
-			     const char *path, const char *name,
-			     HA_CREATE_INFO *create)
+static int write_schema_file(const DatabasePathName &path, const message::Schema &db)
 {
-  message::Schema db;
   char schema_file_tmp[FN_REFLEN];
-  string schema_file(path);
-
-  assert(path);
-  assert(name);
-  assert(create);
-
-  snprintf(schema_file_tmp, FN_REFLEN, "%s%c%s.tmpXXXXXX", path, FN_LIBCHAR, MY_DB_OPT_FILE);
+  string schema_file(path.to_string());
+
+  snprintf(schema_file_tmp, FN_REFLEN, "%s%c%s.tmpXXXXXX", path.to_string().c_str(), FN_LIBCHAR, MY_DB_OPT_FILE);
 
   schema_file.append(1, FN_LIBCHAR);
   schema_file.append(MY_DB_OPT_FILE);
@@ -114,11 +104,6 @@
   if (fd==-1)
     return errno;
 
-  if (!create->default_table_charset)
-    create->default_table_charset= session->variables.collation_server;
-
-  db.set_name(name);
-  db.set_collation(create->default_table_charset->name);
 
   if (!db.SerializeToFileDescriptor(fd))
   {
@@ -186,22 +171,24 @@
 
 */
 
-bool mysql_create_db(Session *session, const char *db, HA_CREATE_INFO *create_info, bool is_if_not_exists)
+bool mysql_create_db(Session *session, const NormalisedDatabaseName &database_name, message::Schema *schema_message, bool is_if_not_exists)
 {
   ReplicationServices &replication_services= ReplicationServices::singleton();
-  char	 path[FN_REFLEN+16];
   long result= 1;
   int error_erno;
   bool error= false;
-  uint32_t path_len;
+  DatabasePathName database_path(database_name);
 
   /* do not create 'information_schema' db */
-  if (!my_strcasecmp(system_charset_info, db, INFORMATION_SCHEMA_NAME.c_str()))
+  if (!my_strcasecmp(system_charset_info, database_name.to_string().c_str(), INFORMATION_SCHEMA_NAME.c_str()))
   {
-    my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
+    my_error(ER_DB_CREATE_EXISTS, MYF(0), database_name.to_string().c_str());
     return(-1);
   }
 
+  assert(database_name.isValid());
+  schema_message->set_name(database_name.to_string());
+
   /*
     Do not create database if another thread is holding read lock.
     Wait for global read lock before acquiring LOCK_create_db.
@@ -222,36 +209,33 @@
 
   pthread_mutex_lock(&LOCK_create_db);
 
-  /* Check directory */
-  path_len= build_table_filename(path, sizeof(path), db, "", false);
-  path[path_len-1]= 0;                    // Remove last '/' from path
-
-  if (mkdir(path,0777) == -1)
+  if (mkdir(database_path.to_string().c_str(),0777) == -1)
   {
     if (errno == EEXIST)
     {
       if (! is_if_not_exists)
       {
-	my_error(ER_DB_CREATE_EXISTS, MYF(0), db);
+	my_error(ER_DB_CREATE_EXISTS, MYF(0), database_name.to_string().c_str());
 	error= true;
 	goto exit;
       }
       push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
-			  ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS), db);
+			  ER_DB_CREATE_EXISTS, ER(ER_DB_CREATE_EXISTS),
+                          database_name.to_string().c_str());
       session->my_ok();
       error= false;
       goto exit;
     }
 
-    my_error(ER_CANT_CREATE_DB, MYF(0), db, my_errno);
+    my_error(ER_CANT_CREATE_DB, MYF(0), database_name.to_string().c_str(), my_errno);
     error= true;
     goto exit;
   }
 
-  error_erno= write_schema_file(session, path, db, create_info);
+  error_erno= write_schema_file(database_path, *schema_message);
   if (error_erno && error_erno != EEXIST)
   {
-    if (rmdir(path) >= 0)
+    if (rmdir(database_path.to_string().c_str()) >= 0)
     {
       error= true;
       goto exit;
@@ -273,13 +257,12 @@
 
 /* db-name is already validated when we come here */
 
-bool mysql_alter_db(Session *session, const char *db, HA_CREATE_INFO *create_info)
+bool mysql_alter_db(Session *session, const NormalisedDatabaseName &database_name, message::Schema *schema_message)
 {
   ReplicationServices &replication_services= ReplicationServices::singleton();
   long result=1;
   int error= 0;
-  char	 path[FN_REFLEN+16];
-  uint32_t path_len;
+  DatabasePathName database_path(database_name);
 
   /*
     Do not alter database if another thread is holding read lock.
@@ -296,13 +279,14 @@
   if ((error=wait_if_global_read_lock(session,0,1)))
     goto exit;
 
+  assert(schema_message);
+  assert(database_name.isValid());
+
+  schema_message->set_name(database_name.to_string());
+
   pthread_mutex_lock(&LOCK_create_db);
 
-  /* Change options if current database is being altered. */
-  path_len= build_table_filename(path, sizeof(path), db, "", false);
-  path[path_len-1]= 0;                    // Remove last '/' from path
-
-  error= write_schema_file(session, path, db, create_info);
+  error= write_schema_file(database_path, *schema_message);
   if (error && error != EEXIST)
   {
     /* TODO: find some witty way of getting back an error message */
@@ -337,7 +321,7 @@
     ERROR Error
 */
 
-bool mysql_rm_db(Session *session,char *db,bool if_exists)
+bool mysql_rm_db(Session *session, const NormalisedDatabaseName &database_name, bool if_exists)
 {
   long deleted=0;
   int error= false;
@@ -346,7 +330,7 @@
   uint32_t length;
   TableList *dropped_tables= NULL;
 
-  if (db && (strcmp(db, "information_schema") == 0))
+  if (database_name.to_string().compare(INFORMATION_SCHEMA_NAME) == 0)
   {
     my_error(ER_DBACCESS_DENIED_ERROR, MYF(0), "", "", INFORMATION_SCHEMA_NAME.c_str());
     return true;
@@ -372,7 +356,8 @@
 
   pthread_mutex_lock(&LOCK_create_db);
 
-  length= build_table_filename(path, sizeof(path), db, "", false);
+  length= build_table_filename(path, sizeof(path),
+                               database_name.to_string().c_str(), "", false);
   strcpy(path+length, MY_DB_OPT_FILE);         // Append db option file name
   unlink(path);
   path[length]= '\0';				// Remove file name
@@ -383,22 +368,23 @@
     if (!if_exists)
     {
       error= -1;
-      my_error(ER_DB_DROP_EXISTS, MYF(0), db);
+      my_error(ER_DB_DROP_EXISTS, MYF(0), database_name.to_string().c_str());
       goto exit;
     }
     else
       push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
-			  ER_DB_DROP_EXISTS, ER(ER_DB_DROP_EXISTS), db);
+			  ER_DB_DROP_EXISTS, ER(ER_DB_DROP_EXISTS),
+                          database_name.to_string().c_str());
   }
   else
   {
     pthread_mutex_lock(&LOCK_open); /* After deleting database, remove all cache entries related to schema */
-    remove_db_from_cache(db);
+    remove_db_from_cache(database_name.to_string().c_str());
     pthread_mutex_unlock(&LOCK_open);
 
 
     error= -1;
-    if ((deleted= mysql_rm_known_files(session, dirp, db, path, &dropped_tables)) >= 0)
+    if ((deleted= mysql_rm_known_files(session, dirp, database_name.to_string().c_str(), path, &dropped_tables)) >= 0)
     {
       plugin::StorageEngine::dropDatabase(path);
       error = 0;
@@ -408,17 +394,12 @@
   {
     const char *query;
     uint32_t query_length;
-    if (!session->query)
-    {
-      /* The client used the old obsolete mysql_drop_db() call */
-      query= path;
-      query_length= sprintf(path, "drop database `%s`", db);
-    }
-    else
-    {
-      query= session->query;
-      query_length= session->query_length;
-    }
+
+    assert(session->query);
+
+    query= session->query;
+    query_length= session->query_length;
+
     ReplicationServices &replication_services= ReplicationServices::singleton();
     replication_services.rawStatement(session, session->getQueryString(), session->getQueryLength());
     session->clear_error();
@@ -436,7 +417,7 @@
       goto exit; /* not much else we can do */
     query_pos= query_data_start= strcpy(query,"drop table ")+11;
     query_end= query + MAX_DROP_TABLE_Q_LEN;
-    db_len= strlen(db);
+    db_len= database_name.to_string().length();
 
     ReplicationServices &replication_services= ReplicationServices::singleton();
     for (tbl= dropped_tables; tbl; tbl= tbl->next_local)
@@ -472,8 +453,8 @@
     SELECT DATABASE() in the future). For this we free() session->db and set
     it to 0.
   */
-  if (! session->db.empty() && ! strcmp(session->db.c_str(), db))
-    mysql_change_db_impl(session, NULL);
+  if (! session->db.empty() && session->db.compare(database_name.to_string()) == 0)
+    session->clear_db();
   pthread_mutex_unlock(&LOCK_create_db);
   start_waiting_global_read_lock(session);
 exit2:
@@ -729,66 +710,6 @@
 }
 
 /**
-  @brief Internal implementation: switch current database to a valid one.
-
-  @param session            Thread context.
-  @param new_db_name    Name of the database to switch to. The function will
-                        take ownership of the name (the caller must not free
-                        the allocated memory). If the name is NULL, we're
-                        going to switch to NULL db.
-  @param new_db_charset Character set of the new database.
-*/
-
-static void mysql_change_db_impl(Session *session, LEX_STRING *new_db_name)
-{
-  /* 1. Change current database in Session. */
-
-  if (new_db_name == NULL)
-  {
-    /*
-      Session::set_db() does all the job -- it frees previous database name and
-      sets the new one.
-    */
-
-    session->set_db(NULL, 0);
-  }
-  else
-  {
-    /*
-      Here we already have a copy of database name to be used in Session. So,
-      we just call Session::reset_db(). Since Session::reset_db() does not releases
-      the previous database name, we should do it explicitly.
-    */
-
-    session->set_db(new_db_name->str, new_db_name->length);
-  }
-}
-
-/**
-  Return true if db1_name is equal to db2_name, false otherwise.
-
-  The function allows to compare database names according to the MySQL
-  rules. The database names db1 and db2 are equal if:
-     - db1 is NULL and db2 is NULL;
-     or
-     - db1 is not-NULL, db2 is not-NULL, db1 is equal (ignoring case) to
-       db2 in system character set (UTF8).
-*/
-
-static inline bool
-cmp_db_names(const char *db1_name,
-             const char *db2_name)
-{
-  return
-         /* db1 is NULL and db2 is NULL */
-         (!db1_name && !db2_name) ||
-
-         /* db1 is not-NULL, db2 is not-NULL, db1 == db2. */
-         (db1_name && db2_name && my_strcasecmp(system_charset_info, db1_name, db2_name) == 0);
-}
-
-
-/**
   @brief Change the current database and its attributes unconditionally.
 
   @param session          thread handle
@@ -850,42 +771,21 @@
     @retval true  Error
 */
 
-bool mysql_change_db(Session *session, const LEX_STRING *new_db_name, bool force_switch)
+bool mysql_change_db(Session *session, const NormalisedDatabaseName &normalised_database_name, bool force_switch)
 {
-  LEX_STRING new_db_file_name;
   const CHARSET_INFO *db_default_cl;
 
-  assert(new_db_name);
-  assert(new_db_name->length);
-
-  if (my_strcasecmp(system_charset_info, new_db_name->str,
+  if (my_strcasecmp(system_charset_info, normalised_database_name.to_string().c_str(),
                     INFORMATION_SCHEMA_NAME.c_str()) == 0)
   {
-    /* Switch the current database to INFORMATION_SCHEMA. */
-    /* const_cast<> is safe here: mysql_change_db_impl does a copy */
-    LEX_STRING is_name= { const_cast<char *>(INFORMATION_SCHEMA_NAME.c_str()),
-                          INFORMATION_SCHEMA_NAME.length() };
-    mysql_change_db_impl(session, &is_name);
+    NonNormalisedDatabaseName non_normalised_i_s(INFORMATION_SCHEMA_NAME);
+    NormalisedDatabaseName is_name(non_normalised_i_s);
 
+    session->set_db(is_name);
     return false;
   }
 
   /*
-    Now we need to make a copy because check_db_name requires a
-    non-constant argument. Actually, it takes database file name.
-
-    TODO: fix check_db_name().
-  */
-
-  new_db_file_name.length= new_db_name->length;
-  new_db_file_name.str= (char *)malloc(new_db_name->length + 1);
-  if (new_db_file_name.str == NULL)
-    return true;                             /* the error is set */
-  memcpy(new_db_file_name.str, new_db_name->str, new_db_name->length);
-  new_db_file_name.str[new_db_name->length]= 0;
-
-
-  /*
     NOTE: if check_db_name() fails, we should throw an error in any case,
     even if we are called from sp_head::execute().
 
@@ -893,19 +793,20 @@
     from sp_head::execute(). But let's switch the current database to NULL
     in this case to be sure.
   */
-
-  if (check_db_name(&new_db_file_name))
+  if (! normalised_database_name.isValid())
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), new_db_file_name.str);
-    free(new_db_file_name.str);
+    my_error(ER_WRONG_DB_NAME, MYF(0),
+             normalised_database_name.to_string().c_str());
 
     if (force_switch)
-      mysql_change_db_impl(session, NULL);
+      session->clear_db();
 
     return true;
   }
 
-  if (check_db_dir_existence(new_db_file_name.str))
+  DatabasePathName database_path(normalised_database_name);
+
+  if (! database_path.exists())
   {
     if (force_switch)
     {
@@ -913,13 +814,11 @@
 
       push_warning_printf(session, DRIZZLE_ERROR::WARN_LEVEL_NOTE,
                           ER_BAD_DB_ERROR, ER(ER_BAD_DB_ERROR),
-                          new_db_file_name.str);
-
-      free(new_db_file_name.str);
+                          normalised_database_name.to_string().c_str());
 
       /* Change db to NULL. */
 
-      mysql_change_db_impl(session, NULL);
+      session->clear_db();
 
       /* The operation succeed. */
 
@@ -929,8 +828,8 @@
     {
       /* Report an error and free new_db_file_name. */
 
-      my_error(ER_BAD_DB_ERROR, MYF(0), new_db_file_name.str);
-      free(new_db_file_name.str);
+      my_error(ER_BAD_DB_ERROR, MYF(0),
+               normalised_database_name.to_string().c_str());
 
       /* The operation failed. */
 
@@ -938,38 +837,62 @@
     }
   }
 
-  db_default_cl= get_default_db_collation(new_db_file_name.str);
+  db_default_cl= get_default_db_collation(normalised_database_name.to_string().c_str());
 
-  mysql_change_db_impl(session, &new_db_file_name);
-  free(new_db_file_name.str);
+  session->set_db(normalised_database_name);
 
   return false;
 }
 
-/*
-  Check if there is directory for the database name.
-
-  SYNOPSIS
-    check_db_dir_existence()
-    db_name   database name
-
-  RETURN VALUES
-    false   There is directory for the specified database name.
-    true    The directory does not exist.
-*/
-
-bool check_db_dir_existence(const char *db_name)
+NormalisedDatabaseName::NormalisedDatabaseName(const NonNormalisedDatabaseName &dbname)
+{
+  const std::string &non_norm_string= dbname.to_string();
+  database_name= (char*)malloc(non_norm_string.size()+1);
+
+  assert(database_name); /* FIXME: should throw exception */
+
+  strncpy(database_name, non_norm_string.c_str(), non_norm_string.size()+1);
+
+  my_casedn_str(files_charset_info, database_name);
+}
+
+NormalisedDatabaseName::~NormalisedDatabaseName()
+{
+  free(database_name);
+}
+
+bool NormalisedDatabaseName::isValid() const
+{
+  LEX_STRING db_lexstring;
+
+  db_lexstring.str= database_name;
+  db_lexstring.length= strlen(database_name);
+
+  if (db_lexstring.length == 0
+      || db_lexstring.length > NAME_LEN
+      || database_name[db_lexstring.length - 1] == ' ')
+    return false;
+
+  return (! check_identifier_name(&db_lexstring));
+}
+
+DatabasePathName::DatabasePathName(const NormalisedDatabaseName &database_name)
 {
   char db_dir_path[FN_REFLEN];
   uint32_t db_dir_path_len;
 
   db_dir_path_len= build_table_filename(db_dir_path, sizeof(db_dir_path),
-                                        db_name, "", false);
+                                        database_name.to_string().c_str(),
+                                        "", false);
 
   if (db_dir_path_len && db_dir_path[db_dir_path_len - 1] == FN_LIBCHAR)
     db_dir_path[db_dir_path_len - 1]= 0;
 
-  /* Check access. */
+  database_path.assign(db_dir_path);
+}
 
-  return my_access(db_dir_path, F_OK);
+bool DatabasePathName::exists() const
+{
+  /* TODO: handle EIO and other fun errors */
+  return access(database_path.c_str(), F_OK) == 0;
 }

=== modified file 'drizzled/db.h'
--- drizzled/db.h	2009-11-19 02:33:17 +0000
+++ drizzled/db.h	2009-12-08 01:43:27 +0000
@@ -23,19 +23,84 @@
 
 namespace drizzled { namespace message { class Schema; } }
 
-bool mysql_create_db(Session *session, const char *db, HA_CREATE_INFO *create_info,
-                     bool is_if_not_exists);
-bool mysql_alter_db(Session *session, const char *db, HA_CREATE_INFO *create);
-bool mysql_rm_db(Session *session,char *db, bool if_exists);
-bool mysql_change_db(Session *session, const LEX_STRING *new_db_name,
-                     bool force_switch);
+class NormalisedDatabaseName;
+
+bool mysql_create_db(Session *session, const NormalisedDatabaseName &database_name, drizzled::message::Schema *schema_message, bool is_if_not_exists);
+bool mysql_alter_db(Session *session, const NormalisedDatabaseName &database_name, drizzled::message::Schema *schema_message);
+bool mysql_rm_db(Session *session, const NormalisedDatabaseName &database_name, bool if_exists);
+bool mysql_change_db(Session *session, const NormalisedDatabaseName &new_db_name, bool force_switch);
 
 bool check_db_dir_existence(const char *db_name);
 int get_database_metadata(const char *dbname, drizzled::message::Schema *db);
 
 const CHARSET_INFO *get_default_db_collation(const char *db_name);
 
-extern int creating_database; // How many database locks are made
-extern int creating_table;    // How many mysql_create_table() are running
+class NonNormalisedDatabaseName
+{
+private:
+  std::string database_name;
+
+  /* Copying a NonNormalisedDatabaseName is always wrong, it's
+     immutable and should be passed by reference */
+  NonNormalisedDatabaseName(const NonNormalisedDatabaseName&);
+
+  NonNormalisedDatabaseName operator=(const NonNormalisedDatabaseName&);
+
+public:
+  explicit NonNormalisedDatabaseName(const std::string db) :
+    database_name(db)
+  {
+  }
+
+  const std::string &to_string(void) const
+  {
+    return database_name;
+  }
+};
+
+class NormalisedDatabaseName
+{
+private:
+  char* database_name;
+
+  /* Copying a NormalisedDatabaseName is always wrong, it's
+     immutable and should be passed by reference */
+  NormalisedDatabaseName(const NormalisedDatabaseName&);
+  NormalisedDatabaseName operator=(const NormalisedDatabaseName&);
+
+public:
+  explicit NormalisedDatabaseName(const NonNormalisedDatabaseName &dbname);
+
+  ~NormalisedDatabaseName();
+
+  const std::string to_string() const
+  {
+    std::string tmp(database_name);
+    return tmp;
+  }
+
+  bool isValid() const;
+};
+
+class DatabasePathName
+{
+private:
+  std::string database_path;
+
+  /* Copying a DatabasePathName is always wrong, it's
+     immutable and should be passed by reference. */
+  DatabasePathName(const DatabasePathName&);
+  DatabasePathName operator=(const DatabasePathName&);
+
+public:
+  explicit DatabasePathName(const NormalisedDatabaseName &database_name);
+
+  const std::string to_string() const
+  {
+    return database_path;
+  }
+
+  bool exists() const;
+};
 
 #endif /* DRIZZLED_DB_H */

=== modified file 'drizzled/definitions.h'
--- drizzled/definitions.h	2009-12-01 00:59:19 +0000
+++ drizzled/definitions.h	2009-12-07 19:18:46 +0000
@@ -451,13 +451,6 @@
 */
 #define HA_MRR_USE_DEFAULT_IMPL 64
 
-/*
-  Used only as parameter to multi_range_read_info():
-  Flag set <=> the caller guarantees that the bounds of the scanned ranges
-  will not have NULL values.
-*/
-#define HA_MRR_NO_NULL_ENDPOINTS 128
-
 typedef int myf;
 #define MYF(v)		(myf) (v)
 

=== modified file 'drizzled/drizzled.cc'
--- drizzled/drizzled.cc	2009-12-02 16:34:51 +0000
+++ drizzled/drizzled.cc	2009-12-08 21:45:43 +0000
@@ -38,7 +38,6 @@
 #include <drizzled/sql_parse.h>
 #include <drizzled/item/cmpfunc.h>
 #include <drizzled/session.h>
-#include <drizzled/db.h>
 #include <drizzled/item/create.h>
 #include <drizzled/unireg.h>
 #include "drizzled/temporal_format.h" /* For init_temporal_formats() */
@@ -1578,7 +1577,10 @@
   select_thread_in_use=1;
 
   if (chdir(drizzle_real_data_home) && !opt_help)
+  {
+    errmsg_printf(ERRMSG_LVL_ERROR, _("Data directory %s does not exist\n"), drizzle_real_data_home);
     unireg_abort(1);
+  }
   drizzle_data_home= drizzle_data_home_buff;
   drizzle_data_home[0]=FN_CURLIB;		// all paths are relative from here
   drizzle_data_home[1]=0;

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2009-12-01 21:58:19 +0000
+++ drizzled/enum.h	2009-12-08 18:42:12 +0000
@@ -73,55 +73,55 @@
 
 enum enum_parsing_place
 {
-  NO_MATTER, 
-  IN_HAVING, 
-  SELECT_LIST, 
-  IN_WHERE, 
-  IN_ON 
+  NO_MATTER,
+  IN_HAVING,
+  SELECT_LIST,
+  IN_WHERE,
+  IN_ON
 };
 
-enum enum_mysql_completiontype 
+enum enum_mysql_completiontype
 {
-  ROLLBACK_RELEASE= -2, 
-  ROLLBACK= 1, 
-  ROLLBACK_AND_CHAIN= 7, 
-  COMMIT_RELEASE= -1, 
-  COMMIT= 0, 
+  ROLLBACK_RELEASE= -2,
+  ROLLBACK= 1,
+  ROLLBACK_AND_CHAIN= 7,
+  COMMIT_RELEASE= -1,
+  COMMIT= 0,
   COMMIT_AND_CHAIN= 6
 };
 
 enum enum_check_fields
 {
-  CHECK_FIELD_IGNORE, 
-  CHECK_FIELD_WARN, 
+  CHECK_FIELD_IGNORE,
+  CHECK_FIELD_WARN,
   CHECK_FIELD_ERROR_FOR_NULL
 };
 
 enum enum_var_type
 {
-  OPT_DEFAULT= 0, 
-  OPT_SESSION, 
+  OPT_DEFAULT= 0,
+  OPT_SESSION,
   OPT_GLOBAL
 };
 
-enum row_type 
-{ 
-  ROW_TYPE_NOT_USED=-1, 
-  ROW_TYPE_DEFAULT, 
+enum row_type
+{
+  ROW_TYPE_NOT_USED=-1,
+  ROW_TYPE_DEFAULT,
   ROW_TYPE_FIXED,
-  ROW_TYPE_DYNAMIC, 
+  ROW_TYPE_DYNAMIC,
   ROW_TYPE_COMPRESSED,
-  ROW_TYPE_REDUNDANT, 
-  ROW_TYPE_COMPACT, 
-  ROW_TYPE_PAGE 
+  ROW_TYPE_REDUNDANT,
+  ROW_TYPE_COMPACT,
+  ROW_TYPE_PAGE
 };
 
-enum column_format_type 
+enum column_format_type
 {
   COLUMN_FORMAT_TYPE_NOT_USED= -1,
   COLUMN_FORMAT_TYPE_DEFAULT= 0,
   COLUMN_FORMAT_TYPE_FIXED= 1,
-  COLUMN_FORMAT_TYPE_DYNAMIC= 2 
+  COLUMN_FORMAT_TYPE_DYNAMIC= 2
 };
 
 
@@ -181,32 +181,32 @@
   TABLE_CATEGORY_INFORMATION
 };
 
-enum enum_enable_or_disable 
+enum enum_enable_or_disable
 {
-  LEAVE_AS_IS, 
-  ENABLE, 
-  DISABLE 
+  LEAVE_AS_IS,
+  ENABLE,
+  DISABLE
 };
 
 
 enum enum_mark_columns
-{ 
-  MARK_COLUMNS_NONE, 
-  MARK_COLUMNS_READ, 
+{
+  MARK_COLUMNS_NONE,
+  MARK_COLUMNS_READ,
   MARK_COLUMNS_WRITE
 };
 
-enum enum_filetype 
-{ 
-  FILETYPE_CSV, 
-  FILETYPE_XML 
+enum enum_filetype
+{
+  FILETYPE_CSV,
+  FILETYPE_XML
 };
 
-enum find_item_error_report_type 
+enum find_item_error_report_type
 {
-  REPORT_ALL_ERRORS, 
+  REPORT_ALL_ERRORS,
   REPORT_EXCEPT_NOT_FOUND,
-  IGNORE_ERRORS, 
+  IGNORE_ERRORS,
   REPORT_EXCEPT_NON_UNIQUE,
   IGNORE_EXCEPT_NON_UNIQUE
 };
@@ -231,35 +231,35 @@
 */
 enum timestamp_auto_set_type
 {
-  TIMESTAMP_NO_AUTO_SET= 0, 
+  TIMESTAMP_NO_AUTO_SET= 0,
   TIMESTAMP_AUTO_SET_ON_INSERT= 1,
-  TIMESTAMP_AUTO_SET_ON_UPDATE= 2, 
+  TIMESTAMP_AUTO_SET_ON_UPDATE= 2,
   TIMESTAMP_AUTO_SET_ON_BOTH= 3
 };
 
-enum enum_ha_read_modes 
-{ 
-  RFIRST, 
-  RNEXT, 
-  RPREV, 
-  RLAST, 
-  RKEY, 
-  RNEXT_SAME 
+enum enum_ha_read_modes
+{
+  RFIRST,
+  RNEXT,
+  RPREV,
+  RLAST,
+  RKEY,
+  RNEXT_SAME
 };
 
-enum enum_tx_isolation 
+enum enum_tx_isolation
 {
-  ISO_READ_UNCOMMITTED, 
+  ISO_READ_UNCOMMITTED,
   ISO_READ_COMMITTED,
-  ISO_REPEATABLE_READ, 
+  ISO_REPEATABLE_READ,
   ISO_SERIALIZABLE
 };
 
 
-enum SHOW_COMP_OPTION 
-{ 
-  SHOW_OPTION_YES, 
-  SHOW_OPTION_NO, 
+enum SHOW_COMP_OPTION
+{
+  SHOW_OPTION_YES,
+  SHOW_OPTION_NO,
   SHOW_OPTION_DISABLED
 };
 
@@ -274,24 +274,24 @@
 */
 
 enum enum_sql_command {
-  SQLCOM_SELECT, 
-  SQLCOM_CREATE_TABLE, 
-  SQLCOM_CREATE_INDEX, 
+  SQLCOM_SELECT,
+  SQLCOM_CREATE_TABLE,
+  SQLCOM_CREATE_INDEX,
   SQLCOM_ALTER_TABLE,
-  SQLCOM_UPDATE, 
-  SQLCOM_INSERT, 
+  SQLCOM_UPDATE,
+  SQLCOM_INSERT,
   SQLCOM_INSERT_SELECT,
-  SQLCOM_DELETE, 
-  SQLCOM_TRUNCATE, 
-  SQLCOM_DROP_TABLE, 
+  SQLCOM_DELETE,
+  SQLCOM_TRUNCATE,
+  SQLCOM_DROP_TABLE,
   SQLCOM_DROP_INDEX,
-  SQLCOM_SHOW_DATABASES, 
-  SQLCOM_SHOW_TABLES, 
+  SQLCOM_SHOW_DATABASES,
+  SQLCOM_SHOW_TABLES,
   SQLCOM_SHOW_FIELDS,
-  SQLCOM_SHOW_KEYS, 
-  SQLCOM_SHOW_VARIABLES, 
+  SQLCOM_SHOW_KEYS,
+  SQLCOM_SHOW_VARIABLES,
   SQLCOM_SHOW_STATUS,
-  SQLCOM_SHOW_ENGINE_STATUS, 
+  SQLCOM_SHOW_ENGINE_STATUS,
   SQLCOM_SHOW_ENGINE_MUTEX,
   SQLCOM_SHOW_PROCESSLIST,
   SQLCOM_SHOW_CREATE,
@@ -300,20 +300,20 @@
   SQLCOM_LOAD,
   SQLCOM_SET_OPTION,
   SQLCOM_UNLOCK_TABLES,
-  SQLCOM_CHANGE_DB, 
-  SQLCOM_CREATE_DB, 
-  SQLCOM_DROP_DB, 
+  SQLCOM_CHANGE_DB,
+  SQLCOM_CREATE_DB,
+  SQLCOM_DROP_DB,
   SQLCOM_ALTER_DB,
-  SQLCOM_REPLACE, 
+  SQLCOM_REPLACE,
   SQLCOM_REPLACE_SELECT,
   SQLCOM_CHECK,
-  SQLCOM_FLUSH, 
-  SQLCOM_KILL, 
+  SQLCOM_FLUSH,
+  SQLCOM_KILL,
   SQLCOM_ANALYZE,
-  SQLCOM_ROLLBACK, 
+  SQLCOM_ROLLBACK,
   SQLCOM_ROLLBACK_TO_SAVEPOINT,
-  SQLCOM_COMMIT, 
-  SQLCOM_SAVEPOINT, 
+  SQLCOM_COMMIT,
+  SQLCOM_SAVEPOINT,
   SQLCOM_RELEASE_SAVEPOINT,
   SQLCOM_BEGIN,
   SQLCOM_RENAME_TABLE,
@@ -330,11 +330,11 @@
   SQLCOM_END
 };
 
-enum enum_duplicates 
-{ 
-  DUP_ERROR, 
-  DUP_REPLACE, 
-  DUP_UPDATE 
+enum enum_duplicates
+{
+  DUP_ERROR,
+  DUP_REPLACE,
+  DUP_UPDATE
 };
 
 #endif /* DRIZZLED_ENUM_H */

=== modified file 'drizzled/filesort.cc'
--- drizzled/filesort.cc	2009-12-05 05:29:33 +0000
+++ drizzled/filesort.cc	2009-12-07 21:50:09 +0000
@@ -46,7 +46,7 @@
                                      unsigned char *buf);
 
 static ha_rows find_all_keys(SORTPARAM *param,
-                             optimizer::SQL_SELECT *select,
+                             optimizer::SqlSelect *select,
 			     unsigned char * *sort_keys, 
                              IO_CACHE *buffer_file,
 			     IO_CACHE *tempfile,
@@ -107,7 +107,7 @@
 */
 
 ha_rows filesort(Session *session, Table *table, SORT_FIELD *sortorder, uint32_t s_length,
-		 optimizer::SQL_SELECT *select, ha_rows max_rows,
+		 optimizer::SqlSelect *select, ha_rows max_rows,
                  bool sort_positions, ha_rows *examined_rows)
 {
   int error;
@@ -134,7 +134,7 @@
 
   /*
     Don't use table->sort in filesort as it is also used by
-    QUICK_INDEX_MERGE_SELECT. Work with a copy and put it back at the end
+    QuickIndexMergeSelect. Work with a copy and put it back at the end
     when index_merge select has finished with it.
   */
   memcpy(&table_sort, &table->sort, sizeof(filesort_info_st));
@@ -452,7 +452,7 @@
 */
 
 static ha_rows find_all_keys(SORTPARAM *param, 
-                             optimizer::SQL_SELECT *select,
+                             optimizer::SqlSelect *select,
 			     unsigned char **sort_keys,
 			     IO_CACHE *buffpek_pointers,
 			     IO_CACHE *tempfile, IO_CACHE *indexfile)

=== modified file 'drizzled/function/set_user_var.cc'
--- drizzled/function/set_user_var.cc	2009-08-06 07:14:37 +0000
+++ drizzled/function/set_user_var.cc	2009-12-08 04:56:41 +0000
@@ -315,17 +315,6 @@
   str->append(')');
 }
 
-
-void Item_func_set_user_var::print_as_stmt(String *str,
-                                           enum_query_type query_type)
-{
-  str->append(STRING_WITH_LEN("set @"));
-  str->append(name.str, name.length);
-  str->append(STRING_WITH_LEN(":="));
-  args[0]->print(str, query_type);
-  str->append(')');
-}
-
 bool Item_func_set_user_var::send(plugin::Client *client, String *str_arg)
 {
   if (result_field)

=== modified file 'drizzled/function/set_user_var.h'
--- drizzled/function/set_user_var.h	2009-08-06 07:14:37 +0000
+++ drizzled/function/set_user_var.h	2009-12-08 04:56:41 +0000
@@ -66,7 +66,7 @@
   bool fix_fields(Session *session, Item **ref);
   void fix_length_and_dec();
   virtual void print(String *str, enum_query_type query_type);
-  void print_as_stmt(String *str, enum_query_type query_type);
+
   const char *func_name() const { return "set_user_var"; }
   int save_in_field(Field *field, bool no_conversions,
                     bool can_use_result_field);

=== modified file 'drizzled/join.cc'
--- drizzled/join.cc	2009-12-05 20:06:30 +0000
+++ drizzled/join.cc	2009-12-07 19:18:46 +0000
@@ -84,7 +84,7 @@
 static uint32_t determine_search_depth(JOIN* join);
 static bool make_simple_join(JOIN *join,Table *tmp_table);
 static void make_outerjoin_info(JOIN *join);
-static bool make_join_select(JOIN *join, optimizer::SQL_SELECT *select,COND *item);
+static bool make_join_select(JOIN *join, optimizer::SqlSelect *select,COND *item);
 static bool make_join_readinfo(JOIN *join, uint64_t options, uint32_t no_jbuf_after);
 static void update_depend_map(JOIN *join);
 static void update_depend_map(JOIN *join, order_st *order);
@@ -660,11 +660,11 @@
      The FROM clause must contain a single non-constant table.
   */
   if (tables - const_tables == 1 && (group_list || select_distinct) &&
-      !tmp_table_param.sum_func_count &&
-      (!join_tab[const_tables].select ||
-       !join_tab[const_tables].select->quick ||
+      ! tmp_table_param.sum_func_count &&
+      (! join_tab[const_tables].select ||
+       ! join_tab[const_tables].select->quick ||
        join_tab[const_tables].select->quick->get_type() !=
-       optimizer::QUICK_SELECT_I::QS_TYPE_GROUP_MIN_MAX))
+       optimizer::QuickSelectInterface::QS_TYPE_GROUP_MIN_MAX))
   {
     if (group_list && list_contains_unique_index(join_tab[const_tables].table, find_field_in_order_list, (void *) group_list))
     {
@@ -683,7 +683,7 @@
           If GROUP BY is a prefix of order_st BY, then it is safe to leave
           'order' as is.
        */
-      if (!order || test_if_subpart(group_list, order))
+      if (! order || test_if_subpart(group_list, order))
           order= skip_sort_order ? 0 : group_list;
       /*
         If we have an IGNORE INDEX FOR GROUP BY(fields) clause, this must be
@@ -1551,7 +1551,7 @@
       if (sort_table_cond)
       {
         if (!curr_table->select)
-          if (!(curr_table->select= new optimizer::SQL_SELECT))
+          if (!(curr_table->select= new optimizer::SqlSelect))
             return;
         if (!curr_table->select->cond)
           curr_table->select->cond= sort_table_cond;
@@ -2609,7 +2609,7 @@
       join->session->send_kill_message();
       return NESTED_LOOP_KILLED;
     }
-    optimizer::SQL_SELECT *select= join_tab->select;
+    optimizer::SqlSelect *select= join_tab->select;
     if (rc == NESTED_LOOP_OK &&
         (!join_tab->cache.select || !join_tab->cache.select->skip_record()))
     {
@@ -4427,7 +4427,7 @@
 }
 
 static bool make_join_select(JOIN *join,
-                             optimizer::SQL_SELECT *select,
+                             optimizer::SqlSelect *select,
                              COND *cond)
 {
   Session *session= join->session;
@@ -4549,7 +4549,7 @@
       if (tmp || !cond || tab->type == AM_REF || tab->type == AM_REF_OR_NULL ||
           tab->type == AM_EQ_REF)
       {
-        optimizer::SQL_SELECT *sel= tab->select= ((optimizer::SQL_SELECT*)
+        optimizer::SqlSelect *sel= tab->select= ((optimizer::SqlSelect*)
             session->memdup((unsigned char*) select,
               sizeof(*select)));
         if (! sel)
@@ -4687,8 +4687,8 @@
                                          current_map,
                                          current_map, 0)))
             {
-              tab->cache.select= (optimizer::SQL_SELECT*)
-                session->memdup((unsigned char*) sel, sizeof(optimizer::SQL_SELECT));
+              tab->cache.select= (optimizer::SqlSelect*)
+                session->memdup((unsigned char*) sel, sizeof(optimizer::SqlSelect));
               tab->cache.select->cond= tmp;
               tab->cache.select->read_tables= join->const_table_map;
             }
@@ -5881,7 +5881,7 @@
         !s->table->pos_in_table_list->embedding)
     {
       ha_rows records;
-      optimizer::SQL_SELECT *select= NULL;
+      optimizer::SqlSelect *select= NULL;
       select= optimizer::make_select(s->table, found_const_table_map, found_const_table_map, *s->on_expr_ref ? *s->on_expr_ref : conds, 1, &error);
       if (! select)
         return 1;

=== modified file 'drizzled/join.h'
--- drizzled/join.h	2009-12-04 20:16:03 +0000
+++ drizzled/join.h	2009-12-07 19:18:46 +0000
@@ -143,7 +143,7 @@
   Select_Lex_Unit *unit;
   /** select that processed */
   Select_Lex *select_lex;
-  drizzled::optimizer::SQL_SELECT *select; /**< created in optimisation phase */
+  drizzled::optimizer::SqlSelect *select; /**< created in optimization phase */
   Array<Item_in_subselect> sj_subselects;
 
   /**

=== modified file 'drizzled/join_cache.h'
--- drizzled/join_cache.h	2009-12-04 20:16:03 +0000
+++ drizzled/join_cache.h	2009-12-07 19:18:46 +0000
@@ -61,7 +61,7 @@
   uint32_t blobs;
   CACHE_FIELD *field;
   CACHE_FIELD **blob_ptr;
-  drizzled::optimizer::SQL_SELECT *select;
+  drizzled::optimizer::SqlSelect *select;
 } JOIN_CACHE;
 
 int join_init_cache(Session *session, JoinTable *tables, uint32_t table_count);

=== modified file 'drizzled/join_table.h'
--- drizzled/join_table.h	2009-12-04 20:16:03 +0000
+++ drizzled/join_table.h	2009-12-07 19:18:46 +0000
@@ -116,9 +116,9 @@
   {}
   Table *table;
   drizzled::optimizer::KeyUse *keyuse; /**< pointer to first used key */
-  drizzled::optimizer::SQL_SELECT *select;
+  drizzled::optimizer::SqlSelect *select;
   COND *select_cond;
-  drizzled::optimizer::QUICK_SELECT_I *quick;
+  drizzled::optimizer::QuickSelectInterface *quick;
   /**
     The value of select_cond before we've attempted to do Index Condition
     Pushdown. We may need to restore everything back if we first choose one
@@ -234,7 +234,7 @@
   {
     return (select && select->quick &&
             (select->quick->get_type() ==
-             drizzled::optimizer::QUICK_SELECT_I::QS_TYPE_GROUP_MIN_MAX));
+             drizzled::optimizer::QuickSelectInterface::QS_TYPE_GROUP_MIN_MAX));
   }
 
   void readCachedRecord();

=== modified file 'drizzled/optimizer/key_field.cc'
--- drizzled/optimizer/key_field.cc	2009-09-30 07:01:32 +0000
+++ drizzled/optimizer/key_field.cc	2009-12-08 18:39:43 +0000
@@ -32,13 +32,13 @@
 namespace drizzled
 {
 
-void optimizer::add_key_part(DYNAMIC_ARRAY *keyuse_array, 
+void optimizer::add_key_part(DYNAMIC_ARRAY *keyuse_array,
                              optimizer::KeyField *key_field)
 {
   Field *field= key_field->getField();
   Table *form= field->table;
 
-  if (key_field->isEqualityCondition() && 
+  if (key_field->isEqualityCondition() &&
       ! (key_field->getOptimizeFlags() & KEY_OPTIMIZE_EXISTS))
   {
     for (uint32_t key= 0; key < form->sizeKeys(); key++)
@@ -102,10 +102,10 @@
   }
   if (nested_join_table->on_expr)
   {
-    add_key_fields(join, 
-                   end, 
-                   and_level, 
-                   nested_join_table->on_expr, 
+    add_key_fields(join,
+                   end,
+                   and_level,
+                   nested_join_table->on_expr,
                    tables,
                    sargables);
   }
@@ -113,7 +113,7 @@
 
 optimizer::KeyField *optimizer::merge_key_fields(optimizer::KeyField *start,
                                                  optimizer::KeyField *new_fields,
-                                                 optimizer::KeyField *end, 
+                                                 optimizer::KeyField *end,
                                                  uint32_t and_level)
 {
   if (start == new_fields)
@@ -151,7 +151,7 @@
           if (old->getValue()->eq(new_fields->getValue(), old->getField()->binary()))
           {
             old->setLevel(and_level);
-            old->setOptimizeFlags(((old->getOptimizeFlags() & 
+            old->setOptimizeFlags(((old->getOptimizeFlags() &
                                     new_fields->getOptimizeFlags() &
                                     KEY_OPTIMIZE_EXISTS) |
                                    ((old->getOptimizeFlags() |
@@ -161,7 +161,7 @@
                                      new_fields->rejectNullValues());
           }
         }
-        else if (old->isEqualityCondition() && 
+        else if (old->isEqualityCondition() &&
                  new_fields->isEqualityCondition() &&
                  old->getValue()->eq_by_collation(new_fields->getValue(),
                                                   old->getField()->binary(),
@@ -169,18 +169,18 @@
 
         {
           old->setLevel(and_level);
-          old->setOptimizeFlags(((old->getOptimizeFlags() & 
+          old->setOptimizeFlags(((old->getOptimizeFlags() &
                                   new_fields->getOptimizeFlags() &
                                   KEY_OPTIMIZE_EXISTS) |
-                                 ((old->getOptimizeFlags() | 
+                                 ((old->getOptimizeFlags() |
                                    new_fields->getOptimizeFlags()) &
                                  KEY_OPTIMIZE_REF_OR_NULL)));
           old->setRejectNullValues(old->rejectNullValues() &&
                                    new_fields->rejectNullValues());
         }
-        else if (old->isEqualityCondition() && 
+        else if (old->isEqualityCondition() &&
                  new_fields->isEqualityCondition() &&
-                 ((old->getValue()->const_item() && 
+                 ((old->getValue()->const_item() &&
                    old->getValue()->is_null()) ||
                    new_fields->getValue()->is_null()))
         {
@@ -191,7 +191,7 @@
             Remember the NOT NULL value unless the value does not depend
             on other tables.
            */
-          if (! old->getValue()->used_tables() && 
+          if (! old->getValue()->used_tables() &&
               old->getValue()->is_null())
           {
             old->setValue(new_fields->getValue());
@@ -413,7 +413,7 @@
   }
 }
 
-void optimizer::add_key_fields(JOIN *join, 
+void optimizer::add_key_fields(JOIN *join,
                                optimizer::KeyField **key_fields,
                                uint32_t *and_level,
                                COND *cond,
@@ -430,10 +430,10 @@
       Item *item;
       while ((item= li++))
       {
-        add_key_fields(join, 
-                       key_fields, 
-                       and_level, 
-                       item, 
+        add_key_fields(join,
+                       key_fields,
+                       and_level,
+                       item,
                        usable_tables,
                        sargables);
       }
@@ -443,10 +443,10 @@
     else
     {
       (*and_level)++;
-      add_key_fields(join, 
-                     key_fields, 
-                     and_level, 
-                     li++, 
+      add_key_fields(join,
+                     key_fields,
+                     and_level,
+                     li++,
                      usable_tables,
                      sargables);
       Item *item;
@@ -454,10 +454,10 @@
       {
         optimizer::KeyField *start_key_fields= *key_fields;
         (*and_level)++;
-        add_key_fields(join, 
-                       key_fields, 
-                       and_level, 
-                       item, 
+        add_key_fields(join,
+                       key_fields,
+                       and_level,
+                       item,
                        usable_tables,
                        sargables);
         *key_fields= merge_key_fields(org_key_fields, start_key_fields,
@@ -477,17 +477,17 @@
         ((Item_func*)cond)->functype() == Item_func::TRIG_COND_FUNC)
     {
       Item *cond_arg= ((Item_func*)cond)->arguments()[0];
-      if (! join->group_list && 
+      if (! join->group_list &&
           ! join->order &&
           join->unit->item &&
           join->unit->item->substype() == Item_subselect::IN_SUBS &&
           ! join->unit->is_union())
       {
         optimizer::KeyField *save= *key_fields;
-        add_key_fields(join, 
-                       key_fields, 
-                       and_level, 
-                       cond_arg, 
+        add_key_fields(join,
+                       key_fields,
+                       and_level,
+                       cond_arg,
                        usable_tables,
                        sargables);
         /* Indicate that this ref access candidate is for subquery lookup */
@@ -502,7 +502,7 @@
   if (cond->type() != Item::FUNC_ITEM)
     return;
   Item_func *cond_func= (Item_func*) cond;
-  switch (cond_func->select_optimize()) 
+  switch (cond_func->select_optimize())
   {
   case Item_func::OPTIMIZE_NONE:
     break;

=== added file 'drizzled/optimizer/quick_index_merge_select.cc'
--- drizzled/optimizer/quick_index_merge_select.cc	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_index_merge_select.cc	2009-12-08 18:39:43 +0000
@@ -0,0 +1,267 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/records.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "drizzled/optimizer/quick_index_merge_select.h"
+
+using namespace drizzled;
+
+static int refpos_order_cmp(void *arg, const void *a, const void *b)
+{
+  Cursor *cursor= (Cursor*)arg;
+  return cursor->cmp_ref((const unsigned char *) a, (const unsigned char *) b);
+}
+
+optimizer::QuickIndexMergeSelect::QuickIndexMergeSelect(Session *session_param,
+                                                        Table *table)
+  :
+    pk_quick_select(NULL),
+    session(session_param)
+{
+  index= MAX_KEY;
+  head= table;
+  memset(&read_record, 0, sizeof(read_record));
+  init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
+  return;
+}
+
+int optimizer::QuickIndexMergeSelect::init()
+{
+  return 0;
+}
+
+int optimizer::QuickIndexMergeSelect::reset()
+{
+  return (read_keys_and_merge());
+}
+
+bool
+optimizer::QuickIndexMergeSelect::push_quick_back(optimizer::QuickRangeSelect *quick_sel_range)
+{
+  /*
+    Save quick_select that does scan on clustered primary key as it will be
+    processed separately.
+  */
+  if (head->cursor->primary_key_is_clustered() &&
+      quick_sel_range->index == head->s->primary_key)
+  {
+    pk_quick_select= quick_sel_range;
+  }
+  else
+  {
+    return quick_selects.push_back(quick_sel_range);
+  }
+  return 0;
+}
+
+optimizer::QuickIndexMergeSelect::~QuickIndexMergeSelect()
+{
+  List_iterator_fast<optimizer::QuickRangeSelect> quick_it(quick_selects);
+  optimizer::QuickRangeSelect* quick;
+  quick_it.rewind();
+  while ((quick= quick_it++))
+  {
+    quick->cursor= NULL;
+  }
+  quick_selects.delete_elements();
+  delete pk_quick_select;
+  free_root(&alloc,MYF(0));
+  return;
+}
+
+
+int optimizer::QuickIndexMergeSelect::read_keys_and_merge()
+{
+  List_iterator_fast<optimizer::QuickRangeSelect> cur_quick_it(quick_selects);
+  optimizer::QuickRangeSelect* cur_quick;
+  int result;
+  Unique *unique;
+  Cursor *cursor= head->cursor;
+
+  cursor->extra(HA_EXTRA_KEYREAD);
+  head->prepare_for_position();
+
+  cur_quick_it.rewind();
+  cur_quick= cur_quick_it++;
+  assert(cur_quick != 0);
+
+  /*
+    We reuse the same instance of Cursor so we need to call both init and
+    reset here.
+  */
+  if (cur_quick->init() || cur_quick->reset())
+    return 0;
+
+  unique= new Unique(refpos_order_cmp,
+                     (void *)cursor,
+                     cursor->ref_length,
+                     session->variables.sortbuff_size);
+  if (!unique)
+    return 0;
+  for (;;)
+  {
+    while ((result= cur_quick->get_next()) == HA_ERR_END_OF_FILE)
+    {
+      cur_quick->range_end();
+      cur_quick= cur_quick_it++;
+      if (!cur_quick)
+        break;
+
+      if (cur_quick->cursor->inited != Cursor::NONE)
+        cur_quick->cursor->ha_index_end();
+      if (cur_quick->init() || cur_quick->reset())
+        return 0;
+    }
+
+    if (result)
+    {
+      if (result != HA_ERR_END_OF_FILE)
+      {
+        cur_quick->range_end();
+        return result;
+      }
+      break;
+    }
+
+    if (session->killed)
+      return 0;
+
+    /* skip row if it will be retrieved by clustered PK scan */
+    if (pk_quick_select && pk_quick_select->row_in_ranges())
+      continue;
+
+    cur_quick->cursor->position(cur_quick->record);
+    result= unique->unique_add((char*)cur_quick->cursor->ref);
+    if (result)
+      return 0;
+
+  }
+
+  /* ok, all row ids are in Unique */
+  result= unique->get(head);
+  delete unique;
+  doing_pk_scan= false;
+  /* index_merge currently doesn't support "using index" at all */
+  cursor->extra(HA_EXTRA_NO_KEYREAD);
+  /* start table scan */
+  init_read_record(&read_record, session, head, (optimizer::SqlSelect*) 0, 1, 1);
+  return result;
+}
+
+
+int optimizer::QuickIndexMergeSelect::get_next()
+{
+  int result;
+
+  if (doing_pk_scan)
+    return(pk_quick_select->get_next());
+
+  if ((result= read_record.read_record(&read_record)) == -1)
+  {
+    result= HA_ERR_END_OF_FILE;
+    end_read_record(&read_record);
+    /* All rows from Unique have been retrieved, do a clustered PK scan */
+    if (pk_quick_select)
+    {
+      doing_pk_scan= true;
+      if ((result= pk_quick_select->init()) ||
+          (result= pk_quick_select->reset()))
+        return result;
+      return(pk_quick_select->get_next());
+    }
+  }
+
+  return result;
+}
+
+bool optimizer::QuickIndexMergeSelect::is_keys_used(const MyBitmap *fields)
+{
+  optimizer::QuickRangeSelect *quick= NULL;
+  List_iterator_fast<QuickRangeSelect> it(quick_selects);
+  while ((quick= it++))
+  {
+    if (is_key_used(head, quick->index, fields))
+      return 1;
+  }
+  return 0;
+}
+
+
+void optimizer::QuickIndexMergeSelect::add_info_string(String *str)
+{
+  optimizer::QuickRangeSelect *quick= NULL;
+  bool first= true;
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
+  str->append(STRING_WITH_LEN("sort_union("));
+  while ((quick= it++))
+  {
+    if (! first)
+      str->append(',');
+    else
+      first= false;
+    quick->add_info_string(str);
+  }
+  if (pk_quick_select)
+  {
+    str->append(',');
+    pk_quick_select->add_info_string(str);
+  }
+  str->append(')');
+}
+
+
+void optimizer::QuickIndexMergeSelect::add_keys_and_lengths(String *key_names,
+                                                            String *used_lengths)
+{
+  char buf[64];
+  uint32_t length;
+  bool first= true;
+  optimizer::QuickRangeSelect *quick= NULL;
+
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
+  while ((quick= it++))
+  {
+    if (first)
+      first= false;
+    else
+    {
+      key_names->append(',');
+      used_lengths->append(',');
+    }
+
+    KEY *key_info= head->key_info + quick->index;
+    key_names->append(key_info->name);
+    length= int64_t2str(quick->max_used_key_length, buf, 10) - buf;
+    used_lengths->append(buf, length);
+  }
+  if (pk_quick_select)
+  {
+    KEY *key_info= head->key_info + pk_quick_select->index;
+    key_names->append(',');
+    key_names->append(key_info->name);
+    length= int64_t2str(pk_quick_select->max_used_key_length, buf, 10) - buf;
+    used_lengths->append(',');
+    used_lengths->append(buf, length);
+  }
+}
+
+

=== added file 'drizzled/optimizer/quick_index_merge_select.h'
--- drizzled/optimizer/quick_index_merge_select.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_index_merge_select.h	2009-12-08 02:07:26 +0000
@@ -0,0 +1,169 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_QUICK_INDEX_MERGE_SELECT_H
+#define DRIZZLED_OPTIMIZER_QUICK_INDEX_MERGE_SELECT_H
+
+#include "drizzled/optimizer/range.h"
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+/**
+  @class QuickIndexMergeSelect - index_merge access method quick select.
+
+    QuickIndexMergeSelect uses
+     * QuickRangeSelects to get rows
+     * Unique class to remove duplicate rows
+
+  INDEX MERGE OPTIMIZER
+    Current implementation doesn't detect all cases where index_merge could
+    be used, in particular:
+     * index_merge will never be used if range scan is possible (even if
+       range scan is more expensive)
+
+     * index_merge+'using index' is not supported (this the consequence of
+       the above restriction)
+
+     * If WHERE part contains complex nested AND and OR conditions, some ways
+       to retrieve rows using index_merge will not be considered. The choice
+       of read plan may depend on the order of conjuncts/disjuncts in WHERE
+       part of the query, see comments near imerge_list_or_list and
+       SEL_IMERGE::or_sel_tree_with_checks functions for details.
+
+     * There is no "index_merge_ref" method (but index_merge on non-first
+       table in join is possible with 'range checked for each record').
+
+    See comments around SEL_IMERGE class and test_quick_select for more
+    details.
+
+  ROW RETRIEVAL ALGORITHM
+
+    index_merge uses Unique class for duplicates removal.  index_merge takes
+    advantage of Clustered Primary Key (CPK) if the table has one.
+    The index_merge algorithm consists of two phases:
+
+    Phase 1 (implemented in QuickIndexMergeSelect::prepare_unique):
+    prepare()
+    {
+      activate 'index only';
+      while(retrieve next row for non-CPK scan)
+      {
+        if (there is a CPK scan and row will be retrieved by it)
+          skip this row;
+        else
+          put its rowid into Unique;
+      }
+      deactivate 'index only';
+    }
+
+    Phase 2 (implemented as sequence of QuickIndexMergeSelect::get_next
+    calls):
+
+    fetch()
+    {
+      retrieve all rows from row pointers stored in Unique;
+      free Unique;
+      retrieve all rows for CPK scan;
+    }
+*/
+class QuickIndexMergeSelect : public QuickSelectInterface
+{
+public:
+
+  QuickIndexMergeSelect(Session *session, Table *table);
+
+  ~QuickIndexMergeSelect();
+
+  int init();
+  int reset(void);
+
+  /**
+   * Get next row for index_merge.
+   * NOTES
+   * The rows are read from
+   * 1. rowids stored in Unique.
+   * 2. QuickRangeSelect with clustered primary key (if any).
+   * The sets of rows retrieved in 1) and 2) are guaranteed to be disjoint.
+   */
+  int get_next();
+
+  bool reverse_sorted()
+  {
+    return false;
+  }
+
+  bool unique_key_range()
+  {
+    return false;
+  }
+
+  int get_type()
+  {
+    return QS_TYPE_INDEX_MERGE;
+  }
+
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+  void add_info_string(String *str);
+  bool is_keys_used(const MyBitmap *fields);
+
+  bool push_quick_back(QuickRangeSelect *quick_sel_range);
+
+  /* range quick selects this index_merge read consists of */
+  List<QuickRangeSelect> quick_selects;
+
+  /* quick select that uses clustered primary key (NULL if none) */
+  QuickRangeSelect* pk_quick_select;
+
+  /* true if this select is currently doing a clustered PK scan */
+  bool  doing_pk_scan;
+
+  MEM_ROOT alloc;
+  Session *session;
+
+  /**
+   * Perform key scans for all used indexes (except CPK), get rowids and merge
+   * them into an ordered non-recurrent sequence of rowids.
+   *
+   * The merge/duplicate removal is performed using Unique class. We put all
+   * rowids into Unique, get the sorted sequence and destroy the Unique.
+   *
+   * If table has a clustered primary key that covers all rows (true for bdb
+   * and innodb currently) and one of the index_merge scans is a scan on PK,
+   * then rows that will be retrieved by PK scan are not put into Unique and
+   * primary key scan is not performed here, it is performed later separately.
+   *
+   * RETURN
+   * @retval 0     OK
+   * @retval other error
+   */
+  int read_keys_and_merge();
+
+  /* used to get rows collected in Unique */
+  READ_RECORD read_record;
+};
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_QUICK_INDEX_MERGE_SELECT_H */

=== added file 'drizzled/optimizer/quick_range.cc'
--- drizzled/optimizer/quick_range.cc	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range.cc	2009-12-08 18:39:43 +0000
@@ -0,0 +1,34 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/optimizer/quick_range.h"
+
+using namespace drizzled;
+
+optimizer::QuickRange::QuickRange()
+  :
+    min_key(0),
+    max_key(0),
+    min_length(0),
+    max_length(0),
+    flag(NO_MIN_RANGE | NO_MAX_RANGE),
+    min_keypart_map(0),
+    max_keypart_map(0)
+{}

=== added file 'drizzled/optimizer/quick_range.h'
--- drizzled/optimizer/quick_range.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range.h	2009-12-08 18:39:43 +0000
@@ -0,0 +1,63 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_QUICK_RANGE_H
+#define DRIZZLED_OPTIMIZER_QUICK_RANGE_H
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+class QuickRange : public Sql_alloc
+{
+public:
+  unsigned char *min_key;
+  unsigned char *max_key;
+  uint16_t min_length;
+  uint16_t max_length;
+  uint16_t flag;
+  key_part_map min_keypart_map; /**< bitmap of used keyparts in min_key */
+  key_part_map max_keypart_map; /**< bitmap of used keyparts in max_key */
+
+  QuickRange(); /**< Constructor for a "full range" */
+  QuickRange(const unsigned char *min_key_arg,
+              uint32_t min_length_arg,
+              key_part_map min_keypart_map_arg,
+	            const unsigned char *max_key_arg,
+              uint32_t max_length_arg,
+              key_part_map max_keypart_map_arg,
+	            uint32_t flag_arg)
+    :
+      min_key((unsigned char*) sql_memdup(min_key_arg,min_length_arg+1)),
+      max_key((unsigned char*) sql_memdup(max_key_arg,max_length_arg+1)),
+      min_length((uint16_t) min_length_arg),
+      max_length((uint16_t) max_length_arg),
+      flag((uint16_t) flag_arg),
+      min_keypart_map(min_keypart_map_arg),
+      max_keypart_map(max_keypart_map_arg)
+    {}
+};
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_QUICK_RANGE_H */

=== added file 'drizzled/optimizer/quick_range_select.cc'
--- drizzled/optimizer/quick_range_select.cc	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range_select.cc	2009-12-08 18:39:43 +0000
@@ -0,0 +1,615 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/session.h"
+#include "drizzled/optimizer/quick_range.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "mysys/my_bitmap.h"
+#include "drizzled/memory/multi_malloc.h"
+
+using namespace std;
+using namespace drizzled;
+
+
+optimizer::QuickRangeSelect::QuickRangeSelect(Session *session,
+                                              Table *table,
+                                              uint32_t key_nr,
+                                              bool no_alloc,
+                                              MEM_ROOT *parent_alloc,
+                                              bool *create_error)
+  :
+    cursor(NULL),
+    ranges(),
+    in_ror_merged_scan(false),
+    column_bitmap(),
+    save_read_set(NULL),
+    save_write_set(NULL),
+    free_file(false),
+    cur_range(NULL),
+    last_range(NULL),
+    qr_traversal_ctx(),
+    mrr_buf_size(0),
+    mrr_buf_desc(NULL),
+    key_parts(NULL),
+    dont_free(false),
+    mrr_flags(0),
+    alloc()
+{
+  my_bitmap_map *bitmap= NULL;
+
+  sorted= 0;
+  index= key_nr;
+  head= table;
+  key_part_info= head->key_info[index].key_part;
+  my_init_dynamic_array(&ranges, sizeof(optimizer::QuickRange*), 16, 16);
+
+  /* 'session' is not accessible in QuickRangeSelect::reset(). */
+  mrr_buf_size= session->variables.read_rnd_buff_size;
+  mrr_buf_desc= NULL;
+
+  if (! no_alloc && ! parent_alloc)
+  {
+    // Allocates everything through the internal memroot
+    init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
+    session->mem_root= &alloc;
+  }
+  else
+  {
+    memset(&alloc, 0, sizeof(alloc));
+  }
+
+  cursor= head->cursor;
+  record= head->record[0];
+  save_read_set= head->read_set;
+  save_write_set= head->write_set;
+
+  /* Allocate a bitmap for used columns. Using sql_alloc instead of malloc
+     simply as a "fix" to the MySQL 6.0 code that also free()s it at the
+     same time we destroy the mem_root.
+   */
+
+  bitmap= reinterpret_cast<my_bitmap_map*>(sql_alloc(head->s->column_bitmap_size));
+  if (! bitmap)
+  {
+    column_bitmap.setBitmap(NULL);
+    *create_error= 1;
+  }
+  else
+  {
+    column_bitmap.init(bitmap, head->s->fields);
+  }
+}
+
+
+int optimizer::QuickRangeSelect::init()
+{
+  if (cursor->inited != Cursor::NONE)
+    cursor->ha_index_or_rnd_end();
+  return (cursor->ha_index_init(index, 1));
+}
+
+
+void optimizer::QuickRangeSelect::range_end()
+{
+  if (cursor->inited != Cursor::NONE)
+    cursor->ha_index_or_rnd_end();
+}
+
+
+optimizer::QuickRangeSelect::~QuickRangeSelect()
+{
+  if (! dont_free)
+  {
+    /* cursor is NULL for CPK scan on covering ROR-intersection */
+    if (cursor)
+    {
+      range_end();
+      if (head->key_read)
+      {
+        head->key_read= 0;
+        cursor->extra(HA_EXTRA_NO_KEYREAD);
+      }
+      if (free_file)
+      {
+        cursor->ha_external_lock(current_session, F_UNLCK);
+        cursor->close();
+        delete cursor;
+      }
+    }
+    delete_dynamic(&ranges); /* ranges are allocated in alloc */
+    free_root(&alloc,MYF(0));
+  }
+  head->column_bitmaps_set(save_read_set, save_write_set);
+  assert(mrr_buf_desc == NULL);
+  if (mrr_buf_desc)
+  {
+    free(mrr_buf_desc);
+  }
+}
+
+
+int optimizer::QuickRangeSelect::init_ror_merged_scan(bool reuse_handler)
+{
+  Cursor *save_file= cursor, *org_file;
+  Session *session;
+
+  in_ror_merged_scan= 1;
+  if (reuse_handler)
+  {
+    if (init() || reset())
+    {
+      return 0;
+    }
+    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+    goto end;
+  }
+
+  /* Create a separate Cursor object for this quick select */
+  if (free_file)
+  {
+    /* already have own 'Cursor' object. */
+    return 0;
+  }
+
+  session= head->in_use;
+  if (! (cursor= head->cursor->clone(session->mem_root)))
+  {
+    /*
+      Manually set the error flag. Note: there seems to be quite a few
+      places where a failure could cause the server to "hang" the client by
+      sending no response to a query. ATM those are not real errors because
+      the storage engine calls in question happen to never fail with the
+      existing storage engines.
+    */
+    my_error(ER_OUT_OF_RESOURCES, MYF(0));
+    /* Caller will free the memory */
+    goto failure;
+  }
+
+  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+
+  if (cursor->ha_external_lock(session, F_RDLCK))
+    goto failure;
+
+  if (init() || reset())
+  {
+    cursor->ha_external_lock(session, F_UNLCK);
+    cursor->close();
+    goto failure;
+  }
+  free_file= true;
+  last_rowid= cursor->ref;
+
+end:
+  /*
+    We are only going to read key fields and call position() on 'cursor'
+    The following sets head->tmp_set to only use this key and then updates
+    head->read_set and head->write_set to use this bitmap.
+    The now bitmap is stored in 'column_bitmap' which is used in ::get_next()
+  */
+  org_file= head->cursor;
+  head->cursor= cursor;
+  /* We don't have to set 'head->keyread' here as the 'cursor' is unique */
+  if (! head->no_keyread)
+  {
+    head->key_read= 1;
+    head->mark_columns_used_by_index(index);
+  }
+  head->prepare_for_position();
+  head->cursor= org_file;
+  column_bitmap= *head->read_set;
+  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+
+  return 0;
+
+failure:
+  head->column_bitmaps_set(save_read_set, save_write_set);
+  delete cursor;
+  cursor= save_file;
+  return 0;
+}
+
+
+void optimizer::QuickRangeSelect::save_last_pos()
+{
+  cursor->position(record);
+}
+
+
+bool optimizer::QuickRangeSelect::unique_key_range()
+{
+  if (ranges.elements == 1)
+  {
+    optimizer::QuickRange *tmp= *((optimizer::QuickRange**)ranges.buffer);
+    if ((tmp->flag & (EQ_RANGE | NULL_RANGE)) == EQ_RANGE)
+    {
+      KEY *key=head->key_info+index;
+      return ((key->flags & (HA_NOSAME)) == HA_NOSAME &&
+	      key->key_length == tmp->min_length);
+    }
+  }
+  return false;
+}
+
+
+int optimizer::QuickRangeSelect::reset()
+{
+  uint32_t buf_size= 0;
+  unsigned char *mrange_buff= NULL;
+  int error= 0;
+  HANDLER_BUFFER empty_buf;
+  last_range= NULL;
+  cur_range= (optimizer::QuickRange**) ranges.buffer;
+
+  if (cursor->inited == Cursor::NONE && (error= cursor->ha_index_init(index, 1)))
+  {
+    return error;
+  }
+
+  /* Allocate buffer if we need one but haven't allocated it yet */
+  if (mrr_buf_size && ! mrr_buf_desc)
+  {
+    buf_size= mrr_buf_size;
+    while (buf_size && ! memory::multi_malloc(false,
+                                              &mrr_buf_desc,
+                                              sizeof(*mrr_buf_desc),
+                                              &mrange_buff,
+                                              buf_size,
+                                              NULL))
+    {
+      /* Try to shrink the buffers until both are 0. */
+      buf_size/= 2;
+    }
+    if (! mrr_buf_desc)
+    {
+      return HA_ERR_OUT_OF_MEM;
+    }
+
+    /* Initialize the Cursor buffer. */
+    mrr_buf_desc->buffer= mrange_buff;
+    mrr_buf_desc->buffer_end= mrange_buff + buf_size;
+    mrr_buf_desc->end_of_used_area= mrange_buff;
+  }
+
+  if (! mrr_buf_desc)
+  {
+    empty_buf.buffer= NULL;
+    empty_buf.buffer_end= NULL;
+    empty_buf.end_of_used_area= NULL;
+  }
+
+  if (sorted)
+  {
+     mrr_flags|= HA_MRR_SORTED;
+  }
+  RANGE_SEQ_IF seq_funcs= {
+    optimizer::quick_range_seq_init,
+    optimizer::quick_range_seq_next
+  };
+  error= cursor->multi_range_read_init(&seq_funcs,
+                                       (void*) this,
+                                       ranges.elements,
+                                       mrr_flags,
+                                       mrr_buf_desc ? mrr_buf_desc : &empty_buf);
+  return error;
+}
+
+
+int optimizer::QuickRangeSelect::get_next()
+{
+  char *dummy= NULL;
+  if (in_ror_merged_scan)
+  {
+    /*
+      We don't need to signal the bitmap change as the bitmap is always the
+      same for this head->cursor
+    */
+    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
+  }
+
+  int result= cursor->multi_range_read_next(&dummy);
+
+  if (in_ror_merged_scan)
+  {
+    /* Restore bitmaps set on entry */
+    head->column_bitmaps_set(save_read_set, save_write_set);
+  }
+  return result;
+}
+
+
+int optimizer::QuickRangeSelect::get_next_prefix(uint32_t prefix_length,
+                                                 key_part_map keypart_map,
+                                                 unsigned char *cur_prefix)
+{
+  for (;;)
+  {
+    int result;
+    key_range start_key, end_key;
+    if (last_range)
+    {
+      /* Read the next record in the same range with prefix after cur_prefix. */
+      assert(cur_prefix != 0);
+      result= cursor->index_read_map(record,
+                                     cur_prefix,
+                                     keypart_map,
+                                     HA_READ_AFTER_KEY);
+      if (result || (cursor->compare_key(cursor->end_range) <= 0))
+        return result;
+    }
+
+    uint32_t count= ranges.elements - (cur_range - (optimizer::QuickRange**) ranges.buffer);
+    if (count == 0)
+    {
+      /* Ranges have already been used up before. None is left for read. */
+      last_range= 0;
+      return HA_ERR_END_OF_FILE;
+    }
+    last_range= *(cur_range++);
+
+    start_key.key= (const unsigned char*) last_range->min_key;
+    start_key.length= min(last_range->min_length, (uint16_t)prefix_length);
+    start_key.keypart_map= last_range->min_keypart_map & keypart_map;
+    start_key.flag= ((last_range->flag & NEAR_MIN) ? HA_READ_AFTER_KEY :
+		                                                (last_range->flag & EQ_RANGE) ?
+		                                                HA_READ_KEY_EXACT : HA_READ_KEY_OR_NEXT);
+    end_key.key= (const unsigned char*) last_range->max_key;
+    end_key.length= min(last_range->max_length, (uint16_t)prefix_length);
+    end_key.keypart_map= last_range->max_keypart_map & keypart_map;
+    /*
+      We use READ_AFTER_KEY here because if we are reading on a key
+      prefix we want to find all keys with this prefix
+    */
+    end_key.flag= (last_range->flag & NEAR_MAX ? HA_READ_BEFORE_KEY :
+		                                             HA_READ_AFTER_KEY);
+
+    result= cursor->read_range_first(last_range->min_keypart_map ? &start_key : 0,
+				                             last_range->max_keypart_map ? &end_key : 0,
+                                     test(last_range->flag & EQ_RANGE),
+				                             sorted);
+    if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
+      last_range= 0; // Stop searching
+
+    if (result != HA_ERR_END_OF_FILE)
+      return result;
+    last_range= 0; // No matching rows; go to next range
+  }
+}
+
+
+bool optimizer::QuickRangeSelect::row_in_ranges()
+{
+  optimizer::QuickRange *res= NULL;
+  uint32_t min= 0;
+  uint32_t max= ranges.elements - 1;
+  uint32_t mid= (max + min) / 2;
+
+  while (min != max)
+  {
+    if (cmp_next(*(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid)))
+    {
+      /* current row value > mid->max */
+      min= mid + 1;
+    }
+    else
+      max= mid;
+    mid= (min + max) / 2;
+  }
+  res= *(optimizer::QuickRange**)dynamic_array_ptr(&ranges, mid);
+  return (! cmp_next(res) && ! cmp_prev(res));
+}
+
+
+int optimizer::QuickRangeSelect::cmp_next(optimizer::QuickRange *range_arg)
+{
+  if (range_arg->flag & NO_MAX_RANGE)
+    return 0;                                   /* key can't be to large */
+
+  KEY_PART *key_part= key_parts;
+  uint32_t store_length;
+
+  for (unsigned char *key=range_arg->max_key, *end=key+range_arg->max_length;
+       key < end;
+       key+= store_length, key_part++)
+  {
+    int cmp;
+    store_length= key_part->store_length;
+    if (key_part->null_bit)
+    {
+      if (*key)
+      {
+        if (! key_part->field->is_null())
+          return 1;
+        continue;
+      }
+      else if (key_part->field->is_null())
+        return 0;
+      key++;					// Skip null byte
+      store_length--;
+    }
+    if ((cmp= key_part->field->key_cmp(key, key_part->length)) < 0)
+      return 0;
+    if (cmp > 0)
+      return 1;
+  }
+  return (range_arg->flag & NEAR_MAX) ? 1 : 0;          // Exact match
+}
+
+
+int optimizer::QuickRangeSelect::cmp_prev(optimizer::QuickRange *range_arg)
+{
+  int cmp;
+  if (range_arg->flag & NO_MIN_RANGE)
+    return 0;					/* key can't be to small */
+
+  cmp= key_cmp(key_part_info,
+               range_arg->min_key,
+               range_arg->min_length);
+  if (cmp > 0 || (cmp == 0 && (range_arg->flag & NEAR_MIN) == false))
+    return 0;
+  return 1;                                     // outside of range
+}
+
+
+void optimizer::QuickRangeSelect::add_info_string(String *str)
+{
+  KEY *key_info= head->key_info + index;
+  str->append(key_info->name);
+}
+
+
+void optimizer::QuickRangeSelect::add_keys_and_lengths(String *key_names,
+                                                       String *used_lengths)
+{
+  char buf[64];
+  uint32_t length;
+  KEY *key_info= head->key_info + index;
+  key_names->append(key_info->name);
+  length= int64_t2str(max_used_key_length, buf, 10) - buf;
+  used_lengths->append(buf, length);
+}
+
+
+/*
+  This is a hack: we inherit from QUICK_SELECT so that we can use the
+  get_next() interface, but we have to hold a pointer to the original
+  QUICK_SELECT because its data are used all over the place.  What
+  should be done is to factor out the data that is needed into a base
+  class (QUICK_SELECT), and then have two subclasses (_ASC and _DESC)
+  which handle the ranges and implement the get_next() function.  But
+  for now, this seems to work right at least.
+ */
+optimizer::QuickSelectDescending::QuickSelectDescending(optimizer::QuickRangeSelect *q, uint32_t, bool *)
+  :
+    optimizer::QuickRangeSelect(*q),
+    rev_it(rev_ranges)
+{
+  optimizer::QuickRange *r= NULL;
+
+  optimizer::QuickRange **pr= (optimizer::QuickRange**)ranges.buffer;
+  optimizer::QuickRange **end_range= pr + ranges.elements;
+  for (; pr != end_range; pr++)
+    rev_ranges.push_front(*pr);
+
+  /* Remove EQ_RANGE flag for keys that are not using the full key */
+  for (r = rev_it++; r; r= rev_it++)
+  {
+    if ((r->flag & EQ_RANGE) &&
+        head->key_info[index].key_length != r->max_length)
+      r->flag&= ~EQ_RANGE;
+  }
+  rev_it.rewind();
+  q->dont_free= 1;				// Don't free shared mem
+  delete q;
+}
+
+
+int optimizer::QuickSelectDescending::get_next()
+{
+  /* The max key is handled as follows:
+   *   - if there is NO_MAX_RANGE, start at the end and move backwards
+   *   - if it is an EQ_RANGE, which means that max key covers the entire
+   *     key, go directly to the key and read through it (sorting backwards is
+   *     same as sorting forwards)
+   *   - if it is NEAR_MAX, go to the key or next, step back once, and
+   *     move backwards
+   *   - otherwise (not NEAR_MAX == include the key), go after the key,
+   *     step back once, and move backwards
+   */
+  for (;;)
+  {
+    int result;
+    if (last_range)
+    {						// Already read through key
+      result= ((last_range->flag & EQ_RANGE) ?
+		           cursor->index_next_same(record, last_range->min_key,
+					                             last_range->min_length) :
+		           cursor->index_prev(record));
+      if (! result)
+      {
+        if (cmp_prev(*rev_it.ref()) == 0)
+          return 0;
+      }
+      else if (result != HA_ERR_END_OF_FILE)
+        return result;
+    }
+
+    if (! (last_range= rev_it++))
+      return HA_ERR_END_OF_FILE;		// All ranges used
+
+    if (last_range->flag & NO_MAX_RANGE)        // Read last record
+    {
+      int local_error;
+      if ((local_error= cursor->index_last(record)))
+        return local_error;	// Empty table
+      if (cmp_prev(last_range) == 0)
+        return 0;
+      last_range= 0; // No match; go to next range
+      continue;
+    }
+
+    if (last_range->flag & EQ_RANGE)
+    {
+      result = cursor->index_read_map(record,
+                                      last_range->max_key,
+                                      last_range->max_keypart_map,
+                                      HA_READ_KEY_EXACT);
+    }
+    else
+    {
+      assert(last_range->flag & NEAR_MAX ||
+             range_reads_after_key(last_range));
+      result= cursor->index_read_map(record,
+                                     last_range->max_key,
+                                     last_range->max_keypart_map,
+                                     ((last_range->flag & NEAR_MAX) ?
+                                      HA_READ_BEFORE_KEY :
+                                      HA_READ_PREFIX_LAST_OR_PREV));
+    }
+    if (result)
+    {
+      if (result != HA_ERR_KEY_NOT_FOUND && result != HA_ERR_END_OF_FILE)
+        return result;
+      last_range= 0;                            // Not found, to next range
+      continue;
+    }
+    if (cmp_prev(last_range) == 0)
+    {
+      if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
+        last_range= 0;				// Stop searching
+      return 0;				// Found key is in range
+    }
+    last_range= 0;                              // To next range
+  }
+}
+
+
+/*
+ * true if this range will require using HA_READ_AFTER_KEY
+   See comment in get_next() about this
+ */
+bool optimizer::QuickSelectDescending::range_reads_after_key(optimizer::QuickRange *range_arg)
+{
+  return ((range_arg->flag & (NO_MAX_RANGE | NEAR_MAX)) ||
+	        ! (range_arg->flag & EQ_RANGE) ||
+	        head->key_info[index].key_length != range_arg->max_length) ? 1 : 0;
+}
+
+

=== added file 'drizzled/optimizer/quick_range_select.h'
--- drizzled/optimizer/quick_range_select.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/quick_range_select.h	2009-12-08 00:38:13 +0000
@@ -0,0 +1,312 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H
+#define DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H
+
+#include "drizzled/optimizer/range.h"
+
+class Cursor;
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+/**
+ * Quick select that does a range scan on a single key. 
+ *
+ * The records are returned in key order.
+ * 
+ */
+class QuickRangeSelect : public QuickSelectInterface
+{
+protected:
+  Cursor *cursor;
+  DYNAMIC_ARRAY ranges; /**< ordered array of range ptrs */
+
+  /** Members to deal with case when this quick select is a ROR-merged scan */
+  bool in_ror_merged_scan;
+  MyBitmap column_bitmap;
+  MyBitmap *save_read_set;
+  MyBitmap *save_write_set;
+  bool free_file; /**< True when this->file is "owned" by this quick select */
+
+  /* Range pointers to be used when not using MRR interface */
+  QuickRange **cur_range; /**< current element in ranges  */
+  QuickRange *last_range;
+
+  /** Members needed to use the MRR interface */
+  QuickRangeSequenceContext qr_traversal_ctx;
+  uint32_t mrr_buf_size; /**< copy from session->variables.read_rnd_buff_size */
+  HANDLER_BUFFER *mrr_buf_desc; /**< the Cursor buffer */
+
+  /** Info about index we're scanning */
+  KEY_PART *key_parts;
+  KEY_PART_INFO *key_part_info;
+
+  bool dont_free; /**< Used by QuickSelectDescending */
+
+  /**
+   * Compare if found key is over max-value
+   * @return 0 if key <= range->max_key
+   * @todo: Figure out why can't this function be as simple as cmp_prev().
+   */
+  int cmp_next(QuickRange *range);
+
+  /**
+   * @return 0 if found key is inside range (found key >= range->min_key).
+   */
+  int cmp_prev(QuickRange *range);
+
+  /**
+   * Check if current row will be retrieved by this QuickRangeSelect
+   *
+   * NOTES
+   * It is assumed that currently a scan is being done on another index
+   * which reads all necessary parts of the index that is scanned by this
+   * quick select.
+   * The implementation does a binary search on sorted array of disjoint
+   * ranges, without taking size of range into account.
+   *
+   * This function is used to filter out clustered PK scan rows in
+   * index_merge quick select.
+   *
+   * RETURN
+   * @retval true  if current row will be retrieved by this quick select
+   * false if not
+   */
+  bool row_in_ranges();
+
+public:
+
+  uint32_t mrr_flags; /**< Flags to be used with MRR interface */
+
+  MEM_ROOT alloc;
+
+  QuickRangeSelect(Session *session,
+                     Table *table,
+                     uint32_t index_arg,
+                     bool no_alloc,
+                     MEM_ROOT *parent_alloc,
+                     bool *create_err);
+
+  ~QuickRangeSelect();
+
+  int init();
+
+  int reset(void);
+
+  /**
+   * Get next possible record using quick-struct.
+   *
+   * SYNOPSIS
+   * QuickRangeSelect::get_next()
+   *
+   * NOTES
+   * Record is read into table->record[0]
+   *
+   * RETURN
+   * @retval 0			Found row
+   * @retval HA_ERR_END_OF_FILE	No (more) rows in range
+   * @retaval #	Error code
+   */
+  int get_next();
+
+  void range_end();
+
+  /**
+   * Get the next record with a different prefix.
+   *
+   * SYNOPSIS
+   * QuickRangeSelect::get_next_prefix()
+   * @param[in] prefix_length  length of cur_prefix
+   * @param[in] cur_prefix     prefix of a key to be searched for
+   *
+   * DESCRIPTION
+   * Each subsequent call to the method retrieves the first record that has a
+   * prefix with length prefix_length different from cur_prefix, such that the
+   * record with the new prefix is within the ranges described by
+   * this->ranges. The record found is stored into the buffer pointed by
+   * this->record.
+   * The method is useful for GROUP-BY queries with range conditions to
+   * discover the prefix of the next group that satisfies the range conditions.
+   *
+   * @todo
+   * This method is a modified copy of QuickRangeSelect::get_next(), so both
+   * methods should be unified into a more general one to reduce code
+   * duplication.
+   *
+   * RETURN
+   * @retval 0                  on success
+   * @retval HA_ERR_END_OF_FILE if returned all keys
+   * @retval other              if some error occurred
+   */
+  int get_next_prefix(uint32_t prefix_length,
+                      key_part_map keypart_map,
+                      unsigned char *cur_prefix);
+
+  bool reverse_sorted()
+  {
+    return false;
+  }
+
+  /**
+   * @return true if there is only one range and this uses the whole primary key
+   */
+  bool unique_key_range();
+
+  /**
+   * Initialize this quick select to be a ROR-merged scan.
+   *
+   * SYNOPSIS
+   * QuickRangeSelect::init_ror_merged_scan()
+   * @param[in] reuse_handler If true, use head->cursor, otherwise create a separate Cursor object
+   *
+   * NOTES
+   * This function creates and prepares for subsequent use a separate Cursor
+   * object if it can't reuse head->cursor. The reason for this is that during
+   * ROR-merge several key scans are performed simultaneously, and a single
+   * Cursor is only capable of preserving context of a single key scan.
+   *
+   * In ROR-merge the quick select doing merge does full records retrieval,
+   * merged quick selects read only keys.
+   *
+   * RETURN
+   * @reval 0  ROR child scan initialized, ok to use.
+   * @retval 1  error
+   */
+  int init_ror_merged_scan(bool reuse_handler);
+
+  void save_last_pos();
+
+  int get_type()
+  {
+    return QS_TYPE_RANGE;
+  }
+
+  void add_keys_and_lengths(String *key_names, String *used_lengths);
+
+  void add_info_string(String *str);
+
+  void resetCursor()
+  {
+    cursor= NULL;
+  }
+
+private:
+
+  /* Used only by QuickSelectDescending */
+  QuickRangeSelect(const QuickRangeSelect& org) : QuickSelectInterface()
+  {
+    memmove(this, &org, sizeof(*this));
+    /*
+      Use default MRR implementation for reverse scans. No table engine
+      currently can do an MRR scan with output in reverse index order.
+    */
+    mrr_buf_desc= NULL;
+    mrr_flags|= HA_MRR_USE_DEFAULT_IMPL;
+    mrr_buf_size= 0;
+  }
+
+  friend class ::TRP_ROR_INTERSECT; 
+
+  friend
+  QuickRangeSelect *get_quick_select_for_ref(Session *session, Table *table,
+                                             struct table_reference_st *ref,
+                                             ha_rows records);
+
+  friend bool get_quick_keys(Parameter *param, 
+                             QuickRangeSelect *quick,
+                             KEY_PART *key, 
+                             SEL_ARG *key_tree,
+                             unsigned char *min_key, 
+                             uint32_t min_key_flag,
+                             unsigned char *max_key, 
+                             uint32_t max_key_flag);
+
+  friend QuickRangeSelect *get_quick_select(Parameter *,
+                                            uint32_t idx,
+                                            SEL_ARG *key_tree,
+                                            uint32_t mrr_flags,
+                                            uint32_t mrr_buf_size,
+                                            MEM_ROOT *alloc);
+  friend class QuickSelectDescending;
+
+  friend class QuickIndexMergeSelect;
+
+  friend class QUICK_ROR_INTERSECT_SELECT;
+
+  friend class QUICK_GROUP_MIN_MAX_SELECT;
+
+  friend uint32_t quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range);
+
+  friend range_seq_t quick_range_seq_init(void *init_param,
+                                          uint32_t n_ranges, 
+                                          uint32_t flags);
+
+  friend void select_describe(JOIN *join, 
+                              bool need_tmp_table, 
+                              bool need_order,
+                              bool distinct,
+                              const char *message);
+};
+
+class QuickSelectDescending : public QuickRangeSelect
+{
+public:
+
+  QuickSelectDescending(QuickRangeSelect *q, 
+                        uint32_t used_key_parts,
+                        bool *create_err);
+
+  int get_next();
+
+  bool reverse_sorted() 
+  { 
+    return true; 
+  }
+
+  int get_type() 
+  { 
+    return QS_TYPE_RANGE_DESC;
+  }
+
+private:
+
+  bool range_reads_after_key(QuickRange *range);
+
+  int reset(void) 
+  { 
+    rev_it.rewind(); 
+    return QuickRangeSelect::reset();
+  }
+
+  List<QuickRange> rev_ranges;
+
+  List_iterator<QuickRange> rev_it;
+
+};
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_QUICK_RANGE_SELECT_H */

=== modified file 'drizzled/optimizer/range.cc'
--- drizzled/optimizer/range.cc	2009-12-05 20:06:30 +0000
+++ drizzled/optimizer/range.cc	2009-12-08 18:39:43 +0000
@@ -117,6 +117,11 @@
 #include "drizzled/check_stack_overrun.h"
 #include "drizzled/optimizer/sum.h"
 #include "drizzled/optimizer/range.h"
+#include "drizzled/optimizer/quick_range.h"
+#include "drizzled/optimizer/quick_range_select.h"
+#include "drizzled/optimizer/quick_index_merge_select.h"
+#include "drizzled/optimizer/sel_arg.h"
+#include "drizzled/optimizer/range_param.h"
 #include "drizzled/records.h"
 
 #include "drizzled/temporal.h" /* Needed in get_mm_leaf() for timestamp -> datetime comparisons */
@@ -141,14 +146,6 @@
     return static_cast<ha_rows>(x);
 }
 
-extern "C" int refpos_order_cmp(void *arg, const void *a, const void *b)
-{
-  Cursor *cursor= (Cursor*)arg;
-  return cursor->cmp_ref((const unsigned char *) a, (const unsigned char *) b);
-}
-
-static int sel_cmp(Field *f, unsigned char *a, unsigned char *b, uint8_t a_flag, uint8_t b_flag);
-
 static unsigned char is_null_string[2]= {1,0};
 
 
@@ -195,8 +192,8 @@
   @param cost         OUT  The cost.
 */
 
-static void get_sweep_read_cost(Table *table, 
-                                ha_rows nrows, 
+static void get_sweep_read_cost(Table *table,
+                                ha_rows nrows,
                                 bool interrupted,
                                 COST_VECT *cost)
 {
@@ -227,479 +224,6 @@
   }
 }
 
-class RANGE_OPT_PARAM;
-/*
-  A construction block of the SEL_ARG-graph.
-
-  The following description only covers graphs of SEL_ARG objects with
-  sel_arg->type==KEY_RANGE:
-
-  One SEL_ARG object represents an "elementary interval" in form
-
-      min_value <=?  table.keypartX  <=? max_value
-
-  The interval is a non-empty interval of any kind: with[out] minimum/maximum
-  bound, [half]open/closed, single-point interval, etc.
-
-  1. SEL_ARG GRAPH STRUCTURE
-
-  SEL_ARG objects are linked together in a graph. The meaning of the graph
-  is better demostrated by an example:
-
-     tree->keys[i]
-      |
-      |             $              $
-      |    part=1   $     part=2   $    part=3
-      |             $              $
-      |  +-------+  $   +-------+  $   +--------+
-      |  | kp1<1 |--$-->| kp2=5 |--$-->| kp3=10 |
-      |  +-------+  $   +-------+  $   +--------+
-      |      |      $              $       |
-      |      |      $              $   +--------+
-      |      |      $              $   | kp3=12 |
-      |      |      $              $   +--------+
-      |  +-------+  $              $
-      \->| kp1=2 |--$--------------$-+
-         +-------+  $              $ |   +--------+
-             |      $              $  ==>| kp3=11 |
-         +-------+  $              $ |   +--------+
-         | kp1=3 |--$--------------$-+       |
-         +-------+  $              $     +--------+
-             |      $              $     | kp3=14 |
-            ...     $              $     +--------+
-
-  The entire graph is partitioned into "interval lists".
-
-  An interval list is a sequence of ordered disjoint intervals over the same
-  key part. SEL_ARG are linked via "next" and "prev" pointers. Additionally,
-  all intervals in the list form an RB-tree, linked via left/right/parent
-  pointers. The RB-tree root SEL_ARG object will be further called "root of the
-  interval list".
-
-    In the example pic, there are 4 interval lists:
-    "kp<1 OR kp1=2 OR kp1=3", "kp2=5", "kp3=10 OR kp3=12", "kp3=11 OR kp3=13".
-    The vertical lines represent SEL_ARG::next/prev pointers.
-
-  In an interval list, each member X may have SEL_ARG::next_key_part pointer
-  pointing to the root of another interval list Y. The pointed interval list
-  must cover a key part with greater number (i.e. Y->part > X->part).
-
-    In the example pic, the next_key_part pointers are represented by
-    horisontal lines.
-
-  2. SEL_ARG GRAPH SEMANTICS
-
-  It represents a condition in a special form (we don't have a name for it ATM)
-  The SEL_ARG::next/prev is "OR", and next_key_part is "AND".
-
-  For example, the picture represents the condition in form:
-   (kp1 < 1 AND kp2=5 AND (kp3=10 OR kp3=12)) OR
-   (kp1=2 AND (kp3=11 OR kp3=14)) OR
-   (kp1=3 AND (kp3=11 OR kp3=14))
-
-
-  3. SEL_ARG GRAPH USE
-
-  Use get_mm_tree() to construct SEL_ARG graph from WHERE condition.
-  Then walk the SEL_ARG graph and get a list of dijsoint ordered key
-  intervals (i.e. intervals in form
-
-   (constA1, .., const1_K) < (keypart1,.., keypartK) < (constB1, .., constB_K)
-
-  Those intervals can be used to access the index. The uses are in:
-   - check_quick_select() - Walk the SEL_ARG graph and find an estimate of
-                            how many table records are contained within all
-                            intervals.
-   - get_quick_select()   - Walk the SEL_ARG, materialize the key intervals,
-                            and create QUICK_RANGE_SELECT object that will
-                            read records within these intervals.
-
-  4. SPACE COMPLEXITY NOTES
-
-    SEL_ARG graph is a representation of an ordered disjoint sequence of
-    intervals over the ordered set of index tuple values.
-
-    For multi-part keys, one can construct a WHERE expression such that its
-    list of intervals will be of combinatorial size. Here is an example:
-
-      (keypart1 IN (1,2, ..., n1)) AND
-      (keypart2 IN (1,2, ..., n2)) AND
-      (keypart3 IN (1,2, ..., n3))
-
-    For this WHERE clause the list of intervals will have n1*n2*n3 intervals
-    of form
-
-      (keypart1, keypart2, keypart3) = (k1, k2, k3), where 1 <= k{i} <= n{i}
-
-    SEL_ARG graph structure aims to reduce the amount of required space by
-    "sharing" the elementary intervals when possible (the pic at the
-    beginning of this comment has examples of such sharing). The sharing may
-    prevent combinatorial blowup:
-
-      There are WHERE clauses that have combinatorial-size interval lists but
-      will be represented by a compact SEL_ARG graph.
-      Example:
-        (keypartN IN (1,2, ..., n1)) AND
-        ...
-        (keypart2 IN (1,2, ..., n2)) AND
-        (keypart1 IN (1,2, ..., n3))
-
-    but not in all cases:
-
-    - There are WHERE clauses that do have a compact SEL_ARG-graph
-      representation but get_mm_tree() and its callees will construct a
-      graph of combinatorial size.
-      Example:
-        (keypart1 IN (1,2, ..., n1)) AND
-        (keypart2 IN (1,2, ..., n2)) AND
-        ...
-        (keypartN IN (1,2, ..., n3))
-
-    - There are WHERE clauses for which the minimal possible SEL_ARG graph
-      representation will have combinatorial size.
-      Example:
-        By induction: Let's take any interval on some keypart in the middle:
-
-           kp15=c0
-
-        Then let's AND it with this interval 'structure' from preceding and
-        following keyparts:
-
-          (kp14=c1 AND kp16=c3) OR keypart14=c2) (*)
-
-        We will obtain this SEL_ARG graph:
-
-             kp14     $      kp15      $      kp16
-                      $                $
-         +---------+  $   +---------+  $   +---------+
-         | kp14=c1 |--$-->| kp15=c0 |--$-->| kp16=c3 |
-         +---------+  $   +---------+  $   +---------+
-              |       $                $
-         +---------+  $   +---------+  $
-         | kp14=c2 |--$-->| kp15=c0 |  $
-         +---------+  $   +---------+  $
-                      $                $
-
-       Note that we had to duplicate "kp15=c0" and there was no way to avoid
-       that.
-       The induction step: AND the obtained expression with another "wrapping"
-       expression like (*).
-       When the process ends because of the limit on max. number of keyparts
-       we'll have:
-
-         WHERE clause length  is O(3*#max_keyparts)
-         SEL_ARG graph size   is O(2^(#max_keyparts/2))
-
-       (it is also possible to construct a case where instead of 2 in 2^n we
-        have a bigger constant, e.g. 4, and get a graph with 4^(31/2)= 2^31
-        nodes)
-
-    We avoid consuming too much memory by setting a limit on the number of
-    SEL_ARG object we can construct during one range analysis invocation.
-*/
-
-class SEL_ARG :public Sql_alloc
-{
-public:
-  uint8_t min_flag,max_flag,maybe_flag;
-  uint8_t part;					// Which key part
-  uint8_t maybe_null;
-  /*
-    Number of children of this element in the RB-tree, plus 1 for this
-    element itself.
-  */
-  uint16_t elements;
-  /*
-    Valid only for elements which are RB-tree roots: Number of times this
-    RB-tree is referred to (it is referred by SEL_ARG::next_key_part or by
-    SEL_TREE::keys[i] or by a temporary SEL_ARG* variable)
-  */
-  ulong use_count;
-
-  Field *field;
-  unsigned char *min_value,*max_value;			// Pointer to range
-
-  /*
-    eq_tree() requires that left == right == 0 if the type is MAYBE_KEY.
-   */
-  SEL_ARG *left,*right;   /* R-B tree children */
-  SEL_ARG *next,*prev;    /* Links for bi-directional interval list */
-  SEL_ARG *parent;        /* R-B tree parent */
-  SEL_ARG *next_key_part;
-  enum leaf_color { BLACK,RED } color;
-  enum Type { IMPOSSIBLE, MAYBE, MAYBE_KEY, KEY_RANGE } type;
-
-  enum { MAX_SEL_ARGS = 16000 };
-
-  SEL_ARG() {}
-  SEL_ARG(SEL_ARG &);
-  SEL_ARG(Field *,const unsigned char *, const unsigned char *);
-  SEL_ARG(Field *field, uint8_t part, unsigned char *min_value, unsigned char *max_value,
-	  uint8_t min_flag, uint8_t max_flag, uint8_t maybe_flag);
-  SEL_ARG(enum Type type_arg)
-    :min_flag(0),elements(1),use_count(1),left(0),right(0),next_key_part(0),
-    color(BLACK), type(type_arg)
-  {}
-  inline bool is_same(SEL_ARG *arg)
-  {
-    if (type != arg->type || part != arg->part)
-      return 0;
-    if (type != KEY_RANGE)
-      return 1;
-    return cmp_min_to_min(arg) == 0 && cmp_max_to_max(arg) == 0;
-  }
-  inline void merge_flags(SEL_ARG *arg) { maybe_flag|=arg->maybe_flag; }
-  inline void maybe_smaller() { maybe_flag=1; }
-  /* Return true iff it's a single-point null interval */
-  inline bool is_null_interval() { return maybe_null && max_value[0] == 1; }
-  inline int cmp_min_to_min(SEL_ARG* arg)
-  {
-    return sel_cmp(field,min_value, arg->min_value, min_flag, arg->min_flag);
-  }
-  inline int cmp_min_to_max(SEL_ARG* arg)
-  {
-    return sel_cmp(field,min_value, arg->max_value, min_flag, arg->max_flag);
-  }
-  inline int cmp_max_to_max(SEL_ARG* arg)
-  {
-    return sel_cmp(field,max_value, arg->max_value, max_flag, arg->max_flag);
-  }
-  inline int cmp_max_to_min(SEL_ARG* arg)
-  {
-    return sel_cmp(field,max_value, arg->min_value, max_flag, arg->min_flag);
-  }
-  SEL_ARG *clone_and(SEL_ARG* arg)
-  {						// Get overlapping range
-    unsigned char *new_min,*new_max;
-    uint8_t flag_min,flag_max;
-    if (cmp_min_to_min(arg) >= 0)
-    {
-      new_min=min_value; flag_min=min_flag;
-    }
-    else
-    {
-      new_min=arg->min_value; flag_min=arg->min_flag;
-    }
-    if (cmp_max_to_max(arg) <= 0)
-    {
-      new_max=max_value; flag_max=max_flag;
-    }
-    else
-    {
-      new_max=arg->max_value; flag_max=arg->max_flag;
-    }
-    return new SEL_ARG(field, part, new_min, new_max, flag_min, flag_max,
-		       test(maybe_flag && arg->maybe_flag));
-  }
-  SEL_ARG *clone_first(SEL_ARG *arg)
-  {						// min <= X < arg->min
-    return new SEL_ARG(field,part, min_value, arg->min_value,
-		       min_flag, arg->min_flag & NEAR_MIN ? 0 : NEAR_MAX,
-		       maybe_flag | arg->maybe_flag);
-  }
-  SEL_ARG *clone_last(SEL_ARG *arg)
-  {						// min <= X <= key_max
-    return new SEL_ARG(field, part, min_value, arg->max_value,
-		       min_flag, arg->max_flag, maybe_flag | arg->maybe_flag);
-  }
-  SEL_ARG *clone(RANGE_OPT_PARAM *param, SEL_ARG *new_parent, SEL_ARG **next);
-
-  bool copy_min(SEL_ARG* arg)
-  {						// Get overlapping range
-    if (cmp_min_to_min(arg) > 0)
-    {
-      min_value=arg->min_value; min_flag=arg->min_flag;
-      if ((max_flag & (NO_MAX_RANGE | NO_MIN_RANGE)) ==
-	  (NO_MAX_RANGE | NO_MIN_RANGE))
-	return 1;				// Full range
-    }
-    maybe_flag|=arg->maybe_flag;
-    return 0;
-  }
-  bool copy_max(SEL_ARG* arg)
-  {						// Get overlapping range
-    if (cmp_max_to_max(arg) <= 0)
-    {
-      max_value=arg->max_value; max_flag=arg->max_flag;
-      if ((max_flag & (NO_MAX_RANGE | NO_MIN_RANGE)) ==
-	  (NO_MAX_RANGE | NO_MIN_RANGE))
-	return 1;				// Full range
-    }
-    maybe_flag|=arg->maybe_flag;
-    return 0;
-  }
-
-  void copy_min_to_min(SEL_ARG *arg)
-  {
-    min_value=arg->min_value; min_flag=arg->min_flag;
-  }
-  void copy_min_to_max(SEL_ARG *arg)
-  {
-    max_value=arg->min_value;
-    max_flag=arg->min_flag & NEAR_MIN ? 0 : NEAR_MAX;
-  }
-  void copy_max_to_min(SEL_ARG *arg)
-  {
-    min_value=arg->max_value;
-    min_flag=arg->max_flag & NEAR_MAX ? 0 : NEAR_MIN;
-  }
-  /* returns a number of keypart values (0 or 1) appended to the key buffer */
-  int store_min(uint32_t length, unsigned char **min_key,uint32_t min_key_flag)
-  {
-    /* "(kp1 > c1) AND (kp2 OP c2) AND ..." -> (kp1 > c1) */
-    if ((!(min_flag & NO_MIN_RANGE) &&
-	!(min_key_flag & (NO_MIN_RANGE | NEAR_MIN))))
-    {
-      if (maybe_null && *min_value)
-      {
-	**min_key=1;
-	memset(*min_key+1, 0, length-1);
-      }
-      else
-	memcpy(*min_key,min_value,length);
-      (*min_key)+= length;
-      return 1;
-    }
-    return 0;
-  }
-  /* returns a number of keypart values (0 or 1) appended to the key buffer */
-  int store_max(uint32_t length, unsigned char **max_key, uint32_t max_key_flag)
-  {
-    if (!(max_flag & NO_MAX_RANGE) &&
-	!(max_key_flag & (NO_MAX_RANGE | NEAR_MAX)))
-    {
-      if (maybe_null && *max_value)
-      {
-	**max_key=1;
-	memset(*max_key+1, 0, length-1);
-      }
-      else
-	memcpy(*max_key,max_value,length);
-      (*max_key)+= length;
-      return 1;
-    }
-    return 0;
-  }
-
-  /* returns a number of keypart values appended to the key buffer */
-  int store_min_key(KEY_PART *key, unsigned char **range_key, uint32_t *range_key_flag)
-  {
-    SEL_ARG *key_tree= first();
-    uint32_t res= key_tree->store_min(key[key_tree->part].store_length,
-                                  range_key, *range_key_flag);
-    *range_key_flag|= key_tree->min_flag;
-
-    if (key_tree->next_key_part &&
-	key_tree->next_key_part->part == key_tree->part+1 &&
-	!(*range_key_flag & (NO_MIN_RANGE | NEAR_MIN)) &&
-	key_tree->next_key_part->type == SEL_ARG::KEY_RANGE)
-      res+= key_tree->next_key_part->store_min_key(key, range_key,
-                                                   range_key_flag);
-    return res;
-  }
-
-  /* returns a number of keypart values appended to the key buffer */
-  int store_max_key(KEY_PART *key, unsigned char **range_key, uint32_t *range_key_flag)
-  {
-    SEL_ARG *key_tree= last();
-    uint32_t res=key_tree->store_max(key[key_tree->part].store_length,
-                                 range_key, *range_key_flag);
-    (*range_key_flag)|= key_tree->max_flag;
-    if (key_tree->next_key_part &&
-	key_tree->next_key_part->part == key_tree->part+1 &&
-	!(*range_key_flag & (NO_MAX_RANGE | NEAR_MAX)) &&
-	key_tree->next_key_part->type == SEL_ARG::KEY_RANGE)
-      res+= key_tree->next_key_part->store_max_key(key, range_key,
-                                                   range_key_flag);
-    return res;
-  }
-
-  SEL_ARG *insert(SEL_ARG *key);
-  SEL_ARG *tree_delete(SEL_ARG *key);
-  SEL_ARG *find_range(SEL_ARG *key);
-  SEL_ARG *rb_insert(SEL_ARG *leaf);
-  friend SEL_ARG *rb_delete_fixup(SEL_ARG *root,SEL_ARG *key, SEL_ARG *par);
-#ifdef EXTRA_DEBUG
-  friend int test_rb_tree(SEL_ARG *element,SEL_ARG *parent);
-  void test_use_count(SEL_ARG *root);
-#endif
-  SEL_ARG *first();
-  SEL_ARG *last();
-  void make_root();
-  inline bool simple_key()
-  {
-    return !next_key_part && elements == 1;
-  }
-  void increment_use_count(long count)
-  {
-    if (next_key_part)
-    {
-      next_key_part->use_count+=count;
-      count*= (next_key_part->use_count-count);
-      for (SEL_ARG *pos=next_key_part->first(); pos ; pos=pos->next)
-	if (pos->next_key_part)
-	  pos->increment_use_count(count);
-    }
-  }
-  void free_tree()
-  {
-    for (SEL_ARG *pos=first(); pos ; pos=pos->next)
-      if (pos->next_key_part)
-      {
-	pos->next_key_part->use_count--;
-	pos->next_key_part->free_tree();
-      }
-  }
-
-  inline SEL_ARG **parent_ptr()
-  {
-    return parent->left == this ? &parent->left : &parent->right;
-  }
-
-
-  /*
-    Check if this SEL_ARG object represents a single-point interval
-
-    SYNOPSIS
-      is_singlepoint()
-
-    DESCRIPTION
-      Check if this SEL_ARG object (not tree) represents a single-point
-      interval, i.e. if it represents a "keypart = const" or
-      "keypart IS NULL".
-
-    RETURN
-      true   This SEL_ARG object represents a singlepoint interval
-      false  Otherwise
-  */
-
-  bool is_singlepoint()
-  {
-    /*
-      Check for NEAR_MIN ("strictly less") and NO_MIN_RANGE (-inf < field)
-      flags, and the same for right edge.
-    */
-    if (min_flag || max_flag)
-      return false;
-    unsigned char *min_val= min_value;
-    unsigned char *max_val= max_value;
-
-    if (maybe_null)
-    {
-      /* First byte is a NULL value indicator */
-      if (*min_val != *max_val)
-        return false;
-
-      if (*min_val)
-        return true; /* This "x IS NULL" */
-      min_val++;
-      max_val++;
-    }
-    return !field->key_cmp(min_val, max_val);
-  }
-  SEL_ARG *clone_tree(RANGE_OPT_PARAM *param);
-};
-
 class SEL_IMERGE;
 
 
@@ -708,10 +232,18 @@
 public:
   /*
     Starting an effort to document this field:
-    (for some i, keys[i]->type == SEL_ARG::IMPOSSIBLE) =>
+    (for some i, keys[i]->type == optimizer::SEL_ARG::IMPOSSIBLE) =>
        (type == SEL_TREE::IMPOSSIBLE)
   */
-  enum Type { IMPOSSIBLE, ALWAYS, MAYBE, KEY, KEY_SMALLER } type;
+  enum Type
+  {
+    IMPOSSIBLE,
+    ALWAYS,
+    MAYBE,
+    KEY,
+    KEY_SMALLER
+  } type;
+
   SEL_TREE(enum Type type_arg) :type(type_arg) {}
   SEL_TREE() :type(KEY)
   {
@@ -724,7 +256,7 @@
     merit in range analyzer functions (e.g. get_mm_parts) returning a
     pointer to such SEL_TREE instead of NULL)
   */
-  SEL_ARG *keys[MAX_KEY];
+  optimizer::SEL_ARG *keys[MAX_KEY];
   key_map keys_map;        /* bitmask of non-NULL elements in keys */
 
   /*
@@ -735,79 +267,13 @@
 
   /* The members below are filled/used only after get_mm_tree is done */
   key_map ror_scans_map;   /* bitmask of ROR scan-able elements in keys */
-  uint32_t    n_ror_scans;     /* number of set bits in ror_scans_map */
+  uint32_t n_ror_scans;     /* number of set bits in ror_scans_map */
 
   struct st_ror_scan_info **ror_scans;     /* list of ROR key scans */
   struct st_ror_scan_info **ror_scans_end; /* last ROR scan */
   /* Note that #records for each key scan is stored in table->quick_rows */
 };
 
-class RANGE_OPT_PARAM
-{
-public:
-  Session	*session;   /* Current thread handle */
-  Table *table; /* Table being analyzed */
-  COND *cond;   /* Used inside get_mm_tree(). */
-  table_map prev_tables;
-  table_map read_tables;
-  table_map current_table; /* Bit of the table being analyzed */
-
-  /* Array of parts of all keys for which range analysis is performed */
-  KEY_PART *key_parts;
-  KEY_PART *key_parts_end;
-  MEM_ROOT *mem_root; /* Memory that will be freed when range analysis completes */
-  MEM_ROOT *old_root; /* Memory that will last until the query end */
-  /*
-    Number of indexes used in range analysis (In SEL_TREE::keys only first
-    #keys elements are not empty)
-  */
-  uint32_t keys;
-
-  /*
-    If true, the index descriptions describe real indexes (and it is ok to
-    call field->optimize_range(real_keynr[...], ...).
-    Otherwise index description describes fake indexes.
-  */
-  bool using_real_indexes;
-
-  bool remove_jump_scans;
-
-  /*
-    used_key_no -> table_key_no translation table. Only makes sense if
-    using_real_indexes==true
-  */
-  uint32_t real_keynr[MAX_KEY];
-  /* Number of SEL_ARG objects allocated by SEL_ARG::clone_tree operations */
-  uint32_t alloced_sel_args;
-  bool force_default_mrr;
-};
-
-class PARAM : public RANGE_OPT_PARAM
-{
-public:
-  KEY_PART *key[MAX_KEY]; /* First key parts of keys used in the query */
-  uint32_t max_key_part;
-  /* Number of ranges in the last checked tree->key */
-  uint32_t range_count;
-  unsigned char min_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH],
-    max_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
-  bool quick;				// Don't calulate possible keys
-
-  uint32_t fields_bitmap_size;
-  MyBitmap needed_fields;    /* bitmask of fields needed by the query */
-  MyBitmap tmp_covered_fields;
-
-  key_map *needed_reg;        /* ptr to SQL_SELECT::needed_reg */
-
-  uint32_t *imerge_cost_buff;     /* buffer for index_merge cost estimates */
-  uint32_t imerge_cost_buff_size; /* size of the buffer */
-
-  /* true if last checked tree->key can be used for ROR-scan */
-  bool is_ror_scan;
-  /* Number of ranges in the last checked tree->key */
-  uint32_t n_ranges;
-};
-
 class TABLE_READ_PLAN;
 class TRP_RANGE;
 class TRP_ROR_INTERSECT;
@@ -817,58 +283,92 @@
 
 struct st_ror_scan_info;
 
-static SEL_TREE * get_mm_parts(RANGE_OPT_PARAM *param,COND *cond_func,Field *field,
-			       Item_func::Functype type,Item *value,
-			       Item_result cmp_type);
-static SEL_ARG *get_mm_leaf(RANGE_OPT_PARAM *param,COND *cond_func,Field *field,
-			    KEY_PART *key_part,
-			    Item_func::Functype type,Item *value);
-static SEL_TREE *get_mm_tree(RANGE_OPT_PARAM *param,COND *cond);
-
-static bool is_key_scan_ror(PARAM *param, uint32_t keynr, uint8_t nparts);
-static ha_rows check_quick_select(PARAM *param, uint32_t idx, bool index_only,
-                                  SEL_ARG *tree, bool update_tbl_stats,
-                                  uint32_t *mrr_flags, uint32_t *bufsize,
+static SEL_TREE * get_mm_parts(optimizer::RangeParameter *param,
+                               COND *cond_func,
+                               Field *field,
+			                         Item_func::Functype type,
+                               Item *value,
+			                         Item_result cmp_type);
+
+static optimizer::SEL_ARG *get_mm_leaf(optimizer::RangeParameter *param,
+                                       COND *cond_func,
+                                       Field *field,
+                                       KEY_PART *key_part,
+			                                 Item_func::Functype type,
+                                       Item *value);
+
+static SEL_TREE *get_mm_tree(optimizer::RangeParameter *param, COND *cond);
+
+static bool is_key_scan_ror(optimizer::Parameter *param, uint32_t keynr, uint8_t nparts);
+
+static ha_rows check_quick_select(optimizer::Parameter *param,
+                                  uint32_t idx,
+                                  bool index_only,
+                                  optimizer::SEL_ARG *tree,
+                                  bool update_tbl_stats,
+                                  uint32_t *mrr_flags,
+                                  uint32_t *bufsize,
                                   COST_VECT *cost);
 
-static TRP_RANGE *get_key_scans_params(PARAM *param, SEL_TREE *tree,
+static TRP_RANGE *get_key_scans_params(optimizer::Parameter *param,
+                                       SEL_TREE *tree,
                                        bool index_read_must_be_used,
                                        bool update_tbl_stats,
                                        double read_time);
+
 static
-TRP_ROR_INTERSECT *get_best_ror_intersect(const PARAM *param, SEL_TREE *tree,
+TRP_ROR_INTERSECT *get_best_ror_intersect(const optimizer::Parameter *param,
+                                          SEL_TREE *tree,
                                           double read_time,
                                           bool *are_all_covering);
+
 static
-TRP_ROR_INTERSECT *get_best_covering_ror_intersect(PARAM *param,
+TRP_ROR_INTERSECT *get_best_covering_ror_intersect(optimizer::Parameter *param,
                                                    SEL_TREE *tree,
                                                    double read_time);
+
 static
-TABLE_READ_PLAN *get_best_disjunct_quick(PARAM *param, SEL_IMERGE *imerge,
+TABLE_READ_PLAN *get_best_disjunct_quick(optimizer::Parameter *param,
+                                         SEL_IMERGE *imerge,
                                          double read_time);
+
 static
-TRP_GROUP_MIN_MAX *get_best_group_min_max(PARAM *param, SEL_TREE *tree);
+TRP_GROUP_MIN_MAX *get_best_group_min_max(optimizer::Parameter *param, SEL_TREE *tree);
 
-static void print_sel_tree(PARAM *param, SEL_TREE *tree, key_map *tree_map,
+static void print_sel_tree(optimizer::Parameter *param,
+                           SEL_TREE *tree,
+                           key_map *tree_map,
                            const char *msg);
-static void print_ror_scans_arr(Table *table, const char *msg,
+
+static void print_ror_scans_arr(Table *table,
+                                const char *msg,
                                 struct st_ror_scan_info **start,
                                 struct st_ror_scan_info **end);
 
-static SEL_TREE *tree_and(RANGE_OPT_PARAM *param,SEL_TREE *tree1,SEL_TREE *tree2);
-static SEL_TREE *tree_or(RANGE_OPT_PARAM *param,SEL_TREE *tree1,SEL_TREE *tree2);
-static SEL_ARG *sel_add(SEL_ARG *key1,SEL_ARG *key2);
-static SEL_ARG *key_or(RANGE_OPT_PARAM *param, SEL_ARG *key1, SEL_ARG *key2);
-static SEL_ARG *key_and(RANGE_OPT_PARAM *param, SEL_ARG *key1, SEL_ARG *key2,
-                        uint32_t clone_flag);
-static bool get_range(SEL_ARG **e1,SEL_ARG **e2,SEL_ARG *root1);
-
-static bool eq_tree(SEL_ARG* a,SEL_ARG *b);
-
-static SEL_ARG null_element(SEL_ARG::IMPOSSIBLE);
-static bool null_part_in_key(KEY_PART *key_part, const unsigned char *key,
+static SEL_TREE *tree_and(optimizer::RangeParameter *param, SEL_TREE *tree1, SEL_TREE *tree2);
+
+static SEL_TREE *tree_or(optimizer::RangeParameter *param, SEL_TREE *tree1, SEL_TREE *tree2);
+
+static optimizer::SEL_ARG *sel_add(optimizer::SEL_ARG *key1, optimizer::SEL_ARG *key2);
+
+static optimizer::SEL_ARG *key_or(optimizer::RangeParameter *param, optimizer::SEL_ARG *key1, optimizer::SEL_ARG *key2);
+
+static optimizer::SEL_ARG *key_and(optimizer::RangeParameter *param,
+                                   optimizer::SEL_ARG *key1,
+                                   optimizer::SEL_ARG *key2,
+                                   uint32_t clone_flag);
+
+static bool get_range(optimizer::SEL_ARG **e1, optimizer::SEL_ARG **e2, optimizer::SEL_ARG *root1);
+
+static bool eq_tree(optimizer::SEL_ARG* a, optimizer::SEL_ARG *b);
+
+optimizer::SEL_ARG drizzled::optimizer::null_element(optimizer::SEL_ARG::IMPOSSIBLE);
+
+static bool null_part_in_key(KEY_PART *key_part,
+                             const unsigned char *key,
                              uint32_t length);
-bool sel_trees_can_be_ored(SEL_TREE *tree1, SEL_TREE *tree2, RANGE_OPT_PARAM* param);
+
+bool sel_trees_can_be_ored(SEL_TREE *tree1, SEL_TREE *tree2, optimizer::RangeParameter *param);
 
 
 /*
@@ -893,16 +393,16 @@
   SEL_TREE **trees_next;        /* last of these trees            */
   SEL_TREE **trees_end;         /* end of allocated space         */
 
-  SEL_ARG  ***best_keys;        /* best keys to read in SEL_TREEs */
+  optimizer::SEL_ARG  ***best_keys;        /* best keys to read in SEL_TREEs */
 
   SEL_IMERGE() :
     trees(&trees_prealloced[0]),
     trees_next(trees),
     trees_end(trees + PREALLOCED_TREES)
   {}
-  int or_sel_tree(RANGE_OPT_PARAM *param, SEL_TREE *tree);
-  int or_sel_tree_with_checks(RANGE_OPT_PARAM *param, SEL_TREE *new_tree);
-  int or_sel_imerge_with_checks(RANGE_OPT_PARAM *param, SEL_IMERGE* imerge);
+  int or_sel_tree(optimizer::RangeParameter *param, SEL_TREE *tree);
+  int or_sel_tree_with_checks(optimizer::RangeParameter *param, SEL_TREE *new_tree);
+  int or_sel_imerge_with_checks(optimizer::RangeParameter *param, SEL_IMERGE* imerge);
 };
 
 
@@ -917,8 +417,7 @@
      0 - OK
     -1 - Out of memory.
 */
-
-int SEL_IMERGE::or_sel_tree(RANGE_OPT_PARAM *param, SEL_TREE *tree)
+int SEL_IMERGE::or_sel_tree(optimizer::RangeParameter *param, SEL_TREE *tree)
 {
   if (trees_next == trees_end)
   {
@@ -930,9 +429,9 @@
     if (!(new_trees= (SEL_TREE**)alloc_root(param->mem_root, new_size)))
       return -1;
     memcpy(new_trees, trees, old_size);
-    trees=      new_trees;
+    trees= new_trees;
     trees_next= trees + old_elements;
-    trees_end=  trees + old_elements * realloc_ratio;
+    trees_end= trees + old_elements * realloc_ratio;
   }
   *(trees_next++)= tree;
   return 0;
@@ -946,7 +445,7 @@
 
   SYNOPSIS
     or_sel_tree_with_checks()
-      param    PARAM from SQL_SELECT::test_quick_select
+      param    Parameter from SqlSelect::test_quick_select
       new_tree SEL_TREE with type KEY or KEY_SMALLER.
 
   NOTES
@@ -968,8 +467,7 @@
        and (*this) should be discarded.
    -1  An error occurred.
 */
-
-int SEL_IMERGE::or_sel_tree_with_checks(RANGE_OPT_PARAM *param, SEL_TREE *new_tree)
+int SEL_IMERGE::or_sel_tree_with_checks(optimizer::RangeParameter *param, SEL_TREE *new_tree)
 {
   for (SEL_TREE** tree = trees;
        tree != trees_next;
@@ -1003,7 +501,7 @@
    -1 - An error occurred
 */
 
-int SEL_IMERGE::or_sel_imerge_with_checks(RANGE_OPT_PARAM *param, SEL_IMERGE* imerge)
+int SEL_IMERGE::or_sel_imerge_with_checks(optimizer::RangeParameter *param, SEL_IMERGE* imerge)
 {
   for (SEL_TREE** tree= imerge->trees;
        tree != imerge->trees_next;
@@ -1049,7 +547,7 @@
     other Error, both passed lists are unusable
 */
 
-static int imerge_list_or_list(RANGE_OPT_PARAM *param,
+static int imerge_list_or_list(optimizer::RangeParameter *param,
                                List<SEL_IMERGE> *im1,
                                List<SEL_IMERGE> *im2)
 {
@@ -1068,8 +566,8 @@
      0     OK, result is stored in *im1.
      other Error
  */
- 
-static int imerge_list_or_tree(RANGE_OPT_PARAM *param,
+
+static int imerge_list_or_tree(optimizer::RangeParameter *param,
                                List<SEL_IMERGE> *im1,
                                SEL_TREE *tree)
 {
@@ -1085,7 +583,7 @@
 
 
 /***************************************************************************
-** Basic functions for SQL_SELECT and QUICK_RANGE_SELECT
+** Basic functions for SqlSelect and QuickRangeSelect
 ***************************************************************************/
 
 	/* make a select from mysql info
@@ -1094,14 +592,14 @@
 	   1 = Got some error (out of memory?)
 	   */
 
-optimizer::SQL_SELECT *optimizer::make_select(Table *head, 
+optimizer::SqlSelect *optimizer::make_select(Table *head,
                                               table_map const_tables,
-                                              table_map read_tables, 
+                                              table_map read_tables,
                                               COND *conds,
                                               bool allow_null_cond,
                                               int *error)
 {
-  optimizer::SQL_SELECT *select= NULL;
+  optimizer::SqlSelect *select= NULL;
 
   *error= 0;
 
@@ -1109,7 +607,7 @@
   {
     return 0;
   }
-  if (! (select= new optimizer::SQL_SELECT))
+  if (! (select= new optimizer::SqlSelect))
   {
     *error= 1;			// out of memory
     return 0;
@@ -1131,15 +629,15 @@
 }
 
 
-optimizer::SQL_SELECT::SQL_SELECT() :quick(0),cond(0),free_cond(0)
+optimizer::SqlSelect::SqlSelect() :quick(0),cond(0),free_cond(0)
 {
-  quick_keys.reset(); 
+  quick_keys.reset();
   needed_reg.reset();
   my_b_clear(&file);
 }
 
 
-void optimizer::SQL_SELECT::cleanup()
+void optimizer::SqlSelect::cleanup()
 {
   delete quick;
   quick= 0;
@@ -1153,13 +651,13 @@
 }
 
 
-optimizer::SQL_SELECT::~SQL_SELECT()
+optimizer::SqlSelect::~SqlSelect()
 {
   cleanup();
 }
 
 
-bool optimizer::SQL_SELECT::check_quick(Session *session, bool force_quick_range,
+bool optimizer::SqlSelect::check_quick(Session *session, bool force_quick_range,
                              ha_rows limit)
 {
   key_map tmp;
@@ -1169,186 +667,27 @@
 }
 
 
-bool optimizer::SQL_SELECT::skip_record()
+bool optimizer::SqlSelect::skip_record()
 {
   return cond ? cond->val_int() == 0 : 0;
 }
 
 
-optimizer::QUICK_SELECT_I::QUICK_SELECT_I()
+optimizer::QuickSelectInterface::QuickSelectInterface()
   :
     max_used_key_length(0),
     used_key_parts(0)
 {}
 
-optimizer::QUICK_RANGE_SELECT::QUICK_RANGE_SELECT(Session *session, 
-                                                  Table *table, 
-                                                  uint32_t key_nr,
-                                                  bool no_alloc, 
-                                                  MEM_ROOT *parent_alloc,
-                                                  bool *create_error)
-  :
-    free_file(0),
-    cur_range(NULL),
-    last_range(0),
-    dont_free(0)
-{
-  my_bitmap_map *bitmap= NULL;
-
-  in_ror_merged_scan= 0;
-  sorted= 0;
-  index= key_nr;
-  head= table;
-  key_part_info= head->key_info[index].key_part;
-  my_init_dynamic_array(&ranges, sizeof(optimizer::QUICK_RANGE*), 16, 16);
-
-  /* 'session' is not accessible in QUICK_RANGE_SELECT::reset(). */
-  mrr_buf_size= session->variables.read_rnd_buff_size;
-  mrr_buf_desc= NULL;
-
-  if (!no_alloc && !parent_alloc)
-  {
-    // Allocates everything through the internal memroot
-    init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
-    session->mem_root= &alloc;
-  }
-  else
-    memset(&alloc, 0, sizeof(alloc));
-  cursor= head->cursor;
-  record= head->record[0];
-  save_read_set= head->read_set;
-  save_write_set= head->write_set;
-
-  /* Allocate a bitmap for used columns. Using sql_alloc instead of malloc
-     simply as a "fix" to the MySQL 6.0 code that also free()s it at the
-     same time we destroy the mem_root.
-   */
-
-  bitmap= reinterpret_cast<my_bitmap_map*>(sql_alloc(head->s->column_bitmap_size));
-  if (! bitmap)
-  {
-    column_bitmap.setBitmap(NULL);
-    *create_error= 1;
-  }
-  else
-  {
-    column_bitmap.init(bitmap, head->s->fields);
-  }
-}
-
-
-int optimizer::QUICK_RANGE_SELECT::init()
-{
-  if (cursor->inited != Cursor::NONE)
-    cursor->ha_index_or_rnd_end();
-  return (cursor->ha_index_init(index, 1));
-}
-
-
-void optimizer::QUICK_RANGE_SELECT::range_end()
-{
-  if (cursor->inited != Cursor::NONE)
-    cursor->ha_index_or_rnd_end();
-}
-
-
-optimizer::QUICK_RANGE_SELECT::~QUICK_RANGE_SELECT()
-{
-  if (!dont_free)
-  {
-    /* cursor is NULL for CPK scan on covering ROR-intersection */
-    if (cursor)
-    {
-      range_end();
-      if (head->key_read)
-      {
-        head->key_read= 0;
-        cursor->extra(HA_EXTRA_NO_KEYREAD);
-      }
-      if (free_file)
-      {
-        cursor->ha_external_lock(current_session, F_UNLCK);
-        cursor->close();
-        delete cursor;
-      }
-    }
-    delete_dynamic(&ranges); /* ranges are allocated in alloc */
-    free_root(&alloc,MYF(0));
-  }
-  head->column_bitmaps_set(save_read_set, save_write_set);
-  assert(mrr_buf_desc == NULL);
-  if (mrr_buf_desc)
-  {
-    free(mrr_buf_desc);
-  }
-}
-
-
-optimizer::QUICK_INDEX_MERGE_SELECT::QUICK_INDEX_MERGE_SELECT(Session *session_param,
-                                                              Table *table)
-  :
-    pk_quick_select(NULL), 
-    session(session_param)
-{
-  index= MAX_KEY;
-  head= table;
-  memset(&read_record, 0, sizeof(read_record));
-  init_sql_alloc(&alloc, session->variables.range_alloc_block_size, 0);
-  return;
-}
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::init()
-{
-  return 0;
-}
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::reset()
-{
-  return (read_keys_and_merge());
-}
-
-bool
-optimizer::QUICK_INDEX_MERGE_SELECT::push_quick_back(optimizer::QUICK_RANGE_SELECT *quick_sel_range)
-{
-  /*
-    Save quick_select that does scan on clustered primary key as it will be
-    processed separately.
-  */
-  if (head->cursor->primary_key_is_clustered() &&
-      quick_sel_range->index == head->s->primary_key)
-  {
-    pk_quick_select= quick_sel_range;
-  }
-  else
-  {
-    return quick_selects.push_back(quick_sel_range);
-  }
-  return 0;
-}
-
-optimizer::QUICK_INDEX_MERGE_SELECT::~QUICK_INDEX_MERGE_SELECT()
-{
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> quick_it(quick_selects);
-  optimizer::QUICK_RANGE_SELECT* quick;
-  quick_it.rewind();
-  while ((quick= quick_it++))
-  {
-    quick->cursor= NULL;
-  }
-  quick_selects.delete_elements();
-  delete pk_quick_select;
-  free_root(&alloc,MYF(0));
-  return;
-}
 
 
 optimizer::QUICK_ROR_INTERSECT_SELECT::QUICK_ROR_INTERSECT_SELECT(Session *session_param,
                                                                   Table *table,
                                                                   bool retrieve_full_rows,
                                                                   MEM_ROOT *parent_alloc)
-  : 
-    cpk_quick(NULL), 
-    session(session_param), 
+  :
+    cpk_quick(NULL),
+    session(session_param),
     need_to_fetch_row(retrieve_full_rows),
     scans_inited(false)
 {
@@ -1386,116 +725,6 @@
 
 
 /*
-  Initialize this quick select to be a ROR-merged scan.
-
-  SYNOPSIS
-    QUICK_RANGE_SELECT::init_ror_merged_scan()
-      reuse_handler If true, use head->cursor, otherwise create a separate
-                    Cursor object
-
-  NOTES
-    This function creates and prepares for subsequent use a separate Cursor
-    object if it can't reuse head->cursor. The reason for this is that during
-    ROR-merge several key scans are performed simultaneously, and a single
-    Cursor is only capable of preserving context of a single key scan.
-
-    In ROR-merge the quick select doing merge does full records retrieval,
-    merged quick selects read only keys.
-
-  RETURN
-    0  ROR child scan initialized, ok to use.
-    1  error
-*/
-
-int optimizer::QUICK_RANGE_SELECT::init_ror_merged_scan(bool reuse_handler)
-{
-  Cursor *save_file= cursor, *org_file;
-  Session *session;
-
-  in_ror_merged_scan= 1;
-  if (reuse_handler)
-  {
-    if (init() || reset())
-    {
-      return 0;
-    }
-    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-    goto end;
-  }
-
-  /* Create a separate Cursor object for this quick select */
-  if (free_file)
-  {
-    /* already have own 'Cursor' object. */
-    return 0;
-  }
-
-  session= head->in_use;
-  if (! (cursor= head->cursor->clone(session->mem_root)))
-  {
-    /*
-      Manually set the error flag. Note: there seems to be quite a few
-      places where a failure could cause the server to "hang" the client by
-      sending no response to a query. ATM those are not real errors because
-      the storage engine calls in question happen to never fail with the
-      existing storage engines.
-    */
-    my_error(ER_OUT_OF_RESOURCES, MYF(0));
-    /* Caller will free the memory */
-    goto failure;
-  }
-
-  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-
-  if (cursor->ha_external_lock(session, F_RDLCK))
-    goto failure;
-
-  if (init() || reset())
-  {
-    cursor->ha_external_lock(session, F_UNLCK);
-    cursor->close();
-    goto failure;
-  }
-  free_file= true;
-  last_rowid= cursor->ref;
-
-end:
-  /*
-    We are only going to read key fields and call position() on 'cursor'
-    The following sets head->tmp_set to only use this key and then updates
-    head->read_set and head->write_set to use this bitmap.
-    The now bitmap is stored in 'column_bitmap' which is used in ::get_next()
-  */
-  org_file= head->cursor;
-  head->cursor= cursor;
-  /* We don't have to set 'head->keyread' here as the 'cursor' is unique */
-  if (! head->no_keyread)
-  {
-    head->key_read= 1;
-    head->mark_columns_used_by_index(index);
-  }
-  head->prepare_for_position();
-  head->cursor= org_file;
-  column_bitmap= *head->read_set;
-  head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-
-  return 0;
-
-failure:
-  head->column_bitmaps_set(save_read_set, save_write_set);
-  delete cursor;
-  cursor= save_file;
-  return 0;
-}
-
-
-void optimizer::QUICK_RANGE_SELECT::save_last_pos()
-{
-  cursor->position(record);
-}
-
-
-/*
   Initialize this quick select to be a part of a ROR-merged scan.
   SYNOPSIS
     QUICK_ROR_INTERSECT_SELECT::init_ror_merged_scan()
@@ -1507,8 +736,8 @@
 */
 int optimizer::QUICK_ROR_INTERSECT_SELECT::init_ror_merged_scan(bool reuse_handler)
 {
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> quick_it(quick_selects);
-  optimizer::QUICK_RANGE_SELECT* quick;
+  List_iterator_fast<optimizer::QuickRangeSelect> quick_it(quick_selects);
+  optimizer::QuickRangeSelect *quick= NULL;
 
   /* Initialize all merged "children" quick selects */
   assert(!need_to_fetch_row || reuse_handler);
@@ -1558,8 +787,8 @@
     return 0;
   }
   scans_inited= true;
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> it(quick_selects);
-  optimizer::QUICK_RANGE_SELECT *quick;
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
+  optimizer::QuickRangeSelect *quick= NULL;
   while ((quick= it++))
   {
     quick->reset();
@@ -1584,7 +813,7 @@
 */
 
 bool
-optimizer::QUICK_ROR_INTERSECT_SELECT::push_quick_back(optimizer::QUICK_RANGE_SELECT *quick)
+optimizer::QUICK_ROR_INTERSECT_SELECT::push_quick_back(optimizer::QuickRangeSelect *quick)
 {
   return quick_selects.push_back(quick);
 }
@@ -1604,8 +833,8 @@
 
 optimizer::QUICK_ROR_UNION_SELECT::QUICK_ROR_UNION_SELECT(Session *session_param,
                                                           Table *table)
-  : 
-    session(session_param), 
+  :
+    session(session_param),
     scans_inited(false)
 {
   index= MAX_KEY;
@@ -1627,7 +856,7 @@
   public:
   compare_functor(optimizer::QUICK_ROR_UNION_SELECT *in_arg)
     : self(in_arg) { }
-  inline bool operator()(const optimizer::QUICK_SELECT_I *i, const optimizer::QUICK_SELECT_I *j) const
+  inline bool operator()(const optimizer::QuickSelectInterface *i, const optimizer::QuickSelectInterface *j) const
   {
     int val= self->head->cursor->cmp_ref(i->last_rowid,
                                          j->last_rowid);
@@ -1647,9 +876,9 @@
 
 int optimizer::QUICK_ROR_UNION_SELECT::init()
 {
-  queue= 
-    new priority_queue<optimizer::QUICK_SELECT_I *, 
-                       vector<optimizer::QUICK_SELECT_I *>, 
+  queue=
+    new priority_queue<optimizer::QuickSelectInterface *,
+                       vector<optimizer::QuickSelectInterface *>,
                        optimizer::compare_functor >(optimizer::compare_functor(this));
   if (! (cur_rowid= (unsigned char*) alloc_root(&alloc, 2*head->cursor->ref_length)))
   {
@@ -1672,12 +901,12 @@
 
 int optimizer::QUICK_ROR_UNION_SELECT::reset()
 {
-  QUICK_SELECT_I *quick= NULL;
+  QuickSelectInterface *quick= NULL;
   int error;
   have_prev_rowid= false;
   if (! scans_inited)
   {
-    List_iterator_fast<QUICK_SELECT_I> it(quick_selects);
+    List_iterator_fast<QuickSelectInterface> it(quick_selects);
     while ((quick= it++))
     {
       if (quick->init_ror_merged_scan(false))
@@ -1695,7 +924,7 @@
     Initialize scans for merged quick selects and put all merged quick
     selects into the queue.
   */
-  List_iterator_fast<QUICK_SELECT_I> it(quick_selects);
+  List_iterator_fast<QuickSelectInterface> it(quick_selects);
   while ((quick= it++))
   {
     if (quick->reset())
@@ -1724,7 +953,7 @@
 
 
 bool
-optimizer::QUICK_ROR_UNION_SELECT::push_quick_back(QUICK_SELECT_I *quick_sel_range)
+optimizer::QUICK_ROR_UNION_SELECT::push_quick_back(QuickSelectInterface *quick_sel_range)
 {
   return quick_selects.push_back(quick_sel_range);
 }
@@ -1746,188 +975,6 @@
 }
 
 
-optimizer::QUICK_RANGE::QUICK_RANGE()
-  :
-    min_key(0),
-    max_key(0),
-    min_length(0),
-    max_length(0),
-    flag(NO_MIN_RANGE | NO_MAX_RANGE),
-    min_keypart_map(0), 
-    max_keypart_map(0)
-{}
-
-SEL_ARG::SEL_ARG(SEL_ARG &arg) :Sql_alloc()
-{
-  type=arg.type;
-  min_flag=arg.min_flag;
-  max_flag=arg.max_flag;
-  maybe_flag=arg.maybe_flag;
-  maybe_null=arg.maybe_null;
-  part=arg.part;
-  field=arg.field;
-  min_value=arg.min_value;
-  max_value=arg.max_value;
-  next_key_part=arg.next_key_part;
-  use_count=1; elements=1;
-}
-
-
-inline void SEL_ARG::make_root()
-{
-  left=right= &null_element;
-  color=BLACK;
-  next=prev=0;
-  use_count=0; elements=1;
-}
-
-SEL_ARG::SEL_ARG(Field *f,const unsigned char *min_value_arg,
-                 const unsigned char *max_value_arg)
-  :min_flag(0), max_flag(0), maybe_flag(0), maybe_null(f->real_maybe_null()),
-   elements(1), use_count(1), field(f), min_value((unsigned char*) min_value_arg),
-   max_value((unsigned char*) max_value_arg), next(0),prev(0),
-   next_key_part(0),color(BLACK),type(KEY_RANGE)
-{
-  left=right= &null_element;
-}
-
-SEL_ARG::SEL_ARG(Field *field_,uint8_t part_,
-                 unsigned char *min_value_, unsigned char *max_value_,
-		 uint8_t min_flag_,uint8_t max_flag_,uint8_t maybe_flag_)
-  :min_flag(min_flag_),max_flag(max_flag_),maybe_flag(maybe_flag_),
-   part(part_),maybe_null(field_->real_maybe_null()), elements(1),use_count(1),
-   field(field_), min_value(min_value_), max_value(max_value_),
-   next(0),prev(0),next_key_part(0),color(BLACK),type(KEY_RANGE)
-{
-  left=right= &null_element;
-}
-
-SEL_ARG *SEL_ARG::clone(RANGE_OPT_PARAM *param, SEL_ARG *new_parent,
-                        SEL_ARG **next_arg)
-{
-  SEL_ARG *tmp;
-
-  /* Bail out if we have already generated too many SEL_ARGs */
-  if (++param->alloced_sel_args > MAX_SEL_ARGS)
-    return 0;
-
-  if (type != KEY_RANGE)
-  {
-    if (!(tmp= new (param->mem_root) SEL_ARG(type)))
-      return 0;					// out of memory
-    tmp->prev= *next_arg;			// Link into next/prev chain
-    (*next_arg)->next=tmp;
-    (*next_arg)= tmp;
-  }
-  else
-  {
-    if (!(tmp= new (param->mem_root) SEL_ARG(field,part, min_value,max_value,
-                                             min_flag, max_flag, maybe_flag)))
-      return 0;					// OOM
-    tmp->parent=new_parent;
-    tmp->next_key_part=next_key_part;
-    if (left != &null_element)
-      if (!(tmp->left=left->clone(param, tmp, next_arg)))
-	return 0;				// OOM
-
-    tmp->prev= *next_arg;			// Link into next/prev chain
-    (*next_arg)->next=tmp;
-    (*next_arg)= tmp;
-
-    if (right != &null_element)
-      if (!(tmp->right= right->clone(param, tmp, next_arg)))
-	return 0;				// OOM
-  }
-  increment_use_count(1);
-  tmp->color= color;
-  tmp->elements= this->elements;
-  return tmp;
-}
-
-SEL_ARG *SEL_ARG::first()
-{
-  SEL_ARG *next_arg=this;
-  if (!next_arg->left)
-    return 0;					// MAYBE_KEY
-  while (next_arg->left != &null_element)
-    next_arg=next_arg->left;
-  return next_arg;
-}
-
-SEL_ARG *SEL_ARG::last()
-{
-  SEL_ARG *next_arg=this;
-  if (!next_arg->right)
-    return 0;					// MAYBE_KEY
-  while (next_arg->right != &null_element)
-    next_arg=next_arg->right;
-  return next_arg;
-}
-
-
-/*
-  Check if a compare is ok, when one takes ranges in account
-  Returns -2 or 2 if the ranges where 'joined' like  < 2 and >= 2
-*/
-
-static int sel_cmp(Field *field, unsigned char *a, unsigned char *b, uint8_t a_flag,
-                   uint8_t b_flag)
-{
-  int cmp;
-  /* First check if there was a compare to a min or max element */
-  if (a_flag & (NO_MIN_RANGE | NO_MAX_RANGE))
-  {
-    if ((a_flag & (NO_MIN_RANGE | NO_MAX_RANGE)) ==
-	(b_flag & (NO_MIN_RANGE | NO_MAX_RANGE)))
-      return 0;
-    return (a_flag & NO_MIN_RANGE) ? -1 : 1;
-  }
-  if (b_flag & (NO_MIN_RANGE | NO_MAX_RANGE))
-    return (b_flag & NO_MIN_RANGE) ? 1 : -1;
-
-  if (field->real_maybe_null())			// If null is part of key
-  {
-    if (*a != *b)
-    {
-      return *a ? -1 : 1;
-    }
-    if (*a)
-      goto end;					// NULL where equal
-    a++; b++;					// Skip NULL marker
-  }
-  cmp=field->key_cmp(a , b);
-  if (cmp) return cmp < 0 ? -1 : 1;		// The values differed
-
-  // Check if the compared equal arguments was defined with open/closed range
- end:
-  if (a_flag & (NEAR_MIN | NEAR_MAX))
-  {
-    if ((a_flag & (NEAR_MIN | NEAR_MAX)) == (b_flag & (NEAR_MIN | NEAR_MAX)))
-      return 0;
-    if (!(b_flag & (NEAR_MIN | NEAR_MAX)))
-      return (a_flag & NEAR_MIN) ? 2 : -2;
-    return (a_flag & NEAR_MIN) ? 1 : -1;
-  }
-  if (b_flag & (NEAR_MIN | NEAR_MAX))
-    return (b_flag & NEAR_MIN) ? -2 : 2;
-  return 0;					// The elements where equal
-}
-
-
-SEL_ARG *SEL_ARG::clone_tree(RANGE_OPT_PARAM *param)
-{
-  SEL_ARG tmp_link,*next_arg,*root;
-  next_arg= &tmp_link;
-  if (!(root= clone(param, (SEL_ARG *) 0, &next_arg)))
-    return 0;
-  next_arg->next=0;				// Fix last link
-  tmp_link.next->prev=0;			// Fix first link
-  if (root)					// If not OOM
-    root->use_count= 0;
-  return root;
-}
-
-
 /*
   Find the best index to retrieve first N records in given order
 
@@ -1985,12 +1032,12 @@
     for (ord= order; ord && partno < n_parts; ord= ord->next, partno++)
     {
       Item *item= order->item[0];
-      if (!(item->type() == Item::FIELD_ITEM &&
+      if (! (item->type() == Item::FIELD_ITEM &&
            ((Item_field*)item)->field->eq(keyinfo[partno].field)))
         break;
     }
 
-    if (!ord && table->key_info[idx].key_length < match_key_len)
+    if (! ord && table->key_info[idx].key_length < match_key_len)
     {
       /*
         Ok, the ordering is compatible and this key is shorter then
@@ -2019,7 +1066,7 @@
 
 
 /*
-  Table rows retrieval plan. Range optimizer creates QUICK_SELECT_I-derived
+  Table rows retrieval plan. Range optimizer creates QuickSelectInterface-derived
   objects from table read plans.
 */
 class TABLE_READ_PLAN
@@ -2054,9 +1101,9 @@
       created quick select
       NULL on any error.
   */
-  virtual optimizer::QUICK_SELECT_I *make_quick(PARAM *param,
-                                                bool retrieve_full_rows,
-                                                MEM_ROOT *parent_alloc=NULL) = 0;
+  virtual optimizer::QuickSelectInterface *make_quick(optimizer::Parameter *param,
+                                                      bool retrieve_full_rows,
+                                                      MEM_ROOT *parent_alloc=NULL) = 0;
 
   /* Table read plans are allocated on MEM_ROOT and are never deleted */
   static void *operator new(size_t size, MEM_ROOT *mem_root)
@@ -2075,32 +1122,35 @@
 
 
 /*
-  Plan for a QUICK_RANGE_SELECT scan.
+  Plan for a QuickRangeSelect scan.
   TRP_RANGE::make_quick ignores retrieve_full_rows parameter because
-  QUICK_RANGE_SELECT doesn't distinguish between 'index only' scans and full
+  QuickRangeSelect doesn't distinguish between 'index only' scans and full
   record retrieval scans.
 */
 
 class TRP_RANGE : public TABLE_READ_PLAN
 {
 public:
-  SEL_ARG *key; /* set of intervals to be used in "range" method retrieval */
-  uint32_t     key_idx; /* key number in PARAM::key */
+  optimizer::SEL_ARG *key; /* set of intervals to be used in "range" method retrieval */
+  uint32_t     key_idx; /* key number in Parameter::key */
   uint32_t     mrr_flags;
   uint32_t     mrr_buf_size;
 
-  TRP_RANGE(SEL_ARG *key_arg, uint32_t idx_arg, uint32_t mrr_flags_arg)
-   : key(key_arg), key_idx(idx_arg), mrr_flags(mrr_flags_arg)
+  TRP_RANGE(optimizer::SEL_ARG *key_arg, uint32_t idx_arg, uint32_t mrr_flags_arg)
+    :
+      key(key_arg),
+      key_idx(idx_arg),
+      mrr_flags(mrr_flags_arg)
   {}
   virtual ~TRP_RANGE() {}                     /* Remove gcc warning */
 
-  optimizer::QUICK_SELECT_I *make_quick(PARAM *param, bool, MEM_ROOT *parent_alloc)
+  optimizer::QuickSelectInterface *make_quick(optimizer::Parameter *param, bool, MEM_ROOT *parent_alloc)
   {
-    optimizer::QUICK_RANGE_SELECT *quick;
-    if ((quick= optimizer::get_quick_select(param, 
-                                            key_idx, 
-                                            key, 
-                                            mrr_flags, 
+    optimizer::QuickRangeSelect *quick= NULL;
+    if ((quick= optimizer::get_quick_select(param,
+                                            key_idx,
+                                            key,
+                                            mrr_flags,
                                             mrr_buf_size,
                                             parent_alloc)))
     {
@@ -2119,9 +1169,9 @@
 public:
   TRP_ROR_INTERSECT() {}                      /* Remove gcc warning */
   virtual ~TRP_ROR_INTERSECT() {}             /* Remove gcc warning */
-  optimizer::QUICK_SELECT_I *make_quick(PARAM *param, 
-                                        bool retrieve_full_rows,
-                                        MEM_ROOT *parent_alloc);
+  optimizer::QuickSelectInterface *make_quick(optimizer::Parameter *param,
+                                              bool retrieve_full_rows,
+                                              MEM_ROOT *parent_alloc);
 
   /* Array of pointers to ROR range scans used in this intersection */
   struct st_ror_scan_info **first_scan;
@@ -2143,16 +1193,16 @@
 public:
   TRP_ROR_UNION() {}                          /* Remove gcc warning */
   virtual ~TRP_ROR_UNION() {}                 /* Remove gcc warning */
-  optimizer::QUICK_SELECT_I *make_quick(PARAM *param, 
-                                        bool retrieve_full_rows,
-                                        MEM_ROOT *parent_alloc);
+  optimizer::QuickSelectInterface *make_quick(optimizer::Parameter *param,
+                                              bool retrieve_full_rows,
+                                              MEM_ROOT *parent_alloc);
   TABLE_READ_PLAN **first_ror; /* array of ptrs to plans for merged scans */
   TABLE_READ_PLAN **last_ror;  /* end of the above array */
 };
 
 
 /*
-  Plan for QUICK_INDEX_MERGE_SELECT scan.
+  Plan for QuickIndexMergeSelect scan.
   QUICK_ROR_INTERSECT_SELECT always retrieves full rows, so retrieve_full_rows
   is ignored by make_quick.
 */
@@ -2162,9 +1212,9 @@
 public:
   TRP_INDEX_MERGE() {}                        /* Remove gcc warning */
   virtual ~TRP_INDEX_MERGE() {}               /* Remove gcc warning */
-  optimizer::QUICK_SELECT_I *make_quick(PARAM *param, 
-                                        bool retrieve_full_rows,
-                                        MEM_ROOT *parent_alloc);
+  optimizer::QuickSelectInterface *make_quick(optimizer::Parameter *param,
+                                              bool retrieve_full_rows,
+                                              MEM_ROOT *parent_alloc);
   TRP_RANGE **range_scans; /* array of ptrs to plans of merged scans */
   TRP_RANGE **range_scans_end; /* end of the array */
 };
@@ -2187,7 +1237,7 @@
   uint32_t key_infix_len;
   unsigned char key_infix[MAX_KEY_LENGTH];
   SEL_TREE *range_tree; /* Represents all range predicates in the query. */
-  SEL_ARG  *index_tree; /* The SEL_ARG sub-tree corresponding to index_info. */
+  optimizer::SEL_ARG  *index_tree; /* The SEL_ARG sub-tree corresponding to index_info. */
   uint32_t param_idx; /* Index of used key in param->key. */
   /* Number of records selected by the ranges in index_tree. */
 public:
@@ -2199,24 +1249,31 @@
                     uint32_t group_key_parts_arg, KEY *index_info_arg,
                     uint32_t index_arg, uint32_t key_infix_len_arg,
                     unsigned char *key_infix_arg,
-                    SEL_TREE *tree_arg, SEL_ARG *index_tree_arg,
+                    SEL_TREE *tree_arg, optimizer::SEL_ARG *index_tree_arg,
                     uint32_t param_idx_arg, ha_rows quick_prefix_records_arg)
-  : have_min(have_min_arg), have_max(have_max_arg),
-    min_max_arg_part(min_max_arg_part_arg),
-    group_prefix_len(group_prefix_len_arg), used_key_parts(used_key_parts_arg),
-    group_key_parts(group_key_parts_arg), index_info(index_info_arg),
-    index(index_arg), key_infix_len(key_infix_len_arg), range_tree(tree_arg),
-    index_tree(index_tree_arg), param_idx(param_idx_arg),
-    quick_prefix_records(quick_prefix_records_arg)
+    :
+      have_min(have_min_arg),
+      have_max(have_max_arg),
+      min_max_arg_part(min_max_arg_part_arg),
+      group_prefix_len(group_prefix_len_arg),
+      used_key_parts(used_key_parts_arg),
+      group_key_parts(group_key_parts_arg),
+      index_info(index_info_arg),
+      index(index_arg),
+      key_infix_len(key_infix_len_arg),
+      range_tree(tree_arg),
+      index_tree(index_tree_arg),
+      param_idx(param_idx_arg),
+      quick_prefix_records(quick_prefix_records_arg)
     {
       if (key_infix_len)
         memcpy(this->key_infix, key_infix_arg, key_infix_len);
     }
   virtual ~TRP_GROUP_MIN_MAX() {}             /* Remove gcc warning */
 
-  optimizer::QUICK_SELECT_I *make_quick(PARAM *param, 
-                                        bool retrieve_full_rows,
-                                        MEM_ROOT *parent_alloc);
+  optimizer::QuickSelectInterface *make_quick(optimizer::Parameter *param,
+                                              bool retrieve_full_rows,
+                                              MEM_ROOT *parent_alloc);
 };
 
 
@@ -2234,7 +1291,7 @@
     1  Out of memory.
 */
 
-static int fill_used_fields_bitmap(PARAM *param)
+static int fill_used_fields_bitmap(optimizer::Parameter *param)
 {
   Table *table= param->table;
   my_bitmap_map *tmp;
@@ -2267,7 +1324,7 @@
   Test if a key can be used in different ranges
 
   SYNOPSIS
-    SQL_SELECT::test_quick_select()
+    SqlSelect::test_quick_select()
       session               Current thread
       keys_to_use       Keys to use for range retrieval
       prev_tables       Tables assumed to be already read when the scan is
@@ -2329,10 +1386,10 @@
     1 if found usable ranges and quick select has been successfully created.
 */
 
-int optimizer::SQL_SELECT::test_quick_select(Session *session, 
+int optimizer::SqlSelect::test_quick_select(Session *session,
                                              key_map keys_to_use,
 				                                     table_map prev_tables,
-				                                     ha_rows limit, 
+				                                     ha_rows limit,
                                              bool force_quick_range,
                                              bool ordered_output)
 {
@@ -2363,7 +1420,7 @@
     SEL_TREE *tree= NULL;
     KEY_PART *key_parts;
     KEY *key_info;
-    PARAM param;
+    optimizer::Parameter param;
 
     if (check_stack_overrun(session, 2*STACK_MIN_SIZE, NULL))
       return 0;                           // Fatal error flag is set
@@ -2553,7 +1610,7 @@
     if (best_trp)
     {
       records= best_trp->records;
-      if (!(quick= best_trp->make_quick(&param, true)) || quick->init())
+      if (! (quick= best_trp->make_quick(&param, true)) || quick->init())
       {
         delete quick;
         quick= NULL;
@@ -2639,7 +1696,8 @@
 */
 
 static
-TABLE_READ_PLAN *get_best_disjunct_quick(PARAM *param, SEL_IMERGE *imerge,
+TABLE_READ_PLAN *get_best_disjunct_quick(optimizer::Parameter *param,
+                                         SEL_IMERGE *imerge,
                                          double read_time)
 {
   SEL_TREE **ptree;
@@ -2682,7 +1740,7 @@
         One of index scans in this index_merge is more expensive than entire
         table read for another available option. The entire index_merge (and
         any possible ROR-union) will be more expensive then, too. We continue
-        here only to update SQL_SELECT members.
+        here only to update SqlSelect members.
       */
       imerge_too_expensive= true;
     }
@@ -2721,7 +1779,7 @@
   {
     /*
       Add one ROWID comparison for each row retrieved on non-CPK scan.  (it
-      is done in QUICK_RANGE_SELECT::row_in_ranges)
+      is done in QuickRangeSelect::row_in_ranges)
      */
     imerge_cost += non_cpk_scan_records / TIME_FOR_COMPARE_ROWID;
   }
@@ -2878,7 +1936,7 @@
   ha_rows   records;  /* estimate of # records this scan will return */
 
   /* Set of intervals over key fields that will be used for row retrieval. */
-  SEL_ARG   *sel_arg;
+  optimizer::SEL_ARG   *sel_arg;
 
   /* Fields used in the query and covered by this ROR scan. */
   MyBitmap covered_fields;
@@ -2911,7 +1969,7 @@
 */
 
 static
-ROR_SCAN_INFO *make_ror_scan(const PARAM *param, int idx, SEL_ARG *sel_arg)
+ROR_SCAN_INFO *make_ror_scan(const optimizer::Parameter *param, int idx, optimizer::SEL_ARG *sel_arg)
 {
   ROR_SCAN_INFO *ror_scan;
   my_bitmap_map *bitmap_buf;
@@ -3011,7 +2069,7 @@
 /* Auxiliary structure for incremental ROR-intersection creation */
 typedef struct
 {
-  const PARAM *param;
+  const optimizer::Parameter *param;
   MyBitmap covered_fields; /* union of fields covered by all scans */
   /*
     Fraction of table records that satisfies conditions of all scans.
@@ -3041,7 +2099,7 @@
 */
 
 static
-ROR_INTERSECT_INFO* ror_intersect_init(const PARAM *param)
+ROR_INTERSECT_INFO* ror_intersect_init(const optimizer::Parameter *param)
 {
   ROR_INTERSECT_INFO *info;
   my_bitmap_map* buf;
@@ -3172,7 +2230,8 @@
   KEY_PART_INFO *key_part= info->param->table->key_info[scan->keynr].key_part;
   unsigned char key_val[MAX_KEY_LENGTH+MAX_FIELD_WIDTH]; /* key values tuple */
   unsigned char *key_ptr= key_val;
-  SEL_ARG *sel_arg, *tuple_arg= NULL;
+  optimizer::SEL_ARG *sel_arg= NULL;
+  optimizer::SEL_ARG *tuple_arg= NULL;
   key_part_map keypart_map= 0;
   bool cur_covered;
   bool prev_covered= test(info->covered_fields.isBitSet(key_part->fieldnr-1));
@@ -3187,7 +2246,7 @@
   for (sel_arg= scan->sel_arg; sel_arg;
        sel_arg= sel_arg->next_key_part)
   {
-    cur_covered= 
+    cur_covered=
       test(info->covered_fields.isBitSet(key_part[sel_arg->part].fieldnr-1));
     if (cur_covered != prev_covered)
     {
@@ -3387,10 +2446,10 @@
 */
 
 static
-TRP_ROR_INTERSECT *get_best_ror_intersect(const PARAM *param, 
-                                                     SEL_TREE *tree,
-                                                     double read_time,
-                                                     bool *are_all_covering)
+TRP_ROR_INTERSECT *get_best_ror_intersect(const optimizer::Parameter *param,
+                                          SEL_TREE *tree,
+                                          double read_time,
+                                          bool *are_all_covering)
 {
   uint32_t idx;
   double min_cost= DBL_MAX;
@@ -3573,9 +2632,9 @@
 */
 
 static
-TRP_ROR_INTERSECT *get_best_covering_ror_intersect(PARAM *param,
-                                                              SEL_TREE *tree,
-                                                              double read_time)
+TRP_ROR_INTERSECT *get_best_covering_ror_intersect(optimizer::Parameter *param,
+                                                   SEL_TREE *tree,
+                                                   double read_time)
 {
   ROR_SCAN_INFO **ror_scan_mark;
   ROR_SCAN_INFO **ror_scans_end= tree->ror_scans_end;
@@ -3711,13 +2770,14 @@
     NULL if no plan found or error occurred
 */
 
-static TRP_RANGE *get_key_scans_params(PARAM *param, SEL_TREE *tree,
+static TRP_RANGE *get_key_scans_params(optimizer::Parameter *param,
+                                       SEL_TREE *tree,
                                        bool index_read_must_be_used,
                                        bool update_tbl_stats,
                                        double read_time)
 {
   uint32_t idx;
-  SEL_ARG **key,**end, **key_to_read= NULL;
+  optimizer::SEL_ARG **key,**end, **key_to_read= NULL;
   ha_rows best_records= 0;
   uint32_t    best_mrr_flags= 0, best_buf_size= 0;
   TRP_RANGE* read_plan= NULL;
@@ -3738,7 +2798,7 @@
       double found_read_time= 0.0;
       uint32_t mrr_flags, buf_size;
       uint32_t keynr= param->real_keynr[idx];
-      if ((*key)->type == SEL_ARG::MAYBE_KEY ||
+      if ((*key)->type == optimizer::SEL_ARG::MAYBE_KEY ||
           (*key)->maybe_flag)
         param->needed_reg->set(keynr);
 
@@ -3783,12 +2843,12 @@
 }
 
 
-optimizer::QUICK_SELECT_I *TRP_INDEX_MERGE::make_quick(PARAM *param, bool, MEM_ROOT *)
+optimizer::QuickSelectInterface *TRP_INDEX_MERGE::make_quick(optimizer::Parameter *param, bool, MEM_ROOT *)
 {
-  optimizer::QUICK_INDEX_MERGE_SELECT *quick_imerge;
-  optimizer::QUICK_RANGE_SELECT *quick;
+  optimizer::QuickIndexMergeSelect *quick_imerge;
+  optimizer::QuickRangeSelect *quick= NULL;
   /* index_merge always retrieves full rows, ignore retrieve_full_rows */
-  if (! (quick_imerge= new optimizer::QUICK_INDEX_MERGE_SELECT(param->session, param->table)))
+  if (! (quick_imerge= new optimizer::QuickIndexMergeSelect(param->session, param->table)))
   {
     return NULL;
   }
@@ -3798,7 +2858,7 @@
   for (TRP_RANGE **range_scan= range_scans; range_scan != range_scans_end;
        range_scan++)
   {
-    if (!(quick= (optimizer::QUICK_RANGE_SELECT*)
+    if (!(quick= (optimizer::QuickRangeSelect*)
           ((*range_scan)->make_quick(param, false, &quick_imerge->alloc)))||
         quick_imerge->push_quick_back(quick))
     {
@@ -3810,65 +2870,66 @@
   return quick_imerge;
 }
 
-optimizer::QUICK_SELECT_I *TRP_ROR_INTERSECT::make_quick(PARAM *param,
+optimizer::QuickSelectInterface *TRP_ROR_INTERSECT::make_quick(optimizer::Parameter *param,
                                                          bool retrieve_full_rows,
                                                          MEM_ROOT *parent_alloc)
 {
-  optimizer::QUICK_ROR_INTERSECT_SELECT *quick_intrsect= NULL;
-  optimizer::QUICK_RANGE_SELECT *quick= NULL;
+  optimizer::QUICK_ROR_INTERSECT_SELECT *quick_intersect= NULL;
+  optimizer::QuickRangeSelect *quick= NULL;
   MEM_ROOT *alloc= NULL;
 
-  if ((quick_intrsect=
-         new optimizer::QUICK_ROR_INTERSECT_SELECT(param->session, param->table,
-                                                   (retrieve_full_rows? (!is_covering) :
-                                                    false),
+  if ((quick_intersect=
+         new optimizer::QUICK_ROR_INTERSECT_SELECT(param->session,
+                                                   param->table,
+                                                   (retrieve_full_rows? (! is_covering) : false),
                                                    parent_alloc)))
   {
     print_ror_scans_arr(param->table,
-                                             "creating ROR-intersect",
-                                             first_scan, last_scan);
-    alloc= parent_alloc? parent_alloc: &quick_intrsect->alloc;
-    for (; first_scan != last_scan;++first_scan)
+                        "creating ROR-intersect",
+                        first_scan,
+                        last_scan);
+    alloc= parent_alloc ? parent_alloc : &quick_intersect->alloc;
+    for (; first_scan != last_scan; ++first_scan)
     {
-      if (! (quick= optimizer::get_quick_select(param, 
+      if (! (quick= optimizer::get_quick_select(param,
                                                 (*first_scan)->idx,
                                                 (*first_scan)->sel_arg,
                                                 HA_MRR_USE_DEFAULT_IMPL | HA_MRR_SORTED,
-                                                0, 
+                                                0,
                                                 alloc)) ||
-          quick_intrsect->push_quick_back(quick))
+          quick_intersect->push_quick_back(quick))
       {
-        delete quick_intrsect;
+        delete quick_intersect;
         return NULL;
       }
     }
     if (cpk_scan)
     {
-      if (! (quick= optimizer::get_quick_select(param, 
+      if (! (quick= optimizer::get_quick_select(param,
                                                 cpk_scan->idx,
                                                 cpk_scan->sel_arg,
                                                 HA_MRR_USE_DEFAULT_IMPL | HA_MRR_SORTED,
-                                                0, 
+                                                0,
                                                 alloc)))
       {
-        delete quick_intrsect;
+        delete quick_intersect;
         return NULL;
       }
       quick->resetCursor();
-      quick_intrsect->cpk_quick= quick;
+      quick_intersect->cpk_quick= quick;
     }
-    quick_intrsect->records= records;
-    quick_intrsect->read_time= read_cost;
+    quick_intersect->records= records;
+    quick_intersect->read_time= read_cost;
   }
-  return quick_intrsect;
+  return quick_intersect;
 }
 
 
-optimizer::QUICK_SELECT_I *TRP_ROR_UNION::make_quick(PARAM *param, bool, MEM_ROOT *)
+optimizer::QuickSelectInterface *TRP_ROR_UNION::make_quick(optimizer::Parameter *param, bool, MEM_ROOT *)
 {
   optimizer::QUICK_ROR_UNION_SELECT *quick_roru;
   TABLE_READ_PLAN **scan;
-  optimizer::QUICK_SELECT_I *quick;
+  optimizer::QuickSelectInterface *quick;
   /*
     It is impossible to construct a ROR-union that will not retrieve full
     rows, ignore retrieve_full_rows parameter.
@@ -3895,7 +2956,7 @@
 
   SYNOPSIS
     get_ne_mm_tree()
-      param       PARAM from SQL_SELECT::test_quick_select
+      param       Parameter from SqlSelect::test_quick_select
       cond_func   item for the predicate
       field       field in the predicate
       lt_value    constant that field should be smaller
@@ -3907,7 +2968,8 @@
     0  on error
 */
 
-static SEL_TREE *get_ne_mm_tree(RANGE_OPT_PARAM *param, Item_func *cond_func,
+static SEL_TREE *get_ne_mm_tree(optimizer::RangeParameter *param,
+                                Item_func *cond_func,
                                 Field *field,
                                 Item *lt_value, Item *gt_value,
                                 Item_result cmp_type)
@@ -3917,9 +2979,12 @@
                      lt_value, cmp_type);
   if (tree)
   {
-    tree= tree_or(param, tree, get_mm_parts(param, cond_func, field,
-					    Item_func::GT_FUNC,
-					    gt_value, cmp_type));
+    tree= tree_or(param,
+                  tree,
+                  get_mm_parts(param, cond_func, field,
+					        Item_func::GT_FUNC,
+					        gt_value,
+                  cmp_type));
   }
   return tree;
 }
@@ -3930,7 +2995,7 @@
 
   SYNOPSIS
     get_func_mm_tree()
-      param       PARAM from SQL_SELECT::test_quick_select
+      param       Parameter from SqlSelect::test_quick_select
       cond_func   item for the predicate
       field       field in the predicate
       value       constant in the predicate
@@ -3942,7 +3007,8 @@
     Pointer to the tree built tree
 */
 
-static SEL_TREE *get_func_mm_tree(RANGE_OPT_PARAM *param, Item_func *cond_func,
+static SEL_TREE *get_func_mm_tree(optimizer::RangeParameter *param,
+                                  Item_func *cond_func,
                                   Field *field, Item *value,
                                   Item_result cmp_type, bool inv)
 {
@@ -4082,7 +3148,7 @@
             /* Change all intervals to be "c_{i-1} < X < c_i" */
             for (uint32_t idx= 0; idx < param->keys; idx++)
             {
-              SEL_ARG *new_interval, *last_val;
+              optimizer::SEL_ARG *new_interval, *last_val;
               if (((new_interval= tree2->keys[idx])) &&
                   (tree->keys[idx]) &&
                   ((last_val= tree->keys[idx]->last())))
@@ -4170,7 +3236,7 @@
 
   SYNOPSIS
     get_full_func_mm_tree()
-      param       PARAM from SQL_SELECT::test_quick_select
+      param       Parameter from SqlSelect::test_quick_select
       cond_func   item for the predicate
       field_item  field in the predicate
       value       constant in the predicate
@@ -4235,7 +3301,7 @@
     Pointer to the tree representing the built conjunction of SEL_TREEs
 */
 
-static SEL_TREE *get_full_func_mm_tree(RANGE_OPT_PARAM *param,
+static SEL_TREE *get_full_func_mm_tree(optimizer::RangeParameter *param,
                                        Item_func *cond_func,
                                        Item_field *field_item, Item *value,
                                        bool inv)
@@ -4283,7 +3349,7 @@
 
 	/* make a select tree of all keys in condition */
 
-static SEL_TREE *get_mm_tree(RANGE_OPT_PARAM *param,COND *cond)
+static SEL_TREE *get_mm_tree(optimizer::RangeParameter *param, COND *cond)
 {
   SEL_TREE *tree=0;
   SEL_TREE *ftree= 0;
@@ -4301,9 +3367,9 @@
       Item *item;
       while ((item=li++))
       {
-	SEL_TREE *new_tree=get_mm_tree(param,item);
+	SEL_TREE *new_tree= get_mm_tree(param,item);
 	if (param->session->is_fatal_error ||
-            param->alloced_sel_args > SEL_ARG::MAX_SEL_ARGS)
+            param->alloced_sel_args > optimizer::SEL_ARG::MAX_SEL_ARGS)
 	  return 0;	// out of memory
 	tree=tree_and(param,tree,new_tree);
 	if (tree && tree->type == SEL_TREE::IMPOSSIBLE)
@@ -4312,13 +3378,13 @@
     }
     else
     {						// COND OR
-      tree=get_mm_tree(param,li++);
+      tree= get_mm_tree(param,li++);
       if (tree)
       {
 	Item *item;
 	while ((item=li++))
 	{
-	  SEL_TREE *new_tree=get_mm_tree(param,item);
+	  SEL_TREE *new_tree= get_mm_tree(param,item);
 	  if (!new_tree)
 	    return 0;	// out of memory
 	  tree=tree_or(param,tree,new_tree);
@@ -4455,9 +3521,11 @@
 
 
 static SEL_TREE *
-get_mm_parts(RANGE_OPT_PARAM *param, COND *cond_func, Field *field,
-	     Item_func::Functype type,
-	     Item *value, Item_result)
+get_mm_parts(optimizer::RangeParameter *param,
+             COND *cond_func,
+             Field *field,
+	           Item_func::Functype type,
+	           Item *value, Item_result)
 {
   if (field->table != param->table)
     return 0;
@@ -4468,30 +3536,30 @@
   if (value &&
       value->used_tables() & ~(param->prev_tables | param->read_tables))
     return 0;
-  for (; key_part != end ; key_part++)
+  for (; key_part != end; key_part++)
   {
     if (field->eq(key_part->field))
     {
-      SEL_ARG *sel_arg=0;
+      optimizer::SEL_ARG *sel_arg=0;
       if (!tree && !(tree=new SEL_TREE()))
-	return 0;				// OOM
+        return 0;				// OOM
       if (!value || !(value->used_tables() & ~param->read_tables))
       {
-	sel_arg=get_mm_leaf(param,cond_func,
-			    key_part->field,key_part,type,value);
-	if (!sel_arg)
-	  continue;
-	if (sel_arg->type == SEL_ARG::IMPOSSIBLE)
-	{
-	  tree->type=SEL_TREE::IMPOSSIBLE;
-	  return(tree);
-	}
+        sel_arg= get_mm_leaf(param,cond_func,
+            key_part->field,key_part,type,value);
+        if (! sel_arg)
+          continue;
+        if (sel_arg->type == optimizer::SEL_ARG::IMPOSSIBLE)
+        {
+          tree->type=SEL_TREE::IMPOSSIBLE;
+          return(tree);
+        }
       }
       else
       {
-	// This key may be used later
-	if (!(sel_arg= new SEL_ARG(SEL_ARG::MAYBE_KEY)))
-	  return 0;			// OOM
+        // This key may be used later
+        if (! (sel_arg= new optimizer::SEL_ARG(optimizer::SEL_ARG::MAYBE_KEY)))
+          return 0;			// OOM
       }
       sel_arg->part=(unsigned char) key_part->part;
       tree->keys[key_part->key]=sel_add(tree->keys[key_part->key],sel_arg);
@@ -4499,17 +3567,21 @@
     }
   }
 
-  return(tree);
+  return tree;
 }
 
 
-static SEL_ARG *
-get_mm_leaf(RANGE_OPT_PARAM *param, COND *conf_func, Field *field,
-            KEY_PART *key_part, Item_func::Functype type,Item *value)
+static optimizer::SEL_ARG *
+get_mm_leaf(optimizer::RangeParameter *param,
+            COND *conf_func,
+            Field *field,
+            KEY_PART *key_part,
+            Item_func::Functype type,
+            Item *value)
 {
   uint32_t maybe_null=(uint32_t) field->real_maybe_null();
   bool optimize_range;
-  SEL_ARG *tree= 0;
+  optimizer::SEL_ARG *tree= NULL;
   MEM_ROOT *alloc= param->mem_root;
   unsigned char *str;
   int err= 0;
@@ -4530,10 +3602,10 @@
     if (!maybe_null)				// Not null field
     {
       if (type == Item_func::ISNULL_FUNC)
-        tree= &null_element;
+        tree= &optimizer::null_element;
       goto end;
     }
-    if (!(tree= new (alloc) SEL_ARG(field,is_null_string,is_null_string)))
+    if (!(tree= new (alloc) optimizer::SEL_ARG(field,is_null_string,is_null_string)))
       goto end;                                 // out of memory
     if (type == Item_func::ISNOTNULL_FUNC)
     {
@@ -4580,7 +3652,7 @@
       goto end;
     if (!(res= value->val_str(&tmp)))
     {
-      tree= &null_element;
+      tree= &optimizer::null_element;
       goto end;
     }
 
@@ -4646,11 +3718,11 @@
       int2store(min_str+maybe_null,min_length);
       int2store(max_str+maybe_null,max_length);
     }
-    tree= new (alloc) SEL_ARG(field, min_str, max_str);
+    tree= new (alloc) optimizer::SEL_ARG(field, min_str, max_str);
     goto end;
   }
 
-  if (!optimize_range &&
+  if (! optimize_range &&
       type != Item_func::EQ_FUNC &&
       type != Item_func::EQUAL_FUNC)
     goto end;                                   // Can't optimize this
@@ -4670,14 +3742,14 @@
    * OK, so previously, and in MySQL, what the optimizer does here is
    * override the sql_mode variable to ignore out-of-range or bad date-
    * time values.  It does this because the optimizer is populating the
-   * field variable with the incoming value from the comparison field, 
+   * field variable with the incoming value from the comparison field,
    * and the value may exceed the bounds of a proper column type.
    *
    * For instance, assume the following:
    *
-   * CREATE TABLE t1 (ts TIMESTAMP); 
+   * CREATE TABLE t1 (ts TIMESTAMP);
    * INSERT INTO t1 ('2009-03-04 00:00:00');
-   * CREATE TABLE t2 (dt1 DATETIME, dt2 DATETIME); 
+   * CREATE TABLE t2 (dt1 DATETIME, dt2 DATETIME);
    * INSERT INT t2 ('2003-12-31 00:00:00','2999-12-31 00:00:00');
    *
    * If we issue this query:
@@ -4690,15 +3762,15 @@
    * but Drizzle always throws errors on bad data storage in a Field class.
    *
    * Therefore, to get around the problem of the Field class being used for
-   * "storage" here without actually storing anything...we must check to see 
+   * "storage" here without actually storing anything...we must check to see
    * if the value being stored in a Field_timestamp here is out of range.  If
    * it is, then we must convert to the highest Timestamp value (or lowest,
    * depending on whether the datetime is before or after the epoch.
    */
   if (field->type() == DRIZZLE_TYPE_TIMESTAMP)
   {
-    /* 
-     * The left-side of the range comparison is a timestamp field.  Therefore, 
+    /*
+     * The left-side of the range comparison is a timestamp field.  Therefore,
      * we must check to see if the value in the right-hand side is outside the
      * range of the UNIX epoch, and cut to the epoch bounds if it is.
      */
@@ -4714,7 +3786,7 @@
         goto end;
       else
       {
-        /* 
+        /*
          * Create a datetime from the string and compare to fixed timestamp
          * instances representing the epoch boundaries.
          */
@@ -4732,7 +3804,7 @@
         /* We rely on Temporal class operator overloads to do our comparisons. */
         if (value_datetime < min_timestamp)
         {
-          /* 
+          /*
            * Datetime in right-hand side column is before UNIX epoch, so adjust to
            * lower bound.
            */
@@ -4780,8 +3852,8 @@
           value->result_type() == item_cmp_type(field->result_type(),
                                                 value->result_type()))
       {
-        tree= new (alloc) SEL_ARG(field, 0, 0);
-        tree->type= SEL_ARG::IMPOSSIBLE;
+        tree= new (alloc) optimizer::SEL_ARG(field, 0, 0);
+        tree->type= optimizer::SEL_ARG::IMPOSSIBLE;
         goto end;
       }
       else
@@ -4838,7 +3910,7 @@
   else if (err < 0)
   {
     /* This happens when we try to insert a NULL field in a not null column */
-    tree= &null_element;                        // cmp with NULL is never true
+    tree= &optimizer::null_element;                        // cmp with NULL is never true
     goto end;
   }
   str= (unsigned char*) alloc_root(alloc, key_part->store_length+1);
@@ -4847,8 +3919,8 @@
   if (maybe_null)
     *str= (unsigned char) field->is_real_null();        // Set to 1 if null
   field->get_key_image(str+maybe_null, key_part->length);
-  if (!(tree= new (alloc) SEL_ARG(field, str, str)))
-    goto end;                                   // out of memory
+  if (! (tree= new (alloc) optimizer::SEL_ARG(field, str, str)))
+    goto end; // out of memory
 
   /*
     Check if we are comparing an UNSIGNED integer with a negative constant.
@@ -4870,7 +3942,7 @@
     {
       if (type == Item_func::LT_FUNC || type == Item_func::LE_FUNC)
       {
-        tree->type= SEL_ARG::IMPOSSIBLE;
+        tree->type= optimizer::SEL_ARG::IMPOSSIBLE;
         goto end;
       }
       if (type == Item_func::GT_FUNC || type == Item_func::GE_FUNC)
@@ -4931,10 +4003,11 @@
   This will never be called for same key parts.
 */
 
-static SEL_ARG *
-sel_add(SEL_ARG *key1,SEL_ARG *key2)
+static optimizer::SEL_ARG *
+sel_add(optimizer::SEL_ARG *key1, optimizer::SEL_ARG *key2)
 {
-  SEL_ARG *root,**key_link;
+  optimizer::SEL_ARG *root= NULL;
+  optimizer::SEL_ARG **key_link= NULL;
 
   if (!key1)
     return key2;
@@ -4967,7 +4040,7 @@
 
 
 static SEL_TREE *
-tree_and(RANGE_OPT_PARAM *param,SEL_TREE *tree1,SEL_TREE *tree2)
+tree_and(optimizer::RangeParameter *param, SEL_TREE *tree1, SEL_TREE *tree2)
 {
   if (!tree1)
     return(tree2);
@@ -4992,7 +4065,7 @@
   result_keys.reset();
 
   /* Join the trees key per key */
-  SEL_ARG **key1,**key2,**end;
+  optimizer::SEL_ARG **key1,**key2,**end;
   for (key1= tree1->keys,key2= tree2->keys,end=key1+param->keys ;
        key1 != end ; key1++,key2++)
   {
@@ -5000,20 +4073,16 @@
     if (*key1 || *key2)
     {
       if (*key1 && !(*key1)->simple_key())
-	flag|=CLONE_KEY1_MAYBE;
+        flag|=CLONE_KEY1_MAYBE;
       if (*key2 && !(*key2)->simple_key())
-	flag|=CLONE_KEY2_MAYBE;
+        flag|=CLONE_KEY2_MAYBE;
       *key1=key_and(param, *key1, *key2, flag);
-      if (*key1 && (*key1)->type == SEL_ARG::IMPOSSIBLE)
+      if (*key1 && (*key1)->type == optimizer::SEL_ARG::IMPOSSIBLE)
       {
-	tree1->type= SEL_TREE::IMPOSSIBLE;
+        tree1->type= SEL_TREE::IMPOSSIBLE;
         return(tree1);
       }
       result_keys.set(key1 - tree1->keys);
-#ifdef EXTRA_DEBUG
-        if (*key1 && param->alloced_sel_args < SEL_ARG::MAX_SEL_ARGS)
-          (*key1)->test_use_count(*key1);
-#endif
     }
   }
   tree1->keys_map= result_keys;
@@ -5037,7 +4106,7 @@
 */
 
 bool sel_trees_can_be_ored(SEL_TREE *tree1, SEL_TREE *tree2,
-                           RANGE_OPT_PARAM* param)
+                           optimizer::RangeParameter* param)
 {
   key_map common_keys= tree1->keys_map;
   common_keys&= tree2->keys_map;
@@ -5046,7 +4115,7 @@
     return false;
 
   /* trees have a common key, check if they refer to same key part */
-  SEL_ARG **key1,**key2;
+  optimizer::SEL_ARG **key1,**key2;
   for (uint32_t key_no=0; key_no < param->keys; key_no++)
   {
     if (common_keys.test(key_no))
@@ -5118,7 +4187,7 @@
     1  No tree->keys[] left.
 */
 
-static bool remove_nonrange_trees(RANGE_OPT_PARAM *param, SEL_TREE *tree)
+static bool remove_nonrange_trees(optimizer::RangeParameter *param, SEL_TREE *tree)
 {
   bool res= false;
   for (uint32_t i=0; i < param->keys; i++)
@@ -5139,7 +4208,7 @@
 
 
 static SEL_TREE *
-tree_or(RANGE_OPT_PARAM *param,SEL_TREE *tree1,SEL_TREE *tree2)
+tree_or(optimizer::RangeParameter *param,SEL_TREE *tree1,SEL_TREE *tree2)
 {
   if (!tree1 || !tree2)
     return 0;
@@ -5158,7 +4227,7 @@
   if (sel_trees_can_be_ored(tree1, tree2, param))
   {
     /* Join the trees key per key */
-    SEL_ARG **key1,**key2,**end;
+    optimizer::SEL_ARG **key1,**key2,**end;
     for (key1= tree1->keys,key2= tree2->keys,end= key1+param->keys ;
          key1 != end ; key1++,key2++)
     {
@@ -5167,10 +4236,6 @@
       {
         result=tree1;				// Added to tree1
         result_keys.set(key1 - tree1->keys);
-#ifdef EXTRA_DEBUG
-        if (param->alloced_sel_args < SEL_ARG::MAX_SEL_ARGS)
-          (*key1)->test_use_count(*key1);
-#endif
       }
     }
     if (result)
@@ -5226,11 +4291,13 @@
 
 /* And key trees where key1->part < key2 -> part */
 
-static SEL_ARG *
-and_all_keys(RANGE_OPT_PARAM *param, SEL_ARG *key1, SEL_ARG *key2,
+static optimizer::SEL_ARG *
+and_all_keys(optimizer::RangeParameter *param,
+             optimizer::SEL_ARG *key1,
+             optimizer::SEL_ARG *key2,
              uint32_t clone_flag)
 {
-  SEL_ARG *next;
+  optimizer::SEL_ARG *next= NULL;
   ulong use_count=key1->use_count;
 
   if (key1->elements != 1)
@@ -5238,32 +4305,32 @@
     key2->use_count+=key1->elements-1; //psergey: why we don't count that key1 has n-k-p?
     key2->increment_use_count((int) key1->elements-1);
   }
-  if (key1->type == SEL_ARG::MAYBE_KEY)
+  if (key1->type == optimizer::SEL_ARG::MAYBE_KEY)
   {
-    key1->right= key1->left= &null_element;
+    key1->right= key1->left= &optimizer::null_element;
     key1->next= key1->prev= 0;
   }
-  for (next=key1->first(); next ; next=next->next)
+  for (next= key1->first(); next ; next=next->next)
   {
     if (next->next_key_part)
     {
-      SEL_ARG *tmp= key_and(param, next->next_key_part, key2, clone_flag);
-      if (tmp && tmp->type == SEL_ARG::IMPOSSIBLE)
+      optimizer::SEL_ARG *tmp= key_and(param, next->next_key_part, key2, clone_flag);
+      if (tmp && tmp->type == optimizer::SEL_ARG::IMPOSSIBLE)
       {
-	key1=key1->tree_delete(next);
-	continue;
+        key1=key1->tree_delete(next);
+        continue;
       }
       next->next_key_part=tmp;
       if (use_count)
-	next->increment_use_count(use_count);
-      if (param->alloced_sel_args > SEL_ARG::MAX_SEL_ARGS)
+        next->increment_use_count(use_count);
+      if (param->alloced_sel_args > optimizer::SEL_ARG::MAX_SEL_ARGS)
         break;
     }
     else
       next->next_key_part=key2;
   }
-  if (!key1)
-    return &null_element;			// Impossible ranges
+  if (! key1)
+    return &optimizer::null_element;			// Impossible ranges
   key1->use_count++;
   return key1;
 }
@@ -5284,12 +4351,15 @@
     NULL if the result of AND operation is an empty interval {0}.
 */
 
-static SEL_ARG *
-key_and(RANGE_OPT_PARAM *param, SEL_ARG *key1, SEL_ARG *key2, uint32_t clone_flag)
+static optimizer::SEL_ARG *
+key_and(optimizer::RangeParameter *param,
+        optimizer::SEL_ARG *key1,
+        optimizer::SEL_ARG *key2,
+        uint32_t clone_flag)
 {
-  if (!key1)
+  if (! key1)
     return key2;
-  if (!key2)
+  if (! key2)
     return key1;
   if (key1->part != key2->part)
   {
@@ -5301,45 +4371,47 @@
     // key1->part < key2->part
     key1->use_count--;
     if (key1->use_count > 0)
-      if (!(key1= key1->clone_tree(param)))
-	return 0;				// OOM
+      if (! (key1= key1->clone_tree(param)))
+        return 0;				// OOM
     return and_all_keys(param, key1, key2, clone_flag);
   }
 
   if (((clone_flag & CLONE_KEY2_MAYBE) &&
-       !(clone_flag & CLONE_KEY1_MAYBE) &&
-       key2->type != SEL_ARG::MAYBE_KEY) ||
-      key1->type == SEL_ARG::MAYBE_KEY)
+       ! (clone_flag & CLONE_KEY1_MAYBE) &&
+       key2->type != optimizer::SEL_ARG::MAYBE_KEY) ||
+      key1->type == optimizer::SEL_ARG::MAYBE_KEY)
   {						// Put simple key in key2
     std::swap(key1, key2);
-    clone_flag=swap_clone_flag(clone_flag);
+    clone_flag= swap_clone_flag(clone_flag);
   }
 
   /* If one of the key is MAYBE_KEY then the found region may be smaller */
-  if (key2->type == SEL_ARG::MAYBE_KEY)
+  if (key2->type == optimizer::SEL_ARG::MAYBE_KEY)
   {
     if (key1->use_count > 1)
     {
       key1->use_count--;
-      if (!(key1=key1->clone_tree(param)))
-	return 0;				// OOM
+      if (! (key1=key1->clone_tree(param)))
+        return 0;				// OOM
       key1->use_count++;
     }
-    if (key1->type == SEL_ARG::MAYBE_KEY)
+    if (key1->type == optimizer::SEL_ARG::MAYBE_KEY)
     {						// Both are maybe key
-      key1->next_key_part=key_and(param, key1->next_key_part,
-                                  key2->next_key_part, clone_flag);
+      key1->next_key_part= key_and(param,
+                                   key1->next_key_part,
+                                   key2->next_key_part,
+                                   clone_flag);
       if (key1->next_key_part &&
-	  key1->next_key_part->type == SEL_ARG::IMPOSSIBLE)
-	return key1;
+          key1->next_key_part->type == optimizer::SEL_ARG::IMPOSSIBLE)
+        return key1;
     }
     else
     {
       key1->maybe_smaller();
       if (key2->next_key_part)
       {
-	key1->use_count--;			// Incremented in and_all_keys
-	return and_all_keys(param, key1, key2, clone_flag);
+        key1->use_count--;			// Incremented in and_all_keys
+        return and_all_keys(param, key1, key2, clone_flag);
       }
       key2->use_count--;			// Key2 doesn't have a tree
     }
@@ -5348,34 +4420,38 @@
 
   key1->use_count--;
   key2->use_count--;
-  SEL_ARG *e1=key1->first(), *e2=key2->first(), *new_tree=0;
+  optimizer::SEL_ARG *e1= key1->first();
+  optimizer::SEL_ARG *e2= key2->first();
+  optimizer::SEL_ARG *new_tree= NULL;
 
   while (e1 && e2)
   {
-    int cmp=e1->cmp_min_to_min(e2);
+    int cmp= e1->cmp_min_to_min(e2);
     if (cmp < 0)
     {
-      if (get_range(&e1,&e2,key1))
-	continue;
+      if (get_range(&e1, &e2, key1))
+        continue;
     }
-    else if (get_range(&e2,&e1,key2))
+    else if (get_range(&e2, &e1, key2))
       continue;
-    SEL_ARG *next=key_and(param, e1->next_key_part, e2->next_key_part,
-                          clone_flag);
+    optimizer::SEL_ARG *next= key_and(param,
+                                      e1->next_key_part,
+                                      e2->next_key_part,
+                                      clone_flag);
     e1->increment_use_count(1);
     e2->increment_use_count(1);
-    if (!next || next->type != SEL_ARG::IMPOSSIBLE)
+    if (! next || next->type != optimizer::SEL_ARG::IMPOSSIBLE)
     {
-      SEL_ARG *new_arg= e1->clone_and(e2);
-      if (!new_arg)
-	return &null_element;			// End of memory
+      optimizer::SEL_ARG *new_arg= e1->clone_and(e2);
+      if (! new_arg)
+        return &optimizer::null_element;			// End of memory
       new_arg->next_key_part=next;
-      if (!new_tree)
+      if (! new_tree)
       {
-	new_tree=new_arg;
+        new_tree=new_arg;
       }
       else
-	new_tree=new_tree->insert(new_arg);
+        new_tree=new_tree->insert(new_arg);
     }
     if (e1->cmp_max_to_max(e2) < 0)
       e1=e1->next;				// e1 can't overlapp next e2
@@ -5384,19 +4460,19 @@
   }
   key1->free_tree();
   key2->free_tree();
-  if (!new_tree)
-    return &null_element;			// Impossible range
+  if (! new_tree)
+    return &optimizer::null_element;			// Impossible range
   return new_tree;
 }
 
 
 static bool
-get_range(SEL_ARG **e1,SEL_ARG **e2,SEL_ARG *root1)
+get_range(optimizer::SEL_ARG **e1, optimizer::SEL_ARG **e2, optimizer::SEL_ARG *root1)
 {
-  (*e1)=root1->find_range(*e2);			// first e1->min < e2->min
+  (*e1)= root1->find_range(*e2);			// first e1->min < e2->min
   if ((*e1)->cmp_max_to_min(*e2) < 0)
   {
-    if (!((*e1)=(*e1)->next))
+    if (! ((*e1)=(*e1)->next))
       return 1;
     if ((*e1)->cmp_min_to_max(*e2) > 0)
     {
@@ -5408,10 +4484,10 @@
 }
 
 
-static SEL_ARG *
-key_or(RANGE_OPT_PARAM *param, SEL_ARG *key1,SEL_ARG *key2)
+static optimizer::SEL_ARG *
+key_or(optimizer::RangeParameter *param, optimizer::SEL_ARG *key1, optimizer::SEL_ARG *key2)
 {
-  if (!key1)
+  if (! key1)
   {
     if (key2)
     {
@@ -5420,7 +4496,7 @@
     }
     return 0;
   }
-  if (!key2)
+  if (! key2)
   {
     key1->use_count--;
     key1->free_tree();
@@ -5437,13 +4513,13 @@
   }
 
   // If one of the key is MAYBE_KEY then the found region may be bigger
-  if (key1->type == SEL_ARG::MAYBE_KEY)
+  if (key1->type == optimizer::SEL_ARG::MAYBE_KEY)
   {
     key2->free_tree();
     key1->use_count++;
     return key1;
   }
-  if (key2->type == SEL_ARG::MAYBE_KEY)
+  if (key2->type == optimizer::SEL_ARG::MAYBE_KEY)
   {
     key1->free_tree();
     key2->use_count++;
@@ -5461,81 +4537,81 @@
   }
 
   // Add tree at key2 to tree at key1
-  bool key2_shared=key2->use_count != 0;
-  key1->maybe_flag|=key2->maybe_flag;
+  bool key2_shared= key2->use_count != 0;
+  key1->maybe_flag|= key2->maybe_flag;
 
   for (key2=key2->first(); key2; )
   {
-    SEL_ARG *tmp=key1->find_range(key2);	// Find key1.min <= key2.min
+    optimizer::SEL_ARG *tmp= key1->find_range(key2); // Find key1.min <= key2.min
     int cmp;
 
-    if (!tmp)
+    if (! tmp)
     {
-      tmp=key1->first();			// tmp.min > key2.min
+      tmp=key1->first(); // tmp.min > key2.min
       cmp= -1;
     }
     else if ((cmp=tmp->cmp_max_to_min(key2)) < 0)
     {						// Found tmp.max < key2.min
-      SEL_ARG *next=tmp->next;
+      optimizer::SEL_ARG *next= tmp->next;
       if (cmp == -2 && eq_tree(tmp->next_key_part,key2->next_key_part))
       {
-	// Join near ranges like tmp.max < 0 and key2.min >= 0
-	SEL_ARG *key2_next=key2->next;
-	if (key2_shared)
-	{
-	  if (!(key2=new SEL_ARG(*key2)))
-	    return 0;		// out of memory
-	  key2->increment_use_count(key1->use_count+1);
-	  key2->next=key2_next;			// New copy of key2
-	}
-	key2->copy_min(tmp);
-	if (!(key1=key1->tree_delete(tmp)))
-	{					// Only one key in tree
-	  key1=key2;
-	  key1->make_root();
-	  key2=key2_next;
-	  break;
-	}
+        // Join near ranges like tmp.max < 0 and key2.min >= 0
+        optimizer::SEL_ARG *key2_next=key2->next;
+        if (key2_shared)
+        {
+          if (! (key2=new optimizer::SEL_ARG(*key2)))
+            return 0;		// out of memory
+          key2->increment_use_count(key1->use_count+1);
+          key2->next= key2_next; // New copy of key2
+        }
+        key2->copy_min(tmp);
+        if (! (key1=key1->tree_delete(tmp)))
+        {					// Only one key in tree
+          key1= key2;
+          key1->make_root();
+          key2= key2_next;
+          break;
+        }
       }
-      if (!(tmp=next))				// tmp.min > key2.min
-	break;					// Copy rest of key2
+      if (! (tmp= next)) // tmp.min > key2.min
+        break; // Copy rest of key2
     }
     if (cmp < 0)
     {						// tmp.min > key2.min
       int tmp_cmp;
-      if ((tmp_cmp=tmp->cmp_min_to_max(key2)) > 0) // if tmp.min > key2.max
+      if ((tmp_cmp= tmp->cmp_min_to_max(key2)) > 0) // if tmp.min > key2.max
       {
-	if (tmp_cmp == 2 && eq_tree(tmp->next_key_part,key2->next_key_part))
-	{					// ranges are connected
-	  tmp->copy_min_to_min(key2);
-	  key1->merge_flags(key2);
-	  if (tmp->min_flag & NO_MIN_RANGE &&
-	      tmp->max_flag & NO_MAX_RANGE)
-	  {
-	    if (key1->maybe_flag)
-	      return new SEL_ARG(SEL_ARG::MAYBE_KEY);
-	    return 0;
-	  }
-	  key2->increment_use_count(-1);	// Free not used tree
-	  key2=key2->next;
-	  continue;
-	}
-	else
-	{
-	  SEL_ARG *next=key2->next;		// Keys are not overlapping
-	  if (key2_shared)
-	  {
-	    SEL_ARG *cpy= new SEL_ARG(*key2);	// Must make copy
-	    if (!cpy)
-	      return 0;				// OOM
-	    key1=key1->insert(cpy);
-	    key2->increment_use_count(key1->use_count+1);
-	  }
-	  else
-	    key1=key1->insert(key2);		// Will destroy key2_root
-	  key2=next;
-	  continue;
-	}
+        if (tmp_cmp == 2 && eq_tree(tmp->next_key_part,key2->next_key_part))
+        {					// ranges are connected
+          tmp->copy_min_to_min(key2);
+          key1->merge_flags(key2);
+          if (tmp->min_flag & NO_MIN_RANGE &&
+              tmp->max_flag & NO_MAX_RANGE)
+          {
+            if (key1->maybe_flag)
+              return new optimizer::SEL_ARG(optimizer::SEL_ARG::MAYBE_KEY);
+            return 0;
+          }
+          key2->increment_use_count(-1);	// Free not used tree
+          key2= key2->next;
+          continue;
+        }
+        else
+        {
+          optimizer::SEL_ARG *next= key2->next; // Keys are not overlapping
+          if (key2_shared)
+          {
+            optimizer::SEL_ARG *cpy= new optimizer::SEL_ARG(*key2); // Must make copy
+            if (! cpy)
+              return 0; // OOM
+            key1= key1->insert(cpy);
+            key2->increment_use_count(key1->use_count+1);
+          }
+          else
+            key1= key1->insert(key2);		// Will destroy key2_root
+          key2= next;
+          continue;
+        }
       }
     }
 
@@ -5544,116 +4620,116 @@
     {
       if (tmp->is_same(key2))
       {
-	tmp->merge_flags(key2);			// Copy maybe flags
-	key2->increment_use_count(-1);		// Free not used tree
+        tmp->merge_flags(key2);			// Copy maybe flags
+        key2->increment_use_count(-1);		// Free not used tree
       }
       else
       {
-	SEL_ARG *last=tmp;
-	while (last->next && last->next->cmp_min_to_max(key2) <= 0 &&
-	       eq_tree(last->next->next_key_part,key2->next_key_part))
-	{
-	  SEL_ARG *save=last;
-	  last=last->next;
-	  key1=key1->tree_delete(save);
-	}
+        optimizer::SEL_ARG *last= tmp;
+        while (last->next && last->next->cmp_min_to_max(key2) <= 0 &&
+               eq_tree(last->next->next_key_part,key2->next_key_part))
+        {
+          optimizer::SEL_ARG *save= last;
+          last= last->next;
+          key1= key1->tree_delete(save);
+        }
         last->copy_min(tmp);
-	if (last->copy_min(key2) || last->copy_max(key2))
-	{					// Full range
-	  key1->free_tree();
-	  for (; key2 ; key2=key2->next)
-	    key2->increment_use_count(-1);	// Free not used tree
-	  if (key1->maybe_flag)
-	    return new SEL_ARG(SEL_ARG::MAYBE_KEY);
-	  return 0;
-	}
+        if (last->copy_min(key2) || last->copy_max(key2))
+        {					// Full range
+          key1->free_tree();
+          for (; key2; key2= key2->next)
+            key2->increment_use_count(-1);	// Free not used tree
+          if (key1->maybe_flag)
+            return new optimizer::SEL_ARG(optimizer::SEL_ARG::MAYBE_KEY);
+          return 0;
+        }
       }
-      key2=key2->next;
+      key2= key2->next;
       continue;
     }
 
     if (cmp >= 0 && tmp->cmp_min_to_min(key2) < 0)
     {						// tmp.min <= x < key2.min
-      SEL_ARG *new_arg=tmp->clone_first(key2);
-      if (!new_arg)
-	return 0;				// OOM
+      optimizer::SEL_ARG *new_arg= tmp->clone_first(key2);
+      if (! new_arg)
+        return 0;				// OOM
       if ((new_arg->next_key_part= key1->next_key_part))
-	new_arg->increment_use_count(key1->use_count+1);
+        new_arg->increment_use_count(key1->use_count+1);
       tmp->copy_min_to_min(key2);
-      key1=key1->insert(new_arg);
+      key1= key1->insert(new_arg);
     }
 
     // tmp.min >= key2.min && tmp.min <= key2.max
-    SEL_ARG key(*key2);				// Get copy we can modify
+    optimizer::SEL_ARG key(*key2); // Get copy we can modify
     for (;;)
     {
       if (tmp->cmp_min_to_min(&key) > 0)
       {						// key.min <= x < tmp.min
-	SEL_ARG *new_arg=key.clone_first(tmp);
-	if (!new_arg)
-	  return 0;				// OOM
-	if ((new_arg->next_key_part=key.next_key_part))
-	  new_arg->increment_use_count(key1->use_count+1);
-	key1=key1->insert(new_arg);
+        optimizer::SEL_ARG *new_arg= key.clone_first(tmp);
+        if (! new_arg)
+          return 0;				// OOM
+        if ((new_arg->next_key_part=key.next_key_part))
+          new_arg->increment_use_count(key1->use_count+1);
+        key1= key1->insert(new_arg);
       }
       if ((cmp=tmp->cmp_max_to_max(&key)) <= 0)
       {						// tmp.min. <= x <= tmp.max
-	tmp->maybe_flag|= key.maybe_flag;
-	key.increment_use_count(key1->use_count+1);
-	tmp->next_key_part= key_or(param, tmp->next_key_part, key.next_key_part);
-	if (!cmp)				// Key2 is ready
-	  break;
-	key.copy_max_to_min(tmp);
-	if (!(tmp=tmp->next))
-	{
-	  SEL_ARG *tmp2= new SEL_ARG(key);
-	  if (!tmp2)
-	    return 0;				// OOM
-	  key1=key1->insert(tmp2);
-	  key2=key2->next;
-	  goto end;
-	}
-	if (tmp->cmp_min_to_max(&key) > 0)
-	{
-	  SEL_ARG *tmp2= new SEL_ARG(key);
-	  if (!tmp2)
-	    return 0;				// OOM
-	  key1=key1->insert(tmp2);
-	  break;
-	}
+        tmp->maybe_flag|= key.maybe_flag;
+        key.increment_use_count(key1->use_count+1);
+        tmp->next_key_part= key_or(param, tmp->next_key_part, key.next_key_part);
+        if (! cmp)				// Key2 is ready
+          break;
+        key.copy_max_to_min(tmp);
+        if (! (tmp= tmp->next))
+        {
+          optimizer::SEL_ARG *tmp2= new optimizer::SEL_ARG(key);
+          if (! tmp2)
+            return 0;				// OOM
+          key1= key1->insert(tmp2);
+          key2= key2->next;
+          goto end;
+        }
+        if (tmp->cmp_min_to_max(&key) > 0)
+        {
+          optimizer::SEL_ARG *tmp2= new optimizer::SEL_ARG(key);
+          if (! tmp2)
+            return 0;				// OOM
+          key1= key1->insert(tmp2);
+          break;
+        }
       }
       else
       {
-	SEL_ARG *new_arg=tmp->clone_last(&key); // tmp.min <= x <= key.max
-	if (!new_arg)
-	  return 0;				// OOM
-	tmp->copy_max_to_min(&key);
-	tmp->increment_use_count(key1->use_count+1);
-	/* Increment key count as it may be used for next loop */
-	key.increment_use_count(1);
-	new_arg->next_key_part= key_or(param, tmp->next_key_part, key.next_key_part);
-	key1=key1->insert(new_arg);
-	break;
+        optimizer::SEL_ARG *new_arg= tmp->clone_last(&key); // tmp.min <= x <= key.max
+        if (! new_arg)
+          return 0;				// OOM
+        tmp->copy_max_to_min(&key);
+        tmp->increment_use_count(key1->use_count+1);
+        /* Increment key count as it may be used for next loop */
+        key.increment_use_count(1);
+        new_arg->next_key_part= key_or(param, tmp->next_key_part, key.next_key_part);
+        key1= key1->insert(new_arg);
+        break;
       }
     }
-    key2=key2->next;
+    key2= key2->next;
   }
 
 end:
   while (key2)
   {
-    SEL_ARG *next=key2->next;
+    optimizer::SEL_ARG *next= key2->next;
     if (key2_shared)
     {
-      SEL_ARG *tmp=new SEL_ARG(*key2);		// Must make copy
-      if (!tmp)
-	return 0;
+      optimizer::SEL_ARG *tmp= new optimizer::SEL_ARG(*key2);		// Must make copy
+      if (! tmp)
+        return 0;
       key2->increment_use_count(key1->use_count+1);
-      key1=key1->insert(tmp);
+      key1= key1->insert(tmp);
     }
     else
-      key1=key1->insert(key2);			// Will destroy key2_root
-    key2=next;
+      key1= key1->insert(key2);			// Will destroy key2_root
+    key2= next;
   }
   key1->use_count++;
   return key1;
@@ -5661,511 +4737,48 @@
 
 
 /* Compare if two trees are equal */
-
-static bool eq_tree(SEL_ARG* a,SEL_ARG *b)
+static bool eq_tree(optimizer::SEL_ARG *a, optimizer::SEL_ARG *b)
 {
   if (a == b)
-    return 1;
-  if (!a || !b || !a->is_same(b))
-    return 0;
-  if (a->left != &null_element && b->left != &null_element)
-  {
-    if (!eq_tree(a->left,b->left))
-      return 0;
-  }
-  else if (a->left != &null_element || b->left != &null_element)
-    return 0;
-  if (a->right != &null_element && b->right != &null_element)
-  {
-    if (!eq_tree(a->right,b->right))
-      return 0;
-  }
-  else if (a->right != &null_element || b->right != &null_element)
-    return 0;
+  {
+    return true;
+  }
+
+  if (! a || ! b || ! a->is_same(b))
+  {
+    return false;
+  }
+
+  if (a->left != &optimizer::null_element && b->left != &optimizer::null_element)
+  {
+    if (! eq_tree(a->left,b->left))
+      return false;
+  }
+  else if (a->left != &optimizer::null_element || b->left != &optimizer::null_element)
+  {
+    return false;
+  }
+
+  if (a->right != &optimizer::null_element && b->right != &optimizer::null_element)
+  {
+    if (! eq_tree(a->right,b->right))
+      return false;
+  }
+  else if (a->right != &optimizer::null_element || b->right != &optimizer::null_element)
+  {
+    return false;
+  }
+
   if (a->next_key_part != b->next_key_part)
   {						// Sub range
-    if (!a->next_key_part != !b->next_key_part ||
-	!eq_tree(a->next_key_part, b->next_key_part))
-      return 0;
-  }
-  return 1;
-}
-
-
-SEL_ARG *
-SEL_ARG::insert(SEL_ARG *key)
-{
-  SEL_ARG *element, **par= NULL, *last_element= NULL;
-
-  for (element= this; element != &null_element ; )
-  {
-    last_element=element;
-    if (key->cmp_min_to_min(element) > 0)
-    {
-      par= &element->right; element= element->right;
-    }
-    else
-    {
-      par = &element->left; element= element->left;
-    }
-  }
-  *par=key;
-  key->parent=last_element;
-	/* Link in list */
-  if (par == &last_element->left)
-  {
-    key->next=last_element;
-    if ((key->prev=last_element->prev))
-      key->prev->next=key;
-    last_element->prev=key;
-  }
-  else
-  {
-    if ((key->next=last_element->next))
-      key->next->prev=key;
-    key->prev=last_element;
-    last_element->next=key;
-  }
-  key->left=key->right= &null_element;
-  SEL_ARG *root=rb_insert(key);			// rebalance tree
-  root->use_count=this->use_count;		// copy root info
-  root->elements= this->elements+1;
-  root->maybe_flag=this->maybe_flag;
-  return root;
-}
-
-
-/*
-** Find best key with min <= given key
-** Because the call context this should never return 0 to get_range
-*/
-
-SEL_ARG *
-SEL_ARG::find_range(SEL_ARG *key)
-{
-  SEL_ARG *element=this,*found=0;
-
-  for (;;)
-  {
-    if (element == &null_element)
-      return found;
-    int cmp=element->cmp_min_to_min(key);
-    if (cmp == 0)
-      return element;
-    if (cmp < 0)
-    {
-      found=element;
-      element=element->right;
-    }
-    else
-      element=element->left;
-  }
-}
-
-
-/*
-  Remove a element from the tree
-
-  SYNOPSIS
-    tree_delete()
-    key		Key that is to be deleted from tree (this)
-
-  NOTE
-    This also frees all sub trees that is used by the element
-
-  RETURN
-    root of new tree (with key deleted)
-*/
-
-SEL_ARG *
-SEL_ARG::tree_delete(SEL_ARG *key)
-{
-  enum leaf_color remove_color;
-  SEL_ARG *root,*nod,**par,*fix_par;
-
-  root=this;
-  this->parent= 0;
-
-  /* Unlink from list */
-  if (key->prev)
-    key->prev->next=key->next;
-  if (key->next)
-    key->next->prev=key->prev;
-  key->increment_use_count(-1);
-  if (!key->parent)
-    par= &root;
-  else
-    par=key->parent_ptr();
-
-  if (key->left == &null_element)
-  {
-    *par=nod=key->right;
-    fix_par=key->parent;
-    if (nod != &null_element)
-      nod->parent=fix_par;
-    remove_color= key->color;
-  }
-  else if (key->right == &null_element)
-  {
-    *par= nod=key->left;
-    nod->parent=fix_par=key->parent;
-    remove_color= key->color;
-  }
-  else
-  {
-    SEL_ARG *tmp=key->next;			// next bigger key (exist!)
-    nod= *tmp->parent_ptr()= tmp->right;	// unlink tmp from tree
-    fix_par=tmp->parent;
-    if (nod != &null_element)
-      nod->parent=fix_par;
-    remove_color= tmp->color;
-
-    tmp->parent=key->parent;			// Move node in place of key
-    (tmp->left=key->left)->parent=tmp;
-    if ((tmp->right=key->right) != &null_element)
-      tmp->right->parent=tmp;
-    tmp->color=key->color;
-    *par=tmp;
-    if (fix_par == key)				// key->right == key->next
-      fix_par=tmp;				// new parent of nod
-  }
-
-  if (root == &null_element)
-    return 0;				// Maybe root later
-  if (remove_color == BLACK)
-    root=rb_delete_fixup(root,nod,fix_par);
-#ifdef EXTRA_DEBUG
-  test_rb_tree(root,root->parent);
-#endif /* EXTRA_DEBUG */
-
-  root->use_count=this->use_count;		// Fix root counters
-  root->elements=this->elements-1;
-  root->maybe_flag=this->maybe_flag;
-  return(root);
-}
-
-
-	/* Functions to fix up the tree after insert and delete */
-
-static void left_rotate(SEL_ARG **root,SEL_ARG *leaf)
-{
-  SEL_ARG *y=leaf->right;
-  leaf->right=y->left;
-  if (y->left != &null_element)
-    y->left->parent=leaf;
-  if (!(y->parent=leaf->parent))
-    *root=y;
-  else
-    *leaf->parent_ptr()=y;
-  y->left=leaf;
-  leaf->parent=y;
-}
-
-static void right_rotate(SEL_ARG **root,SEL_ARG *leaf)
-{
-  SEL_ARG *y=leaf->left;
-  leaf->left=y->right;
-  if (y->right != &null_element)
-    y->right->parent=leaf;
-  if (!(y->parent=leaf->parent))
-    *root=y;
-  else
-    *leaf->parent_ptr()=y;
-  y->right=leaf;
-  leaf->parent=y;
-}
-
-
-SEL_ARG *
-SEL_ARG::rb_insert(SEL_ARG *leaf)
-{
-  SEL_ARG *y,*par,*par2,*root;
-  root= this; root->parent= 0;
-
-  leaf->color=RED;
-  while (leaf != root && (par= leaf->parent)->color == RED)
-  {					// This can't be root or 1 level under
-    if (par == (par2= leaf->parent->parent)->left)
-    {
-      y= par2->right;
-      if (y->color == RED)
-      {
-	par->color=BLACK;
-	y->color=BLACK;
-	leaf=par2;
-	leaf->color=RED;		/* And the loop continues */
-      }
-      else
-      {
-	if (leaf == par->right)
-	{
-	  left_rotate(&root,leaf->parent);
-	  par=leaf;			/* leaf is now parent to old leaf */
-	}
-	par->color=BLACK;
-	par2->color=RED;
-	right_rotate(&root,par2);
-	break;
-      }
-    }
-    else
-    {
-      y= par2->left;
-      if (y->color == RED)
-      {
-	par->color=BLACK;
-	y->color=BLACK;
-	leaf=par2;
-	leaf->color=RED;		/* And the loop continues */
-      }
-      else
-      {
-	if (leaf == par->left)
-	{
-	  right_rotate(&root,par);
-	  par=leaf;
-	}
-	par->color=BLACK;
-	par2->color=RED;
-	left_rotate(&root,par2);
-	break;
-      }
-    }
-  }
-  root->color=BLACK;
-#ifdef EXTRA_DEBUG
-  test_rb_tree(root,root->parent);
-#endif /* EXTRA_DEBUG */
-
-  return root;
-}
-
-
-SEL_ARG *rb_delete_fixup(SEL_ARG *root,SEL_ARG *key,SEL_ARG *par)
-{
-  SEL_ARG *x,*w;
-  root->parent=0;
-
-  x= key;
-  while (x != root && x->color == SEL_ARG::BLACK)
-  {
-    if (x == par->left)
-    {
-      w=par->right;
-      if (w->color == SEL_ARG::RED)
-      {
-	w->color=SEL_ARG::BLACK;
-	par->color=SEL_ARG::RED;
-	left_rotate(&root,par);
-	w=par->right;
-      }
-      if (w->left->color == SEL_ARG::BLACK && w->right->color == SEL_ARG::BLACK)
-      {
-	w->color=SEL_ARG::RED;
-	x=par;
-      }
-      else
-      {
-	if (w->right->color == SEL_ARG::BLACK)
-	{
-	  w->left->color=SEL_ARG::BLACK;
-	  w->color=SEL_ARG::RED;
-	  right_rotate(&root,w);
-	  w=par->right;
-	}
-	w->color=par->color;
-	par->color=SEL_ARG::BLACK;
-	w->right->color=SEL_ARG::BLACK;
-	left_rotate(&root,par);
-	x=root;
-	break;
-      }
-    }
-    else
-    {
-      w=par->left;
-      if (w->color == SEL_ARG::RED)
-      {
-	w->color=SEL_ARG::BLACK;
-	par->color=SEL_ARG::RED;
-	right_rotate(&root,par);
-	w=par->left;
-      }
-      if (w->right->color == SEL_ARG::BLACK && w->left->color == SEL_ARG::BLACK)
-      {
-	w->color=SEL_ARG::RED;
-	x=par;
-      }
-      else
-      {
-	if (w->left->color == SEL_ARG::BLACK)
-	{
-	  w->right->color=SEL_ARG::BLACK;
-	  w->color=SEL_ARG::RED;
-	  left_rotate(&root,w);
-	  w=par->left;
-	}
-	w->color=par->color;
-	par->color=SEL_ARG::BLACK;
-	w->left->color=SEL_ARG::BLACK;
-	right_rotate(&root,par);
-	x=root;
-	break;
-      }
-    }
-    par=x->parent;
-  }
-  x->color=SEL_ARG::BLACK;
-  return root;
-}
-
-
-	/* Test that the properties for a red-black tree hold */
-
-#ifdef EXTRA_DEBUG
-int test_rb_tree(SEL_ARG *element,SEL_ARG *parent)
-{
-  int count_l,count_r;
-
-  if (element == &null_element)
-    return 0;					// Found end of tree
-  if (element->parent != parent)
-  {
-    errmsg_printf(ERRMSG_LVL_ERROR, "Wrong tree: Parent doesn't point at parent");
-    return -1;
-  }
-  if (element->color == SEL_ARG::RED &&
-      (element->left->color == SEL_ARG::RED ||
-       element->right->color == SEL_ARG::RED))
-  {
-    errmsg_printf(ERRMSG_LVL_ERROR, "Wrong tree: Found two red in a row");
-    return -1;
-  }
-  if (element->left == element->right && element->left != &null_element)
-  {						// Dummy test
-    errmsg_printf(ERRMSG_LVL_ERROR, "Wrong tree: Found right == left");
-    return -1;
-  }
-  count_l=test_rb_tree(element->left,element);
-  count_r=test_rb_tree(element->right,element);
-  if (count_l >= 0 && count_r >= 0)
-  {
-    if (count_l == count_r)
-      return count_l+(element->color == SEL_ARG::BLACK);
-    errmsg_printf(ERRMSG_LVL_ERROR, "Wrong tree: Incorrect black-count: %d - %d",
-	    count_l,count_r);
-  }
-  return -1;					// Error, no more warnings
-}
-
-
-/*
-  Count how many times SEL_ARG graph "root" refers to its part "key"
-
-  SYNOPSIS
-    count_key_part_usage()
-      root  An RB-Root node in a SEL_ARG graph.
-      key   Another RB-Root node in that SEL_ARG graph.
-
-  DESCRIPTION
-    The passed "root" node may refer to "key" node via root->next_key_part,
-    root->next->n
-
-    This function counts how many times the node "key" is referred (via
-    SEL_ARG::next_key_part) by
-     - intervals of RB-tree pointed by "root",
-     - intervals of RB-trees that are pointed by SEL_ARG::next_key_part from
-       intervals of RB-tree pointed by "root",
-     - and so on.
-
-    Here is an example (horizontal links represent next_key_part pointers,
-    vertical links - next/prev prev pointers):
-
-         +----+               $
-         |root|-----------------+
-         +----+               $ |
-           |                  $ |
-           |                  $ |
-         +----+       +---+   $ |     +---+    Here the return value
-         |    |- ... -|   |---$-+--+->|key|    will be 4.
-         +----+       +---+   $ |  |  +---+
-           |                  $ |  |
-          ...                 $ |  |
-           |                  $ |  |
-         +----+   +---+       $ |  |
-         |    |---|   |---------+  |
-         +----+   +---+       $    |
-           |        |         $    |
-          ...     +---+       $    |
-                  |   |------------+
-                  +---+       $
-  RETURN
-    Number of links to "key" from nodes reachable from "root".
-*/
-
-static ulong count_key_part_usage(SEL_ARG *root, SEL_ARG *key)
-{
-  ulong count= 0;
-  for (root=root->first(); root ; root=root->next)
-  {
-    if (root->next_key_part)
-    {
-      if (root->next_key_part == key)
-	count++;
-      if (root->next_key_part->part < key->part)
-	count+=count_key_part_usage(root->next_key_part,key);
-    }
-  }
-  return count;
-}
-
-
-/*
-  Check if SEL_ARG::use_count value is correct
-
-  SYNOPSIS
-    SEL_ARG::test_use_count()
-      root  The root node of the SEL_ARG graph (an RB-tree root node that
-            has the least value of sel_arg->part in the entire graph, and
-            thus is the "origin" of the graph)
-
-  DESCRIPTION
-    Check if SEL_ARG::use_count value is correct. See the definition of
-    use_count for what is "correct".
-*/
-
-void SEL_ARG::test_use_count(SEL_ARG *root)
-{
-  uint32_t e_count=0;
-  if (this == root && use_count != 1)
-  {
-    errmsg_printf(ERRMSG_LVL_INFO, "Use_count: Wrong count %lu for root",use_count);
-    return;
-  }
-  if (this->type != SEL_ARG::KEY_RANGE)
-    return;
-  for (SEL_ARG *pos=first(); pos ; pos=pos->next)
-  {
-    e_count++;
-    if (pos->next_key_part)
-    {
-      ulong count=count_key_part_usage(root,pos->next_key_part);
-      if (count > pos->next_key_part->use_count)
-      {
-        errmsg_printf(ERRMSG_LVL_INFO, "Use_count: Wrong count for key at 0x%lx, %lu "
-                              "should be %lu", (long unsigned int)pos,
-                              pos->next_key_part->use_count, count);
-	return;
-      }
-      pos->next_key_part->test_use_count(root);
-    }
-  }
-  if (e_count != elements)
-    errmsg_printf(ERRMSG_LVL_WARN, "Wrong use count: %u (should be %u) for tree at 0x%lx",
-                      e_count, elements, (long unsigned int) this);
-}
-
-#endif
+    if (! a->next_key_part != ! b->next_key_part ||
+	      ! eq_tree(a->next_key_part, b->next_key_part))
+      return false;
+  }
+
+  return true;
+}
+
 
 /****************************************************************************
   MRR Range Sequence Interface implementation that walks a SEL_ARG* tree.
@@ -6188,7 +4801,7 @@
 
   /* Number of key parts */
   uint32_t min_key_parts, max_key_parts;
-  SEL_ARG *key_tree;
+  optimizer::SEL_ARG *key_tree;
 } RANGE_SEQ_ENTRY;
 
 
@@ -6199,8 +4812,8 @@
 {
   uint32_t keyno;      /* index of used tree in SEL_TREE structure */
   uint32_t real_keyno; /* Number of the index in tables */
-  PARAM *param;
-  SEL_ARG *start; /* Root node of the traversed SEL_ARG* graph */
+  optimizer::Parameter *param;
+  optimizer::SEL_ARG *start; /* Root node of the traversed SEL_ARG* graph */
 
   RANGE_SEQ_ENTRY stack[MAX_REF_PARTS];
   int i; /* Index of last used element in the above array */
@@ -6239,7 +4852,7 @@
 }
 
 
-static void step_down_to(SEL_ARG_RANGE_SEQ *arg, SEL_ARG *key_tree)
+static void step_down_to(SEL_ARG_RANGE_SEQ *arg, optimizer::SEL_ARG *key_tree)
 {
   RANGE_SEQ_ENTRY *cur= &arg->stack[arg->i+1];
   RANGE_SEQ_ENTRY *prev= &arg->stack[arg->i];
@@ -6291,7 +4904,7 @@
 //psergey-merge-todo: support check_quick_keys:max_keypart
 static uint32_t sel_arg_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range)
 {
-  SEL_ARG *key_tree;
+  optimizer::SEL_ARG *key_tree;
   SEL_ARG_RANGE_SEQ *seq= (SEL_ARG_RANGE_SEQ*)rseq;
   if (seq->at_start)
   {
@@ -6304,7 +4917,7 @@
   /* Ok, we're at some "full tuple" position in the tree */
 
   /* Step down if we can */
-  if (key_tree->next && key_tree->next != &null_element)
+  if (key_tree->next && key_tree->next != &optimizer::null_element)
   {
     //step down; (update the tuple, we'll step right and stay there)
     seq->i--;
@@ -6324,7 +4937,7 @@
     key_tree= seq->stack[seq->i].key_tree;
 
     /* Step down if we can */
-    if (key_tree->next && key_tree->next != &null_element)
+    if (key_tree->next && key_tree->next != &optimizer::null_element)
     {
       // Step down; update the tuple
       seq->i--;
@@ -6339,26 +4952,26 @@
     Walk right-up while we can
   */
 walk_right_n_up:
-  while (key_tree->next_key_part && key_tree->next_key_part != &null_element &&
+  while (key_tree->next_key_part && key_tree->next_key_part != &optimizer::null_element &&
          key_tree->next_key_part->part == key_tree->part + 1 &&
-         key_tree->next_key_part->type == SEL_ARG::KEY_RANGE)
+         key_tree->next_key_part->type == optimizer::SEL_ARG::KEY_RANGE)
   {
     {
       RANGE_SEQ_ENTRY *cur= &seq->stack[seq->i];
       uint32_t min_key_length= cur->min_key - seq->param->min_key;
       uint32_t max_key_length= cur->max_key - seq->param->max_key;
       uint32_t len= cur->min_key - cur[-1].min_key;
-      if (!(min_key_length == max_key_length &&
-            !memcmp(cur[-1].min_key, cur[-1].max_key, len) &&
-            !key_tree->min_flag && !key_tree->max_flag))
+      if (! (min_key_length == max_key_length &&
+          ! memcmp(cur[-1].min_key, cur[-1].max_key, len) &&
+          ! key_tree->min_flag && !key_tree->max_flag))
       {
         seq->param->is_ror_scan= false;
-        if (!key_tree->min_flag)
+        if (! key_tree->min_flag)
           cur->min_key_parts +=
             key_tree->next_key_part->store_min_key(seq->param->key[seq->keyno],
                                                    &cur->min_key,
                                                    &cur->min_key_flag);
-        if (!key_tree->max_flag)
+        if (! key_tree->max_flag)
           cur->max_key_parts +=
             key_tree->next_key_part->store_max_key(seq->param->key[seq->keyno],
                                                    &cur->max_key,
@@ -6374,7 +4987,7 @@
     key_tree= key_tree->next_key_part;
 
 walk_up_n_right:
-    while (key_tree->prev && key_tree->prev != &null_element)
+    while (key_tree->prev && key_tree->prev != &optimizer::null_element)
     {
       /* Step up */
       key_tree= key_tree->prev;
@@ -6439,7 +5052,7 @@
   SYNOPSIS
     check_quick_select()
       param             Parameter from test_quick_select
-      idx               Number of index to use in PARAM::key SEL_TREE::key
+      idx               Number of index to use in Parameter::key SEL_TREE::key
       index_only        true  - assume only index tuples will be accessed
                         false - assume full table rows will be read
       tree              Transformed selection condition, tree->key[idx] holds
@@ -6461,9 +5074,14 @@
 */
 
 static
-ha_rows check_quick_select(PARAM *param, uint32_t idx, bool index_only,
-                           SEL_ARG *tree, bool update_tbl_stats,
-                           uint32_t *mrr_flags, uint32_t *bufsize, COST_VECT *cost)
+ha_rows check_quick_select(optimizer::Parameter *param,
+                           uint32_t idx,
+                           bool index_only,
+                           optimizer::SEL_ARG *tree,
+                           bool update_tbl_stats,
+                           uint32_t *mrr_flags,
+                           uint32_t *bufsize,
+                           COST_VECT *cost)
 {
   SEL_ARG_RANGE_SEQ seq;
   RANGE_SEQ_IF seq_if = {sel_arg_range_seq_init, sel_arg_range_seq_next};
@@ -6472,11 +5090,11 @@
   uint32_t keynr= param->real_keynr[idx];
 
   /* Handle cases when we don't have a valid non-empty list of range */
-  if (!tree)
+  if (! tree)
     return(HA_POS_ERROR);
-  if (tree->type == SEL_ARG::IMPOSSIBLE)
+  if (tree->type == optimizer::SEL_ARG::IMPOSSIBLE)
     return(0L);
-  if (tree->type != SEL_ARG::KEY_RANGE || tree->part != 0)
+  if (tree->type != optimizer::SEL_ARG::KEY_RANGE || tree->part != 0)
     return(HA_POS_ERROR);
 
   seq.keyno= idx;
@@ -6577,7 +5195,7 @@
     false  Otherwise
 */
 
-static bool is_key_scan_ror(PARAM *param, uint32_t keynr, uint8_t nparts)
+static bool is_key_scan_ror(optimizer::Parameter *param, uint32_t keynr, uint8_t nparts)
 {
   KEY *table_key= param->table->key_info + keynr;
   KEY_PART_INFO *key_part= table_key->key_part + nparts;
@@ -6615,20 +5233,23 @@
 }
 
 
-optimizer::QUICK_RANGE_SELECT *
-optimizer::get_quick_select(PARAM *param,
+optimizer::QuickRangeSelect *
+optimizer::get_quick_select(Parameter *param,
                             uint32_t idx,
-                            SEL_ARG *key_tree, 
+                            optimizer::SEL_ARG *key_tree,
                             uint32_t mrr_flags,
-                            uint32_t mrr_buf_size, 
+                            uint32_t mrr_buf_size,
                             MEM_ROOT *parent_alloc)
 {
-  optimizer::QUICK_RANGE_SELECT *quick= NULL;
+  optimizer::QuickRangeSelect *quick= NULL;
   bool create_err= false;
 
-  quick=new optimizer::QUICK_RANGE_SELECT(param->session, param->table,
-                               param->real_keynr[idx],
-                               test(parent_alloc), NULL, &create_err);
+  quick= new optimizer::QuickRangeSelect(param->session,
+                                         param->table,
+                                         param->real_keynr[idx],
+                                         test(parent_alloc),
+                                         NULL,
+                                         &create_err);
 
   if (quick)
   {
@@ -6643,7 +5264,7 @@
                        0))
     {
       delete quick;
-      quick=0;
+      quick= NULL;
     }
     else
     {
@@ -6664,21 +5285,21 @@
 ** Fix this to get all possible sub_ranges
 */
 bool
-optimizer::get_quick_keys(PARAM *param,
-                          optimizer::QUICK_RANGE_SELECT *quick,
+optimizer::get_quick_keys(optimizer::Parameter *param,
+                          optimizer::QuickRangeSelect *quick,
                           KEY_PART *key,
-	                        SEL_ARG *key_tree, 
+	                        optimizer::SEL_ARG *key_tree,
                           unsigned char *min_key,
                           uint32_t min_key_flag,
 	                        unsigned char *max_key,
                           uint32_t max_key_flag)
 {
-  optimizer::QUICK_RANGE *range= NULL;
+  optimizer::QuickRange *range= NULL;
   uint32_t flag;
   int min_part= key_tree->part - 1; // # of keypart values in min_key buffer
   int max_part= key_tree->part - 1; // # of keypart values in max_key buffer
 
-  if (key_tree->left != &null_element)
+  if (key_tree->left != &optimizer::null_element)
   {
     if (get_quick_keys(param,
                        quick,
@@ -6692,7 +5313,7 @@
       return 1;
     }
   }
-  unsigned char *tmp_min_key=min_key,*tmp_max_key=max_key;
+  unsigned char *tmp_min_key= min_key,*tmp_max_key= max_key;
   min_part+= key_tree->store_min(key[key_tree->part].store_length,
                                  &tmp_min_key,min_key_flag);
   max_part+= key_tree->store_max(key[key_tree->part].store_length,
@@ -6700,7 +5321,7 @@
 
   if (key_tree->next_key_part &&
       key_tree->next_key_part->part == key_tree->part+1 &&
-      key_tree->next_key_part->type == SEL_ARG::KEY_RANGE)
+      key_tree->next_key_part->type == optimizer::SEL_ARG::KEY_RANGE)
   {						  // const key as prefix
     if ((tmp_min_key - min_key) == (tmp_max_key - max_key) &&
         memcmp(min_key, max_key, (uint32_t)(tmp_max_key - max_key))==0 &&
@@ -6710,9 +5331,9 @@
                          quick,
                          key,
                          key_tree->next_key_part,
-                         tmp_min_key, 
+                         tmp_min_key,
                          min_key_flag | key_tree->min_flag,
-                         tmp_max_key, 
+                         tmp_max_key,
                          max_key_flag | key_tree->max_flag))
       {
         return 1;
@@ -6723,13 +5344,13 @@
       uint32_t tmp_min_flag=key_tree->min_flag,tmp_max_flag=key_tree->max_flag;
       if (! tmp_min_flag)
       {
-        min_part+= key_tree->next_key_part->store_min_key(key, 
+        min_part+= key_tree->next_key_part->store_min_key(key,
                                                           &tmp_min_key,
                                                           &tmp_min_flag);
       }
       if (! tmp_max_flag)
       {
-        max_part+= key_tree->next_key_part->store_max_key(key, 
+        max_part+= key_tree->next_key_part->store_max_key(key,
                                                           &tmp_max_key,
                                                           &tmp_max_flag);
       }
@@ -6788,7 +5409,7 @@
   }
 
   /* Get range for retrieving rows in QUICK_SELECT::get_next */
-  if (! (range= new optimizer::QUICK_RANGE(param->min_key,
+  if (! (range= new optimizer::QuickRange(param->min_key,
 			                                     (uint32_t) (tmp_min_key - param->min_key),
                                            min_part >=0 ? make_keypart_map(min_part) : 0,
 			                                     param->max_key,
@@ -6808,7 +5429,7 @@
   }
 
  end:
-  if (key_tree->right != &null_element)
+  if (key_tree->right != &optimizer::null_element)
   {
     return get_quick_keys(param,
                           quick,
@@ -6823,27 +5444,6 @@
 }
 
 /*
-  Return 1 if there is only one range and this uses the whole primary key
-*/
-
-bool optimizer::QUICK_RANGE_SELECT::unique_key_range()
-{
-  if (ranges.elements == 1)
-  {
-    optimizer::QUICK_RANGE *tmp= *((optimizer::QUICK_RANGE**)ranges.buffer);
-    if ((tmp->flag & (EQ_RANGE | NULL_RANGE)) == EQ_RANGE)
-    {
-      KEY *key=head->key_info+index;
-      return ((key->flags & (HA_NOSAME)) == HA_NOSAME &&
-	      key->key_length == tmp->min_length);
-    }
-  }
-  return 0;
-}
-
-
-
-/*
   Return true if any part of the key is NULL
 
   SYNOPSIS
@@ -6870,27 +5470,16 @@
 }
 
 
-bool optimizer::QUICK_SELECT_I::is_keys_used(const MyBitmap *fields)
+bool optimizer::QuickSelectInterface::is_keys_used(const MyBitmap *fields)
 {
   return is_key_used(head, index, fields);
 }
 
-bool optimizer::QUICK_INDEX_MERGE_SELECT::is_keys_used(const MyBitmap *fields)
-{
-  optimizer::QUICK_RANGE_SELECT *quick= NULL;
-  List_iterator_fast<QUICK_RANGE_SELECT> it(quick_selects);
-  while ((quick= it++))
-  {
-    if (is_key_used(head, quick->index, fields))
-      return 1;
-  }
-  return 0;
-}
 
 bool optimizer::QUICK_ROR_INTERSECT_SELECT::is_keys_used(const MyBitmap *fields)
 {
-  optimizer::QUICK_RANGE_SELECT *quick;
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> it(quick_selects);
+  optimizer::QuickRangeSelect *quick;
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
   while ((quick= it++))
   {
     if (is_key_used(head, quick->index, fields))
@@ -6901,8 +5490,8 @@
 
 bool optimizer::QUICK_ROR_UNION_SELECT::is_keys_used(const MyBitmap *fields)
 {
-  optimizer::QUICK_SELECT_I *quick;
-  List_iterator_fast<optimizer::QUICK_SELECT_I> it(quick_selects);
+  optimizer::QuickSelectInterface *quick;
+  List_iterator_fast<optimizer::QuickSelectInterface> it(quick_selects);
   while ((quick= it++))
   {
     if (quick->is_keys_used(fields))
@@ -6931,28 +5520,28 @@
     NULL on error.
 */
 
-optimizer::QUICK_RANGE_SELECT *optimizer::get_quick_select_for_ref(Session *session, 
-                                                        Table *table,
-                                                        table_reference_st *ref, 
-                                                        ha_rows records)
+optimizer::QuickRangeSelect *optimizer::get_quick_select_for_ref(Session *session,
+                                                                 Table *table,
+                                                                 table_reference_st *ref,
+                                                                 ha_rows records)
 {
   MEM_ROOT *old_root, *alloc;
-  optimizer::QUICK_RANGE_SELECT *quick= NULL;
+  optimizer::QuickRangeSelect *quick= NULL;
   KEY *key_info = &table->key_info[ref->key];
   KEY_PART *key_part;
-  optimizer::QUICK_RANGE *range;
+  optimizer::QuickRange *range= NULL;
   uint32_t part;
   bool create_err= false;
   COST_VECT cost;
 
   old_root= session->mem_root;
   /* The following call may change session->mem_root */
-  quick= new optimizer::QUICK_RANGE_SELECT(session, table, ref->key, 0, 0, &create_err);
-  /* save mem_root set by QUICK_RANGE_SELECT constructor */
+  quick= new optimizer::QuickRangeSelect(session, table, ref->key, 0, 0, &create_err);
+  /* save mem_root set by QuickRangeSelect constructor */
   alloc= session->mem_root;
   /*
     return back default mem_root (session->mem_root) changed by
-    QUICK_RANGE_SELECT constructor
+    QuickRangeSelect constructor
   */
   session->mem_root= old_root;
 
@@ -6963,7 +5552,7 @@
   quick->records= records;
 
   if ((cp_buffer_from_ref(session, ref) && session->is_fatal_error) ||
-      !(range= new(alloc) optimizer::QUICK_RANGE()))
+      !(range= new(alloc) optimizer::QuickRange()))
     goto err;                                   // out of memory
 
   range->min_key= range->max_key= ref->key_buff;
@@ -6998,11 +5587,11 @@
   */
   if (ref->null_ref_key)
   {
-    optimizer::QUICK_RANGE *null_range;
+    optimizer::QuickRange *null_range= NULL;
 
     *ref->null_ref_key= 1;		// Set null byte then create a range
     if (!(null_range= new (alloc)
-          optimizer::QUICK_RANGE(ref->key_buff, ref->key_length,
+          optimizer::QuickRange(ref->key_buff, ref->key_length,
                                  make_prev_keypart_map(ref->key_parts),
                                  ref->key_buff, ref->key_length,
                                  make_prev_keypart_map(ref->key_parts), EQ_RANGE)))
@@ -7032,136 +5621,6 @@
 
 
 /*
-  Perform key scans for all used indexes (except CPK), get rowids and merge
-  them into an ordered non-recurrent sequence of rowids.
-
-  The merge/duplicate removal is performed using Unique class. We put all
-  rowids into Unique, get the sorted sequence and destroy the Unique.
-
-  If table has a clustered primary key that covers all rows (true for bdb
-  and innodb currently) and one of the index_merge scans is a scan on PK,
-  then rows that will be retrieved by PK scan are not put into Unique and
-  primary key scan is not performed here, it is performed later separately.
-
-  RETURN
-    0     OK
-    other error
-*/
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::read_keys_and_merge()
-{
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> cur_quick_it(quick_selects);
-  optimizer::QUICK_RANGE_SELECT* cur_quick;
-  int result;
-  Unique *unique;
-  Cursor *cursor= head->cursor;
-
-  cursor->extra(HA_EXTRA_KEYREAD);
-  head->prepare_for_position();
-
-  cur_quick_it.rewind();
-  cur_quick= cur_quick_it++;
-  assert(cur_quick != 0);
-
-  /*
-    We reuse the same instance of Cursor so we need to call both init and
-    reset here.
-  */
-  if (cur_quick->init() || cur_quick->reset())
-    return 0;
-
-  unique= new Unique(refpos_order_cmp, (void *)cursor,
-                     cursor->ref_length,
-                     session->variables.sortbuff_size);
-  if (!unique)
-    return 0;
-  for (;;)
-  {
-    while ((result= cur_quick->get_next()) == HA_ERR_END_OF_FILE)
-    {
-      cur_quick->range_end();
-      cur_quick= cur_quick_it++;
-      if (!cur_quick)
-        break;
-
-      if (cur_quick->cursor->inited != Cursor::NONE)
-        cur_quick->cursor->ha_index_end();
-      if (cur_quick->init() || cur_quick->reset())
-        return 0;
-    }
-
-    if (result)
-    {
-      if (result != HA_ERR_END_OF_FILE)
-      {
-        cur_quick->range_end();
-        return result;
-      }
-      break;
-    }
-
-    if (session->killed)
-      return 0;
-
-    /* skip row if it will be retrieved by clustered PK scan */
-    if (pk_quick_select && pk_quick_select->row_in_ranges())
-      continue;
-
-    cur_quick->cursor->position(cur_quick->record);
-    result= unique->unique_add((char*)cur_quick->cursor->ref);
-    if (result)
-      return 0;
-
-  }
-
-  /* ok, all row ids are in Unique */
-  result= unique->get(head);
-  delete unique;
-  doing_pk_scan= false;
-  /* index_merge currently doesn't support "using index" at all */
-  cursor->extra(HA_EXTRA_NO_KEYREAD);
-  /* start table scan */
-  init_read_record(&read_record, session, head, (optimizer::SQL_SELECT*) 0, 1, 1);
-  return result;
-}
-
-
-/*
-  Get next row for index_merge.
-  NOTES
-    The rows are read from
-      1. rowids stored in Unique.
-      2. QUICK_RANGE_SELECT with clustered primary key (if any).
-    The sets of rows retrieved in 1) and 2) are guaranteed to be disjoint.
-*/
-
-int optimizer::QUICK_INDEX_MERGE_SELECT::get_next()
-{
-  int result;
-
-  if (doing_pk_scan)
-    return(pk_quick_select->get_next());
-
-  if ((result= read_record.read_record(&read_record)) == -1)
-  {
-    result= HA_ERR_END_OF_FILE;
-    end_read_record(&read_record);
-    /* All rows from Unique have been retrieved, do a clustered PK scan */
-    if (pk_quick_select)
-    {
-      doing_pk_scan= true;
-      if ((result= pk_quick_select->init()) ||
-          (result= pk_quick_select->reset()))
-        return result;
-      return(pk_quick_select->get_next());
-    }
-  }
-
-  return result;
-}
-
-
-/*
   Retrieve next record.
   SYNOPSIS
      QUICK_ROR_INTERSECT_SELECT::get_next()
@@ -7183,8 +5642,8 @@
 
 int optimizer::QUICK_ROR_INTERSECT_SELECT::get_next()
 {
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> quick_it(quick_selects);
-  optimizer::QUICK_RANGE_SELECT* quick;
+  List_iterator_fast<optimizer::QuickRangeSelect> quick_it(quick_selects);
+  optimizer::QuickRangeSelect* quick;
   int error, cmp;
   uint32_t last_rowid_count=0;
 
@@ -7269,7 +5728,7 @@
 int optimizer::QUICK_ROR_UNION_SELECT::get_next()
 {
   int error, dup_row;
-  optimizer::QUICK_SELECT_I *quick;
+  optimizer::QuickSelectInterface *quick;
   unsigned char *tmp;
 
   do
@@ -7317,58 +5776,12 @@
 }
 
 
-int optimizer::QUICK_RANGE_SELECT::reset()
-{
-  uint32_t  buf_size;
-  unsigned char *mrange_buff;
-  int   error;
-  HANDLER_BUFFER empty_buf;
-  last_range= NULL;
-  cur_range= (optimizer::QUICK_RANGE**) ranges.buffer;
-
-  if (cursor->inited == Cursor::NONE && (error= cursor->ha_index_init(index,1)))
-    return(error);
-
-  /* Allocate buffer if we need one but haven't allocated it yet */
-  if (mrr_buf_size && !mrr_buf_desc)
-  {
-    buf_size= mrr_buf_size;
-    while (buf_size && ! memory::multi_malloc(false,
-                                        &mrr_buf_desc, sizeof(*mrr_buf_desc),
-                                        &mrange_buff, buf_size,
-                                        NULL))
-    {
-      /* Try to shrink the buffers until both are 0. */
-      buf_size/= 2;
-    }
-    if (!mrr_buf_desc)
-      return(HA_ERR_OUT_OF_MEM);
-
-    /* Initialize the Cursor buffer. */
-    mrr_buf_desc->buffer= mrange_buff;
-    mrr_buf_desc->buffer_end= mrange_buff + buf_size;
-    mrr_buf_desc->end_of_used_area= mrange_buff;
-  }
-
-  if (!mrr_buf_desc)
-    empty_buf.buffer= empty_buf.buffer_end= empty_buf.end_of_used_area= NULL;
-
-  if (sorted)
-     mrr_flags |= HA_MRR_SORTED;
-  RANGE_SEQ_IF seq_funcs= {optimizer::quick_range_seq_init, optimizer::quick_range_seq_next};
-  error= cursor->multi_range_read_init(&seq_funcs, (void*)this, ranges.elements,
-                                     mrr_flags, mrr_buf_desc? mrr_buf_desc:
-                                                              &empty_buf);
-  return(error);
-}
-
-
 /*
-  Range sequence interface implementation for array<QUICK_RANGE>: initialize
+  Range sequence interface implementation for array<QuickRange>: initialize
 
   SYNOPSIS
     quick_range_seq_init()
-      init_param  Caller-opaque paramenter: QUICK_RANGE_SELECT* pointer
+      init_param  Caller-opaque paramenter: QuickRangeSelect* pointer
       n_ranges    Number of ranges in the sequence (ignored)
       flags       MRR flags (currently not used)
 
@@ -7378,9 +5791,9 @@
 
 range_seq_t optimizer::quick_range_seq_init(void *init_param, uint32_t, uint32_t)
 {
-  optimizer::QUICK_RANGE_SELECT *quick= (optimizer::QUICK_RANGE_SELECT*)init_param;
-  quick->qr_traversal_ctx.first=  (optimizer::QUICK_RANGE**)quick->ranges.buffer;
-  quick->qr_traversal_ctx.cur=    (optimizer::QUICK_RANGE**)quick->ranges.buffer;
+  optimizer::QuickRangeSelect *quick= (optimizer::QuickRangeSelect*)init_param;
+  quick->qr_traversal_ctx.first=  (optimizer::QuickRange**)quick->ranges.buffer;
+  quick->qr_traversal_ctx.cur=    (optimizer::QuickRange**)quick->ranges.buffer;
   quick->qr_traversal_ctx.last=   quick->qr_traversal_ctx.cur +
                                   quick->ranges.elements;
   return &quick->qr_traversal_ctx;
@@ -7388,7 +5801,7 @@
 
 
 /*
-  Range sequence interface implementation for array<QUICK_RANGE>: get next
+  Range sequence interface implementation for array<QuickRange>: get next
 
   SYNOPSIS
     quick_range_seq_next()
@@ -7399,427 +5812,48 @@
     0  Ok
     1  No more ranges in the sequence
 */
-
 uint32_t optimizer::quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range)
 {
-  QUICK_RANGE_SEQ_CTX *ctx= (QUICK_RANGE_SEQ_CTX*)rseq;
+  QuickRangeSequenceContext *ctx= (QuickRangeSequenceContext*) rseq;
 
   if (ctx->cur == ctx->last)
     return 1; /* no more ranges */
 
-  optimizer::QUICK_RANGE *cur= *(ctx->cur);
+  optimizer::QuickRange *cur= *(ctx->cur);
   key_range *start_key= &range->start_key;
-  key_range *end_key=   &range->end_key;
+  key_range *end_key= &range->end_key;
 
-  start_key->key=    cur->min_key;
+  start_key->key= cur->min_key;
   start_key->length= cur->min_length;
   start_key->keypart_map= cur->min_keypart_map;
-  start_key->flag=   ((cur->flag & NEAR_MIN) ? HA_READ_AFTER_KEY :
-                      (cur->flag & EQ_RANGE) ?
-                      HA_READ_KEY_EXACT : HA_READ_KEY_OR_NEXT);
-  end_key->key=      cur->max_key;
-  end_key->length=   cur->max_length;
+  start_key->flag= ((cur->flag & NEAR_MIN) ? HA_READ_AFTER_KEY :
+                                             (cur->flag & EQ_RANGE) ?
+                                             HA_READ_KEY_EXACT : HA_READ_KEY_OR_NEXT);
+  end_key->key= cur->max_key;
+  end_key->length= cur->max_length;
   end_key->keypart_map= cur->max_keypart_map;
   /*
     We use HA_READ_AFTER_KEY here because if we are reading on a key
     prefix. We want to find all keys with this prefix.
   */
-  end_key->flag=     (cur->flag & NEAR_MAX ? HA_READ_BEFORE_KEY :
-                      HA_READ_AFTER_KEY);
+  end_key->flag= (cur->flag & NEAR_MAX ? HA_READ_BEFORE_KEY :
+                                         HA_READ_AFTER_KEY);
   range->range_flag= cur->flag;
   ctx->cur++;
   return 0;
 }
 
 
-/*
-  Get next possible record using quick-struct.
-
-  SYNOPSIS
-    QUICK_RANGE_SELECT::get_next()
-
-  NOTES
-    Record is read into table->record[0]
-
-  RETURN
-    0			Found row
-    HA_ERR_END_OF_FILE	No (more) rows in range
-    #			Error code
-*/
-
-int optimizer::QUICK_RANGE_SELECT::get_next()
-{
-  char *dummy;
-  if (in_ror_merged_scan)
-  {
-    /*
-      We don't need to signal the bitmap change as the bitmap is always the
-      same for this head->cursor
-    */
-    head->column_bitmaps_set(&column_bitmap, &column_bitmap);
-  }
-
-  int result= cursor->multi_range_read_next(&dummy);
-
-  if (in_ror_merged_scan)
-  {
-    /* Restore bitmaps set on entry */
-    head->column_bitmaps_set(save_read_set, save_write_set);
-  }
-  return result;
-}
-
-
-/*
-  Get the next record with a different prefix.
-
-  SYNOPSIS
-    QUICK_RANGE_SELECT::get_next_prefix()
-    prefix_length  length of cur_prefix
-    cur_prefix     prefix of a key to be searched for
-
-  DESCRIPTION
-    Each subsequent call to the method retrieves the first record that has a
-    prefix with length prefix_length different from cur_prefix, such that the
-    record with the new prefix is within the ranges described by
-    this->ranges. The record found is stored into the buffer pointed by
-    this->record.
-    The method is useful for GROUP-BY queries with range conditions to
-    discover the prefix of the next group that satisfies the range conditions.
-
-  TODO
-    This method is a modified copy of QUICK_RANGE_SELECT::get_next(), so both
-    methods should be unified into a more general one to reduce code
-    duplication.
-
-  RETURN
-    0                  on success
-    HA_ERR_END_OF_FILE if returned all keys
-    other              if some error occurred
-*/
-
-int optimizer::QUICK_RANGE_SELECT::get_next_prefix(uint32_t prefix_length,
-                                        key_part_map keypart_map,
-                                        unsigned char *cur_prefix)
-{
-  for (;;)
-  {
-    int result;
-    key_range start_key, end_key;
-    if (last_range)
-    {
-      /* Read the next record in the same range with prefix after cur_prefix. */
-      assert(cur_prefix != 0);
-      result= cursor->index_read_map(record, cur_prefix, keypart_map,
-                                   HA_READ_AFTER_KEY);
-      if (result || (cursor->compare_key(cursor->end_range) <= 0))
-        return result;
-    }
-
-    uint32_t count= ranges.elements - (cur_range - (optimizer::QUICK_RANGE**) ranges.buffer);
-    if (count == 0)
-    {
-      /* Ranges have already been used up before. None is left for read. */
-      last_range= 0;
-      return HA_ERR_END_OF_FILE;
-    }
-    last_range= *(cur_range++);
-
-    start_key.key=    (const unsigned char*) last_range->min_key;
-    start_key.length= min(last_range->min_length, (uint16_t)prefix_length);
-    start_key.keypart_map= last_range->min_keypart_map & keypart_map;
-    start_key.flag=   ((last_range->flag & NEAR_MIN) ? HA_READ_AFTER_KEY :
-		       (last_range->flag & EQ_RANGE) ?
-		       HA_READ_KEY_EXACT : HA_READ_KEY_OR_NEXT);
-    end_key.key=      (const unsigned char*) last_range->max_key;
-    end_key.length=   min(last_range->max_length, (uint16_t)prefix_length);
-    end_key.keypart_map= last_range->max_keypart_map & keypart_map;
-    /*
-      We use READ_AFTER_KEY here because if we are reading on a key
-      prefix we want to find all keys with this prefix
-    */
-    end_key.flag=     (last_range->flag & NEAR_MAX ? HA_READ_BEFORE_KEY :
-		       HA_READ_AFTER_KEY);
-
-    result= cursor->read_range_first(last_range->min_keypart_map ? &start_key : 0,
-				   last_range->max_keypart_map ? &end_key : 0,
-                                   test(last_range->flag & EQ_RANGE),
-				   sorted);
-    if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
-      last_range= 0;			// Stop searching
-
-    if (result != HA_ERR_END_OF_FILE)
-      return result;
-    last_range= 0;			// No matching rows; go to next range
-  }
-}
-
-
-/*
-  Check if current row will be retrieved by this QUICK_RANGE_SELECT
-
-  NOTES
-    It is assumed that currently a scan is being done on another index
-    which reads all necessary parts of the index that is scanned by this
-    quick select.
-    The implementation does a binary search on sorted array of disjoint
-    ranges, without taking size of range into account.
-
-    This function is used to filter out clustered PK scan rows in
-    index_merge quick select.
-
-  RETURN
-    true  if current row will be retrieved by this quick select
-    false if not
-*/
-
-bool optimizer::QUICK_RANGE_SELECT::row_in_ranges()
-{
-  optimizer::QUICK_RANGE *res;
-  uint32_t min= 0;
-  uint32_t max= ranges.elements - 1;
-  uint32_t mid= (max + min)/2;
-
-  while (min != max)
-  {
-    if (cmp_next(*(optimizer::QUICK_RANGE**)dynamic_array_ptr(&ranges, mid)))
-    {
-      /* current row value > mid->max */
-      min= mid + 1;
-    }
-    else
-      max= mid;
-    mid= (min + max) / 2;
-  }
-  res= *(optimizer::QUICK_RANGE**)dynamic_array_ptr(&ranges, mid);
-  return (!cmp_next(res) && !cmp_prev(res));
-}
-
-/*
-  This is a hack: we inherit from QUICK_SELECT so that we can use the
-  get_next() interface, but we have to hold a pointer to the original
-  QUICK_SELECT because its data are used all over the place.  What
-  should be done is to factor out the data that is needed into a base
-  class (QUICK_SELECT), and then have two subclasses (_ASC and _DESC)
-  which handle the ranges and implement the get_next() function.  But
-  for now, this seems to work right at least.
- */
-
-optimizer::QUICK_SELECT_DESC::QUICK_SELECT_DESC(optimizer::QUICK_RANGE_SELECT *q, uint32_t, bool *)
-  :
-    optimizer::QUICK_RANGE_SELECT(*q), 
-    rev_it(rev_ranges)
-{
-  optimizer::QUICK_RANGE *r;
-
-  optimizer::QUICK_RANGE **pr= (optimizer::QUICK_RANGE**)ranges.buffer;
-  optimizer::QUICK_RANGE **end_range= pr + ranges.elements;
-  for (; pr!=end_range; pr++)
-    rev_ranges.push_front(*pr);
-
-  /* Remove EQ_RANGE flag for keys that are not using the full key */
-  for (r = rev_it++; r; r = rev_it++)
-  {
-    if ((r->flag & EQ_RANGE) &&
-	head->key_info[index].key_length != r->max_length)
-      r->flag&= ~EQ_RANGE;
-  }
-  rev_it.rewind();
-  q->dont_free=1;				// Don't free shared mem
-  delete q;
-}
-
-
-int optimizer::QUICK_SELECT_DESC::get_next()
-{
-  /* The max key is handled as follows:
-   *   - if there is NO_MAX_RANGE, start at the end and move backwards
-   *   - if it is an EQ_RANGE, which means that max key covers the entire
-   *     key, go directly to the key and read through it (sorting backwards is
-   *     same as sorting forwards)
-   *   - if it is NEAR_MAX, go to the key or next, step back once, and
-   *     move backwards
-   *   - otherwise (not NEAR_MAX == include the key), go after the key,
-   *     step back once, and move backwards
-   */
-
-  for (;;)
-  {
-    int result;
-    if (last_range)
-    {						// Already read through key
-      result = ((last_range->flag & EQ_RANGE)
-		? cursor->index_next_same(record, last_range->min_key,
-					last_range->min_length) :
-		cursor->index_prev(record));
-      if (!result)
-      {
-	if (cmp_prev(*rev_it.ref()) == 0)
-	  return 0;
-      }
-      else if (result != HA_ERR_END_OF_FILE)
-	return result;
-    }
-
-    if (!(last_range= rev_it++))
-      return HA_ERR_END_OF_FILE;		// All ranges used
-
-    if (last_range->flag & NO_MAX_RANGE)        // Read last record
-    {
-      int local_error;
-      if ((local_error=cursor->index_last(record)))
-	return(local_error);		// Empty table
-      if (cmp_prev(last_range) == 0)
-	return 0;
-      last_range= 0;                            // No match; go to next range
-      continue;
-    }
-
-    if (last_range->flag & EQ_RANGE)
-    {
-      result = cursor->index_read_map(record, last_range->max_key,
-                                    last_range->max_keypart_map,
-                                    HA_READ_KEY_EXACT);
-    }
-    else
-    {
-      assert(last_range->flag & NEAR_MAX ||
-                  range_reads_after_key(last_range));
-      result=cursor->index_read_map(record, last_range->max_key,
-                                  last_range->max_keypart_map,
-                                  ((last_range->flag & NEAR_MAX) ?
-                                   HA_READ_BEFORE_KEY :
-                                   HA_READ_PREFIX_LAST_OR_PREV));
-    }
-    if (result)
-    {
-      if (result != HA_ERR_KEY_NOT_FOUND && result != HA_ERR_END_OF_FILE)
-	return result;
-      last_range= 0;                            // Not found, to next range
-      continue;
-    }
-    if (cmp_prev(last_range) == 0)
-    {
-      if (last_range->flag == (UNIQUE_RANGE | EQ_RANGE))
-	last_range= 0;				// Stop searching
-      return 0;				// Found key is in range
-    }
-    last_range= 0;                              // To next range
-  }
-}
-
-
-/*
-  Compare if found key is over max-value
-  Returns 0 if key <= range->max_key
-  TODO: Figure out why can't this function be as simple as cmp_prev().
-*/
-
-int optimizer::QUICK_RANGE_SELECT::cmp_next(optimizer::QUICK_RANGE *range_arg)
-{
-  if (range_arg->flag & NO_MAX_RANGE)
-    return 0;                                   /* key can't be to large */
-
-  KEY_PART *key_part=key_parts;
-  uint32_t store_length;
-
-  for (unsigned char *key=range_arg->max_key, *end=key+range_arg->max_length;
-       key < end;
-       key+= store_length, key_part++)
-  {
-    int cmp;
-    store_length= key_part->store_length;
-    if (key_part->null_bit)
-    {
-      if (*key)
-      {
-        if (!key_part->field->is_null())
-          return 1;
-        continue;
-      }
-      else if (key_part->field->is_null())
-        return 0;
-      key++;					// Skip null byte
-      store_length--;
-    }
-    if ((cmp=key_part->field->key_cmp(key, key_part->length)) < 0)
-      return 0;
-    if (cmp > 0)
-      return 1;
-  }
-  return (range_arg->flag & NEAR_MAX) ? 1 : 0;          // Exact match
-}
-
-
-/*
-  Returns 0 if found key is inside range (found key >= range->min_key).
-*/
-
-int optimizer::QUICK_RANGE_SELECT::cmp_prev(optimizer::QUICK_RANGE *range_arg)
-{
-  int cmp;
-  if (range_arg->flag & NO_MIN_RANGE)
-    return 0;					/* key can't be to small */
-
-  cmp= key_cmp(key_part_info, range_arg->min_key,
-               range_arg->min_length);
-  if (cmp > 0 || (cmp == 0 && (range_arg->flag & NEAR_MIN) == false))
-    return 0;
-  return 1;                                     // outside of range
-}
-
-
-/*
- * true if this range will require using HA_READ_AFTER_KEY
-   See comment in get_next() about this
- */
-
-bool optimizer::QUICK_SELECT_DESC::range_reads_after_key(optimizer::QUICK_RANGE *range_arg)
-{
-  return ((range_arg->flag & (NO_MAX_RANGE | NEAR_MAX)) ||
-	  !(range_arg->flag & EQ_RANGE) ||
-	  head->key_info[index].key_length != range_arg->max_length) ? 1 : 0;
-}
-
-
-void optimizer::QUICK_RANGE_SELECT::add_info_string(String *str)
-{
-  KEY *key_info= head->key_info + index;
-  str->append(key_info->name);
-}
-
-void optimizer::QUICK_INDEX_MERGE_SELECT::add_info_string(String *str)
-{
-  optimizer::QUICK_RANGE_SELECT *quick;
-  bool first= true;
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> it(quick_selects);
-  str->append(STRING_WITH_LEN("sort_union("));
-  while ((quick= it++))
-  {
-    if (!first)
-      str->append(',');
-    else
-      first= false;
-    quick->add_info_string(str);
-  }
-  if (pk_quick_select)
-  {
-    str->append(',');
-    pk_quick_select->add_info_string(str);
-  }
-  str->append(')');
-}
-
 void optimizer::QUICK_ROR_INTERSECT_SELECT::add_info_string(String *str)
 {
   bool first= true;
-  optimizer::QUICK_RANGE_SELECT *quick;
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> it(quick_selects);
+  optimizer::QuickRangeSelect *quick;
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
   str->append(STRING_WITH_LEN("intersect("));
   while ((quick= it++))
   {
     KEY *key_info= head->key_info + quick->index;
-    if (!first)
+    if (! first)
       str->append(',');
     else
       first= false;
@@ -7834,15 +5868,16 @@
   str->append(')');
 }
 
+
 void optimizer::QUICK_ROR_UNION_SELECT::add_info_string(String *str)
 {
   bool first= true;
-  optimizer::QUICK_SELECT_I *quick;
-  List_iterator_fast<optimizer::QUICK_SELECT_I> it(quick_selects);
+  optimizer::QuickSelectInterface *quick;
+  List_iterator_fast<optimizer::QuickSelectInterface> it(quick_selects);
   str->append(STRING_WITH_LEN("union("));
   while ((quick= it++))
   {
-    if (!first)
+    if (! first)
       str->append(',');
     else
       first= false;
@@ -7852,60 +5887,14 @@
 }
 
 
-void optimizer::QUICK_RANGE_SELECT::add_keys_and_lengths(String *key_names,
-                                              String *used_lengths)
-{
-  char buf[64];
-  uint32_t length;
-  KEY *key_info= head->key_info + index;
-  key_names->append(key_info->name);
-  length= int64_t2str(max_used_key_length, buf, 10) - buf;
-  used_lengths->append(buf, length);
-}
-
-void optimizer::QUICK_INDEX_MERGE_SELECT::add_keys_and_lengths(String *key_names,
-                                                               String *used_lengths)
-{
-  char buf[64];
-  uint32_t length;
-  bool first= true;
-  optimizer::QUICK_RANGE_SELECT *quick;
-
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> it(quick_selects);
-  while ((quick= it++))
-  {
-    if (first)
-      first= false;
-    else
-    {
-      key_names->append(',');
-      used_lengths->append(',');
-    }
-
-    KEY *key_info= head->key_info + quick->index;
-    key_names->append(key_info->name);
-    length= int64_t2str(quick->max_used_key_length, buf, 10) - buf;
-    used_lengths->append(buf, length);
-  }
-  if (pk_quick_select)
-  {
-    KEY *key_info= head->key_info + pk_quick_select->index;
-    key_names->append(',');
-    key_names->append(key_info->name);
-    length= int64_t2str(pk_quick_select->max_used_key_length, buf, 10) - buf;
-    used_lengths->append(',');
-    used_lengths->append(buf, length);
-  }
-}
-
 void optimizer::QUICK_ROR_INTERSECT_SELECT::add_keys_and_lengths(String *key_names,
                                                                  String *used_lengths)
 {
   char buf[64];
   uint32_t length;
   bool first= true;
-  optimizer::QUICK_RANGE_SELECT *quick;
-  List_iterator_fast<optimizer::QUICK_RANGE_SELECT> it(quick_selects);
+  optimizer::QuickRangeSelect *quick;
+  List_iterator_fast<optimizer::QuickRangeSelect> it(quick_selects);
   while ((quick= it++))
   {
     KEY *key_info= head->key_info + quick->index;
@@ -7932,12 +5921,13 @@
   }
 }
 
+
 void optimizer::QUICK_ROR_UNION_SELECT::add_keys_and_lengths(String *key_names,
                                                              String *used_lengths)
 {
   bool first= true;
-  optimizer::QUICK_SELECT_I *quick;
-  List_iterator_fast<optimizer::QUICK_SELECT_I> it(quick_selects);
+  optimizer::QuickSelectInterface *quick;
+  List_iterator_fast<optimizer::QuickSelectInterface> it(quick_selects);
   while ((quick= it++))
   {
     if (first)
@@ -7957,22 +5947,36 @@
 *******************************************************************************/
 
 static inline uint32_t get_field_keypart(KEY *index, Field *field);
-static inline SEL_ARG * get_index_range_tree(uint32_t index, SEL_TREE* range_tree,
-                                             PARAM *param, uint32_t *param_idx);
-static bool get_constant_key_infix(KEY *index_info, SEL_ARG *index_range_tree,
-                       KEY_PART_INFO *first_non_group_part,
-                       KEY_PART_INFO *min_max_arg_part,
-                       KEY_PART_INFO *last_part, Session *session,
-                       unsigned char *key_infix, uint32_t *key_infix_len,
-                       KEY_PART_INFO **first_non_infix_part);
+
+static inline optimizer::SEL_ARG * get_index_range_tree(uint32_t index,
+                                                        SEL_TREE *range_tree,
+                                                        optimizer::Parameter *param,
+                                                        uint32_t *param_idx);
+
+static bool get_constant_key_infix(KEY *index_info,
+                                   optimizer::SEL_ARG *index_range_tree,
+                                   KEY_PART_INFO *first_non_group_part,
+                                   KEY_PART_INFO *min_max_arg_part,
+                                   KEY_PART_INFO *last_part,
+                                   Session *session,
+                                   unsigned char *key_infix,
+                                   uint32_t *key_infix_len,
+                                   KEY_PART_INFO **first_non_infix_part);
+
 static bool check_group_min_max_predicates(COND *cond, Item_field *min_max_arg_item);
 
 static void
-cost_group_min_max(Table* table, KEY *index_info, uint32_t used_key_parts,
-                   uint32_t group_key_parts, SEL_TREE *range_tree,
-                   SEL_ARG *index_tree, ha_rows quick_prefix_records,
-                   bool have_min, bool have_max,
-                   double *read_cost, ha_rows *records);
+cost_group_min_max(Table* table,
+                   KEY *index_info,
+                   uint32_t used_key_parts,
+                   uint32_t group_key_parts,
+                   SEL_TREE *range_tree,
+                   optimizer::SEL_ARG *index_tree,
+                   ha_rows quick_prefix_records,
+                   bool have_min,
+                   bool have_max,
+                   double *read_cost,
+                   ha_rows *records);
 
 
 /*
@@ -8102,9 +6106,8 @@
     If mem_root == NULL
     - NULL
 */
-
 static TRP_GROUP_MIN_MAX *
-get_best_group_min_max(PARAM *param, SEL_TREE *tree)
+get_best_group_min_max(optimizer::Parameter *param, SEL_TREE *tree)
 {
   Session *session= param->session;
   JOIN *join= session->lex->current_select->join;
@@ -8122,16 +6125,17 @@
   uint32_t key_infix_len= 0;          /* Length of key_infix. */
   TRP_GROUP_MIN_MAX *read_plan= NULL; /* The eventually constructed TRP. */
   uint32_t key_part_nr;
-  order_st *tmp_group;
-  Item *item;
-  Item_field *item_field;
+  order_st *tmp_group= NULL;
+  Item *item= NULL;
+  Item_field *item_field= NULL;
 
   /* Perform few 'cheap' tests whether this access method is applicable. */
-  if (!join)
+  if (! join)
     return NULL;        /* This is not a select statement. */
+
   if ((join->tables != 1) ||  /* The query must reference one table. */
-      ((!join->group_list) && /* Neither GROUP BY nor a DISTINCT query. */
-       (!join->select_distinct)) ||
+      ((! join->group_list) && /* Neither GROUP BY nor a DISTINCT query. */
+       (! join->select_distinct)) ||
       (join->select_lex->olap == ROLLUP_TYPE)) /* Check (B3) for ROLLUP */
     return NULL;
   if (table->s->keys == 0)        /* There are no indexes to use. */
@@ -8143,9 +6147,10 @@
   /* Check (SA1,SA4) and store the only MIN/MAX argument - the C attribute.*/
   if (join->make_sum_func_list(join->all_fields, join->fields_list, 1))
     return NULL;
+
   if (join->sum_funcs[0])
   {
-    Item_sum *min_max_item;
+    Item_sum *min_max_item= NULL;
     Item_sum **func_ptr= join->sum_funcs;
     while ((min_max_item= *(func_ptr++)))
     {
@@ -8195,7 +6200,7 @@
   KEY *cur_index_info= table->key_info;
   KEY *cur_index_info_end= cur_index_info + table->s->keys;
   KEY_PART_INFO *cur_part= NULL;
-  KEY_PART_INFO *end_part; /* Last part for loops. */
+  KEY_PART_INFO *end_part= NULL; /* Last part for loops. */
   /* Last index part. */
   KEY_PART_INFO *last_part= NULL;
   KEY_PART_INFO *first_non_group_part= NULL;
@@ -8206,25 +6211,26 @@
   /* Cost-related variables for the best index so far. */
   double best_read_cost= DBL_MAX;
   ha_rows best_records= 0;
-  SEL_ARG *best_index_tree= NULL;
+  optimizer::SEL_ARG *best_index_tree= NULL;
   ha_rows best_quick_prefix_records= 0;
   uint32_t best_param_idx= 0;
   double cur_read_cost= DBL_MAX;
   ha_rows cur_records;
-  SEL_ARG *cur_index_tree= NULL;
+  optimizer::SEL_ARG *cur_index_tree= NULL;
   ha_rows cur_quick_prefix_records= 0;
-  uint32_t cur_param_idx=MAX_KEY;
+  uint32_t cur_param_idx= MAX_KEY;
   key_map used_key_parts_map;
   uint32_t cur_key_infix_len= 0;
   unsigned char cur_key_infix[MAX_KEY_LENGTH];
   uint32_t cur_used_key_parts= 0;
   uint32_t pk= param->table->s->primary_key;
 
-  for (uint32_t cur_index= 0 ; cur_index_info != cur_index_info_end ;
+  for (uint32_t cur_index= 0;
+       cur_index_info != cur_index_info_end;
        cur_index_info++, cur_index++)
   {
     /* Check (B1) - if current index is covering. */
-    if (!table->covering_keys.test(cur_index))
+    if (! table->covering_keys.test(cur_index))
       goto next_index;
 
     /*
@@ -8248,7 +6254,7 @@
           part of 'cur_index'
         */
         if ((cur_field->isReadSet()) &&
-            !cur_field->part_of_key_not_clustered.test(cur_index))
+            ! cur_field->part_of_key_not_clustered.test(cur_index))
           goto next_index;                  // Field was not part of key
       }
     }
@@ -8261,7 +6267,8 @@
       cur_part= cur_index_info->key_part;
       end_part= cur_part + cur_index_info->key_parts;
       /* Iterate in parallel over the GROUP list and the index parts. */
-      for (tmp_group= join->group_list; tmp_group && (cur_part != end_part);
+      for (tmp_group= join->group_list;
+           tmp_group && (cur_part != end_part);
            tmp_group= tmp_group->next, cur_part++)
       {
         /*
@@ -8319,7 +6326,8 @@
         all_parts have all bits set from 0 to (max_key_part-1).
         cur_parts have bits set for only used keyparts.
       */
-      key_map all_parts, cur_parts;
+      key_map all_parts;
+      key_map cur_parts;
       for (uint32_t pos= 0; pos < max_key_part; pos++)
         all_parts.set(pos);
       cur_parts= used_key_parts_map >> 1;
@@ -8363,19 +6371,27 @@
                              NULL :
                            NULL;
     if (first_non_group_part &&
-        (!min_max_arg_part || (min_max_arg_part - first_non_group_part > 0)))
+        (! min_max_arg_part || (min_max_arg_part - first_non_group_part > 0)))
     {
       if (tree)
       {
         uint32_t dummy;
-        SEL_ARG *index_range_tree= get_index_range_tree(cur_index, tree, param,
-                                                        &dummy);
-        if (!get_constant_key_infix(cur_index_info, index_range_tree,
-                                    first_non_group_part, min_max_arg_part,
-                                    last_part, session, cur_key_infix, 
-                                    &cur_key_infix_len,
-                                    &first_non_infix_part))
+        optimizer::SEL_ARG *index_range_tree= get_index_range_tree(cur_index,
+                                                                   tree,
+                                                                   param,
+                                                                   &dummy);
+        if (! get_constant_key_infix(cur_index_info,
+                                     index_range_tree,
+                                     first_non_group_part,
+                                     min_max_arg_part,
+                                     last_part,
+                                     session,
+                                     cur_key_infix,
+                                     &cur_key_infix_len,
+                                     &first_non_infix_part))
+        {
           goto next_index;
+        }
       }
       else if (min_max_arg_part &&
                (min_max_arg_part - first_non_group_part > 0))
@@ -8405,7 +6421,8 @@
         key_part_range[1]= last_part;
 
         /* Check if cur_part is referenced in the WHERE clause. */
-        if (join->conds->walk(&Item::find_item_in_field_list_processor, 0,
+        if (join->conds->walk(&Item::find_item_in_field_list_processor,
+                              0,
                               (unsigned char*) key_part_range))
           goto next_index;
       }
@@ -8435,22 +6452,34 @@
     if (tree)
     {
       /* Find the SEL_ARG sub-tree that corresponds to the chosen index. */
-      cur_index_tree= get_index_range_tree(cur_index, tree, param,
+      cur_index_tree= get_index_range_tree(cur_index,
+                                           tree,
+                                           param,
                                            &cur_param_idx);
       /* Check if this range tree can be used for prefix retrieval. */
       COST_VECT dummy_cost;
       uint32_t mrr_flags= HA_MRR_USE_DEFAULT_IMPL;
-      uint32_t mrr_bufsize=0;
-      cur_quick_prefix_records= check_quick_select(param, cur_param_idx,
+      uint32_t mrr_bufsize= 0;
+      cur_quick_prefix_records= check_quick_select(param,
+                                                   cur_param_idx,
                                                    false /*don't care*/,
-                                                   cur_index_tree, true,
-                                                   &mrr_flags, &mrr_bufsize,
+                                                   cur_index_tree,
+                                                   true,
+                                                   &mrr_flags,
+                                                   &mrr_bufsize,
                                                    &dummy_cost);
     }
-    cost_group_min_max(table, cur_index_info, cur_used_key_parts,
-                       cur_group_key_parts, tree, cur_index_tree,
-                       cur_quick_prefix_records, have_min, have_max,
-                       &cur_read_cost, &cur_records);
+    cost_group_min_max(table,
+                       cur_index_info,
+                       cur_used_key_parts,
+                       cur_group_key_parts,
+                       tree,
+                       cur_index_tree,
+                       cur_quick_prefix_records,
+                       have_min,
+                       have_max,
+                       &cur_read_cost,
+                       &cur_records);
     /*
       If cur_read_cost is lower than best_read_cost use cur_index.
       Do not compare doubles directly because they may have different
@@ -8469,8 +6498,12 @@
       group_key_parts= cur_group_key_parts;
       group_prefix_len= cur_group_prefix_len;
       key_infix_len= cur_key_infix_len;
+
       if (key_infix_len)
-        memcpy (key_infix, cur_key_infix, sizeof (key_infix));
+      {
+        memcpy(key_infix, cur_key_infix, sizeof (key_infix));
+      }
+
       used_key_parts= cur_used_key_parts;
     }
 
@@ -8479,7 +6512,7 @@
     cur_group_prefix_len= 0;
     cur_key_infix_len= 0;
   }
-  if (!index_info) /* No usable index found. */
+  if (! index_info) /* No usable index found. */
     return NULL;
 
   /* Check (SA3) for the where clause. */
@@ -8488,21 +6521,28 @@
     return NULL;
 
   /* The query passes all tests, so construct a new TRP object. */
-  read_plan= new (param->mem_root)
-                 TRP_GROUP_MIN_MAX(have_min, have_max, min_max_arg_part,
-                                   group_prefix_len, used_key_parts,
-                                   group_key_parts, index_info, index,
-                                   key_infix_len,
-                                   (key_infix_len > 0) ? key_infix : NULL,
-                                   tree, best_index_tree, best_param_idx,
-                                   best_quick_prefix_records);
+  read_plan=
+    new(param->mem_root) TRP_GROUP_MIN_MAX(have_min,
+                                           have_max,
+                                           min_max_arg_part,
+                                           group_prefix_len,
+                                           used_key_parts,
+                                           group_key_parts,
+                                           index_info,
+                                           index,
+                                           key_infix_len,
+                                           (key_infix_len > 0) ? key_infix : NULL,
+                                           tree,
+                                           best_index_tree,
+                                           best_param_idx,
+                                           best_quick_prefix_records);
   if (read_plan)
   {
     if (tree && read_plan->quick_prefix_records == 0)
       return NULL;
 
     read_plan->read_cost= best_read_cost;
-    read_plan->records=   best_records;
+    read_plan->records= best_records;
   }
 
   return read_plan;
@@ -8539,10 +6579,10 @@
   if (cond_type == Item::COND_ITEM) /* 'AND' or 'OR' */
   {
     List_iterator_fast<Item> li(*((Item_cond*) cond)->argument_list());
-    Item *and_or_arg;
+    Item *and_or_arg= NULL;
     while ((and_or_arg= li++))
     {
-      if (!check_group_min_max_predicates(and_or_arg, min_max_arg_item))
+      if (! check_group_min_max_predicates(and_or_arg, min_max_arg_item))
         return false;
     }
     return true;
@@ -8566,7 +6606,7 @@
   /* Test if cond references only group-by or non-group fields. */
   Item_func *pred= (Item_func*) cond;
   Item **arguments= pred->arguments();
-  Item *cur_arg;
+  Item *cur_arg= NULL;
   for (uint32_t arg_idx= 0; arg_idx < pred->argument_count (); arg_idx++)
   {
     cur_arg= arguments[arg_idx]->real_item();
@@ -8594,9 +6634,9 @@
         /* Check that pred compares min_max_arg_item with a constant. */
         Item *args[3];
         memset(args, 0, 3 * sizeof(Item*));
-        bool inv;
+        bool inv= false;
         /* Test if this is a comparison of a field and a constant. */
-        if (! optimizer::simple_pred(pred, args, &inv))
+        if (! optimizer::simple_pred(pred, args, inv))
           return false;
 
         /* Check for compatible string comparisons - similar to get_mm_leaf. */
@@ -8615,7 +6655,9 @@
              */
              (args[1]->result_type() != STRING_RESULT &&
               min_max_arg_item->field->cmp_type() != args[1]->result_type())))
+        {
           return false;
+        }
       }
     }
     else if (cur_arg->type() == Item::FUNC_ITEM)
@@ -8665,17 +6707,19 @@
     true  if the index passes the test
     false o/w
 */
-
 static bool
-get_constant_key_infix(KEY *, SEL_ARG *index_range_tree,
+get_constant_key_infix(KEY *,
+                       optimizer::SEL_ARG *index_range_tree,
                        KEY_PART_INFO *first_non_group_part,
                        KEY_PART_INFO *min_max_arg_part,
                        KEY_PART_INFO *last_part,
-                       Session *, unsigned char *key_infix, uint32_t *key_infix_len,
+                       Session *,
+                       unsigned char *key_infix,
+                       uint32_t *key_infix_len,
                        KEY_PART_INFO **first_non_infix_part)
 {
-  SEL_ARG       *cur_range;
-  KEY_PART_INFO *cur_part;
+  optimizer::SEL_ARG *cur_range= NULL;
+  KEY_PART_INFO *cur_part= NULL;
   /* End part for the first loop below. */
   KEY_PART_INFO *end_part= min_max_arg_part ? min_max_arg_part : last_part;
 
@@ -8693,7 +6737,7 @@
       if (cur_range->field->eq(cur_part->field))
         break;
     }
-    if (!cur_range)
+    if (! cur_range)
     {
       if (min_max_arg_part)
         return false; /* The current keypart has no range predicates at all. */
@@ -8709,7 +6753,8 @@
       return false; /* This is not the only range predicate for the field. */
     if ((cur_range->min_flag & NO_MIN_RANGE) ||
         (cur_range->max_flag & NO_MAX_RANGE) ||
-        (cur_range->min_flag & NEAR_MIN) || (cur_range->max_flag & NEAR_MAX))
+        (cur_range->min_flag & NEAR_MIN) ||
+        (cur_range->max_flag & NEAR_MAX))
       return false;
 
     uint32_t field_length= cur_part->store_length;
@@ -8749,11 +6794,11 @@
     Positive number which is the consecutive number of the key part, or
     0 if field does not reference any index field.
 */
-
 static inline uint
 get_field_keypart(KEY *index, Field *field)
 {
-  KEY_PART_INFO *part, *end;
+  KEY_PART_INFO *part= NULL;
+  KEY_PART_INFO *end= NULL;
 
   for (part= index->key_part, end= part + index->key_parts; part < end; part++)
   {
@@ -8771,23 +6816,24 @@
     get_index_range_tree()
     index     [in]  The ID of the index being looked for
     range_tree[in]  Tree of ranges being searched
-    param     [in]  PARAM from SQL_SELECT::test_quick_select
-    param_idx [out] Index in the array PARAM::key that corresponds to 'index'
+    param     [in]  Parameter from SqlSelect::test_quick_select
+    param_idx [out] Index in the array Parameter::key that corresponds to 'index'
 
   DESCRIPTION
 
     A SEL_TREE contains range trees for all usable indexes. This procedure
     finds the SEL_ARG sub-tree for 'index'. The members of a SEL_TREE are
-    ordered in the same way as the members of PARAM::key, thus we first find
-    the corresponding index in the array PARAM::key. This index is returned
+    ordered in the same way as the members of Parameter::key, thus we first find
+    the corresponding index in the array Parameter::key. This index is returned
     through the variable param_idx, to be used later as argument of
     check_quick_select().
 
   RETURN
     Pointer to the SEL_ARG subtree that corresponds to index.
 */
-
-SEL_ARG * get_index_range_tree(uint32_t index, SEL_TREE* range_tree, PARAM *param,
+optimizer::SEL_ARG * get_index_range_tree(uint32_t index,
+                               SEL_TREE* range_tree,
+                               optimizer::Parameter *param,
                                uint32_t *param_idx)
 {
   uint32_t idx= 0; /* Index nr in param->key_parts */
@@ -8861,12 +6907,17 @@
   RETURN
     None
 */
-
-void cost_group_min_max(Table* table, KEY *index_info, uint32_t used_key_parts,
-                        uint32_t group_key_parts, SEL_TREE *range_tree,
-                        SEL_ARG *, ha_rows quick_prefix_records,
-                        bool have_min, bool have_max,
-                        double *read_cost, ha_rows *records)
+void cost_group_min_max(Table* table,
+                        KEY *index_info,
+                        uint32_t used_key_parts,
+                        uint32_t group_key_parts,
+                        SEL_TREE *range_tree,
+                        optimizer::SEL_ARG *,
+                        ha_rows quick_prefix_records,
+                        bool have_min,
+                        bool have_max,
+                        double *read_cost,
+                        ha_rows *records)
 {
   ha_rows table_records;
   uint32_t num_groups;
@@ -8884,7 +6935,7 @@
   keys_per_block= (table->cursor->stats.block_size / 2 /
                    (index_info->key_length + table->cursor->ref_length)
                         + 1);
-  num_blocks= (uint32_t)(table_records / keys_per_block) + 1;
+  num_blocks= (uint32_t) (table_records / keys_per_block) + 1;
 
   /* Compute the number of keys in a group. */
   keys_per_group= index_info->rec_per_key[group_key_parts - 1];
@@ -8927,7 +6978,7 @@
   /*
     TODO: If there is no WHERE clause and no other expressions, there should be
     no CPU cost. We leave it here to make this cost comparable to that of index
-    scan as computed in SQL_SELECT::test_quick_select().
+    scan as computed in SqlSelect::test_quick_select().
   */
   cpu_cost= (double) num_groups / TIME_FOR_COMPARE;
 
@@ -8956,26 +7007,25 @@
     New QUICK_GROUP_MIN_MAX_SELECT object if successfully created,
     NULL otherwise.
 */
-
-optimizer::QUICK_SELECT_I *
-TRP_GROUP_MIN_MAX::make_quick(PARAM *param, bool, MEM_ROOT *parent_alloc)
+optimizer::QuickSelectInterface *
+TRP_GROUP_MIN_MAX::make_quick(optimizer::Parameter *param, bool, MEM_ROOT *parent_alloc)
 {
-  optimizer::QUICK_GROUP_MIN_MAX_SELECT *quick;
+  optimizer::QUICK_GROUP_MIN_MAX_SELECT *quick= NULL;
 
   quick= new optimizer::QUICK_GROUP_MIN_MAX_SELECT(param->table,
                                                    param->session->lex->current_select->join,
-                                                   have_min, 
-                                                   have_max, 
+                                                   have_min,
+                                                   have_max,
                                                    min_max_arg_part,
-                                                   group_prefix_len, 
+                                                   group_prefix_len,
                                                    group_key_parts,
-                                                   used_key_parts, 
-                                                   index_info, 
+                                                   used_key_parts,
+                                                   index_info,
                                                    index,
-                                                   read_cost, 
-                                                   records, 
+                                                   read_cost,
+                                                   records,
                                                    key_infix_len,
-                                                   key_infix, 
+                                                   key_infix,
                                                    parent_alloc);
   if (! quick)
   {
@@ -8997,23 +7047,23 @@
     }
     else
     {
-      /* Make a QUICK_RANGE_SELECT to be used for group prefix retrieval. */
-      quick->quick_prefix_select= optimizer::get_quick_select(param, 
+      /* Make a QuickRangeSelect to be used for group prefix retrieval. */
+      quick->quick_prefix_select= optimizer::get_quick_select(param,
                                                               param_idx,
                                                               index_tree,
-                                                              HA_MRR_USE_DEFAULT_IMPL, 
+                                                              HA_MRR_USE_DEFAULT_IMPL,
                                                               0,
                                                               &quick->alloc);
     }
 
     /*
       Extract the SEL_ARG subtree that contains only ranges for the MIN/MAX
-      attribute, and create an array of QUICK_RANGES to be used by the
+      attribute, and create an array of QuickRanges to be used by the
       new quick select.
     */
     if (min_max_arg_part)
     {
-      SEL_ARG *min_max_range= index_tree;
+      optimizer::SEL_ARG *min_max_range= index_tree;
       while (min_max_range) /* Find the tree for the MIN/MAX key part. */
       {
         if (min_max_range->field->eq(min_max_arg_part->field))
@@ -9023,7 +7073,7 @@
       /* Scroll to the leftmost interval for the MIN/MAX argument. */
       while (min_max_range && min_max_range->prev)
         min_max_range= min_max_range->prev;
-      /* Create an array of QUICK_RANGEs for the MIN/MAX argument. */
+      /* Create an array of QuickRanges for the MIN/MAX argument. */
       while (min_max_range)
       {
         if (quick->add_range(min_max_range))
@@ -9069,41 +7119,40 @@
   RETURN
     None
 */
-
 optimizer::QUICK_GROUP_MIN_MAX_SELECT::
-QUICK_GROUP_MIN_MAX_SELECT(Table *table, 
-                           JOIN *join_arg, 
+QUICK_GROUP_MIN_MAX_SELECT(Table *table,
+                           JOIN *join_arg,
                            bool have_min_arg,
                            bool have_max_arg,
                            KEY_PART_INFO *min_max_arg_part_arg,
-                           uint32_t group_prefix_len_arg, 
+                           uint32_t group_prefix_len_arg,
                            uint32_t group_key_parts_arg,
-                           uint32_t used_key_parts_arg, 
+                           uint32_t used_key_parts_arg,
                            KEY *index_info_arg,
-                           uint32_t use_index, 
+                           uint32_t use_index,
                            double read_cost_arg,
-                           ha_rows records_arg, 
+                           ha_rows records_arg,
                            uint32_t key_infix_len_arg,
-                           unsigned char *key_infix_arg, 
+                           unsigned char *key_infix_arg,
                            MEM_ROOT *parent_alloc)
   :
-    join(join_arg), 
+    join(join_arg),
     index_info(index_info_arg),
-   group_prefix_len(group_prefix_len_arg),
-   group_key_parts(group_key_parts_arg), 
-   have_min(have_min_arg),
-   have_max(have_max_arg), 
-   seen_first_key(false),
-   min_max_arg_part(min_max_arg_part_arg), 
-   key_infix(key_infix_arg),
-   key_infix_len(key_infix_len_arg), 
-   min_functions_it(NULL),
-   max_functions_it(NULL)
+    group_prefix_len(group_prefix_len_arg),
+    group_key_parts(group_key_parts_arg),
+    have_min(have_min_arg),
+    have_max(have_max_arg),
+    seen_first_key(false),
+    min_max_arg_part(min_max_arg_part_arg),
+    key_infix(key_infix_arg),
+    key_infix_len(key_infix_len_arg),
+    min_functions_it(NULL),
+    max_functions_it(NULL)
 {
-  head=       table;
-  cursor=       head->cursor;
-  index=      use_index;
-  record=     head->record[0];
+  head= table;
+  cursor= head->cursor;
+  index= use_index;
+  record= head->record[0];
   tmp_record= head->record[1];
   read_time= read_cost_arg;
   records= records_arg;
@@ -9117,8 +7166,8 @@
     We can't have parent_alloc set as the init function can't handle this case
     yet.
   */
-  assert(!parent_alloc);
-  if (!parent_alloc)
+  assert(! parent_alloc);
+  if (! parent_alloc)
   {
     init_sql_alloc(&alloc, join->session->variables.range_alloc_block_size, 0);
     join->session->mem_root= &alloc;
@@ -9144,20 +7193,19 @@
     0      OK
     other  Error code
 */
-
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::init()
 {
   if (group_prefix) /* Already initialized. */
     return 0;
 
-  if (!(last_prefix= (unsigned char*) alloc_root(&alloc, group_prefix_len)))
+  if (! (last_prefix= (unsigned char*) alloc_root(&alloc, group_prefix_len)))
       return 1;
   /*
     We may use group_prefix to store keys with all select fields, so allocate
     enough space for it.
   */
-  if (!(group_prefix= (unsigned char*) alloc_root(&alloc,
-                                         real_prefix_len + min_max_arg_len)))
+  if (! (group_prefix= (unsigned char*) alloc_root(&alloc,
+                                                   real_prefix_len + min_max_arg_len)))
     return 1;
 
   if (key_infix_len > 0)
@@ -9167,7 +7215,7 @@
       allocate a new buffer and copy the key_infix into it.
     */
     unsigned char *tmp_key_infix= (unsigned char*) alloc_root(&alloc, key_infix_len);
-    if (!tmp_key_infix)
+    if (! tmp_key_infix)
       return 1;
     memcpy(tmp_key_infix, this->key_infix, key_infix_len);
     this->key_infix= tmp_key_infix;
@@ -9175,25 +7223,25 @@
 
   if (min_max_arg_part)
   {
-    if (my_init_dynamic_array(&min_max_ranges, sizeof(optimizer::QUICK_RANGE*), 16, 16))
+    if (my_init_dynamic_array(&min_max_ranges, sizeof(optimizer::QuickRange*), 16, 16))
       return 1;
 
     if (have_min)
     {
-      if (!(min_functions= new List<Item_sum>))
+      if (! (min_functions= new List<Item_sum>))
         return 1;
     }
     else
       min_functions= NULL;
     if (have_max)
     {
-      if (!(max_functions= new List<Item_sum>))
+      if (! (max_functions= new List<Item_sum>))
         return 1;
     }
     else
       max_functions= NULL;
 
-    Item_sum *min_max_item;
+    Item_sum *min_max_item= NULL;
     Item_sum **func_ptr= join->sum_funcs;
     while ((min_max_item= *(func_ptr++)))
     {
@@ -9205,13 +7253,13 @@
 
     if (have_min)
     {
-      if (!(min_functions_it= new List_iterator<Item_sum>(*min_functions)))
+      if (! (min_functions_it= new List_iterator<Item_sum>(*min_functions)))
         return 1;
     }
 
     if (have_max)
     {
-      if (!(max_functions_it= new List_iterator<Item_sum>(*max_functions)))
+      if (! (max_functions_it= new List_iterator<Item_sum>(*max_functions)))
         return 1;
     }
   }
@@ -9247,7 +7295,7 @@
     sel_range  Range object from which a
 
   NOTES
-    Construct a new QUICK_RANGE object from a SEL_ARG object, and
+    Construct a new QuickRange object from a SEL_ARG object, and
     add it to the array min_max_ranges. If sel_arg is an infinite
     range, e.g. (x < 5 or x > 4), then skip it and do not construct
     a quick range.
@@ -9256,18 +7304,17 @@
     false on success
     true  otherwise
 */
-
 bool optimizer::QUICK_GROUP_MIN_MAX_SELECT::add_range(SEL_ARG *sel_range)
 {
-  optimizer::QUICK_RANGE *range;
+  optimizer::QuickRange *range= NULL;
   uint32_t range_flag= sel_range->min_flag | sel_range->max_flag;
 
   /* Skip (-inf,+inf) ranges, e.g. (x < 5 or x > 4). */
   if ((range_flag & NO_MIN_RANGE) && (range_flag & NO_MAX_RANGE))
     return false;
 
-  if (!(sel_range->min_flag & NO_MIN_RANGE) &&
-      !(sel_range->max_flag & NO_MAX_RANGE))
+  if (! (sel_range->min_flag & NO_MIN_RANGE) &&
+      ! (sel_range->max_flag & NO_MAX_RANGE))
   {
     if (sel_range->maybe_null &&
         sel_range->min_value[0] && sel_range->max_value[0])
@@ -9276,12 +7323,14 @@
                     min_max_arg_len) == 0)
       range_flag|= EQ_RANGE;  /* equality condition */
   }
-  range= new optimizer::QUICK_RANGE(sel_range->min_value, min_max_arg_len,
-                         make_keypart_map(sel_range->part),
-                         sel_range->max_value, min_max_arg_len,
-                         make_keypart_map(sel_range->part),
-                         range_flag);
-  if (!range)
+  range= new optimizer::QuickRange(sel_range->min_value,
+                                   min_max_arg_len,
+                                   make_keypart_map(sel_range->part),
+                                   sel_range->max_value,
+                                   min_max_arg_len,
+                                   make_keypart_map(sel_range->part),
+                                   range_flag);
+  if (! range)
     return true;
   if (insert_dynamic(&min_max_ranges, (unsigned char*)&range))
     return true;
@@ -9311,12 +7360,12 @@
   if (quick_prefix_select &&
       group_prefix_len < quick_prefix_select->max_used_key_length)
   {
-    DYNAMIC_ARRAY *arr;
+    DYNAMIC_ARRAY *arr= NULL;
     uint32_t inx;
 
     for (inx= 0, arr= &quick_prefix_select->ranges; inx < arr->elements; inx++)
     {
-      optimizer::QUICK_RANGE *range;
+      optimizer::QuickRange *range= NULL;
 
       get_dynamic(arr, (unsigned char*)&range, inx);
       range->flag &= ~(NEAR_MIN | NEAR_MAX);
@@ -9345,18 +7394,18 @@
   RETURN
     None
 */
-
 void optimizer::QUICK_GROUP_MIN_MAX_SELECT::update_key_stat()
 {
   max_used_key_length= real_prefix_len;
   if (min_max_ranges.elements > 0)
   {
-    optimizer::QUICK_RANGE *cur_range;
+    optimizer::QuickRange *cur_range= NULL;
     if (have_min)
     { /* Check if the right-most range has a lower boundary. */
-      get_dynamic(&min_max_ranges, (unsigned char*)&cur_range,
+      get_dynamic(&min_max_ranges,
+                  (unsigned char*) &cur_range,
                   min_max_ranges.elements - 1);
-      if (!(cur_range->flag & NO_MIN_RANGE))
+      if (! (cur_range->flag & NO_MIN_RANGE))
       {
         max_used_key_length+= min_max_arg_len;
         used_key_parts++;
@@ -9366,7 +7415,7 @@
     if (have_max)
     { /* Check if the left-most range has an upper boundary. */
       get_dynamic(&min_max_ranges, (unsigned char*)&cur_range, 0);
-      if (!(cur_range->flag & NO_MAX_RANGE))
+      if (! (cur_range->flag & NO_MAX_RANGE))
       {
         max_used_key_length+= min_max_arg_len;
         used_key_parts++;
@@ -9405,7 +7454,6 @@
     0      OK
     other  Error code
 */
-
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::reset(void)
 {
   int result;
@@ -9452,12 +7500,11 @@
     HA_ERR_END_OF_FILE if returned all keys
     other              if some error occurred
 */
-
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::get_next()
 {
   int min_res= 0;
   int max_res= 0;
-  int result;
+  int result= 0;
   int is_last_prefix= 0;
 
   /*
@@ -9471,7 +7518,7 @@
       Check if this is the last group prefix. Notice that at this point
       this->record contains the current prefix in record format.
     */
-    if (!result)
+    if (! result)
     {
       is_last_prefix= key_cmp(index_info->key_part, last_prefix,
                               group_prefix_len);
@@ -9506,10 +7553,11 @@
       are equality predicates for the key parts after the group, find the
       first sub-group with the extended prefix.
     */
-    if (!have_min && !have_max && key_infix_len > 0)
-      result= cursor->index_read_map(record, group_prefix,
-                                   make_prev_keypart_map(real_key_parts),
-                                   HA_READ_KEY_EXACT);
+    if (! have_min && ! have_max && key_infix_len > 0)
+      result= cursor->index_read_map(record,
+                                     group_prefix,
+                                     make_prev_keypart_map(real_key_parts),
+                                     HA_READ_KEY_EXACT);
 
     result= have_min ? min_res : have_max ? max_res : result;
   } while ((result == HA_ERR_KEY_NOT_FOUND || result == HA_ERR_END_OF_FILE) &&
@@ -9554,7 +7602,6 @@
     HA_ERR_END_OF_FILE   - "" -
     other                if some error occurred
 */
-
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::next_min()
 {
   int result= 0;
@@ -9570,9 +7617,10 @@
     /* Apply the constant equality conditions to the non-group select fields */
     if (key_infix_len > 0)
     {
-      if ((result= cursor->index_read_map(record, group_prefix,
-                                        make_prev_keypart_map(real_key_parts),
-                                        HA_READ_KEY_EXACT)))
+      if ((result= cursor->index_read_map(record,
+                                          group_prefix,
+                                          make_prev_keypart_map(real_key_parts),
+                                          HA_READ_KEY_EXACT)))
         return result;
     }
 
@@ -9587,9 +7635,10 @@
     {
       /* Find the first subsequent record without NULL in the MIN/MAX field. */
       key_copy(tmp_record, record, index_info, 0);
-      result= cursor->index_read_map(record, tmp_record,
-                                   make_keypart_map(real_key_parts),
-                                   HA_READ_AFTER_KEY);
+      result= cursor->index_read_map(record,
+                                     tmp_record,
+                                     make_keypart_map(real_key_parts),
+                                     HA_READ_AFTER_KEY);
       /*
         Check if the new record belongs to the current group by comparing its
         prefix with the group's prefix. If it is from the next group, then the
@@ -9600,7 +7649,7 @@
         next call to next_min(), and to save one lookup in the next call. For
         this add a new member 'this->next_group_prefix'.
       */
-      if (!result)
+      if (! result)
       {
         if (key_cmp(index_info->key_part, group_prefix, real_prefix_len))
           key_restore(record, tmp_record, index_info, 0);
@@ -9633,18 +7682,18 @@
     HA_ERR_END_OF_FILE	 - "" -
     other                if some error occurred
 */
-
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::next_max()
 {
-  int result;
+  int result= 0;
 
   /* Get the last key in the (possibly extended) group. */
   if (min_max_ranges.elements > 0)
     result= next_max_in_range();
   else
-    result= cursor->index_read_map(record, group_prefix,
-                                 make_prev_keypart_map(real_key_parts),
-                                 HA_READ_PREFIX_LAST);
+    result= cursor->index_read_map(record,
+                                   group_prefix,
+                                   make_prev_keypart_map(real_key_parts),
+                                   HA_READ_PREFIX_LAST);
   return result;
 }
 
@@ -9658,7 +7707,7 @@
   DESCRIPTION
     Determine the prefix of the next group that satisfies the query conditions.
     If there is a range condition referencing the group attributes, use a
-    QUICK_RANGE_SELECT object to retrieve the *first* key that satisfies the
+    QuickRangeSelect object to retrieve the *first* key that satisfies the
     condition. If there is a key infix of constants, append this infix
     immediately after the group attributes. The possibly extended prefix is
     stored in this->group_prefix. The first key of the found group is stored in
@@ -9672,19 +7721,20 @@
 */
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::next_prefix()
 {
-  int result;
+  int result= 0;
 
   if (quick_prefix_select)
   {
     unsigned char *cur_prefix= seen_first_key ? group_prefix : NULL;
     if ((result= quick_prefix_select->get_next_prefix(group_prefix_len,
-                         make_prev_keypart_map(group_key_parts), cur_prefix)))
+                                                      make_prev_keypart_map(group_key_parts),
+                                                      cur_prefix)))
       return result;
     seen_first_key= true;
   }
   else
   {
-    if (!seen_first_key)
+    if (! seen_first_key)
     {
       result= cursor->index_first(record);
       if (result)
@@ -9694,9 +7744,10 @@
     else
     {
       /* Load the first key in this group into record. */
-      result= cursor->index_read_map(record, group_prefix,
-                                   make_prev_keypart_map(group_key_parts),
-                                   HA_READ_AFTER_KEY);
+      result= cursor->index_read_map(record,
+                                     group_prefix,
+                                     make_prev_keypart_map(group_key_parts),
+                                     HA_READ_AFTER_KEY);
       if (result)
         return result;
     }
@@ -9707,7 +7758,8 @@
   /* Append key_infix to group_prefix. */
   if (key_infix_len > 0)
     memcpy(group_prefix + group_prefix_len,
-           key_infix, key_infix_len);
+           key_infix,
+           key_infix_len);
 
   return 0;
 }
@@ -9733,12 +7785,11 @@
     HA_ERR_END_OF_FILE   - "" -
     other                if some error
 */
-
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::next_min_in_range()
 {
   ha_rkey_function find_flag;
   key_part_map keypart_map;
-  optimizer::QUICK_RANGE *cur_range;
+  optimizer::QuickRange *cur_range= NULL;
   bool found_null= false;
   int result= HA_ERR_KEY_NOT_FOUND;
   basic_string<unsigned char> max_key;
@@ -9756,7 +7807,8 @@
       boundary of cur_range, there is no need to check this range.
     */
     if (range_idx != 0 && !(cur_range->flag & NO_MAX_RANGE) &&
-        (key_cmp(min_max_arg_part, (const unsigned char*) cur_range->max_key,
+        (key_cmp(min_max_arg_part,
+                 (const unsigned char*) cur_range->max_key,
                  min_max_arg_len) == 1))
       continue;
 
@@ -9768,7 +7820,8 @@
     else
     {
       /* Extend the search key with the lower boundary for this range. */
-      memcpy(group_prefix + real_prefix_len, cur_range->min_key,
+      memcpy(group_prefix + real_prefix_len,
+             cur_range->min_key,
              cur_range->min_length);
       keypart_map= make_keypart_map(real_key_parts);
       find_flag= (cur_range->flag & (EQ_RANGE | NULL_RANGE)) ?
@@ -9814,7 +7867,7 @@
     }
 
     /* If there is an upper limit, check if the found key is in the range. */
-    if ( !(cur_range->flag & NO_MAX_RANGE) )
+    if (! (cur_range->flag & NO_MAX_RANGE) )
     {
       /* Compose the MAX key for the range. */
       max_key.clear();
@@ -9824,8 +7877,8 @@
       int cmp_res= key_cmp(index_info->key_part,
                            max_key.data(),
                            real_prefix_len + min_max_arg_len);
-      if (!(((cur_range->flag & NEAR_MAX) && (cmp_res == -1)) ||
-            (cmp_res <= 0)))
+      if (! (((cur_range->flag & NEAR_MAX) && (cmp_res == -1)) ||
+          (cmp_res <= 0)))
       {
         result= HA_ERR_KEY_NOT_FOUND;
         continue;
@@ -9869,13 +7922,12 @@
     HA_ERR_END_OF_FILE   - "" -
     other                if some error
 */
-
 int optimizer::QUICK_GROUP_MIN_MAX_SELECT::next_max_in_range()
 {
   ha_rkey_function find_flag;
   key_part_map keypart_map;
-  optimizer::QUICK_RANGE *cur_range;
-  int result;
+  optimizer::QuickRange *cur_range= NULL;
+  int result= 0;
   basic_string<unsigned char> min_key;
   min_key.reserve(real_prefix_len + min_max_arg_len);
 
@@ -9890,8 +7942,9 @@
       boundary of cur_range, there is no need to check this range.
     */
     if (range_idx != min_max_ranges.elements &&
-        !(cur_range->flag & NO_MIN_RANGE) &&
-        (key_cmp(min_max_arg_part, (const unsigned char*) cur_range->min_key,
+        ! (cur_range->flag & NO_MIN_RANGE) &&
+        (key_cmp(min_max_arg_part,
+                 (const unsigned char*) cur_range->min_key,
                  min_max_arg_len) == -1))
       continue;
 
@@ -9903,7 +7956,8 @@
     else
     {
       /* Extend the search key with the upper boundary for this range. */
-      memcpy(group_prefix + real_prefix_len, cur_range->max_key,
+      memcpy(group_prefix + real_prefix_len,
+             cur_range->max_key,
              cur_range->max_length);
       keypart_map= make_keypart_map(real_key_parts);
       find_flag= (cur_range->flag & EQ_RANGE) ?
@@ -9934,7 +7988,7 @@
       continue;                                 // Row not found
 
     /* If there is a lower limit, check if the found key is in the range. */
-    if ( !(cur_range->flag & NO_MIN_RANGE) )
+    if (! (cur_range->flag & NO_MIN_RANGE) )
     {
       /* Compose the MIN key for the range. */
       min_key.clear();
@@ -9945,8 +7999,8 @@
       int cmp_res= key_cmp(index_info->key_part,
                            min_key.data(),
                            real_prefix_len + min_max_arg_len);
-      if (!(((cur_range->flag & NEAR_MIN) && (cmp_res == 1)) ||
-            (cmp_res >= 0)))
+      if (! (((cur_range->flag & NEAR_MIN) && (cmp_res == 1)) ||
+          (cmp_res >= 0)))
         continue;
     }
     /* If we got to this point, the current key qualifies as MAX. */
@@ -9978,10 +8032,9 @@
   RETURN
     None
 */
-
 void optimizer::QUICK_GROUP_MIN_MAX_SELECT::update_min_result()
 {
-  Item_sum *min_func;
+  Item_sum *min_func= NULL;
 
   min_functions_it->rewind();
   while ((min_func= (*min_functions_it)++))
@@ -10010,10 +8063,9 @@
   RETURN
     None
 */
-
 void optimizer::QUICK_GROUP_MIN_MAX_SELECT::update_max_result()
 {
-  Item_sum *max_func;
+  Item_sum *max_func= NULL;
 
   max_functions_it->rewind();
   while ((max_func= (*max_functions_it)++))
@@ -10035,7 +8087,6 @@
     indexes used by a quick select.
 
 */
-
 void optimizer::QUICK_GROUP_MIN_MAX_SELECT::add_keys_and_lengths(String *key_names,
                                                                  String *used_lengths)
 {
@@ -10046,17 +8097,18 @@
   used_lengths->append(buf, length);
 }
 
-static void print_sel_tree(PARAM *param, SEL_TREE *tree, key_map *tree_map, const char *)
+static void print_sel_tree(optimizer::Parameter *param, SEL_TREE *tree, key_map *tree_map, const char *)
 {
-  SEL_ARG **key,**end;
-  int idx;
+  optimizer::SEL_ARG **key= NULL;
+  optimizer::SEL_ARG **end= NULL;
+  int idx= 0;
   char buff[1024];
 
   String tmp(buff,sizeof(buff),&my_charset_bin);
   tmp.length(0);
-  for (idx= 0,key=tree->keys, end=key+param->keys ;
-       key != end ;
-       key++,idx++)
+  for (idx= 0, key=tree->keys, end= key + param->keys;
+       key != end;
+       key++, idx++)
   {
     if (tree_map->test(idx))
     {
@@ -10066,13 +8118,14 @@
       tmp.append(param->table->key_info[keynr].name);
     }
   }
-  if (!tmp.length())
+  if (! tmp.length())
     tmp.append(STRING_WITH_LEN("(empty)"));
 }
 
 
 static void print_ror_scans_arr(Table *table,
-                                const char *, struct st_ror_scan_info **start,
+                                const char *,
+                                struct st_ror_scan_info **start,
                                 struct st_ror_scan_info **end)
 {
   char buff[1024];
@@ -10093,6 +8146,6 @@
 *****************************************************************************/
 
 #ifdef HAVE_EXPLICIT_TEMPLATE_INSTANTIATION
-template class List<optimizer::QUICK_RANGE>;
-template class List_iterator<optimizer::QUICK_RANGE>;
+template class List<optimizer::QuickRange>;
+template class List_iterator<optimizer::QuickRange>;
 #endif

=== modified file 'drizzled/optimizer/range.h'
--- drizzled/optimizer/range.h	2009-12-05 23:29:59 +0000
+++ drizzled/optimizer/range.h	2009-12-08 00:38:13 +0000
@@ -27,7 +27,6 @@
 
 #include <queue>
 
-class PARAM;
 class JOIN;
 class TRP_ROR_INTERSECT; 
 typedef class Item COND;
@@ -50,7 +49,6 @@
   Field *field;
 } KEY_PART;
 
-class SEL_ARG;
 
 namespace drizzled
 {
@@ -58,34 +56,8 @@
 namespace optimizer
 {
 
-class QUICK_RANGE : public Sql_alloc 
-{
-public:
-  unsigned char *min_key;
-  unsigned char *max_key;
-  uint16_t min_length;
-  uint16_t max_length;
-  uint16_t flag;
-  key_part_map min_keypart_map; /**< bitmap of used keyparts in min_key */
-  key_part_map max_keypart_map; /**< bitmap of used keyparts in max_key */
-  QUICK_RANGE(); /**< Constructor for a "full range" */
-  QUICK_RANGE(const unsigned char *min_key_arg,
-              uint32_t min_length_arg,
-              key_part_map min_keypart_map_arg,
-	            const unsigned char *max_key_arg, 
-              uint32_t max_length_arg,
-              key_part_map max_keypart_map_arg,
-	            uint32_t flag_arg)
-    : 
-      min_key((unsigned char*) sql_memdup(min_key_arg,min_length_arg+1)),
-      max_key((unsigned char*) sql_memdup(max_key_arg,max_length_arg+1)),
-      min_length((uint16_t) min_length_arg),
-      max_length((uint16_t) max_length_arg),
-      flag((uint16_t) flag_arg),
-      min_keypart_map(min_keypart_map_arg),
-      max_keypart_map(max_keypart_map_arg)
-    {}
-};
+class Parameter;
+class SEL_ARG;
 
 /**
   Quick select interface.
@@ -129,7 +101,7 @@
     delete quick;
 
 */
-class QUICK_SELECT_I
+class QuickSelectInterface
 {
 public:
   bool sorted;
@@ -165,8 +137,8 @@
    */
   unsigned char *record;
 
-  QUICK_SELECT_I();
-  virtual ~QUICK_SELECT_I(){};
+  QuickSelectInterface();
+  virtual ~QuickSelectInterface(){};
 
   /**
    * Do post-constructor initialization.
@@ -284,241 +256,31 @@
 };
 
 struct st_qsel_param;
+class QuickRange;
+class QuickRangeSelect;
 
 /**
- * MRR range sequence, array<QUICK_RANGE> implementation: sequence traversal
+ * MRR range sequence, array<QuickRange> implementation: sequence traversal
  * context.
  */
 typedef struct st_quick_range_seq_ctx
 {
-  QUICK_RANGE **first;
-  QUICK_RANGE **cur;
-  QUICK_RANGE **last;
-} QUICK_RANGE_SEQ_CTX;
+  QuickRange **first;
+  QuickRange **cur;
+  QuickRange **last;
+} QuickRangeSequenceContext;
 
 range_seq_t quick_range_seq_init(void *init_param, uint32_t n_ranges, uint32_t flags);
+
 uint32_t quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range);
 
-/**
- * Quick select that does a range scan on a single key. 
- *
- * The records are returned in key order.
- * 
- */
-class QUICK_RANGE_SELECT : public QUICK_SELECT_I
-{
-protected:
-  Cursor *cursor;
-  DYNAMIC_ARRAY ranges; /**< ordered array of range ptrs */
-
-  /** Members to deal with case when this quick select is a ROR-merged scan */
-  bool in_ror_merged_scan;
-  MyBitmap column_bitmap;
-  MyBitmap *save_read_set;
-  MyBitmap *save_write_set;
-  bool free_file; /**< True when this->file is "owned" by this quick select */
-
-  /* Range pointers to be used when not using MRR interface */
-  QUICK_RANGE **cur_range; /**< current element in ranges  */
-  QUICK_RANGE *last_range;
-
-  /** Members needed to use the MRR interface */
-  QUICK_RANGE_SEQ_CTX qr_traversal_ctx;
-  uint32_t mrr_buf_size; /**< copy from session->variables.read_rnd_buff_size */
-  HANDLER_BUFFER *mrr_buf_desc; /**< the Cursor buffer */
-
-  /** Info about index we're scanning */
-  KEY_PART *key_parts;
-  KEY_PART_INFO *key_part_info;
-
-  bool dont_free; /**< Used by QUICK_SELECT_DESC */
-
-  int cmp_next(QUICK_RANGE *range);
-  int cmp_prev(QUICK_RANGE *range);
-  bool row_in_ranges();
-public:
-  uint32_t mrr_flags; /**< Flags to be used with MRR interface */
-  MEM_ROOT alloc;
-
-  QUICK_RANGE_SELECT(Session *session,
-                     Table *table,
-                     uint32_t index_arg,
-                     bool no_alloc,
-                     MEM_ROOT *parent_alloc,
-                     bool *create_err);
-  ~QUICK_RANGE_SELECT();
-
-  int init();
-  int reset(void);
-  int get_next();
-  void range_end();
-  int get_next_prefix(uint32_t prefix_length,
-                      key_part_map keypart_map,
-                      unsigned char *cur_prefix);
-  bool reverse_sorted()
-  {
-    return false;
-  }
-  bool unique_key_range();
-  int init_ror_merged_scan(bool reuse_handler);
-  void save_last_pos();
-  int get_type()
-  {
-    return QS_TYPE_RANGE;
-  }
-  void add_keys_and_lengths(String *key_names, String *used_lengths);
-  void add_info_string(String *str);
-  void resetCursor()
-  {
-    cursor= NULL;
-  }
-private:
-  /* Used only by QUICK_SELECT_DESC */
-  QUICK_RANGE_SELECT(const QUICK_RANGE_SELECT& org) : QUICK_SELECT_I()
-  {
-    memmove(this, &org, sizeof(*this));
-    /*
-      Use default MRR implementation for reverse scans. No table engine
-      currently can do an MRR scan with output in reverse index order.
-    */
-    mrr_buf_desc= NULL;
-    mrr_flags|= HA_MRR_USE_DEFAULT_IMPL;
-    mrr_buf_size= 0;
-  }
-  friend class ::TRP_ROR_INTERSECT; 
-  friend
-  QUICK_RANGE_SELECT *get_quick_select_for_ref(Session *session, Table *table,
-                                               struct table_reference_st *ref,
-                                               ha_rows records);
-  friend bool get_quick_keys(PARAM *param, QUICK_RANGE_SELECT *quick,
-                             KEY_PART *key, SEL_ARG *key_tree,
-                             unsigned char *min_key, uint32_t min_key_flag,
-                             unsigned char *max_key, uint32_t max_key_flag);
-  friend QUICK_RANGE_SELECT *get_quick_select(PARAM*,uint32_t idx,
-                                              SEL_ARG *key_tree,
-                                              uint32_t mrr_flags,
-                                              uint32_t mrr_buf_size,
-                                              MEM_ROOT *alloc);
-  friend class QUICK_SELECT_DESC;
-  friend class QUICK_INDEX_MERGE_SELECT;
-  friend class QUICK_ROR_INTERSECT_SELECT;
-  friend class QUICK_GROUP_MIN_MAX_SELECT;
-  friend uint32_t quick_range_seq_next(range_seq_t rseq, KEY_MULTI_RANGE *range);
-  friend range_seq_t quick_range_seq_init(void *init_param,
-                                          uint32_t n_ranges, uint32_t flags);
-  friend void select_describe(JOIN *join, bool need_tmp_table, bool need_order,
-                              bool distinct,const char *message);
-};
-
-/**
-  QUICK_INDEX_MERGE_SELECT - index_merge access method quick select.
-
-    QUICK_INDEX_MERGE_SELECT uses
-     * QUICK_RANGE_SELECTs to get rows
-     * Unique class to remove duplicate rows
-
-  INDEX MERGE OPTIMIZER
-    Current implementation doesn't detect all cases where index_merge could
-    be used, in particular:
-     * index_merge will never be used if range scan is possible (even if
-       range scan is more expensive)
-
-     * index_merge+'using index' is not supported (this the consequence of
-       the above restriction)
-
-     * If WHERE part contains complex nested AND and OR conditions, some ways
-       to retrieve rows using index_merge will not be considered. The choice
-       of read plan may depend on the order of conjuncts/disjuncts in WHERE
-       part of the query, see comments near imerge_list_or_list and
-       SEL_IMERGE::or_sel_tree_with_checks functions for details.
-
-     * There is no "index_merge_ref" method (but index_merge on non-first
-       table in join is possible with 'range checked for each record').
-
-    See comments around SEL_IMERGE class and test_quick_select for more
-    details.
-
-  ROW RETRIEVAL ALGORITHM
-
-    index_merge uses Unique class for duplicates removal.  index_merge takes
-    advantage of Clustered Primary Key (CPK) if the table has one.
-    The index_merge algorithm consists of two phases:
-
-    Phase 1 (implemented in QUICK_INDEX_MERGE_SELECT::prepare_unique):
-    prepare()
-    {
-      activate 'index only';
-      while(retrieve next row for non-CPK scan)
-      {
-        if (there is a CPK scan and row will be retrieved by it)
-          skip this row;
-        else
-          put its rowid into Unique;
-      }
-      deactivate 'index only';
-    }
-
-    Phase 2 (implemented as sequence of QUICK_INDEX_MERGE_SELECT::get_next
-    calls):
-
-    fetch()
-    {
-      retrieve all rows from row pointers stored in Unique;
-      free Unique;
-      retrieve all rows for CPK scan;
-    }
-*/
-class QUICK_INDEX_MERGE_SELECT : public QUICK_SELECT_I
-{
-public:
-  QUICK_INDEX_MERGE_SELECT(Session *session, Table *table);
-  ~QUICK_INDEX_MERGE_SELECT();
-
-  int init();
-  int reset(void);
-  int get_next();
-  bool reverse_sorted()
-  {
-    return false;
-  }
-  bool unique_key_range()
-  {
-    return false;
-  }
-  int get_type()
-  {
-    return QS_TYPE_INDEX_MERGE;
-  }
-  void add_keys_and_lengths(String *key_names, String *used_lengths);
-  void add_info_string(String *str);
-  bool is_keys_used(const MyBitmap *fields);
-
-  bool push_quick_back(QUICK_RANGE_SELECT *quick_sel_range);
-
-  /* range quick selects this index_merge read consists of */
-  List<QUICK_RANGE_SELECT> quick_selects;
-
-  /* quick select that uses clustered primary key (NULL if none) */
-  QUICK_RANGE_SELECT* pk_quick_select;
-
-  /* true if this select is currently doing a clustered PK scan */
-  bool  doing_pk_scan;
-
-  MEM_ROOT alloc;
-  Session *session;
-  int read_keys_and_merge();
-
-  /* used to get rows collected in Unique */
-  READ_RECORD read_record;
-};
-
 
 /**
   Rowid-Ordered Retrieval (ROR) index intersection quick select.
   This quick select produces intersection of row sequences returned
-  by several QUICK_RANGE_SELECTs it "merges".
+  by several QuickRangeSelects it "merges".
 
-  All merged QUICK_RANGE_SELECTs must return rowids in rowid order.
+  All merged QuickRangeSelects must return rowids in rowid order.
   QUICK_ROR_INTERSECT_SELECT will return rows in rowid order, too.
 
   All merged quick selects retrieve {rowid, covered_fields} tuples (not full
@@ -530,7 +292,7 @@
   If one of the merged quick selects is a Clustered PK range scan, it is
   used only to filter rowid sequence produced by other merged quick selects.
 */
-class QUICK_ROR_INTERSECT_SELECT : public QUICK_SELECT_I
+class QUICK_ROR_INTERSECT_SELECT : public QuickSelectInterface
 {
 public:
   QUICK_ROR_INTERSECT_SELECT(Session *session, Table *table,
@@ -557,19 +319,19 @@
   void add_info_string(String *str);
   bool is_keys_used(const MyBitmap *fields);
   int init_ror_merged_scan(bool reuse_handler);
-  bool push_quick_back(QUICK_RANGE_SELECT *quick_sel_range);
+  bool push_quick_back(QuickRangeSelect *quick_sel_range);
 
   /**
    * Range quick selects this intersection consists of, not including
    * cpk_quick.
    */
-  List<QUICK_RANGE_SELECT> quick_selects;
+  List<QuickRangeSelect> quick_selects;
 
   /**
    * Merged quick select that uses Clustered PK, if there is one. This quick
    * select is not used for row retrieval, it is used for row retrieval.
    */
-  QUICK_RANGE_SELECT *cpk_quick;
+  QuickRangeSelect *cpk_quick;
 
   MEM_ROOT alloc; /**< Memory pool for this and merged quick selects data. */
   Session *session; /**< Pointer to the current session */
@@ -598,7 +360,7 @@
   ROR-union quick select always retrieves full records.
 
 */
-class QUICK_ROR_UNION_SELECT : public QUICK_SELECT_I
+class QUICK_ROR_UNION_SELECT : public QuickSelectInterface
 {
 public:
   QUICK_ROR_UNION_SELECT(Session *session, Table *table);
@@ -623,12 +385,12 @@
   void add_info_string(String *str);
   bool is_keys_used(const MyBitmap *fields);
 
-  bool push_quick_back(QUICK_SELECT_I *quick_sel_range);
+  bool push_quick_back(QuickSelectInterface *quick_sel_range);
 
-  List<QUICK_SELECT_I> quick_selects; /**< Merged quick selects */
+  List<QuickSelectInterface> quick_selects; /**< Merged quick selects */
 
   /** Priority queue for merge operation */
-  std::priority_queue<QUICK_SELECT_I *, std::vector<QUICK_SELECT_I *>, compare_functor > *queue;
+  std::priority_queue<QuickSelectInterface *, std::vector<QuickSelectInterface *>, compare_functor > *queue;
   MEM_ROOT alloc; /**< Memory pool for this and merged quick selects data. */
 
   Session *session; /**< current thread */
@@ -672,7 +434,7 @@
   Since one of the requirements is that all select fields are part of the same
   index, this class produces only index keys, and not complete records.
 */
-class QUICK_GROUP_MIN_MAX_SELECT : public QUICK_SELECT_I
+class QUICK_GROUP_MIN_MAX_SELECT : public QuickSelectInterface
 {
 private:
   Cursor *cursor; /**< The Cursor used to get data. */
@@ -704,7 +466,7 @@
     TRP_GROUP_MIN_MAX::make_quick()
   */
   MEM_ROOT alloc; /**< Memory pool for this and quick_prefix_select data. */
-  QUICK_RANGE_SELECT *quick_prefix_select; /**< For retrieval of group prefixes. */
+  QuickRangeSelect *quick_prefix_select; /**< For retrieval of group prefixes. */
 private:
   int next_prefix();
   int next_min_in_range();
@@ -744,33 +506,18 @@
   void add_keys_and_lengths(String *key_names, String *used_lengths);
 };
 
-class QUICK_SELECT_DESC: public QUICK_RANGE_SELECT
-{
-public:
-  QUICK_SELECT_DESC(QUICK_RANGE_SELECT *q, uint32_t used_key_parts,
-                    bool *create_err);
-  int get_next();
-  bool reverse_sorted() { return 1; }
-  int get_type() { return QS_TYPE_RANGE_DESC; }
-private:
-  bool range_reads_after_key(QUICK_RANGE *range);
-  int reset(void) { rev_it.rewind(); return QUICK_RANGE_SELECT::reset(); }
-  List<QUICK_RANGE> rev_ranges;
-  List_iterator<QUICK_RANGE> rev_it;
-};
-
 /**
  * Executor class for SELECT statements.
  *
  * @details
  *
- * The QUICK_SELECT_I member variable is the implementor
+ * The QuickSelectInterface member variable is the implementor
  * of the SELECT execution.
  */
-class SQL_SELECT : public Sql_alloc 
+class SqlSelect : public Sql_alloc 
 {
  public:
-  QUICK_SELECT_I *quick; /**< If quick-select used */
+  QuickSelectInterface *quick; /**< If quick-select used */
   COND *cond; /**< where condition */
   Table	*head;
   IO_CACHE file; /**< Positions to used records */
@@ -782,8 +529,8 @@
   table_map read_tables;
   bool free_cond;
 
-  SQL_SELECT();
-  ~SQL_SELECT();
+  SqlSelect();
+  ~SqlSelect();
   void cleanup();
   bool check_quick(Session *session, bool force_quick_range, ha_rows limit);
   bool skip_record();
@@ -792,13 +539,13 @@
                         bool ordered_output);
 };
 
-QUICK_RANGE_SELECT *get_quick_select_for_ref(Session *session, 
+QuickRangeSelect *get_quick_select_for_ref(Session *session, 
                                              Table *table,
                                              struct table_reference_st *ref,
                                              ha_rows records);
 
 /*
-  Create a QUICK_RANGE_SELECT from given key and SEL_ARG tree for that key.
+  Create a QuickRangeSelect from given key and SEL_ARG tree for that key.
 
   SYNOPSIS
     get_quick_select()
@@ -819,24 +566,25 @@
     NULL on error
     otherwise created quick select
 */
-QUICK_RANGE_SELECT *get_quick_select(PARAM *param,
-                                     uint32_t index,
-                                     SEL_ARG *key_tree, 
-                                     uint32_t mrr_flags,
-                                     uint32_t mrr_buf_size, 
-                                     MEM_ROOT *alloc);
+QuickRangeSelect *get_quick_select(Parameter *param,
+                                   uint32_t index,
+                                   SEL_ARG *key_tree, 
+                                   uint32_t mrr_flags,
+                                   uint32_t mrr_buf_size, 
+                                   MEM_ROOT *alloc);
 
 uint32_t get_index_for_order(Table *table, order_st *order, ha_rows limit);
 
-SQL_SELECT *make_select(Table *head, 
+SqlSelect *make_select(Table *head, 
                         table_map const_tables,
 			                  table_map read_tables, 
                         COND *conds,
                         bool allow_null_cond,
                         int *error);
 
-bool get_quick_keys(PARAM *param, 
-                    QUICK_RANGE_SELECT *quick,KEY_PART *key,
+bool get_quick_keys(Parameter *param, 
+                    QuickRangeSelect *quick,
+                    KEY_PART *key,
                     SEL_ARG *key_tree, 
                     unsigned char *min_key,
                     uint32_t min_key_flag,

=== added file 'drizzled/optimizer/range_param.h'
--- drizzled/optimizer/range_param.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/range_param.h	2009-12-08 02:21:16 +0000
@@ -0,0 +1,104 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_RANGE_PARAM_H
+#define DRIZZLED_OPTIMIZER_RANGE_PARAM_H
+
+#include "drizzled/field.h"
+
+typedef class Item COND;
+typedef struct st_key_part KEY_PART;
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+class RangeParameter
+{
+public:
+  Session	*session;   /* Current thread handle */
+  Table *table; /* Table being analyzed */
+  COND *cond;   /* Used inside get_mm_tree(). */
+  table_map prev_tables;
+  table_map read_tables;
+  table_map current_table; /* Bit of the table being analyzed */
+
+  /* Array of parts of all keys for which range analysis is performed */
+  KEY_PART *key_parts;
+  KEY_PART *key_parts_end;
+  MEM_ROOT *mem_root; /* Memory that will be freed when range analysis completes */
+  MEM_ROOT *old_root; /* Memory that will last until the query end */
+  /*
+    Number of indexes used in range analysis (In SEL_TREE::keys only first
+    #keys elements are not empty)
+  */
+  uint32_t keys;
+
+  /*
+    If true, the index descriptions describe real indexes (and it is ok to
+    call field->optimize_range(real_keynr[...], ...).
+    Otherwise index description describes fake indexes.
+  */
+  bool using_real_indexes;
+
+  bool remove_jump_scans;
+
+  /*
+    used_key_no -> table_key_no translation table. Only makes sense if
+    using_real_indexes==true
+  */
+  uint32_t real_keynr[MAX_KEY];
+  /* Number of SEL_ARG objects allocated by optimizer::SEL_ARG::clone_tree operations */
+  uint32_t alloced_sel_args;
+  bool force_default_mrr;
+};
+
+class Parameter : public RangeParameter
+{
+public:
+  KEY_PART *key[MAX_KEY]; /* First key parts of keys used in the query */
+  uint32_t max_key_part;
+  /* Number of ranges in the last checked tree->key */
+  uint32_t range_count;
+  unsigned char min_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH],
+    max_key[MAX_KEY_LENGTH+MAX_FIELD_WIDTH];
+  bool quick;				// Don't calulate possible keys
+
+  uint32_t fields_bitmap_size;
+  MyBitmap needed_fields;    /* bitmask of fields needed by the query */
+  MyBitmap tmp_covered_fields;
+
+  key_map *needed_reg;        /* ptr to SqlSelect::needed_reg */
+
+  uint32_t *imerge_cost_buff;     /* buffer for index_merge cost estimates */
+  uint32_t imerge_cost_buff_size; /* size of the buffer */
+
+  /* true if last checked tree->key can be used for ROR-scan */
+  bool is_ror_scan;
+  /* Number of ranges in the last checked tree->key */
+  uint32_t n_ranges;
+};
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_RANGE_PARAM_H */

=== added file 'drizzled/optimizer/sel_arg.cc'
--- drizzled/optimizer/sel_arg.cc	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/sel_arg.cc	2009-12-08 18:39:43 +0000
@@ -0,0 +1,721 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#include "drizzled/server_includes.h"
+#include "drizzled/optimizer/range.h"
+#include "drizzled/optimizer/range_param.h"
+#include "drizzled/optimizer/sel_arg.h"
+
+using namespace drizzled;
+
+
+/* Functions to fix up the tree after insert and delete */
+static void left_rotate(optimizer::SEL_ARG **root, optimizer::SEL_ARG *leaf)
+{
+  optimizer::SEL_ARG *y= leaf->right;
+  leaf->right= y->left;
+  if (y->left != &optimizer::null_element)
+    y->left->parent= leaf;
+  if (! (y->parent=leaf->parent))
+    *root= y;
+  else
+    *leaf->parent_ptr()= y;
+  y->left= leaf;
+  leaf->parent= y;
+}
+
+
+static void right_rotate(optimizer::SEL_ARG **root, optimizer::SEL_ARG *leaf)
+{
+  optimizer::SEL_ARG *y= leaf->left;
+  leaf->left= y->right;
+  if (y->right != &optimizer::null_element)
+    y->right->parent= leaf;
+  if (! (y->parent=leaf->parent))
+    *root= y;
+  else
+    *leaf->parent_ptr()= y;
+  y->right= leaf;
+  leaf->parent= y;
+}
+
+
+/* Get overlapping range */
+optimizer::SEL_ARG *optimizer::SEL_ARG::clone_and(optimizer::SEL_ARG *arg)
+{
+  unsigned char *new_min= NULL;
+  unsigned char *new_max= NULL;
+  uint8_t flag_min= 0;
+  uint8_t flag_max= 0;
+
+  if (cmp_min_to_min(arg) >= 0)
+  {
+    new_min= min_value;
+    flag_min= min_flag;
+  }
+  else
+  {
+    new_min=arg->min_value; flag_min=arg->min_flag;
+  }
+  if (cmp_max_to_max(arg) <= 0)
+  {
+    new_max= max_value;
+    flag_max= max_flag;
+  }
+  else
+  {
+    new_max= arg->max_value;
+    flag_max= arg->max_flag;
+  }
+  return (new SEL_ARG(field,
+                      part,
+                      new_min,
+                      new_max,
+                      flag_min,
+                      flag_max,
+                      test(maybe_flag && arg->maybe_flag)));
+}
+
+
+/* min <= X , arg->min */
+optimizer::SEL_ARG *optimizer::SEL_ARG::clone_first(optimizer::SEL_ARG *arg)
+{
+  return (new SEL_ARG(field,part,
+                      min_value,
+                      arg->min_value,
+                      min_flag,
+                      arg->min_flag & NEAR_MIN ? 0 : NEAR_MAX,
+		                                     maybe_flag | arg->maybe_flag));
+}
+
+
+/* min <= X <= key_max */
+optimizer::SEL_ARG *optimizer::SEL_ARG::clone_last(optimizer::SEL_ARG *arg)
+{
+  return (new SEL_ARG(field,
+                      part,
+                      min_value,
+                      arg->max_value,
+                      min_flag,
+                      arg->max_flag,
+                      maybe_flag | arg->maybe_flag));
+}
+
+
+/* Get overlapping range */
+bool optimizer::SEL_ARG::copy_min(optimizer::SEL_ARG *arg)
+{
+  if (cmp_min_to_min(arg) > 0)
+  {
+    min_value= arg->min_value;
+    min_flag=arg->min_flag;
+    if ((max_flag & (NO_MAX_RANGE | NO_MIN_RANGE)) ==
+        (NO_MAX_RANGE | NO_MIN_RANGE))
+    {
+      return 1;	// Full range
+    }
+  }
+  maybe_flag|= arg->maybe_flag;
+  return 0;
+}
+
+/* Get overlapping range */
+bool optimizer::SEL_ARG::copy_max(optimizer::SEL_ARG *arg)
+{
+  if (cmp_max_to_max(arg) <= 0)
+  {
+    max_value= arg->max_value;
+    max_flag= arg->max_flag;
+    if ((max_flag & (NO_MAX_RANGE | NO_MIN_RANGE)) ==
+        (NO_MAX_RANGE | NO_MIN_RANGE))
+    {
+      return 1;	// Full range
+    }
+  }
+  maybe_flag|= arg->maybe_flag;
+  return 0;
+}
+
+void optimizer::SEL_ARG::copy_min_to_min(optimizer::SEL_ARG *arg)
+{
+  min_value= arg->min_value;
+  min_flag= arg->min_flag;
+}
+
+
+void optimizer::SEL_ARG::copy_min_to_max(optimizer::SEL_ARG *arg)
+{
+  max_value= arg->min_value;
+  max_flag= arg->min_flag & NEAR_MIN ? 0 : NEAR_MAX;
+}
+
+void optimizer::SEL_ARG::copy_max_to_min(optimizer::SEL_ARG *arg)
+{
+  min_value= arg->max_value;
+  min_flag= arg->max_flag & NEAR_MAX ? 0 : NEAR_MIN;
+}
+
+
+int optimizer::SEL_ARG::store_min(uint32_t length, unsigned char **min_key, uint32_t min_key_flag)
+{
+  /* "(kp1 > c1) AND (kp2 OP c2) AND ..." -> (kp1 > c1) */
+  if ((! (min_flag & NO_MIN_RANGE) &&
+      ! (min_key_flag & (NO_MIN_RANGE | NEAR_MIN))))
+  {
+    if (maybe_null && *min_value)
+    {
+      **min_key= 1;
+      memset(*min_key+1, 0, length-1);
+    }
+    else
+    {
+      memcpy(*min_key,min_value,length);
+    }
+    (*min_key)+= length;
+    return 1;
+  }
+  return 0;
+}
+
+
+int optimizer::SEL_ARG::store_max(uint32_t length, unsigned char **max_key, uint32_t max_key_flag)
+{
+  if (! (max_flag & NO_MAX_RANGE) &&
+      ! (max_key_flag & (NO_MAX_RANGE | NEAR_MAX)))
+  {
+    if (maybe_null && *max_value)
+    {
+      **max_key= 1;
+      memset(*max_key + 1, 0, length-1);
+    }
+    else
+    {
+      memcpy(*max_key,max_value,length);
+    }
+    (*max_key)+= length;
+    return 1;
+  }
+  return 0;
+}
+
+
+int optimizer::SEL_ARG::store_min_key(KEY_PART *key, unsigned char **range_key, uint32_t *range_key_flag)
+{
+  optimizer::SEL_ARG *key_tree= first();
+  uint32_t res= key_tree->store_min(key[key_tree->part].store_length,
+                                    range_key,
+                                    *range_key_flag);
+  *range_key_flag|= key_tree->min_flag;
+
+  if (key_tree->next_key_part &&
+      key_tree->next_key_part->part == key_tree->part+1 &&
+      ! (*range_key_flag & (NO_MIN_RANGE | NEAR_MIN)) &&
+      key_tree->next_key_part->type == optimizer::SEL_ARG::KEY_RANGE)
+  {
+    res+= key_tree->next_key_part->store_min_key(key,
+                                                 range_key,
+                                                 range_key_flag);
+  }
+  return res;
+}
+
+int optimizer::SEL_ARG::store_max_key(KEY_PART *key, unsigned char **range_key, uint32_t *range_key_flag)
+{
+  SEL_ARG *key_tree= last();
+  uint32_t res= key_tree->store_max(key[key_tree->part].store_length,
+                                    range_key,
+                                    *range_key_flag);
+  (*range_key_flag)|= key_tree->max_flag;
+  if (key_tree->next_key_part &&
+      key_tree->next_key_part->part == key_tree->part+1 &&
+      ! (*range_key_flag & (NO_MAX_RANGE | NEAR_MAX)) &&
+      key_tree->next_key_part->type == SEL_ARG::KEY_RANGE)
+    res+= key_tree->next_key_part->store_max_key(key,
+                                                 range_key,
+                                                 range_key_flag);
+  return res;
+}
+
+optimizer::SEL_ARG::SEL_ARG(optimizer::SEL_ARG &arg)
+  :
+    Sql_alloc()
+{
+  type= arg.type;
+  min_flag= arg.min_flag;
+  max_flag= arg.max_flag;
+  maybe_flag= arg.maybe_flag;
+  maybe_null= arg.maybe_null;
+  part= arg.part;
+  field= arg.field;
+  min_value= arg.min_value;
+  max_value= arg.max_value;
+  next_key_part= arg.next_key_part;
+  use_count=1;
+  elements=1;
+}
+
+
+void optimizer::SEL_ARG::make_root()
+{
+  left= right= &optimizer::null_element;
+  color= BLACK;
+  next= prev =0;
+  use_count= 0;
+  elements= 1;
+}
+
+optimizer::SEL_ARG::SEL_ARG(Field *f,
+                            const unsigned char *min_value_arg,
+                            const unsigned char *max_value_arg)
+  :
+    min_flag(0),
+    max_flag(0),
+    maybe_flag(0),
+    maybe_null(f->real_maybe_null()),
+    elements(1),
+    use_count(1),
+    field(f),
+    min_value((unsigned char*) min_value_arg),
+    max_value((unsigned char*) max_value_arg),
+    next(0),
+    prev(0),
+    next_key_part(0),
+    color(BLACK),
+    type(KEY_RANGE)
+{
+  left= right= &optimizer::null_element;
+}
+
+optimizer::SEL_ARG::SEL_ARG(Field *field_,
+                            uint8_t part_,
+                            unsigned char *min_value_,
+                            unsigned char *max_value_,
+		            uint8_t min_flag_,
+                            uint8_t max_flag_,
+                            uint8_t maybe_flag_)
+  :
+    min_flag(min_flag_),
+    max_flag(max_flag_),
+    maybe_flag(maybe_flag_),
+    part(part_),
+    maybe_null(field_->real_maybe_null()),
+    elements(1),
+    use_count(1),
+    field(field_),
+    min_value(min_value_),
+    max_value(max_value_),
+    next(0),
+    prev(0),
+    next_key_part(0),
+    color(BLACK),
+    type(KEY_RANGE)
+{
+  left= right= &optimizer::null_element;
+}
+
+optimizer::SEL_ARG *optimizer::SEL_ARG::clone(RangeParameter *param,
+                                              optimizer::SEL_ARG *new_parent,
+                                              optimizer::SEL_ARG **next_arg)
+{
+  optimizer::SEL_ARG *tmp= NULL;
+
+  /* Bail out if we have already generated too many SEL_ARGs */
+  if (++param->alloced_sel_args > MAX_SEL_ARGS)
+    return 0;
+
+  if (type != KEY_RANGE)
+  {
+    if (! (tmp= new (param->mem_root) optimizer::SEL_ARG(type)))
+      return 0;	// out of memory
+    tmp->prev= *next_arg; // Link into next/prev chain
+    (*next_arg)->next= tmp;
+    (*next_arg)= tmp;
+  }
+  else
+  {
+    if (! (tmp= new (param->mem_root) optimizer::SEL_ARG(field,
+                                                         part,
+                                                         min_value,
+                                                         max_value,
+                                                         min_flag,
+                                                         max_flag,
+                                                         maybe_flag)))
+      return 0; // OOM
+    tmp->parent= new_parent;
+    tmp->next_key_part= next_key_part;
+    if (left != &optimizer::null_element)
+      if (! (tmp->left= left->clone(param, tmp, next_arg)))
+	return 0; // OOM
+
+    tmp->prev= *next_arg; // Link into next/prev chain
+    (*next_arg)->next= tmp;
+    (*next_arg)= tmp;
+
+    if (right != &optimizer::null_element)
+      if (! (tmp->right= right->clone(param, tmp, next_arg)))
+	return 0; // OOM
+  }
+  increment_use_count(1);
+  tmp->color= color;
+  tmp->elements= this->elements;
+  return tmp;
+}
+
+optimizer::SEL_ARG *optimizer::SEL_ARG::first()
+{
+  optimizer::SEL_ARG *next_arg= this;
+  if (! next_arg->left)
+    return 0; // MAYBE_KEY
+  while (next_arg->left != &optimizer::null_element)
+    next_arg= next_arg->left;
+  return next_arg;
+}
+
+optimizer::SEL_ARG *optimizer::SEL_ARG::last()
+{
+  SEL_ARG *next_arg= this;
+  if (! next_arg->right)
+    return 0; // MAYBE_KEY
+  while (next_arg->right != &optimizer::null_element)
+    next_arg=next_arg->right;
+  return next_arg;
+}
+
+
+optimizer::SEL_ARG *optimizer::SEL_ARG::clone_tree(RangeParameter *param)
+{
+  optimizer::SEL_ARG tmp_link;
+  optimizer::SEL_ARG*next_arg= NULL;
+  optimizer::SEL_ARG *root= NULL;
+  next_arg= &tmp_link;
+  if (! (root= clone(param, (SEL_ARG *) 0, &next_arg)))
+    return 0;
+  next_arg->next= 0; // Fix last link
+  tmp_link.next->prev= 0; // Fix first link
+  if (root) // If not OOM
+    root->use_count= 0;
+  return root;
+}
+
+
+optimizer::SEL_ARG *
+optimizer::SEL_ARG::insert(optimizer::SEL_ARG *key)
+{
+  optimizer::SEL_ARG *element= NULL;
+  optimizer::SEL_ARG **par= NULL;
+  optimizer::SEL_ARG *last_element= NULL;
+
+  for (element= this; element != &optimizer::null_element; )
+  {
+    last_element= element;
+    if (key->cmp_min_to_min(element) > 0)
+    {
+      par= &element->right; element= element->right;
+    }
+    else
+    {
+      par= &element->left; element= element->left;
+    }
+  }
+  *par= key;
+  key->parent= last_element;
+	/* Link in list */
+  if (par == &last_element->left)
+  {
+    key->next= last_element;
+    if ((key->prev=last_element->prev))
+      key->prev->next= key;
+    last_element->prev= key;
+  }
+  else
+  {
+    if ((key->next= last_element->next))
+      key->next->prev= key;
+    key->prev= last_element;
+    last_element->next= key;
+  }
+  key->left= key->right= &optimizer::null_element;
+  optimizer::SEL_ARG *root= rb_insert(key); // rebalance tree
+  root->use_count= this->use_count;		// copy root info
+  root->elements= this->elements+1;
+  root->maybe_flag= this->maybe_flag;
+  return root;
+}
+
+
+/*
+** Find best key with min <= given key
+** Because the call context this should never return 0 to get_range
+*/
+optimizer::SEL_ARG *
+optimizer::SEL_ARG::find_range(optimizer::SEL_ARG *key)
+{
+  optimizer::SEL_ARG *element= this;
+  optimizer::SEL_ARG *found= NULL;
+
+  for (;;)
+  {
+    if (element == &optimizer::null_element)
+      return found;
+    int cmp= element->cmp_min_to_min(key);
+    if (cmp == 0)
+      return element;
+    if (cmp < 0)
+    {
+      found= element;
+      element= element->right;
+    }
+    else
+      element= element->left;
+  }
+}
+
+
+/*
+  Remove a element from the tree
+
+  SYNOPSIS
+    tree_delete()
+    key		Key that is to be deleted from tree (this)
+
+  NOTE
+    This also frees all sub trees that is used by the element
+
+  RETURN
+    root of new tree (with key deleted)
+*/
+optimizer::SEL_ARG *
+optimizer::SEL_ARG::tree_delete(optimizer::SEL_ARG *key)
+{
+  enum leaf_color remove_color;
+  optimizer::SEL_ARG *root= NULL;
+  optimizer::SEL_ARG *nod= NULL;
+  optimizer::SEL_ARG **par= NULL;
+  optimizer::SEL_ARG *fix_par= NULL;
+
+  root= this;
+  this->parent= 0;
+
+  /* Unlink from list */
+  if (key->prev)
+    key->prev->next= key->next;
+  if (key->next)
+    key->next->prev= key->prev;
+  key->increment_use_count(-1);
+  if (! key->parent)
+    par= &root;
+  else
+    par= key->parent_ptr();
+
+  if (key->left == &optimizer::null_element)
+  {
+    *par= nod= key->right;
+    fix_par= key->parent;
+    if (nod != &optimizer::null_element)
+      nod->parent= fix_par;
+    remove_color= key->color;
+  }
+  else if (key->right == &optimizer::null_element)
+  {
+    *par= nod= key->left;
+    nod->parent= fix_par= key->parent;
+    remove_color= key->color;
+  }
+  else
+  {
+    optimizer::SEL_ARG *tmp= key->next; // next bigger key (exist!)
+    nod= *tmp->parent_ptr()= tmp->right;	// unlink tmp from tree
+    fix_par= tmp->parent;
+    if (nod != &optimizer::null_element)
+      nod->parent= fix_par;
+    remove_color= tmp->color;
+
+    tmp->parent= key->parent;			// Move node in place of key
+    (tmp->left= key->left)->parent= tmp;
+    if ((tmp->right=key->right) != &optimizer::null_element)
+      tmp->right->parent= tmp;
+    tmp->color= key->color;
+    *par= tmp;
+    if (fix_par == key)				// key->right == key->next
+      fix_par= tmp;				// new parent of nod
+  }
+
+  if (root == &optimizer::null_element)
+    return 0;				// Maybe root later
+  if (remove_color == BLACK)
+    root= rb_delete_fixup(root, nod, fix_par);
+
+  root->use_count= this->use_count;		// Fix root counters
+  root->elements= this->elements-1;
+  root->maybe_flag= this->maybe_flag;
+  return root;
+}
+
+
+optimizer::SEL_ARG *
+optimizer::SEL_ARG::rb_insert(optimizer::SEL_ARG *leaf)
+{
+  optimizer::SEL_ARG *y= NULL;
+  optimizer::SEL_ARG *par= NULL;
+  optimizer::SEL_ARG *par2= NULL;
+  optimizer::SEL_ARG *root= NULL;
+
+  root= this;
+  root->parent= 0;
+
+  leaf->color= RED;
+  while (leaf != root && (par= leaf->parent)->color == RED)
+  {					// This can't be root or 1 level under
+    if (par == (par2= leaf->parent->parent)->left)
+    {
+      y= par2->right;
+      if (y->color == RED)
+      {
+        par->color= BLACK;
+        y->color= BLACK;
+        leaf= par2;
+        leaf->color= RED;		/* And the loop continues */
+      }
+      else
+      {
+        if (leaf == par->right)
+        {
+          left_rotate(&root,leaf->parent);
+          par= leaf; /* leaf is now parent to old leaf */
+        }
+        par->color= BLACK;
+        par2->color= RED;
+        right_rotate(&root, par2);
+        break;
+      }
+    }
+    else
+    {
+      y= par2->left;
+      if (y->color == RED)
+      {
+        par->color= BLACK;
+        y->color= BLACK;
+        leaf= par2;
+        leaf->color= RED;		/* And the loop continues */
+      }
+      else
+      {
+        if (leaf == par->left)
+        {
+          right_rotate(&root,par);
+          par= leaf;
+        }
+        par->color= BLACK;
+        par2->color= RED;
+        left_rotate(&root, par2);
+        break;
+      }
+    }
+  }
+  root->color= BLACK;
+
+  return root;
+}
+
+
+optimizer::SEL_ARG *optimizer::rb_delete_fixup(optimizer::SEL_ARG *root,
+                                               optimizer::SEL_ARG *key,
+                                               optimizer::SEL_ARG *par)
+{
+  optimizer::SEL_ARG *x= NULL;
+  optimizer::SEL_ARG *w= NULL;
+  root->parent= 0;
+
+  x= key;
+  while (x != root && x->color == optimizer::SEL_ARG::BLACK)
+  {
+    if (x == par->left)
+    {
+      w= par->right;
+      if (w->color == optimizer::SEL_ARG::RED)
+      {
+        w->color= optimizer::SEL_ARG::BLACK;
+        par->color= optimizer::SEL_ARG::RED;
+        left_rotate(&root, par);
+        w= par->right;
+      }
+      if (w->left->color == optimizer::SEL_ARG::BLACK &&
+          w->right->color == optimizer::SEL_ARG::BLACK)
+      {
+        w->color= optimizer::SEL_ARG::RED;
+        x= par;
+      }
+      else
+      {
+        if (w->right->color == optimizer::SEL_ARG::BLACK)
+        {
+          w->left->color= optimizer::SEL_ARG::BLACK;
+          w->color= optimizer::SEL_ARG::RED;
+          right_rotate(&root, w);
+          w= par->right;
+        }
+        w->color= par->color;
+        par->color= optimizer::SEL_ARG::BLACK;
+        w->right->color= optimizer::SEL_ARG::BLACK;
+        left_rotate(&root, par);
+        x= root;
+        break;
+      }
+    }
+    else
+    {
+      w= par->left;
+      if (w->color == optimizer::SEL_ARG::RED)
+      {
+        w->color= optimizer::SEL_ARG::BLACK;
+        par->color= optimizer::SEL_ARG::RED;
+        right_rotate(&root, par);
+        w= par->left;
+      }
+      if (w->right->color == optimizer::SEL_ARG::BLACK &&
+          w->left->color == optimizer::SEL_ARG::BLACK)
+      {
+        w->color= optimizer::SEL_ARG::RED;
+        x= par;
+      }
+      else
+      {
+        if (w->left->color == SEL_ARG::BLACK)
+        {
+          w->right->color= optimizer::SEL_ARG::BLACK;
+          w->color= optimizer::SEL_ARG::RED;
+          left_rotate(&root, w);
+          w= par->left;
+        }
+        w->color= par->color;
+        par->color= optimizer::SEL_ARG::BLACK;
+        w->left->color= optimizer::SEL_ARG::BLACK;
+        right_rotate(&root, par);
+        x= root;
+        break;
+      }
+    }
+    par= x->parent;
+  }
+  x->color= optimizer::SEL_ARG::BLACK;
+  return root;
+}
+
+

=== added file 'drizzled/optimizer/sel_arg.h'
--- drizzled/optimizer/sel_arg.h	1970-01-01 00:00:00 +0000
+++ drizzled/optimizer/sel_arg.h	2009-12-08 00:38:13 +0000
@@ -0,0 +1,492 @@
+/* -*- mode: c++; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ *  Copyright (C) 2008-2009 Sun Microsystems
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; version 2 of the License.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program; if not, write to the Free Software
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+ */
+
+#ifndef DRIZZLED_OPTIMIZER_SEL_ARG_H
+#define DRIZZLED_OPTIMIZER_SEL_ARG_H
+
+namespace drizzled
+{
+
+namespace optimizer
+{
+
+class RangeParameter;
+
+/*
+  A construction block of the SEL_ARG-graph.
+
+  The following description only covers graphs of SEL_ARG objects with
+  sel_arg->type==KEY_RANGE:
+
+  One SEL_ARG object represents an "elementary interval" in form
+
+      min_value <=?  table.keypartX  <=? max_value
+
+  The interval is a non-empty interval of any kind: with[out] minimum/maximum
+  bound, [half]open/closed, single-point interval, etc.
+
+  1. SEL_ARG GRAPH STRUCTURE
+
+  SEL_ARG objects are linked together in a graph. The meaning of the graph
+  is better demostrated by an example:
+
+     tree->keys[i]
+      |
+      |             $              $
+      |    part=1   $     part=2   $    part=3
+      |             $              $
+      |  +-------+  $   +-------+  $   +--------+
+      |  | kp1<1 |--$-->| kp2=5 |--$-->| kp3=10 |
+      |  +-------+  $   +-------+  $   +--------+
+      |      |      $              $       |
+      |      |      $              $   +--------+
+      |      |      $              $   | kp3=12 |
+      |      |      $              $   +--------+
+      |  +-------+  $              $
+      \->| kp1=2 |--$--------------$-+
+         +-------+  $              $ |   +--------+
+             |      $              $  ==>| kp3=11 |
+         +-------+  $              $ |   +--------+
+         | kp1=3 |--$--------------$-+       |
+         +-------+  $              $     +--------+
+             |      $              $     | kp3=14 |
+            ...     $              $     +--------+
+
+  The entire graph is partitioned into "interval lists".
+
+  An interval list is a sequence of ordered disjoint intervals over the same
+  key part. SEL_ARG are linked via "next" and "prev" pointers. Additionally,
+  all intervals in the list form an RB-tree, linked via left/right/parent
+  pointers. The RB-tree root SEL_ARG object will be further called "root of the
+  interval list".
+
+    In the example pic, there are 4 interval lists:
+    "kp<1 OR kp1=2 OR kp1=3", "kp2=5", "kp3=10 OR kp3=12", "kp3=11 OR kp3=13".
+    The vertical lines represent SEL_ARG::next/prev pointers.
+
+  In an interval list, each member X may have SEL_ARG::next_key_part pointer
+  pointing to the root of another interval list Y. The pointed interval list
+  must cover a key part with greater number (i.e. Y->part > X->part).
+
+    In the example pic, the next_key_part pointers are represented by
+    horisontal lines.
+
+  2. SEL_ARG GRAPH SEMANTICS
+
+  It represents a condition in a special form (we don't have a name for it ATM)
+  The SEL_ARG::next/prev is "OR", and next_key_part is "AND".
+
+  For example, the picture represents the condition in form:
+   (kp1 < 1 AND kp2=5 AND (kp3=10 OR kp3=12)) OR
+   (kp1=2 AND (kp3=11 OR kp3=14)) OR
+   (kp1=3 AND (kp3=11 OR kp3=14))
+
+
+  3. SEL_ARG GRAPH USE
+
+  Use get_mm_tree() to construct SEL_ARG graph from WHERE condition.
+  Then walk the SEL_ARG graph and get a list of dijsoint ordered key
+  intervals (i.e. intervals in form
+
+   (constA1, .., const1_K) < (keypart1,.., keypartK) < (constB1, .., constB_K)
+
+  Those intervals can be used to access the index. The uses are in:
+   - check_quick_select() - Walk the SEL_ARG graph and find an estimate of
+                            how many table records are contained within all
+                            intervals.
+   - get_quick_select()   - Walk the SEL_ARG, materialize the key intervals,
+                            and create QuickRangeSelect object that will
+                            read records within these intervals.
+
+  4. SPACE COMPLEXITY NOTES
+
+    SEL_ARG graph is a representation of an ordered disjoint sequence of
+    intervals over the ordered set of index tuple values.
+
+    For multi-part keys, one can construct a WHERE expression such that its
+    list of intervals will be of combinatorial size. Here is an example:
+
+      (keypart1 IN (1,2, ..., n1)) AND
+      (keypart2 IN (1,2, ..., n2)) AND
+      (keypart3 IN (1,2, ..., n3))
+
+    For this WHERE clause the list of intervals will have n1*n2*n3 intervals
+    of form
+
+      (keypart1, keypart2, keypart3) = (k1, k2, k3), where 1 <= k{i} <= n{i}
+
+    SEL_ARG graph structure aims to reduce the amount of required space by
+    "sharing" the elementary intervals when possible (the pic at the
+    beginning of this comment has examples of such sharing). The sharing may
+    prevent combinatorial blowup:
+
+      There are WHERE clauses that have combinatorial-size interval lists but
+      will be represented by a compact SEL_ARG graph.
+      Example:
+        (keypartN IN (1,2, ..., n1)) AND
+        ...
+        (keypart2 IN (1,2, ..., n2)) AND
+        (keypart1 IN (1,2, ..., n3))
+
+    but not in all cases:
+
+    - There are WHERE clauses that do have a compact SEL_ARG-graph
+      representation but get_mm_tree() and its callees will construct a
+      graph of combinatorial size.
+      Example:
+        (keypart1 IN (1,2, ..., n1)) AND
+        (keypart2 IN (1,2, ..., n2)) AND
+        ...
+        (keypartN IN (1,2, ..., n3))
+
+    - There are WHERE clauses for which the minimal possible SEL_ARG graph
+      representation will have combinatorial size.
+      Example:
+        By induction: Let's take any interval on some keypart in the middle:
+
+           kp15=c0
+
+        Then let's AND it with this interval 'structure' from preceding and
+        following keyparts:
+
+          (kp14=c1 AND kp16=c3) OR keypart14=c2) (*)
+
+        We will obtain this SEL_ARG graph:
+
+             kp14     $      kp15      $      kp16
+                      $                $
+         +---------+  $   +---------+  $   +---------+
+         | kp14=c1 |--$-->| kp15=c0 |--$-->| kp16=c3 |
+         +---------+  $   +---------+  $   +---------+
+              |       $                $
+         +---------+  $   +---------+  $
+         | kp14=c2 |--$-->| kp15=c0 |  $
+         +---------+  $   +---------+  $
+                      $                $
+
+       Note that we had to duplicate "kp15=c0" and there was no way to avoid
+       that.
+       The induction step: AND the obtained expression with another "wrapping"
+       expression like (*).
+       When the process ends because of the limit on max. number of keyparts
+       we'll have:
+
+         WHERE clause length  is O(3*#max_keyparts)
+         SEL_ARG graph size   is O(2^(#max_keyparts/2))
+
+       (it is also possible to construct a case where instead of 2 in 2^n we
+        have a bigger constant, e.g. 4, and get a graph with 4^(31/2)= 2^31
+        nodes)
+
+    We avoid consuming too much memory by setting a limit on the number of
+    SEL_ARG object we can construct during one range analysis invocation.
+*/
+
+class SEL_ARG :public Sql_alloc
+{
+public:
+  uint8_t min_flag,max_flag,maybe_flag;
+  uint8_t part;					// Which key part
+  uint8_t maybe_null;
+  /*
+    Number of children of this element in the RB-tree, plus 1 for this
+    element itself.
+  */
+  uint16_t elements;
+  /*
+    Valid only for elements which are RB-tree roots: Number of times this
+    RB-tree is referred to (it is referred by SEL_ARG::next_key_part or by
+    SEL_TREE::keys[i] or by a temporary SEL_ARG* variable)
+  */
+  ulong use_count;
+
+  Field *field;
+  unsigned char *min_value,*max_value;			// Pointer to range
+
+  /*
+    eq_tree() requires that left == right == 0 if the type is MAYBE_KEY.
+   */
+  SEL_ARG *left,*right;   /* R-B tree children */
+  SEL_ARG *next,*prev;    /* Links for bi-directional interval list */
+  SEL_ARG *parent;        /* R-B tree parent */
+  SEL_ARG *next_key_part;
+  enum leaf_color { BLACK,RED } color;
+  enum Type { IMPOSSIBLE, MAYBE, MAYBE_KEY, KEY_RANGE } type;
+
+  enum 
+  { 
+    MAX_SEL_ARGS = 16000 
+  };
+
+  SEL_ARG() {}
+
+  SEL_ARG(SEL_ARG &);
+
+  SEL_ARG(Field *,const unsigned char *, const unsigned char *);
+
+  SEL_ARG(Field *field, 
+          uint8_t part, 
+          unsigned char *min_value, 
+          unsigned char *max_value,
+	        uint8_t min_flag, 
+          uint8_t max_flag, 
+          uint8_t maybe_flag);
+
+  SEL_ARG(enum Type type_arg)
+    :
+      min_flag(0),
+      elements(1),
+      use_count(1),
+      left(0),
+      right(0),
+      next_key_part(0),
+      color(BLACK), 
+      type(type_arg)
+  {}
+
+  inline bool is_same(SEL_ARG *arg)
+  {
+    if (type != arg->type || part != arg->part)
+      return 0;
+    if (type != KEY_RANGE)
+      return 1;
+    return (cmp_min_to_min(arg) == 0 && cmp_max_to_max(arg) == 0);
+  }
+
+  inline void merge_flags(SEL_ARG *arg) 
+  { 
+    maybe_flag|= arg->maybe_flag; 
+  }
+
+  inline void maybe_smaller() 
+  { 
+    maybe_flag= 1; 
+  }
+
+  /* Return true iff it's a single-point null interval */
+  inline bool is_null_interval() 
+  { 
+    return (maybe_null && max_value[0] == 1);
+  }
+
+  inline int cmp_min_to_min(SEL_ARG *arg)
+  {
+    return sel_cmp(field,min_value, arg->min_value, min_flag, arg->min_flag);
+  }
+
+  inline int cmp_min_to_max(SEL_ARG *arg)
+  {
+    return sel_cmp(field,min_value, arg->max_value, min_flag, arg->max_flag);
+  }
+
+  inline int cmp_max_to_max(SEL_ARG *arg)
+  {
+    return sel_cmp(field,max_value, arg->max_value, max_flag, arg->max_flag);
+  }
+
+  inline int cmp_max_to_min(SEL_ARG *arg)
+  {
+    return sel_cmp(field,max_value, arg->min_value, max_flag, arg->min_flag);
+  }
+
+  SEL_ARG *clone_and(SEL_ARG *arg);
+
+  SEL_ARG *clone_first(SEL_ARG *arg);
+
+  SEL_ARG *clone_last(SEL_ARG *arg);
+
+  SEL_ARG *clone(RangeParameter *param, SEL_ARG *new_parent, SEL_ARG **next);
+
+  bool copy_min(SEL_ARG *arg);
+
+  bool copy_max(SEL_ARG *arg);
+
+  void copy_min_to_min(SEL_ARG *arg);
+
+  void copy_min_to_max(SEL_ARG *arg);
+
+  void copy_max_to_min(SEL_ARG *arg);
+
+  /* returns a number of keypart values (0 or 1) appended to the key buffer */
+  int store_min(uint32_t length, unsigned char **min_key, uint32_t min_key_flag);
+
+  /* returns a number of keypart values (0 or 1) appended to the key buffer */
+  int store_max(uint32_t length, unsigned char **max_key, uint32_t max_key_flag);
+
+  /* returns a number of keypart values appended to the key buffer */
+  int store_min_key(KEY_PART *key, unsigned char **range_key, uint32_t *range_key_flag);
+
+  /* returns a number of keypart values appended to the key buffer */
+  int store_max_key(KEY_PART *key, unsigned char **range_key, uint32_t *range_key_flag);
+
+  SEL_ARG *insert(SEL_ARG *key);
+  SEL_ARG *tree_delete(SEL_ARG *key);
+  SEL_ARG *find_range(SEL_ARG *key);
+  SEL_ARG *rb_insert(SEL_ARG *leaf);
+
+  friend SEL_ARG *rb_delete_fixup(SEL_ARG *root,SEL_ARG *key, SEL_ARG *par);
+
+  SEL_ARG *first();
+
+  SEL_ARG *last();
+
+  void make_root();
+
+  inline bool simple_key()
+  {
+    return (! next_key_part && elements == 1);
+  }
+
+  void increment_use_count(long count)
+  {
+    if (next_key_part)
+    {
+      next_key_part->use_count+= count;
+      count*= (next_key_part->use_count - count);
+      for (SEL_ARG *pos= next_key_part->first(); pos; pos= pos->next)
+        if (pos->next_key_part)
+          pos->increment_use_count(count);
+    }
+  }
+
+  void free_tree()
+  {
+    for (SEL_ARG *pos= first(); pos; pos= pos->next)
+      if (pos->next_key_part)
+      {
+        pos->next_key_part->use_count--;
+        pos->next_key_part->free_tree();
+      }
+  }
+
+  inline SEL_ARG **parent_ptr()
+  {
+    return parent->left == this ? &parent->left : &parent->right;
+  }
+
+
+  /*
+    Check if this SEL_ARG object represents a single-point interval
+
+    SYNOPSIS
+      is_singlepoint()
+
+    DESCRIPTION
+      Check if this SEL_ARG object (not tree) represents a single-point
+      interval, i.e. if it represents a "keypart = const" or
+      "keypart IS NULL".
+
+    RETURN
+      true   This SEL_ARG object represents a singlepoint interval
+      false  Otherwise
+  */
+
+  bool is_singlepoint()
+  {
+    /*
+      Check for NEAR_MIN ("strictly less") and NO_MIN_RANGE (-inf < field)
+      flags, and the same for right edge.
+    */
+    if (min_flag || max_flag)
+      return false;
+    unsigned char *min_val= min_value;
+    unsigned char *max_val= max_value;
+
+    if (maybe_null)
+    {
+      /* First byte is a NULL value indicator */
+      if (*min_val != *max_val)
+        return false;
+
+      if (*min_val)
+        return true; /* This "x IS NULL" */
+      min_val++;
+      max_val++;
+    }
+    return ! field->key_cmp(min_val, max_val);
+  }
+
+  SEL_ARG *clone_tree(RangeParameter *param);
+
+private:
+
+  /*
+     Check if a compare is ok, when one takes ranges in account
+     Returns -2 or 2 if the ranges where 'joined' like  < 2 and >= 2
+   */
+  int sel_cmp(Field *in_field, 
+              unsigned char *a, 
+              unsigned char *b, 
+              uint8_t a_flag,
+              uint8_t b_flag)
+  {
+    int cmp= 0;
+    /* First check if there was a compare to a min or max element */
+    if (a_flag & (NO_MIN_RANGE | NO_MAX_RANGE))
+    {
+      if ((a_flag & (NO_MIN_RANGE | NO_MAX_RANGE)) ==
+          (b_flag & (NO_MIN_RANGE | NO_MAX_RANGE)))
+        return 0;
+      return (a_flag & NO_MIN_RANGE) ? -1 : 1;
+    }
+    if (b_flag & (NO_MIN_RANGE | NO_MAX_RANGE))
+      return (b_flag & NO_MIN_RANGE) ? 1 : -1;
+
+    if (in_field->real_maybe_null())			// If null is part of key
+    {
+      if (*a != *b)
+      {
+        return *a ? -1 : 1;
+      }
+      if (*a)
+        goto end;					// NULL where equal
+      a++; b++;					// Skip NULL marker
+    }
+    cmp= in_field->key_cmp(a , b);
+    if (cmp) return cmp < 0 ? -1 : 1;		// The values differed
+
+    // Check if the compared equal arguments was defined with open/closed range
+end:
+    if (a_flag & (NEAR_MIN | NEAR_MAX))
+    {
+      if ((a_flag & (NEAR_MIN | NEAR_MAX)) == (b_flag & (NEAR_MIN | NEAR_MAX)))
+        return 0;
+      if (! (b_flag & (NEAR_MIN | NEAR_MAX)))
+        return (a_flag & NEAR_MIN) ? 2 : -2;
+      return (a_flag & NEAR_MIN) ? 1 : -1;
+    }
+    if (b_flag & (NEAR_MIN | NEAR_MAX))
+      return (b_flag & NEAR_MIN) ? -2 : 2;
+    return 0;					// The elements where equal
+  }
+
+  
+};
+
+SEL_ARG *rb_delete_fixup(SEL_ARG *root,
+                         SEL_ARG *key,
+                         SEL_ARG *par);
+
+extern SEL_ARG null_element;
+
+} /* namespace optimizer */
+
+} /* namespace drizzled */
+
+#endif /* DRIZZLED_OPTIMIZER_SEL_ARG_H */

=== modified file 'drizzled/optimizer/sum.cc'
--- drizzled/optimizer/sum.cc	2009-12-05 20:06:30 +0000
+++ drizzled/optimizer/sum.cc	2009-12-08 18:39:43 +0000
@@ -60,21 +60,21 @@
 namespace drizzled
 {
 
-static bool find_key_for_maxmin(bool max_fl, 
-                                table_reference_st *ref, 
+static bool find_key_for_maxmin(bool max_fl,
+                                table_reference_st *ref,
                                 Field* field,
-                                COND *cond, 
+                                COND *cond,
                                 uint32_t *range_fl,
                                 uint32_t *key_prefix_length);
 
-static int reckey_in_range(bool max_fl, 
-                           table_reference_st *ref, 
+static int reckey_in_range(bool max_fl,
+                           table_reference_st *ref,
                            Field* field,
-                           COND *cond, 
-                           uint32_t range_fl, 
+                           COND *cond,
+                           uint32_t range_fl,
                            uint32_t prefix_len);
 
-static int maxmin_in_range(bool max_fl, Field* field, COND *cond);
+static int maxmin_in_range(bool max_fl, Field *field, COND *cond);
 
 
 /*
@@ -92,7 +92,6 @@
     UINT64_MAX	Error: Could not calculate number of rows
     #			Multiplication of number of rows in all tables
 */
-
 static uint64_t get_exact_record_count(TableList *tables)
 {
   uint64_t count= 1;
@@ -115,13 +114,13 @@
   int const_result= 1;
   bool recalc_const_item= false;
   uint64_t count= 1;
-  bool is_exact_count= true; 
+  bool is_exact_count= true;
   bool maybe_exact_count= true;
   table_map removed_tables= 0;
   table_map outer_tables= 0;
   table_map used_tables= 0;
   table_map where_tables= 0;
-  Item *item;
+  Item *item= NULL;
   int error;
 
   if (conds)
@@ -135,8 +134,8 @@
    */
   for (TableList *tl= tables; tl; tl= tl->next_leaf)
   {
-    TableList *embedded;
-    for (embedded= tl ; embedded; embedded= embedded->embedding)
+    TableList *embedded= NULL;
+    for (embedded= tl; embedded; embedded= embedded->embedding)
     {
       if (embedded->on_expr)
         break;
@@ -195,7 +194,7 @@
     if (item->type() == Item::SUM_FUNC_ITEM)
     {
       Item_sum *item_sum= (((Item_sum*) item));
-      switch (item_sum->sum_func()) 
+      switch (item_sum->sum_func())
       {
         case Item_sum::COUNT_FUNC:
           /*
@@ -250,13 +249,13 @@
                  Type of range for the key part for this field will be
                  returned in range_fl.
                */
-              if (table->cursor->inited || 
+              if (table->cursor->inited ||
                   (outer_tables & table->map) ||
-                  ! find_key_for_maxmin(0, 
-                                        &ref, 
-                                        item_field->field, 
+                  ! find_key_for_maxmin(0,
+                                        &ref,
+                                        item_field->field,
                                         conds,
-                                        &range_fl, 
+                                        &range_fl,
                                         &prefix_len))
               {
                 const_result= 0;
@@ -285,6 +284,7 @@
                    nullable column, test if there is an exact match for the key.
                  */
                 if (! (range_fl & NEAR_MIN))
+                {
                   /*
                      Closed interval: Either The MIN argument is non-nullable, or
                      we have a >= predicate for the MIN argument.
@@ -293,6 +293,7 @@
                                                        ref.key_buff,
                                                        make_prev_keypart_map(ref.key_parts),
                                                        HA_READ_KEY_OR_NEXT);
+                }
                 else
                 {
                   /*
@@ -335,12 +336,12 @@
                 }
               }
               /* Verify that the read tuple indeed matches the search key */
-              if (! error && 
-                  reckey_in_range(0, 
-                                  &ref, 
+              if (! error &&
+                  reckey_in_range(0,
+                                  &ref,
                                   item_field->field,
-                                  conds, 
-                                  range_fl, 
+                                  conds,
+                                  range_fl,
                                   prefix_len))
               {
                 error= HA_ERR_KEY_NOT_FOUND;
@@ -416,13 +417,13 @@
                  Type of range for the key part for this field will be
                  returned in range_fl.
                */
-              if (table->cursor->inited || 
+              if (table->cursor->inited ||
                   (outer_tables & table->map) ||
-                  ! find_key_for_maxmin(1, 
-                                        &ref, 
-                                        item_field->field, 
+                  ! find_key_for_maxmin(1,
+                                        &ref,
+                                        item_field->field,
                                         conds,
-                                        &range_fl, 
+                                        &range_fl,
                                         &prefix_len))
               {
                 const_result= 0;
@@ -436,19 +437,19 @@
               }
               else
               {
-                error= table->cursor->index_read_map(table->record[0], 
+                error= table->cursor->index_read_map(table->record[0],
                                                      key_buff,
                                                      make_prev_keypart_map(ref.key_parts),
                                                      range_fl & NEAR_MAX ?
                                                      HA_READ_BEFORE_KEY :
                                                      HA_READ_PREFIX_LAST_OR_PREV);
               }
-              if (! error && 
-                  reckey_in_range(1, 
-                                  &ref, 
+              if (! error &&
+                  reckey_in_range(1,
+                                  &ref,
                                   item_field->field,
-                                  conds, 
-                                  range_fl, 
+                                  conds,
+                                  range_fl,
                                   prefix_len))
               {
                 error= HA_ERR_KEY_NOT_FOUND;
@@ -530,11 +531,12 @@
 }
 
 
-bool optimizer::simple_pred(Item_func *func_item, Item **args, bool *inv_order)
+bool optimizer::simple_pred(Item_func *func_item, Item **args, bool &inv_order)
 {
   Item *item= NULL;
-  *inv_order= 0;
-  switch (func_item->argument_count()) {
+  inv_order= false;
+  switch (func_item->argument_count())
+  {
   case 0:
     /* MULT_EQUAL_FUNC */
     {
@@ -582,7 +584,7 @@
         return 0;
       }
       args[0]= item;
-      *inv_order= 1;
+      inv_order= true;
     }
     else
     {
@@ -643,13 +645,12 @@
   @retval
     1        We can use index to get MIN/MAX value
 */
-
-static bool matching_cond(bool max_fl, 
-                          table_reference_st *ref, 
+static bool matching_cond(bool max_fl,
+                          table_reference_st *ref,
                           KEY *keyinfo,
-                          KEY_PART_INFO *field_part, 
+                          KEY_PART_INFO *field_part,
                           COND *cond,
-                          key_part_map *key_part_used, 
+                          key_part_map *key_part_used,
                           uint32_t *range_fl,
                           uint32_t *prefix_len)
 {
@@ -678,13 +679,13 @@
     Item *item;
     while ((item= li++))
     {
-      if (! matching_cond(max_fl, 
-                          ref, 
-                          keyinfo, 
-                          field_part, 
+      if (! matching_cond(max_fl,
+                          ref,
+                          keyinfo,
+                          field_part,
                           item,
-                          key_part_used, 
-                          range_fl, 
+                          key_part_used,
+                          range_fl,
                           prefix_len))
       {
         return 0;
@@ -704,7 +705,7 @@
   bool is_null= false;
   bool between= false;
 
-  switch (((Item_func*) cond)->functype()) 
+  switch (((Item_func*) cond)->functype())
   {
   case Item_func::ISNULL_FUNC:
     is_null= 1;     /* fall through */
@@ -735,7 +736,7 @@
   bool inv;
 
   /* Test if this is a comparison of a field and constant */
-  if (! optimizer::simple_pred((Item_func*) cond, args, &inv))
+  if (! optimizer::simple_pred((Item_func*) cond, args, inv))
   {
     return 0;
   }
@@ -894,13 +895,11 @@
     1   Can use key to optimize MIN()/MAX().
     In this case ref, range_fl and prefix_len are updated
 */
-
-
-static bool find_key_for_maxmin(bool max_fl, 
+static bool find_key_for_maxmin(bool max_fl,
                                 table_reference_st *ref,
-                                Field* field, 
+                                Field* field,
                                 COND *cond,
-                                uint32_t *range_fl, 
+                                uint32_t *range_fl,
                                 uint32_t *prefix_len)
 {
   if (! (field->flags & PART_KEY_FLAG))
@@ -955,13 +954,13 @@
         ref->key_parts= 0;
         key_part_map key_part_used= 0;
         *range_fl= NO_MIN_RANGE | NO_MAX_RANGE;
-        if (matching_cond(max_fl, 
-                          ref, 
-                          keyinfo, 
-                          part, 
+        if (matching_cond(max_fl,
+                          ref,
+                          keyinfo,
+                          part,
                           cond,
-                          &key_part_used, 
-                          range_fl, 
+                          &key_part_used,
+                          range_fl,
                           prefix_len) &&
             ! (key_part_to_use & ~key_part_used))
         {
@@ -1022,11 +1021,11 @@
   @retval
     1        WHERE was not true for the found row
 */
-static int reckey_in_range(bool max_fl, 
-                           table_reference_st *ref, 
+static int reckey_in_range(bool max_fl,
+                           table_reference_st *ref,
                            Field* field,
-                           COND *cond, 
-                           uint32_t range_fl, 
+                           COND *cond,
+                           uint32_t range_fl,
                            uint32_t prefix_len)
 {
   if (key_cmp_if_same(field->table, ref->key_buff, ref->key, prefix_len))
@@ -1075,7 +1074,7 @@
     return 0;
   }
   bool less_fl= false;
-  switch (((Item_func*) cond)->functype()) 
+  switch (((Item_func*) cond)->functype())
   {
   case Item_func::BETWEEN:
     return cond->val_int() == 0;                // Return 1 if WHERE is false

=== modified file 'drizzled/optimizer/sum.h'
--- drizzled/optimizer/sum.h	2009-12-04 07:02:49 +0000
+++ drizzled/optimizer/sum.h	2009-12-07 21:50:09 +0000
@@ -40,7 +40,7 @@
   @retval
     1        Otherwise
 */
-bool simple_pred(Item_func *func_item, Item **args, bool *inv_order);
+bool simple_pred(Item_func *func_item, Item **args, bool &inv_order);
 
 /**
   Substitutes constants for some COUNT(), MIN() and MAX() functions.

=== modified file 'drizzled/records.cc'
--- drizzled/records.cc	2009-12-05 05:29:33 +0000
+++ drizzled/records.cc	2009-12-07 19:18:46 +0000
@@ -64,7 +64,7 @@
 void init_read_record(READ_RECORD *info,
                       Session *session, 
                       Table *table,
-                      optimizer::SQL_SELECT *select,
+                      optimizer::SqlSelect *select,
                       int use_record_cache, 
                       bool print_error)
 {

=== modified file 'drizzled/records.h'
--- drizzled/records.h	2009-12-05 05:29:33 +0000
+++ drizzled/records.h	2009-12-07 19:18:46 +0000
@@ -112,7 +112,7 @@
 void init_read_record(READ_RECORD *info, 
                       Session *session, 
                       Table *reg_form,
-                      drizzled::optimizer::SQL_SELECT *select,
+                      drizzled::optimizer::SqlSelect *select,
                       int use_record_cache, 
                       bool print_errors);
 

=== modified file 'drizzled/server_includes.h'
--- drizzled/server_includes.h	2009-12-05 20:24:57 +0000
+++ drizzled/server_includes.h	2009-12-08 21:44:46 +0000
@@ -172,14 +172,14 @@
 int test_if_number(char *str,int *res,bool allow_wildcards);
 void change_byte(unsigned char *,uint,char,char);
 
-namespace drizzled { namespace optimizer { class SQL_SELECT; } }
+namespace drizzled { namespace optimizer { class SqlSelect; } }
 
-ha_rows filesort(Session *session, 
+ha_rows filesort(Session *session,
                  Table *form,
                  struct st_sort_field *sortorder,
-		             uint32_t s_length, 
-                 drizzled::optimizer::SQL_SELECT *select,
-		             ha_rows max_rows, 
+		             uint32_t s_length,
+                 drizzled::optimizer::SqlSelect *select,
+		             ha_rows max_rows,
                  bool sort_positions,
                  ha_rows *examined_rows);
 
@@ -187,7 +187,7 @@
 void change_double_for_sort(double nr,unsigned char *to);
 double my_double_round(double value, int64_t dec, bool dec_unsigned,
                        bool truncate);
-int get_quick_record(drizzled::optimizer::SQL_SELECT *select);
+int get_quick_record(drizzled::optimizer::SqlSelect *select);
 
 void find_date(char *pos,uint32_t *vek,uint32_t flag);
 TYPELIB *convert_strings_to_array_type(char * *typelibs, char * *end);
@@ -198,7 +198,6 @@
 
 
 int rename_file_ext(const char * from,const char * to,const char * ext);
-bool check_db_name(LEX_STRING *db);
 bool check_column_name(const char *name);
 bool check_table_name(const char *name, uint32_t length);
 

=== modified file 'drizzled/session.cc'
--- drizzled/session.cc	2009-12-05 01:45:30 +0000
+++ drizzled/session.cc	2009-12-08 21:44:46 +0000
@@ -41,6 +41,7 @@
 #include "drizzled/plugin/authentication.h"
 #include "drizzled/probes.h"
 #include "drizzled/table_proto.h"
+#include "drizzled/db.h"
 
 #include <algorithm>
 
@@ -646,7 +647,6 @@
 
 bool Session::checkUser(const char *passwd, uint32_t passwd_len, const char *in_db)
 {
-  LEX_STRING db_str= { (char *) in_db, in_db ? strlen(in_db) : 0 };
   bool is_authenticated;
 
   if (passwd_len != 0 && passwd_len != SCRAMBLE_LENGTH)
@@ -672,7 +672,11 @@
   /* Change database if necessary */
   if (in_db && in_db[0])
   {
-    if (mysql_change_db(this, &db_str, false))
+    const string database_name_string(in_db);
+    NonNormalisedDatabaseName database_name(database_name_string);
+    NormalisedDatabaseName normalised_database_name(database_name);
+
+    if (mysql_change_db(this, normalised_database_name, false))
     {
       /* mysql_change_db() has pushed the error message. */
       return false;
@@ -1692,17 +1696,18 @@
 }
 
 
-bool Session::set_db(const char *new_db, size_t length)
+bool Session::set_db(const NormalisedDatabaseName &new_db)
 {
-  /* Do not reallocate memory if current chunk is big enough. */
-  if (length)
-    db= new_db;
-  else
-    db.clear();
+  db= new_db.to_string();
 
   return false;
 }
 
+void Session::clear_db()
+{
+  db.clear();
+}
+
 
 /**
   Check the killed state of a user thread

=== modified file 'drizzled/session.h'
--- drizzled/session.h	2009-12-05 21:19:51 +0000
+++ drizzled/session.h	2009-12-08 21:44:46 +0000
@@ -33,7 +33,6 @@
 #include <drizzled/sql_error.h>
 #include <drizzled/file_exchange.h>
 #include <drizzled/select_result_interceptor.h>
-#include <drizzled/db.h>
 #include <drizzled/xid.h>
 
 #include <netdb.h>
@@ -1139,7 +1138,9 @@
       @retval false Success
       @retval true  Out-of-memory error
   */
-  bool set_db(const char *new_db, size_t new_db_len);
+  bool set_db(const NormalisedDatabaseName &new_db);
+
+  void clear_db();
 
   /*
     Copy the current database to the argument. Use the current arena to

=== modified file 'drizzled/show.cc'
--- drizzled/show.cc	2009-11-26 04:33:51 +0000
+++ drizzled/show.cc	2009-12-04 07:43:11 +0000
@@ -296,18 +296,18 @@
   return false;
 }
 
-bool mysqld_show_create_db(Session *session, char *dbname, bool if_not_exists)
+bool mysqld_show_create_db(Session *session, const NormalisedDatabaseName &database_name, bool if_not_exists)
 {
   char buff[2048];
   String buffer(buff, sizeof(buff), system_charset_info);
 
-  if (store_db_create_info(dbname, &buffer, if_not_exists))
+  if (store_db_create_info(database_name.to_string().c_str(), &buffer, if_not_exists))
   {
     /*
       This assumes that the only reason for which store_db_create_info()
       can fail is incorrect database name (which is the case now).
     */
-    my_error(ER_BAD_DB_ERROR, MYF(0), dbname);
+    my_error(ER_BAD_DB_ERROR, MYF(0), database_name.to_string().c_str());
     return true;
   }
 
@@ -318,7 +318,7 @@
   if (session->client->sendFields(&field_list))
     return true;
 
-  session->client->store(dbname, strlen(dbname));
+  session->client->store(database_name.to_string().c_str(), database_name.to_string().length());
   session->client->store(buffer.ptr(), buffer.length());
 
   if (session->client->flush())

=== modified file 'drizzled/show.h'
--- drizzled/show.h	2009-12-05 20:24:57 +0000
+++ drizzled/show.h	2009-12-08 21:44:46 +0000
@@ -52,7 +52,8 @@
 class Table;
 typedef class Item COND;
 
-typedef drizzle_show_var SHOW_VAR;
+class NormalisedDatabaseName;
+
 typedef struct system_status_var STATUS_VAR;
 
 typedef struct st_lookup_field_values
@@ -85,7 +86,7 @@
 void mysqld_list_fields(Session *session,TableList *table, const char *wild);
 int mysqld_dump_create_info(Session *session, TableList *table_list, int fd);
 bool drizzled_show_create(Session *session, TableList *table_list, bool is_if_not_exists);
-bool mysqld_show_create_db(Session *session, char *dbname, bool if_not_exists);
+bool mysqld_show_create_db(Session *session, const NormalisedDatabaseName &database_name, bool if_not_exists);
 
 int mysqld_show_status(Session *session);
 int mysqld_show_variables(Session *session,const char *wild);

=== modified file 'drizzled/sql_delete.cc'
--- drizzled/sql_delete.cc	2009-12-05 20:06:30 +0000
+++ drizzled/sql_delete.cc	2009-12-07 19:18:46 +0000
@@ -45,7 +45,7 @@
 {
   int		error;
   Table		*table;
-  optimizer::SQL_SELECT *select= NULL;
+  optimizer::SqlSelect *select= NULL;
   READ_RECORD	info;
   bool          using_limit=limit != HA_POS_ERROR;
   bool		transactional_table, const_cond;

=== modified file 'drizzled/sql_load.cc'
--- drizzled/sql_load.cc	2009-11-17 18:11:45 +0000
+++ drizzled/sql_load.cc	2009-12-07 07:10:27 +0000
@@ -23,6 +23,7 @@
 #include <drizzled/session.h>
 #include <drizzled/sql_base.h>
 #include <drizzled/field/timestamp.h>
+#include <drizzled/db.h>
 
 #include <algorithm>
 

=== modified file 'drizzled/sql_parse.cc'
--- drizzled/sql_parse.cc	2009-11-28 22:02:19 +0000
+++ drizzled/sql_parse.cc	2009-12-08 01:51:27 +0000
@@ -16,7 +16,6 @@
 #define DRIZZLE_LEX 1
 #include <drizzled/server_includes.h>
 #include <mysys/hash.h>
-#include <drizzled/db.h>
 #include <drizzled/error.h>
 #include <drizzled/nested_join.h>
 #include <drizzled/query_id.h>
@@ -24,6 +23,7 @@
 #include <drizzled/data_home.h>
 #include <drizzled/sql_base.h>
 #include <drizzled/show.h>
+#include <drizzled/db.h>
 #include <drizzled/plugin/info_schema_table.h>
 #include <drizzled/function/time/unix_timestamp.h>
 #include <drizzled/function/get_system_var.h>
@@ -193,11 +193,13 @@
   switch (command) {
   case COM_INIT_DB:
   {
-    LEX_STRING tmp;
+    string database_name(packet);
+    NonNormalisedDatabaseName non_normalised_database_name(database_name);
+    NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+
     status_var_increment(session->status_var.com_stat[SQLCOM_CHANGE_DB]);
-    tmp.str= packet;
-    tmp.length= packet_length;
-    if (!mysql_change_db(session, &tmp, false))
+
+    if (! mysql_change_db(session, normalised_database_name, false))
     {
       session->my_ok();
     }
@@ -356,7 +358,6 @@
   if (schema_table_name.compare("TABLES") == 0 ||
       schema_table_name.compare("TABLE_NAMES") == 0)
   {
-    LEX_STRING db;
     size_t dummy;
     if (lex->select_lex.db == NULL &&
         lex->copy_db_to(&lex->select_lex.db, &dummy))
@@ -364,13 +365,18 @@
       return (1);
     }
     schema_select_lex= new Select_Lex();
-    db.str= schema_select_lex->db= lex->select_lex.db;
+    schema_select_lex->db= lex->select_lex.db;
+
+    string database_name(schema_select_lex->db);
+    NonNormalisedDatabaseName non_normalised_database_name(database_name);
+    NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+
     schema_select_lex->table_list.first= NULL;
-    db.length= strlen(db.str);
 
-    if (check_db_name(&db))
+    if (! normalised_database_name.isValid())
     {
-      my_error(ER_WRONG_DB_NAME, MYF(0), db.str);
+      my_error(ER_WRONG_DB_NAME, MYF(0),
+               normalised_database_name.to_string().c_str());
       return (1);
     }
   }
@@ -942,11 +948,20 @@
     return NULL;
   }
 
-  if (table->is_derived_table() == false && table->db.str &&
-      check_db_name(&table->db))
+  if (table->is_derived_table() == false && table->db.str)
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), table->db.str);
-    return NULL;
+    string database_name(table->db.str);
+    NonNormalisedDatabaseName non_normalised_database_name(database_name);
+    NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+
+    if (! normalised_database_name.isValid())
+    {
+      my_error(ER_WRONG_DB_NAME, MYF(0), normalised_database_name.to_string().c_str());
+      return NULL;
+    }
+
+    strncpy(table->db.str, normalised_database_name.to_string().c_str(),
+            table->db.length);
   }
 
   if (!alias)					/* Alias is case sensitive */

=== modified file 'drizzled/sql_select.cc'
--- drizzled/sql_select.cc	2009-12-05 20:06:30 +0000
+++ drizzled/sql_select.cc	2009-12-07 21:50:09 +0000
@@ -59,6 +59,7 @@
 #include "drizzled/optimizer/sargable_param.h"
 #include "drizzled/optimizer/key_use.h"
 #include "drizzled/optimizer/range.h"
+#include "drizzled/optimizer/quick_range_select.h"
 
 using namespace std;
 using namespace drizzled;
@@ -475,7 +476,7 @@
   Create JoinTableS, make a guess about the table types,
   Approximate how many records will be used in each table
 *****************************************************************************/
-ha_rows get_quick_record_count(Session *session, optimizer::SQL_SELECT *select, Table *table, const key_map *keys,ha_rows limit)
+ha_rows get_quick_record_count(Session *session, optimizer::SqlSelect *select, Table *table, const key_map *keys,ha_rows limit)
 {
   int error;
   if (check_stack_overrun(session, STACK_MIN_SIZE, NULL))
@@ -4791,9 +4792,9 @@
   int order_direction;
   uint32_t used_key_parts;
   Table *table=tab->table;
-  optimizer::SQL_SELECT *select= tab->select;
+  optimizer::SqlSelect *select= tab->select;
   key_map usable_keys;
-  optimizer::QUICK_SELECT_I *save_quick= NULL;
+  optimizer::QuickSelectInterface *save_quick= NULL;
 
   /*
     Keys disabled by ALTER Table ... DISABLE KEYS should have already
@@ -4833,9 +4834,9 @@
       by clustered PK values.
     */
 
-    if (quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_INDEX_MERGE ||
-        quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_UNION ||
-        quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_INTERSECT)
+    if (quick_type == optimizer::QuickSelectInterface::QS_TYPE_INDEX_MERGE ||
+        quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_UNION ||
+        quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_INTERSECT)
       return(0);
     ref_key=	   select->quick->index;
     ref_key_parts= select->quick->used_key_parts;
@@ -4885,7 +4886,7 @@
         else
         {
           /*
-            The range optimizer constructed QUICK_RANGE for ref_key, and
+            The range optimizer constructed QuickRange for ref_key, and
             we want to use instead new_ref_key as the index. We can't
             just change the index of the quick select, because this may
             result in an incosistent QUICK_SELECT object. Below we
@@ -5154,28 +5155,29 @@
       */
       if (! select->quick->reverse_sorted())
       {
-        optimizer::QUICK_SELECT_DESC *tmp= NULL;
+        optimizer::QuickSelectDescending *tmp= NULL;
         bool error= false;
         int quick_type= select->quick->get_type();
-        if (quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_INDEX_MERGE ||
-            quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_INTERSECT ||
-            quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_UNION ||
-            quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_GROUP_MIN_MAX)
+        if (quick_type == optimizer::QuickSelectInterface::QS_TYPE_INDEX_MERGE ||
+            quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_INTERSECT ||
+            quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_UNION ||
+            quick_type == optimizer::QuickSelectInterface::QS_TYPE_GROUP_MIN_MAX)
         {
           tab->limit= 0;
           select->quick= save_quick;
-          return(0);                   // Use filesort
+          return 0; // Use filesort
         }
 
         /* order_st BY range_key DESC */
-        tmp= new optimizer::QUICK_SELECT_DESC((optimizer::QUICK_RANGE_SELECT*)(select->quick),
-                                              used_key_parts, &error);
-        if (!tmp || error)
+        tmp= new optimizer::QuickSelectDescending((optimizer::QuickRangeSelect*)(select->quick),
+                                                  used_key_parts, 
+                                                  &error);
+        if (! tmp || error)
         {
           delete tmp;
-                select->quick= save_quick;
-                tab->limit= 0;
-          return(0);		// Reverse sort not supported
+          select->quick= save_quick;
+          tab->limit= 0;
+          return 0; // Reverse sort not supported
         }
         select->quick=tmp;
       }
@@ -5195,7 +5197,7 @@
   }
   else if (select && select->quick)
     select->quick->sorted= 1;
-  return(1);
+  return 1;
 }
 
 /*
@@ -5231,7 +5233,7 @@
   uint32_t length= 0;
   ha_rows examined_rows;
   Table *table;
-  optimizer::SQL_SELECT *select= NULL;
+  optimizer::SqlSelect *select= NULL;
   JoinTable *tab;
 
   if (join->tables == join->const_tables)
@@ -5248,7 +5250,7 @@
   */
   if ((order != join->group_list ||
        !(join->select_options & SELECT_BIG_RESULT) ||
-       (select && select->quick && (select->quick->get_type() == optimizer::QUICK_SELECT_I::QS_TYPE_GROUP_MIN_MAX))) &&
+       (select && select->quick && (select->quick->get_type() == optimizer::QuickSelectInterface::QS_TYPE_GROUP_MIN_MAX))) &&
       test_if_skip_sort_order(tab,order,select_limit,0,
                               is_order_by ?  &table->keys_in_use_for_order_by :
                               &table->keys_in_use_for_group_by))
@@ -6685,9 +6687,9 @@
       if (tab->type == AM_ALL && tab->select && tab->select->quick)
       {
         quick_type= tab->select->quick->get_type();
-        if ((quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_INDEX_MERGE) ||
-            (quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_INTERSECT) ||
-            (quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_UNION))
+        if ((quick_type == optimizer::QuickSelectInterface::QS_TYPE_INDEX_MERGE) ||
+            (quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_INTERSECT) ||
+            (quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_UNION))
           tab->type = AM_INDEX_MERGE;
         else
 	  tab->type = AM_RANGE;
@@ -6815,7 +6817,7 @@
       if ((tab->type == AM_NEXT || tab->type == AM_CONST) &&
           table->covering_keys.test(tab->index))
 	key_read=1;
-      if (quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_INTERSECT &&
+      if (quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_INTERSECT &&
           !((optimizer::QUICK_ROR_INTERSECT_SELECT*)tab->select->quick)->need_to_fetch_row)
         key_read=1;
 
@@ -6847,9 +6849,9 @@
         else if (tab->select && tab->select->quick)
           keyno = tab->select->quick->index;
 
-        if (quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_UNION ||
-            quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_ROR_INTERSECT ||
-            quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_INDEX_MERGE)
+        if (quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_UNION ||
+            quick_type == optimizer::QuickSelectInterface::QS_TYPE_ROR_INTERSECT ||
+            quick_type == optimizer::QuickSelectInterface::QS_TYPE_INDEX_MERGE)
         {
           extra.append(STRING_WITH_LEN("; Using "));
           tab->select->quick->add_info_string(&extra);
@@ -6888,7 +6890,7 @@
         }
         if (key_read)
         {
-          if (quick_type == optimizer::QUICK_SELECT_I::QS_TYPE_GROUP_MIN_MAX)
+          if (quick_type == optimizer::QuickSelectInterface::QS_TYPE_GROUP_MIN_MAX)
             extra.append(STRING_WITH_LEN("; Using index for group-by"));
           else
             extra.append(STRING_WITH_LEN("; Using index"));

=== modified file 'drizzled/sql_select.h'
--- drizzled/sql_select.h	2009-12-04 20:16:03 +0000
+++ drizzled/sql_select.h	2009-12-07 21:50:09 +0000
@@ -201,7 +201,7 @@
                          table_map normal_tables,
                          Select_Lex *select_lex,
                          std::vector<drizzled::optimizer::SargableParam> &sargables);
-ha_rows get_quick_record_count(Session *session, drizzled::optimizer::SQL_SELECT *select, Table *table, const key_map *keys,ha_rows limit);
+ha_rows get_quick_record_count(Session *session, drizzled::optimizer::SqlSelect *select, Table *table, const key_map *keys,ha_rows limit);
 void optimize_keyuse(JOIN *join, DYNAMIC_ARRAY *keyuse_array);
 void add_group_and_distinct_keys(JOIN *join, JoinTable *join_tab);
 void read_cached_record(JoinTable *tab);
@@ -213,9 +213,6 @@
                         drizzled::optimizer::KeyUse *org_keyuse, 
                         table_map used_tables);
 
-/* from sql_delete.cc, used by optimizer/range.cc */
-extern "C" int refpos_order_cmp(void* arg, const void *a,const void *b);
-
 #include "drizzled/stored_key.h"
 
 bool cp_buffer_from_ref(Session *session, table_reference_st *ref);

=== modified file 'drizzled/sql_update.cc'
--- drizzled/sql_update.cc	2009-12-05 20:06:30 +0000
+++ drizzled/sql_update.cc	2009-12-07 19:18:46 +0000
@@ -134,7 +134,7 @@
   ha_rows	updated, found;
   key_map	old_covering_keys;
   Table		*table;
-  optimizer::SQL_SELECT *select= NULL;
+  optimizer::SqlSelect *select= NULL;
   READ_RECORD	info;
   Select_Lex    *select_lex= &session->lex->select_lex;
   uint64_t     id;
@@ -383,7 +383,7 @@
       }
       else
       {
-	select= new optimizer::SQL_SELECT;
+	select= new optimizer::SqlSelect;
 	select->head=table;
       }
       if (reinit_io_cache(&tempfile,READ_CACHE,0L,0,0))

=== modified file 'drizzled/sql_yacc.yy'
--- drizzled/sql_yacc.yy	2009-12-05 20:06:30 +0000
+++ drizzled/sql_yacc.yy	2009-12-08 21:44:46 +0000
@@ -88,6 +88,7 @@
 #include <drizzled/function/get_system_var.h>
 #include <mysys/thr_lock.h>
 #include <drizzled/message/table.pb.h>
+#include <drizzled/message/schema.pb.h>
 #include <drizzled/statement.h>
 #include <drizzled/statement/alter_schema.h>
 #include <drizzled/statement/alter_table.h>
@@ -129,6 +130,7 @@
 #include <drizzled/statement/truncate.h>
 #include <drizzled/statement/unlock_tables.h>
 #include <drizzled/statement/update.h>
+#include <drizzled/db.h>
 
 using namespace drizzled;
 
@@ -1366,8 +1368,8 @@
           {
             statement::CreateSchema *statement= (statement::CreateSchema *)Lex->statement;
 
-            statement->create_info.default_table_charset= $4;
-            statement->create_info.used_fields|= HA_CREATE_USED_DEFAULT_CHARSET;
+            message::Schema &schema_message= statement->schema_message;
+            schema_message.set_collation($4->name);
           }
         ;
 
@@ -2416,11 +2418,24 @@
             {
               DRIZZLE_YYABORT;
             }
-            if (check_table_name($3->table.str,$3->table.length) || ($3->db.str && check_db_name(&$3->db)))
+
+            if (check_table_name($3->table.str,$3->table.length))
             {
               my_error(ER_WRONG_TABLE_NAME, MYF(0), $3->table.str);
               DRIZZLE_YYABORT;
             }
+
+            if ($3->db.str)
+            {
+              std::string database_name($3->db.str);
+              NonNormalisedDatabaseName non_normalised_database_name(database_name);
+              NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+              if (! normalised_database_name.isValid())
+              {
+                my_error(ER_WRONG_TABLE_NAME, MYF(0), $3->table.str);
+                DRIZZLE_YYABORT;
+              }
+            }
             lex->name= $3->table;
             statement->alter_info.flags.set(ALTER_RENAME);
           }

=== modified file 'drizzled/statement/alter_schema.cc'
--- drizzled/statement/alter_schema.cc	2009-09-27 23:05:13 +0000
+++ drizzled/statement/alter_schema.cc	2009-12-08 01:51:02 +0000
@@ -22,16 +22,24 @@
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/alter_schema.h>
+#include <drizzled/db.h>
+
+#include <string>
+
+using namespace std;
 
 namespace drizzled
 {
 
 bool statement::AlterSchema::execute()
 {
-  LEX_STRING *db= &session->lex->name;
-  if (check_db_name(db))
+  string database_name(session->lex->name.str);
+  NonNormalisedDatabaseName non_normalised_database_name(database_name);
+  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+
+  if (! normalised_database_name.isValid())
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), db->str);
+    my_error(ER_WRONG_DB_NAME, MYF(0), normalised_database_name.to_string().c_str());
     return false;
   }
   if (session->inTransaction())
@@ -41,7 +49,7 @@
                MYF(0));
     return true;
   }
-  bool res= mysql_alter_db(session, db->str, &create_info);
+  bool res= mysql_alter_db(session, normalised_database_name, &schema_message);
   return res;
 }
 

=== modified file 'drizzled/statement/alter_table.cc'
--- drizzled/statement/alter_table.cc	2009-12-05 23:29:59 +0000
+++ drizzled/statement/alter_table.cc	2009-12-07 19:18:46 +0000
@@ -1333,7 +1333,7 @@
                       &tables, fields, all_fields, order) ||
           !(sortorder= make_unireg_sortorder(order, &length, NULL)) ||
           (from->sort.found_records= filesort(session, from, sortorder, length,
-                                              (optimizer::SQL_SELECT *) 0, HA_POS_ERROR,
+                                              (optimizer::SqlSelect *) 0, HA_POS_ERROR,
                                               1, &examined_rows)) ==
           HA_POS_ERROR)
         goto err;
@@ -1342,7 +1342,7 @@
 
   /* Tell handler that we have values for all columns in the to table */
   to->use_all_columns();
-  init_read_record(&info, session, from, (optimizer::SQL_SELECT *) 0, 1,1);
+  init_read_record(&info, session, from, (optimizer::SqlSelect *) 0, 1,1);
   if (ignore)
     to->cursor->extra(HA_EXTRA_IGNORE_DUP_KEY);
   session->row_count= 0;

=== modified file 'drizzled/statement/change_schema.cc'
--- drizzled/statement/change_schema.cc	2009-09-25 22:17:36 +0000
+++ drizzled/statement/change_schema.cc	2009-12-08 01:51:02 +0000
@@ -22,6 +22,11 @@
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/change_schema.h>
+#include <drizzled/db.h>
+
+#include <string>
+
+using namespace std;
 
 namespace drizzled
 {
@@ -29,9 +34,11 @@
 bool statement::ChangeSchema::execute()
 {
   Select_Lex *select_lex= &session->lex->select_lex;
-  LEX_STRING db_str= { (char *) select_lex->db, strlen(select_lex->db) };
+  string database_name(select_lex->db);
+  NonNormalisedDatabaseName non_normalised_database_name(database_name);
+  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
 
-  if (! mysql_change_db(session, &db_str, false))
+  if (! mysql_change_db(session, normalised_database_name, false))
   {
     session->my_ok();
   }

=== modified file 'drizzled/statement/create_index.cc'
--- drizzled/statement/create_index.cc	2009-11-17 02:26:31 +0000
+++ drizzled/statement/create_index.cc	2009-12-07 07:10:27 +0000
@@ -23,6 +23,7 @@
 #include "drizzled/session.h"
 #include "drizzled/statement/create_index.h"
 #include "drizzled/statement/alter_table.h"
+#include "drizzled/db.h"
 
 namespace drizzled
 {

=== modified file 'drizzled/statement/create_schema.cc'
--- drizzled/statement/create_schema.cc	2009-11-19 02:33:17 +0000
+++ drizzled/statement/create_schema.cc	2009-12-08 01:51:02 +0000
@@ -22,25 +22,33 @@
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/create_schema.h>
+#include <drizzled/db.h>
+
+#include <string>
+
+using namespace std;
 
 namespace drizzled
 {
 
 bool statement::CreateSchema::execute()
 {
+  string database_name(session->lex->name.str);
+  NonNormalisedDatabaseName non_normalised_database_name(database_name);
+  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+
+
   if (! session->endActiveTransaction())
   {
     return true;
   }
-  char *alias= session->strmake(session->lex->name.str,
-                                session->lex->name.length);
-  if (! alias ||
-      check_db_name(&session->lex->name))
+  if (! session->lex->name.str ||
+      ! normalised_database_name.isValid())
   {
     my_error(ER_WRONG_DB_NAME, MYF(0), session->lex->name.str);
     return false;
   }
-  bool res= mysql_create_db(session, session->lex->name.str, &create_info, is_if_not_exists);
+  bool res= mysql_create_db(session, normalised_database_name, &schema_message, is_if_not_exists);
   return res;
 }
 

=== modified file 'drizzled/statement/create_schema.h'
--- drizzled/statement/create_schema.h	2009-11-19 02:33:17 +0000
+++ drizzled/statement/create_schema.h	2009-12-07 05:09:37 +0000
@@ -22,6 +22,7 @@
 #define DRIZZLED_STATEMENT_CREATE_SCHEMA_H
 
 #include <drizzled/statement.h>
+#include <drizzled/message/schema.pb.h>
 
 class Session;
 
@@ -38,12 +39,11 @@
       Statement(in_session),
       is_if_not_exists(false)
   {
-    memset(&create_info, 0, sizeof(create_info));
   }
 
   bool execute();
   bool is_if_not_exists;
-  HA_CREATE_INFO create_info;
+  drizzled::message::Schema schema_message;
 };
 
 } /* end namespace statement */

=== modified file 'drizzled/statement/drop_index.cc'
--- drizzled/statement/drop_index.cc	2009-11-17 02:26:31 +0000
+++ drizzled/statement/drop_index.cc	2009-12-07 07:10:27 +0000
@@ -23,6 +23,7 @@
 #include "drizzled/session.h"
 #include "drizzled/statement/drop_index.h"
 #include "drizzled/statement/alter_table.h"
+#include "drizzled/db.h"
 
 namespace drizzled
 {

=== modified file 'drizzled/statement/drop_schema.cc'
--- drizzled/statement/drop_schema.cc	2009-09-27 23:05:13 +0000
+++ drizzled/statement/drop_schema.cc	2009-12-08 01:51:02 +0000
@@ -22,19 +22,29 @@
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/drop_schema.h>
+#include <drizzled/db.h>
+
+#include <string>
+
+using namespace std;
 
 namespace drizzled
 {
 
 bool statement::DropSchema::execute()
 {
+  string database_name(session->lex->name.str);
+  NonNormalisedDatabaseName non_normalised_database_name(database_name);
+  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+
   if (! session->endActiveTransaction())
   {
     return true;
   }
-  if (check_db_name(&session->lex->name))
+  if (! normalised_database_name.isValid())
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), session->lex->name.str);
+    my_error(ER_WRONG_DB_NAME, MYF(0),
+             normalised_database_name.to_string().c_str());
     return false;
   }
   if (session->inTransaction())
@@ -44,7 +54,7 @@
         MYF(0));
     return true;
   }
-  bool res= mysql_rm_db(session, session->lex->name.str, drop_if_exists);
+  bool res= mysql_rm_db(session, normalised_database_name, drop_if_exists);
   return res;
 }
 

=== modified file 'drizzled/statement/show_create_schema.cc'
--- drizzled/statement/show_create_schema.cc	2009-11-19 02:33:17 +0000
+++ drizzled/statement/show_create_schema.cc	2009-12-08 01:51:02 +0000
@@ -22,18 +22,28 @@
 #include <drizzled/show.h>
 #include <drizzled/session.h>
 #include <drizzled/statement/show_create_schema.h>
+#include <drizzled/db.h>
+
+#include <string>
+
+using namespace std;
 
 namespace drizzled
 {
 
 bool statement::ShowCreateSchema::execute()
 {
-  if (check_db_name(&session->lex->name))
+  string database_name(session->lex->name.str);
+  NonNormalisedDatabaseName non_normalised_database_name(database_name);
+  NormalisedDatabaseName normalised_database_name(non_normalised_database_name);
+
+  if (! normalised_database_name.isValid())
   {
-    my_error(ER_WRONG_DB_NAME, MYF(0), session->lex->name.str);
+    my_error(ER_WRONG_DB_NAME, MYF(0),
+             normalised_database_name.to_string().c_str());
     return false;
   }
-  bool res= mysqld_show_create_db(session, session->lex->name.str,
+  bool res= mysqld_show_create_db(session, normalised_database_name,
                                   is_if_not_exists);
   return res;
 }

=== modified file 'drizzled/structs.h'
--- drizzled/structs.h	2009-12-05 20:06:30 +0000
+++ drizzled/structs.h	2009-12-07 19:18:46 +0000
@@ -119,7 +119,7 @@
 struct st_read_record;				/* For referense later */
 class Session;
 class Cursor;
-namespace drizzled { namespace optimizer { class SQL_SELECT; } }
+namespace drizzled { namespace optimizer { class SqlSelect; } }
 
 typedef struct st_read_record {			/* Parameter to read_record */
   Table *table;			/* Head-form */
@@ -127,7 +127,7 @@
   Table **forms;			/* head and ref forms */
   int (*read_record)(struct st_read_record *);
   Session *session;
-  drizzled::optimizer::SQL_SELECT *select;
+  drizzled::optimizer::SqlSelect *select;
   uint32_t cache_records;
   uint32_t ref_length,struct_length,reclength,rec_cache_size,error_offset;
   uint32_t index;

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2009-12-05 20:06:30 +0000
+++ drizzled/table.cc	2009-12-08 21:44:46 +0000
@@ -1835,32 +1835,6 @@
 }
 
 /*
-  Check if database name is valid
-
-  SYNPOSIS
-    check_db_name()
-    org_name		Name of database and length
-
-  RETURN
-    0	ok
-    1   error
-*/
-
-bool check_db_name(LEX_STRING *org_name)
-{
-  char *name= org_name->str;
-  uint32_t name_length= org_name->length;
-
-  if (!name_length || name_length > NAME_LEN || name[name_length - 1] == ' ')
-    return 1;
-
-  my_casedn_str(files_charset_info, name);
-
-  return check_identifier_name(org_name);
-}
-
-
-/*
   Allow anything as a table name, as long as it doesn't contain an
   ' ' at the end
   returns 1 on error

=== modified file 'mysys/my_sys.h'
--- mysys/my_sys.h	2009-12-02 06:01:21 +0000
+++ mysys/my_sys.h	2009-12-08 21:44:46 +0000
@@ -277,7 +277,6 @@
 		     myf MyFlags);
 extern int _sanity(const char *sFile, uint32_t uLine);
 
-#define my_access access
 extern int check_if_legal_filename(const char *path);
 extern int check_if_legal_tablename(const char *path);
 

=== modified file 'plugin/info_schema/schemata.cc'
--- plugin/info_schema/schemata.cc	2009-11-26 04:33:51 +0000
+++ plugin/info_schema/schemata.cc	2009-12-07 07:10:27 +0000
@@ -27,7 +27,7 @@
 #include "drizzled/session.h"
 #include "drizzled/show.h"
 #include "drizzled/join_table.h"
-
+#include "drizzled/db.h"
 #include "helper_methods.h"
 #include "schemata.h"
 

=== modified file 'plugin/memcached_stats/analysis_table.cc'
--- plugin/memcached_stats/analysis_table.cc	2009-11-26 03:47:13 +0000
+++ plugin/memcached_stats/analysis_table.cc	2009-12-05 15:21:44 +0000
@@ -66,6 +66,16 @@
     memcached_analysis_st *report= memcached_analyze(serv, stats, &rc);
     table->restoreRecordAsDefault();
 
+    table->setWriteSet(0);
+    table->setWriteSet(1);
+    table->setWriteSet(2);
+    table->setWriteSet(3);
+    table->setWriteSet(4);
+    table->setWriteSet(5);
+    table->setWriteSet(6);
+    table->setWriteSet(7);
+    table->setWriteSet(8);
+
     table->field[0]->store(server_count);
     table->field[1]->store(report->average_item_size);
 

=== modified file 'plugin/memcached_stats/stats_table.cc'
--- plugin/memcached_stats/stats_table.cc	2009-11-26 03:47:13 +0000
+++ plugin/memcached_stats/stats_table.cc	2009-12-05 15:21:44 +0000
@@ -66,6 +66,9 @@
     char **list= memcached_stat_get_keys(serv, &stats[i], &rc);
     char **ptr= NULL;
 
+    table->setWriteSet(0);
+    table->setWriteSet(1);
+
     table->field[0]->store(memcached_server_name(serv, servers[i]),
                            64,
                            scs);
@@ -74,6 +77,8 @@
     for (ptr= list; *ptr; ptr++)
     {
       char *value= memcached_stat_get_value(serv, &stats[i], *ptr, &rc);
+
+      table->setWriteSet(col);
       table->field[col]->store(value,
                                64,
                                scs);

=== added directory 'plugin/show_schema_proto'
=== added file 'plugin/show_schema_proto/plugin.ini'
--- plugin/show_schema_proto/plugin.ini	1970-01-01 00:00:00 +0000
+++ plugin/show_schema_proto/plugin.ini	2009-12-08 04:23:00 +0000
@@ -0,0 +1,6 @@
+[plugin]
+name=show_schema_proto
+title=SHOW_SCHEMA_PROTO(db) Function
+description=Shows a text representation of the schema (database) proto
+load_by_default=yes
+testsuite=show_schema_proto

=== added file 'plugin/show_schema_proto/show_schema_proto.cc'
--- plugin/show_schema_proto/show_schema_proto.cc	1970-01-01 00:00:00 +0000
+++ plugin/show_schema_proto/show_schema_proto.cc	2009-12-08 04:23:00 +0000
@@ -0,0 +1,134 @@
+/* vim: expandtab:shiftwidth=2:tabstop=2:smarttab:
+   Copyright (C) 2009 Sun Microsystems
+
+   This program is free software; you can redistribute it and/or modify
+   it under the terms of the GNU General Public License as published by
+   the Free Software Foundation; version 2 of the License.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+   GNU General Public License for more details.
+
+   You should have received a copy of the GNU General Public License
+   along with this program; if not, write to the Free Software
+   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA */
+
+#include <drizzled/server_includes.h>
+#include <drizzled/plugin/function.h>
+#include <drizzled/item/func.h>
+#include <drizzled/function/str/strfunc.h>
+#include <drizzled/error.h>
+#include <drizzled/current_session.h>
+#include <drizzled/db.h>
+
+#include <stdio.h>
+#include <iostream>
+#include <string>
+#include <drizzled/message/schema.pb.h>
+#include <google/protobuf/io/zero_copy_stream.h>
+#include <google/protobuf/io/zero_copy_stream_impl.h>
+#include <google/protobuf/text_format.h>
+
+using namespace std;
+using namespace drizzled;
+using namespace google;
+
+class ShowSchemaProtoFunction : public Item_str_func
+{
+public:
+  ShowSchemaProtoFunction() : Item_str_func() {}
+  String *val_str(String*);
+
+  void fix_length_and_dec()
+  {
+    max_length= 16384; /* Guesswork? */
+    args[0]->collation.set(
+      get_charset_by_csname(args[0]->collation.collation->csname,
+                            MY_CS_BINSORT), DERIVATION_COERCIBLE);
+  }
+
+  const char *func_name() const
+  {
+    return "show_schema_proto";
+  }
+
+  bool check_argument_count(int n)
+  {
+    return (n == 1);
+  }
+};
+
+
+String *ShowSchemaProtoFunction::val_str(String *str)
+{
+  assert(fixed == true);
+
+  String *db_sptr= args[0]->val_str(str);
+
+  if (db_sptr == NULL)
+  {
+    null_value= true;
+    return NULL;
+  }
+
+  null_value= false;
+
+  const char* db= db_sptr->c_ptr_safe();
+
+  string proto_as_text("");
+  message::Schema proto;
+
+  int err= get_database_metadata(db, &proto);
+
+  if (err != 0)
+  {
+    my_error(ER_BAD_DB_ERROR, MYF(0), db);
+    return NULL;
+  }
+
+  protobuf::TextFormat::PrintToString(proto, &proto_as_text);
+
+  if (str->alloc(proto_as_text.length()))
+  {
+    null_value= true;
+    return NULL;
+  }
+
+  str->length(proto_as_text.length());
+
+  strncpy(str->ptr(),proto_as_text.c_str(), proto_as_text.length());
+
+  return str;
+}
+
+plugin::Create_function<ShowSchemaProtoFunction> *show_schema_proto_func= NULL;
+
+static int initialize(plugin::Registry &registry)
+{
+  show_schema_proto_func= new plugin::Create_function<ShowSchemaProtoFunction>("show_schema_proto");
+  registry.add(show_schema_proto_func);
+  return 0;
+}
+
+static int finalize(plugin::Registry &registry)
+{
+  registry.remove(show_schema_proto_func);
+  delete show_schema_proto_func;
+  return 0;
+}
+
+DRIZZLE_DECLARE_PLUGIN
+{
+  "show_schema_proto",
+  "1.0",
+  "Stewart Smith",
+  "Shows text representation of schema definition proto",
+  PLUGIN_LICENSE_GPL,
+  initialize, /* Plugin Init */
+  finalize,   /* Plugin Deinit */
+  NULL,   /* status variables */
+  NULL,   /* system variables */
+  NULL    /* config options */
+}
+DRIZZLE_DECLARE_PLUGIN_END;

=== added directory 'plugin/show_schema_proto/tests'
=== added directory 'plugin/show_schema_proto/tests/r'
=== added file 'plugin/show_schema_proto/tests/r/basic.result'
--- plugin/show_schema_proto/tests/r/basic.result	1970-01-01 00:00:00 +0000
+++ plugin/show_schema_proto/tests/r/basic.result	2009-12-08 04:23:00 +0000
@@ -0,0 +1,22 @@
+SELECT SHOW_SCHEMA_PROTO('not_exists');
+ERROR 42000: Unknown database 'not_exists'
+CREATE DATABASE foobar COLLATE = utf8_bin;
+SELECT SHOW_SCHEMA_PROTO('foobar');
+SHOW_SCHEMA_PROTO('foobar')
+name: "foobar"
+collation: "utf8_bin"
+
+DROP DATABASE foobar;
+CREATE DATABASE foobar;
+SELECT SHOW_SCHEMA_PROTO('foobar');
+SHOW_SCHEMA_PROTO('foobar')
+name: "foobar"
+collation: "utf8_general_ci"
+
+ALTER DATABASE foobar COLLATE = utf8_bin;
+SELECT SHOW_SCHEMA_PROTO('foobar');
+SHOW_SCHEMA_PROTO('foobar')
+name: "foobar"
+collation: "utf8_bin"
+
+DROP DATABASE foobar;

=== added directory 'plugin/show_schema_proto/tests/t'
=== added file 'plugin/show_schema_proto/tests/t/basic.test'
--- plugin/show_schema_proto/tests/t/basic.test	1970-01-01 00:00:00 +0000
+++ plugin/show_schema_proto/tests/t/basic.test	2009-12-08 04:23:00 +0000
@@ -0,0 +1,11 @@
+#SELECT SHOW_SCHEMA_PROTO('test');
+--error 1049
+SELECT SHOW_SCHEMA_PROTO('not_exists');
+CREATE DATABASE foobar COLLATE = utf8_bin;
+SELECT SHOW_SCHEMA_PROTO('foobar');
+DROP DATABASE foobar;
+CREATE DATABASE foobar;
+SELECT SHOW_SCHEMA_PROTO('foobar');
+ALTER DATABASE foobar COLLATE = utf8_bin;
+SELECT SHOW_SCHEMA_PROTO('foobar');
+DROP DATABASE foobar;

=== added file 'tests/r/create_max_fields.result'
--- tests/r/create_max_fields.result	1970-01-01 00:00:00 +0000
+++ tests/r/create_max_fields.result	2009-12-08 03:43:57 +0000
@@ -0,0 +1,12301 @@
+One column less than MAX_FIELDS
+CREATE TABLE t1 (
+f1 int,
+f2 int,
+f3 int,
+f4 int,
+f5 int,
+f6 int,
+f7 int,
+f8 int,
+f9 int,
+f10 int,
+f11 int,
+f12 int,
+f13 int,
+f14 int,
+f15 int,
+f16 int,
+f17 int,
+f18 int,
+f19 int,
+f20 int,
+f21 int,
+f22 int,
+f23 int,
+f24 int,
+f25 int,
+f26 int,
+f27 int,
+f28 int,
+f29 int,
+f30 int,
+f31 int,
+f32 int,
+f33 int,
+f34 int,
+f35 int,
+f36 int,
+f37 int,
+f38 int,
+f39 int,
+f40 int,
+f41 int,
+f42 int,
+f43 int,
+f44 int,
+f45 int,
+f46 int,
+f47 int,
+f48 int,
+f49 int,
+f50 int,
+f51 int,
+f52 int,
+f53 int,
+f54 int,
+f55 int,
+f56 int,
+f57 int,
+f58 int,
+f59 int,
+f60 int,
+f61 int,
+f62 int,
+f63 int,
+f64 int,
+f65 int,
+f66 int,
+f67 int,
+f68 int,
+f69 int,
+f70 int,
+f71 int,
+f72 int,
+f73 int,
+f74 int,
+f75 int,
+f76 int,
+f77 int,
+f78 int,
+f79 int,
+f80 int,
+f81 int,
+f82 int,
+f83 int,
+f84 int,
+f85 int,
+f86 int,
+f87 int,
+f88 int,
+f89 int,
+f90 int,
+f91 int,
+f92 int,
+f93 int,
+f94 int,
+f95 int,
+f96 int,
+f97 int,
+f98 int,
+f99 int,
+f100 int,
+f101 int,
+f102 int,
+f103 int,
+f104 int,
+f105 int,
+f106 int,
+f107 int,
+f108 int,
+f109 int,
+f110 int,
+f111 int,
+f112 int,
+f113 int,
+f114 int,
+f115 int,
+f116 int,
+f117 int,
+f118 int,
+f119 int,
+f120 int,
+f121 int,
+f122 int,
+f123 int,
+f124 int,
+f125 int,
+f126 int,
+f127 int,
+f128 int,
+f129 int,
+f130 int,
+f131 int,
+f132 int,
+f133 int,
+f134 int,
+f135 int,
+f136 int,
+f137 int,
+f138 int,
+f139 int,
+f140 int,
+f141 int,
+f142 int,
+f143 int,
+f144 int,
+f145 int,
+f146 int,
+f147 int,
+f148 int,
+f149 int,
+f150 int,
+f151 int,
+f152 int,
+f153 int,
+f154 int,
+f155 int,
+f156 int,
+f157 int,
+f158 int,
+f159 int,
+f160 int,
+f161 int,
+f162 int,
+f163 int,
+f164 int,
+f165 int,
+f166 int,
+f167 int,
+f168 int,
+f169 int,
+f170 int,
+f171 int,
+f172 int,
+f173 int,
+f174 int,
+f175 int,
+f176 int,
+f177 int,
+f178 int,
+f179 int,
+f180 int,
+f181 int,
+f182 int,
+f183 int,
+f184 int,
+f185 int,
+f186 int,
+f187 int,
+f188 int,
+f189 int,
+f190 int,
+f191 int,
+f192 int,
+f193 int,
+f194 int,
+f195 int,
+f196 int,
+f197 int,
+f198 int,
+f199 int,
+f200 int,
+f201 int,
+f202 int,
+f203 int,
+f204 int,
+f205 int,
+f206 int,
+f207 int,
+f208 int,
+f209 int,
+f210 int,
+f211 int,
+f212 int,
+f213 int,
+f214 int,
+f215 int,
+f216 int,
+f217 int,
+f218 int,
+f219 int,
+f220 int,
+f221 int,
+f222 int,
+f223 int,
+f224 int,
+f225 int,
+f226 int,
+f227 int,
+f228 int,
+f229 int,
+f230 int,
+f231 int,
+f232 int,
+f233 int,
+f234 int,
+f235 int,
+f236 int,
+f237 int,
+f238 int,
+f239 int,
+f240 int,
+f241 int,
+f242 int,
+f243 int,
+f244 int,
+f245 int,
+f246 int,
+f247 int,
+f248 int,
+f249 int,
+f250 int,
+f251 int,
+f252 int,
+f253 int,
+f254 int,
+f255 int,
+f256 int,
+f257 int,
+f258 int,
+f259 int,
+f260 int,
+f261 int,
+f262 int,
+f263 int,
+f264 int,
+f265 int,
+f266 int,
+f267 int,
+f268 int,
+f269 int,
+f270 int,
+f271 int,
+f272 int,
+f273 int,
+f274 int,
+f275 int,
+f276 int,
+f277 int,
+f278 int,
+f279 int,
+f280 int,
+f281 int,
+f282 int,
+f283 int,
+f284 int,
+f285 int,
+f286 int,
+f287 int,
+f288 int,
+f289 int,
+f290 int,
+f291 int,
+f292 int,
+f293 int,
+f294 int,
+f295 int,
+f296 int,
+f297 int,
+f298 int,
+f299 int,
+f300 int,
+f301 int,
+f302 int,
+f303 int,
+f304 int,
+f305 int,
+f306 int,
+f307 int,
+f308 int,
+f309 int,
+f310 int,
+f311 int,
+f312 int,
+f313 int,
+f314 int,
+f315 int,
+f316 int,
+f317 int,
+f318 int,
+f319 int,
+f320 int,
+f321 int,
+f322 int,
+f323 int,
+f324 int,
+f325 int,
+f326 int,
+f327 int,
+f328 int,
+f329 int,
+f330 int,
+f331 int,
+f332 int,
+f333 int,
+f334 int,
+f335 int,
+f336 int,
+f337 int,
+f338 int,
+f339 int,
+f340 int,
+f341 int,
+f342 int,
+f343 int,
+f344 int,
+f345 int,
+f346 int,
+f347 int,
+f348 int,
+f349 int,
+f350 int,
+f351 int,
+f352 int,
+f353 int,
+f354 int,
+f355 int,
+f356 int,
+f357 int,
+f358 int,
+f359 int,
+f360 int,
+f361 int,
+f362 int,
+f363 int,
+f364 int,
+f365 int,
+f366 int,
+f367 int,
+f368 int,
+f369 int,
+f370 int,
+f371 int,
+f372 int,
+f373 int,
+f374 int,
+f375 int,
+f376 int,
+f377 int,
+f378 int,
+f379 int,
+f380 int,
+f381 int,
+f382 int,
+f383 int,
+f384 int,
+f385 int,
+f386 int,
+f387 int,
+f388 int,
+f389 int,
+f390 int,
+f391 int,
+f392 int,
+f393 int,
+f394 int,
+f395 int,
+f396 int,
+f397 int,
+f398 int,
+f399 int,
+f400 int,
+f401 int,
+f402 int,
+f403 int,
+f404 int,
+f405 int,
+f406 int,
+f407 int,
+f408 int,
+f409 int,
+f410 int,
+f411 int,
+f412 int,
+f413 int,
+f414 int,
+f415 int,
+f416 int,
+f417 int,
+f418 int,
+f419 int,
+f420 int,
+f421 int,
+f422 int,
+f423 int,
+f424 int,
+f425 int,
+f426 int,
+f427 int,
+f428 int,
+f429 int,
+f430 int,
+f431 int,
+f432 int,
+f433 int,
+f434 int,
+f435 int,
+f436 int,
+f437 int,
+f438 int,
+f439 int,
+f440 int,
+f441 int,
+f442 int,
+f443 int,
+f444 int,
+f445 int,
+f446 int,
+f447 int,
+f448 int,
+f449 int,
+f450 int,
+f451 int,
+f452 int,
+f453 int,
+f454 int,
+f455 int,
+f456 int,
+f457 int,
+f458 int,
+f459 int,
+f460 int,
+f461 int,
+f462 int,
+f463 int,
+f464 int,
+f465 int,
+f466 int,
+f467 int,
+f468 int,
+f469 int,
+f470 int,
+f471 int,
+f472 int,
+f473 int,
+f474 int,
+f475 int,
+f476 int,
+f477 int,
+f478 int,
+f479 int,
+f480 int,
+f481 int,
+f482 int,
+f483 int,
+f484 int,
+f485 int,
+f486 int,
+f487 int,
+f488 int,
+f489 int,
+f490 int,
+f491 int,
+f492 int,
+f493 int,
+f494 int,
+f495 int,
+f496 int,
+f497 int,
+f498 int,
+f499 int,
+f500 int,
+f501 int,
+f502 int,
+f503 int,
+f504 int,
+f505 int,
+f506 int,
+f507 int,
+f508 int,
+f509 int,
+f510 int,
+f511 int,
+f512 int,
+f513 int,
+f514 int,
+f515 int,
+f516 int,
+f517 int,
+f518 int,
+f519 int,
+f520 int,
+f521 int,
+f522 int,
+f523 int,
+f524 int,
+f525 int,
+f526 int,
+f527 int,
+f528 int,
+f529 int,
+f530 int,
+f531 int,
+f532 int,
+f533 int,
+f534 int,
+f535 int,
+f536 int,
+f537 int,
+f538 int,
+f539 int,
+f540 int,
+f541 int,
+f542 int,
+f543 int,
+f544 int,
+f545 int,
+f546 int,
+f547 int,
+f548 int,
+f549 int,
+f550 int,
+f551 int,
+f552 int,
+f553 int,
+f554 int,
+f555 int,
+f556 int,
+f557 int,
+f558 int,
+f559 int,
+f560 int,
+f561 int,
+f562 int,
+f563 int,
+f564 int,
+f565 int,
+f566 int,
+f567 int,
+f568 int,
+f569 int,
+f570 int,
+f571 int,
+f572 int,
+f573 int,
+f574 int,
+f575 int,
+f576 int,
+f577 int,
+f578 int,
+f579 int,
+f580 int,
+f581 int,
+f582 int,
+f583 int,
+f584 int,
+f585 int,
+f586 int,
+f587 int,
+f588 int,
+f589 int,
+f590 int,
+f591 int,
+f592 int,
+f593 int,
+f594 int,
+f595 int,
+f596 int,
+f597 int,
+f598 int,
+f599 int,
+f600 int,
+f601 int,
+f602 int,
+f603 int,
+f604 int,
+f605 int,
+f606 int,
+f607 int,
+f608 int,
+f609 int,
+f610 int,
+f611 int,
+f612 int,
+f613 int,
+f614 int,
+f615 int,
+f616 int,
+f617 int,
+f618 int,
+f619 int,
+f620 int,
+f621 int,
+f622 int,
+f623 int,
+f624 int,
+f625 int,
+f626 int,
+f627 int,
+f628 int,
+f629 int,
+f630 int,
+f631 int,
+f632 int,
+f633 int,
+f634 int,
+f635 int,
+f636 int,
+f637 int,
+f638 int,
+f639 int,
+f640 int,
+f641 int,
+f642 int,
+f643 int,
+f644 int,
+f645 int,
+f646 int,
+f647 int,
+f648 int,
+f649 int,
+f650 int,
+f651 int,
+f652 int,
+f653 int,
+f654 int,
+f655 int,
+f656 int,
+f657 int,
+f658 int,
+f659 int,
+f660 int,
+f661 int,
+f662 int,
+f663 int,
+f664 int,
+f665 int,
+f666 int,
+f667 int,
+f668 int,
+f669 int,
+f670 int,
+f671 int,
+f672 int,
+f673 int,
+f674 int,
+f675 int,
+f676 int,
+f677 int,
+f678 int,
+f679 int,
+f680 int,
+f681 int,
+f682 int,
+f683 int,
+f684 int,
+f685 int,
+f686 int,
+f687 int,
+f688 int,
+f689 int,
+f690 int,
+f691 int,
+f692 int,
+f693 int,
+f694 int,
+f695 int,
+f696 int,
+f697 int,
+f698 int,
+f699 int,
+f700 int,
+f701 int,
+f702 int,
+f703 int,
+f704 int,
+f705 int,
+f706 int,
+f707 int,
+f708 int,
+f709 int,
+f710 int,
+f711 int,
+f712 int,
+f713 int,
+f714 int,
+f715 int,
+f716 int,
+f717 int,
+f718 int,
+f719 int,
+f720 int,
+f721 int,
+f722 int,
+f723 int,
+f724 int,
+f725 int,
+f726 int,
+f727 int,
+f728 int,
+f729 int,
+f730 int,
+f731 int,
+f732 int,
+f733 int,
+f734 int,
+f735 int,
+f736 int,
+f737 int,
+f738 int,
+f739 int,
+f740 int,
+f741 int,
+f742 int,
+f743 int,
+f744 int,
+f745 int,
+f746 int,
+f747 int,
+f748 int,
+f749 int,
+f750 int,
+f751 int,
+f752 int,
+f753 int,
+f754 int,
+f755 int,
+f756 int,
+f757 int,
+f758 int,
+f759 int,
+f760 int,
+f761 int,
+f762 int,
+f763 int,
+f764 int,
+f765 int,
+f766 int,
+f767 int,
+f768 int,
+f769 int,
+f770 int,
+f771 int,
+f772 int,
+f773 int,
+f774 int,
+f775 int,
+f776 int,
+f777 int,
+f778 int,
+f779 int,
+f780 int,
+f781 int,
+f782 int,
+f783 int,
+f784 int,
+f785 int,
+f786 int,
+f787 int,
+f788 int,
+f789 int,
+f790 int,
+f791 int,
+f792 int,
+f793 int,
+f794 int,
+f795 int,
+f796 int,
+f797 int,
+f798 int,
+f799 int,
+f800 int,
+f801 int,
+f802 int,
+f803 int,
+f804 int,
+f805 int,
+f806 int,
+f807 int,
+f808 int,
+f809 int,
+f810 int,
+f811 int,
+f812 int,
+f813 int,
+f814 int,
+f815 int,
+f816 int,
+f817 int,
+f818 int,
+f819 int,
+f820 int,
+f821 int,
+f822 int,
+f823 int,
+f824 int,
+f825 int,
+f826 int,
+f827 int,
+f828 int,
+f829 int,
+f830 int,
+f831 int,
+f832 int,
+f833 int,
+f834 int,
+f835 int,
+f836 int,
+f837 int,
+f838 int,
+f839 int,
+f840 int,
+f841 int,
+f842 int,
+f843 int,
+f844 int,
+f845 int,
+f846 int,
+f847 int,
+f848 int,
+f849 int,
+f850 int,
+f851 int,
+f852 int,
+f853 int,
+f854 int,
+f855 int,
+f856 int,
+f857 int,
+f858 int,
+f859 int,
+f860 int,
+f861 int,
+f862 int,
+f863 int,
+f864 int,
+f865 int,
+f866 int,
+f867 int,
+f868 int,
+f869 int,
+f870 int,
+f871 int,
+f872 int,
+f873 int,
+f874 int,
+f875 int,
+f876 int,
+f877 int,
+f878 int,
+f879 int,
+f880 int,
+f881 int,
+f882 int,
+f883 int,
+f884 int,
+f885 int,
+f886 int,
+f887 int,
+f888 int,
+f889 int,
+f890 int,
+f891 int,
+f892 int,
+f893 int,
+f894 int,
+f895 int,
+f896 int,
+f897 int,
+f898 int,
+f899 int,
+f900 int,
+f901 int,
+f902 int,
+f903 int,
+f904 int,
+f905 int,
+f906 int,
+f907 int,
+f908 int,
+f909 int,
+f910 int,
+f911 int,
+f912 int,
+f913 int,
+f914 int,
+f915 int,
+f916 int,
+f917 int,
+f918 int,
+f919 int,
+f920 int,
+f921 int,
+f922 int,
+f923 int,
+f924 int,
+f925 int,
+f926 int,
+f927 int,
+f928 int,
+f929 int,
+f930 int,
+f931 int,
+f932 int,
+f933 int,
+f934 int,
+f935 int,
+f936 int,
+f937 int,
+f938 int,
+f939 int,
+f940 int,
+f941 int,
+f942 int,
+f943 int,
+f944 int,
+f945 int,
+f946 int,
+f947 int,
+f948 int,
+f949 int,
+f950 int,
+f951 int,
+f952 int,
+f953 int,
+f954 int,
+f955 int,
+f956 int,
+f957 int,
+f958 int,
+f959 int,
+f960 int,
+f961 int,
+f962 int,
+f963 int,
+f964 int,
+f965 int,
+f966 int,
+f967 int,
+f968 int,
+f969 int,
+f970 int,
+f971 int,
+f972 int,
+f973 int,
+f974 int,
+f975 int,
+f976 int,
+f977 int,
+f978 int,
+f979 int,
+f980 int,
+f981 int,
+f982 int,
+f983 int,
+f984 int,
+f985 int,
+f986 int,
+f987 int,
+f988 int,
+f989 int,
+f990 int,
+f991 int,
+f992 int,
+f993 int,
+f994 int,
+f995 int,
+f996 int,
+f997 int,
+f998 int,
+f999 int,
+f1000 int,
+f1001 int,
+f1002 int,
+f1003 int,
+f1004 int,
+f1005 int,
+f1006 int,
+f1007 int,
+f1008 int,
+f1009 int,
+f1010 int,
+f1011 int,
+f1012 int,
+f1013 int,
+f1014 int,
+f1015 int,
+f1016 int,
+f1017 int,
+f1018 int,
+f1019 int,
+f1020 int,
+f1021 int,
+f1022 int,
+f1023 int,
+f1024 int,
+f1025 int,
+f1026 int,
+f1027 int,
+f1028 int,
+f1029 int,
+f1030 int,
+f1031 int,
+f1032 int,
+f1033 int,
+f1034 int,
+f1035 int,
+f1036 int,
+f1037 int,
+f1038 int,
+f1039 int,
+f1040 int,
+f1041 int,
+f1042 int,
+f1043 int,
+f1044 int,
+f1045 int,
+f1046 int,
+f1047 int,
+f1048 int,
+f1049 int,
+f1050 int,
+f1051 int,
+f1052 int,
+f1053 int,
+f1054 int,
+f1055 int,
+f1056 int,
+f1057 int,
+f1058 int,
+f1059 int,
+f1060 int,
+f1061 int,
+f1062 int,
+f1063 int,
+f1064 int,
+f1065 int,
+f1066 int,
+f1067 int,
+f1068 int,
+f1069 int,
+f1070 int,
+f1071 int,
+f1072 int,
+f1073 int,
+f1074 int,
+f1075 int,
+f1076 int,
+f1077 int,
+f1078 int,
+f1079 int,
+f1080 int,
+f1081 int,
+f1082 int,
+f1083 int,
+f1084 int,
+f1085 int,
+f1086 int,
+f1087 int,
+f1088 int,
+f1089 int,
+f1090 int,
+f1091 int,
+f1092 int,
+f1093 int,
+f1094 int,
+f1095 int,
+f1096 int,
+f1097 int,
+f1098 int,
+f1099 int,
+f1100 int,
+f1101 int,
+f1102 int,
+f1103 int,
+f1104 int,
+f1105 int,
+f1106 int,
+f1107 int,
+f1108 int,
+f1109 int,
+f1110 int,
+f1111 int,
+f1112 int,
+f1113 int,
+f1114 int,
+f1115 int,
+f1116 int,
+f1117 int,
+f1118 int,
+f1119 int,
+f1120 int,
+f1121 int,
+f1122 int,
+f1123 int,
+f1124 int,
+f1125 int,
+f1126 int,
+f1127 int,
+f1128 int,
+f1129 int,
+f1130 int,
+f1131 int,
+f1132 int,
+f1133 int,
+f1134 int,
+f1135 int,
+f1136 int,
+f1137 int,
+f1138 int,
+f1139 int,
+f1140 int,
+f1141 int,
+f1142 int,
+f1143 int,
+f1144 int,
+f1145 int,
+f1146 int,
+f1147 int,
+f1148 int,
+f1149 int,
+f1150 int,
+f1151 int,
+f1152 int,
+f1153 int,
+f1154 int,
+f1155 int,
+f1156 int,
+f1157 int,
+f1158 int,
+f1159 int,
+f1160 int,
+f1161 int,
+f1162 int,
+f1163 int,
+f1164 int,
+f1165 int,
+f1166 int,
+f1167 int,
+f1168 int,
+f1169 int,
+f1170 int,
+f1171 int,
+f1172 int,
+f1173 int,
+f1174 int,
+f1175 int,
+f1176 int,
+f1177 int,
+f1178 int,
+f1179 int,
+f1180 int,
+f1181 int,
+f1182 int,
+f1183 int,
+f1184 int,
+f1185 int,
+f1186 int,
+f1187 int,
+f1188 int,
+f1189 int,
+f1190 int,
+f1191 int,
+f1192 int,
+f1193 int,
+f1194 int,
+f1195 int,
+f1196 int,
+f1197 int,
+f1198 int,
+f1199 int,
+f1200 int,
+f1201 int,
+f1202 int,
+f1203 int,
+f1204 int,
+f1205 int,
+f1206 int,
+f1207 int,
+f1208 int,
+f1209 int,
+f1210 int,
+f1211 int,
+f1212 int,
+f1213 int,
+f1214 int,
+f1215 int,
+f1216 int,
+f1217 int,
+f1218 int,
+f1219 int,
+f1220 int,
+f1221 int,
+f1222 int,
+f1223 int,
+f1224 int,
+f1225 int,
+f1226 int,
+f1227 int,
+f1228 int,
+f1229 int,
+f1230 int,
+f1231 int,
+f1232 int,
+f1233 int,
+f1234 int,
+f1235 int,
+f1236 int,
+f1237 int,
+f1238 int,
+f1239 int,
+f1240 int,
+f1241 int,
+f1242 int,
+f1243 int,
+f1244 int,
+f1245 int,
+f1246 int,
+f1247 int,
+f1248 int,
+f1249 int,
+f1250 int,
+f1251 int,
+f1252 int,
+f1253 int,
+f1254 int,
+f1255 int,
+f1256 int,
+f1257 int,
+f1258 int,
+f1259 int,
+f1260 int,
+f1261 int,
+f1262 int,
+f1263 int,
+f1264 int,
+f1265 int,
+f1266 int,
+f1267 int,
+f1268 int,
+f1269 int,
+f1270 int,
+f1271 int,
+f1272 int,
+f1273 int,
+f1274 int,
+f1275 int,
+f1276 int,
+f1277 int,
+f1278 int,
+f1279 int,
+f1280 int,
+f1281 int,
+f1282 int,
+f1283 int,
+f1284 int,
+f1285 int,
+f1286 int,
+f1287 int,
+f1288 int,
+f1289 int,
+f1290 int,
+f1291 int,
+f1292 int,
+f1293 int,
+f1294 int,
+f1295 int,
+f1296 int,
+f1297 int,
+f1298 int,
+f1299 int,
+f1300 int,
+f1301 int,
+f1302 int,
+f1303 int,
+f1304 int,
+f1305 int,
+f1306 int,
+f1307 int,
+f1308 int,
+f1309 int,
+f1310 int,
+f1311 int,
+f1312 int,
+f1313 int,
+f1314 int,
+f1315 int,
+f1316 int,
+f1317 int,
+f1318 int,
+f1319 int,
+f1320 int,
+f1321 int,
+f1322 int,
+f1323 int,
+f1324 int,
+f1325 int,
+f1326 int,
+f1327 int,
+f1328 int,
+f1329 int,
+f1330 int,
+f1331 int,
+f1332 int,
+f1333 int,
+f1334 int,
+f1335 int,
+f1336 int,
+f1337 int,
+f1338 int,
+f1339 int,
+f1340 int,
+f1341 int,
+f1342 int,
+f1343 int,
+f1344 int,
+f1345 int,
+f1346 int,
+f1347 int,
+f1348 int,
+f1349 int,
+f1350 int,
+f1351 int,
+f1352 int,
+f1353 int,
+f1354 int,
+f1355 int,
+f1356 int,
+f1357 int,
+f1358 int,
+f1359 int,
+f1360 int,
+f1361 int,
+f1362 int,
+f1363 int,
+f1364 int,
+f1365 int,
+f1366 int,
+f1367 int,
+f1368 int,
+f1369 int,
+f1370 int,
+f1371 int,
+f1372 int,
+f1373 int,
+f1374 int,
+f1375 int,
+f1376 int,
+f1377 int,
+f1378 int,
+f1379 int,
+f1380 int,
+f1381 int,
+f1382 int,
+f1383 int,
+f1384 int,
+f1385 int,
+f1386 int,
+f1387 int,
+f1388 int,
+f1389 int,
+f1390 int,
+f1391 int,
+f1392 int,
+f1393 int,
+f1394 int,
+f1395 int,
+f1396 int,
+f1397 int,
+f1398 int,
+f1399 int,
+f1400 int,
+f1401 int,
+f1402 int,
+f1403 int,
+f1404 int,
+f1405 int,
+f1406 int,
+f1407 int,
+f1408 int,
+f1409 int,
+f1410 int,
+f1411 int,
+f1412 int,
+f1413 int,
+f1414 int,
+f1415 int,
+f1416 int,
+f1417 int,
+f1418 int,
+f1419 int,
+f1420 int,
+f1421 int,
+f1422 int,
+f1423 int,
+f1424 int,
+f1425 int,
+f1426 int,
+f1427 int,
+f1428 int,
+f1429 int,
+f1430 int,
+f1431 int,
+f1432 int,
+f1433 int,
+f1434 int,
+f1435 int,
+f1436 int,
+f1437 int,
+f1438 int,
+f1439 int,
+f1440 int,
+f1441 int,
+f1442 int,
+f1443 int,
+f1444 int,
+f1445 int,
+f1446 int,
+f1447 int,
+f1448 int,
+f1449 int,
+f1450 int,
+f1451 int,
+f1452 int,
+f1453 int,
+f1454 int,
+f1455 int,
+f1456 int,
+f1457 int,
+f1458 int,
+f1459 int,
+f1460 int,
+f1461 int,
+f1462 int,
+f1463 int,
+f1464 int,
+f1465 int,
+f1466 int,
+f1467 int,
+f1468 int,
+f1469 int,
+f1470 int,
+f1471 int,
+f1472 int,
+f1473 int,
+f1474 int,
+f1475 int,
+f1476 int,
+f1477 int,
+f1478 int,
+f1479 int,
+f1480 int,
+f1481 int,
+f1482 int,
+f1483 int,
+f1484 int,
+f1485 int,
+f1486 int,
+f1487 int,
+f1488 int,
+f1489 int,
+f1490 int,
+f1491 int,
+f1492 int,
+f1493 int,
+f1494 int,
+f1495 int,
+f1496 int,
+f1497 int,
+f1498 int,
+f1499 int,
+f1500 int,
+f1501 int,
+f1502 int,
+f1503 int,
+f1504 int,
+f1505 int,
+f1506 int,
+f1507 int,
+f1508 int,
+f1509 int,
+f1510 int,
+f1511 int,
+f1512 int,
+f1513 int,
+f1514 int,
+f1515 int,
+f1516 int,
+f1517 int,
+f1518 int,
+f1519 int,
+f1520 int,
+f1521 int,
+f1522 int,
+f1523 int,
+f1524 int,
+f1525 int,
+f1526 int,
+f1527 int,
+f1528 int,
+f1529 int,
+f1530 int,
+f1531 int,
+f1532 int,
+f1533 int,
+f1534 int,
+f1535 int,
+f1536 int,
+f1537 int,
+f1538 int,
+f1539 int,
+f1540 int,
+f1541 int,
+f1542 int,
+f1543 int,
+f1544 int,
+f1545 int,
+f1546 int,
+f1547 int,
+f1548 int,
+f1549 int,
+f1550 int,
+f1551 int,
+f1552 int,
+f1553 int,
+f1554 int,
+f1555 int,
+f1556 int,
+f1557 int,
+f1558 int,
+f1559 int,
+f1560 int,
+f1561 int,
+f1562 int,
+f1563 int,
+f1564 int,
+f1565 int,
+f1566 int,
+f1567 int,
+f1568 int,
+f1569 int,
+f1570 int,
+f1571 int,
+f1572 int,
+f1573 int,
+f1574 int,
+f1575 int,
+f1576 int,
+f1577 int,
+f1578 int,
+f1579 int,
+f1580 int,
+f1581 int,
+f1582 int,
+f1583 int,
+f1584 int,
+f1585 int,
+f1586 int,
+f1587 int,
+f1588 int,
+f1589 int,
+f1590 int,
+f1591 int,
+f1592 int,
+f1593 int,
+f1594 int,
+f1595 int,
+f1596 int,
+f1597 int,
+f1598 int,
+f1599 int,
+f1600 int,
+f1601 int,
+f1602 int,
+f1603 int,
+f1604 int,
+f1605 int,
+f1606 int,
+f1607 int,
+f1608 int,
+f1609 int,
+f1610 int,
+f1611 int,
+f1612 int,
+f1613 int,
+f1614 int,
+f1615 int,
+f1616 int,
+f1617 int,
+f1618 int,
+f1619 int,
+f1620 int,
+f1621 int,
+f1622 int,
+f1623 int,
+f1624 int,
+f1625 int,
+f1626 int,
+f1627 int,
+f1628 int,
+f1629 int,
+f1630 int,
+f1631 int,
+f1632 int,
+f1633 int,
+f1634 int,
+f1635 int,
+f1636 int,
+f1637 int,
+f1638 int,
+f1639 int,
+f1640 int,
+f1641 int,
+f1642 int,
+f1643 int,
+f1644 int,
+f1645 int,
+f1646 int,
+f1647 int,
+f1648 int,
+f1649 int,
+f1650 int,
+f1651 int,
+f1652 int,
+f1653 int,
+f1654 int,
+f1655 int,
+f1656 int,
+f1657 int,
+f1658 int,
+f1659 int,
+f1660 int,
+f1661 int,
+f1662 int,
+f1663 int,
+f1664 int,
+f1665 int,
+f1666 int,
+f1667 int,
+f1668 int,
+f1669 int,
+f1670 int,
+f1671 int,
+f1672 int,
+f1673 int,
+f1674 int,
+f1675 int,
+f1676 int,
+f1677 int,
+f1678 int,
+f1679 int,
+f1680 int,
+f1681 int,
+f1682 int,
+f1683 int,
+f1684 int,
+f1685 int,
+f1686 int,
+f1687 int,
+f1688 int,
+f1689 int,
+f1690 int,
+f1691 int,
+f1692 int,
+f1693 int,
+f1694 int,
+f1695 int,
+f1696 int,
+f1697 int,
+f1698 int,
+f1699 int,
+f1700 int,
+f1701 int,
+f1702 int,
+f1703 int,
+f1704 int,
+f1705 int,
+f1706 int,
+f1707 int,
+f1708 int,
+f1709 int,
+f1710 int,
+f1711 int,
+f1712 int,
+f1713 int,
+f1714 int,
+f1715 int,
+f1716 int,
+f1717 int,
+f1718 int,
+f1719 int,
+f1720 int,
+f1721 int,
+f1722 int,
+f1723 int,
+f1724 int,
+f1725 int,
+f1726 int,
+f1727 int,
+f1728 int,
+f1729 int,
+f1730 int,
+f1731 int,
+f1732 int,
+f1733 int,
+f1734 int,
+f1735 int,
+f1736 int,
+f1737 int,
+f1738 int,
+f1739 int,
+f1740 int,
+f1741 int,
+f1742 int,
+f1743 int,
+f1744 int,
+f1745 int,
+f1746 int,
+f1747 int,
+f1748 int,
+f1749 int,
+f1750 int,
+f1751 int,
+f1752 int,
+f1753 int,
+f1754 int,
+f1755 int,
+f1756 int,
+f1757 int,
+f1758 int,
+f1759 int,
+f1760 int,
+f1761 int,
+f1762 int,
+f1763 int,
+f1764 int,
+f1765 int,
+f1766 int,
+f1767 int,
+f1768 int,
+f1769 int,
+f1770 int,
+f1771 int,
+f1772 int,
+f1773 int,
+f1774 int,
+f1775 int,
+f1776 int,
+f1777 int,
+f1778 int,
+f1779 int,
+f1780 int,
+f1781 int,
+f1782 int,
+f1783 int,
+f1784 int,
+f1785 int,
+f1786 int,
+f1787 int,
+f1788 int,
+f1789 int,
+f1790 int,
+f1791 int,
+f1792 int,
+f1793 int,
+f1794 int,
+f1795 int,
+f1796 int,
+f1797 int,
+f1798 int,
+f1799 int,
+f1800 int,
+f1801 int,
+f1802 int,
+f1803 int,
+f1804 int,
+f1805 int,
+f1806 int,
+f1807 int,
+f1808 int,
+f1809 int,
+f1810 int,
+f1811 int,
+f1812 int,
+f1813 int,
+f1814 int,
+f1815 int,
+f1816 int,
+f1817 int,
+f1818 int,
+f1819 int,
+f1820 int,
+f1821 int,
+f1822 int,
+f1823 int,
+f1824 int,
+f1825 int,
+f1826 int,
+f1827 int,
+f1828 int,
+f1829 int,
+f1830 int,
+f1831 int,
+f1832 int,
+f1833 int,
+f1834 int,
+f1835 int,
+f1836 int,
+f1837 int,
+f1838 int,
+f1839 int,
+f1840 int,
+f1841 int,
+f1842 int,
+f1843 int,
+f1844 int,
+f1845 int,
+f1846 int,
+f1847 int,
+f1848 int,
+f1849 int,
+f1850 int,
+f1851 int,
+f1852 int,
+f1853 int,
+f1854 int,
+f1855 int,
+f1856 int,
+f1857 int,
+f1858 int,
+f1859 int,
+f1860 int,
+f1861 int,
+f1862 int,
+f1863 int,
+f1864 int,
+f1865 int,
+f1866 int,
+f1867 int,
+f1868 int,
+f1869 int,
+f1870 int,
+f1871 int,
+f1872 int,
+f1873 int,
+f1874 int,
+f1875 int,
+f1876 int,
+f1877 int,
+f1878 int,
+f1879 int,
+f1880 int,
+f1881 int,
+f1882 int,
+f1883 int,
+f1884 int,
+f1885 int,
+f1886 int,
+f1887 int,
+f1888 int,
+f1889 int,
+f1890 int,
+f1891 int,
+f1892 int,
+f1893 int,
+f1894 int,
+f1895 int,
+f1896 int,
+f1897 int,
+f1898 int,
+f1899 int,
+f1900 int,
+f1901 int,
+f1902 int,
+f1903 int,
+f1904 int,
+f1905 int,
+f1906 int,
+f1907 int,
+f1908 int,
+f1909 int,
+f1910 int,
+f1911 int,
+f1912 int,
+f1913 int,
+f1914 int,
+f1915 int,
+f1916 int,
+f1917 int,
+f1918 int,
+f1919 int,
+f1920 int,
+f1921 int,
+f1922 int,
+f1923 int,
+f1924 int,
+f1925 int,
+f1926 int,
+f1927 int,
+f1928 int,
+f1929 int,
+f1930 int,
+f1931 int,
+f1932 int,
+f1933 int,
+f1934 int,
+f1935 int,
+f1936 int,
+f1937 int,
+f1938 int,
+f1939 int,
+f1940 int,
+f1941 int,
+f1942 int,
+f1943 int,
+f1944 int,
+f1945 int,
+f1946 int,
+f1947 int,
+f1948 int,
+f1949 int,
+f1950 int,
+f1951 int,
+f1952 int,
+f1953 int,
+f1954 int,
+f1955 int,
+f1956 int,
+f1957 int,
+f1958 int,
+f1959 int,
+f1960 int,
+f1961 int,
+f1962 int,
+f1963 int,
+f1964 int,
+f1965 int,
+f1966 int,
+f1967 int,
+f1968 int,
+f1969 int,
+f1970 int,
+f1971 int,
+f1972 int,
+f1973 int,
+f1974 int,
+f1975 int,
+f1976 int,
+f1977 int,
+f1978 int,
+f1979 int,
+f1980 int,
+f1981 int,
+f1982 int,
+f1983 int,
+f1984 int,
+f1985 int,
+f1986 int,
+f1987 int,
+f1988 int,
+f1989 int,
+f1990 int,
+f1991 int,
+f1992 int,
+f1993 int,
+f1994 int,
+f1995 int,
+f1996 int,
+f1997 int,
+f1998 int,
+f1999 int,
+f2000 int,
+f2001 int,
+f2002 int,
+f2003 int,
+f2004 int,
+f2005 int,
+f2006 int,
+f2007 int,
+f2008 int,
+f2009 int,
+f2010 int,
+f2011 int,
+f2012 int,
+f2013 int,
+f2014 int,
+f2015 int,
+f2016 int,
+f2017 int,
+f2018 int,
+f2019 int,
+f2020 int,
+f2021 int,
+f2022 int,
+f2023 int,
+f2024 int,
+f2025 int,
+f2026 int,
+f2027 int,
+f2028 int,
+f2029 int,
+f2030 int,
+f2031 int,
+f2032 int,
+f2033 int,
+f2034 int,
+f2035 int,
+f2036 int,
+f2037 int,
+f2038 int,
+f2039 int,
+f2040 int,
+f2041 int,
+f2042 int,
+f2043 int,
+f2044 int,
+f2045 int,
+f2046 int,
+f2047 int,
+f2048 int,
+f2049 int,
+f2050 int,
+f2051 int,
+f2052 int,
+f2053 int,
+f2054 int,
+f2055 int,
+f2056 int,
+f2057 int,
+f2058 int,
+f2059 int,
+f2060 int,
+f2061 int,
+f2062 int,
+f2063 int,
+f2064 int,
+f2065 int,
+f2066 int,
+f2067 int,
+f2068 int,
+f2069 int,
+f2070 int,
+f2071 int,
+f2072 int,
+f2073 int,
+f2074 int,
+f2075 int,
+f2076 int,
+f2077 int,
+f2078 int,
+f2079 int,
+f2080 int,
+f2081 int,
+f2082 int,
+f2083 int,
+f2084 int,
+f2085 int,
+f2086 int,
+f2087 int,
+f2088 int,
+f2089 int,
+f2090 int,
+f2091 int,
+f2092 int,
+f2093 int,
+f2094 int,
+f2095 int,
+f2096 int,
+f2097 int,
+f2098 int,
+f2099 int,
+f2100 int,
+f2101 int,
+f2102 int,
+f2103 int,
+f2104 int,
+f2105 int,
+f2106 int,
+f2107 int,
+f2108 int,
+f2109 int,
+f2110 int,
+f2111 int,
+f2112 int,
+f2113 int,
+f2114 int,
+f2115 int,
+f2116 int,
+f2117 int,
+f2118 int,
+f2119 int,
+f2120 int,
+f2121 int,
+f2122 int,
+f2123 int,
+f2124 int,
+f2125 int,
+f2126 int,
+f2127 int,
+f2128 int,
+f2129 int,
+f2130 int,
+f2131 int,
+f2132 int,
+f2133 int,
+f2134 int,
+f2135 int,
+f2136 int,
+f2137 int,
+f2138 int,
+f2139 int,
+f2140 int,
+f2141 int,
+f2142 int,
+f2143 int,
+f2144 int,
+f2145 int,
+f2146 int,
+f2147 int,
+f2148 int,
+f2149 int,
+f2150 int,
+f2151 int,
+f2152 int,
+f2153 int,
+f2154 int,
+f2155 int,
+f2156 int,
+f2157 int,
+f2158 int,
+f2159 int,
+f2160 int,
+f2161 int,
+f2162 int,
+f2163 int,
+f2164 int,
+f2165 int,
+f2166 int,
+f2167 int,
+f2168 int,
+f2169 int,
+f2170 int,
+f2171 int,
+f2172 int,
+f2173 int,
+f2174 int,
+f2175 int,
+f2176 int,
+f2177 int,
+f2178 int,
+f2179 int,
+f2180 int,
+f2181 int,
+f2182 int,
+f2183 int,
+f2184 int,
+f2185 int,
+f2186 int,
+f2187 int,
+f2188 int,
+f2189 int,
+f2190 int,
+f2191 int,
+f2192 int,
+f2193 int,
+f2194 int,
+f2195 int,
+f2196 int,
+f2197 int,
+f2198 int,
+f2199 int,
+f2200 int,
+f2201 int,
+f2202 int,
+f2203 int,
+f2204 int,
+f2205 int,
+f2206 int,
+f2207 int,
+f2208 int,
+f2209 int,
+f2210 int,
+f2211 int,
+f2212 int,
+f2213 int,
+f2214 int,
+f2215 int,
+f2216 int,
+f2217 int,
+f2218 int,
+f2219 int,
+f2220 int,
+f2221 int,
+f2222 int,
+f2223 int,
+f2224 int,
+f2225 int,
+f2226 int,
+f2227 int,
+f2228 int,
+f2229 int,
+f2230 int,
+f2231 int,
+f2232 int,
+f2233 int,
+f2234 int,
+f2235 int,
+f2236 int,
+f2237 int,
+f2238 int,
+f2239 int,
+f2240 int,
+f2241 int,
+f2242 int,
+f2243 int,
+f2244 int,
+f2245 int,
+f2246 int,
+f2247 int,
+f2248 int,
+f2249 int,
+f2250 int,
+f2251 int,
+f2252 int,
+f2253 int,
+f2254 int,
+f2255 int,
+f2256 int,
+f2257 int,
+f2258 int,
+f2259 int,
+f2260 int,
+f2261 int,
+f2262 int,
+f2263 int,
+f2264 int,
+f2265 int,
+f2266 int,
+f2267 int,
+f2268 int,
+f2269 int,
+f2270 int,
+f2271 int,
+f2272 int,
+f2273 int,
+f2274 int,
+f2275 int,
+f2276 int,
+f2277 int,
+f2278 int,
+f2279 int,
+f2280 int,
+f2281 int,
+f2282 int,
+f2283 int,
+f2284 int,
+f2285 int,
+f2286 int,
+f2287 int,
+f2288 int,
+f2289 int,
+f2290 int,
+f2291 int,
+f2292 int,
+f2293 int,
+f2294 int,
+f2295 int,
+f2296 int,
+f2297 int,
+f2298 int,
+f2299 int,
+f2300 int,
+f2301 int,
+f2302 int,
+f2303 int,
+f2304 int,
+f2305 int,
+f2306 int,
+f2307 int,
+f2308 int,
+f2309 int,
+f2310 int,
+f2311 int,
+f2312 int,
+f2313 int,
+f2314 int,
+f2315 int,
+f2316 int,
+f2317 int,
+f2318 int,
+f2319 int,
+f2320 int,
+f2321 int,
+f2322 int,
+f2323 int,
+f2324 int,
+f2325 int,
+f2326 int,
+f2327 int,
+f2328 int,
+f2329 int,
+f2330 int,
+f2331 int,
+f2332 int,
+f2333 int,
+f2334 int,
+f2335 int,
+f2336 int,
+f2337 int,
+f2338 int,
+f2339 int,
+f2340 int,
+f2341 int,
+f2342 int,
+f2343 int,
+f2344 int,
+f2345 int,
+f2346 int,
+f2347 int,
+f2348 int,
+f2349 int,
+f2350 int,
+f2351 int,
+f2352 int,
+f2353 int,
+f2354 int,
+f2355 int,
+f2356 int,
+f2357 int,
+f2358 int,
+f2359 int,
+f2360 int,
+f2361 int,
+f2362 int,
+f2363 int,
+f2364 int,
+f2365 int,
+f2366 int,
+f2367 int,
+f2368 int,
+f2369 int,
+f2370 int,
+f2371 int,
+f2372 int,
+f2373 int,
+f2374 int,
+f2375 int,
+f2376 int,
+f2377 int,
+f2378 int,
+f2379 int,
+f2380 int,
+f2381 int,
+f2382 int,
+f2383 int,
+f2384 int,
+f2385 int,
+f2386 int,
+f2387 int,
+f2388 int,
+f2389 int,
+f2390 int,
+f2391 int,
+f2392 int,
+f2393 int,
+f2394 int,
+f2395 int,
+f2396 int,
+f2397 int,
+f2398 int,
+f2399 int,
+f2400 int,
+f2401 int,
+f2402 int,
+f2403 int,
+f2404 int,
+f2405 int,
+f2406 int,
+f2407 int,
+f2408 int,
+f2409 int,
+f2410 int,
+f2411 int,
+f2412 int,
+f2413 int,
+f2414 int,
+f2415 int,
+f2416 int,
+f2417 int,
+f2418 int,
+f2419 int,
+f2420 int,
+f2421 int,
+f2422 int,
+f2423 int,
+f2424 int,
+f2425 int,
+f2426 int,
+f2427 int,
+f2428 int,
+f2429 int,
+f2430 int,
+f2431 int,
+f2432 int,
+f2433 int,
+f2434 int,
+f2435 int,
+f2436 int,
+f2437 int,
+f2438 int,
+f2439 int,
+f2440 int,
+f2441 int,
+f2442 int,
+f2443 int,
+f2444 int,
+f2445 int,
+f2446 int,
+f2447 int,
+f2448 int,
+f2449 int,
+f2450 int,
+f2451 int,
+f2452 int,
+f2453 int,
+f2454 int,
+f2455 int,
+f2456 int,
+f2457 int,
+f2458 int,
+f2459 int,
+f2460 int,
+f2461 int,
+f2462 int,
+f2463 int,
+f2464 int,
+f2465 int,
+f2466 int,
+f2467 int,
+f2468 int,
+f2469 int,
+f2470 int,
+f2471 int,
+f2472 int,
+f2473 int,
+f2474 int,
+f2475 int,
+f2476 int,
+f2477 int,
+f2478 int,
+f2479 int,
+f2480 int,
+f2481 int,
+f2482 int,
+f2483 int,
+f2484 int,
+f2485 int,
+f2486 int,
+f2487 int,
+f2488 int,
+f2489 int,
+f2490 int,
+f2491 int,
+f2492 int,
+f2493 int,
+f2494 int,
+f2495 int,
+f2496 int,
+f2497 int,
+f2498 int,
+f2499 int,
+f2500 int,
+f2501 int,
+f2502 int,
+f2503 int,
+f2504 int,
+f2505 int,
+f2506 int,
+f2507 int,
+f2508 int,
+f2509 int,
+f2510 int,
+f2511 int,
+f2512 int,
+f2513 int,
+f2514 int,
+f2515 int,
+f2516 int,
+f2517 int,
+f2518 int,
+f2519 int,
+f2520 int,
+f2521 int,
+f2522 int,
+f2523 int,
+f2524 int,
+f2525 int,
+f2526 int,
+f2527 int,
+f2528 int,
+f2529 int,
+f2530 int,
+f2531 int,
+f2532 int,
+f2533 int,
+f2534 int,
+f2535 int,
+f2536 int,
+f2537 int,
+f2538 int,
+f2539 int,
+f2540 int,
+f2541 int,
+f2542 int,
+f2543 int,
+f2544 int,
+f2545 int,
+f2546 int,
+f2547 int,
+f2548 int,
+f2549 int,
+f2550 int,
+f2551 int,
+f2552 int,
+f2553 int,
+f2554 int,
+f2555 int,
+f2556 int,
+f2557 int,
+f2558 int,
+f2559 int,
+f2560 int,
+f2561 int,
+f2562 int,
+f2563 int,
+f2564 int,
+f2565 int,
+f2566 int,
+f2567 int,
+f2568 int,
+f2569 int,
+f2570 int,
+f2571 int,
+f2572 int,
+f2573 int,
+f2574 int,
+f2575 int,
+f2576 int,
+f2577 int,
+f2578 int,
+f2579 int,
+f2580 int,
+f2581 int,
+f2582 int,
+f2583 int,
+f2584 int,
+f2585 int,
+f2586 int,
+f2587 int,
+f2588 int,
+f2589 int,
+f2590 int,
+f2591 int,
+f2592 int,
+f2593 int,
+f2594 int,
+f2595 int,
+f2596 int,
+f2597 int,
+f2598 int,
+f2599 int,
+f2600 int,
+f2601 int,
+f2602 int,
+f2603 int,
+f2604 int,
+f2605 int,
+f2606 int,
+f2607 int,
+f2608 int,
+f2609 int,
+f2610 int,
+f2611 int,
+f2612 int,
+f2613 int,
+f2614 int,
+f2615 int,
+f2616 int,
+f2617 int,
+f2618 int,
+f2619 int,
+f2620 int,
+f2621 int,
+f2622 int,
+f2623 int,
+f2624 int,
+f2625 int,
+f2626 int,
+f2627 int,
+f2628 int,
+f2629 int,
+f2630 int,
+f2631 int,
+f2632 int,
+f2633 int,
+f2634 int,
+f2635 int,
+f2636 int,
+f2637 int,
+f2638 int,
+f2639 int,
+f2640 int,
+f2641 int,
+f2642 int,
+f2643 int,
+f2644 int,
+f2645 int,
+f2646 int,
+f2647 int,
+f2648 int,
+f2649 int,
+f2650 int,
+f2651 int,
+f2652 int,
+f2653 int,
+f2654 int,
+f2655 int,
+f2656 int,
+f2657 int,
+f2658 int,
+f2659 int,
+f2660 int,
+f2661 int,
+f2662 int,
+f2663 int,
+f2664 int,
+f2665 int,
+f2666 int,
+f2667 int,
+f2668 int,
+f2669 int,
+f2670 int,
+f2671 int,
+f2672 int,
+f2673 int,
+f2674 int,
+f2675 int,
+f2676 int,
+f2677 int,
+f2678 int,
+f2679 int,
+f2680 int,
+f2681 int,
+f2682 int,
+f2683 int,
+f2684 int,
+f2685 int,
+f2686 int,
+f2687 int,
+f2688 int,
+f2689 int,
+f2690 int,
+f2691 int,
+f2692 int,
+f2693 int,
+f2694 int,
+f2695 int,
+f2696 int,
+f2697 int,
+f2698 int,
+f2699 int,
+f2700 int,
+f2701 int,
+f2702 int,
+f2703 int,
+f2704 int,
+f2705 int,
+f2706 int,
+f2707 int,
+f2708 int,
+f2709 int,
+f2710 int,
+f2711 int,
+f2712 int,
+f2713 int,
+f2714 int,
+f2715 int,
+f2716 int,
+f2717 int,
+f2718 int,
+f2719 int,
+f2720 int,
+f2721 int,
+f2722 int,
+f2723 int,
+f2724 int,
+f2725 int,
+f2726 int,
+f2727 int,
+f2728 int,
+f2729 int,
+f2730 int,
+f2731 int,
+f2732 int,
+f2733 int,
+f2734 int,
+f2735 int,
+f2736 int,
+f2737 int,
+f2738 int,
+f2739 int,
+f2740 int,
+f2741 int,
+f2742 int,
+f2743 int,
+f2744 int,
+f2745 int,
+f2746 int,
+f2747 int,
+f2748 int,
+f2749 int,
+f2750 int,
+f2751 int,
+f2752 int,
+f2753 int,
+f2754 int,
+f2755 int,
+f2756 int,
+f2757 int,
+f2758 int,
+f2759 int,
+f2760 int,
+f2761 int,
+f2762 int,
+f2763 int,
+f2764 int,
+f2765 int,
+f2766 int,
+f2767 int,
+f2768 int,
+f2769 int,
+f2770 int,
+f2771 int,
+f2772 int,
+f2773 int,
+f2774 int,
+f2775 int,
+f2776 int,
+f2777 int,
+f2778 int,
+f2779 int,
+f2780 int,
+f2781 int,
+f2782 int,
+f2783 int,
+f2784 int,
+f2785 int,
+f2786 int,
+f2787 int,
+f2788 int,
+f2789 int,
+f2790 int,
+f2791 int,
+f2792 int,
+f2793 int,
+f2794 int,
+f2795 int,
+f2796 int,
+f2797 int,
+f2798 int,
+f2799 int,
+f2800 int,
+f2801 int,
+f2802 int,
+f2803 int,
+f2804 int,
+f2805 int,
+f2806 int,
+f2807 int,
+f2808 int,
+f2809 int,
+f2810 int,
+f2811 int,
+f2812 int,
+f2813 int,
+f2814 int,
+f2815 int,
+f2816 int,
+f2817 int,
+f2818 int,
+f2819 int,
+f2820 int,
+f2821 int,
+f2822 int,
+f2823 int,
+f2824 int,
+f2825 int,
+f2826 int,
+f2827 int,
+f2828 int,
+f2829 int,
+f2830 int,
+f2831 int,
+f2832 int,
+f2833 int,
+f2834 int,
+f2835 int,
+f2836 int,
+f2837 int,
+f2838 int,
+f2839 int,
+f2840 int,
+f2841 int,
+f2842 int,
+f2843 int,
+f2844 int,
+f2845 int,
+f2846 int,
+f2847 int,
+f2848 int,
+f2849 int,
+f2850 int,
+f2851 int,
+f2852 int,
+f2853 int,
+f2854 int,
+f2855 int,
+f2856 int,
+f2857 int,
+f2858 int,
+f2859 int,
+f2860 int,
+f2861 int,
+f2862 int,
+f2863 int,
+f2864 int,
+f2865 int,
+f2866 int,
+f2867 int,
+f2868 int,
+f2869 int,
+f2870 int,
+f2871 int,
+f2872 int,
+f2873 int,
+f2874 int,
+f2875 int,
+f2876 int,
+f2877 int,
+f2878 int,
+f2879 int,
+f2880 int,
+f2881 int,
+f2882 int,
+f2883 int,
+f2884 int,
+f2885 int,
+f2886 int,
+f2887 int,
+f2888 int,
+f2889 int,
+f2890 int,
+f2891 int,
+f2892 int,
+f2893 int,
+f2894 int,
+f2895 int,
+f2896 int,
+f2897 int,
+f2898 int,
+f2899 int,
+f2900 int,
+f2901 int,
+f2902 int,
+f2903 int,
+f2904 int,
+f2905 int,
+f2906 int,
+f2907 int,
+f2908 int,
+f2909 int,
+f2910 int,
+f2911 int,
+f2912 int,
+f2913 int,
+f2914 int,
+f2915 int,
+f2916 int,
+f2917 int,
+f2918 int,
+f2919 int,
+f2920 int,
+f2921 int,
+f2922 int,
+f2923 int,
+f2924 int,
+f2925 int,
+f2926 int,
+f2927 int,
+f2928 int,
+f2929 int,
+f2930 int,
+f2931 int,
+f2932 int,
+f2933 int,
+f2934 int,
+f2935 int,
+f2936 int,
+f2937 int,
+f2938 int,
+f2939 int,
+f2940 int,
+f2941 int,
+f2942 int,
+f2943 int,
+f2944 int,
+f2945 int,
+f2946 int,
+f2947 int,
+f2948 int,
+f2949 int,
+f2950 int,
+f2951 int,
+f2952 int,
+f2953 int,
+f2954 int,
+f2955 int,
+f2956 int,
+f2957 int,
+f2958 int,
+f2959 int,
+f2960 int,
+f2961 int,
+f2962 int,
+f2963 int,
+f2964 int,
+f2965 int,
+f2966 int,
+f2967 int,
+f2968 int,
+f2969 int,
+f2970 int,
+f2971 int,
+f2972 int,
+f2973 int,
+f2974 int,
+f2975 int,
+f2976 int,
+f2977 int,
+f2978 int,
+f2979 int,
+f2980 int,
+f2981 int,
+f2982 int,
+f2983 int,
+f2984 int,
+f2985 int,
+f2986 int,
+f2987 int,
+f2988 int,
+f2989 int,
+f2990 int,
+f2991 int,
+f2992 int,
+f2993 int,
+f2994 int,
+f2995 int,
+f2996 int,
+f2997 int,
+f2998 int,
+f2999 int,
+f3000 int,
+f3001 int,
+f3002 int,
+f3003 int,
+f3004 int,
+f3005 int,
+f3006 int,
+f3007 int,
+f3008 int,
+f3009 int,
+f3010 int,
+f3011 int,
+f3012 int,
+f3013 int,
+f3014 int,
+f3015 int,
+f3016 int,
+f3017 int,
+f3018 int,
+f3019 int,
+f3020 int,
+f3021 int,
+f3022 int,
+f3023 int,
+f3024 int,
+f3025 int,
+f3026 int,
+f3027 int,
+f3028 int,
+f3029 int,
+f3030 int,
+f3031 int,
+f3032 int,
+f3033 int,
+f3034 int,
+f3035 int,
+f3036 int,
+f3037 int,
+f3038 int,
+f3039 int,
+f3040 int,
+f3041 int,
+f3042 int,
+f3043 int,
+f3044 int,
+f3045 int,
+f3046 int,
+f3047 int,
+f3048 int,
+f3049 int,
+f3050 int,
+f3051 int,
+f3052 int,
+f3053 int,
+f3054 int,
+f3055 int,
+f3056 int,
+f3057 int,
+f3058 int,
+f3059 int,
+f3060 int,
+f3061 int,
+f3062 int,
+f3063 int,
+f3064 int,
+f3065 int,
+f3066 int,
+f3067 int,
+f3068 int,
+f3069 int,
+f3070 int,
+f3071 int,
+f3072 int,
+f3073 int,
+f3074 int,
+f3075 int,
+f3076 int,
+f3077 int,
+f3078 int,
+f3079 int,
+f3080 int,
+f3081 int,
+f3082 int,
+f3083 int,
+f3084 int,
+f3085 int,
+f3086 int,
+f3087 int,
+f3088 int,
+f3089 int,
+f3090 int,
+f3091 int,
+f3092 int,
+f3093 int,
+f3094 int,
+f3095 int,
+f3096 int,
+f3097 int,
+f3098 int,
+f3099 int,
+f3100 int,
+f3101 int,
+f3102 int,
+f3103 int,
+f3104 int,
+f3105 int,
+f3106 int,
+f3107 int,
+f3108 int,
+f3109 int,
+f3110 int,
+f3111 int,
+f3112 int,
+f3113 int,
+f3114 int,
+f3115 int,
+f3116 int,
+f3117 int,
+f3118 int,
+f3119 int,
+f3120 int,
+f3121 int,
+f3122 int,
+f3123 int,
+f3124 int,
+f3125 int,
+f3126 int,
+f3127 int,
+f3128 int,
+f3129 int,
+f3130 int,
+f3131 int,
+f3132 int,
+f3133 int,
+f3134 int,
+f3135 int,
+f3136 int,
+f3137 int,
+f3138 int,
+f3139 int,
+f3140 int,
+f3141 int,
+f3142 int,
+f3143 int,
+f3144 int,
+f3145 int,
+f3146 int,
+f3147 int,
+f3148 int,
+f3149 int,
+f3150 int,
+f3151 int,
+f3152 int,
+f3153 int,
+f3154 int,
+f3155 int,
+f3156 int,
+f3157 int,
+f3158 int,
+f3159 int,
+f3160 int,
+f3161 int,
+f3162 int,
+f3163 int,
+f3164 int,
+f3165 int,
+f3166 int,
+f3167 int,
+f3168 int,
+f3169 int,
+f3170 int,
+f3171 int,
+f3172 int,
+f3173 int,
+f3174 int,
+f3175 int,
+f3176 int,
+f3177 int,
+f3178 int,
+f3179 int,
+f3180 int,
+f3181 int,
+f3182 int,
+f3183 int,
+f3184 int,
+f3185 int,
+f3186 int,
+f3187 int,
+f3188 int,
+f3189 int,
+f3190 int,
+f3191 int,
+f3192 int,
+f3193 int,
+f3194 int,
+f3195 int,
+f3196 int,
+f3197 int,
+f3198 int,
+f3199 int,
+f3200 int,
+f3201 int,
+f3202 int,
+f3203 int,
+f3204 int,
+f3205 int,
+f3206 int,
+f3207 int,
+f3208 int,
+f3209 int,
+f3210 int,
+f3211 int,
+f3212 int,
+f3213 int,
+f3214 int,
+f3215 int,
+f3216 int,
+f3217 int,
+f3218 int,
+f3219 int,
+f3220 int,
+f3221 int,
+f3222 int,
+f3223 int,
+f3224 int,
+f3225 int,
+f3226 int,
+f3227 int,
+f3228 int,
+f3229 int,
+f3230 int,
+f3231 int,
+f3232 int,
+f3233 int,
+f3234 int,
+f3235 int,
+f3236 int,
+f3237 int,
+f3238 int,
+f3239 int,
+f3240 int,
+f3241 int,
+f3242 int,
+f3243 int,
+f3244 int,
+f3245 int,
+f3246 int,
+f3247 int,
+f3248 int,
+f3249 int,
+f3250 int,
+f3251 int,
+f3252 int,
+f3253 int,
+f3254 int,
+f3255 int,
+f3256 int,
+f3257 int,
+f3258 int,
+f3259 int,
+f3260 int,
+f3261 int,
+f3262 int,
+f3263 int,
+f3264 int,
+f3265 int,
+f3266 int,
+f3267 int,
+f3268 int,
+f3269 int,
+f3270 int,
+f3271 int,
+f3272 int,
+f3273 int,
+f3274 int,
+f3275 int,
+f3276 int,
+f3277 int,
+f3278 int,
+f3279 int,
+f3280 int,
+f3281 int,
+f3282 int,
+f3283 int,
+f3284 int,
+f3285 int,
+f3286 int,
+f3287 int,
+f3288 int,
+f3289 int,
+f3290 int,
+f3291 int,
+f3292 int,
+f3293 int,
+f3294 int,
+f3295 int,
+f3296 int,
+f3297 int,
+f3298 int,
+f3299 int,
+f3300 int,
+f3301 int,
+f3302 int,
+f3303 int,
+f3304 int,
+f3305 int,
+f3306 int,
+f3307 int,
+f3308 int,
+f3309 int,
+f3310 int,
+f3311 int,
+f3312 int,
+f3313 int,
+f3314 int,
+f3315 int,
+f3316 int,
+f3317 int,
+f3318 int,
+f3319 int,
+f3320 int,
+f3321 int,
+f3322 int,
+f3323 int,
+f3324 int,
+f3325 int,
+f3326 int,
+f3327 int,
+f3328 int,
+f3329 int,
+f3330 int,
+f3331 int,
+f3332 int,
+f3333 int,
+f3334 int,
+f3335 int,
+f3336 int,
+f3337 int,
+f3338 int,
+f3339 int,
+f3340 int,
+f3341 int,
+f3342 int,
+f3343 int,
+f3344 int,
+f3345 int,
+f3346 int,
+f3347 int,
+f3348 int,
+f3349 int,
+f3350 int,
+f3351 int,
+f3352 int,
+f3353 int,
+f3354 int,
+f3355 int,
+f3356 int,
+f3357 int,
+f3358 int,
+f3359 int,
+f3360 int,
+f3361 int,
+f3362 int,
+f3363 int,
+f3364 int,
+f3365 int,
+f3366 int,
+f3367 int,
+f3368 int,
+f3369 int,
+f3370 int,
+f3371 int,
+f3372 int,
+f3373 int,
+f3374 int,
+f3375 int,
+f3376 int,
+f3377 int,
+f3378 int,
+f3379 int,
+f3380 int,
+f3381 int,
+f3382 int,
+f3383 int,
+f3384 int,
+f3385 int,
+f3386 int,
+f3387 int,
+f3388 int,
+f3389 int,
+f3390 int,
+f3391 int,
+f3392 int,
+f3393 int,
+f3394 int,
+f3395 int,
+f3396 int,
+f3397 int,
+f3398 int,
+f3399 int,
+f3400 int,
+f3401 int,
+f3402 int,
+f3403 int,
+f3404 int,
+f3405 int,
+f3406 int,
+f3407 int,
+f3408 int,
+f3409 int,
+f3410 int,
+f3411 int,
+f3412 int,
+f3413 int,
+f3414 int,
+f3415 int,
+f3416 int,
+f3417 int,
+f3418 int,
+f3419 int,
+f3420 int,
+f3421 int,
+f3422 int,
+f3423 int,
+f3424 int,
+f3425 int,
+f3426 int,
+f3427 int,
+f3428 int,
+f3429 int,
+f3430 int,
+f3431 int,
+f3432 int,
+f3433 int,
+f3434 int,
+f3435 int,
+f3436 int,
+f3437 int,
+f3438 int,
+f3439 int,
+f3440 int,
+f3441 int,
+f3442 int,
+f3443 int,
+f3444 int,
+f3445 int,
+f3446 int,
+f3447 int,
+f3448 int,
+f3449 int,
+f3450 int,
+f3451 int,
+f3452 int,
+f3453 int,
+f3454 int,
+f3455 int,
+f3456 int,
+f3457 int,
+f3458 int,
+f3459 int,
+f3460 int,
+f3461 int,
+f3462 int,
+f3463 int,
+f3464 int,
+f3465 int,
+f3466 int,
+f3467 int,
+f3468 int,
+f3469 int,
+f3470 int,
+f3471 int,
+f3472 int,
+f3473 int,
+f3474 int,
+f3475 int,
+f3476 int,
+f3477 int,
+f3478 int,
+f3479 int,
+f3480 int,
+f3481 int,
+f3482 int,
+f3483 int,
+f3484 int,
+f3485 int,
+f3486 int,
+f3487 int,
+f3488 int,
+f3489 int,
+f3490 int,
+f3491 int,
+f3492 int,
+f3493 int,
+f3494 int,
+f3495 int,
+f3496 int,
+f3497 int,
+f3498 int,
+f3499 int,
+f3500 int,
+f3501 int,
+f3502 int,
+f3503 int,
+f3504 int,
+f3505 int,
+f3506 int,
+f3507 int,
+f3508 int,
+f3509 int,
+f3510 int,
+f3511 int,
+f3512 int,
+f3513 int,
+f3514 int,
+f3515 int,
+f3516 int,
+f3517 int,
+f3518 int,
+f3519 int,
+f3520 int,
+f3521 int,
+f3522 int,
+f3523 int,
+f3524 int,
+f3525 int,
+f3526 int,
+f3527 int,
+f3528 int,
+f3529 int,
+f3530 int,
+f3531 int,
+f3532 int,
+f3533 int,
+f3534 int,
+f3535 int,
+f3536 int,
+f3537 int,
+f3538 int,
+f3539 int,
+f3540 int,
+f3541 int,
+f3542 int,
+f3543 int,
+f3544 int,
+f3545 int,
+f3546 int,
+f3547 int,
+f3548 int,
+f3549 int,
+f3550 int,
+f3551 int,
+f3552 int,
+f3553 int,
+f3554 int,
+f3555 int,
+f3556 int,
+f3557 int,
+f3558 int,
+f3559 int,
+f3560 int,
+f3561 int,
+f3562 int,
+f3563 int,
+f3564 int,
+f3565 int,
+f3566 int,
+f3567 int,
+f3568 int,
+f3569 int,
+f3570 int,
+f3571 int,
+f3572 int,
+f3573 int,
+f3574 int,
+f3575 int,
+f3576 int,
+f3577 int,
+f3578 int,
+f3579 int,
+f3580 int,
+f3581 int,
+f3582 int,
+f3583 int,
+f3584 int,
+f3585 int,
+f3586 int,
+f3587 int,
+f3588 int,
+f3589 int,
+f3590 int,
+f3591 int,
+f3592 int,
+f3593 int,
+f3594 int,
+f3595 int,
+f3596 int,
+f3597 int,
+f3598 int,
+f3599 int,
+f3600 int,
+f3601 int,
+f3602 int,
+f3603 int,
+f3604 int,
+f3605 int,
+f3606 int,
+f3607 int,
+f3608 int,
+f3609 int,
+f3610 int,
+f3611 int,
+f3612 int,
+f3613 int,
+f3614 int,
+f3615 int,
+f3616 int,
+f3617 int,
+f3618 int,
+f3619 int,
+f3620 int,
+f3621 int,
+f3622 int,
+f3623 int,
+f3624 int,
+f3625 int,
+f3626 int,
+f3627 int,
+f3628 int,
+f3629 int,
+f3630 int,
+f3631 int,
+f3632 int,
+f3633 int,
+f3634 int,
+f3635 int,
+f3636 int,
+f3637 int,
+f3638 int,
+f3639 int,
+f3640 int,
+f3641 int,
+f3642 int,
+f3643 int,
+f3644 int,
+f3645 int,
+f3646 int,
+f3647 int,
+f3648 int,
+f3649 int,
+f3650 int,
+f3651 int,
+f3652 int,
+f3653 int,
+f3654 int,
+f3655 int,
+f3656 int,
+f3657 int,
+f3658 int,
+f3659 int,
+f3660 int,
+f3661 int,
+f3662 int,
+f3663 int,
+f3664 int,
+f3665 int,
+f3666 int,
+f3667 int,
+f3668 int,
+f3669 int,
+f3670 int,
+f3671 int,
+f3672 int,
+f3673 int,
+f3674 int,
+f3675 int,
+f3676 int,
+f3677 int,
+f3678 int,
+f3679 int,
+f3680 int,
+f3681 int,
+f3682 int,
+f3683 int,
+f3684 int,
+f3685 int,
+f3686 int,
+f3687 int,
+f3688 int,
+f3689 int,
+f3690 int,
+f3691 int,
+f3692 int,
+f3693 int,
+f3694 int,
+f3695 int,
+f3696 int,
+f3697 int,
+f3698 int,
+f3699 int,
+f3700 int,
+f3701 int,
+f3702 int,
+f3703 int,
+f3704 int,
+f3705 int,
+f3706 int,
+f3707 int,
+f3708 int,
+f3709 int,
+f3710 int,
+f3711 int,
+f3712 int,
+f3713 int,
+f3714 int,
+f3715 int,
+f3716 int,
+f3717 int,
+f3718 int,
+f3719 int,
+f3720 int,
+f3721 int,
+f3722 int,
+f3723 int,
+f3724 int,
+f3725 int,
+f3726 int,
+f3727 int,
+f3728 int,
+f3729 int,
+f3730 int,
+f3731 int,
+f3732 int,
+f3733 int,
+f3734 int,
+f3735 int,
+f3736 int,
+f3737 int,
+f3738 int,
+f3739 int,
+f3740 int,
+f3741 int,
+f3742 int,
+f3743 int,
+f3744 int,
+f3745 int,
+f3746 int,
+f3747 int,
+f3748 int,
+f3749 int,
+f3750 int,
+f3751 int,
+f3752 int,
+f3753 int,
+f3754 int,
+f3755 int,
+f3756 int,
+f3757 int,
+f3758 int,
+f3759 int,
+f3760 int,
+f3761 int,
+f3762 int,
+f3763 int,
+f3764 int,
+f3765 int,
+f3766 int,
+f3767 int,
+f3768 int,
+f3769 int,
+f3770 int,
+f3771 int,
+f3772 int,
+f3773 int,
+f3774 int,
+f3775 int,
+f3776 int,
+f3777 int,
+f3778 int,
+f3779 int,
+f3780 int,
+f3781 int,
+f3782 int,
+f3783 int,
+f3784 int,
+f3785 int,
+f3786 int,
+f3787 int,
+f3788 int,
+f3789 int,
+f3790 int,
+f3791 int,
+f3792 int,
+f3793 int,
+f3794 int,
+f3795 int,
+f3796 int,
+f3797 int,
+f3798 int,
+f3799 int,
+f3800 int,
+f3801 int,
+f3802 int,
+f3803 int,
+f3804 int,
+f3805 int,
+f3806 int,
+f3807 int,
+f3808 int,
+f3809 int,
+f3810 int,
+f3811 int,
+f3812 int,
+f3813 int,
+f3814 int,
+f3815 int,
+f3816 int,
+f3817 int,
+f3818 int,
+f3819 int,
+f3820 int,
+f3821 int,
+f3822 int,
+f3823 int,
+f3824 int,
+f3825 int,
+f3826 int,
+f3827 int,
+f3828 int,
+f3829 int,
+f3830 int,
+f3831 int,
+f3832 int,
+f3833 int,
+f3834 int,
+f3835 int,
+f3836 int,
+f3837 int,
+f3838 int,
+f3839 int,
+f3840 int,
+f3841 int,
+f3842 int,
+f3843 int,
+f3844 int,
+f3845 int,
+f3846 int,
+f3847 int,
+f3848 int,
+f3849 int,
+f3850 int,
+f3851 int,
+f3852 int,
+f3853 int,
+f3854 int,
+f3855 int,
+f3856 int,
+f3857 int,
+f3858 int,
+f3859 int,
+f3860 int,
+f3861 int,
+f3862 int,
+f3863 int,
+f3864 int,
+f3865 int,
+f3866 int,
+f3867 int,
+f3868 int,
+f3869 int,
+f3870 int,
+f3871 int,
+f3872 int,
+f3873 int,
+f3874 int,
+f3875 int,
+f3876 int,
+f3877 int,
+f3878 int,
+f3879 int,
+f3880 int,
+f3881 int,
+f3882 int,
+f3883 int,
+f3884 int,
+f3885 int,
+f3886 int,
+f3887 int,
+f3888 int,
+f3889 int,
+f3890 int,
+f3891 int,
+f3892 int,
+f3893 int,
+f3894 int,
+f3895 int,
+f3896 int,
+f3897 int,
+f3898 int,
+f3899 int,
+f3900 int,
+f3901 int,
+f3902 int,
+f3903 int,
+f3904 int,
+f3905 int,
+f3906 int,
+f3907 int,
+f3908 int,
+f3909 int,
+f3910 int,
+f3911 int,
+f3912 int,
+f3913 int,
+f3914 int,
+f3915 int,
+f3916 int,
+f3917 int,
+f3918 int,
+f3919 int,
+f3920 int,
+f3921 int,
+f3922 int,
+f3923 int,
+f3924 int,
+f3925 int,
+f3926 int,
+f3927 int,
+f3928 int,
+f3929 int,
+f3930 int,
+f3931 int,
+f3932 int,
+f3933 int,
+f3934 int,
+f3935 int,
+f3936 int,
+f3937 int,
+f3938 int,
+f3939 int,
+f3940 int,
+f3941 int,
+f3942 int,
+f3943 int,
+f3944 int,
+f3945 int,
+f3946 int,
+f3947 int,
+f3948 int,
+f3949 int,
+f3950 int,
+f3951 int,
+f3952 int,
+f3953 int,
+f3954 int,
+f3955 int,
+f3956 int,
+f3957 int,
+f3958 int,
+f3959 int,
+f3960 int,
+f3961 int,
+f3962 int,
+f3963 int,
+f3964 int,
+f3965 int,
+f3966 int,
+f3967 int,
+f3968 int,
+f3969 int,
+f3970 int,
+f3971 int,
+f3972 int,
+f3973 int,
+f3974 int,
+f3975 int,
+f3976 int,
+f3977 int,
+f3978 int,
+f3979 int,
+f3980 int,
+f3981 int,
+f3982 int,
+f3983 int,
+f3984 int,
+f3985 int,
+f3986 int,
+f3987 int,
+f3988 int,
+f3989 int,
+f3990 int,
+f3991 int,
+f3992 int,
+f3993 int,
+f3994 int,
+f3995 int,
+f3996 int,
+f3997 int,
+f3998 int,
+f3999 int,
+f4000 int,
+f4001 int,
+f4002 int,
+f4003 int,
+f4004 int,
+f4005 int,
+f4006 int,
+f4007 int,
+f4008 int,
+f4009 int,
+f4010 int,
+f4011 int,
+f4012 int,
+f4013 int,
+f4014 int,
+f4015 int,
+f4016 int,
+f4017 int,
+f4018 int,
+f4019 int,
+f4020 int,
+f4021 int,
+f4022 int,
+f4023 int,
+f4024 int,
+f4025 int,
+f4026 int,
+f4027 int,
+f4028 int,
+f4029 int,
+f4030 int,
+f4031 int,
+f4032 int,
+f4033 int,
+f4034 int,
+f4035 int,
+f4036 int,
+f4037 int,
+f4038 int,
+f4039 int,
+f4040 int,
+f4041 int,
+f4042 int,
+f4043 int,
+f4044 int,
+f4045 int,
+f4046 int,
+f4047 int,
+f4048 int,
+f4049 int,
+f4050 int,
+f4051 int,
+f4052 int,
+f4053 int,
+f4054 int,
+f4055 int,
+f4056 int,
+f4057 int,
+f4058 int,
+f4059 int,
+f4060 int,
+f4061 int,
+f4062 int,
+f4063 int,
+f4064 int,
+f4065 int,
+f4066 int,
+f4067 int,
+f4068 int,
+f4069 int,
+f4070 int,
+f4071 int,
+f4072 int,
+f4073 int,
+f4074 int,
+f4075 int,
+f4076 int,
+f4077 int,
+f4078 int,
+f4079 int,
+f4080 int,
+f4081 int,
+f4082 int,
+f4083 int,
+f4084 int,
+f4085 int,
+f4086 int,
+f4087 int,
+f4088 int,
+f4089 int,
+f4090 int,
+f4091 int,
+f4092 int,
+f4093 int,
+f4094 int,
+f4095 int,
+f4096 int
+) ENGINE=BLACKHOLE;
+SHOW CREATE TABLE t1;
+Table	Create Table
+t1	CREATE TABLE `t1` (
+  `f1` int DEFAULT NULL,
+  `f2` int DEFAULT NULL,
+  `f3` int DEFAULT NULL,
+  `f4` int DEFAULT NULL,
+  `f5` int DEFAULT NULL,
+  `f6` int DEFAULT NULL,
+  `f7` int DEFAULT NULL,
+  `f8` int DEFAULT NULL,
+  `f9` int DEFAULT NULL,
+  `f10` int DEFAULT NULL,
+  `f11` int DEFAULT NULL,
+  `f12` int DEFAULT NULL,
+  `f13` int DEFAULT NULL,
+  `f14` int DEFAULT NULL,
+  `f15` int DEFAULT NULL,
+  `f16` int DEFAULT NULL,
+  `f17` int DEFAULT NULL,
+  `f18` int DEFAULT NULL,
+  `f19` int DEFAULT NULL,
+  `f20` int DEFAULT NULL,
+  `f21` int DEFAULT NULL,
+  `f22` int DEFAULT NULL,
+  `f23` int DEFAULT NULL,
+  `f24` int DEFAULT NULL,
+  `f25` int DEFAULT NULL,
+  `f26` int DEFAULT NULL,
+  `f27` int DEFAULT NULL,
+  `f28` int DEFAULT NULL,
+  `f29` int DEFAULT NULL,
+  `f30` int DEFAULT NULL,
+  `f31` int DEFAULT NULL,
+  `f32` int DEFAULT NULL,
+  `f33` int DEFAULT NULL,
+  `f34` int DEFAULT NULL,
+  `f35` int DEFAULT NULL,
+  `f36` int DEFAULT NULL,
+  `f37` int DEFAULT NULL,
+  `f38` int DEFAULT NULL,
+  `f39` int DEFAULT NULL,
+  `f40` int DEFAULT NULL,
+  `f41` int DEFAULT NULL,
+  `f42` int DEFAULT NULL,
+  `f43` int DEFAULT NULL,
+  `f44` int DEFAULT NULL,
+  `f45` int DEFAULT NULL,
+  `f46` int DEFAULT NULL,
+  `f47` int DEFAULT NULL,
+  `f48` int DEFAULT NULL,
+  `f49` int DEFAULT NULL,
+  `f50` int DEFAULT NULL,
+  `f51` int DEFAULT NULL,
+  `f52` int DEFAULT NULL,
+  `f53` int DEFAULT NULL,
+  `f54` int DEFAULT NULL,
+  `f55` int DEFAULT NULL,
+  `f56` int DEFAULT NULL,
+  `f57` int DEFAULT NULL,
+  `f58` int DEFAULT NULL,
+  `f59` int DEFAULT NULL,
+  `f60` int DEFAULT NULL,
+  `f61` int DEFAULT NULL,
+  `f62` int DEFAULT NULL,
+  `f63` int DEFAULT NULL,
+  `f64` int DEFAULT NULL,
+  `f65` int DEFAULT NULL,
+  `f66` int DEFAULT NULL,
+  `f67` int DEFAULT NULL,
+  `f68` int DEFAULT NULL,
+  `f69` int DEFAULT NULL,
+  `f70` int DEFAULT NULL,
+  `f71` int DEFAULT NULL,
+  `f72` int DEFAULT NULL,
+  `f73` int DEFAULT NULL,
+  `f74` int DEFAULT NULL,
+  `f75` int DEFAULT NULL,
+  `f76` int DEFAULT NULL,
+  `f77` int DEFAULT NULL,
+  `f78` int DEFAULT NULL,
+  `f79` int DEFAULT NULL,
+  `f80` int DEFAULT NULL,
+  `f81` int DEFAULT NULL,
+  `f82` int DEFAULT NULL,
+  `f83` int DEFAULT NULL,
+  `f84` int DEFAULT NULL,
+  `f85` int DEFAULT NULL,
+  `f86` int DEFAULT NULL,
+  `f87` int DEFAULT NULL,
+  `f88` int DEFAULT NULL,
+  `f89` int DEFAULT NULL,
+  `f90` int DEFAULT NULL,
+  `f91` int DEFAULT NULL,
+  `f92` int DEFAULT NULL,
+  `f93` int DEFAULT NULL,
+  `f94` int DEFAULT NULL,
+  `f95` int DEFAULT NULL,
+  `f96` int DEFAULT NULL,
+  `f97` int DEFAULT NULL,
+  `f98` int DEFAULT NULL,
+  `f99` int DEFAULT NULL,
+  `f100` int DEFAULT NULL,
+  `f101` int DEFAULT NULL,
+  `f102` int DEFAULT NULL,
+  `f103` int DEFAULT NULL,
+  `f104` int DEFAULT NULL,
+  `f105` int DEFAULT NULL,
+  `f106` int DEFAULT NULL,
+  `f107` int DEFAULT NULL,
+  `f108` int DEFAULT NULL,
+  `f109` int DEFAULT NULL,
+  `f110` int DEFAULT NULL,
+  `f111` int DEFAULT NULL,
+  `f112` int DEFAULT NULL,
+  `f113` int DEFAULT NULL,
+  `f114` int DEFAULT NULL,
+  `f115` int DEFAULT NULL,
+  `f116` int DEFAULT NULL,
+  `f117` int DEFAULT NULL,
+  `f118` int DEFAULT NULL,
+  `f119` int DEFAULT NULL,
+  `f120` int DEFAULT NULL,
+  `f121` int DEFAULT NULL,
+  `f122` int DEFAULT NULL,
+  `f123` int DEFAULT NULL,
+  `f124` int DEFAULT NULL,
+  `f125` int DEFAULT NULL,
+  `f126` int DEFAULT NULL,
+  `f127` int DEFAULT NULL,
+  `f128` int DEFAULT NULL,
+  `f129` int DEFAULT NULL,
+  `f130` int DEFAULT NULL,
+  `f131` int DEFAULT NULL,
+  `f132` int DEFAULT NULL,
+  `f133` int DEFAULT NULL,
+  `f134` int DEFAULT NULL,
+  `f135` int DEFAULT NULL,
+  `f136` int DEFAULT NULL,
+  `f137` int DEFAULT NULL,
+  `f138` int DEFAULT NULL,
+  `f139` int DEFAULT NULL,
+  `f140` int DEFAULT NULL,
+  `f141` int DEFAULT NULL,
+  `f142` int DEFAULT NULL,
+  `f143` int DEFAULT NULL,
+  `f144` int DEFAULT NULL,
+  `f145` int DEFAULT NULL,
+  `f146` int DEFAULT NULL,
+  `f147` int DEFAULT NULL,
+  `f148` int DEFAULT NULL,
+  `f149` int DEFAULT NULL,
+  `f150` int DEFAULT NULL,
+  `f151` int DEFAULT NULL,
+  `f152` int DEFAULT NULL,
+  `f153` int DEFAULT NULL,
+  `f154` int DEFAULT NULL,
+  `f155` int DEFAULT NULL,
+  `f156` int DEFAULT NULL,
+  `f157` int DEFAULT NULL,
+  `f158` int DEFAULT NULL,
+  `f159` int DEFAULT NULL,
+  `f160` int DEFAULT NULL,
+  `f161` int DEFAULT NULL,
+  `f162` int DEFAULT NULL,
+  `f163` int DEFAULT NULL,
+  `f164` int DEFAULT NULL,
+  `f165` int DEFAULT NULL,
+  `f166` int DEFAULT NULL,
+  `f167` int DEFAULT NULL,
+  `f168` int DEFAULT NULL,
+  `f169` int DEFAULT NULL,
+  `f170` int DEFAULT NULL,
+  `f171` int DEFAULT NULL,
+  `f172` int DEFAULT NULL,
+  `f173` int DEFAULT NULL,
+  `f174` int DEFAULT NULL,
+  `f175` int DEFAULT NULL,
+  `f176` int DEFAULT NULL,
+  `f177` int DEFAULT NULL,
+  `f178` int DEFAULT NULL,
+  `f179` int DEFAULT NULL,
+  `f180` int DEFAULT NULL,
+  `f181` int DEFAULT NULL,
+  `f182` int DEFAULT NULL,
+  `f183` int DEFAULT NULL,
+  `f184` int DEFAULT NULL,
+  `f185` int DEFAULT NULL,
+  `f186` int DEFAULT NULL,
+  `f187` int DEFAULT NULL,
+  `f188` int DEFAULT NULL,
+  `f189` int DEFAULT NULL,
+  `f190` int DEFAULT NULL,
+  `f191` int DEFAULT NULL,
+  `f192` int DEFAULT NULL,
+  `f193` int DEFAULT NULL,
+  `f194` int DEFAULT NULL,
+  `f195` int DEFAULT NULL,
+  `f196` int DEFAULT NULL,
+  `f197` int DEFAULT NULL,
+  `f198` int DEFAULT NULL,
+  `f199` int DEFAULT NULL,
+  `f200` int DEFAULT NULL,
+  `f201` int DEFAULT NULL,
+  `f202` int DEFAULT NULL,
+  `f203` int DEFAULT NULL,
+  `f204` int DEFAULT NULL,
+  `f205` int DEFAULT NULL,
+  `f206` int DEFAULT NULL,
+  `f207` int DEFAULT NULL,
+  `f208` int DEFAULT NULL,
+  `f209` int DEFAULT NULL,
+  `f210` int DEFAULT NULL,
+  `f211` int DEFAULT NULL,
+  `f212` int DEFAULT NULL,
+  `f213` int DEFAULT NULL,
+  `f214` int DEFAULT NULL,
+  `f215` int DEFAULT NULL,
+  `f216` int DEFAULT NULL,
+  `f217` int DEFAULT NULL,
+  `f218` int DEFAULT NULL,
+  `f219` int DEFAULT NULL,
+  `f220` int DEFAULT NULL,
+  `f221` int DEFAULT NULL,
+  `f222` int DEFAULT NULL,
+  `f223` int DEFAULT NULL,
+  `f224` int DEFAULT NULL,
+  `f225` int DEFAULT NULL,
+  `f226` int DEFAULT NULL,
+  `f227` int DEFAULT NULL,
+  `f228` int DEFAULT NULL,
+  `f229` int DEFAULT NULL,
+  `f230` int DEFAULT NULL,
+  `f231` int DEFAULT NULL,
+  `f232` int DEFAULT NULL,
+  `f233` int DEFAULT NULL,
+  `f234` int DEFAULT NULL,
+  `f235` int DEFAULT NULL,
+  `f236` int DEFAULT NULL,
+  `f237` int DEFAULT NULL,
+  `f238` int DEFAULT NULL,
+  `f239` int DEFAULT NULL,
+  `f240` int DEFAULT NULL,
+  `f241` int DEFAULT NULL,
+  `f242` int DEFAULT NULL,
+  `f243` int DEFAULT NULL,
+  `f244` int DEFAULT NULL,
+  `f245` int DEFAULT NULL,
+  `f246` int DEFAULT NULL,
+  `f247` int DEFAULT NULL,
+  `f248` int DEFAULT NULL,
+  `f249` int DEFAULT NULL,
+  `f250` int DEFAULT NULL,
+  `f251` int DEFAULT NULL,
+  `f252` int DEFAULT NULL,
+  `f253` int DEFAULT NULL,
+  `f254` int DEFAULT NULL,
+  `f255` int DEFAULT NULL,
+  `f256` int DEFAULT NULL,
+  `f257` int DEFAULT NULL,
+  `f258` int DEFAULT NULL,
+  `f259` int DEFAULT NULL,
+  `f260` int DEFAULT NULL,
+  `f261` int DEFAULT NULL,
+  `f262` int DEFAULT NULL,
+  `f263` int DEFAULT NULL,
+  `f264` int DEFAULT NULL,
+  `f265` int DEFAULT NULL,
+  `f266` int DEFAULT NULL,
+  `f267` int DEFAULT NULL,
+  `f268` int DEFAULT NULL,
+  `f269` int DEFAULT NULL,
+  `f270` int DEFAULT NULL,
+  `f271` int DEFAULT NULL,
+  `f272` int DEFAULT NULL,
+  `f273` int DEFAULT NULL,
+  `f274` int DEFAULT NULL,
+  `f275` int DEFAULT NULL,
+  `f276` int DEFAULT NULL,
+  `f277` int DEFAULT NULL,
+  `f278` int DEFAULT NULL,
+  `f279` int DEFAULT NULL,
+  `f280` int DEFAULT NULL,
+  `f281` int DEFAULT NULL,
+  `f282` int DEFAULT NULL,
+  `f283` int DEFAULT NULL,
+  `f284` int DEFAULT NULL,
+  `f285` int DEFAULT NULL,
+  `f286` int DEFAULT NULL,
+  `f287` int DEFAULT NULL,
+  `f288` int DEFAULT NULL,
+  `f289` int DEFAULT NULL,
+  `f290` int DEFAULT NULL,
+  `f291` int DEFAULT NULL,
+  `f292` int DEFAULT NULL,
+  `f293` int DEFAULT NULL,
+  `f294` int DEFAULT NULL,
+  `f295` int DEFAULT NULL,
+  `f296` int DEFAULT NULL,
+  `f297` int DEFAULT NULL,
+  `f298` int DEFAULT NULL,
+  `f299` int DEFAULT NULL,
+  `f300` int DEFAULT NULL,
+  `f301` int DEFAULT NULL,
+  `f302` int DEFAULT NULL,
+  `f303` int DEFAULT NULL,
+  `f304` int DEFAULT NULL,
+  `f305` int DEFAULT NULL,
+  `f306` int DEFAULT NULL,
+  `f307` int DEFAULT NULL,
+  `f308` int DEFAULT NULL,
+  `f309` int DEFAULT NULL,
+  `f310` int DEFAULT NULL,
+  `f311` int DEFAULT NULL,
+  `f312` int DEFAULT NULL,
+  `f313` int DEFAULT NULL,
+  `f314` int DEFAULT NULL,
+  `f315` int DEFAULT NULL,
+  `f316` int DEFAULT NULL,
+  `f317` int DEFAULT NULL,
+  `f318` int DEFAULT NULL,
+  `f319` int DEFAULT NULL,
+  `f320` int DEFAULT NULL,
+  `f321` int DEFAULT NULL,
+  `f322` int DEFAULT NULL,
+  `f323` int DEFAULT NULL,
+  `f324` int DEFAULT NULL,
+  `f325` int DEFAULT NULL,
+  `f326` int DEFAULT NULL,
+  `f327` int DEFAULT NULL,
+  `f328` int DEFAULT NULL,
+  `f329` int DEFAULT NULL,
+  `f330` int DEFAULT NULL,
+  `f331` int DEFAULT NULL,
+  `f332` int DEFAULT NULL,
+  `f333` int DEFAULT NULL,
+  `f334` int DEFAULT NULL,
+  `f335` int DEFAULT NULL,
+  `f336` int DEFAULT NULL,
+  `f337` int DEFAULT NULL,
+  `f338` int DEFAULT NULL,
+  `f339` int DEFAULT NULL,
+  `f340` int DEFAULT NULL,
+  `f341` int DEFAULT NULL,
+  `f342` int DEFAULT NULL,
+  `f343` int DEFAULT NULL,
+  `f344` int DEFAULT NULL,
+  `f345` int DEFAULT NULL,
+  `f346` int DEFAULT NULL,
+  `f347` int DEFAULT NULL,
+  `f348` int DEFAULT NULL,
+  `f349` int DEFAULT NULL,
+  `f350` int DEFAULT NULL,
+  `f351` int DEFAULT NULL,
+  `f352` int DEFAULT NULL,
+  `f353` int DEFAULT NULL,
+  `f354` int DEFAULT NULL,
+  `f355` int DEFAULT NULL,
+  `f356` int DEFAULT NULL,
+  `f357` int DEFAULT NULL,
+  `f358` int DEFAULT NULL,
+  `f359` int DEFAULT NULL,
+  `f360` int DEFAULT NULL,
+  `f361` int DEFAULT NULL,
+  `f362` int DEFAULT NULL,
+  `f363` int DEFAULT NULL,
+  `f364` int DEFAULT NULL,
+  `f365` int DEFAULT NULL,
+  `f366` int DEFAULT NULL,
+  `f367` int DEFAULT NULL,
+  `f368` int DEFAULT NULL,
+  `f369` int DEFAULT NULL,
+  `f370` int DEFAULT NULL,
+  `f371` int DEFAULT NULL,
+  `f372` int DEFAULT NULL,
+  `f373` int DEFAULT NULL,
+  `f374` int DEFAULT NULL,
+  `f375` int DEFAULT NULL,
+  `f376` int DEFAULT NULL,
+  `f377` int DEFAULT NULL,
+  `f378` int DEFAULT NULL,
+  `f379` int DEFAULT NULL,
+  `f380` int DEFAULT NULL,
+  `f381` int DEFAULT NULL,
+  `f382` int DEFAULT NULL,
+  `f383` int DEFAULT NULL,
+  `f384` int DEFAULT NULL,
+  `f385` int DEFAULT NULL,
+  `f386` int DEFAULT NULL,
+  `f387` int DEFAULT NULL,
+  `f388` int DEFAULT NULL,
+  `f389` int DEFAULT NULL,
+  `f390` int DEFAULT NULL,
+  `f391` int DEFAULT NULL,
+  `f392` int DEFAULT NULL,
+  `f393` int DEFAULT NULL,
+  `f394` int DEFAULT NULL,
+  `f395` int DEFAULT NULL,
+  `f396` int DEFAULT NULL,
+  `f397` int DEFAULT NULL,
+  `f398` int DEFAULT NULL,
+  `f399` int DEFAULT NULL,
+  `f400` int DEFAULT NULL,
+  `f401` int DEFAULT NULL,
+  `f402` int DEFAULT NULL,
+  `f403` int DEFAULT NULL,
+  `f404` int DEFAULT NULL,
+  `f405` int DEFAULT NULL,
+  `f406` int DEFAULT NULL,
+  `f407` int DEFAULT NULL,
+  `f408` int DEFAULT NULL,
+  `f409` int DEFAULT NULL,
+  `f410` int DEFAULT NULL,
+  `f411` int DEFAULT NULL,
+  `f412` int DEFAULT NULL,
+  `f413` int DEFAULT NULL,
+  `f414` int DEFAULT NULL,
+  `f415` int DEFAULT NULL,
+  `f416` int DEFAULT NULL,
+  `f417` int DEFAULT NULL,
+  `f418` int DEFAULT NULL,
+  `f419` int DEFAULT NULL,
+  `f420` int DEFAULT NULL,
+  `f421` int DEFAULT NULL,
+  `f422` int DEFAULT NULL,
+  `f423` int DEFAULT NULL,
+  `f424` int DEFAULT NULL,
+  `f425` int DEFAULT NULL,
+  `f426` int DEFAULT NULL,
+  `f427` int DEFAULT NULL,
+  `f428` int DEFAULT NULL,
+  `f429` int DEFAULT NULL,
+  `f430` int DEFAULT NULL,
+  `f431` int DEFAULT NULL,
+  `f432` int DEFAULT NULL,
+  `f433` int DEFAULT NULL,
+  `f434` int DEFAULT NULL,
+  `f435` int DEFAULT NULL,
+  `f436` int DEFAULT NULL,
+  `f437` int DEFAULT NULL,
+  `f438` int DEFAULT NULL,
+  `f439` int DEFAULT NULL,
+  `f440` int DEFAULT NULL,
+  `f441` int DEFAULT NULL,
+  `f442` int DEFAULT NULL,
+  `f443` int DEFAULT NULL,
+  `f444` int DEFAULT NULL,
+  `f445` int DEFAULT NULL,
+  `f446` int DEFAULT NULL,
+  `f447` int DEFAULT NULL,
+  `f448` int DEFAULT NULL,
+  `f449` int DEFAULT NULL,
+  `f450` int DEFAULT NULL,
+  `f451` int DEFAULT NULL,
+  `f452` int DEFAULT NULL,
+  `f453` int DEFAULT NULL,
+  `f454` int DEFAULT NULL,
+  `f455` int DEFAULT NULL,
+  `f456` int DEFAULT NULL,
+  `f457` int DEFAULT NULL,
+  `f458` int DEFAULT NULL,
+  `f459` int DEFAULT NULL,
+  `f460` int DEFAULT NULL,
+  `f461` int DEFAULT NULL,
+  `f462` int DEFAULT NULL,
+  `f463` int DEFAULT NULL,
+  `f464` int DEFAULT NULL,
+  `f465` int DEFAULT NULL,
+  `f466` int DEFAULT NULL,
+  `f467` int DEFAULT NULL,
+  `f468` int DEFAULT NULL,
+  `f469` int DEFAULT NULL,
+  `f470` int DEFAULT NULL,
+  `f471` int DEFAULT NULL,
+  `f472` int DEFAULT NULL,
+  `f473` int DEFAULT NULL,
+  `f474` int DEFAULT NULL,
+  `f475` int DEFAULT NULL,
+  `f476` int DEFAULT NULL,
+  `f477` int DEFAULT NULL,
+  `f478` int DEFAULT NULL,
+  `f479` int DEFAULT NULL,
+  `f480` int DEFAULT NULL,
+  `f481` int DEFAULT NULL,
+  `f482` int DEFAULT NULL,
+  `f483` int DEFAULT NULL,
+  `f484` int DEFAULT NULL,
+  `f485` int DEFAULT NULL,
+  `f486` int DEFAULT NULL,
+  `f487` int DEFAULT NULL,
+  `f488` int DEFAULT NULL,
+  `f489` int DEFAULT NULL,
+  `f490` int DEFAULT NULL,
+  `f491` int DEFAULT NULL,
+  `f492` int DEFAULT NULL,
+  `f493` int DEFAULT NULL,
+  `f494` int DEFAULT NULL,
+  `f495` int DEFAULT NULL,
+  `f496` int DEFAULT NULL,
+  `f497` int DEFAULT NULL,
+  `f498` int DEFAULT NULL,
+  `f499` int DEFAULT NULL,
+  `f500` int DEFAULT NULL,
+  `f501` int DEFAULT NULL,
+  `f502` int DEFAULT NULL,
+  `f503` int DEFAULT NULL,
+  `f504` int DEFAULT NULL,
+  `f505` int DEFAULT NULL,
+  `f506` int DEFAULT NULL,
+  `f507` int DEFAULT NULL,
+  `f508` int DEFAULT NULL,
+  `f509` int DEFAULT NULL,
+  `f510` int DEFAULT NULL,
+  `f511` int DEFAULT NULL,
+  `f512` int DEFAULT NULL,
+  `f513` int DEFAULT NULL,
+  `f514` int DEFAULT NULL,
+  `f515` int DEFAULT NULL,
+  `f516` int DEFAULT NULL,
+  `f517` int DEFAULT NULL,
+  `f518` int DEFAULT NULL,
+  `f519` int DEFAULT NULL,
+  `f520` int DEFAULT NULL,
+  `f521` int DEFAULT NULL,
+  `f522` int DEFAULT NULL,
+  `f523` int DEFAULT NULL,
+  `f524` int DEFAULT NULL,
+  `f525` int DEFAULT NULL,
+  `f526` int DEFAULT NULL,
+  `f527` int DEFAULT NULL,
+  `f528` int DEFAULT NULL,
+  `f529` int DEFAULT NULL,
+  `f530` int DEFAULT NULL,
+  `f531` int DEFAULT NULL,
+  `f532` int DEFAULT NULL,
+  `f533` int DEFAULT NULL,
+  `f534` int DEFAULT NULL,
+  `f535` int DEFAULT NULL,
+  `f536` int DEFAULT NULL,
+  `f537` int DEFAULT NULL,
+  `f538` int DEFAULT NULL,
+  `f539` int DEFAULT NULL,
+  `f540` int DEFAULT NULL,
+  `f541` int DEFAULT NULL,
+  `f542` int DEFAULT NULL,
+  `f543` int DEFAULT NULL,
+  `f544` int DEFAULT NULL,
+  `f545` int DEFAULT NULL,
+  `f546` int DEFAULT NULL,
+  `f547` int DEFAULT NULL,
+  `f548` int DEFAULT NULL,
+  `f549` int DEFAULT NULL,
+  `f550` int DEFAULT NULL,
+  `f551` int DEFAULT NULL,
+  `f552` int DEFAULT NULL,
+  `f553` int DEFAULT NULL,
+  `f554` int DEFAULT NULL,
+  `f555` int DEFAULT NULL,
+  `f556` int DEFAULT NULL,
+  `f557` int DEFAULT NULL,
+  `f558` int DEFAULT NULL,
+  `f559` int DEFAULT NULL,
+  `f560` int DEFAULT NULL,
+  `f561` int DEFAULT NULL,
+  `f562` int DEFAULT NULL,
+  `f563` int DEFAULT NULL,
+  `f564` int DEFAULT NULL,
+  `f565` int DEFAULT NULL,
+  `f566` int DEFAULT NULL,
+  `f567` int DEFAULT NULL,
+  `f568` int DEFAULT NULL,
+  `f569` int DEFAULT NULL,
+  `f570` int DEFAULT NULL,
+  `f571` int DEFAULT NULL,
+  `f572` int DEFAULT NULL,
+  `f573` int DEFAULT NULL,
+  `f574` int DEFAULT NULL,
+  `f575` int DEFAULT NULL,
+  `f576` int DEFAULT NULL,
+  `f577` int DEFAULT NULL,
+  `f578` int DEFAULT NULL,
+  `f579` int DEFAULT NULL,
+  `f580` int DEFAULT NULL,
+  `f581` int DEFAULT NULL,
+  `f582` int DEFAULT NULL,
+  `f583` int DEFAULT NULL,
+  `f584` int DEFAULT NULL,
+  `f585` int DEFAULT NULL,
+  `f586` int DEFAULT NULL,
+  `f587` int DEFAULT NULL,
+  `f588` int DEFAULT NULL,
+  `f589` int DEFAULT NULL,
+  `f590` int DEFAULT NULL,
+  `f591` int DEFAULT NULL,
+  `f592` int DEFAULT NULL,
+  `f593` int DEFAULT NULL,
+  `f594` int DEFAULT NULL,
+  `f595` int DEFAULT NULL,
+  `f596` int DEFAULT NULL,
+  `f597` int DEFAULT NULL,
+  `f598` int DEFAULT NULL,
+  `f599` int DEFAULT NULL,
+  `f600` int DEFAULT NULL,
+  `f601` int DEFAULT NULL,
+  `f602` int DEFAULT NULL,
+  `f603` int DEFAULT NULL,
+  `f604` int DEFAULT NULL,
+  `f605` int DEFAULT NULL,
+  `f606` int DEFAULT NULL,
+  `f607` int DEFAULT NULL,
+  `f608` int DEFAULT NULL,
+  `f609` int DEFAULT NULL,
+  `f610` int DEFAULT NULL,
+  `f611` int DEFAULT NULL,
+  `f612` int DEFAULT NULL,
+  `f613` int DEFAULT NULL,
+  `f614` int DEFAULT NULL,
+  `f615` int DEFAULT NULL,
+  `f616` int DEFAULT NULL,
+  `f617` int DEFAULT NULL,
+  `f618` int DEFAULT NULL,
+  `f619` int DEFAULT NULL,
+  `f620` int DEFAULT NULL,
+  `f621` int DEFAULT NULL,
+  `f622` int DEFAULT NULL,
+  `f623` int DEFAULT NULL,
+  `f624` int DEFAULT NULL,
+  `f625` int DEFAULT NULL,
+  `f626` int DEFAULT NULL,
+  `f627` int DEFAULT NULL,
+  `f628` int DEFAULT NULL,
+  `f629` int DEFAULT NULL,
+  `f630` int DEFAULT NULL,
+  `f631` int DEFAULT NULL,
+  `f632` int DEFAULT NULL,
+  `f633` int DEFAULT NULL,
+  `f634` int DEFAULT NULL,
+  `f635` int DEFAULT NULL,
+  `f636` int DEFAULT NULL,
+  `f637` int DEFAULT NULL,
+  `f638` int DEFAULT NULL,
+  `f639` int DEFAULT NULL,
+  `f640` int DEFAULT NULL,
+  `f641` int DEFAULT NULL,
+  `f642` int DEFAULT NULL,
+  `f643` int DEFAULT NULL,
+  `f644` int DEFAULT NULL,
+  `f645` int DEFAULT NULL,
+  `f646` int DEFAULT NULL,
+  `f647` int DEFAULT NULL,
+  `f648` int DEFAULT NULL,
+  `f649` int DEFAULT NULL,
+  `f650` int DEFAULT NULL,
+  `f651` int DEFAULT NULL,
+  `f652` int DEFAULT NULL,
+  `f653` int DEFAULT NULL,
+  `f654` int DEFAULT NULL,
+  `f655` int DEFAULT NULL,
+  `f656` int DEFAULT NULL,
+  `f657` int DEFAULT NULL,
+  `f658` int DEFAULT NULL,
+  `f659` int DEFAULT NULL,
+  `f660` int DEFAULT NULL,
+  `f661` int DEFAULT NULL,
+  `f662` int DEFAULT NULL,
+  `f663` int DEFAULT NULL,
+  `f664` int DEFAULT NULL,
+  `f665` int DEFAULT NULL,
+  `f666` int DEFAULT NULL,
+  `f667` int DEFAULT NULL,
+  `f668` int DEFAULT NULL,
+  `f669` int DEFAULT NULL,
+  `f670` int DEFAULT NULL,
+  `f671` int DEFAULT NULL,
+  `f672` int DEFAULT NULL,
+  `f673` int DEFAULT NULL,
+  `f674` int DEFAULT NULL,
+  `f675` int DEFAULT NULL,
+  `f676` int DEFAULT NULL,
+  `f677` int DEFAULT NULL,
+  `f678` int DEFAULT NULL,
+  `f679` int DEFAULT NULL,
+  `f680` int DEFAULT NULL,
+  `f681` int DEFAULT NULL,
+  `f682` int DEFAULT NULL,
+  `f683` int DEFAULT NULL,
+  `f684` int DEFAULT NULL,
+  `f685` int DEFAULT NULL,
+  `f686` int DEFAULT NULL,
+  `f687` int DEFAULT NULL,
+  `f688` int DEFAULT NULL,
+  `f689` int DEFAULT NULL,
+  `f690` int DEFAULT NULL,
+  `f691` int DEFAULT NULL,
+  `f692` int DEFAULT NULL,
+  `f693` int DEFAULT NULL,
+  `f694` int DEFAULT NULL,
+  `f695` int DEFAULT NULL,
+  `f696` int DEFAULT NULL,
+  `f697` int DEFAULT NULL,
+  `f698` int DEFAULT NULL,
+  `f699` int DEFAULT NULL,
+  `f700` int DEFAULT NULL,
+  `f701` int DEFAULT NULL,
+  `f702` int DEFAULT NULL,
+  `f703` int DEFAULT NULL,
+  `f704` int DEFAULT NULL,
+  `f705` int DEFAULT NULL,
+  `f706` int DEFAULT NULL,
+  `f707` int DEFAULT NULL,
+  `f708` int DEFAULT NULL,
+  `f709` int DEFAULT NULL,
+  `f710` int DEFAULT NULL,
+  `f711` int DEFAULT NULL,
+  `f712` int DEFAULT NULL,
+  `f713` int DEFAULT NULL,
+  `f714` int DEFAULT NULL,
+  `f715` int DEFAULT NULL,
+  `f716` int DEFAULT NULL,
+  `f717` int DEFAULT NULL,
+  `f718` int DEFAULT NULL,
+  `f719` int DEFAULT NULL,
+  `f720` int DEFAULT NULL,
+  `f721` int DEFAULT NULL,
+  `f722` int DEFAULT NULL,
+  `f723` int DEFAULT NULL,
+  `f724` int DEFAULT NULL,
+  `f725` int DEFAULT NULL,
+  `f726` int DEFAULT NULL,
+  `f727` int DEFAULT NULL,
+  `f728` int DEFAULT NULL,
+  `f729` int DEFAULT NULL,
+  `f730` int DEFAULT NULL,
+  `f731` int DEFAULT NULL,
+  `f732` int DEFAULT NULL,
+  `f733` int DEFAULT NULL,
+  `f734` int DEFAULT NULL,
+  `f735` int DEFAULT NULL,
+  `f736` int DEFAULT NULL,
+  `f737` int DEFAULT NULL,
+  `f738` int DEFAULT NULL,
+  `f739` int DEFAULT NULL,
+  `f740` int DEFAULT NULL,
+  `f741` int DEFAULT NULL,
+  `f742` int DEFAULT NULL,
+  `f743` int DEFAULT NULL,
+  `f744` int DEFAULT NULL,
+  `f745` int DEFAULT NULL,
+  `f746` int DEFAULT NULL,
+  `f747` int DEFAULT NULL,
+  `f748` int DEFAULT NULL,
+  `f749` int DEFAULT NULL,
+  `f750` int DEFAULT NULL,
+  `f751` int DEFAULT NULL,
+  `f752` int DEFAULT NULL,
+  `f753` int DEFAULT NULL,
+  `f754` int DEFAULT NULL,
+  `f755` int DEFAULT NULL,
+  `f756` int DEFAULT NULL,
+  `f757` int DEFAULT NULL,
+  `f758` int DEFAULT NULL,
+  `f759` int DEFAULT NULL,
+  `f760` int DEFAULT NULL,
+  `f761` int DEFAULT NULL,
+  `f762` int DEFAULT NULL,
+  `f763` int DEFAULT NULL,
+  `f764` int DEFAULT NULL,
+  `f765` int DEFAULT NULL,
+  `f766` int DEFAULT NULL,
+  `f767` int DEFAULT NULL,
+  `f768` int DEFAULT NULL,
+  `f769` int DEFAULT NULL,
+  `f770` int DEFAULT NULL,
+  `f771` int DEFAULT NULL,
+  `f772` int DEFAULT NULL,
+  `f773` int DEFAULT NULL,
+  `f774` int DEFAULT NULL,
+  `f775` int DEFAULT NULL,
+  `f776` int DEFAULT NULL,
+  `f777` int DEFAULT NULL,
+  `f778` int DEFAULT NULL,
+  `f779` int DEFAULT NULL,
+  `f780` int DEFAULT NULL,
+  `f781` int DEFAULT NULL,
+  `f782` int DEFAULT NULL,
+  `f783` int DEFAULT NULL,
+  `f784` int DEFAULT NULL,
+  `f785` int DEFAULT NULL,
+  `f786` int DEFAULT NULL,
+  `f787` int DEFAULT NULL,
+  `f788` int DEFAULT NULL,
+  `f789` int DEFAULT NULL,
+  `f790` int DEFAULT NULL,
+  `f791` int DEFAULT NULL,
+  `f792` int DEFAULT NULL,
+  `f793` int DEFAULT NULL,
+  `f794` int DEFAULT NULL,
+  `f795` int DEFAULT NULL,
+  `f796` int DEFAULT NULL,
+  `f797` int DEFAULT NULL,
+  `f798` int DEFAULT NULL,
+  `f799` int DEFAULT NULL,
+  `f800` int DEFAULT NULL,
+  `f801` int DEFAULT NULL,
+  `f802` int DEFAULT NULL,
+  `f803` int DEFAULT NULL,
+  `f804` int DEFAULT NULL,
+  `f805` int DEFAULT NULL,
+  `f806` int DEFAULT NULL,
+  `f807` int DEFAULT NULL,
+  `f808` int DEFAULT NULL,
+  `f809` int DEFAULT NULL,
+  `f810` int DEFAULT NULL,
+  `f811` int DEFAULT NULL,
+  `f812` int DEFAULT NULL,
+  `f813` int DEFAULT NULL,
+  `f814` int DEFAULT NULL,
+  `f815` int DEFAULT NULL,
+  `f816` int DEFAULT NULL,
+  `f817` int DEFAULT NULL,
+  `f818` int DEFAULT NULL,
+  `f819` int DEFAULT NULL,
+  `f820` int DEFAULT NULL,
+  `f821` int DEFAULT NULL,
+  `f822` int DEFAULT NULL,
+  `f823` int DEFAULT NULL,
+  `f824` int DEFAULT NULL,
+  `f825` int DEFAULT NULL,
+  `f826` int DEFAULT NULL,
+  `f827` int DEFAULT NULL,
+  `f828` int DEFAULT NULL,
+  `f829` int DEFAULT NULL,
+  `f830` int DEFAULT NULL,
+  `f831` int DEFAULT NULL,
+  `f832` int DEFAULT NULL,
+  `f833` int DEFAULT NULL,
+  `f834` int DEFAULT NULL,
+  `f835` int DEFAULT NULL,
+  `f836` int DEFAULT NULL,
+  `f837` int DEFAULT NULL,
+  `f838` int DEFAULT NULL,
+  `f839` int DEFAULT NULL,
+  `f840` int DEFAULT NULL,
+  `f841` int DEFAULT NULL,
+  `f842` int DEFAULT NULL,
+  `f843` int DEFAULT NULL,
+  `f844` int DEFAULT NULL,
+  `f845` int DEFAULT NULL,
+  `f846` int DEFAULT NULL,
+  `f847` int DEFAULT NULL,
+  `f848` int DEFAULT NULL,
+  `f849` int DEFAULT NULL,
+  `f850` int DEFAULT NULL,
+  `f851` int DEFAULT NULL,
+  `f852` int DEFAULT NULL,
+  `f853` int DEFAULT NULL,
+  `f854` int DEFAULT NULL,
+  `f855` int DEFAULT NULL,
+  `f856` int DEFAULT NULL,
+  `f857` int DEFAULT NULL,
+  `f858` int DEFAULT NULL,
+  `f859` int DEFAULT NULL,
+  `f860` int DEFAULT NULL,
+  `f861` int DEFAULT NULL,
+  `f862` int DEFAULT NULL,
+  `f863` int DEFAULT NULL,
+  `f864` int DEFAULT NULL,
+  `f865` int DEFAULT NULL,
+  `f866` int DEFAULT NULL,
+  `f867` int DEFAULT NULL,
+  `f868` int DEFAULT NULL,
+  `f869` int DEFAULT NULL,
+  `f870` int DEFAULT NULL,
+  `f871` int DEFAULT NULL,
+  `f872` int DEFAULT NULL,
+  `f873` int DEFAULT NULL,
+  `f874` int DEFAULT NULL,
+  `f875` int DEFAULT NULL,
+  `f876` int DEFAULT NULL,
+  `f877` int DEFAULT NULL,
+  `f878` int DEFAULT NULL,
+  `f879` int DEFAULT NULL,
+  `f880` int DEFAULT NULL,
+  `f881` int DEFAULT NULL,
+  `f882` int DEFAULT NULL,
+  `f883` int DEFAULT NULL,
+  `f884` int DEFAULT NULL,
+  `f885` int DEFAULT NULL,
+  `f886` int DEFAULT NULL,
+  `f887` int DEFAULT NULL,
+  `f888` int DEFAULT NULL,
+  `f889` int DEFAULT NULL,
+  `f890` int DEFAULT NULL,
+  `f891` int DEFAULT NULL,
+  `f892` int DEFAULT NULL,
+  `f893` int DEFAULT NULL,
+  `f894` int DEFAULT NULL,
+  `f895` int DEFAULT NULL,
+  `f896` int DEFAULT NULL,
+  `f897` int DEFAULT NULL,
+  `f898` int DEFAULT NULL,
+  `f899` int DEFAULT NULL,
+  `f900` int DEFAULT NULL,
+  `f901` int DEFAULT NULL,
+  `f902` int DEFAULT NULL,
+  `f903` int DEFAULT NULL,
+  `f904` int DEFAULT NULL,
+  `f905` int DEFAULT NULL,
+  `f906` int DEFAULT NULL,
+  `f907` int DEFAULT NULL,
+  `f908` int DEFAULT NULL,
+  `f909` int DEFAULT NULL,
+  `f910` int DEFAULT NULL,
+  `f911` int DEFAULT NULL,
+  `f912` int DEFAULT NULL,
+  `f913` int DEFAULT NULL,
+  `f914` int DEFAULT NULL,
+  `f915` int DEFAULT NULL,
+  `f916` int DEFAULT NULL,
+  `f917` int DEFAULT NULL,
+  `f918` int DEFAULT NULL,
+  `f919` int DEFAULT NULL,
+  `f920` int DEFAULT NULL,
+  `f921` int DEFAULT NULL,
+  `f922` int DEFAULT NULL,
+  `f923` int DEFAULT NULL,
+  `f924` int DEFAULT NULL,
+  `f925` int DEFAULT NULL,
+  `f926` int DEFAULT NULL,
+  `f927` int DEFAULT NULL,
+  `f928` int DEFAULT NULL,
+  `f929` int DEFAULT NULL,
+  `f930` int DEFAULT NULL,
+  `f931` int DEFAULT NULL,
+  `f932` int DEFAULT NULL,
+  `f933` int DEFAULT NULL,
+  `f934` int DEFAULT NULL,
+  `f935` int DEFAULT NULL,
+  `f936` int DEFAULT NULL,
+  `f937` int DEFAULT NULL,
+  `f938` int DEFAULT NULL,
+  `f939` int DEFAULT NULL,
+  `f940` int DEFAULT NULL,
+  `f941` int DEFAULT NULL,
+  `f942` int DEFAULT NULL,
+  `f943` int DEFAULT NULL,
+  `f944` int DEFAULT NULL,
+  `f945` int DEFAULT NULL,
+  `f946` int DEFAULT NULL,
+  `f947` int DEFAULT NULL,
+  `f948` int DEFAULT NULL,
+  `f949` int DEFAULT NULL,
+  `f950` int DEFAULT NULL,
+  `f951` int DEFAULT NULL,
+  `f952` int DEFAULT NULL,
+  `f953` int DEFAULT NULL,
+  `f954` int DEFAULT NULL,
+  `f955` int DEFAULT NULL,
+  `f956` int DEFAULT NULL,
+  `f957` int DEFAULT NULL,
+  `f958` int DEFAULT NULL,
+  `f959` int DEFAULT NULL,
+  `f960` int DEFAULT NULL,
+  `f961` int DEFAULT NULL,
+  `f962` int DEFAULT NULL,
+  `f963` int DEFAULT NULL,
+  `f964` int DEFAULT NULL,
+  `f965` int DEFAULT NULL,
+  `f966` int DEFAULT NULL,
+  `f967` int DEFAULT NULL,
+  `f968` int DEFAULT NULL,
+  `f969` int DEFAULT NULL,
+  `f970` int DEFAULT NULL,
+  `f971` int DEFAULT NULL,
+  `f972` int DEFAULT NULL,
+  `f973` int DEFAULT NULL,
+  `f974` int DEFAULT NULL,
+  `f975` int DEFAULT NULL,
+  `f976` int DEFAULT NULL,
+  `f977` int DEFAULT NULL,
+  `f978` int DEFAULT NULL,
+  `f979` int DEFAULT NULL,
+  `f980` int DEFAULT NULL,
+  `f981` int DEFAULT NULL,
+  `f982` int DEFAULT NULL,
+  `f983` int DEFAULT NULL,
+  `f984` int DEFAULT NULL,
+  `f985` int DEFAULT NULL,
+  `f986` int DEFAULT NULL,
+  `f987` int DEFAULT NULL,
+  `f988` int DEFAULT NULL,
+  `f989` int DEFAULT NULL,
+  `f990` int DEFAULT NULL,
+  `f991` int DEFAULT NULL,
+  `f992` int DEFAULT NULL,
+  `f993` int DEFAULT NULL,
+  `f994` int DEFAULT NULL,
+  `f995` int DEFAULT NULL,
+  `f996` int DEFAULT NULL,
+  `f997` int DEFAULT NULL,
+  `f998` int DEFAULT NULL,
+  `f999` int DEFAULT NULL,
+  `f1000` int DEFAULT NULL,
+  `f1001` int DEFAULT NULL,
+  `f1002` int DEFAULT NULL,
+  `f1003` int DEFAULT NULL,
+  `f1004` int DEFAULT NULL,
+  `f1005` int DEFAULT NULL,
+  `f1006` int DEFAULT NULL,
+  `f1007` int DEFAULT NULL,
+  `f1008` int DEFAULT NULL,
+  `f1009` int DEFAULT NULL,
+  `f1010` int DEFAULT NULL,
+  `f1011` int DEFAULT NULL,
+  `f1012` int DEFAULT NULL,
+  `f1013` int DEFAULT NULL,
+  `f1014` int DEFAULT NULL,
+  `f1015` int DEFAULT NULL,
+  `f1016` int DEFAULT NULL,
+  `f1017` int DEFAULT NULL,
+  `f1018` int DEFAULT NULL,
+  `f1019` int DEFAULT NULL,
+  `f1020` int DEFAULT NULL,
+  `f1021` int DEFAULT NULL,
+  `f1022` int DEFAULT NULL,
+  `f1023` int DEFAULT NULL,
+  `f1024` int DEFAULT NULL,
+  `f1025` int DEFAULT NULL,
+  `f1026` int DEFAULT NULL,
+  `f1027` int DEFAULT NULL,
+  `f1028` int DEFAULT NULL,
+  `f1029` int DEFAULT NULL,
+  `f1030` int DEFAULT NULL,
+  `f1031` int DEFAULT NULL,
+  `f1032` int DEFAULT NULL,
+  `f1033` int DEFAULT NULL,
+  `f1034` int DEFAULT NULL,
+  `f1035` int DEFAULT NULL,
+  `f1036` int DEFAULT NULL,
+  `f1037` int DEFAULT NULL,
+  `f1038` int DEFAULT NULL,
+  `f1039` int DEFAULT NULL,
+  `f1040` int DEFAULT NULL,
+  `f1041` int DEFAULT NULL,
+  `f1042` int DEFAULT NULL,
+  `f1043` int DEFAULT NULL,
+  `f1044` int DEFAULT NULL,
+  `f1045` int DEFAULT NULL,
+  `f1046` int DEFAULT NULL,
+  `f1047` int DEFAULT NULL,
+  `f1048` int DEFAULT NULL,
+  `f1049` int DEFAULT NULL,
+  `f1050` int DEFAULT NULL,
+  `f1051` int DEFAULT NULL,
+  `f1052` int DEFAULT NULL,
+  `f1053` int DEFAULT NULL,
+  `f1054` int DEFAULT NULL,
+  `f1055` int DEFAULT NULL,
+  `f1056` int DEFAULT NULL,
+  `f1057` int DEFAULT NULL,
+  `f1058` int DEFAULT NULL,
+  `f1059` int DEFAULT NULL,
+  `f1060` int DEFAULT NULL,
+  `f1061` int DEFAULT NULL,
+  `f1062` int DEFAULT NULL,
+  `f1063` int DEFAULT NULL,
+  `f1064` int DEFAULT NULL,
+  `f1065` int DEFAULT NULL,
+  `f1066` int DEFAULT NULL,
+  `f1067` int DEFAULT NULL,
+  `f1068` int DEFAULT NULL,
+  `f1069` int DEFAULT NULL,
+  `f1070` int DEFAULT NULL,
+  `f1071` int DEFAULT NULL,
+  `f1072` int DEFAULT NULL,
+  `f1073` int DEFAULT NULL,
+  `f1074` int DEFAULT NULL,
+  `f1075` int DEFAULT NULL,
+  `f1076` int DEFAULT NULL,
+  `f1077` int DEFAULT NULL,
+  `f1078` int DEFAULT NULL,
+  `f1079` int DEFAULT NULL,
+  `f1080` int DEFAULT NULL,
+  `f1081` int DEFAULT NULL,
+  `f1082` int DEFAULT NULL,
+  `f1083` int DEFAULT NULL,
+  `f1084` int DEFAULT NULL,
+  `f1085` int DEFAULT NULL,
+  `f1086` int DEFAULT NULL,
+  `f1087` int DEFAULT NULL,
+  `f1088` int DEFAULT NULL,
+  `f1089` int DEFAULT NULL,
+  `f1090` int DEFAULT NULL,
+  `f1091` int DEFAULT NULL,
+  `f1092` int DEFAULT NULL,
+  `f1093` int DEFAULT NULL,
+  `f1094` int DEFAULT NULL,
+  `f1095` int DEFAULT NULL,
+  `f1096` int DEFAULT NULL,
+  `f1097` int DEFAULT NULL,
+  `f1098` int DEFAULT NULL,
+  `f1099` int DEFAULT NULL,
+  `f1100` int DEFAULT NULL,
+  `f1101` int DEFAULT NULL,
+  `f1102` int DEFAULT NULL,
+  `f1103` int DEFAULT NULL,
+  `f1104` int DEFAULT NULL,
+  `f1105` int DEFAULT NULL,
+  `f1106` int DEFAULT NULL,
+  `f1107` int DEFAULT NULL,
+  `f1108` int DEFAULT NULL,
+  `f1109` int DEFAULT NULL,
+  `f1110` int DEFAULT NULL,
+  `f1111` int DEFAULT NULL,
+  `f1112` int DEFAULT NULL,
+  `f1113` int DEFAULT NULL,
+  `f1114` int DEFAULT NULL,
+  `f1115` int DEFAULT NULL,
+  `f1116` int DEFAULT NULL,
+  `f1117` int DEFAULT NULL,
+  `f1118` int DEFAULT NULL,
+  `f1119` int DEFAULT NULL,
+  `f1120` int DEFAULT NULL,
+  `f1121` int DEFAULT NULL,
+  `f1122` int DEFAULT NULL,
+  `f1123` int DEFAULT NULL,
+  `f1124` int DEFAULT NULL,
+  `f1125` int DEFAULT NULL,
+  `f1126` int DEFAULT NULL,
+  `f1127` int DEFAULT NULL,
+  `f1128` int DEFAULT NULL,
+  `f1129` int DEFAULT NULL,
+  `f1130` int DEFAULT NULL,
+  `f1131` int DEFAULT NULL,
+  `f1132` int DEFAULT NULL,
+  `f1133` int DEFAULT NULL,
+  `f1134` int DEFAULT NULL,
+  `f1135` int DEFAULT NULL,
+  `f1136` int DEFAULT NULL,
+  `f1137` int DEFAULT NULL,
+  `f1138` int DEFAULT NULL,
+  `f1139` int DEFAULT NULL,
+  `f1140` int DEFAULT NULL,
+  `f1141` int DEFAULT NULL,
+  `f1142` int DEFAULT NULL,
+  `f1143` int DEFAULT NULL,
+  `f1144` int DEFAULT NULL,
+  `f1145` int DEFAULT NULL,
+  `f1146` int DEFAULT NULL,
+  `f1147` int DEFAULT NULL,
+  `f1148` int DEFAULT NULL,
+  `f1149` int DEFAULT NULL,
+  `f1150` int DEFAULT NULL,
+  `f1151` int DEFAULT NULL,
+  `f1152` int DEFAULT NULL,
+  `f1153` int DEFAULT NULL,
+  `f1154` int DEFAULT NULL,
+  `f1155` int DEFAULT NULL,
+  `f1156` int DEFAULT NULL,
+  `f1157` int DEFAULT NULL,
+  `f1158` int DEFAULT NULL,
+  `f1159` int DEFAULT NULL,
+  `f1160` int DEFAULT NULL,
+  `f1161` int DEFAULT NULL,
+  `f1162` int DEFAULT NULL,
+  `f1163` int DEFAULT NULL,
+  `f1164` int DEFAULT NULL,
+  `f1165` int DEFAULT NULL,
+  `f1166` int DEFAULT NULL,
+  `f1167` int DEFAULT NULL,
+  `f1168` int DEFAULT NULL,
+  `f1169` int DEFAULT NULL,
+  `f1170` int DEFAULT NULL,
+  `f1171` int DEFAULT NULL,
+  `f1172` int DEFAULT NULL,
+  `f1173` int DEFAULT NULL,
+  `f1174` int DEFAULT NULL,
+  `f1175` int DEFAULT NULL,
+  `f1176` int DEFAULT NULL,
+  `f1177` int DEFAULT NULL,
+  `f1178` int DEFAULT NULL,
+  `f1179` int DEFAULT NULL,
+  `f1180` int DEFAULT NULL,
+  `f1181` int DEFAULT NULL,
+  `f1182` int DEFAULT NULL,
+  `f1183` int DEFAULT NULL,
+  `f1184` int DEFAULT NULL,
+  `f1185` int DEFAULT NULL,
+  `f1186` int DEFAULT NULL,
+  `f1187` int DEFAULT NULL,
+  `f1188` int DEFAULT NULL,
+  `f1189` int DEFAULT NULL,
+  `f1190` int DEFAULT NULL,
+  `f1191` int DEFAULT NULL,
+  `f1192` int DEFAULT NULL,
+  `f1193` int DEFAULT NULL,
+  `f1194` int DEFAULT NULL,
+  `f1195` int DEFAULT NULL,
+  `f1196` int DEFAULT NULL,
+  `f1197` int DEFAULT NULL,
+  `f1198` int DEFAULT NULL,
+  `f1199` int DEFAULT NULL,
+  `f1200` int DEFAULT NULL,
+  `f1201` int DEFAULT NULL,
+  `f1202` int DEFAULT NULL,
+  `f1203` int DEFAULT NULL,
+  `f1204` int DEFAULT NULL,
+  `f1205` int DEFAULT NULL,
+  `f1206` int DEFAULT NULL,
+  `f1207` int DEFAULT NULL,
+  `f1208` int DEFAULT NULL,
+  `f1209` int DEFAULT NULL,
+  `f1210` int DEFAULT NULL,
+  `f1211` int DEFAULT NULL,
+  `f1212` int DEFAULT NULL,
+  `f1213` int DEFAULT NULL,
+  `f1214` int DEFAULT NULL,
+  `f1215` int DEFAULT NULL,
+  `f1216` int DEFAULT NULL,
+  `f1217` int DEFAULT NULL,
+  `f1218` int DEFAULT NULL,
+  `f1219` int DEFAULT NULL,
+  `f1220` int DEFAULT NULL,
+  `f1221` int DEFAULT NULL,
+  `f1222` int DEFAULT NULL,
+  `f1223` int DEFAULT NULL,
+  `f1224` int DEFAULT NULL,
+  `f1225` int DEFAULT NULL,
+  `f1226` int DEFAULT NULL,
+  `f1227` int DEFAULT NULL,
+  `f1228` int DEFAULT NULL,
+  `f1229` int DEFAULT NULL,
+  `f1230` int DEFAULT NULL,
+  `f1231` int DEFAULT NULL,
+  `f1232` int DEFAULT NULL,
+  `f1233` int DEFAULT NULL,
+  `f1234` int DEFAULT NULL,
+  `f1235` int DEFAULT NULL,
+  `f1236` int DEFAULT NULL,
+  `f1237` int DEFAULT NULL,
+  `f1238` int DEFAULT NULL,
+  `f1239` int DEFAULT NULL,
+  `f1240` int DEFAULT NULL,
+  `f1241` int DEFAULT NULL,
+  `f1242` int DEFAULT NULL,
+  `f1243` int DEFAULT NULL,
+  `f1244` int DEFAULT NULL,
+  `f1245` int DEFAULT NULL,
+  `f1246` int DEFAULT NULL,
+  `f1247` int DEFAULT NULL,
+  `f1248` int DEFAULT NULL,
+  `f1249` int DEFAULT NULL,
+  `f1250` int DEFAULT NULL,
+  `f1251` int DEFAULT NULL,
+  `f1252` int DEFAULT NULL,
+  `f1253` int DEFAULT NULL,
+  `f1254` int DEFAULT NULL,
+  `f1255` int DEFAULT NULL,
+  `f1256` int DEFAULT NULL,
+  `f1257` int DEFAULT NULL,
+  `f1258` int DEFAULT NULL,
+  `f1259` int DEFAULT NULL,
+  `f1260` int DEFAULT NULL,
+  `f1261` int DEFAULT NULL,
+  `f1262` int DEFAULT NULL,
+  `f1263` int DEFAULT NULL,
+  `f1264` int DEFAULT NULL,
+  `f1265` int DEFAULT NULL,
+  `f1266` int DEFAULT NULL,
+  `f1267` int DEFAULT NULL,
+  `f1268` int DEFAULT NULL,
+  `f1269` int DEFAULT NULL,
+  `f1270` int DEFAULT NULL,
+  `f1271` int DEFAULT NULL,
+  `f1272` int DEFAULT NULL,
+  `f1273` int DEFAULT NULL,
+  `f1274` int DEFAULT NULL,
+  `f1275` int DEFAULT NULL,
+  `f1276` int DEFAULT NULL,
+  `f1277` int DEFAULT NULL,
+  `f1278` int DEFAULT NULL,
+  `f1279` int DEFAULT NULL,
+  `f1280` int DEFAULT NULL,
+  `f1281` int DEFAULT NULL,
+  `f1282` int DEFAULT NULL,
+  `f1283` int DEFAULT NULL,
+  `f1284` int DEFAULT NULL,
+  `f1285` int DEFAULT NULL,
+  `f1286` int DEFAULT NULL,
+  `f1287` int DEFAULT NULL,
+  `f1288` int DEFAULT NULL,
+  `f1289` int DEFAULT NULL,
+  `f1290` int DEFAULT NULL,
+  `f1291` int DEFAULT NULL,
+  `f1292` int DEFAULT NULL,
+  `f1293` int DEFAULT NULL,
+  `f1294` int DEFAULT NULL,
+  `f1295` int DEFAULT NULL,
+  `f1296` int DEFAULT NULL,
+  `f1297` int DEFAULT NULL,
+  `f1298` int DEFAULT NULL,
+  `f1299` int DEFAULT NULL,
+  `f1300` int DEFAULT NULL,
+  `f1301` int DEFAULT NULL,
+  `f1302` int DEFAULT NULL,
+  `f1303` int DEFAULT NULL,
+  `f1304` int DEFAULT NULL,
+  `f1305` int DEFAULT NULL,
+  `f1306` int DEFAULT NULL,
+  `f1307` int DEFAULT NULL,
+  `f1308` int DEFAULT NULL,
+  `f1309` int DEFAULT NULL,
+  `f1310` int DEFAULT NULL,
+  `f1311` int DEFAULT NULL,
+  `f1312` int DEFAULT NULL,
+  `f1313` int DEFAULT NULL,
+  `f1314` int DEFAULT NULL,
+  `f1315` int DEFAULT NULL,
+  `f1316` int DEFAULT NULL,
+  `f1317` int DEFAULT NULL,
+  `f1318` int DEFAULT NULL,
+  `f1319` int DEFAULT NULL,
+  `f1320` int DEFAULT NULL,
+  `f1321` int DEFAULT NULL,
+  `f1322` int DEFAULT NULL,
+  `f1323` int DEFAULT NULL,
+  `f1324` int DEFAULT NULL,
+  `f1325` int DEFAULT NULL,
+  `f1326` int DEFAULT NULL,
+  `f1327` int DEFAULT NULL,
+  `f1328` int DEFAULT NULL,
+  `f1329` int DEFAULT NULL,
+  `f1330` int DEFAULT NULL,
+  `f1331` int DEFAULT NULL,
+  `f1332` int DEFAULT NULL,
+  `f1333` int DEFAULT NULL,
+  `f1334` int DEFAULT NULL,
+  `f1335` int DEFAULT NULL,
+  `f1336` int DEFAULT NULL,
+  `f1337` int DEFAULT NULL,
+  `f1338` int DEFAULT NULL,
+  `f1339` int DEFAULT NULL,
+  `f1340` int DEFAULT NULL,
+  `f1341` int DEFAULT NULL,
+  `f1342` int DEFAULT NULL,
+  `f1343` int DEFAULT NULL,
+  `f1344` int DEFAULT NULL,
+  `f1345` int DEFAULT NULL,
+  `f1346` int DEFAULT NULL,
+  `f1347` int DEFAULT NULL,
+  `f1348` int DEFAULT NULL,
+  `f1349` int DEFAULT NULL,
+  `f1350` int DEFAULT NULL,
+  `f1351` int DEFAULT NULL,
+  `f1352` int DEFAULT NULL,
+  `f1353` int DEFAULT NULL,
+  `f1354` int DEFAULT NULL,
+  `f1355` int DEFAULT NULL,
+  `f1356` int DEFAULT NULL,
+  `f1357` int DEFAULT NULL,
+  `f1358` int DEFAULT NULL,
+  `f1359` int DEFAULT NULL,
+  `f1360` int DEFAULT NULL,
+  `f1361` int DEFAULT NULL,
+  `f1362` int DEFAULT NULL,
+  `f1363` int DEFAULT NULL,
+  `f1364` int DEFAULT NULL,
+  `f1365` int DEFAULT NULL,
+  `f1366` int DEFAULT NULL,
+  `f1367` int DEFAULT NULL,
+  `f1368` int DEFAULT NULL,
+  `f1369` int DEFAULT NULL,
+  `f1370` int DEFAULT NULL,
+  `f1371` int DEFAULT NULL,
+  `f1372` int DEFAULT NULL,
+  `f1373` int DEFAULT NULL,
+  `f1374` int DEFAULT NULL,
+  `f1375` int DEFAULT NULL,
+  `f1376` int DEFAULT NULL,
+  `f1377` int DEFAULT NULL,
+  `f1378` int DEFAULT NULL,
+  `f1379` int DEFAULT NULL,
+  `f1380` int DEFAULT NULL,
+  `f1381` int DEFAULT NULL,
+  `f1382` int DEFAULT NULL,
+  `f1383` int DEFAULT NULL,
+  `f1384` int DEFAULT NULL,
+  `f1385` int DEFAULT NULL,
+  `f1386` int DEFAULT NULL,
+  `f1387` int DEFAULT NULL,
+  `f1388` int DEFAULT NULL,
+  `f1389` int DEFAULT NULL,
+  `f1390` int DEFAULT NULL,
+  `f1391` int DEFAULT NULL,
+  `f1392` int DEFAULT NULL,
+  `f1393` int DEFAULT NULL,
+  `f1394` int DEFAULT NULL,
+  `f1395` int DEFAULT NULL,
+  `f1396` int DEFAULT NULL,
+  `f1397` int DEFAULT NULL,
+  `f1398` int DEFAULT NULL,
+  `f1399` int DEFAULT NULL,
+  `f1400` int DEFAULT NULL,
+  `f1401` int DEFAULT NULL,
+  `f1402` int DEFAULT NULL,
+  `f1403` int DEFAULT NULL,
+  `f1404` int DEFAULT NULL,
+  `f1405` int DEFAULT NULL,
+  `f1406` int DEFAULT NULL,
+  `f1407` int DEFAULT NULL,
+  `f1408` int DEFAULT NULL,
+  `f1409` int DEFAULT NULL,
+  `f1410` int DEFAULT NULL,
+  `f1411` int DEFAULT NULL,
+  `f1412` int DEFAULT NULL,
+  `f1413` int DEFAULT NULL,
+  `f1414` int DEFAULT NULL,
+  `f1415` int DEFAULT NULL,
+  `f1416` int DEFAULT NULL,
+  `f1417` int DEFAULT NULL,
+  `f1418` int DEFAULT NULL,
+  `f1419` int DEFAULT NULL,
+  `f1420` int DEFAULT NULL,
+  `f1421` int DEFAULT NULL,
+  `f1422` int DEFAULT NULL,
+  `f1423` int DEFAULT NULL,
+  `f1424` int DEFAULT NULL,
+  `f1425` int DEFAULT NULL,
+  `f1426` int DEFAULT NULL,
+  `f1427` int DEFAULT NULL,
+  `f1428` int DEFAULT NULL,
+  `f1429` int DEFAULT NULL,
+  `f1430` int DEFAULT NULL,
+  `f1431` int DEFAULT NULL,
+  `f1432` int DEFAULT NULL,
+  `f1433` int DEFAULT NULL,
+  `f1434` int DEFAULT NULL,
+  `f1435` int DEFAULT NULL,
+  `f1436` int DEFAULT NULL,
+  `f1437` int DEFAULT NULL,
+  `f1438` int DEFAULT NULL,
+  `f1439` int DEFAULT NULL,
+  `f1440` int DEFAULT NULL,
+  `f1441` int DEFAULT NULL,
+  `f1442` int DEFAULT NULL,
+  `f1443` int DEFAULT NULL,
+  `f1444` int DEFAULT NULL,
+  `f1445` int DEFAULT NULL,
+  `f1446` int DEFAULT NULL,
+  `f1447` int DEFAULT NULL,
+  `f1448` int DEFAULT NULL,
+  `f1449` int DEFAULT NULL,
+  `f1450` int DEFAULT NULL,
+  `f1451` int DEFAULT NULL,
+  `f1452` int DEFAULT NULL,
+  `f1453` int DEFAULT NULL,
+  `f1454` int DEFAULT NULL,
+  `f1455` int DEFAULT NULL,
+  `f1456` int DEFAULT NULL,
+  `f1457` int DEFAULT NULL,
+  `f1458` int DEFAULT NULL,
+  `f1459` int DEFAULT NULL,
+  `f1460` int DEFAULT NULL,
+  `f1461` int DEFAULT NULL,
+  `f1462` int DEFAULT NULL,
+  `f1463` int DEFAULT NULL,
+  `f1464` int DEFAULT NULL,
+  `f1465` int DEFAULT NULL,
+  `f1466` int DEFAULT NULL,
+  `f1467` int DEFAULT NULL,
+  `f1468` int DEFAULT NULL,
+  `f1469` int DEFAULT NULL,
+  `f1470` int DEFAULT NULL,
+  `f1471` int DEFAULT NULL,
+  `f1472` int DEFAULT NULL,
+  `f1473` int DEFAULT NULL,
+  `f1474` int DEFAULT NULL,
+  `f1475` int DEFAULT NULL,
+  `f1476` int DEFAULT NULL,
+  `f1477` int DEFAULT NULL,
+  `f1478` int DEFAULT NULL,
+  `f1479` int DEFAULT NULL,
+  `f1480` int DEFAULT NULL,
+  `f1481` int DEFAULT NULL,
+  `f1482` int DEFAULT NULL,
+  `f1483` int DEFAULT NULL,
+  `f1484` int DEFAULT NULL,
+  `f1485` int DEFAULT NULL,
+  `f1486` int DEFAULT NULL,
+  `f1487` int DEFAULT NULL,
+  `f1488` int DEFAULT NULL,
+  `f1489` int DEFAULT NULL,
+  `f1490` int DEFAULT NULL,
+  `f1491` int DEFAULT NULL,
+  `f1492` int DEFAULT NULL,
+  `f1493` int DEFAULT NULL,
+  `f1494` int DEFAULT NULL,
+  `f1495` int DEFAULT NULL,
+  `f1496` int DEFAULT NULL,
+  `f1497` int DEFAULT NULL,
+  `f1498` int DEFAULT NULL,
+  `f1499` int DEFAULT NULL,
+  `f1500` int DEFAULT NULL,
+  `f1501` int DEFAULT NULL,
+  `f1502` int DEFAULT NULL,
+  `f1503` int DEFAULT NULL,
+  `f1504` int DEFAULT NULL,
+  `f1505` int DEFAULT NULL,
+  `f1506` int DEFAULT NULL,
+  `f1507` int DEFAULT NULL,
+  `f1508` int DEFAULT NULL,
+  `f1509` int DEFAULT NULL,
+  `f1510` int DEFAULT NULL,
+  `f1511` int DEFAULT NULL,
+  `f1512` int DEFAULT NULL,
+  `f1513` int DEFAULT NULL,
+  `f1514` int DEFAULT NULL,
+  `f1515` int DEFAULT NULL,
+  `f1516` int DEFAULT NULL,
+  `f1517` int DEFAULT NULL,
+  `f1518` int DEFAULT NULL,
+  `f1519` int DEFAULT NULL,
+  `f1520` int DEFAULT NULL,
+  `f1521` int DEFAULT NULL,
+  `f1522` int DEFAULT NULL,
+  `f1523` int DEFAULT NULL,
+  `f1524` int DEFAULT NULL,
+  `f1525` int DEFAULT NULL,
+  `f1526` int DEFAULT NULL,
+  `f1527` int DEFAULT NULL,
+  `f1528` int DEFAULT NULL,
+  `f1529` int DEFAULT NULL,
+  `f1530` int DEFAULT NULL,
+  `f1531` int DEFAULT NULL,
+  `f1532` int DEFAULT NULL,
+  `f1533` int DEFAULT NULL,
+  `f1534` int DEFAULT NULL,
+  `f1535` int DEFAULT NULL,
+  `f1536` int DEFAULT NULL,
+  `f1537` int DEFAULT NULL,
+  `f1538` int DEFAULT NULL,
+  `f1539` int DEFAULT NULL,
+  `f1540` int DEFAULT NULL,
+  `f1541` int DEFAULT NULL,
+  `f1542` int DEFAULT NULL,
+  `f1543` int DEFAULT NULL,
+  `f1544` int DEFAULT NULL,
+  `f1545` int DEFAULT NULL,
+  `f1546` int DEFAULT NULL,
+  `f1547` int DEFAULT NULL,
+  `f1548` int DEFAULT NULL,
+  `f1549` int DEFAULT NULL,
+  `f1550` int DEFAULT NULL,
+  `f1551` int DEFAULT NULL,
+  `f1552` int DEFAULT NULL,
+  `f1553` int DEFAULT NULL,
+  `f1554` int DEFAULT NULL,
+  `f1555` int DEFAULT NULL,
+  `f1556` int DEFAULT NULL,
+  `f1557` int DEFAULT NULL,
+  `f1558` int DEFAULT NULL,
+  `f1559` int DEFAULT NULL,
+  `f1560` int DEFAULT NULL,
+  `f1561` int DEFAULT NULL,
+  `f1562` int DEFAULT NULL,
+  `f1563` int DEFAULT NULL,
+  `f1564` int DEFAULT NULL,
+  `f1565` int DEFAULT NULL,
+  `f1566` int DEFAULT NULL,
+  `f1567` int DEFAULT NULL,
+  `f1568` int DEFAULT NULL,
+  `f1569` int DEFAULT NULL,
+  `f1570` int DEFAULT NULL,
+  `f1571` int DEFAULT NULL,
+  `f1572` int DEFAULT NULL,
+  `f1573` int DEFAULT NULL,
+  `f1574` int DEFAULT NULL,
+  `f1575` int DEFAULT NULL,
+  `f1576` int DEFAULT NULL,
+  `f1577` int DEFAULT NULL,
+  `f1578` int DEFAULT NULL,
+  `f1579` int DEFAULT NULL,
+  `f1580` int DEFAULT NULL,
+  `f1581` int DEFAULT NULL,
+  `f1582` int DEFAULT NULL,
+  `f1583` int DEFAULT NULL,
+  `f1584` int DEFAULT NULL,
+  `f1585` int DEFAULT NULL,
+  `f1586` int DEFAULT NULL,
+  `f1587` int DEFAULT NULL,
+  `f1588` int DEFAULT NULL,
+  `f1589` int DEFAULT NULL,
+  `f1590` int DEFAULT NULL,
+  `f1591` int DEFAULT NULL,
+  `f1592` int DEFAULT NULL,
+  `f1593` int DEFAULT NULL,
+  `f1594` int DEFAULT NULL,
+  `f1595` int DEFAULT NULL,
+  `f1596` int DEFAULT NULL,
+  `f1597` int DEFAULT NULL,
+  `f1598` int DEFAULT NULL,
+  `f1599` int DEFAULT NULL,
+  `f1600` int DEFAULT NULL,
+  `f1601` int DEFAULT NULL,
+  `f1602` int DEFAULT NULL,
+  `f1603` int DEFAULT NULL,
+  `f1604` int DEFAULT NULL,
+  `f1605` int DEFAULT NULL,
+  `f1606` int DEFAULT NULL,
+  `f1607` int DEFAULT NULL,
+  `f1608` int DEFAULT NULL,
+  `f1609` int DEFAULT NULL,
+  `f1610` int DEFAULT NULL,
+  `f1611` int DEFAULT NULL,
+  `f1612` int DEFAULT NULL,
+  `f1613` int DEFAULT NULL,
+  `f1614` int DEFAULT NULL,
+  `f1615` int DEFAULT NULL,
+  `f1616` int DEFAULT NULL,
+  `f1617` int DEFAULT NULL,
+  `f1618` int DEFAULT NULL,
+  `f1619` int DEFAULT NULL,
+  `f1620` int DEFAULT NULL,
+  `f1621` int DEFAULT NULL,
+  `f1622` int DEFAULT NULL,
+  `f1623` int DEFAULT NULL,
+  `f1624` int DEFAULT NULL,
+  `f1625` int DEFAULT NULL,
+  `f1626` int DEFAULT NULL,
+  `f1627` int DEFAULT NULL,
+  `f1628` int DEFAULT NULL,
+  `f1629` int DEFAULT NULL,
+  `f1630` int DEFAULT NULL,
+  `f1631` int DEFAULT NULL,
+  `f1632` int DEFAULT NULL,
+  `f1633` int DEFAULT NULL,
+  `f1634` int DEFAULT NULL,
+  `f1635` int DEFAULT NULL,
+  `f1636` int DEFAULT NULL,
+  `f1637` int DEFAULT NULL,
+  `f1638` int DEFAULT NULL,
+  `f1639` int DEFAULT NULL,
+  `f1640` int DEFAULT NULL,
+  `f1641` int DEFAULT NULL,
+  `f1642` int DEFAULT NULL,
+  `f1643` int DEFAULT NULL,
+  `f1644` int DEFAULT NULL,
+  `f1645` int DEFAULT NULL,
+  `f1646` int DEFAULT NULL,
+  `f1647` int DEFAULT NULL,
+  `f1648` int DEFAULT NULL,
+  `f1649` int DEFAULT NULL,
+  `f1650` int DEFAULT NULL,
+  `f1651` int DEFAULT NULL,
+  `f1652` int DEFAULT NULL,
+  `f1653` int DEFAULT NULL,
+  `f1654` int DEFAULT NULL,
+  `f1655` int DEFAULT NULL,
+  `f1656` int DEFAULT NULL,
+  `f1657` int DEFAULT NULL,
+  `f1658` int DEFAULT NULL,
+  `f1659` int DEFAULT NULL,
+  `f1660` int DEFAULT NULL,
+  `f1661` int DEFAULT NULL,
+  `f1662` int DEFAULT NULL,
+  `f1663` int DEFAULT NULL,
+  `f1664` int DEFAULT NULL,
+  `f1665` int DEFAULT NULL,
+  `f1666` int DEFAULT NULL,
+  `f1667` int DEFAULT NULL,
+  `f1668` int DEFAULT NULL,
+  `f1669` int DEFAULT NULL,
+  `f1670` int DEFAULT NULL,
+  `f1671` int DEFAULT NULL,
+  `f1672` int DEFAULT NULL,
+  `f1673` int DEFAULT NULL,
+  `f1674` int DEFAULT NULL,
+  `f1675` int DEFAULT NULL,
+  `f1676` int DEFAULT NULL,
+  `f1677` int DEFAULT NULL,
+  `f1678` int DEFAULT NULL,
+  `f1679` int DEFAULT NULL,
+  `f1680` int DEFAULT NULL,
+  `f1681` int DEFAULT NULL,
+  `f1682` int DEFAULT NULL,
+  `f1683` int DEFAULT NULL,
+  `f1684` int DEFAULT NULL,
+  `f1685` int DEFAULT NULL,
+  `f1686` int DEFAULT NULL,
+  `f1687` int DEFAULT NULL,
+  `f1688` int DEFAULT NULL,
+  `f1689` int DEFAULT NULL,
+  `f1690` int DEFAULT NULL,
+  `f1691` int DEFAULT NULL,
+  `f1692` int DEFAULT NULL,
+  `f1693` int DEFAULT NULL,
+  `f1694` int DEFAULT NULL,
+  `f1695` int DEFAULT NULL,
+  `f1696` int DEFAULT NULL,
+  `f1697` int DEFAULT NULL,
+  `f1698` int DEFAULT NULL,
+  `f1699` int DEFAULT NULL,
+  `f1700` int DEFAULT NULL,
+  `f1701` int DEFAULT NULL,
+  `f1702` int DEFAULT NULL,
+  `f1703` int DEFAULT NULL,
+  `f1704` int DEFAULT NULL,
+  `f1705` int DEFAULT NULL,
+  `f1706` int DEFAULT NULL,
+  `f1707` int DEFAULT NULL,
+  `f1708` int DEFAULT NULL,
+  `f1709` int DEFAULT NULL,
+  `f1710` int DEFAULT NULL,
+  `f1711` int DEFAULT NULL,
+  `f1712` int DEFAULT NULL,
+  `f1713` int DEFAULT NULL,
+  `f1714` int DEFAULT NULL,
+  `f1715` int DEFAULT NULL,
+  `f1716` int DEFAULT NULL,
+  `f1717` int DEFAULT NULL,
+  `f1718` int DEFAULT NULL,
+  `f1719` int DEFAULT NULL,
+  `f1720` int DEFAULT NULL,
+  `f1721` int DEFAULT NULL,
+  `f1722` int DEFAULT NULL,
+  `f1723` int DEFAULT NULL,
+  `f1724` int DEFAULT NULL,
+  `f1725` int DEFAULT NULL,
+  `f1726` int DEFAULT NULL,
+  `f1727` int DEFAULT NULL,
+  `f1728` int DEFAULT NULL,
+  `f1729` int DEFAULT NULL,
+  `f1730` int DEFAULT NULL,
+  `f1731` int DEFAULT NULL,
+  `f1732` int DEFAULT NULL,
+  `f1733` int DEFAULT NULL,
+  `f1734` int DEFAULT NULL,
+  `f1735` int DEFAULT NULL,
+  `f1736` int DEFAULT NULL,
+  `f1737` int DEFAULT NULL,
+  `f1738` int DEFAULT NULL,
+  `f1739` int DEFAULT NULL,
+  `f1740` int DEFAULT NULL,
+  `f1741` int DEFAULT NULL,
+  `f1742` int DEFAULT NULL,
+  `f1743` int DEFAULT NULL,
+  `f1744` int DEFAULT NULL,
+  `f1745` int DEFAULT NULL,
+  `f1746` int DEFAULT NULL,
+  `f1747` int DEFAULT NULL,
+  `f1748` int DEFAULT NULL,
+  `f1749` int DEFAULT NULL,
+  `f1750` int DEFAULT NULL,
+  `f1751` int DEFAULT NULL,
+  `f1752` int DEFAULT NULL,
+  `f1753` int DEFAULT NULL,
+  `f1754` int DEFAULT NULL,
+  `f1755` int DEFAULT NULL,
+  `f1756` int DEFAULT NULL,
+  `f1757` int DEFAULT NULL,
+  `f1758` int DEFAULT NULL,
+  `f1759` int DEFAULT NULL,
+  `f1760` int DEFAULT NULL,
+  `f1761` int DEFAULT NULL,
+  `f1762` int DEFAULT NULL,
+  `f1763` int DEFAULT NULL,
+  `f1764` int DEFAULT NULL,
+  `f1765` int DEFAULT NULL,
+  `f1766` int DEFAULT NULL,
+  `f1767` int DEFAULT NULL,
+  `f1768` int DEFAULT NULL,
+  `f1769` int DEFAULT NULL,
+  `f1770` int DEFAULT NULL,
+  `f1771` int DEFAULT NULL,
+  `f1772` int DEFAULT NULL,
+  `f1773` int DEFAULT NULL,
+  `f1774` int DEFAULT NULL,
+  `f1775` int DEFAULT NULL,
+  `f1776` int DEFAULT NULL,
+  `f1777` int DEFAULT NULL,
+  `f1778` int DEFAULT NULL,
+  `f1779` int DEFAULT NULL,
+  `f1780` int DEFAULT NULL,
+  `f1781` int DEFAULT NULL,
+  `f1782` int DEFAULT NULL,
+  `f1783` int DEFAULT NULL,
+  `f1784` int DEFAULT NULL,
+  `f1785` int DEFAULT NULL,
+  `f1786` int DEFAULT NULL,
+  `f1787` int DEFAULT NULL,
+  `f1788` int DEFAULT NULL,
+  `f1789` int DEFAULT NULL,
+  `f1790` int DEFAULT NULL,
+  `f1791` int DEFAULT NULL,
+  `f1792` int DEFAULT NULL,
+  `f1793` int DEFAULT NULL,
+  `f1794` int DEFAULT NULL,
+  `f1795` int DEFAULT NULL,
+  `f1796` int DEFAULT NULL,
+  `f1797` int DEFAULT NULL,
+  `f1798` int DEFAULT NULL,
+  `f1799` int DEFAULT NULL,
+  `f1800` int DEFAULT NULL,
+  `f1801` int DEFAULT NULL,
+  `f1802` int DEFAULT NULL,
+  `f1803` int DEFAULT NULL,
+  `f1804` int DEFAULT NULL,
+  `f1805` int DEFAULT NULL,
+  `f1806` int DEFAULT NULL,
+  `f1807` int DEFAULT NULL,
+  `f1808` int DEFAULT NULL,
+  `f1809` int DEFAULT NULL,
+  `f1810` int DEFAULT NULL,
+  `f1811` int DEFAULT NULL,
+  `f1812` int DEFAULT NULL,
+  `f1813` int DEFAULT NULL,
+  `f1814` int DEFAULT NULL,
+  `f1815` int DEFAULT NULL,
+  `f1816` int DEFAULT NULL,
+  `f1817` int DEFAULT NULL,
+  `f1818` int DEFAULT NULL,
+  `f1819` int DEFAULT NULL,
+  `f1820` int DEFAULT NULL,
+  `f1821` int DEFAULT NULL,
+  `f1822` int DEFAULT NULL,
+  `f1823` int DEFAULT NULL,
+  `f1824` int DEFAULT NULL,
+  `f1825` int DEFAULT NULL,
+  `f1826` int DEFAULT NULL,
+  `f1827` int DEFAULT NULL,
+  `f1828` int DEFAULT NULL,
+  `f1829` int DEFAULT NULL,
+  `f1830` int DEFAULT NULL,
+  `f1831` int DEFAULT NULL,
+  `f1832` int DEFAULT NULL,
+  `f1833` int DEFAULT NULL,
+  `f1834` int DEFAULT NULL,
+  `f1835` int DEFAULT NULL,
+  `f1836` int DEFAULT NULL,
+  `f1837` int DEFAULT NULL,
+  `f1838` int DEFAULT NULL,
+  `f1839` int DEFAULT NULL,
+  `f1840` int DEFAULT NULL,
+  `f1841` int DEFAULT NULL,
+  `f1842` int DEFAULT NULL,
+  `f1843` int DEFAULT NULL,
+  `f1844` int DEFAULT NULL,
+  `f1845` int DEFAULT NULL,
+  `f1846` int DEFAULT NULL,
+  `f1847` int DEFAULT NULL,
+  `f1848` int DEFAULT NULL,
+  `f1849` int DEFAULT NULL,
+  `f1850` int DEFAULT NULL,
+  `f1851` int DEFAULT NULL,
+  `f1852` int DEFAULT NULL,
+  `f1853` int DEFAULT NULL,
+  `f1854` int DEFAULT NULL,
+  `f1855` int DEFAULT NULL,
+  `f1856` int DEFAULT NULL,
+  `f1857` int DEFAULT NULL,
+  `f1858` int DEFAULT NULL,
+  `f1859` int DEFAULT NULL,
+  `f1860` int DEFAULT NULL,
+  `f1861` int DEFAULT NULL,
+  `f1862` int DEFAULT NULL,
+  `f1863` int DEFAULT NULL,
+  `f1864` int DEFAULT NULL,
+  `f1865` int DEFAULT NULL,
+  `f1866` int DEFAULT NULL,
+  `f1867` int DEFAULT NULL,
+  `f1868` int DEFAULT NULL,
+  `f1869` int DEFAULT NULL,
+  `f1870` int DEFAULT NULL,
+  `f1871` int DEFAULT NULL,
+  `f1872` int DEFAULT NULL,
+  `f1873` int DEFAULT NULL,
+  `f1874` int DEFAULT NULL,
+  `f1875` int DEFAULT NULL,
+  `f1876` int DEFAULT NULL,
+  `f1877` int DEFAULT NULL,
+  `f1878` int DEFAULT NULL,
+  `f1879` int DEFAULT NULL,
+  `f1880` int DEFAULT NULL,
+  `f1881` int DEFAULT NULL,
+  `f1882` int DEFAULT NULL,
+  `f1883` int DEFAULT NULL,
+  `f1884` int DEFAULT NULL,
+  `f1885` int DEFAULT NULL,
+  `f1886` int DEFAULT NULL,
+  `f1887` int DEFAULT NULL,
+  `f1888` int DEFAULT NULL,
+  `f1889` int DEFAULT NULL,
+  `f1890` int DEFAULT NULL,
+  `f1891` int DEFAULT NULL,
+  `f1892` int DEFAULT NULL,
+  `f1893` int DEFAULT NULL,
+  `f1894` int DEFAULT NULL,
+  `f1895` int DEFAULT NULL,
+  `f1896` int DEFAULT NULL,
+  `f1897` int DEFAULT NULL,
+  `f1898` int DEFAULT NULL,
+  `f1899` int DEFAULT NULL,
+  `f1900` int DEFAULT NULL,
+  `f1901` int DEFAULT NULL,
+  `f1902` int DEFAULT NULL,
+  `f1903` int DEFAULT NULL,
+  `f1904` int DEFAULT NULL,
+  `f1905` int DEFAULT NULL,
+  `f1906` int DEFAULT NULL,
+  `f1907` int DEFAULT NULL,
+  `f1908` int DEFAULT NULL,
+  `f1909` int DEFAULT NULL,
+  `f1910` int DEFAULT NULL,
+  `f1911` int DEFAULT NULL,
+  `f1912` int DEFAULT NULL,
+  `f1913` int DEFAULT NULL,
+  `f1914` int DEFAULT NULL,
+  `f1915` int DEFAULT NULL,
+  `f1916` int DEFAULT NULL,
+  `f1917` int DEFAULT NULL,
+  `f1918` int DEFAULT NULL,
+  `f1919` int DEFAULT NULL,
+  `f1920` int DEFAULT NULL,
+  `f1921` int DEFAULT NULL,
+  `f1922` int DEFAULT NULL,
+  `f1923` int DEFAULT NULL,
+  `f1924` int DEFAULT NULL,
+  `f1925` int DEFAULT NULL,
+  `f1926` int DEFAULT NULL,
+  `f1927` int DEFAULT NULL,
+  `f1928` int DEFAULT NULL,
+  `f1929` int DEFAULT NULL,
+  `f1930` int DEFAULT NULL,
+  `f1931` int DEFAULT NULL,
+  `f1932` int DEFAULT NULL,
+  `f1933` int DEFAULT NULL,
+  `f1934` int DEFAULT NULL,
+  `f1935` int DEFAULT NULL,
+  `f1936` int DEFAULT NULL,
+  `f1937` int DEFAULT NULL,
+  `f1938` int DEFAULT NULL,
+  `f1939` int DEFAULT NULL,
+  `f1940` int DEFAULT NULL,
+  `f1941` int DEFAULT NULL,
+  `f1942` int DEFAULT NULL,
+  `f1943` int DEFAULT NULL,
+  `f1944` int DEFAULT NULL,
+  `f1945` int DEFAULT NULL,
+  `f1946` int DEFAULT NULL,
+  `f1947` int DEFAULT NULL,
+  `f1948` int DEFAULT NULL,
+  `f1949` int DEFAULT NULL,
+  `f1950` int DEFAULT NULL,
+  `f1951` int DEFAULT NULL,
+  `f1952` int DEFAULT NULL,
+  `f1953` int DEFAULT NULL,
+  `f1954` int DEFAULT NULL,
+  `f1955` int DEFAULT NULL,
+  `f1956` int DEFAULT NULL,
+  `f1957` int DEFAULT NULL,
+  `f1958` int DEFAULT NULL,
+  `f1959` int DEFAULT NULL,
+  `f1960` int DEFAULT NULL,
+  `f1961` int DEFAULT NULL,
+  `f1962` int DEFAULT NULL,
+  `f1963` int DEFAULT NULL,
+  `f1964` int DEFAULT NULL,
+  `f1965` int DEFAULT NULL,
+  `f1966` int DEFAULT NULL,
+  `f1967` int DEFAULT NULL,
+  `f1968` int DEFAULT NULL,
+  `f1969` int DEFAULT NULL,
+  `f1970` int DEFAULT NULL,
+  `f1971` int DEFAULT NULL,
+  `f1972` int DEFAULT NULL,
+  `f1973` int DEFAULT NULL,
+  `f1974` int DEFAULT NULL,
+  `f1975` int DEFAULT NULL,
+  `f1976` int DEFAULT NULL,
+  `f1977` int DEFAULT NULL,
+  `f1978` int DEFAULT NULL,
+  `f1979` int DEFAULT NULL,
+  `f1980` int DEFAULT NULL,
+  `f1981` int DEFAULT NULL,
+  `f1982` int DEFAULT NULL,
+  `f1983` int DEFAULT NULL,
+  `f1984` int DEFAULT NULL,
+  `f1985` int DEFAULT NULL,
+  `f1986` int DEFAULT NULL,
+  `f1987` int DEFAULT NULL,
+  `f1988` int DEFAULT NULL,
+  `f1989` int DEFAULT NULL,
+  `f1990` int DEFAULT NULL,
+  `f1991` int DEFAULT NULL,
+  `f1992` int DEFAULT NULL,
+  `f1993` int DEFAULT NULL,
+  `f1994` int DEFAULT NULL,
+  `f1995` int DEFAULT NULL,
+  `f1996` int DEFAULT NULL,
+  `f1997` int DEFAULT NULL,
+  `f1998` int DEFAULT NULL,
+  `f1999` int DEFAULT NULL,
+  `f2000` int DEFAULT NULL,
+  `f2001` int DEFAULT NULL,
+  `f2002` int DEFAULT NULL,
+  `f2003` int DEFAULT NULL,
+  `f2004` int DEFAULT NULL,
+  `f2005` int DEFAULT NULL,
+  `f2006` int DEFAULT NULL,
+  `f2007` int DEFAULT NULL,
+  `f2008` int DEFAULT NULL,
+  `f2009` int DEFAULT NULL,
+  `f2010` int DEFAULT NULL,
+  `f2011` int DEFAULT NULL,
+  `f2012` int DEFAULT NULL,
+  `f2013` int DEFAULT NULL,
+  `f2014` int DEFAULT NULL,
+  `f2015` int DEFAULT NULL,
+  `f2016` int DEFAULT NULL,
+  `f2017` int DEFAULT NULL,
+  `f2018` int DEFAULT NULL,
+  `f2019` int DEFAULT NULL,
+  `f2020` int DEFAULT NULL,
+  `f2021` int DEFAULT NULL,
+  `f2022` int DEFAULT NULL,
+  `f2023` int DEFAULT NULL,
+  `f2024` int DEFAULT NULL,
+  `f2025` int DEFAULT NULL,
+  `f2026` int DEFAULT NULL,
+  `f2027` int DEFAULT NULL,
+  `f2028` int DEFAULT NULL,
+  `f2029` int DEFAULT NULL,
+  `f2030` int DEFAULT NULL,
+  `f2031` int DEFAULT NULL,
+  `f2032` int DEFAULT NULL,
+  `f2033` int DEFAULT NULL,
+  `f2034` int DEFAULT NULL,
+  `f2035` int DEFAULT NULL,
+  `f2036` int DEFAULT NULL,
+  `f2037` int DEFAULT NULL,
+  `f2038` int DEFAULT NULL,
+  `f2039` int DEFAULT NULL,
+  `f2040` int DEFAULT NULL,
+  `f2041` int DEFAULT NULL,
+  `f2042` int DEFAULT NULL,
+  `f2043` int DEFAULT NULL,
+  `f2044` int DEFAULT NULL,
+  `f2045` int DEFAULT NULL,
+  `f2046` int DEFAULT NULL,
+  `f2047` int DEFAULT NULL,
+  `f2048` int DEFAULT NULL,
+  `f2049` int DEFAULT NULL,
+  `f2050` int DEFAULT NULL,
+  `f2051` int DEFAULT NULL,
+  `f2052` int DEFAULT NULL,
+  `f2053` int DEFAULT NULL,
+  `f2054` int DEFAULT NULL,
+  `f2055` int DEFAULT NULL,
+  `f2056` int DEFAULT NULL,
+  `f2057` int DEFAULT NULL,
+  `f2058` int DEFAULT NULL,
+  `f2059` int DEFAULT NULL,
+  `f2060` int DEFAULT NULL,
+  `f2061` int DEFAULT NULL,
+  `f2062` int DEFAULT NULL,
+  `f2063` int DEFAULT NULL,
+  `f2064` int DEFAULT NULL,
+  `f2065` int DEFAULT NULL,
+  `f2066` int DEFAULT NULL,
+  `f2067` int DEFAULT NULL,
+  `f2068` int DEFAULT NULL,
+  `f2069` int DEFAULT NULL,
+  `f2070` int DEFAULT NULL,
+  `f2071` int DEFAULT NULL,
+  `f2072` int DEFAULT NULL,
+  `f2073` int DEFAULT NULL,
+  `f2074` int DEFAULT NULL,
+  `f2075` int DEFAULT NULL,
+  `f2076` int DEFAULT NULL,
+  `f2077` int DEFAULT NULL,
+  `f2078` int DEFAULT NULL,
+  `f2079` int DEFAULT NULL,
+  `f2080` int DEFAULT NULL,
+  `f2081` int DEFAULT NULL,
+  `f2082` int DEFAULT NULL,
+  `f2083` int DEFAULT NULL,
+  `f2084` int DEFAULT NULL,
+  `f2085` int DEFAULT NULL,
+  `f2086` int DEFAULT NULL,
+  `f2087` int DEFAULT NULL,
+  `f2088` int DEFAULT NULL,
+  `f2089` int DEFAULT NULL,
+  `f2090` int DEFAULT NULL,
+  `f2091` int DEFAULT NULL,
+  `f2092` int DEFAULT NULL,
+  `f2093` int DEFAULT NULL,
+  `f2094` int DEFAULT NULL,
+  `f2095` int DEFAULT NULL,
+  `f2096` int DEFAULT NULL,
+  `f2097` int DEFAULT NULL,
+  `f2098` int DEFAULT NULL,
+  `f2099` int DEFAULT NULL,
+  `f2100` int DEFAULT NULL,
+  `f2101` int DEFAULT NULL,
+  `f2102` int DEFAULT NULL,
+  `f2103` int DEFAULT NULL,
+  `f2104` int DEFAULT NULL,
+  `f2105` int DEFAULT NULL,
+  `f2106` int DEFAULT NULL,
+  `f2107` int DEFAULT NULL,
+  `f2108` int DEFAULT NULL,
+  `f2109` int DEFAULT NULL,
+  `f2110` int DEFAULT NULL,
+  `f2111` int DEFAULT NULL,
+  `f2112` int DEFAULT NULL,
+  `f2113` int DEFAULT NULL,
+  `f2114` int DEFAULT NULL,
+  `f2115` int DEFAULT NULL,
+  `f2116` int DEFAULT NULL,
+  `f2117` int DEFAULT NULL,
+  `f2118` int DEFAULT NULL,
+  `f2119` int DEFAULT NULL,
+  `f2120` int DEFAULT NULL,
+  `f2121` int DEFAULT NULL,
+  `f2122` int DEFAULT NULL,
+  `f2123` int DEFAULT NULL,
+  `f2124` int DEFAULT NULL,
+  `f2125` int DEFAULT NULL,
+  `f2126` int DEFAULT NULL,
+  `f2127` int DEFAULT NULL,
+  `f2128` int DEFAULT NULL,
+  `f2129` int DEFAULT NULL,
+  `f2130` int DEFAULT NULL,
+  `f2131` int DEFAULT NULL,
+  `f2132` int DEFAULT NULL,
+  `f2133` int DEFAULT NULL,
+  `f2134` int DEFAULT NULL,
+  `f2135` int DEFAULT NULL,
+  `f2136` int DEFAULT NULL,
+  `f2137` int DEFAULT NULL,
+  `f2138` int DEFAULT NULL,
+  `f2139` int DEFAULT NULL,
+  `f2140` int DEFAULT NULL,
+  `f2141` int DEFAULT NULL,
+  `f2142` int DEFAULT NULL,
+  `f2143` int DEFAULT NULL,
+  `f2144` int DEFAULT NULL,
+  `f2145` int DEFAULT NULL,
+  `f2146` int DEFAULT NULL,
+  `f2147` int DEFAULT NULL,
+  `f2148` int DEFAULT NULL,
+  `f2149` int DEFAULT NULL,
+  `f2150` int DEFAULT NULL,
+  `f2151` int DEFAULT NULL,
+  `f2152` int DEFAULT NULL,
+  `f2153` int DEFAULT NULL,
+  `f2154` int DEFAULT NULL,
+  `f2155` int DEFAULT NULL,
+  `f2156` int DEFAULT NULL,
+  `f2157` int DEFAULT NULL,
+  `f2158` int DEFAULT NULL,
+  `f2159` int DEFAULT NULL,
+  `f2160` int DEFAULT NULL,
+  `f2161` int DEFAULT NULL,
+  `f2162` int DEFAULT NULL,
+  `f2163` int DEFAULT NULL,
+  `f2164` int DEFAULT NULL,
+  `f2165` int DEFAULT NULL,
+  `f2166` int DEFAULT NULL,
+  `f2167` int DEFAULT NULL,
+  `f2168` int DEFAULT NULL,
+  `f2169` int DEFAULT NULL,
+  `f2170` int DEFAULT NULL,
+  `f2171` int DEFAULT NULL,
+  `f2172` int DEFAULT NULL,
+  `f2173` int DEFAULT NULL,
+  `f2174` int DEFAULT NULL,
+  `f2175` int DEFAULT NULL,
+  `f2176` int DEFAULT NULL,
+  `f2177` int DEFAULT NULL,
+  `f2178` int DEFAULT NULL,
+  `f2179` int DEFAULT NULL,
+  `f2180` int DEFAULT NULL,
+  `f2181` int DEFAULT NULL,
+  `f2182` int DEFAULT NULL,
+  `f2183` int DEFAULT NULL,
+  `f2184` int DEFAULT NULL,
+  `f2185` int DEFAULT NULL,
+  `f2186` int DEFAULT NULL,
+  `f2187` int DEFAULT NULL,
+  `f2188` int DEFAULT NULL,
+  `f2189` int DEFAULT NULL,
+  `f2190` int DEFAULT NULL,
+  `f2191` int DEFAULT NULL,
+  `f2192` int DEFAULT NULL,
+  `f2193` int DEFAULT NULL,
+  `f2194` int DEFAULT NULL,
+  `f2195` int DEFAULT NULL,
+  `f2196` int DEFAULT NULL,
+  `f2197` int DEFAULT NULL,
+  `f2198` int DEFAULT NULL,
+  `f2199` int DEFAULT NULL,
+  `f2200` int DEFAULT NULL,
+  `f2201` int DEFAULT NULL,
+  `f2202` int DEFAULT NULL,
+  `f2203` int DEFAULT NULL,
+  `f2204` int DEFAULT NULL,
+  `f2205` int DEFAULT NULL,
+  `f2206` int DEFAULT NULL,
+  `f2207` int DEFAULT NULL,
+  `f2208` int DEFAULT NULL,
+  `f2209` int DEFAULT NULL,
+  `f2210` int DEFAULT NULL,
+  `f2211` int DEFAULT NULL,
+  `f2212` int DEFAULT NULL,
+  `f2213` int DEFAULT NULL,
+  `f2214` int DEFAULT NULL,
+  `f2215` int DEFAULT NULL,
+  `f2216` int DEFAULT NULL,
+  `f2217` int DEFAULT NULL,
+  `f2218` int DEFAULT NULL,
+  `f2219` int DEFAULT NULL,
+  `f2220` int DEFAULT NULL,
+  `f2221` int DEFAULT NULL,
+  `f2222` int DEFAULT NULL,
+  `f2223` int DEFAULT NULL,
+  `f2224` int DEFAULT NULL,
+  `f2225` int DEFAULT NULL,
+  `f2226` int DEFAULT NULL,
+  `f2227` int DEFAULT NULL,
+  `f2228` int DEFAULT NULL,
+  `f2229` int DEFAULT NULL,
+  `f2230` int DEFAULT NULL,
+  `f2231` int DEFAULT NULL,
+  `f2232` int DEFAULT NULL,
+  `f2233` int DEFAULT NULL,
+  `f2234` int DEFAULT NULL,
+  `f2235` int DEFAULT NULL,
+  `f2236` int DEFAULT NULL,
+  `f2237` int DEFAULT NULL,
+  `f2238` int DEFAULT NULL,
+  `f2239` int DEFAULT NULL,
+  `f2240` int DEFAULT NULL,
+  `f2241` int DEFAULT NULL,
+  `f2242` int DEFAULT NULL,
+  `f2243` int DEFAULT NULL,
+  `f2244` int DEFAULT NULL,
+  `f2245` int DEFAULT NULL,
+  `f2246` int DEFAULT NULL,
+  `f2247` int DEFAULT NULL,
+  `f2248` int DEFAULT NULL,
+  `f2249` int DEFAULT NULL,
+  `f2250` int DEFAULT NULL,
+  `f2251` int DEFAULT NULL,
+  `f2252` int DEFAULT NULL,
+  `f2253` int DEFAULT NULL,
+  `f2254` int DEFAULT NULL,
+  `f2255` int DEFAULT NULL,
+  `f2256` int DEFAULT NULL,
+  `f2257` int DEFAULT NULL,
+  `f2258` int DEFAULT NULL,
+  `f2259` int DEFAULT NULL,
+  `f2260` int DEFAULT NULL,
+  `f2261` int DEFAULT NULL,
+  `f2262` int DEFAULT NULL,
+  `f2263` int DEFAULT NULL,
+  `f2264` int DEFAULT NULL,
+  `f2265` int DEFAULT NULL,
+  `f2266` int DEFAULT NULL,
+  `f2267` int DEFAULT NULL,
+  `f2268` int DEFAULT NULL,
+  `f2269` int DEFAULT NULL,
+  `f2270` int DEFAULT NULL,
+  `f2271` int DEFAULT NULL,
+  `f2272` int DEFAULT NULL,
+  `f2273` int DEFAULT NULL,
+  `f2274` int DEFAULT NULL,
+  `f2275` int DEFAULT NULL,
+  `f2276` int DEFAULT NULL,
+  `f2277` int DEFAULT NULL,
+  `f2278` int DEFAULT NULL,
+  `f2279` int DEFAULT NULL,
+  `f2280` int DEFAULT NULL,
+  `f2281` int DEFAULT NULL,
+  `f2282` int DEFAULT NULL,
+  `f2283` int DEFAULT NULL,
+  `f2284` int DEFAULT NULL,
+  `f2285` int DEFAULT NULL,
+  `f2286` int DEFAULT NULL,
+  `f2287` int DEFAULT NULL,
+  `f2288` int DEFAULT NULL,
+  `f2289` int DEFAULT NULL,
+  `f2290` int DEFAULT NULL,
+  `f2291` int DEFAULT NULL,
+  `f2292` int DEFAULT NULL,
+  `f2293` int DEFAULT NULL,
+  `f2294` int DEFAULT NULL,
+  `f2295` int DEFAULT NULL,
+  `f2296` int DEFAULT NULL,
+  `f2297` int DEFAULT NULL,
+  `f2298` int DEFAULT NULL,
+  `f2299` int DEFAULT NULL,
+  `f2300` int DEFAULT NULL,
+  `f2301` int DEFAULT NULL,
+  `f2302` int DEFAULT NULL,
+  `f2303` int DEFAULT NULL,
+  `f2304` int DEFAULT NULL,
+  `f2305` int DEFAULT NULL,
+  `f2306` int DEFAULT NULL,
+  `f2307` int DEFAULT NULL,
+  `f2308` int DEFAULT NULL,
+  `f2309` int DEFAULT NULL,
+  `f2310` int DEFAULT NULL,
+  `f2311` int DEFAULT NULL,
+  `f2312` int DEFAULT NULL,
+  `f2313` int DEFAULT NULL,
+  `f2314` int DEFAULT NULL,
+  `f2315` int DEFAULT NULL,
+  `f2316` int DEFAULT NULL,
+  `f2317` int DEFAULT NULL,
+  `f2318` int DEFAULT NULL,
+  `f2319` int DEFAULT NULL,
+  `f2320` int DEFAULT NULL,
+  `f2321` int DEFAULT NULL,
+  `f2322` int DEFAULT NULL,
+  `f2323` int DEFAULT NULL,
+  `f2324` int DEFAULT NULL,
+  `f2325` int DEFAULT NULL,
+  `f2326` int DEFAULT NULL,
+  `f2327` int DEFAULT NULL,
+  `f2328` int DEFAULT NULL,
+  `f2329` int DEFAULT NULL,
+  `f2330` int DEFAULT NULL,
+  `f2331` int DEFAULT NULL,
+  `f2332` int DEFAULT NULL,
+  `f2333` int DEFAULT NULL,
+  `f2334` int DEFAULT NULL,
+  `f2335` int DEFAULT NULL,
+  `f2336` int DEFAULT NULL,
+  `f2337` int DEFAULT NULL,
+  `f2338` int DEFAULT NULL,
+  `f2339` int DEFAULT NULL,
+  `f2340` int DEFAULT NULL,
+  `f2341` int DEFAULT NULL,
+  `f2342` int DEFAULT NULL,
+  `f2343` int DEFAULT NULL,
+  `f2344` int DEFAULT NULL,
+  `f2345` int DEFAULT NULL,
+  `f2346` int DEFAULT NULL,
+  `f2347` int DEFAULT NULL,
+  `f2348` int DEFAULT NULL,
+  `f2349` int DEFAULT NULL,
+  `f2350` int DEFAULT NULL,
+  `f2351` int DEFAULT NULL,
+  `f2352` int DEFAULT NULL,
+  `f2353` int DEFAULT NULL,
+  `f2354` int DEFAULT NULL,
+  `f2355` int DEFAULT NULL,
+  `f2356` int DEFAULT NULL,
+  `f2357` int DEFAULT NULL,
+  `f2358` int DEFAULT NULL,
+  `f2359` int DEFAULT NULL,
+  `f2360` int DEFAULT NULL,
+  `f2361` int DEFAULT NULL,
+  `f2362` int DEFAULT NULL,
+  `f2363` int DEFAULT NULL,
+  `f2364` int DEFAULT NULL,
+  `f2365` int DEFAULT NULL,
+  `f2366` int DEFAULT NULL,
+  `f2367` int DEFAULT NULL,
+  `f2368` int DEFAULT NULL,
+  `f2369` int DEFAULT NULL,
+  `f2370` int DEFAULT NULL,
+  `f2371` int DEFAULT NULL,
+  `f2372` int DEFAULT NULL,
+  `f2373` int DEFAULT NULL,
+  `f2374` int DEFAULT NULL,
+  `f2375` int DEFAULT NULL,
+  `f2376` int DEFAULT NULL,
+  `f2377` int DEFAULT NULL,
+  `f2378` int DEFAULT NULL,
+  `f2379` int DEFAULT NULL,
+  `f2380` int DEFAULT NULL,
+  `f2381` int DEFAULT NULL,
+  `f2382` int DEFAULT NULL,
+  `f2383` int DEFAULT NULL,
+  `f2384` int DEFAULT NULL,
+  `f2385` int DEFAULT NULL,
+  `f2386` int DEFAULT NULL,
+  `f2387` int DEFAULT NULL,
+  `f2388` int DEFAULT NULL,
+  `f2389` int DEFAULT NULL,
+  `f2390` int DEFAULT NULL,
+  `f2391` int DEFAULT NULL,
+  `f2392` int DEFAULT NULL,
+  `f2393` int DEFAULT NULL,
+  `f2394` int DEFAULT NULL,
+  `f2395` int DEFAULT NULL,
+  `f2396` int DEFAULT NULL,
+  `f2397` int DEFAULT NULL,
+  `f2398` int DEFAULT NULL,
+  `f2399` int DEFAULT NULL,
+  `f2400` int DEFAULT NULL,
+  `f2401` int DEFAULT NULL,
+  `f2402` int DEFAULT NULL,
+  `f2403` int DEFAULT NULL,
+  `f2404` int DEFAULT NULL,
+  `f2405` int DEFAULT NULL,
+  `f2406` int DEFAULT NULL,
+  `f2407` int DEFAULT NULL,
+  `f2408` int DEFAULT NULL,
+  `f2409` int DEFAULT NULL,
+  `f2410` int DEFAULT NULL,
+  `f2411` int DEFAULT NULL,
+  `f2412` int DEFAULT NULL,
+  `f2413` int DEFAULT NULL,
+  `f2414` int DEFAULT NULL,
+  `f2415` int DEFAULT NULL,
+  `f2416` int DEFAULT NULL,
+  `f2417` int DEFAULT NULL,
+  `f2418` int DEFAULT NULL,
+  `f2419` int DEFAULT NULL,
+  `f2420` int DEFAULT NULL,
+  `f2421` int DEFAULT NULL,
+  `f2422` int DEFAULT NULL,
+  `f2423` int DEFAULT NULL,
+  `f2424` int DEFAULT NULL,
+  `f2425` int DEFAULT NULL,
+  `f2426` int DEFAULT NULL,
+  `f2427` int DEFAULT NULL,
+  `f2428` int DEFAULT NULL,
+  `f2429` int DEFAULT NULL,
+  `f2430` int DEFAULT NULL,
+  `f2431` int DEFAULT NULL,
+  `f2432` int DEFAULT NULL,
+  `f2433` int DEFAULT NULL,
+  `f2434` int DEFAULT NULL,
+  `f2435` int DEFAULT NULL,
+  `f2436` int DEFAULT NULL,
+  `f2437` int DEFAULT NULL,
+  `f2438` int DEFAULT NULL,
+  `f2439` int DEFAULT NULL,
+  `f2440` int DEFAULT NULL,
+  `f2441` int DEFAULT NULL,
+  `f2442` int DEFAULT NULL,
+  `f2443` int DEFAULT NULL,
+  `f2444` int DEFAULT NULL,
+  `f2445` int DEFAULT NULL,
+  `f2446` int DEFAULT NULL,
+  `f2447` int DEFAULT NULL,
+  `f2448` int DEFAULT NULL,
+  `f2449` int DEFAULT NULL,
+  `f2450` int DEFAULT NULL,
+  `f2451` int DEFAULT NULL,
+  `f2452` int DEFAULT NULL,
+  `f2453` int DEFAULT NULL,
+  `f2454` int DEFAULT NULL,
+  `f2455` int DEFAULT NULL,
+  `f2456` int DEFAULT NULL,
+  `f2457` int DEFAULT NULL,
+  `f2458` int DEFAULT NULL,
+  `f2459` int DEFAULT NULL,
+  `f2460` int DEFAULT NULL,
+  `f2461` int DEFAULT NULL,
+  `f2462` int DEFAULT NULL,
+  `f2463` int DEFAULT NULL,
+  `f2464` int DEFAULT NULL,
+  `f2465` int DEFAULT NULL,
+  `f2466` int DEFAULT NULL,
+  `f2467` int DEFAULT NULL,
+  `f2468` int DEFAULT NULL,
+  `f2469` int DEFAULT NULL,
+  `f2470` int DEFAULT NULL,
+  `f2471` int DEFAULT NULL,
+  `f2472` int DEFAULT NULL,
+  `f2473` int DEFAULT NULL,
+  `f2474` int DEFAULT NULL,
+  `f2475` int DEFAULT NULL,
+  `f2476` int DEFAULT NULL,
+  `f2477` int DEFAULT NULL,
+  `f2478` int DEFAULT NULL,
+  `f2479` int DEFAULT NULL,
+  `f2480` int DEFAULT NULL,
+  `f2481` int DEFAULT NULL,
+  `f2482` int DEFAULT NULL,
+  `f2483` int DEFAULT NULL,
+  `f2484` int DEFAULT NULL,
+  `f2485` int DEFAULT NULL,
+  `f2486` int DEFAULT NULL,
+  `f2487` int DEFAULT NULL,
+  `f2488` int DEFAULT NULL,
+  `f2489` int DEFAULT NULL,
+  `f2490` int DEFAULT NULL,
+  `f2491` int DEFAULT NULL,
+  `f2492` int DEFAULT NULL,
+  `f2493` int DEFAULT NULL,
+  `f2494` int DEFAULT NULL,
+  `f2495` int DEFAULT NULL,
+  `f2496` int DEFAULT NULL,
+  `f2497` int DEFAULT NULL,
+  `f2498` int DEFAULT NULL,
+  `f2499` int DEFAULT NULL,
+  `f2500` int DEFAULT NULL,
+  `f2501` int DEFAULT NULL,
+  `f2502` int DEFAULT NULL,
+  `f2503` int DEFAULT NULL,
+  `f2504` int DEFAULT NULL,
+  `f2505` int DEFAULT NULL,
+  `f2506` int DEFAULT NULL,
+  `f2507` int DEFAULT NULL,
+  `f2508` int DEFAULT NULL,
+  `f2509` int DEFAULT NULL,
+  `f2510` int DEFAULT NULL,
+  `f2511` int DEFAULT NULL,
+  `f2512` int DEFAULT NULL,
+  `f2513` int DEFAULT NULL,
+  `f2514` int DEFAULT NULL,
+  `f2515` int DEFAULT NULL,
+  `f2516` int DEFAULT NULL,
+  `f2517` int DEFAULT NULL,
+  `f2518` int DEFAULT NULL,
+  `f2519` int DEFAULT NULL,
+  `f2520` int DEFAULT NULL,
+  `f2521` int DEFAULT NULL,
+  `f2522` int DEFAULT NULL,
+  `f2523` int DEFAULT NULL,
+  `f2524` int DEFAULT NULL,
+  `f2525` int DEFAULT NULL,
+  `f2526` int DEFAULT NULL,
+  `f2527` int DEFAULT NULL,
+  `f2528` int DEFAULT NULL,
+  `f2529` int DEFAULT NULL,
+  `f2530` int DEFAULT NULL,
+  `f2531` int DEFAULT NULL,
+  `f2532` int DEFAULT NULL,
+  `f2533` int DEFAULT NULL,
+  `f2534` int DEFAULT NULL,
+  `f2535` int DEFAULT NULL,
+  `f2536` int DEFAULT NULL,
+  `f2537` int DEFAULT NULL,
+  `f2538` int DEFAULT NULL,
+  `f2539` int DEFAULT NULL,
+  `f2540` int DEFAULT NULL,
+  `f2541` int DEFAULT NULL,
+  `f2542` int DEFAULT NULL,
+  `f2543` int DEFAULT NULL,
+  `f2544` int DEFAULT NULL,
+  `f2545` int DEFAULT NULL,
+  `f2546` int DEFAULT NULL,
+  `f2547` int DEFAULT NULL,
+  `f2548` int DEFAULT NULL,
+  `f2549` int DEFAULT NULL,
+  `f2550` int DEFAULT NULL,
+  `f2551` int DEFAULT NULL,
+  `f2552` int DEFAULT NULL,
+  `f2553` int DEFAULT NULL,
+  `f2554` int DEFAULT NULL,
+  `f2555` int DEFAULT NULL,
+  `f2556` int DEFAULT NULL,
+  `f2557` int DEFAULT NULL,
+  `f2558` int DEFAULT NULL,
+  `f2559` int DEFAULT NULL,
+  `f2560` int DEFAULT NULL,
+  `f2561` int DEFAULT NULL,
+  `f2562` int DEFAULT NULL,
+  `f2563` int DEFAULT NULL,
+  `f2564` int DEFAULT NULL,
+  `f2565` int DEFAULT NULL,
+  `f2566` int DEFAULT NULL,
+  `f2567` int DEFAULT NULL,
+  `f2568` int DEFAULT NULL,
+  `f2569` int DEFAULT NULL,
+  `f2570` int DEFAULT NULL,
+  `f2571` int DEFAULT NULL,
+  `f2572` int DEFAULT NULL,
+  `f2573` int DEFAULT NULL,
+  `f2574` int DEFAULT NULL,
+  `f2575` int DEFAULT NULL,
+  `f2576` int DEFAULT NULL,
+  `f2577` int DEFAULT NULL,
+  `f2578` int DEFAULT NULL,
+  `f2579` int DEFAULT NULL,
+  `f2580` int DEFAULT NULL,
+  `f2581` int DEFAULT NULL,
+  `f2582` int DEFAULT NULL,
+  `f2583` int DEFAULT NULL,
+  `f2584` int DEFAULT NULL,
+  `f2585` int DEFAULT NULL,
+  `f2586` int DEFAULT NULL,
+  `f2587` int DEFAULT NULL,
+  `f2588` int DEFAULT NULL,
+  `f2589` int DEFAULT NULL,
+  `f2590` int DEFAULT NULL,
+  `f2591` int DEFAULT NULL,
+  `f2592` int DEFAULT NULL,
+  `f2593` int DEFAULT NULL,
+  `f2594` int DEFAULT NULL,
+  `f2595` int DEFAULT NULL,
+  `f2596` int DEFAULT NULL,
+  `f2597` int DEFAULT NULL,
+  `f2598` int DEFAULT NULL,
+  `f2599` int DEFAULT NULL,
+  `f2600` int DEFAULT NULL,
+  `f2601` int DEFAULT NULL,
+  `f2602` int DEFAULT NULL,
+  `f2603` int DEFAULT NULL,
+  `f2604` int DEFAULT NULL,
+  `f2605` int DEFAULT NULL,
+  `f2606` int DEFAULT NULL,
+  `f2607` int DEFAULT NULL,
+  `f2608` int DEFAULT NULL,
+  `f2609` int DEFAULT NULL,
+  `f2610` int DEFAULT NULL,
+  `f2611` int DEFAULT NULL,
+  `f2612` int DEFAULT NULL,
+  `f2613` int DEFAULT NULL,
+  `f2614` int DEFAULT NULL,
+  `f2615` int DEFAULT NULL,
+  `f2616` int DEFAULT NULL,
+  `f2617` int DEFAULT NULL,
+  `f2618` int DEFAULT NULL,
+  `f2619` int DEFAULT NULL,
+  `f2620` int DEFAULT NULL,
+  `f2621` int DEFAULT NULL,
+  `f2622` int DEFAULT NULL,
+  `f2623` int DEFAULT NULL,
+  `f2624` int DEFAULT NULL,
+  `f2625` int DEFAULT NULL,
+  `f2626` int DEFAULT NULL,
+  `f2627` int DEFAULT NULL,
+  `f2628` int DEFAULT NULL,
+  `f2629` int DEFAULT NULL,
+  `f2630` int DEFAULT NULL,
+  `f2631` int DEFAULT NULL,
+  `f2632` int DEFAULT NULL,
+  `f2633` int DEFAULT NULL,
+  `f2634` int DEFAULT NULL,
+  `f2635` int DEFAULT NULL,
+  `f2636` int DEFAULT NULL,
+  `f2637` int DEFAULT NULL,
+  `f2638` int DEFAULT NULL,
+  `f2639` int DEFAULT NULL,
+  `f2640` int DEFAULT NULL,
+  `f2641` int DEFAULT NULL,
+  `f2642` int DEFAULT NULL,
+  `f2643` int DEFAULT NULL,
+  `f2644` int DEFAULT NULL,
+  `f2645` int DEFAULT NULL,
+  `f2646` int DEFAULT NULL,
+  `f2647` int DEFAULT NULL,
+  `f2648` int DEFAULT NULL,
+  `f2649` int DEFAULT NULL,
+  `f2650` int DEFAULT NULL,
+  `f2651` int DEFAULT NULL,
+  `f2652` int DEFAULT NULL,
+  `f2653` int DEFAULT NULL,
+  `f2654` int DEFAULT NULL,
+  `f2655` int DEFAULT NULL,
+  `f2656` int DEFAULT NULL,
+  `f2657` int DEFAULT NULL,
+  `f2658` int DEFAULT NULL,
+  `f2659` int DEFAULT NULL,
+  `f2660` int DEFAULT NULL,
+  `f2661` int DEFAULT NULL,
+  `f2662` int DEFAULT NULL,
+  `f2663` int DEFAULT NULL,
+  `f2664` int DEFAULT NULL,
+  `f2665` int DEFAULT NULL,
+  `f2666` int DEFAULT NULL,
+  `f2667` int DEFAULT NULL,
+  `f2668` int DEFAULT NULL,
+  `f2669` int DEFAULT NULL,
+  `f2670` int DEFAULT NULL,
+  `f2671` int DEFAULT NULL,
+  `f2672` int DEFAULT NULL,
+  `f2673` int DEFAULT NULL,
+  `f2674` int DEFAULT NULL,
+  `f2675` int DEFAULT NULL,
+  `f2676` int DEFAULT NULL,
+  `f2677` int DEFAULT NULL,
+  `f2678` int DEFAULT NULL,
+  `f2679` int DEFAULT NULL,
+  `f2680` int DEFAULT NULL,
+  `f2681` int DEFAULT NULL,
+  `f2682` int DEFAULT NULL,
+  `f2683` int DEFAULT NULL,
+  `f2684` int DEFAULT NULL,
+  `f2685` int DEFAULT NULL,
+  `f2686` int DEFAULT NULL,
+  `f2687` int DEFAULT NULL,
+  `f2688` int DEFAULT NULL,
+  `f2689` int DEFAULT NULL,
+  `f2690` int DEFAULT NULL,
+  `f2691` int DEFAULT NULL,
+  `f2692` int DEFAULT NULL,
+  `f2693` int DEFAULT NULL,
+  `f2694` int DEFAULT NULL,
+  `f2695` int DEFAULT NULL,
+  `f2696` int DEFAULT NULL,
+  `f2697` int DEFAULT NULL,
+  `f2698` int DEFAULT NULL,
+  `f2699` int DEFAULT NULL,
+  `f2700` int DEFAULT NULL,
+  `f2701` int DEFAULT NULL,
+  `f2702` int DEFAULT NULL,
+  `f2703` int DEFAULT NULL,
+  `f2704` int DEFAULT NULL,
+  `f2705` int DEFAULT NULL,
+  `f2706` int DEFAULT NULL,
+  `f2707` int DEFAULT NULL,
+  `f2708` int DEFAULT NULL,
+  `f2709` int DEFAULT NULL,
+  `f2710` int DEFAULT NULL,
+  `f2711` int DEFAULT NULL,
+  `f2712` int DEFAULT NULL,
+  `f2713` int DEFAULT NULL,
+  `f2714` int DEFAULT NULL,
+  `f2715` int DEFAULT NULL,
+  `f2716` int DEFAULT NULL,
+  `f2717` int DEFAULT NULL,
+  `f2718` int DEFAULT NULL,
+  `f2719` int DEFAULT NULL,
+  `f2720` int DEFAULT NULL,
+  `f2721` int DEFAULT NULL,
+  `f2722` int DEFAULT NULL,
+  `f2723` int DEFAULT NULL,
+  `f2724` int DEFAULT NULL,
+  `f2725` int DEFAULT NULL,
+  `f2726` int DEFAULT NULL,
+  `f2727` int DEFAULT NULL,
+  `f2728` int DEFAULT NULL,
+  `f2729` int DEFAULT NULL,
+  `f2730` int DEFAULT NULL,
+  `f2731` int DEFAULT NULL,
+  `f2732` int DEFAULT NULL,
+  `f2733` int DEFAULT NULL,
+  `f2734` int DEFAULT NULL,
+  `f2735` int DEFAULT NULL,
+  `f2736` int DEFAULT NULL,
+  `f2737` int DEFAULT NULL,
+  `f2738` int DEFAULT NULL,
+  `f2739` int DEFAULT NULL,
+  `f2740` int DEFAULT NULL,
+  `f2741` int DEFAULT NULL,
+  `f2742` int DEFAULT NULL,
+  `f2743` int DEFAULT NULL,
+  `f2744` int DEFAULT NULL,
+  `f2745` int DEFAULT NULL,
+  `f2746` int DEFAULT NULL,
+  `f2747` int DEFAULT NULL,
+  `f2748` int DEFAULT NULL,
+  `f2749` int DEFAULT NULL,
+  `f2750` int DEFAULT NULL,
+  `f2751` int DEFAULT NULL,
+  `f2752` int DEFAULT NULL,
+  `f2753` int DEFAULT NULL,
+  `f2754` int DEFAULT NULL,
+  `f2755` int DEFAULT NULL,
+  `f2756` int DEFAULT NULL,
+  `f2757` int DEFAULT NULL,
+  `f2758` int DEFAULT NULL,
+  `f2759` int DEFAULT NULL,
+  `f2760` int DEFAULT NULL,
+  `f2761` int DEFAULT NULL,
+  `f2762` int DEFAULT NULL,
+  `f2763` int DEFAULT NULL,
+  `f2764` int DEFAULT NULL,
+  `f2765` int DEFAULT NULL,
+  `f2766` int DEFAULT NULL,
+  `f2767` int DEFAULT NULL,
+  `f2768` int DEFAULT NULL,
+  `f2769` int DEFAULT NULL,
+  `f2770` int DEFAULT NULL,
+  `f2771` int DEFAULT NULL,
+  `f2772` int DEFAULT NULL,
+  `f2773` int DEFAULT NULL,
+  `f2774` int DEFAULT NULL,
+  `f2775` int DEFAULT NULL,
+  `f2776` int DEFAULT NULL,
+  `f2777` int DEFAULT NULL,
+  `f2778` int DEFAULT NULL,
+  `f2779` int DEFAULT NULL,
+  `f2780` int DEFAULT NULL,
+  `f2781` int DEFAULT NULL,
+  `f2782` int DEFAULT NULL,
+  `f2783` int DEFAULT NULL,
+  `f2784` int DEFAULT NULL,
+  `f2785` int DEFAULT NULL,
+  `f2786` int DEFAULT NULL,
+  `f2787` int DEFAULT NULL,
+  `f2788` int DEFAULT NULL,
+  `f2789` int DEFAULT NULL,
+  `f2790` int DEFAULT NULL,
+  `f2791` int DEFAULT NULL,
+  `f2792` int DEFAULT NULL,
+  `f2793` int DEFAULT NULL,
+  `f2794` int DEFAULT NULL,
+  `f2795` int DEFAULT NULL,
+  `f2796` int DEFAULT NULL,
+  `f2797` int DEFAULT NULL,
+  `f2798` int DEFAULT NULL,
+  `f2799` int DEFAULT NULL,
+  `f2800` int DEFAULT NULL,
+  `f2801` int DEFAULT NULL,
+  `f2802` int DEFAULT NULL,
+  `f2803` int DEFAULT NULL,
+  `f2804` int DEFAULT NULL,
+  `f2805` int DEFAULT NULL,
+  `f2806` int DEFAULT NULL,
+  `f2807` int DEFAULT NULL,
+  `f2808` int DEFAULT NULL,
+  `f2809` int DEFAULT NULL,
+  `f2810` int DEFAULT NULL,
+  `f2811` int DEFAULT NULL,
+  `f2812` int DEFAULT NULL,
+  `f2813` int DEFAULT NULL,
+  `f2814` int DEFAULT NULL,
+  `f2815` int DEFAULT NULL,
+  `f2816` int DEFAULT NULL,
+  `f2817` int DEFAULT NULL,
+  `f2818` int DEFAULT NULL,
+  `f2819` int DEFAULT NULL,
+  `f2820` int DEFAULT NULL,
+  `f2821` int DEFAULT NULL,
+  `f2822` int DEFAULT NULL,
+  `f2823` int DEFAULT NULL,
+  `f2824` int DEFAULT NULL,
+  `f2825` int DEFAULT NULL,
+  `f2826` int DEFAULT NULL,
+  `f2827` int DEFAULT NULL,
+  `f2828` int DEFAULT NULL,
+  `f2829` int DEFAULT NULL,
+  `f2830` int DEFAULT NULL,
+  `f2831` int DEFAULT NULL,
+  `f2832` int DEFAULT NULL,
+  `f2833` int DEFAULT NULL,
+  `f2834` int DEFAULT NULL,
+  `f2835` int DEFAULT NULL,
+  `f2836` int DEFAULT NULL,
+  `f2837` int DEFAULT NULL,
+  `f2838` int DEFAULT NULL,
+  `f2839` int DEFAULT NULL,
+  `f2840` int DEFAULT NULL,
+  `f2841` int DEFAULT NULL,
+  `f2842` int DEFAULT NULL,
+  `f2843` int DEFAULT NULL,
+  `f2844` int DEFAULT NULL,
+  `f2845` int DEFAULT NULL,
+  `f2846` int DEFAULT NULL,
+  `f2847` int DEFAULT NULL,
+  `f2848` int DEFAULT NULL,
+  `f2849` int DEFAULT NULL,
+  `f2850` int DEFAULT NULL,
+  `f2851` int DEFAULT NULL,
+  `f2852` int DEFAULT NULL,
+  `f2853` int DEFAULT NULL,
+  `f2854` int DEFAULT NULL,
+  `f2855` int DEFAULT NULL,
+  `f2856` int DEFAULT NULL,
+  `f2857` int DEFAULT NULL,
+  `f2858` int DEFAULT NULL,
+  `f2859` int DEFAULT NULL,
+  `f2860` int DEFAULT NULL,
+  `f2861` int DEFAULT NULL,
+  `f2862` int DEFAULT NULL,
+  `f2863` int DEFAULT NULL,
+  `f2864` int DEFAULT NULL,
+  `f2865` int DEFAULT NULL,
+  `f2866` int DEFAULT NULL,
+  `f2867` int DEFAULT NULL,
+  `f2868` int DEFAULT NULL,
+  `f2869` int DEFAULT NULL,
+  `f2870` int DEFAULT NULL,
+  `f2871` int DEFAULT NULL,
+  `f2872` int DEFAULT NULL,
+  `f2873` int DEFAULT NULL,
+  `f2874` int DEFAULT NULL,
+  `f2875` int DEFAULT NULL,
+  `f2876` int DEFAULT NULL,
+  `f2877` int DEFAULT NULL,
+  `f2878` int DEFAULT NULL,
+  `f2879` int DEFAULT NULL,
+  `f2880` int DEFAULT NULL,
+  `f2881` int DEFAULT NULL,
+  `f2882` int DEFAULT NULL,
+  `f2883` int DEFAULT NULL,
+  `f2884` int DEFAULT NULL,
+  `f2885` int DEFAULT NULL,
+  `f2886` int DEFAULT NULL,
+  `f2887` int DEFAULT NULL,
+  `f2888` int DEFAULT NULL,
+  `f2889` int DEFAULT NULL,
+  `f2890` int DEFAULT NULL,
+  `f2891` int DEFAULT NULL,
+  `f2892` int DEFAULT NULL,
+  `f2893` int DEFAULT NULL,
+  `f2894` int DEFAULT NULL,
+  `f2895` int DEFAULT NULL,
+  `f2896` int DEFAULT NULL,
+  `f2897` int DEFAULT NULL,
+  `f2898` int DEFAULT NULL,
+  `f2899` int DEFAULT NULL,
+  `f2900` int DEFAULT NULL,
+  `f2901` int DEFAULT NULL,
+  `f2902` int DEFAULT NULL,
+  `f2903` int DEFAULT NULL,
+  `f2904` int DEFAULT NULL,
+  `f2905` int DEFAULT NULL,
+  `f2906` int DEFAULT NULL,
+  `f2907` int DEFAULT NULL,
+  `f2908` int DEFAULT NULL,
+  `f2909` int DEFAULT NULL,
+  `f2910` int DEFAULT NULL,
+  `f2911` int DEFAULT NULL,
+  `f2912` int DEFAULT NULL,
+  `f2913` int DEFAULT NULL,
+  `f2914` int DEFAULT NULL,
+  `f2915` int DEFAULT NULL,
+  `f2916` int DEFAULT NULL,
+  `f2917` int DEFAULT NULL,
+  `f2918` int DEFAULT NULL,
+  `f2919` int DEFAULT NULL,
+  `f2920` int DEFAULT NULL,
+  `f2921` int DEFAULT NULL,
+  `f2922` int DEFAULT NULL,
+  `f2923` int DEFAULT NULL,
+  `f2924` int DEFAULT NULL,
+  `f2925` int DEFAULT NULL,
+  `f2926` int DEFAULT NULL,
+  `f2927` int DEFAULT NULL,
+  `f2928` int DEFAULT NULL,
+  `f2929` int DEFAULT NULL,
+  `f2930` int DEFAULT NULL,
+  `f2931` int DEFAULT NULL,
+  `f2932` int DEFAULT NULL,
+  `f2933` int DEFAULT NULL,
+  `f2934` int DEFAULT NULL,
+  `f2935` int DEFAULT NULL,
+  `f2936` int DEFAULT NULL,
+  `f2937` int DEFAULT NULL,
+  `f2938` int DEFAULT NULL,
+  `f2939` int DEFAULT NULL,
+  `f2940` int DEFAULT NULL,
+  `f2941` int DEFAULT NULL,
+  `f2942` int DEFAULT NULL,
+  `f2943` int DEFAULT NULL,
+  `f2944` int DEFAULT NULL,
+  `f2945` int DEFAULT NULL,
+  `f2946` int DEFAULT NULL,
+  `f2947` int DEFAULT NULL,
+  `f2948` int DEFAULT NULL,
+  `f2949` int DEFAULT NULL,
+  `f2950` int DEFAULT NULL,
+  `f2951` int DEFAULT NULL,
+  `f2952` int DEFAULT NULL,
+  `f2953` int DEFAULT NULL,
+  `f2954` int DEFAULT NULL,
+  `f2955` int DEFAULT NULL,
+  `f2956` int DEFAULT NULL,
+  `f2957` int DEFAULT NULL,
+  `f2958` int DEFAULT NULL,
+  `f2959` int DEFAULT NULL,
+  `f2960` int DEFAULT NULL,
+  `f2961` int DEFAULT NULL,
+  `f2962` int DEFAULT NULL,
+  `f2963` int DEFAULT NULL,
+  `f2964` int DEFAULT NULL,
+  `f2965` int DEFAULT NULL,
+  `f2966` int DEFAULT NULL,
+  `f2967` int DEFAULT NULL,
+  `f2968` int DEFAULT NULL,
+  `f2969` int DEFAULT NULL,
+  `f2970` int DEFAULT NULL,
+  `f2971` int DEFAULT NULL,
+  `f2972` int DEFAULT NULL,
+  `f2973` int DEFAULT NULL,
+  `f2974` int DEFAULT NULL,
+  `f2975` int DEFAULT NULL,
+  `f2976` int DEFAULT NULL,
+  `f2977` int DEFAULT NULL,
+  `f2978` int DEFAULT NULL,
+  `f2979` int DEFAULT NULL,
+  `f2980` int DEFAULT NULL,
+  `f2981` int DEFAULT NULL,
+  `f2982` int DEFAULT NULL,
+  `f2983` int DEFAULT NULL,
+  `f2984` int DEFAULT NULL,
+  `f2985` int DEFAULT NULL,
+  `f2986` int DEFAULT NULL,
+  `f2987` int DEFAULT NULL,
+  `f2988` int DEFAULT NULL,
+  `f2989` int DEFAULT NULL,
+  `f2990` int DEFAULT NULL,
+  `f2991` int DEFAULT NULL,
+  `f2992` int DEFAULT NULL,
+  `f2993` int DEFAULT NULL,
+  `f2994` int DEFAULT NULL,
+  `f2995` int DEFAULT NULL,
+  `f2996` int DEFAULT NULL,
+  `f2997` int DEFAULT NULL,
+  `f2998` int DEFAULT NULL,
+  `f2999` int DEFAULT NULL,
+  `f3000` int DEFAULT NULL,
+  `f3001` int DEFAULT NULL,
+  `f3002` int DEFAULT NULL,
+  `f3003` int DEFAULT NULL,
+  `f3004` int DEFAULT NULL,
+  `f3005` int DEFAULT NULL,
+  `f3006` int DEFAULT NULL,
+  `f3007` int DEFAULT NULL,
+  `f3008` int DEFAULT NULL,
+  `f3009` int DEFAULT NULL,
+  `f3010` int DEFAULT NULL,
+  `f3011` int DEFAULT NULL,
+  `f3012` int DEFAULT NULL,
+  `f3013` int DEFAULT NULL,
+  `f3014` int DEFAULT NULL,
+  `f3015` int DEFAULT NULL,
+  `f3016` int DEFAULT NULL,
+  `f3017` int DEFAULT NULL,
+  `f3018` int DEFAULT NULL,
+  `f3019` int DEFAULT NULL,
+  `f3020` int DEFAULT NULL,
+  `f3021` int DEFAULT NULL,
+  `f3022` int DEFAULT NULL,
+  `f3023` int DEFAULT NULL,
+  `f3024` int DEFAULT NULL,
+  `f3025` int DEFAULT NULL,
+  `f3026` int DEFAULT NULL,
+  `f3027` int DEFAULT NULL,
+  `f3028` int DEFAULT NULL,
+  `f3029` int DEFAULT NULL,
+  `f3030` int DEFAULT NULL,
+  `f3031` int DEFAULT NULL,
+  `f3032` int DEFAULT NULL,
+  `f3033` int DEFAULT NULL,
+  `f3034` int DEFAULT NULL,
+  `f3035` int DEFAULT NULL,
+  `f3036` int DEFAULT NULL,
+  `f3037` int DEFAULT NULL,
+  `f3038` int DEFAULT NULL,
+  `f3039` int DEFAULT NULL,
+  `f3040` int DEFAULT NULL,
+  `f3041` int DEFAULT NULL,
+  `f3042` int DEFAULT NULL,
+  `f3043` int DEFAULT NULL,
+  `f3044` int DEFAULT NULL,
+  `f3045` int DEFAULT NULL,
+  `f3046` int DEFAULT NULL,
+  `f3047` int DEFAULT NULL,
+  `f3048` int DEFAULT NULL,
+  `f3049` int DEFAULT NULL,
+  `f3050` int DEFAULT NULL,
+  `f3051` int DEFAULT NULL,
+  `f3052` int DEFAULT NULL,
+  `f3053` int DEFAULT NULL,
+  `f3054` int DEFAULT NULL,
+  `f3055` int DEFAULT NULL,
+  `f3056` int DEFAULT NULL,
+  `f3057` int DEFAULT NULL,
+  `f3058` int DEFAULT NULL,
+  `f3059` int DEFAULT NULL,
+  `f3060` int DEFAULT NULL,
+  `f3061` int DEFAULT NULL,
+  `f3062` int DEFAULT NULL,
+  `f3063` int DEFAULT NULL,
+  `f3064` int DEFAULT NULL,
+  `f3065` int DEFAULT NULL,
+  `f3066` int DEFAULT NULL,
+  `f3067` int DEFAULT NULL,
+  `f3068` int DEFAULT NULL,
+  `f3069` int DEFAULT NULL,
+  `f3070` int DEFAULT NULL,
+  `f3071` int DEFAULT NULL,
+  `f3072` int DEFAULT NULL,
+  `f3073` int DEFAULT NULL,
+  `f3074` int DEFAULT NULL,
+  `f3075` int DEFAULT NULL,
+  `f3076` int DEFAULT NULL,
+  `f3077` int DEFAULT NULL,
+  `f3078` int DEFAULT NULL,
+  `f3079` int DEFAULT NULL,
+  `f3080` int DEFAULT NULL,
+  `f3081` int DEFAULT NULL,
+  `f3082` int DEFAULT NULL,
+  `f3083` int DEFAULT NULL,
+  `f3084` int DEFAULT NULL,
+  `f3085` int DEFAULT NULL,
+  `f3086` int DEFAULT NULL,
+  `f3087` int DEFAULT NULL,
+  `f3088` int DEFAULT NULL,
+  `f3089` int DEFAULT NULL,
+  `f3090` int DEFAULT NULL,
+  `f3091` int DEFAULT NULL,
+  `f3092` int DEFAULT NULL,
+  `f3093` int DEFAULT NULL,
+  `f3094` int DEFAULT NULL,
+  `f3095` int DEFAULT NULL,
+  `f3096` int DEFAULT NULL,
+  `f3097` int DEFAULT NULL,
+  `f3098` int DEFAULT NULL,
+  `f3099` int DEFAULT NULL,
+  `f3100` int DEFAULT NULL,
+  `f3101` int DEFAULT NULL,
+  `f3102` int DEFAULT NULL,
+  `f3103` int DEFAULT NULL,
+  `f3104` int DEFAULT NULL,
+  `f3105` int DEFAULT NULL,
+  `f3106` int DEFAULT NULL,
+  `f3107` int DEFAULT NULL,
+  `f3108` int DEFAULT NULL,
+  `f3109` int DEFAULT NULL,
+  `f3110` int DEFAULT NULL,
+  `f3111` int DEFAULT NULL,
+  `f3112` int DEFAULT NULL,
+  `f3113` int DEFAULT NULL,
+  `f3114` int DEFAULT NULL,
+  `f3115` int DEFAULT NULL,
+  `f3116` int DEFAULT NULL,
+  `f3117` int DEFAULT NULL,
+  `f3118` int DEFAULT NULL,
+  `f3119` int DEFAULT NULL,
+  `f3120` int DEFAULT NULL,
+  `f3121` int DEFAULT NULL,
+  `f3122` int DEFAULT NULL,
+  `f3123` int DEFAULT NULL,
+  `f3124` int DEFAULT NULL,
+  `f3125` int DEFAULT NULL,
+  `f3126` int DEFAULT NULL,
+  `f3127` int DEFAULT NULL,
+  `f3128` int DEFAULT NULL,
+  `f3129` int DEFAULT NULL,
+  `f3130` int DEFAULT NULL,
+  `f3131` int DEFAULT NULL,
+  `f3132` int DEFAULT NULL,
+  `f3133` int DEFAULT NULL,
+  `f3134` int DEFAULT NULL,
+  `f3135` int DEFAULT NULL,
+  `f3136` int DEFAULT NULL,
+  `f3137` int DEFAULT NULL,
+  `f3138` int DEFAULT NULL,
+  `f3139` int DEFAULT NULL,
+  `f3140` int DEFAULT NULL,
+  `f3141` int DEFAULT NULL,
+  `f3142` int DEFAULT NULL,
+  `f3143` int DEFAULT NULL,
+  `f3144` int DEFAULT NULL,
+  `f3145` int DEFAULT NULL,
+  `f3146` int DEFAULT NULL,
+  `f3147` int DEFAULT NULL,
+  `f3148` int DEFAULT NULL,
+  `f3149` int DEFAULT NULL,
+  `f3150` int DEFAULT NULL,
+  `f3151` int DEFAULT NULL,
+  `f3152` int DEFAULT NULL,
+  `f3153` int DEFAULT NULL,
+  `f3154` int DEFAULT NULL,
+  `f3155` int DEFAULT NULL,
+  `f3156` int DEFAULT NULL,
+  `f3157` int DEFAULT NULL,
+  `f3158` int DEFAULT NULL,
+  `f3159` int DEFAULT NULL,
+  `f3160` int DEFAULT NULL,
+  `f3161` int DEFAULT NULL,
+  `f3162` int DEFAULT NULL,
+  `f3163` int DEFAULT NULL,
+  `f3164` int DEFAULT NULL,
+  `f3165` int DEFAULT NULL,
+  `f3166` int DEFAULT NULL,
+  `f3167` int DEFAULT NULL,
+  `f3168` int DEFAULT NULL,
+  `f3169` int DEFAULT NULL,
+  `f3170` int DEFAULT NULL,
+  `f3171` int DEFAULT NULL,
+  `f3172` int DEFAULT NULL,
+  `f3173` int DEFAULT NULL,
+  `f3174` int DEFAULT NULL,
+  `f3175` int DEFAULT NULL,
+  `f3176` int DEFAULT NULL,
+  `f3177` int DEFAULT NULL,
+  `f3178` int DEFAULT NULL,
+  `f3179` int DEFAULT NULL,
+  `f3180` int DEFAULT NULL,
+  `f3181` int DEFAULT NULL,
+  `f3182` int DEFAULT NULL,
+  `f3183` int DEFAULT NULL,
+  `f3184` int DEFAULT NULL,
+  `f3185` int DEFAULT NULL,
+  `f3186` int DEFAULT NULL,
+  `f3187` int DEFAULT NULL,
+  `f3188` int DEFAULT NULL,
+  `f3189` int DEFAULT NULL,
+  `f3190` int DEFAULT NULL,
+  `f3191` int DEFAULT NULL,
+  `f3192` int DEFAULT NULL,
+  `f3193` int DEFAULT NULL,
+  `f3194` int DEFAULT NULL,
+  `f3195` int DEFAULT NULL,
+  `f3196` int DEFAULT NULL,
+  `f3197` int DEFAULT NULL,
+  `f3198` int DEFAULT NULL,
+  `f3199` int DEFAULT NULL,
+  `f3200` int DEFAULT NULL,
+  `f3201` int DEFAULT NULL,
+  `f3202` int DEFAULT NULL,
+  `f3203` int DEFAULT NULL,
+  `f3204` int DEFAULT NULL,
+  `f3205` int DEFAULT NULL,
+  `f3206` int DEFAULT NULL,
+  `f3207` int DEFAULT NULL,
+  `f3208` int DEFAULT NULL,
+  `f3209` int DEFAULT NULL,
+  `f3210` int DEFAULT NULL,
+  `f3211` int DEFAULT NULL,
+  `f3212` int DEFAULT NULL,
+  `f3213` int DEFAULT NULL,
+  `f3214` int DEFAULT NULL,
+  `f3215` int DEFAULT NULL,
+  `f3216` int DEFAULT NULL,
+  `f3217` int DEFAULT NULL,
+  `f3218` int DEFAULT NULL,
+  `f3219` int DEFAULT NULL,
+  `f3220` int DEFAULT NULL,
+  `f3221` int DEFAULT NULL,
+  `f3222` int DEFAULT NULL,
+  `f3223` int DEFAULT NULL,
+  `f3224` int DEFAULT NULL,
+  `f3225` int DEFAULT NULL,
+  `f3226` int DEFAULT NULL,
+  `f3227` int DEFAULT NULL,
+  `f3228` int DEFAULT NULL,
+  `f3229` int DEFAULT NULL,
+  `f3230` int DEFAULT NULL,
+  `f3231` int DEFAULT NULL,
+  `f3232` int DEFAULT NULL,
+  `f3233` int DEFAULT NULL,
+  `f3234` int DEFAULT NULL,
+  `f3235` int DEFAULT NULL,
+  `f3236` int DEFAULT NULL,
+  `f3237` int DEFAULT NULL,
+  `f3238` int DEFAULT NULL,
+  `f3239` int DEFAULT NULL,
+  `f3240` int DEFAULT NULL,
+  `f3241` int DEFAULT NULL,
+  `f3242` int DEFAULT NULL,
+  `f3243` int DEFAULT NULL,
+  `f3244` int DEFAULT NULL,
+  `f3245` int DEFAULT NULL,
+  `f3246` int DEFAULT NULL,
+  `f3247` int DEFAULT NULL,
+  `f3248` int DEFAULT NULL,
+  `f3249` int DEFAULT NULL,
+  `f3250` int DEFAULT NULL,
+  `f3251` int DEFAULT NULL,
+  `f3252` int DEFAULT NULL,
+  `f3253` int DEFAULT NULL,
+  `f3254` int DEFAULT NULL,
+  `f3255` int DEFAULT NULL,
+  `f3256` int DEFAULT NULL,
+  `f3257` int DEFAULT NULL,
+  `f3258` int DEFAULT NULL,
+  `f3259` int DEFAULT NULL,
+  `f3260` int DEFAULT NULL,
+  `f3261` int DEFAULT NULL,
+  `f3262` int DEFAULT NULL,
+  `f3263` int DEFAULT NULL,
+  `f3264` int DEFAULT NULL,
+  `f3265` int DEFAULT NULL,
+  `f3266` int DEFAULT NULL,
+  `f3267` int DEFAULT NULL,
+  `f3268` int DEFAULT NULL,
+  `f3269` int DEFAULT NULL,
+  `f3270` int DEFAULT NULL,
+  `f3271` int DEFAULT NULL,
+  `f3272` int DEFAULT NULL,
+  `f3273` int DEFAULT NULL,
+  `f3274` int DEFAULT NULL,
+  `f3275` int DEFAULT NULL,
+  `f3276` int DEFAULT NULL,
+  `f3277` int DEFAULT NULL,
+  `f3278` int DEFAULT NULL,
+  `f3279` int DEFAULT NULL,
+  `f3280` int DEFAULT NULL,
+  `f3281` int DEFAULT NULL,
+  `f3282` int DEFAULT NULL,
+  `f3283` int DEFAULT NULL,
+  `f3284` int DEFAULT NULL,
+  `f3285` int DEFAULT NULL,
+  `f3286` int DEFAULT NULL,
+  `f3287` int DEFAULT NULL,
+  `f3288` int DEFAULT NULL,
+  `f3289` int DEFAULT NULL,
+  `f3290` int DEFAULT NULL,
+  `f3291` int DEFAULT NULL,
+  `f3292` int DEFAULT NULL,
+  `f3293` int DEFAULT NULL,
+  `f3294` int DEFAULT NULL,
+  `f3295` int DEFAULT NULL,
+  `f3296` int DEFAULT NULL,
+  `f3297` int DEFAULT NULL,
+  `f3298` int DEFAULT NULL,
+  `f3299` int DEFAULT NULL,
+  `f3300` int DEFAULT NULL,
+  `f3301` int DEFAULT NULL,
+  `f3302` int DEFAULT NULL,
+  `f3303` int DEFAULT NULL,
+  `f3304` int DEFAULT NULL,
+  `f3305` int DEFAULT NULL,
+  `f3306` int DEFAULT NULL,
+  `f3307` int DEFAULT NULL,
+  `f3308` int DEFAULT NULL,
+  `f3309` int DEFAULT NULL,
+  `f3310` int DEFAULT NULL,
+  `f3311` int DEFAULT NULL,
+  `f3312` int DEFAULT NULL,
+  `f3313` int DEFAULT NULL,
+  `f3314` int DEFAULT NULL,
+  `f3315` int DEFAULT NULL,
+  `f3316` int DEFAULT NULL,
+  `f3317` int DEFAULT NULL,
+  `f3318` int DEFAULT NULL,
+  `f3319` int DEFAULT NULL,
+  `f3320` int DEFAULT NULL,
+  `f3321` int DEFAULT NULL,
+  `f3322` int DEFAULT NULL,
+  `f3323` int DEFAULT NULL,
+  `f3324` int DEFAULT NULL,
+  `f3325` int DEFAULT NULL,
+  `f3326` int DEFAULT NULL,
+  `f3327` int DEFAULT NULL,
+  `f3328` int DEFAULT NULL,
+  `f3329` int DEFAULT NULL,
+  `f3330` int DEFAULT NULL,
+  `f3331` int DEFAULT NULL,
+  `f3332` int DEFAULT NULL,
+  `f3333` int DEFAULT NULL,
+  `f3334` int DEFAULT NULL,
+  `f3335` int DEFAULT NULL,
+  `f3336` int DEFAULT NULL,
+  `f3337` int DEFAULT NULL,
+  `f3338` int DEFAULT NULL,
+  `f3339` int DEFAULT NULL,
+  `f3340` int DEFAULT NULL,
+  `f3341` int DEFAULT NULL,
+  `f3342` int DEFAULT NULL,
+  `f3343` int DEFAULT NULL,
+  `f3344` int DEFAULT NULL,
+  `f3345` int DEFAULT NULL,
+  `f3346` int DEFAULT NULL,
+  `f3347` int DEFAULT NULL,
+  `f3348` int DEFAULT NULL,
+  `f3349` int DEFAULT NULL,
+  `f3350` int DEFAULT NULL,
+  `f3351` int DEFAULT NULL,
+  `f3352` int DEFAULT NULL,
+  `f3353` int DEFAULT NULL,
+  `f3354` int DEFAULT NULL,
+  `f3355` int DEFAULT NULL,
+  `f3356` int DEFAULT NULL,
+  `f3357` int DEFAULT NULL,
+  `f3358` int DEFAULT NULL,
+  `f3359` int DEFAULT NULL,
+  `f3360` int DEFAULT NULL,
+  `f3361` int DEFAULT NULL,
+  `f3362` int DEFAULT NULL,
+  `f3363` int DEFAULT NULL,
+  `f3364` int DEFAULT NULL,
+  `f3365` int DEFAULT NULL,
+  `f3366` int DEFAULT NULL,
+  `f3367` int DEFAULT NULL,
+  `f3368` int DEFAULT NULL,
+  `f3369` int DEFAULT NULL,
+  `f3370` int DEFAULT NULL,
+  `f3371` int DEFAULT NULL,
+  `f3372` int DEFAULT NULL,
+  `f3373` int DEFAULT NULL,
+  `f3374` int DEFAULT NULL,
+  `f3375` int DEFAULT NULL,
+  `f3376` int DEFAULT NULL,
+  `f3377` int DEFAULT NULL,
+  `f3378` int DEFAULT NULL,
+  `f3379` int DEFAULT NULL,
+  `f3380` int DEFAULT NULL,
+  `f3381` int DEFAULT NULL,
+  `f3382` int DEFAULT NULL,
+  `f3383` int DEFAULT NULL,
+  `f3384` int DEFAULT NULL,
+  `f3385` int DEFAULT NULL,
+  `f3386` int DEFAULT NULL,
+  `f3387` int DEFAULT NULL,
+  `f3388` int DEFAULT NULL,
+  `f3389` int DEFAULT NULL,
+  `f3390` int DEFAULT NULL,
+  `f3391` int DEFAULT NULL,
+  `f3392` int DEFAULT NULL,
+  `f3393` int DEFAULT NULL,
+  `f3394` int DEFAULT NULL,
+  `f3395` int DEFAULT NULL,
+  `f3396` int DEFAULT NULL,
+  `f3397` int DEFAULT NULL,
+  `f3398` int DEFAULT NULL,
+  `f3399` int DEFAULT NULL,
+  `f3400` int DEFAULT NULL,
+  `f3401` int DEFAULT NULL,
+  `f3402` int DEFAULT NULL,
+  `f3403` int DEFAULT NULL,
+  `f3404` int DEFAULT NULL,
+  `f3405` int DEFAULT NULL,
+  `f3406` int DEFAULT NULL,
+  `f3407` int DEFAULT NULL,
+  `f3408` int DEFAULT NULL,
+  `f3409` int DEFAULT NULL,
+  `f3410` int DEFAULT NULL,
+  `f3411` int DEFAULT NULL,
+  `f3412` int DEFAULT NULL,
+  `f3413` int DEFAULT NULL,
+  `f3414` int DEFAULT NULL,
+  `f3415` int DEFAULT NULL,
+  `f3416` int DEFAULT NULL,
+  `f3417` int DEFAULT NULL,
+  `f3418` int DEFAULT NULL,
+  `f3419` int DEFAULT NULL,
+  `f3420` int DEFAULT NULL,
+  `f3421` int DEFAULT NULL,
+  `f3422` int DEFAULT NULL,
+  `f3423` int DEFAULT NULL,
+  `f3424` int DEFAULT NULL,
+  `f3425` int DEFAULT NULL,
+  `f3426` int DEFAULT NULL,
+  `f3427` int DEFAULT NULL,
+  `f3428` int DEFAULT NULL,
+  `f3429` int DEFAULT NULL,
+  `f3430` int DEFAULT NULL,
+  `f3431` int DEFAULT NULL,
+  `f3432` int DEFAULT NULL,
+  `f3433` int DEFAULT NULL,
+  `f3434` int DEFAULT NULL,
+  `f3435` int DEFAULT NULL,
+  `f3436` int DEFAULT NULL,
+  `f3437` int DEFAULT NULL,
+  `f3438` int DEFAULT NULL,
+  `f3439` int DEFAULT NULL,
+  `f3440` int DEFAULT NULL,
+  `f3441` int DEFAULT NULL,
+  `f3442` int DEFAULT NULL,
+  `f3443` int DEFAULT NULL,
+  `f3444` int DEFAULT NULL,
+  `f3445` int DEFAULT NULL,
+  `f3446` int DEFAULT NULL,
+  `f3447` int DEFAULT NULL,
+  `f3448` int DEFAULT NULL,
+  `f3449` int DEFAULT NULL,
+  `f3450` int DEFAULT NULL,
+  `f3451` int DEFAULT NULL,
+  `f3452` int DEFAULT NULL,
+  `f3453` int DEFAULT NULL,
+  `f3454` int DEFAULT NULL,
+  `f3455` int DEFAULT NULL,
+  `f3456` int DEFAULT NULL,
+  `f3457` int DEFAULT NULL,
+  `f3458` int DEFAULT NULL,
+  `f3459` int DEFAULT NULL,
+  `f3460` int DEFAULT NULL,
+  `f3461` int DEFAULT NULL,
+  `f3462` int DEFAULT NULL,
+  `f3463` int DEFAULT NULL,
+  `f3464` int DEFAULT NULL,
+  `f3465` int DEFAULT NULL,
+  `f3466` int DEFAULT NULL,
+  `f3467` int DEFAULT NULL,
+  `f3468` int DEFAULT NULL,
+  `f3469` int DEFAULT NULL,
+  `f3470` int DEFAULT NULL,
+  `f3471` int DEFAULT NULL,
+  `f3472` int DEFAULT NULL,
+  `f3473` int DEFAULT NULL,
+  `f3474` int DEFAULT NULL,
+  `f3475` int DEFAULT NULL,
+  `f3476` int DEFAULT NULL,
+  `f3477` int DEFAULT NULL,
+  `f3478` int DEFAULT NULL,
+  `f3479` int DEFAULT NULL,
+  `f3480` int DEFAULT NULL,
+  `f3481` int DEFAULT NULL,
+  `f3482` int DEFAULT NULL,
+  `f3483` int DEFAULT NULL,
+  `f3484` int DEFAULT NULL,
+  `f3485` int DEFAULT NULL,
+  `f3486` int DEFAULT NULL,
+  `f3487` int DEFAULT NULL,
+  `f3488` int DEFAULT NULL,
+  `f3489` int DEFAULT NULL,
+  `f3490` int DEFAULT NULL,
+  `f3491` int DEFAULT NULL,
+  `f3492` int DEFAULT NULL,
+  `f3493` int DEFAULT NULL,
+  `f3494` int DEFAULT NULL,
+  `f3495` int DEFAULT NULL,
+  `f3496` int DEFAULT NULL,
+  `f3497` int DEFAULT NULL,
+  `f3498` int DEFAULT NULL,
+  `f3499` int DEFAULT NULL,
+  `f3500` int DEFAULT NULL,
+  `f3501` int DEFAULT NULL,
+  `f3502` int DEFAULT NULL,
+  `f3503` int DEFAULT NULL,
+  `f3504` int DEFAULT NULL,
+  `f3505` int DEFAULT NULL,
+  `f3506` int DEFAULT NULL,
+  `f3507` int DEFAULT NULL,
+  `f3508` int DEFAULT NULL,
+  `f3509` int DEFAULT NULL,
+  `f3510` int DEFAULT NULL,
+  `f3511` int DEFAULT NULL,
+  `f3512` int DEFAULT NULL,
+  `f3513` int DEFAULT NULL,
+  `f3514` int DEFAULT NULL,
+  `f3515` int DEFAULT NULL,
+  `f3516` int DEFAULT NULL,
+  `f3517` int DEFAULT NULL,
+  `f3518` int DEFAULT NULL,
+  `f3519` int DEFAULT NULL,
+  `f3520` int DEFAULT NULL,
+  `f3521` int DEFAULT NULL,
+  `f3522` int DEFAULT NULL,
+  `f3523` int DEFAULT NULL,
+  `f3524` int DEFAULT NULL,
+  `f3525` int DEFAULT NULL,
+  `f3526` int DEFAULT NULL,
+  `f3527` int DEFAULT NULL,
+  `f3528` int DEFAULT NULL,
+  `f3529` int DEFAULT NULL,
+  `f3530` int DEFAULT NULL,
+  `f3531` int DEFAULT NULL,
+  `f3532` int DEFAULT NULL,
+  `f3533` int DEFAULT NULL,
+  `f3534` int DEFAULT NULL,
+  `f3535` int DEFAULT NULL,
+  `f3536` int DEFAULT NULL,
+  `f3537` int DEFAULT NULL,
+  `f3538` int DEFAULT NULL,
+  `f3539` int DEFAULT NULL,
+  `f3540` int DEFAULT NULL,
+  `f3541` int DEFAULT NULL,
+  `f3542` int DEFAULT NULL,
+  `f3543` int DEFAULT NULL,
+  `f3544` int DEFAULT NULL,
+  `f3545` int DEFAULT NULL,
+  `f3546` int DEFAULT NULL,
+  `f3547` int DEFAULT NULL,
+  `f3548` int DEFAULT NULL,
+  `f3549` int DEFAULT NULL,
+  `f3550` int DEFAULT NULL,
+  `f3551` int DEFAULT NULL,
+  `f3552` int DEFAULT NULL,
+  `f3553` int DEFAULT NULL,
+  `f3554` int DEFAULT NULL,
+  `f3555` int DEFAULT NULL,
+  `f3556` int DEFAULT NULL,
+  `f3557` int DEFAULT NULL,
+  `f3558` int DEFAULT NULL,
+  `f3559` int DEFAULT NULL,
+  `f3560` int DEFAULT NULL,
+  `f3561` int DEFAULT NULL,
+  `f3562` int DEFAULT NULL,
+  `f3563` int DEFAULT NULL,
+  `f3564` int DEFAULT NULL,
+  `f3565` int DEFAULT NULL,
+  `f3566` int DEFAULT NULL,
+  `f3567` int DEFAULT NULL,
+  `f3568` int DEFAULT NULL,
+  `f3569` int DEFAULT NULL,
+  `f3570` int DEFAULT NULL,
+  `f3571` int DEFAULT NULL,
+  `f3572` int DEFAULT NULL,
+  `f3573` int DEFAULT NULL,
+  `f3574` int DEFAULT NULL,
+  `f3575` int DEFAULT NULL,
+  `f3576` int DEFAULT NULL,
+  `f3577` int DEFAULT NULL,
+  `f3578` int DEFAULT NULL,
+  `f3579` int DEFAULT NULL,
+  `f3580` int DEFAULT NULL,
+  `f3581` int DEFAULT NULL,
+  `f3582` int DEFAULT NULL,
+  `f3583` int DEFAULT NULL,
+  `f3584` int DEFAULT NULL,
+  `f3585` int DEFAULT NULL,
+  `f3586` int DEFAULT NULL,
+  `f3587` int DEFAULT NULL,
+  `f3588` int DEFAULT NULL,
+  `f3589` int DEFAULT NULL,
+  `f3590` int DEFAULT NULL,
+  `f3591` int DEFAULT NULL,
+  `f3592` int DEFAULT NULL,
+  `f3593` int DEFAULT NULL,
+  `f3594` int DEFAULT NULL,
+  `f3595` int DEFAULT NULL,
+  `f3596` int DEFAULT NULL,
+  `f3597` int DEFAULT NULL,
+  `f3598` int DEFAULT NULL,
+  `f3599` int DEFAULT NULL,
+  `f3600` int DEFAULT NULL,
+  `f3601` int DEFAULT NULL,
+  `f3602` int DEFAULT NULL,
+  `f3603` int DEFAULT NULL,
+  `f3604` int DEFAULT NULL,
+  `f3605` int DEFAULT NULL,
+  `f3606` int DEFAULT NULL,
+  `f3607` int DEFAULT NULL,
+  `f3608` int DEFAULT NULL,
+  `f3609` int DEFAULT NULL,
+  `f3610` int DEFAULT NULL,
+  `f3611` int DEFAULT NULL,
+  `f3612` int DEFAULT NULL,
+  `f3613` int DEFAULT NULL,
+  `f3614` int DEFAULT NULL,
+  `f3615` int DEFAULT NULL,
+  `f3616` int DEFAULT NULL,
+  `f3617` int DEFAULT NULL,
+  `f3618` int DEFAULT NULL,
+  `f3619` int DEFAULT NULL,
+  `f3620` int DEFAULT NULL,
+  `f3621` int DEFAULT NULL,
+  `f3622` int DEFAULT NULL,
+  `f3623` int DEFAULT NULL,
+  `f3624` int DEFAULT NULL,
+  `f3625` int DEFAULT NULL,
+  `f3626` int DEFAULT NULL,
+  `f3627` int DEFAULT NULL,
+  `f3628` int DEFAULT NULL,
+  `f3629` int DEFAULT NULL,
+  `f3630` int DEFAULT NULL,
+  `f3631` int DEFAULT NULL,
+  `f3632` int DEFAULT NULL,
+  `f3633` int DEFAULT NULL,
+  `f3634` int DEFAULT NULL,
+  `f3635` int DEFAULT NULL,
+  `f3636` int DEFAULT NULL,
+  `f3637` int DEFAULT NULL,
+  `f3638` int DEFAULT NULL,
+  `f3639` int DEFAULT NULL,
+  `f3640` int DEFAULT NULL,
+  `f3641` int DEFAULT NULL,
+  `f3642` int DEFAULT NULL,
+  `f3643` int DEFAULT NULL,
+  `f3644` int DEFAULT NULL,
+  `f3645` int DEFAULT NULL,
+  `f3646` int DEFAULT NULL,
+  `f3647` int DEFAULT NULL,
+  `f3648` int DEFAULT NULL,
+  `f3649` int DEFAULT NULL,
+  `f3650` int DEFAULT NULL,
+  `f3651` int DEFAULT NULL,
+  `f3652` int DEFAULT NULL,
+  `f3653` int DEFAULT NULL,
+  `f3654` int DEFAULT NULL,
+  `f3655` int DEFAULT NULL,
+  `f3656` int DEFAULT NULL,
+  `f3657` int DEFAULT NULL,
+  `f3658` int DEFAULT NULL,
+  `f3659` int DEFAULT NULL,
+  `f3660` int DEFAULT NULL,
+  `f3661` int DEFAULT NULL,
+  `f3662` int DEFAULT NULL,
+  `f3663` int DEFAULT NULL,
+  `f3664` int DEFAULT NULL,
+  `f3665` int DEFAULT NULL,
+  `f3666` int DEFAULT NULL,
+  `f3667` int DEFAULT NULL,
+  `f3668` int DEFAULT NULL,
+  `f3669` int DEFAULT NULL,
+  `f3670` int DEFAULT NULL,
+  `f3671` int DEFAULT NULL,
+  `f3672` int DEFAULT NULL,
+  `f3673` int DEFAULT NULL,
+  `f3674` int DEFAULT NULL,
+  `f3675` int DEFAULT NULL,
+  `f3676` int DEFAULT NULL,
+  `f3677` int DEFAULT NULL,
+  `f3678` int DEFAULT NULL,
+  `f3679` int DEFAULT NULL,
+  `f3680` int DEFAULT NULL,
+  `f3681` int DEFAULT NULL,
+  `f3682` int DEFAULT NULL,
+  `f3683` int DEFAULT NULL,
+  `f3684` int DEFAULT NULL,
+  `f3685` int DEFAULT NULL,
+  `f3686` int DEFAULT NULL,
+  `f3687` int DEFAULT NULL,
+  `f3688` int DEFAULT NULL,
+  `f3689` int DEFAULT NULL,
+  `f3690` int DEFAULT NULL,
+  `f3691` int DEFAULT NULL,
+  `f3692` int DEFAULT NULL,
+  `f3693` int DEFAULT NULL,
+  `f3694` int DEFAULT NULL,
+  `f3695` int DEFAULT NULL,
+  `f3696` int DEFAULT NULL,
+  `f3697` int DEFAULT NULL,
+  `f3698` int DEFAULT NULL,
+  `f3699` int DEFAULT NULL,
+  `f3700` int DEFAULT NULL,
+  `f3701` int DEFAULT NULL,
+  `f3702` int DEFAULT NULL,
+  `f3703` int DEFAULT NULL,
+  `f3704` int DEFAULT NULL,
+  `f3705` int DEFAULT NULL,
+  `f3706` int DEFAULT NULL,
+  `f3707` int DEFAULT NULL,
+  `f3708` int DEFAULT NULL,
+  `f3709` int DEFAULT NULL,
+  `f3710` int DEFAULT NULL,
+  `f3711` int DEFAULT NULL,
+  `f3712` int DEFAULT NULL,
+  `f3713` int DEFAULT NULL,
+  `f3714` int DEFAULT NULL,
+  `f3715` int DEFAULT NULL,
+  `f3716` int DEFAULT NULL,
+  `f3717` int DEFAULT NULL,
+  `f3718` int DEFAULT NULL,
+  `f3719` int DEFAULT NULL,
+  `f3720` int DEFAULT NULL,
+  `f3721` int DEFAULT NULL,
+  `f3722` int DEFAULT NULL,
+  `f3723` int DEFAULT NULL,
+  `f3724` int DEFAULT NULL,
+  `f3725` int DEFAULT NULL,
+  `f3726` int DEFAULT NULL,
+  `f3727` int DEFAULT NULL,
+  `f3728` int DEFAULT NULL,
+  `f3729` int DEFAULT NULL,
+  `f3730` int DEFAULT NULL,
+  `f3731` int DEFAULT NULL,
+  `f3732` int DEFAULT NULL,
+  `f3733` int DEFAULT NULL,
+  `f3734` int DEFAULT NULL,
+  `f3735` int DEFAULT NULL,
+  `f3736` int DEFAULT NULL,
+  `f3737` int DEFAULT NULL,
+  `f3738` int DEFAULT NULL,
+  `f3739` int DEFAULT NULL,
+  `f3740` int DEFAULT NULL,
+  `f3741` int DEFAULT NULL,
+  `f3742` int DEFAULT NULL,
+  `f3743` int DEFAULT NULL,
+  `f3744` int DEFAULT NULL,
+  `f3745` int DEFAULT NULL,
+  `f3746` int DEFAULT NULL,
+  `f3747` int DEFAULT NULL,
+  `f3748` int DEFAULT NULL,
+  `f3749` int DEFAULT NULL,
+  `f3750` int DEFAULT NULL,
+  `f3751` int DEFAULT NULL,
+  `f3752` int DEFAULT NULL,
+  `f3753` int DEFAULT NULL,
+  `f3754` int DEFAULT NULL,
+  `f3755` int DEFAULT NULL,
+  `f3756` int DEFAULT NULL,
+  `f3757` int DEFAULT NULL,
+  `f3758` int DEFAULT NULL,
+  `f3759` int DEFAULT NULL,
+  `f3760` int DEFAULT NULL,
+  `f3761` int DEFAULT NULL,
+  `f3762` int DEFAULT NULL,
+  `f3763` int DEFAULT NULL,
+  `f3764` int DEFAULT NULL,
+  `f3765` int DEFAULT NULL,
+  `f3766` int DEFAULT NULL,
+  `f3767` int DEFAULT NULL,
+  `f3768` int DEFAULT NULL,
+  `f3769` int DEFAULT NULL,
+  `f3770` int DEFAULT NULL,
+  `f3771` int DEFAULT NULL,
+  `f3772` int DEFAULT NULL,
+  `f3773` int DEFAULT NULL,
+  `f3774` int DEFAULT NULL,
+  `f3775` int DEFAULT NULL,
+  `f3776` int DEFAULT NULL,
+  `f3777` int DEFAULT NULL,
+  `f3778` int DEFAULT NULL,
+  `f3779` int DEFAULT NULL,
+  `f3780` int DEFAULT NULL,
+  `f3781` int DEFAULT NULL,
+  `f3782` int DEFAULT NULL,
+  `f3783` int DEFAULT NULL,
+  `f3784` int DEFAULT NULL,
+  `f3785` int DEFAULT NULL,
+  `f3786` int DEFAULT NULL,
+  `f3787` int DEFAULT NULL,
+  `f3788` int DEFAULT NULL,
+  `f3789` int DEFAULT NULL,
+  `f3790` int DEFAULT NULL,
+  `f3791` int DEFAULT NULL,
+  `f3792` int DEFAULT NULL,
+  `f3793` int DEFAULT NULL,
+  `f3794` int DEFAULT NULL,
+  `f3795` int DEFAULT NULL,
+  `f3796` int DEFAULT NULL,
+  `f3797` int DEFAULT NULL,
+  `f3798` int DEFAULT NULL,
+  `f3799` int DEFAULT NULL,
+  `f3800` int DEFAULT NULL,
+  `f3801` int DEFAULT NULL,
+  `f3802` int DEFAULT NULL,
+  `f3803` int DEFAULT NULL,
+  `f3804` int DEFAULT NULL,
+  `f3805` int DEFAULT NULL,
+  `f3806` int DEFAULT NULL,
+  `f3807` int DEFAULT NULL,
+  `f3808` int DEFAULT NULL,
+  `f3809` int DEFAULT NULL,
+  `f3810` int DEFAULT NULL,
+  `f3811` int DEFAULT NULL,
+  `f3812` int DEFAULT NULL,
+  `f3813` int DEFAULT NULL,
+  `f3814` int DEFAULT NULL,
+  `f3815` int DEFAULT NULL,
+  `f3816` int DEFAULT NULL,
+  `f3817` int DEFAULT NULL,
+  `f3818` int DEFAULT NULL,
+  `f3819` int DEFAULT NULL,
+  `f3820` int DEFAULT NULL,
+  `f3821` int DEFAULT NULL,
+  `f3822` int DEFAULT NULL,
+  `f3823` int DEFAULT NULL,
+  `f3824` int DEFAULT NULL,
+  `f3825` int DEFAULT NULL,
+  `f3826` int DEFAULT NULL,
+  `f3827` int DEFAULT NULL,
+  `f3828` int DEFAULT NULL,
+  `f3829` int DEFAULT NULL,
+  `f3830` int DEFAULT NULL,
+  `f3831` int DEFAULT NULL,
+  `f3832` int DEFAULT NULL,
+  `f3833` int DEFAULT NULL,
+  `f3834` int DEFAULT NULL,
+  `f3835` int DEFAULT NULL,
+  `f3836` int DEFAULT NULL,
+  `f3837` int DEFAULT NULL,
+  `f3838` int DEFAULT NULL,
+  `f3839` int DEFAULT NULL,
+  `f3840` int DEFAULT NULL,
+  `f3841` int DEFAULT NULL,
+  `f3842` int DEFAULT NULL,
+  `f3843` int DEFAULT NULL,
+  `f3844` int DEFAULT NULL,
+  `f3845` int DEFAULT NULL,
+  `f3846` int DEFAULT NULL,
+  `f3847` int DEFAULT NULL,
+  `f3848` int DEFAULT NULL,
+  `f3849` int DEFAULT NULL,
+  `f3850` int DEFAULT NULL,
+  `f3851` int DEFAULT NULL,
+  `f3852` int DEFAULT NULL,
+  `f3853` int DEFAULT NULL,
+  `f3854` int DEFAULT NULL,
+  `f3855` int DEFAULT NULL,
+  `f3856` int DEFAULT NULL,
+  `f3857` int DEFAULT NULL,
+  `f3858` int DEFAULT NULL,
+  `f3859` int DEFAULT NULL,
+  `f3860` int DEFAULT NULL,
+  `f3861` int DEFAULT NULL,
+  `f3862` int DEFAULT NULL,
+  `f3863` int DEFAULT NULL,
+  `f3864` int DEFAULT NULL,
+  `f3865` int DEFAULT NULL,
+  `f3866` int DEFAULT NULL,
+  `f3867` int DEFAULT NULL,
+  `f3868` int DEFAULT NULL,
+  `f3869` int DEFAULT NULL,
+  `f3870` int DEFAULT NULL,
+  `f3871` int DEFAULT NULL,
+  `f3872` int DEFAULT NULL,
+  `f3873` int DEFAULT NULL,
+  `f3874` int DEFAULT NULL,
+  `f3875` int DEFAULT NULL,
+  `f3876` int DEFAULT NULL,
+  `f3877` int DEFAULT NULL,
+  `f3878` int DEFAULT NULL,
+  `f3879` int DEFAULT NULL,
+  `f3880` int DEFAULT NULL,
+  `f3881` int DEFAULT NULL,
+  `f3882` int DEFAULT NULL,
+  `f3883` int DEFAULT NULL,
+  `f3884` int DEFAULT NULL,
+  `f3885` int DEFAULT NULL,
+  `f3886` int DEFAULT NULL,
+  `f3887` int DEFAULT NULL,
+  `f3888` int DEFAULT NULL,
+  `f3889` int DEFAULT NULL,
+  `f3890` int DEFAULT NULL,
+  `f3891` int DEFAULT NULL,
+  `f3892` int DEFAULT NULL,
+  `f3893` int DEFAULT NULL,
+  `f3894` int DEFAULT NULL,
+  `f3895` int DEFAULT NULL,
+  `f3896` int DEFAULT NULL,
+  `f3897` int DEFAULT NULL,
+  `f3898` int DEFAULT NULL,
+  `f3899` int DEFAULT NULL,
+  `f3900` int DEFAULT NULL,
+  `f3901` int DEFAULT NULL,
+  `f3902` int DEFAULT NULL,
+  `f3903` int DEFAULT NULL,
+  `f3904` int DEFAULT NULL,
+  `f3905` int DEFAULT NULL,
+  `f3906` int DEFAULT NULL,
+  `f3907` int DEFAULT NULL,
+  `f3908` int DEFAULT NULL,
+  `f3909` int DEFAULT NULL,
+  `f3910` int DEFAULT NULL,
+  `f3911` int DEFAULT NULL,
+  `f3912` int DEFAULT NULL,
+  `f3913` int DEFAULT NULL,
+  `f3914` int DEFAULT NULL,
+  `f3915` int DEFAULT NULL,
+  `f3916` int DEFAULT NULL,
+  `f3917` int DEFAULT NULL,
+  `f3918` int DEFAULT NULL,
+  `f3919` int DEFAULT NULL,
+  `f3920` int DEFAULT NULL,
+  `f3921` int DEFAULT NULL,
+  `f3922` int DEFAULT NULL,
+  `f3923` int DEFAULT NULL,
+  `f3924` int DEFAULT NULL,
+  `f3925` int DEFAULT NULL,
+  `f3926` int DEFAULT NULL,
+  `f3927` int DEFAULT NULL,
+  `f3928` int DEFAULT NULL,
+  `f3929` int DEFAULT NULL,
+  `f3930` int DEFAULT NULL,
+  `f3931` int DEFAULT NULL,
+  `f3932` int DEFAULT NULL,
+  `f3933` int DEFAULT NULL,
+  `f3934` int DEFAULT NULL,
+  `f3935` int DEFAULT NULL,
+  `f3936` int DEFAULT NULL,
+  `f3937` int DEFAULT NULL,
+  `f3938` int DEFAULT NULL,
+  `f3939` int DEFAULT NULL,
+  `f3940` int DEFAULT NULL,
+  `f3941` int DEFAULT NULL,
+  `f3942` int DEFAULT NULL,
+  `f3943` int DEFAULT NULL,
+  `f3944` int DEFAULT NULL,
+  `f3945` int DEFAULT NULL,
+  `f3946` int DEFAULT NULL,
+  `f3947` int DEFAULT NULL,
+  `f3948` int DEFAULT NULL,
+  `f3949` int DEFAULT NULL,
+  `f3950` int DEFAULT NULL,
+  `f3951` int DEFAULT NULL,
+  `f3952` int DEFAULT NULL,
+  `f3953` int DEFAULT NULL,
+  `f3954` int DEFAULT NULL,
+  `f3955` int DEFAULT NULL,
+  `f3956` int DEFAULT NULL,
+  `f3957` int DEFAULT NULL,
+  `f3958` int DEFAULT NULL,
+  `f3959` int DEFAULT NULL,
+  `f3960` int DEFAULT NULL,
+  `f3961` int DEFAULT NULL,
+  `f3962` int DEFAULT NULL,
+  `f3963` int DEFAULT NULL,
+  `f3964` int DEFAULT NULL,
+  `f3965` int DEFAULT NULL,
+  `f3966` int DEFAULT NULL,
+  `f3967` int DEFAULT NULL,
+  `f3968` int DEFAULT NULL,
+  `f3969` int DEFAULT NULL,
+  `f3970` int DEFAULT NULL,
+  `f3971` int DEFAULT NULL,
+  `f3972` int DEFAULT NULL,
+  `f3973` int DEFAULT NULL,
+  `f3974` int DEFAULT NULL,
+  `f3975` int DEFAULT NULL,
+  `f3976` int DEFAULT NULL,
+  `f3977` int DEFAULT NULL,
+  `f3978` int DEFAULT NULL,
+  `f3979` int DEFAULT NULL,
+  `f3980` int DEFAULT NULL,
+  `f3981` int DEFAULT NULL,
+  `f3982` int DEFAULT NULL,
+  `f3983` int DEFAULT NULL,
+  `f3984` int DEFAULT NULL,
+  `f3985` int DEFAULT NULL,
+  `f3986` int DEFAULT NULL,
+  `f3987` int DEFAULT NULL,
+  `f3988` int DEFAULT NULL,
+  `f3989` int DEFAULT NULL,
+  `f3990` int DEFAULT NULL,
+  `f3991` int DEFAULT NULL,
+  `f3992` int DEFAULT NULL,
+  `f3993` int DEFAULT NULL,
+  `f3994` int DEFAULT NULL,
+  `f3995` int DEFAULT NULL,
+  `f3996` int DEFAULT NULL,
+  `f3997` int DEFAULT NULL,
+  `f3998` int DEFAULT NULL,
+  `f3999` int DEFAULT NULL,
+  `f4000` int DEFAULT NULL,
+  `f4001` int DEFAULT NULL,
+  `f4002` int DEFAULT NULL,
+  `f4003` int DEFAULT NULL,
+  `f4004` int DEFAULT NULL,
+  `f4005` int DEFAULT NULL,
+  `f4006` int DEFAULT NULL,
+  `f4007` int DEFAULT NULL,
+  `f4008` int DEFAULT NULL,
+  `f4009` int DEFAULT NULL,
+  `f4010` int DEFAULT NULL,
+  `f4011` int DEFAULT NULL,
+  `f4012` int DEFAULT NULL,
+  `f4013` int DEFAULT NULL,
+  `f4014` int DEFAULT NULL,
+  `f4015` int DEFAULT NULL,
+  `f4016` int DEFAULT NULL,
+  `f4017` int DEFAULT NULL,
+  `f4018` int DEFAULT NULL,
+  `f4019` int DEFAULT NULL,
+  `f4020` int DEFAULT NULL,
+  `f4021` int DEFAULT NULL,
+  `f4022` int DEFAULT NULL,
+  `f4023` int DEFAULT NULL,
+  `f4024` int DEFAULT NULL,
+  `f4025` int DEFAULT NULL,
+  `f4026` int DEFAULT NULL,
+  `f4027` int DEFAULT NULL,
+  `f4028` int DEFAULT NULL,
+  `f4029` int DEFAULT NULL,
+  `f4030` int DEFAULT NULL,
+  `f4031` int DEFAULT NULL,
+  `f4032` int DEFAULT NULL,
+  `f4033` int DEFAULT NULL,
+  `f4034` int DEFAULT NULL,
+  `f4035` int DEFAULT NULL,
+  `f4036` int DEFAULT NULL,
+  `f4037` int DEFAULT NULL,
+  `f4038` int DEFAULT NULL,
+  `f4039` int DEFAULT NULL,
+  `f4040` int DEFAULT NULL,
+  `f4041` int DEFAULT NULL,
+  `f4042` int DEFAULT NULL,
+  `f4043` int DEFAULT NULL,
+  `f4044` int DEFAULT NULL,
+  `f4045` int DEFAULT NULL,
+  `f4046` int DEFAULT NULL,
+  `f4047` int DEFAULT NULL,
+  `f4048` int DEFAULT NULL,
+  `f4049` int DEFAULT NULL,
+  `f4050` int DEFAULT NULL,
+  `f4051` int DEFAULT NULL,
+  `f4052` int DEFAULT NULL,
+  `f4053` int DEFAULT NULL,
+  `f4054` int DEFAULT NULL,
+  `f4055` int DEFAULT NULL,
+  `f4056` int DEFAULT NULL,
+  `f4057` int DEFAULT NULL,
+  `f4058` int DEFAULT NULL,
+  `f4059` int DEFAULT NULL,
+  `f4060` int DEFAULT NULL,
+  `f4061` int DEFAULT NULL,
+  `f4062` int DEFAULT NULL,
+  `f4063` int DEFAULT NULL,
+  `f4064` int DEFAULT NULL,
+  `f4065` int DEFAULT NULL,
+  `f4066` int DEFAULT NULL,
+  `f4067` int DEFAULT NULL,
+  `f4068` int DEFAULT NULL,
+  `f4069` int DEFAULT NULL,
+  `f4070` int DEFAULT NULL,
+  `f4071` int DEFAULT NULL,
+  `f4072` int DEFAULT NULL,
+  `f4073` int DEFAULT NULL,
+  `f4074` int DEFAULT NULL,
+  `f4075` int DEFAULT NULL,
+  `f4076` int DEFAULT NULL,
+  `f4077` int DEFAULT NULL,
+  `f4078` int DEFAULT NULL,
+  `f4079` int DEFAULT NULL,
+  `f4080` int DEFAULT NULL,
+  `f4081` int DEFAULT NULL,
+  `f4082` int DEFAULT NULL,
+  `f4083` int DEFAULT NULL,
+  `f4084` int DEFAULT NULL,
+  `f4085` int DEFAULT NULL,
+  `f4086` int DEFAULT NULL,
+  `f4087` int DEFAULT NULL,
+  `f4088` int DEFAULT NULL,
+  `f4089` int DEFAULT NULL,
+  `f4090` int DEFAULT NULL,
+  `f4091` int DEFAULT NULL,
+  `f4092` int DEFAULT NULL,
+  `f4093` int DEFAULT NULL,
+  `f4094` int DEFAULT NULL,
+  `f4095` int DEFAULT NULL,
+  `f4096` int DEFAULT NULL
+) ENGINE=BLACKHOLE
+DROP TABLE t1;
+Now, MAX_FIELDS. This fails.
+CREATE TABLE t1 (
+f0 int,
+f1 int,
+f2 int,
+f3 int,
+f4 int,
+f5 int,
+f6 int,
+f7 int,
+f8 int,
+f9 int,
+f10 int,
+f11 int,
+f12 int,
+f13 int,
+f14 int,
+f15 int,
+f16 int,
+f17 int,
+f18 int,
+f19 int,
+f20 int,
+f21 int,
+f22 int,
+f23 int,
+f24 int,
+f25 int,
+f26 int,
+f27 int,
+f28 int,
+f29 int,
+f30 int,
+f31 int,
+f32 int,
+f33 int,
+f34 int,
+f35 int,
+f36 int,
+f37 int,
+f38 int,
+f39 int,
+f40 int,
+f41 int,
+f42 int,
+f43 int,
+f44 int,
+f45 int,
+f46 int,
+f47 int,
+f48 int,
+f49 int,
+f50 int,
+f51 int,
+f52 int,
+f53 int,
+f54 int,
+f55 int,
+f56 int,
+f57 int,
+f58 int,
+f59 int,
+f60 int,
+f61 int,
+f62 int,
+f63 int,
+f64 int,
+f65 int,
+f66 int,
+f67 int,
+f68 int,
+f69 int,
+f70 int,
+f71 int,
+f72 int,
+f73 int,
+f74 int,
+f75 int,
+f76 int,
+f77 int,
+f78 int,
+f79 int,
+f80 int,
+f81 int,
+f82 int,
+f83 int,
+f84 int,
+f85 int,
+f86 int,
+f87 int,
+f88 int,
+f89 int,
+f90 int,
+f91 int,
+f92 int,
+f93 int,
+f94 int,
+f95 int,
+f96 int,
+f97 int,
+f98 int,
+f99 int,
+f100 int,
+f101 int,
+f102 int,
+f103 int,
+f104 int,
+f105 int,
+f106 int,
+f107 int,
+f108 int,
+f109 int,
+f110 int,
+f111 int,
+f112 int,
+f113 int,
+f114 int,
+f115 int,
+f116 int,
+f117 int,
+f118 int,
+f119 int,
+f120 int,
+f121 int,
+f122 int,
+f123 int,
+f124 int,
+f125 int,
+f126 int,
+f127 int,
+f128 int,
+f129 int,
+f130 int,
+f131 int,
+f132 int,
+f133 int,
+f134 int,
+f135 int,
+f136 int,
+f137 int,
+f138 int,
+f139 int,
+f140 int,
+f141 int,
+f142 int,
+f143 int,
+f144 int,
+f145 int,
+f146 int,
+f147 int,
+f148 int,
+f149 int,
+f150 int,
+f151 int,
+f152 int,
+f153 int,
+f154 int,
+f155 int,
+f156 int,
+f157 int,
+f158 int,
+f159 int,
+f160 int,
+f161 int,
+f162 int,
+f163 int,
+f164 int,
+f165 int,
+f166 int,
+f167 int,
+f168 int,
+f169 int,
+f170 int,
+f171 int,
+f172 int,
+f173 int,
+f174 int,
+f175 int,
+f176 int,
+f177 int,
+f178 int,
+f179 int,
+f180 int,
+f181 int,
+f182 int,
+f183 int,
+f184 int,
+f185 int,
+f186 int,
+f187 int,
+f188 int,
+f189 int,
+f190 int,
+f191 int,
+f192 int,
+f193 int,
+f194 int,
+f195 int,
+f196 int,
+f197 int,
+f198 int,
+f199 int,
+f200 int,
+f201 int,
+f202 int,
+f203 int,
+f204 int,
+f205 int,
+f206 int,
+f207 int,
+f208 int,
+f209 int,
+f210 int,
+f211 int,
+f212 int,
+f213 int,
+f214 int,
+f215 int,
+f216 int,
+f217 int,
+f218 int,
+f219 int,
+f220 int,
+f221 int,
+f222 int,
+f223 int,
+f224 int,
+f225 int,
+f226 int,
+f227 int,
+f228 int,
+f229 int,
+f230 int,
+f231 int,
+f232 int,
+f233 int,
+f234 int,
+f235 int,
+f236 int,
+f237 int,
+f238 int,
+f239 int,
+f240 int,
+f241 int,
+f242 int,
+f243 int,
+f244 int,
+f245 int,
+f246 int,
+f247 int,
+f248 int,
+f249 int,
+f250 int,
+f251 int,
+f252 int,
+f253 int,
+f254 int,
+f255 int,
+f256 int,
+f257 int,
+f258 int,
+f259 int,
+f260 int,
+f261 int,
+f262 int,
+f263 int,
+f264 int,
+f265 int,
+f266 int,
+f267 int,
+f268 int,
+f269 int,
+f270 int,
+f271 int,
+f272 int,
+f273 int,
+f274 int,
+f275 int,
+f276 int,
+f277 int,
+f278 int,
+f279 int,
+f280 int,
+f281 int,
+f282 int,
+f283 int,
+f284 int,
+f285 int,
+f286 int,
+f287 int,
+f288 int,
+f289 int,
+f290 int,
+f291 int,
+f292 int,
+f293 int,
+f294 int,
+f295 int,
+f296 int,
+f297 int,
+f298 int,
+f299 int,
+f300 int,
+f301 int,
+f302 int,
+f303 int,
+f304 int,
+f305 int,
+f306 int,
+f307 int,
+f308 int,
+f309 int,
+f310 int,
+f311 int,
+f312 int,
+f313 int,
+f314 int,
+f315 int,
+f316 int,
+f317 int,
+f318 int,
+f319 int,
+f320 int,
+f321 int,
+f322 int,
+f323 int,
+f324 int,
+f325 int,
+f326 int,
+f327 int,
+f328 int,
+f329 int,
+f330 int,
+f331 int,
+f332 int,
+f333 int,
+f334 int,
+f335 int,
+f336 int,
+f337 int,
+f338 int,
+f339 int,
+f340 int,
+f341 int,
+f342 int,
+f343 int,
+f344 int,
+f345 int,
+f346 int,
+f347 int,
+f348 int,
+f349 int,
+f350 int,
+f351 int,
+f352 int,
+f353 int,
+f354 int,
+f355 int,
+f356 int,
+f357 int,
+f358 int,
+f359 int,
+f360 int,
+f361 int,
+f362 int,
+f363 int,
+f364 int,
+f365 int,
+f366 int,
+f367 int,
+f368 int,
+f369 int,
+f370 int,
+f371 int,
+f372 int,
+f373 int,
+f374 int,
+f375 int,
+f376 int,
+f377 int,
+f378 int,
+f379 int,
+f380 int,
+f381 int,
+f382 int,
+f383 int,
+f384 int,
+f385 int,
+f386 int,
+f387 int,
+f388 int,
+f389 int,
+f390 int,
+f391 int,
+f392 int,
+f393 int,
+f394 int,
+f395 int,
+f396 int,
+f397 int,
+f398 int,
+f399 int,
+f400 int,
+f401 int,
+f402 int,
+f403 int,
+f404 int,
+f405 int,
+f406 int,
+f407 int,
+f408 int,
+f409 int,
+f410 int,
+f411 int,
+f412 int,
+f413 int,
+f414 int,
+f415 int,
+f416 int,
+f417 int,
+f418 int,
+f419 int,
+f420 int,
+f421 int,
+f422 int,
+f423 int,
+f424 int,
+f425 int,
+f426 int,
+f427 int,
+f428 int,
+f429 int,
+f430 int,
+f431 int,
+f432 int,
+f433 int,
+f434 int,
+f435 int,
+f436 int,
+f437 int,
+f438 int,
+f439 int,
+f440 int,
+f441 int,
+f442 int,
+f443 int,
+f444 int,
+f445 int,
+f446 int,
+f447 int,
+f448 int,
+f449 int,
+f450 int,
+f451 int,
+f452 int,
+f453 int,
+f454 int,
+f455 int,
+f456 int,
+f457 int,
+f458 int,
+f459 int,
+f460 int,
+f461 int,
+f462 int,
+f463 int,
+f464 int,
+f465 int,
+f466 int,
+f467 int,
+f468 int,
+f469 int,
+f470 int,
+f471 int,
+f472 int,
+f473 int,
+f474 int,
+f475 int,
+f476 int,
+f477 int,
+f478 int,
+f479 int,
+f480 int,
+f481 int,
+f482 int,
+f483 int,
+f484 int,
+f485 int,
+f486 int,
+f487 int,
+f488 int,
+f489 int,
+f490 int,
+f491 int,
+f492 int,
+f493 int,
+f494 int,
+f495 int,
+f496 int,
+f497 int,
+f498 int,
+f499 int,
+f500 int,
+f501 int,
+f502 int,
+f503 int,
+f504 int,
+f505 int,
+f506 int,
+f507 int,
+f508 int,
+f509 int,
+f510 int,
+f511 int,
+f512 int,
+f513 int,
+f514 int,
+f515 int,
+f516 int,
+f517 int,
+f518 int,
+f519 int,
+f520 int,
+f521 int,
+f522 int,
+f523 int,
+f524 int,
+f525 int,
+f526 int,
+f527 int,
+f528 int,
+f529 int,
+f530 int,
+f531 int,
+f532 int,
+f533 int,
+f534 int,
+f535 int,
+f536 int,
+f537 int,
+f538 int,
+f539 int,
+f540 int,
+f541 int,
+f542 int,
+f543 int,
+f544 int,
+f545 int,
+f546 int,
+f547 int,
+f548 int,
+f549 int,
+f550 int,
+f551 int,
+f552 int,
+f553 int,
+f554 int,
+f555 int,
+f556 int,
+f557 int,
+f558 int,
+f559 int,
+f560 int,
+f561 int,
+f562 int,
+f563 int,
+f564 int,
+f565 int,
+f566 int,
+f567 int,
+f568 int,
+f569 int,
+f570 int,
+f571 int,
+f572 int,
+f573 int,
+f574 int,
+f575 int,
+f576 int,
+f577 int,
+f578 int,
+f579 int,
+f580 int,
+f581 int,
+f582 int,
+f583 int,
+f584 int,
+f585 int,
+f586 int,
+f587 int,
+f588 int,
+f589 int,
+f590 int,
+f591 int,
+f592 int,
+f593 int,
+f594 int,
+f595 int,
+f596 int,
+f597 int,
+f598 int,
+f599 int,
+f600 int,
+f601 int,
+f602 int,
+f603 int,
+f604 int,
+f605 int,
+f606 int,
+f607 int,
+f608 int,
+f609 int,
+f610 int,
+f611 int,
+f612 int,
+f613 int,
+f614 int,
+f615 int,
+f616 int,
+f617 int,
+f618 int,
+f619 int,
+f620 int,
+f621 int,
+f622 int,
+f623 int,
+f624 int,
+f625 int,
+f626 int,
+f627 int,
+f628 int,
+f629 int,
+f630 int,
+f631 int,
+f632 int,
+f633 int,
+f634 int,
+f635 int,
+f636 int,
+f637 int,
+f638 int,
+f639 int,
+f640 int,
+f641 int,
+f642 int,
+f643 int,
+f644 int,
+f645 int,
+f646 int,
+f647 int,
+f648 int,
+f649 int,
+f650 int,
+f651 int,
+f652 int,
+f653 int,
+f654 int,
+f655 int,
+f656 int,
+f657 int,
+f658 int,
+f659 int,
+f660 int,
+f661 int,
+f662 int,
+f663 int,
+f664 int,
+f665 int,
+f666 int,
+f667 int,
+f668 int,
+f669 int,
+f670 int,
+f671 int,
+f672 int,
+f673 int,
+f674 int,
+f675 int,
+f676 int,
+f677 int,
+f678 int,
+f679 int,
+f680 int,
+f681 int,
+f682 int,
+f683 int,
+f684 int,
+f685 int,
+f686 int,
+f687 int,
+f688 int,
+f689 int,
+f690 int,
+f691 int,
+f692 int,
+f693 int,
+f694 int,
+f695 int,
+f696 int,
+f697 int,
+f698 int,
+f699 int,
+f700 int,
+f701 int,
+f702 int,
+f703 int,
+f704 int,
+f705 int,
+f706 int,
+f707 int,
+f708 int,
+f709 int,
+f710 int,
+f711 int,
+f712 int,
+f713 int,
+f714 int,
+f715 int,
+f716 int,
+f717 int,
+f718 int,
+f719 int,
+f720 int,
+f721 int,
+f722 int,
+f723 int,
+f724 int,
+f725 int,
+f726 int,
+f727 int,
+f728 int,
+f729 int,
+f730 int,
+f731 int,
+f732 int,
+f733 int,
+f734 int,
+f735 int,
+f736 int,
+f737 int,
+f738 int,
+f739 int,
+f740 int,
+f741 int,
+f742 int,
+f743 int,
+f744 int,
+f745 int,
+f746 int,
+f747 int,
+f748 int,
+f749 int,
+f750 int,
+f751 int,
+f752 int,
+f753 int,
+f754 int,
+f755 int,
+f756 int,
+f757 int,
+f758 int,
+f759 int,
+f760 int,
+f761 int,
+f762 int,
+f763 int,
+f764 int,
+f765 int,
+f766 int,
+f767 int,
+f768 int,
+f769 int,
+f770 int,
+f771 int,
+f772 int,
+f773 int,
+f774 int,
+f775 int,
+f776 int,
+f777 int,
+f778 int,
+f779 int,
+f780 int,
+f781 int,
+f782 int,
+f783 int,
+f784 int,
+f785 int,
+f786 int,
+f787 int,
+f788 int,
+f789 int,
+f790 int,
+f791 int,
+f792 int,
+f793 int,
+f794 int,
+f795 int,
+f796 int,
+f797 int,
+f798 int,
+f799 int,
+f800 int,
+f801 int,
+f802 int,
+f803 int,
+f804 int,
+f805 int,
+f806 int,
+f807 int,
+f808 int,
+f809 int,
+f810 int,
+f811 int,
+f812 int,
+f813 int,
+f814 int,
+f815 int,
+f816 int,
+f817 int,
+f818 int,
+f819 int,
+f820 int,
+f821 int,
+f822 int,
+f823 int,
+f824 int,
+f825 int,
+f826 int,
+f827 int,
+f828 int,
+f829 int,
+f830 int,
+f831 int,
+f832 int,
+f833 int,
+f834 int,
+f835 int,
+f836 int,
+f837 int,
+f838 int,
+f839 int,
+f840 int,
+f841 int,
+f842 int,
+f843 int,
+f844 int,
+f845 int,
+f846 int,
+f847 int,
+f848 int,
+f849 int,
+f850 int,
+f851 int,
+f852 int,
+f853 int,
+f854 int,
+f855 int,
+f856 int,
+f857 int,
+f858 int,
+f859 int,
+f860 int,
+f861 int,
+f862 int,
+f863 int,
+f864 int,
+f865 int,
+f866 int,
+f867 int,
+f868 int,
+f869 int,
+f870 int,
+f871 int,
+f872 int,
+f873 int,
+f874 int,
+f875 int,
+f876 int,
+f877 int,
+f878 int,
+f879 int,
+f880 int,
+f881 int,
+f882 int,
+f883 int,
+f884 int,
+f885 int,
+f886 int,
+f887 int,
+f888 int,
+f889 int,
+f890 int,
+f891 int,
+f892 int,
+f893 int,
+f894 int,
+f895 int,
+f896 int,
+f897 int,
+f898 int,
+f899 int,
+f900 int,
+f901 int,
+f902 int,
+f903 int,
+f904 int,
+f905 int,
+f906 int,
+f907 int,
+f908 int,
+f909 int,
+f910 int,
+f911 int,
+f912 int,
+f913 int,
+f914 int,
+f915 int,
+f916 int,
+f917 int,
+f918 int,
+f919 int,
+f920 int,
+f921 int,
+f922 int,
+f923 int,
+f924 int,
+f925 int,
+f926 int,
+f927 int,
+f928 int,
+f929 int,
+f930 int,
+f931 int,
+f932 int,
+f933 int,
+f934 int,
+f935 int,
+f936 int,
+f937 int,
+f938 int,
+f939 int,
+f940 int,
+f941 int,
+f942 int,
+f943 int,
+f944 int,
+f945 int,
+f946 int,
+f947 int,
+f948 int,
+f949 int,
+f950 int,
+f951 int,
+f952 int,
+f953 int,
+f954 int,
+f955 int,
+f956 int,
+f957 int,
+f958 int,
+f959 int,
+f960 int,
+f961 int,
+f962 int,
+f963 int,
+f964 int,
+f965 int,
+f966 int,
+f967 int,
+f968 int,
+f969 int,
+f970 int,
+f971 int,
+f972 int,
+f973 int,
+f974 int,
+f975 int,
+f976 int,
+f977 int,
+f978 int,
+f979 int,
+f980 int,
+f981 int,
+f982 int,
+f983 int,
+f984 int,
+f985 int,
+f986 int,
+f987 int,
+f988 int,
+f989 int,
+f990 int,
+f991 int,
+f992 int,
+f993 int,
+f994 int,
+f995 int,
+f996 int,
+f997 int,
+f998 int,
+f999 int,
+f1000 int,
+f1001 int,
+f1002 int,
+f1003 int,
+f1004 int,
+f1005 int,
+f1006 int,
+f1007 int,
+f1008 int,
+f1009 int,
+f1010 int,
+f1011 int,
+f1012 int,
+f1013 int,
+f1014 int,
+f1015 int,
+f1016 int,
+f1017 int,
+f1018 int,
+f1019 int,
+f1020 int,
+f1021 int,
+f1022 int,
+f1023 int,
+f1024 int,
+f1025 int,
+f1026 int,
+f1027 int,
+f1028 int,
+f1029 int,
+f1030 int,
+f1031 int,
+f1032 int,
+f1033 int,
+f1034 int,
+f1035 int,
+f1036 int,
+f1037 int,
+f1038 int,
+f1039 int,
+f1040 int,
+f1041 int,
+f1042 int,
+f1043 int,
+f1044 int,
+f1045 int,
+f1046 int,
+f1047 int,
+f1048 int,
+f1049 int,
+f1050 int,
+f1051 int,
+f1052 int,
+f1053 int,
+f1054 int,
+f1055 int,
+f1056 int,
+f1057 int,
+f1058 int,
+f1059 int,
+f1060 int,
+f1061 int,
+f1062 int,
+f1063 int,
+f1064 int,
+f1065 int,
+f1066 int,
+f1067 int,
+f1068 int,
+f1069 int,
+f1070 int,
+f1071 int,
+f1072 int,
+f1073 int,
+f1074 int,
+f1075 int,
+f1076 int,
+f1077 int,
+f1078 int,
+f1079 int,
+f1080 int,
+f1081 int,
+f1082 int,
+f1083 int,
+f1084 int,
+f1085 int,
+f1086 int,
+f1087 int,
+f1088 int,
+f1089 int,
+f1090 int,
+f1091 int,
+f1092 int,
+f1093 int,
+f1094 int,
+f1095 int,
+f1096 int,
+f1097 int,
+f1098 int,
+f1099 int,
+f1100 int,
+f1101 int,
+f1102 int,
+f1103 int,
+f1104 int,
+f1105 int,
+f1106 int,
+f1107 int,
+f1108 int,
+f1109 int,
+f1110 int,
+f1111 int,
+f1112 int,
+f1113 int,
+f1114 int,
+f1115 int,
+f1116 int,
+f1117 int,
+f1118 int,
+f1119 int,
+f1120 int,
+f1121 int,
+f1122 int,
+f1123 int,
+f1124 int,
+f1125 int,
+f1126 int,
+f1127 int,
+f1128 int,
+f1129 int,
+f1130 int,
+f1131 int,
+f1132 int,
+f1133 int,
+f1134 int,
+f1135 int,
+f1136 int,
+f1137 int,
+f1138 int,
+f1139 int,
+f1140 int,
+f1141 int,
+f1142 int,
+f1143 int,
+f1144 int,
+f1145 int,
+f1146 int,
+f1147 int,
+f1148 int,
+f1149 int,
+f1150 int,
+f1151 int,
+f1152 int,
+f1153 int,
+f1154 int,
+f1155 int,
+f1156 int,
+f1157 int,
+f1158 int,
+f1159 int,
+f1160 int,
+f1161 int,
+f1162 int,
+f1163 int,
+f1164 int,
+f1165 int,
+f1166 int,
+f1167 int,
+f1168 int,
+f1169 int,
+f1170 int,
+f1171 int,
+f1172 int,
+f1173 int,
+f1174 int,
+f1175 int,
+f1176 int,
+f1177 int,
+f1178 int,
+f1179 int,
+f1180 int,
+f1181 int,
+f1182 int,
+f1183 int,
+f1184 int,
+f1185 int,
+f1186 int,
+f1187 int,
+f1188 int,
+f1189 int,
+f1190 int,
+f1191 int,
+f1192 int,
+f1193 int,
+f1194 int,
+f1195 int,
+f1196 int,
+f1197 int,
+f1198 int,
+f1199 int,
+f1200 int,
+f1201 int,
+f1202 int,
+f1203 int,
+f1204 int,
+f1205 int,
+f1206 int,
+f1207 int,
+f1208 int,
+f1209 int,
+f1210 int,
+f1211 int,
+f1212 int,
+f1213 int,
+f1214 int,
+f1215 int,
+f1216 int,
+f1217 int,
+f1218 int,
+f1219 int,
+f1220 int,
+f1221 int,
+f1222 int,
+f1223 int,
+f1224 int,
+f1225 int,
+f1226 int,
+f1227 int,
+f1228 int,
+f1229 int,
+f1230 int,
+f1231 int,
+f1232 int,
+f1233 int,
+f1234 int,
+f1235 int,
+f1236 int,
+f1237 int,
+f1238 int,
+f1239 int,
+f1240 int,
+f1241 int,
+f1242 int,
+f1243 int,
+f1244 int,
+f1245 int,
+f1246 int,
+f1247 int,
+f1248 int,
+f1249 int,
+f1250 int,
+f1251 int,
+f1252 int,
+f1253 int,
+f1254 int,
+f1255 int,
+f1256 int,
+f1257 int,
+f1258 int,
+f1259 int,
+f1260 int,
+f1261 int,
+f1262 int,
+f1263 int,
+f1264 int,
+f1265 int,
+f1266 int,
+f1267 int,
+f1268 int,
+f1269 int,
+f1270 int,
+f1271 int,
+f1272 int,
+f1273 int,
+f1274 int,
+f1275 int,
+f1276 int,
+f1277 int,
+f1278 int,
+f1279 int,
+f1280 int,
+f1281 int,
+f1282 int,
+f1283 int,
+f1284 int,
+f1285 int,
+f1286 int,
+f1287 int,
+f1288 int,
+f1289 int,
+f1290 int,
+f1291 int,
+f1292 int,
+f1293 int,
+f1294 int,
+f1295 int,
+f1296 int,
+f1297 int,
+f1298 int,
+f1299 int,
+f1300 int,
+f1301 int,
+f1302 int,
+f1303 int,
+f1304 int,
+f1305 int,
+f1306 int,
+f1307 int,
+f1308 int,
+f1309 int,
+f1310 int,
+f1311 int,
+f1312 int,
+f1313 int,
+f1314 int,
+f1315 int,
+f1316 int,
+f1317 int,
+f1318 int,
+f1319 int,
+f1320 int,
+f1321 int,
+f1322 int,
+f1323 int,
+f1324 int,
+f1325 int,
+f1326 int,
+f1327 int,
+f1328 int,
+f1329 int,
+f1330 int,
+f1331 int,
+f1332 int,
+f1333 int,
+f1334 int,
+f1335 int,
+f1336 int,
+f1337 int,
+f1338 int,
+f1339 int,
+f1340 int,
+f1341 int,
+f1342 int,
+f1343 int,
+f1344 int,
+f1345 int,
+f1346 int,
+f1347 int,
+f1348 int,
+f1349 int,
+f1350 int,
+f1351 int,
+f1352 int,
+f1353 int,
+f1354 int,
+f1355 int,
+f1356 int,
+f1357 int,
+f1358 int,
+f1359 int,
+f1360 int,
+f1361 int,
+f1362 int,
+f1363 int,
+f1364 int,
+f1365 int,
+f1366 int,
+f1367 int,
+f1368 int,
+f1369 int,
+f1370 int,
+f1371 int,
+f1372 int,
+f1373 int,
+f1374 int,
+f1375 int,
+f1376 int,
+f1377 int,
+f1378 int,
+f1379 int,
+f1380 int,
+f1381 int,
+f1382 int,
+f1383 int,
+f1384 int,
+f1385 int,
+f1386 int,
+f1387 int,
+f1388 int,
+f1389 int,
+f1390 int,
+f1391 int,
+f1392 int,
+f1393 int,
+f1394 int,
+f1395 int,
+f1396 int,
+f1397 int,
+f1398 int,
+f1399 int,
+f1400 int,
+f1401 int,
+f1402 int,
+f1403 int,
+f1404 int,
+f1405 int,
+f1406 int,
+f1407 int,
+f1408 int,
+f1409 int,
+f1410 int,
+f1411 int,
+f1412 int,
+f1413 int,
+f1414 int,
+f1415 int,
+f1416 int,
+f1417 int,
+f1418 int,
+f1419 int,
+f1420 int,
+f1421 int,
+f1422 int,
+f1423 int,
+f1424 int,
+f1425 int,
+f1426 int,
+f1427 int,
+f1428 int,
+f1429 int,
+f1430 int,
+f1431 int,
+f1432 int,
+f1433 int,
+f1434 int,
+f1435 int,
+f1436 int,
+f1437 int,
+f1438 int,
+f1439 int,
+f1440 int,
+f1441 int,
+f1442 int,
+f1443 int,
+f1444 int,
+f1445 int,
+f1446 int,
+f1447 int,
+f1448 int,
+f1449 int,
+f1450 int,
+f1451 int,
+f1452 int,
+f1453 int,
+f1454 int,
+f1455 int,
+f1456 int,
+f1457 int,
+f1458 int,
+f1459 int,
+f1460 int,
+f1461 int,
+f1462 int,
+f1463 int,
+f1464 int,
+f1465 int,
+f1466 int,
+f1467 int,
+f1468 int,
+f1469 int,
+f1470 int,
+f1471 int,
+f1472 int,
+f1473 int,
+f1474 int,
+f1475 int,
+f1476 int,
+f1477 int,
+f1478 int,
+f1479 int,
+f1480 int,
+f1481 int,
+f1482 int,
+f1483 int,
+f1484 int,
+f1485 int,
+f1486 int,
+f1487 int,
+f1488 int,
+f1489 int,
+f1490 int,
+f1491 int,
+f1492 int,
+f1493 int,
+f1494 int,
+f1495 int,
+f1496 int,
+f1497 int,
+f1498 int,
+f1499 int,
+f1500 int,
+f1501 int,
+f1502 int,
+f1503 int,
+f1504 int,
+f1505 int,
+f1506 int,
+f1507 int,
+f1508 int,
+f1509 int,
+f1510 int,
+f1511 int,
+f1512 int,
+f1513 int,
+f1514 int,
+f1515 int,
+f1516 int,
+f1517 int,
+f1518 int,
+f1519 int,
+f1520 int,
+f1521 int,
+f1522 int,
+f1523 int,
+f1524 int,
+f1525 int,
+f1526 int,
+f1527 int,
+f1528 int,
+f1529 int,
+f1530 int,
+f1531 int,
+f1532 int,
+f1533 int,
+f1534 int,
+f1535 int,
+f1536 int,
+f1537 int,
+f1538 int,
+f1539 int,
+f1540 int,
+f1541 int,
+f1542 int,
+f1543 int,
+f1544 int,
+f1545 int,
+f1546 int,
+f1547 int,
+f1548 int,
+f1549 int,
+f1550 int,
+f1551 int,
+f1552 int,
+f1553 int,
+f1554 int,
+f1555 int,
+f1556 int,
+f1557 int,
+f1558 int,
+f1559 int,
+f1560 int,
+f1561 int,
+f1562 int,
+f1563 int,
+f1564 int,
+f1565 int,
+f1566 int,
+f1567 int,
+f1568 int,
+f1569 int,
+f1570 int,
+f1571 int,
+f1572 int,
+f1573 int,
+f1574 int,
+f1575 int,
+f1576 int,
+f1577 int,
+f1578 int,
+f1579 int,
+f1580 int,
+f1581 int,
+f1582 int,
+f1583 int,
+f1584 int,
+f1585 int,
+f1586 int,
+f1587 int,
+f1588 int,
+f1589 int,
+f1590 int,
+f1591 int,
+f1592 int,
+f1593 int,
+f1594 int,
+f1595 int,
+f1596 int,
+f1597 int,
+f1598 int,
+f1599 int,
+f1600 int,
+f1601 int,
+f1602 int,
+f1603 int,
+f1604 int,
+f1605 int,
+f1606 int,
+f1607 int,
+f1608 int,
+f1609 int,
+f1610 int,
+f1611 int,
+f1612 int,
+f1613 int,
+f1614 int,
+f1615 int,
+f1616 int,
+f1617 int,
+f1618 int,
+f1619 int,
+f1620 int,
+f1621 int,
+f1622 int,
+f1623 int,
+f1624 int,
+f1625 int,
+f1626 int,
+f1627 int,
+f1628 int,
+f1629 int,
+f1630 int,
+f1631 int,
+f1632 int,
+f1633 int,
+f1634 int,
+f1635 int,
+f1636 int,
+f1637 int,
+f1638 int,
+f1639 int,
+f1640 int,
+f1641 int,
+f1642 int,
+f1643 int,
+f1644 int,
+f1645 int,
+f1646 int,
+f1647 int,
+f1648 int,
+f1649 int,
+f1650 int,
+f1651 int,
+f1652 int,
+f1653 int,
+f1654 int,
+f1655 int,
+f1656 int,
+f1657 int,
+f1658 int,
+f1659 int,
+f1660 int,
+f1661 int,
+f1662 int,
+f1663 int,
+f1664 int,
+f1665 int,
+f1666 int,
+f1667 int,
+f1668 int,
+f1669 int,
+f1670 int,
+f1671 int,
+f1672 int,
+f1673 int,
+f1674 int,
+f1675 int,
+f1676 int,
+f1677 int,
+f1678 int,
+f1679 int,
+f1680 int,
+f1681 int,
+f1682 int,
+f1683 int,
+f1684 int,
+f1685 int,
+f1686 int,
+f1687 int,
+f1688 int,
+f1689 int,
+f1690 int,
+f1691 int,
+f1692 int,
+f1693 int,
+f1694 int,
+f1695 int,
+f1696 int,
+f1697 int,
+f1698 int,
+f1699 int,
+f1700 int,
+f1701 int,
+f1702 int,
+f1703 int,
+f1704 int,
+f1705 int,
+f1706 int,
+f1707 int,
+f1708 int,
+f1709 int,
+f1710 int,
+f1711 int,
+f1712 int,
+f1713 int,
+f1714 int,
+f1715 int,
+f1716 int,
+f1717 int,
+f1718 int,
+f1719 int,
+f1720 int,
+f1721 int,
+f1722 int,
+f1723 int,
+f1724 int,
+f1725 int,
+f1726 int,
+f1727 int,
+f1728 int,
+f1729 int,
+f1730 int,
+f1731 int,
+f1732 int,
+f1733 int,
+f1734 int,
+f1735 int,
+f1736 int,
+f1737 int,
+f1738 int,
+f1739 int,
+f1740 int,
+f1741 int,
+f1742 int,
+f1743 int,
+f1744 int,
+f1745 int,
+f1746 int,
+f1747 int,
+f1748 int,
+f1749 int,
+f1750 int,
+f1751 int,
+f1752 int,
+f1753 int,
+f1754 int,
+f1755 int,
+f1756 int,
+f1757 int,
+f1758 int,
+f1759 int,
+f1760 int,
+f1761 int,
+f1762 int,
+f1763 int,
+f1764 int,
+f1765 int,
+f1766 int,
+f1767 int,
+f1768 int,
+f1769 int,
+f1770 int,
+f1771 int,
+f1772 int,
+f1773 int,
+f1774 int,
+f1775 int,
+f1776 int,
+f1777 int,
+f1778 int,
+f1779 int,
+f1780 int,
+f1781 int,
+f1782 int,
+f1783 int,
+f1784 int,
+f1785 int,
+f1786 int,
+f1787 int,
+f1788 int,
+f1789 int,
+f1790 int,
+f1791 int,
+f1792 int,
+f1793 int,
+f1794 int,
+f1795 int,
+f1796 int,
+f1797 int,
+f1798 int,
+f1799 int,
+f1800 int,
+f1801 int,
+f1802 int,
+f1803 int,
+f1804 int,
+f1805 int,
+f1806 int,
+f1807 int,
+f1808 int,
+f1809 int,
+f1810 int,
+f1811 int,
+f1812 int,
+f1813 int,
+f1814 int,
+f1815 int,
+f1816 int,
+f1817 int,
+f1818 int,
+f1819 int,
+f1820 int,
+f1821 int,
+f1822 int,
+f1823 int,
+f1824 int,
+f1825 int,
+f1826 int,
+f1827 int,
+f1828 int,
+f1829 int,
+f1830 int,
+f1831 int,
+f1832 int,
+f1833 int,
+f1834 int,
+f1835 int,
+f1836 int,
+f1837 int,
+f1838 int,
+f1839 int,
+f1840 int,
+f1841 int,
+f1842 int,
+f1843 int,
+f1844 int,
+f1845 int,
+f1846 int,
+f1847 int,
+f1848 int,
+f1849 int,
+f1850 int,
+f1851 int,
+f1852 int,
+f1853 int,
+f1854 int,
+f1855 int,
+f1856 int,
+f1857 int,
+f1858 int,
+f1859 int,
+f1860 int,
+f1861 int,
+f1862 int,
+f1863 int,
+f1864 int,
+f1865 int,
+f1866 int,
+f1867 int,
+f1868 int,
+f1869 int,
+f1870 int,
+f1871 int,
+f1872 int,
+f1873 int,
+f1874 int,
+f1875 int,
+f1876 int,
+f1877 int,
+f1878 int,
+f1879 int,
+f1880 int,
+f1881 int,
+f1882 int,
+f1883 int,
+f1884 int,
+f1885 int,
+f1886 int,
+f1887 int,
+f1888 int,
+f1889 int,
+f1890 int,
+f1891 int,
+f1892 int,
+f1893 int,
+f1894 int,
+f1895 int,
+f1896 int,
+f1897 int,
+f1898 int,
+f1899 int,
+f1900 int,
+f1901 int,
+f1902 int,
+f1903 int,
+f1904 int,
+f1905 int,
+f1906 int,
+f1907 int,
+f1908 int,
+f1909 int,
+f1910 int,
+f1911 int,
+f1912 int,
+f1913 int,
+f1914 int,
+f1915 int,
+f1916 int,
+f1917 int,
+f1918 int,
+f1919 int,
+f1920 int,
+f1921 int,
+f1922 int,
+f1923 int,
+f1924 int,
+f1925 int,
+f1926 int,
+f1927 int,
+f1928 int,
+f1929 int,
+f1930 int,
+f1931 int,
+f1932 int,
+f1933 int,
+f1934 int,
+f1935 int,
+f1936 int,
+f1937 int,
+f1938 int,
+f1939 int,
+f1940 int,
+f1941 int,
+f1942 int,
+f1943 int,
+f1944 int,
+f1945 int,
+f1946 int,
+f1947 int,
+f1948 int,
+f1949 int,
+f1950 int,
+f1951 int,
+f1952 int,
+f1953 int,
+f1954 int,
+f1955 int,
+f1956 int,
+f1957 int,
+f1958 int,
+f1959 int,
+f1960 int,
+f1961 int,
+f1962 int,
+f1963 int,
+f1964 int,
+f1965 int,
+f1966 int,
+f1967 int,
+f1968 int,
+f1969 int,
+f1970 int,
+f1971 int,
+f1972 int,
+f1973 int,
+f1974 int,
+f1975 int,
+f1976 int,
+f1977 int,
+f1978 int,
+f1979 int,
+f1980 int,
+f1981 int,
+f1982 int,
+f1983 int,
+f1984 int,
+f1985 int,
+f1986 int,
+f1987 int,
+f1988 int,
+f1989 int,
+f1990 int,
+f1991 int,
+f1992 int,
+f1993 int,
+f1994 int,
+f1995 int,
+f1996 int,
+f1997 int,
+f1998 int,
+f1999 int,
+f2000 int,
+f2001 int,
+f2002 int,
+f2003 int,
+f2004 int,
+f2005 int,
+f2006 int,
+f2007 int,
+f2008 int,
+f2009 int,
+f2010 int,
+f2011 int,
+f2012 int,
+f2013 int,
+f2014 int,
+f2015 int,
+f2016 int,
+f2017 int,
+f2018 int,
+f2019 int,
+f2020 int,
+f2021 int,
+f2022 int,
+f2023 int,
+f2024 int,
+f2025 int,
+f2026 int,
+f2027 int,
+f2028 int,
+f2029 int,
+f2030 int,
+f2031 int,
+f2032 int,
+f2033 int,
+f2034 int,
+f2035 int,
+f2036 int,
+f2037 int,
+f2038 int,
+f2039 int,
+f2040 int,
+f2041 int,
+f2042 int,
+f2043 int,
+f2044 int,
+f2045 int,
+f2046 int,
+f2047 int,
+f2048 int,
+f2049 int,
+f2050 int,
+f2051 int,
+f2052 int,
+f2053 int,
+f2054 int,
+f2055 int,
+f2056 int,
+f2057 int,
+f2058 int,
+f2059 int,
+f2060 int,
+f2061 int,
+f2062 int,
+f2063 int,
+f2064 int,
+f2065 int,
+f2066 int,
+f2067 int,
+f2068 int,
+f2069 int,
+f2070 int,
+f2071 int,
+f2072 int,
+f2073 int,
+f2074 int,
+f2075 int,
+f2076 int,
+f2077 int,
+f2078 int,
+f2079 int,
+f2080 int,
+f2081 int,
+f2082 int,
+f2083 int,
+f2084 int,
+f2085 int,
+f2086 int,
+f2087 int,
+f2088 int,
+f2089 int,
+f2090 int,
+f2091 int,
+f2092 int,
+f2093 int,
+f2094 int,
+f2095 int,
+f2096 int,
+f2097 int,
+f2098 int,
+f2099 int,
+f2100 int,
+f2101 int,
+f2102 int,
+f2103 int,
+f2104 int,
+f2105 int,
+f2106 int,
+f2107 int,
+f2108 int,
+f2109 int,
+f2110 int,
+f2111 int,
+f2112 int,
+f2113 int,
+f2114 int,
+f2115 int,
+f2116 int,
+f2117 int,
+f2118 int,
+f2119 int,
+f2120 int,
+f2121 int,
+f2122 int,
+f2123 int,
+f2124 int,
+f2125 int,
+f2126 int,
+f2127 int,
+f2128 int,
+f2129 int,
+f2130 int,
+f2131 int,
+f2132 int,
+f2133 int,
+f2134 int,
+f2135 int,
+f2136 int,
+f2137 int,
+f2138 int,
+f2139 int,
+f2140 int,
+f2141 int,
+f2142 int,
+f2143 int,
+f2144 int,
+f2145 int,
+f2146 int,
+f2147 int,
+f2148 int,
+f2149 int,
+f2150 int,
+f2151 int,
+f2152 int,
+f2153 int,
+f2154 int,
+f2155 int,
+f2156 int,
+f2157 int,
+f2158 int,
+f2159 int,
+f2160 int,
+f2161 int,
+f2162 int,
+f2163 int,
+f2164 int,
+f2165 int,
+f2166 int,
+f2167 int,
+f2168 int,
+f2169 int,
+f2170 int,
+f2171 int,
+f2172 int,
+f2173 int,
+f2174 int,
+f2175 int,
+f2176 int,
+f2177 int,
+f2178 int,
+f2179 int,
+f2180 int,
+f2181 int,
+f2182 int,
+f2183 int,
+f2184 int,
+f2185 int,
+f2186 int,
+f2187 int,
+f2188 int,
+f2189 int,
+f2190 int,
+f2191 int,
+f2192 int,
+f2193 int,
+f2194 int,
+f2195 int,
+f2196 int,
+f2197 int,
+f2198 int,
+f2199 int,
+f2200 int,
+f2201 int,
+f2202 int,
+f2203 int,
+f2204 int,
+f2205 int,
+f2206 int,
+f2207 int,
+f2208 int,
+f2209 int,
+f2210 int,
+f2211 int,
+f2212 int,
+f2213 int,
+f2214 int,
+f2215 int,
+f2216 int,
+f2217 int,
+f2218 int,
+f2219 int,
+f2220 int,
+f2221 int,
+f2222 int,
+f2223 int,
+f2224 int,
+f2225 int,
+f2226 int,
+f2227 int,
+f2228 int,
+f2229 int,
+f2230 int,
+f2231 int,
+f2232 int,
+f2233 int,
+f2234 int,
+f2235 int,
+f2236 int,
+f2237 int,
+f2238 int,
+f2239 int,
+f2240 int,
+f2241 int,
+f2242 int,
+f2243 int,
+f2244 int,
+f2245 int,
+f2246 int,
+f2247 int,
+f2248 int,
+f2249 int,
+f2250 int,
+f2251 int,
+f2252 int,
+f2253 int,
+f2254 int,
+f2255 int,
+f2256 int,
+f2257 int,
+f2258 int,
+f2259 int,
+f2260 int,
+f2261 int,
+f2262 int,
+f2263 int,
+f2264 int,
+f2265 int,
+f2266 int,
+f2267 int,
+f2268 int,
+f2269 int,
+f2270 int,
+f2271 int,
+f2272 int,
+f2273 int,
+f2274 int,
+f2275 int,
+f2276 int,
+f2277 int,
+f2278 int,
+f2279 int,
+f2280 int,
+f2281 int,
+f2282 int,
+f2283 int,
+f2284 int,
+f2285 int,
+f2286 int,
+f2287 int,
+f2288 int,
+f2289 int,
+f2290 int,
+f2291 int,
+f2292 int,
+f2293 int,
+f2294 int,
+f2295 int,
+f2296 int,
+f2297 int,
+f2298 int,
+f2299 int,
+f2300 int,
+f2301 int,
+f2302 int,
+f2303 int,
+f2304 int,
+f2305 int,
+f2306 int,
+f2307 int,
+f2308 int,
+f2309 int,
+f2310 int,
+f2311 int,
+f2312 int,
+f2313 int,
+f2314 int,
+f2315 int,
+f2316 int,
+f2317 int,
+f2318 int,
+f2319 int,
+f2320 int,
+f2321 int,
+f2322 int,
+f2323 int,
+f2324 int,
+f2325 int,
+f2326 int,
+f2327 int,
+f2328 int,
+f2329 int,
+f2330 int,
+f2331 int,
+f2332 int,
+f2333 int,
+f2334 int,
+f2335 int,
+f2336 int,
+f2337 int,
+f2338 int,
+f2339 int,
+f2340 int,
+f2341 int,
+f2342 int,
+f2343 int,
+f2344 int,
+f2345 int,
+f2346 int,
+f2347 int,
+f2348 int,
+f2349 int,
+f2350 int,
+f2351 int,
+f2352 int,
+f2353 int,
+f2354 int,
+f2355 int,
+f2356 int,
+f2357 int,
+f2358 int,
+f2359 int,
+f2360 int,
+f2361 int,
+f2362 int,
+f2363 int,
+f2364 int,
+f2365 int,
+f2366 int,
+f2367 int,
+f2368 int,
+f2369 int,
+f2370 int,
+f2371 int,
+f2372 int,
+f2373 int,
+f2374 int,
+f2375 int,
+f2376 int,
+f2377 int,
+f2378 int,
+f2379 int,
+f2380 int,
+f2381 int,
+f2382 int,
+f2383 int,
+f2384 int,
+f2385 int,
+f2386 int,
+f2387 int,
+f2388 int,
+f2389 int,
+f2390 int,
+f2391 int,
+f2392 int,
+f2393 int,
+f2394 int,
+f2395 int,
+f2396 int,
+f2397 int,
+f2398 int,
+f2399 int,
+f2400 int,
+f2401 int,
+f2402 int,
+f2403 int,
+f2404 int,
+f2405 int,
+f2406 int,
+f2407 int,
+f2408 int,
+f2409 int,
+f2410 int,
+f2411 int,
+f2412 int,
+f2413 int,
+f2414 int,
+f2415 int,
+f2416 int,
+f2417 int,
+f2418 int,
+f2419 int,
+f2420 int,
+f2421 int,
+f2422 int,
+f2423 int,
+f2424 int,
+f2425 int,
+f2426 int,
+f2427 int,
+f2428 int,
+f2429 int,
+f2430 int,
+f2431 int,
+f2432 int,
+f2433 int,
+f2434 int,
+f2435 int,
+f2436 int,
+f2437 int,
+f2438 int,
+f2439 int,
+f2440 int,
+f2441 int,
+f2442 int,
+f2443 int,
+f2444 int,
+f2445 int,
+f2446 int,
+f2447 int,
+f2448 int,
+f2449 int,
+f2450 int,
+f2451 int,
+f2452 int,
+f2453 int,
+f2454 int,
+f2455 int,
+f2456 int,
+f2457 int,
+f2458 int,
+f2459 int,
+f2460 int,
+f2461 int,
+f2462 int,
+f2463 int,
+f2464 int,
+f2465 int,
+f2466 int,
+f2467 int,
+f2468 int,
+f2469 int,
+f2470 int,
+f2471 int,
+f2472 int,
+f2473 int,
+f2474 int,
+f2475 int,
+f2476 int,
+f2477 int,
+f2478 int,
+f2479 int,
+f2480 int,
+f2481 int,
+f2482 int,
+f2483 int,
+f2484 int,
+f2485 int,
+f2486 int,
+f2487 int,
+f2488 int,
+f2489 int,
+f2490 int,
+f2491 int,
+f2492 int,
+f2493 int,
+f2494 int,
+f2495 int,
+f2496 int,
+f2497 int,
+f2498 int,
+f2499 int,
+f2500 int,
+f2501 int,
+f2502 int,
+f2503 int,
+f2504 int,
+f2505 int,
+f2506 int,
+f2507 int,
+f2508 int,
+f2509 int,
+f2510 int,
+f2511 int,
+f2512 int,
+f2513 int,
+f2514 int,
+f2515 int,
+f2516 int,
+f2517 int,
+f2518 int,
+f2519 int,
+f2520 int,
+f2521 int,
+f2522 int,
+f2523 int,
+f2524 int,
+f2525 int,
+f2526 int,
+f2527 int,
+f2528 int,
+f2529 int,
+f2530 int,
+f2531 int,
+f2532 int,
+f2533 int,
+f2534 int,
+f2535 int,
+f2536 int,
+f2537 int,
+f2538 int,
+f2539 int,
+f2540 int,
+f2541 int,
+f2542 int,
+f2543 int,
+f2544 int,
+f2545 int,
+f2546 int,
+f2547 int,
+f2548 int,
+f2549 int,
+f2550 int,
+f2551 int,
+f2552 int,
+f2553 int,
+f2554 int,
+f2555 int,
+f2556 int,
+f2557 int,
+f2558 int,
+f2559 int,
+f2560 int,
+f2561 int,
+f2562 int,
+f2563 int,
+f2564 int,
+f2565 int,
+f2566 int,
+f2567 int,
+f2568 int,
+f2569 int,
+f2570 int,
+f2571 int,
+f2572 int,
+f2573 int,
+f2574 int,
+f2575 int,
+f2576 int,
+f2577 int,
+f2578 int,
+f2579 int,
+f2580 int,
+f2581 int,
+f2582 int,
+f2583 int,
+f2584 int,
+f2585 int,
+f2586 int,
+f2587 int,
+f2588 int,
+f2589 int,
+f2590 int,
+f2591 int,
+f2592 int,
+f2593 int,
+f2594 int,
+f2595 int,
+f2596 int,
+f2597 int,
+f2598 int,
+f2599 int,
+f2600 int,
+f2601 int,
+f2602 int,
+f2603 int,
+f2604 int,
+f2605 int,
+f2606 int,
+f2607 int,
+f2608 int,
+f2609 int,
+f2610 int,
+f2611 int,
+f2612 int,
+f2613 int,
+f2614 int,
+f2615 int,
+f2616 int,
+f2617 int,
+f2618 int,
+f2619 int,
+f2620 int,
+f2621 int,
+f2622 int,
+f2623 int,
+f2624 int,
+f2625 int,
+f2626 int,
+f2627 int,
+f2628 int,
+f2629 int,
+f2630 int,
+f2631 int,
+f2632 int,
+f2633 int,
+f2634 int,
+f2635 int,
+f2636 int,
+f2637 int,
+f2638 int,
+f2639 int,
+f2640 int,
+f2641 int,
+f2642 int,
+f2643 int,
+f2644 int,
+f2645 int,
+f2646 int,
+f2647 int,
+f2648 int,
+f2649 int,
+f2650 int,
+f2651 int,
+f2652 int,
+f2653 int,
+f2654 int,
+f2655 int,
+f2656 int,
+f2657 int,
+f2658 int,
+f2659 int,
+f2660 int,
+f2661 int,
+f2662 int,
+f2663 int,
+f2664 int,
+f2665 int,
+f2666 int,
+f2667 int,
+f2668 int,
+f2669 int,
+f2670 int,
+f2671 int,
+f2672 int,
+f2673 int,
+f2674 int,
+f2675 int,
+f2676 int,
+f2677 int,
+f2678 int,
+f2679 int,
+f2680 int,
+f2681 int,
+f2682 int,
+f2683 int,
+f2684 int,
+f2685 int,
+f2686 int,
+f2687 int,
+f2688 int,
+f2689 int,
+f2690 int,
+f2691 int,
+f2692 int,
+f2693 int,
+f2694 int,
+f2695 int,
+f2696 int,
+f2697 int,
+f2698 int,
+f2699 int,
+f2700 int,
+f2701 int,
+f2702 int,
+f2703 int,
+f2704 int,
+f2705 int,
+f2706 int,
+f2707 int,
+f2708 int,
+f2709 int,
+f2710 int,
+f2711 int,
+f2712 int,
+f2713 int,
+f2714 int,
+f2715 int,
+f2716 int,
+f2717 int,
+f2718 int,
+f2719 int,
+f2720 int,
+f2721 int,
+f2722 int,
+f2723 int,
+f2724 int,
+f2725 int,
+f2726 int,
+f2727 int,
+f2728 int,
+f2729 int,
+f2730 int,
+f2731 int,
+f2732 int,
+f2733 int,
+f2734 int,
+f2735 int,
+f2736 int,
+f2737 int,
+f2738 int,
+f2739 int,
+f2740 int,
+f2741 int,
+f2742 int,
+f2743 int,
+f2744 int,
+f2745 int,
+f2746 int,
+f2747 int,
+f2748 int,
+f2749 int,
+f2750 int,
+f2751 int,
+f2752 int,
+f2753 int,
+f2754 int,
+f2755 int,
+f2756 int,
+f2757 int,
+f2758 int,
+f2759 int,
+f2760 int,
+f2761 int,
+f2762 int,
+f2763 int,
+f2764 int,
+f2765 int,
+f2766 int,
+f2767 int,
+f2768 int,
+f2769 int,
+f2770 int,
+f2771 int,
+f2772 int,
+f2773 int,
+f2774 int,
+f2775 int,
+f2776 int,
+f2777 int,
+f2778 int,
+f2779 int,
+f2780 int,
+f2781 int,
+f2782 int,
+f2783 int,
+f2784 int,
+f2785 int,
+f2786 int,
+f2787 int,
+f2788 int,
+f2789 int,
+f2790 int,
+f2791 int,
+f2792 int,
+f2793 int,
+f2794 int,
+f2795 int,
+f2796 int,
+f2797 int,
+f2798 int,
+f2799 int,
+f2800 int,
+f2801 int,
+f2802 int,
+f2803 int,
+f2804 int,
+f2805 int,
+f2806 int,
+f2807 int,
+f2808 int,
+f2809 int,
+f2810 int,
+f2811 int,
+f2812 int,
+f2813 int,
+f2814 int,
+f2815 int,
+f2816 int,
+f2817 int,
+f2818 int,
+f2819 int,
+f2820 int,
+f2821 int,
+f2822 int,
+f2823 int,
+f2824 int,
+f2825 int,
+f2826 int,
+f2827 int,
+f2828 int,
+f2829 int,
+f2830 int,
+f2831 int,
+f2832 int,
+f2833 int,
+f2834 int,
+f2835 int,
+f2836 int,
+f2837 int,
+f2838 int,
+f2839 int,
+f2840 int,
+f2841 int,
+f2842 int,
+f2843 int,
+f2844 int,
+f2845 int,
+f2846 int,
+f2847 int,
+f2848 int,
+f2849 int,
+f2850 int,
+f2851 int,
+f2852 int,
+f2853 int,
+f2854 int,
+f2855 int,
+f2856 int,
+f2857 int,
+f2858 int,
+f2859 int,
+f2860 int,
+f2861 int,
+f2862 int,
+f2863 int,
+f2864 int,
+f2865 int,
+f2866 int,
+f2867 int,
+f2868 int,
+f2869 int,
+f2870 int,
+f2871 int,
+f2872 int,
+f2873 int,
+f2874 int,
+f2875 int,
+f2876 int,
+f2877 int,
+f2878 int,
+f2879 int,
+f2880 int,
+f2881 int,
+f2882 int,
+f2883 int,
+f2884 int,
+f2885 int,
+f2886 int,
+f2887 int,
+f2888 int,
+f2889 int,
+f2890 int,
+f2891 int,
+f2892 int,
+f2893 int,
+f2894 int,
+f2895 int,
+f2896 int,
+f2897 int,
+f2898 int,
+f2899 int,
+f2900 int,
+f2901 int,
+f2902 int,
+f2903 int,
+f2904 int,
+f2905 int,
+f2906 int,
+f2907 int,
+f2908 int,
+f2909 int,
+f2910 int,
+f2911 int,
+f2912 int,
+f2913 int,
+f2914 int,
+f2915 int,
+f2916 int,
+f2917 int,
+f2918 int,
+f2919 int,
+f2920 int,
+f2921 int,
+f2922 int,
+f2923 int,
+f2924 int,
+f2925 int,
+f2926 int,
+f2927 int,
+f2928 int,
+f2929 int,
+f2930 int,
+f2931 int,
+f2932 int,
+f2933 int,
+f2934 int,
+f2935 int,
+f2936 int,
+f2937 int,
+f2938 int,
+f2939 int,
+f2940 int,
+f2941 int,
+f2942 int,
+f2943 int,
+f2944 int,
+f2945 int,
+f2946 int,
+f2947 int,
+f2948 int,
+f2949 int,
+f2950 int,
+f2951 int,
+f2952 int,
+f2953 int,
+f2954 int,
+f2955 int,
+f2956 int,
+f2957 int,
+f2958 int,
+f2959 int,
+f2960 int,
+f2961 int,
+f2962 int,
+f2963 int,
+f2964 int,
+f2965 int,
+f2966 int,
+f2967 int,
+f2968 int,
+f2969 int,
+f2970 int,
+f2971 int,
+f2972 int,
+f2973 int,
+f2974 int,
+f2975 int,
+f2976 int,
+f2977 int,
+f2978 int,
+f2979 int,
+f2980 int,
+f2981 int,
+f2982 int,
+f2983 int,
+f2984 int,
+f2985 int,
+f2986 int,
+f2987 int,
+f2988 int,
+f2989 int,
+f2990 int,
+f2991 int,
+f2992 int,
+f2993 int,
+f2994 int,
+f2995 int,
+f2996 int,
+f2997 int,
+f2998 int,
+f2999 int,
+f3000 int,
+f3001 int,
+f3002 int,
+f3003 int,
+f3004 int,
+f3005 int,
+f3006 int,
+f3007 int,
+f3008 int,
+f3009 int,
+f3010 int,
+f3011 int,
+f3012 int,
+f3013 int,
+f3014 int,
+f3015 int,
+f3016 int,
+f3017 int,
+f3018 int,
+f3019 int,
+f3020 int,
+f3021 int,
+f3022 int,
+f3023 int,
+f3024 int,
+f3025 int,
+f3026 int,
+f3027 int,
+f3028 int,
+f3029 int,
+f3030 int,
+f3031 int,
+f3032 int,
+f3033 int,
+f3034 int,
+f3035 int,
+f3036 int,
+f3037 int,
+f3038 int,
+f3039 int,
+f3040 int,
+f3041 int,
+f3042 int,
+f3043 int,
+f3044 int,
+f3045 int,
+f3046 int,
+f3047 int,
+f3048 int,
+f3049 int,
+f3050 int,
+f3051 int,
+f3052 int,
+f3053 int,
+f3054 int,
+f3055 int,
+f3056 int,
+f3057 int,
+f3058 int,
+f3059 int,
+f3060 int,
+f3061 int,
+f3062 int,
+f3063 int,
+f3064 int,
+f3065 int,
+f3066 int,
+f3067 int,
+f3068 int,
+f3069 int,
+f3070 int,
+f3071 int,
+f3072 int,
+f3073 int,
+f3074 int,
+f3075 int,
+f3076 int,
+f3077 int,
+f3078 int,
+f3079 int,
+f3080 int,
+f3081 int,
+f3082 int,
+f3083 int,
+f3084 int,
+f3085 int,
+f3086 int,
+f3087 int,
+f3088 int,
+f3089 int,
+f3090 int,
+f3091 int,
+f3092 int,
+f3093 int,
+f3094 int,
+f3095 int,
+f3096 int,
+f3097 int,
+f3098 int,
+f3099 int,
+f3100 int,
+f3101 int,
+f3102 int,
+f3103 int,
+f3104 int,
+f3105 int,
+f3106 int,
+f3107 int,
+f3108 int,
+f3109 int,
+f3110 int,
+f3111 int,
+f3112 int,
+f3113 int,
+f3114 int,
+f3115 int,
+f3116 int,
+f3117 int,
+f3118 int,
+f3119 int,
+f3120 int,
+f3121 int,
+f3122 int,
+f3123 int,
+f3124 int,
+f3125 int,
+f3126 int,
+f3127 int,
+f3128 int,
+f3129 int,
+f3130 int,
+f3131 int,
+f3132 int,
+f3133 int,
+f3134 int,
+f3135 int,
+f3136 int,
+f3137 int,
+f3138 int,
+f3139 int,
+f3140 int,
+f3141 int,
+f3142 int,
+f3143 int,
+f3144 int,
+f3145 int,
+f3146 int,
+f3147 int,
+f3148 int,
+f3149 int,
+f3150 int,
+f3151 int,
+f3152 int,
+f3153 int,
+f3154 int,
+f3155 int,
+f3156 int,
+f3157 int,
+f3158 int,
+f3159 int,
+f3160 int,
+f3161 int,
+f3162 int,
+f3163 int,
+f3164 int,
+f3165 int,
+f3166 int,
+f3167 int,
+f3168 int,
+f3169 int,
+f3170 int,
+f3171 int,
+f3172 int,
+f3173 int,
+f3174 int,
+f3175 int,
+f3176 int,
+f3177 int,
+f3178 int,
+f3179 int,
+f3180 int,
+f3181 int,
+f3182 int,
+f3183 int,
+f3184 int,
+f3185 int,
+f3186 int,
+f3187 int,
+f3188 int,
+f3189 int,
+f3190 int,
+f3191 int,
+f3192 int,
+f3193 int,
+f3194 int,
+f3195 int,
+f3196 int,
+f3197 int,
+f3198 int,
+f3199 int,
+f3200 int,
+f3201 int,
+f3202 int,
+f3203 int,
+f3204 int,
+f3205 int,
+f3206 int,
+f3207 int,
+f3208 int,
+f3209 int,
+f3210 int,
+f3211 int,
+f3212 int,
+f3213 int,
+f3214 int,
+f3215 int,
+f3216 int,
+f3217 int,
+f3218 int,
+f3219 int,
+f3220 int,
+f3221 int,
+f3222 int,
+f3223 int,
+f3224 int,
+f3225 int,
+f3226 int,
+f3227 int,
+f3228 int,
+f3229 int,
+f3230 int,
+f3231 int,
+f3232 int,
+f3233 int,
+f3234 int,
+f3235 int,
+f3236 int,
+f3237 int,
+f3238 int,
+f3239 int,
+f3240 int,
+f3241 int,
+f3242 int,
+f3243 int,
+f3244 int,
+f3245 int,
+f3246 int,
+f3247 int,
+f3248 int,
+f3249 int,
+f3250 int,
+f3251 int,
+f3252 int,
+f3253 int,
+f3254 int,
+f3255 int,
+f3256 int,
+f3257 int,
+f3258 int,
+f3259 int,
+f3260 int,
+f3261 int,
+f3262 int,
+f3263 int,
+f3264 int,
+f3265 int,
+f3266 int,
+f3267 int,
+f3268 int,
+f3269 int,
+f3270 int,
+f3271 int,
+f3272 int,
+f3273 int,
+f3274 int,
+f3275 int,
+f3276 int,
+f3277 int,
+f3278 int,
+f3279 int,
+f3280 int,
+f3281 int,
+f3282 int,
+f3283 int,
+f3284 int,
+f3285 int,
+f3286 int,
+f3287 int,
+f3288 int,
+f3289 int,
+f3290 int,
+f3291 int,
+f3292 int,
+f3293 int,
+f3294 int,
+f3295 int,
+f3296 int,
+f3297 int,
+f3298 int,
+f3299 int,
+f3300 int,
+f3301 int,
+f3302 int,
+f3303 int,
+f3304 int,
+f3305 int,
+f3306 int,
+f3307 int,
+f3308 int,
+f3309 int,
+f3310 int,
+f3311 int,
+f3312 int,
+f3313 int,
+f3314 int,
+f3315 int,
+f3316 int,
+f3317 int,
+f3318 int,
+f3319 int,
+f3320 int,
+f3321 int,
+f3322 int,
+f3323 int,
+f3324 int,
+f3325 int,
+f3326 int,
+f3327 int,
+f3328 int,
+f3329 int,
+f3330 int,
+f3331 int,
+f3332 int,
+f3333 int,
+f3334 int,
+f3335 int,
+f3336 int,
+f3337 int,
+f3338 int,
+f3339 int,
+f3340 int,
+f3341 int,
+f3342 int,
+f3343 int,
+f3344 int,
+f3345 int,
+f3346 int,
+f3347 int,
+f3348 int,
+f3349 int,
+f3350 int,
+f3351 int,
+f3352 int,
+f3353 int,
+f3354 int,
+f3355 int,
+f3356 int,
+f3357 int,
+f3358 int,
+f3359 int,
+f3360 int,
+f3361 int,
+f3362 int,
+f3363 int,
+f3364 int,
+f3365 int,
+f3366 int,
+f3367 int,
+f3368 int,
+f3369 int,
+f3370 int,
+f3371 int,
+f3372 int,
+f3373 int,
+f3374 int,
+f3375 int,
+f3376 int,
+f3377 int,
+f3378 int,
+f3379 int,
+f3380 int,
+f3381 int,
+f3382 int,
+f3383 int,
+f3384 int,
+f3385 int,
+f3386 int,
+f3387 int,
+f3388 int,
+f3389 int,
+f3390 int,
+f3391 int,
+f3392 int,
+f3393 int,
+f3394 int,
+f3395 int,
+f3396 int,
+f3397 int,
+f3398 int,
+f3399 int,
+f3400 int,
+f3401 int,
+f3402 int,
+f3403 int,
+f3404 int,
+f3405 int,
+f3406 int,
+f3407 int,
+f3408 int,
+f3409 int,
+f3410 int,
+f3411 int,
+f3412 int,
+f3413 int,
+f3414 int,
+f3415 int,
+f3416 int,
+f3417 int,
+f3418 int,
+f3419 int,
+f3420 int,
+f3421 int,
+f3422 int,
+f3423 int,
+f3424 int,
+f3425 int,
+f3426 int,
+f3427 int,
+f3428 int,
+f3429 int,
+f3430 int,
+f3431 int,
+f3432 int,
+f3433 int,
+f3434 int,
+f3435 int,
+f3436 int,
+f3437 int,
+f3438 int,
+f3439 int,
+f3440 int,
+f3441 int,
+f3442 int,
+f3443 int,
+f3444 int,
+f3445 int,
+f3446 int,
+f3447 int,
+f3448 int,
+f3449 int,
+f3450 int,
+f3451 int,
+f3452 int,
+f3453 int,
+f3454 int,
+f3455 int,
+f3456 int,
+f3457 int,
+f3458 int,
+f3459 int,
+f3460 int,
+f3461 int,
+f3462 int,
+f3463 int,
+f3464 int,
+f3465 int,
+f3466 int,
+f3467 int,
+f3468 int,
+f3469 int,
+f3470 int,
+f3471 int,
+f3472 int,
+f3473 int,
+f3474 int,
+f3475 int,
+f3476 int,
+f3477 int,
+f3478 int,
+f3479 int,
+f3480 int,
+f3481 int,
+f3482 int,
+f3483 int,
+f3484 int,
+f3485 int,
+f3486 int,
+f3487 int,
+f3488 int,
+f3489 int,
+f3490 int,
+f3491 int,
+f3492 int,
+f3493 int,
+f3494 int,
+f3495 int,
+f3496 int,
+f3497 int,
+f3498 int,
+f3499 int,
+f3500 int,
+f3501 int,
+f3502 int,
+f3503 int,
+f3504 int,
+f3505 int,
+f3506 int,
+f3507 int,
+f3508 int,
+f3509 int,
+f3510 int,
+f3511 int,
+f3512 int,
+f3513 int,
+f3514 int,
+f3515 int,
+f3516 int,
+f3517 int,
+f3518 int,
+f3519 int,
+f3520 int,
+f3521 int,
+f3522 int,
+f3523 int,
+f3524 int,
+f3525 int,
+f3526 int,
+f3527 int,
+f3528 int,
+f3529 int,
+f3530 int,
+f3531 int,
+f3532 int,
+f3533 int,
+f3534 int,
+f3535 int,
+f3536 int,
+f3537 int,
+f3538 int,
+f3539 int,
+f3540 int,
+f3541 int,
+f3542 int,
+f3543 int,
+f3544 int,
+f3545 int,
+f3546 int,
+f3547 int,
+f3548 int,
+f3549 int,
+f3550 int,
+f3551 int,
+f3552 int,
+f3553 int,
+f3554 int,
+f3555 int,
+f3556 int,
+f3557 int,
+f3558 int,
+f3559 int,
+f3560 int,
+f3561 int,
+f3562 int,
+f3563 int,
+f3564 int,
+f3565 int,
+f3566 int,
+f3567 int,
+f3568 int,
+f3569 int,
+f3570 int,
+f3571 int,
+f3572 int,
+f3573 int,
+f3574 int,
+f3575 int,
+f3576 int,
+f3577 int,
+f3578 int,
+f3579 int,
+f3580 int,
+f3581 int,
+f3582 int,
+f3583 int,
+f3584 int,
+f3585 int,
+f3586 int,
+f3587 int,
+f3588 int,
+f3589 int,
+f3590 int,
+f3591 int,
+f3592 int,
+f3593 int,
+f3594 int,
+f3595 int,
+f3596 int,
+f3597 int,
+f3598 int,
+f3599 int,
+f3600 int,
+f3601 int,
+f3602 int,
+f3603 int,
+f3604 int,
+f3605 int,
+f3606 int,
+f3607 int,
+f3608 int,
+f3609 int,
+f3610 int,
+f3611 int,
+f3612 int,
+f3613 int,
+f3614 int,
+f3615 int,
+f3616 int,
+f3617 int,
+f3618 int,
+f3619 int,
+f3620 int,
+f3621 int,
+f3622 int,
+f3623 int,
+f3624 int,
+f3625 int,
+f3626 int,
+f3627 int,
+f3628 int,
+f3629 int,
+f3630 int,
+f3631 int,
+f3632 int,
+f3633 int,
+f3634 int,
+f3635 int,
+f3636 int,
+f3637 int,
+f3638 int,
+f3639 int,
+f3640 int,
+f3641 int,
+f3642 int,
+f3643 int,
+f3644 int,
+f3645 int,
+f3646 int,
+f3647 int,
+f3648 int,
+f3649 int,
+f3650 int,
+f3651 int,
+f3652 int,
+f3653 int,
+f3654 int,
+f3655 int,
+f3656 int,
+f3657 int,
+f3658 int,
+f3659 int,
+f3660 int,
+f3661 int,
+f3662 int,
+f3663 int,
+f3664 int,
+f3665 int,
+f3666 int,
+f3667 int,
+f3668 int,
+f3669 int,
+f3670 int,
+f3671 int,
+f3672 int,
+f3673 int,
+f3674 int,
+f3675 int,
+f3676 int,
+f3677 int,
+f3678 int,
+f3679 int,
+f3680 int,
+f3681 int,
+f3682 int,
+f3683 int,
+f3684 int,
+f3685 int,
+f3686 int,
+f3687 int,
+f3688 int,
+f3689 int,
+f3690 int,
+f3691 int,
+f3692 int,
+f3693 int,
+f3694 int,
+f3695 int,
+f3696 int,
+f3697 int,
+f3698 int,
+f3699 int,
+f3700 int,
+f3701 int,
+f3702 int,
+f3703 int,
+f3704 int,
+f3705 int,
+f3706 int,
+f3707 int,
+f3708 int,
+f3709 int,
+f3710 int,
+f3711 int,
+f3712 int,
+f3713 int,
+f3714 int,
+f3715 int,
+f3716 int,
+f3717 int,
+f3718 int,
+f3719 int,
+f3720 int,
+f3721 int,
+f3722 int,
+f3723 int,
+f3724 int,
+f3725 int,
+f3726 int,
+f3727 int,
+f3728 int,
+f3729 int,
+f3730 int,
+f3731 int,
+f3732 int,
+f3733 int,
+f3734 int,
+f3735 int,
+f3736 int,
+f3737 int,
+f3738 int,
+f3739 int,
+f3740 int,
+f3741 int,
+f3742 int,
+f3743 int,
+f3744 int,
+f3745 int,
+f3746 int,
+f3747 int,
+f3748 int,
+f3749 int,
+f3750 int,
+f3751 int,
+f3752 int,
+f3753 int,
+f3754 int,
+f3755 int,
+f3756 int,
+f3757 int,
+f3758 int,
+f3759 int,
+f3760 int,
+f3761 int,
+f3762 int,
+f3763 int,
+f3764 int,
+f3765 int,
+f3766 int,
+f3767 int,
+f3768 int,
+f3769 int,
+f3770 int,
+f3771 int,
+f3772 int,
+f3773 int,
+f3774 int,
+f3775 int,
+f3776 int,
+f3777 int,
+f3778 int,
+f3779 int,
+f3780 int,
+f3781 int,
+f3782 int,
+f3783 int,
+f3784 int,
+f3785 int,
+f3786 int,
+f3787 int,
+f3788 int,
+f3789 int,
+f3790 int,
+f3791 int,
+f3792 int,
+f3793 int,
+f3794 int,
+f3795 int,
+f3796 int,
+f3797 int,
+f3798 int,
+f3799 int,
+f3800 int,
+f3801 int,
+f3802 int,
+f3803 int,
+f3804 int,
+f3805 int,
+f3806 int,
+f3807 int,
+f3808 int,
+f3809 int,
+f3810 int,
+f3811 int,
+f3812 int,
+f3813 int,
+f3814 int,
+f3815 int,
+f3816 int,
+f3817 int,
+f3818 int,
+f3819 int,
+f3820 int,
+f3821 int,
+f3822 int,
+f3823 int,
+f3824 int,
+f3825 int,
+f3826 int,
+f3827 int,
+f3828 int,
+f3829 int,
+f3830 int,
+f3831 int,
+f3832 int,
+f3833 int,
+f3834 int,
+f3835 int,
+f3836 int,
+f3837 int,
+f3838 int,
+f3839 int,
+f3840 int,
+f3841 int,
+f3842 int,
+f3843 int,
+f3844 int,
+f3845 int,
+f3846 int,
+f3847 int,
+f3848 int,
+f3849 int,
+f3850 int,
+f3851 int,
+f3852 int,
+f3853 int,
+f3854 int,
+f3855 int,
+f3856 int,
+f3857 int,
+f3858 int,
+f3859 int,
+f3860 int,
+f3861 int,
+f3862 int,
+f3863 int,
+f3864 int,
+f3865 int,
+f3866 int,
+f3867 int,
+f3868 int,
+f3869 int,
+f3870 int,
+f3871 int,
+f3872 int,
+f3873 int,
+f3874 int,
+f3875 int,
+f3876 int,
+f3877 int,
+f3878 int,
+f3879 int,
+f3880 int,
+f3881 int,
+f3882 int,
+f3883 int,
+f3884 int,
+f3885 int,
+f3886 int,
+f3887 int,
+f3888 int,
+f3889 int,
+f3890 int,
+f3891 int,
+f3892 int,
+f3893 int,
+f3894 int,
+f3895 int,
+f3896 int,
+f3897 int,
+f3898 int,
+f3899 int,
+f3900 int,
+f3901 int,
+f3902 int,
+f3903 int,
+f3904 int,
+f3905 int,
+f3906 int,
+f3907 int,
+f3908 int,
+f3909 int,
+f3910 int,
+f3911 int,
+f3912 int,
+f3913 int,
+f3914 int,
+f3915 int,
+f3916 int,
+f3917 int,
+f3918 int,
+f3919 int,
+f3920 int,
+f3921 int,
+f3922 int,
+f3923 int,
+f3924 int,
+f3925 int,
+f3926 int,
+f3927 int,
+f3928 int,
+f3929 int,
+f3930 int,
+f3931 int,
+f3932 int,
+f3933 int,
+f3934 int,
+f3935 int,
+f3936 int,
+f3937 int,
+f3938 int,
+f3939 int,
+f3940 int,
+f3941 int,
+f3942 int,
+f3943 int,
+f3944 int,
+f3945 int,
+f3946 int,
+f3947 int,
+f3948 int,
+f3949 int,
+f3950 int,
+f3951 int,
+f3952 int,
+f3953 int,
+f3954 int,
+f3955 int,
+f3956 int,
+f3957 int,
+f3958 int,
+f3959 int,
+f3960 int,
+f3961 int,
+f3962 int,
+f3963 int,
+f3964 int,
+f3965 int,
+f3966 int,
+f3967 int,
+f3968 int,
+f3969 int,
+f3970 int,
+f3971 int,
+f3972 int,
+f3973 int,
+f3974 int,
+f3975 int,
+f3976 int,
+f3977 int,
+f3978 int,
+f3979 int,
+f3980 int,
+f3981 int,
+f3982 int,
+f3983 int,
+f3984 int,
+f3985 int,
+f3986 int,
+f3987 int,
+f3988 int,
+f3989 int,
+f3990 int,
+f3991 int,
+f3992 int,
+f3993 int,
+f3994 int,
+f3995 int,
+f3996 int,
+f3997 int,
+f3998 int,
+f3999 int,
+f4000 int,
+f4001 int,
+f4002 int,
+f4003 int,
+f4004 int,
+f4005 int,
+f4006 int,
+f4007 int,
+f4008 int,
+f4009 int,
+f4010 int,
+f4011 int,
+f4012 int,
+f4013 int,
+f4014 int,
+f4015 int,
+f4016 int,
+f4017 int,
+f4018 int,
+f4019 int,
+f4020 int,
+f4021 int,
+f4022 int,
+f4023 int,
+f4024 int,
+f4025 int,
+f4026 int,
+f4027 int,
+f4028 int,
+f4029 int,
+f4030 int,
+f4031 int,
+f4032 int,
+f4033 int,
+f4034 int,
+f4035 int,
+f4036 int,
+f4037 int,
+f4038 int,
+f4039 int,
+f4040 int,
+f4041 int,
+f4042 int,
+f4043 int,
+f4044 int,
+f4045 int,
+f4046 int,
+f4047 int,
+f4048 int,
+f4049 int,
+f4050 int,
+f4051 int,
+f4052 int,
+f4053 int,
+f4054 int,
+f4055 int,
+f4056 int,
+f4057 int,
+f4058 int,
+f4059 int,
+f4060 int,
+f4061 int,
+f4062 int,
+f4063 int,
+f4064 int,
+f4065 int,
+f4066 int,
+f4067 int,
+f4068 int,
+f4069 int,
+f4070 int,
+f4071 int,
+f4072 int,
+f4073 int,
+f4074 int,
+f4075 int,
+f4076 int,
+f4077 int,
+f4078 int,
+f4079 int,
+f4080 int,
+f4081 int,
+f4082 int,
+f4083 int,
+f4084 int,
+f4085 int,
+f4086 int,
+f4087 int,
+f4088 int,
+f4089 int,
+f4090 int,
+f4091 int,
+f4092 int,
+f4093 int,
+f4094 int,
+f4095 int,
+f4096 int
+) ENGINE=BLACKHOLE;
+ERROR HY000: Too many columns

=== modified file 'tests/r/default_collation.result'
--- tests/r/default_collation.result	2009-07-09 01:35:10 +0000
+++ tests/r/default_collation.result	2009-12-07 05:09:37 +0000
@@ -2,6 +2,14 @@
 show create database mysqltest;
 Database	Create Database
 mysqltest	CREATE DATABASE `mysqltest` COLLATE = utf8_bin
+alter database mysqltest collate = utf8_general_ci;
+show create database mysqltest;
+Database	Create Database
+mysqltest	CREATE DATABASE `mysqltest`
+alter database mysqltest collate = utf8_bin;
+show create database mysqltest;
+Database	Create Database
+mysqltest	CREATE DATABASE `mysqltest` COLLATE = utf8_bin
 use mysqltest;
 create table t1 (a varchar(10));
 create table t2 (a varchar(10)) collate = utf8_general_ci;

=== added file 'tests/t/create_max_fields.test'
--- tests/t/create_max_fields.test	1970-01-01 00:00:00 +0000
+++ tests/t/create_max_fields.test	2009-12-08 03:43:57 +0000
@@ -0,0 +1,8202 @@
+--echo One column less than MAX_FIELDS
+CREATE TABLE t1 (
+f1 int,
+f2 int,
+f3 int,
+f4 int,
+f5 int,
+f6 int,
+f7 int,
+f8 int,
+f9 int,
+f10 int,
+f11 int,
+f12 int,
+f13 int,
+f14 int,
+f15 int,
+f16 int,
+f17 int,
+f18 int,
+f19 int,
+f20 int,
+f21 int,
+f22 int,
+f23 int,
+f24 int,
+f25 int,
+f26 int,
+f27 int,
+f28 int,
+f29 int,
+f30 int,
+f31 int,
+f32 int,
+f33 int,
+f34 int,
+f35 int,
+f36 int,
+f37 int,
+f38 int,
+f39 int,
+f40 int,
+f41 int,
+f42 int,
+f43 int,
+f44 int,
+f45 int,
+f46 int,
+f47 int,
+f48 int,
+f49 int,
+f50 int,
+f51 int,
+f52 int,
+f53 int,
+f54 int,
+f55 int,
+f56 int,
+f57 int,
+f58 int,
+f59 int,
+f60 int,
+f61 int,
+f62 int,
+f63 int,
+f64 int,
+f65 int,
+f66 int,
+f67 int,
+f68 int,
+f69 int,
+f70 int,
+f71 int,
+f72 int,
+f73 int,
+f74 int,
+f75 int,
+f76 int,
+f77 int,
+f78 int,
+f79 int,
+f80 int,
+f81 int,
+f82 int,
+f83 int,
+f84 int,
+f85 int,
+f86 int,
+f87 int,
+f88 int,
+f89 int,
+f90 int,
+f91 int,
+f92 int,
+f93 int,
+f94 int,
+f95 int,
+f96 int,
+f97 int,
+f98 int,
+f99 int,
+f100 int,
+f101 int,
+f102 int,
+f103 int,
+f104 int,
+f105 int,
+f106 int,
+f107 int,
+f108 int,
+f109 int,
+f110 int,
+f111 int,
+f112 int,
+f113 int,
+f114 int,
+f115 int,
+f116 int,
+f117 int,
+f118 int,
+f119 int,
+f120 int,
+f121 int,
+f122 int,
+f123 int,
+f124 int,
+f125 int,
+f126 int,
+f127 int,
+f128 int,
+f129 int,
+f130 int,
+f131 int,
+f132 int,
+f133 int,
+f134 int,
+f135 int,
+f136 int,
+f137 int,
+f138 int,
+f139 int,
+f140 int,
+f141 int,
+f142 int,
+f143 int,
+f144 int,
+f145 int,
+f146 int,
+f147 int,
+f148 int,
+f149 int,
+f150 int,
+f151 int,
+f152 int,
+f153 int,
+f154 int,
+f155 int,
+f156 int,
+f157 int,
+f158 int,
+f159 int,
+f160 int,
+f161 int,
+f162 int,
+f163 int,
+f164 int,
+f165 int,
+f166 int,
+f167 int,
+f168 int,
+f169 int,
+f170 int,
+f171 int,
+f172 int,
+f173 int,
+f174 int,
+f175 int,
+f176 int,
+f177 int,
+f178 int,
+f179 int,
+f180 int,
+f181 int,
+f182 int,
+f183 int,
+f184 int,
+f185 int,
+f186 int,
+f187 int,
+f188 int,
+f189 int,
+f190 int,
+f191 int,
+f192 int,
+f193 int,
+f194 int,
+f195 int,
+f196 int,
+f197 int,
+f198 int,
+f199 int,
+f200 int,
+f201 int,
+f202 int,
+f203 int,
+f204 int,
+f205 int,
+f206 int,
+f207 int,
+f208 int,
+f209 int,
+f210 int,
+f211 int,
+f212 int,
+f213 int,
+f214 int,
+f215 int,
+f216 int,
+f217 int,
+f218 int,
+f219 int,
+f220 int,
+f221 int,
+f222 int,
+f223 int,
+f224 int,
+f225 int,
+f226 int,
+f227 int,
+f228 int,
+f229 int,
+f230 int,
+f231 int,
+f232 int,
+f233 int,
+f234 int,
+f235 int,
+f236 int,
+f237 int,
+f238 int,
+f239 int,
+f240 int,
+f241 int,
+f242 int,
+f243 int,
+f244 int,
+f245 int,
+f246 int,
+f247 int,
+f248 int,
+f249 int,
+f250 int,
+f251 int,
+f252 int,
+f253 int,
+f254 int,
+f255 int,
+f256 int,
+f257 int,
+f258 int,
+f259 int,
+f260 int,
+f261 int,
+f262 int,
+f263 int,
+f264 int,
+f265 int,
+f266 int,
+f267 int,
+f268 int,
+f269 int,
+f270 int,
+f271 int,
+f272 int,
+f273 int,
+f274 int,
+f275 int,
+f276 int,
+f277 int,
+f278 int,
+f279 int,
+f280 int,
+f281 int,
+f282 int,
+f283 int,
+f284 int,
+f285 int,
+f286 int,
+f287 int,
+f288 int,
+f289 int,
+f290 int,
+f291 int,
+f292 int,
+f293 int,
+f294 int,
+f295 int,
+f296 int,
+f297 int,
+f298 int,
+f299 int,
+f300 int,
+f301 int,
+f302 int,
+f303 int,
+f304 int,
+f305 int,
+f306 int,
+f307 int,
+f308 int,
+f309 int,
+f310 int,
+f311 int,
+f312 int,
+f313 int,
+f314 int,
+f315 int,
+f316 int,
+f317 int,
+f318 int,
+f319 int,
+f320 int,
+f321 int,
+f322 int,
+f323 int,
+f324 int,
+f325 int,
+f326 int,
+f327 int,
+f328 int,
+f329 int,
+f330 int,
+f331 int,
+f332 int,
+f333 int,
+f334 int,
+f335 int,
+f336 int,
+f337 int,
+f338 int,
+f339 int,
+f340 int,
+f341 int,
+f342 int,
+f343 int,
+f344 int,
+f345 int,
+f346 int,
+f347 int,
+f348 int,
+f349 int,
+f350 int,
+f351 int,
+f352 int,
+f353 int,
+f354 int,
+f355 int,
+f356 int,
+f357 int,
+f358 int,
+f359 int,
+f360 int,
+f361 int,
+f362 int,
+f363 int,
+f364 int,
+f365 int,
+f366 int,
+f367 int,
+f368 int,
+f369 int,
+f370 int,
+f371 int,
+f372 int,
+f373 int,
+f374 int,
+f375 int,
+f376 int,
+f377 int,
+f378 int,
+f379 int,
+f380 int,
+f381 int,
+f382 int,
+f383 int,
+f384 int,
+f385 int,
+f386 int,
+f387 int,
+f388 int,
+f389 int,
+f390 int,
+f391 int,
+f392 int,
+f393 int,
+f394 int,
+f395 int,
+f396 int,
+f397 int,
+f398 int,
+f399 int,
+f400 int,
+f401 int,
+f402 int,
+f403 int,
+f404 int,
+f405 int,
+f406 int,
+f407 int,
+f408 int,
+f409 int,
+f410 int,
+f411 int,
+f412 int,
+f413 int,
+f414 int,
+f415 int,
+f416 int,
+f417 int,
+f418 int,
+f419 int,
+f420 int,
+f421 int,
+f422 int,
+f423 int,
+f424 int,
+f425 int,
+f426 int,
+f427 int,
+f428 int,
+f429 int,
+f430 int,
+f431 int,
+f432 int,
+f433 int,
+f434 int,
+f435 int,
+f436 int,
+f437 int,
+f438 int,
+f439 int,
+f440 int,
+f441 int,
+f442 int,
+f443 int,
+f444 int,
+f445 int,
+f446 int,
+f447 int,
+f448 int,
+f449 int,
+f450 int,
+f451 int,
+f452 int,
+f453 int,
+f454 int,
+f455 int,
+f456 int,
+f457 int,
+f458 int,
+f459 int,
+f460 int,
+f461 int,
+f462 int,
+f463 int,
+f464 int,
+f465 int,
+f466 int,
+f467 int,
+f468 int,
+f469 int,
+f470 int,
+f471 int,
+f472 int,
+f473 int,
+f474 int,
+f475 int,
+f476 int,
+f477 int,
+f478 int,
+f479 int,
+f480 int,
+f481 int,
+f482 int,
+f483 int,
+f484 int,
+f485 int,
+f486 int,
+f487 int,
+f488 int,
+f489 int,
+f490 int,
+f491 int,
+f492 int,
+f493 int,
+f494 int,
+f495 int,
+f496 int,
+f497 int,
+f498 int,
+f499 int,
+f500 int,
+f501 int,
+f502 int,
+f503 int,
+f504 int,
+f505 int,
+f506 int,
+f507 int,
+f508 int,
+f509 int,
+f510 int,
+f511 int,
+f512 int,
+f513 int,
+f514 int,
+f515 int,
+f516 int,
+f517 int,
+f518 int,
+f519 int,
+f520 int,
+f521 int,
+f522 int,
+f523 int,
+f524 int,
+f525 int,
+f526 int,
+f527 int,
+f528 int,
+f529 int,
+f530 int,
+f531 int,
+f532 int,
+f533 int,
+f534 int,
+f535 int,
+f536 int,
+f537 int,
+f538 int,
+f539 int,
+f540 int,
+f541 int,
+f542 int,
+f543 int,
+f544 int,
+f545 int,
+f546 int,
+f547 int,
+f548 int,
+f549 int,
+f550 int,
+f551 int,
+f552 int,
+f553 int,
+f554 int,
+f555 int,
+f556 int,
+f557 int,
+f558 int,
+f559 int,
+f560 int,
+f561 int,
+f562 int,
+f563 int,
+f564 int,
+f565 int,
+f566 int,
+f567 int,
+f568 int,
+f569 int,
+f570 int,
+f571 int,
+f572 int,
+f573 int,
+f574 int,
+f575 int,
+f576 int,
+f577 int,
+f578 int,
+f579 int,
+f580 int,
+f581 int,
+f582 int,
+f583 int,
+f584 int,
+f585 int,
+f586 int,
+f587 int,
+f588 int,
+f589 int,
+f590 int,
+f591 int,
+f592 int,
+f593 int,
+f594 int,
+f595 int,
+f596 int,
+f597 int,
+f598 int,
+f599 int,
+f600 int,
+f601 int,
+f602 int,
+f603 int,
+f604 int,
+f605 int,
+f606 int,
+f607 int,
+f608 int,
+f609 int,
+f610 int,
+f611 int,
+f612 int,
+f613 int,
+f614 int,
+f615 int,
+f616 int,
+f617 int,
+f618 int,
+f619 int,
+f620 int,
+f621 int,
+f622 int,
+f623 int,
+f624 int,
+f625 int,
+f626 int,
+f627 int,
+f628 int,
+f629 int,
+f630 int,
+f631 int,
+f632 int,
+f633 int,
+f634 int,
+f635 int,
+f636 int,
+f637 int,
+f638 int,
+f639 int,
+f640 int,
+f641 int,
+f642 int,
+f643 int,
+f644 int,
+f645 int,
+f646 int,
+f647 int,
+f648 int,
+f649 int,
+f650 int,
+f651 int,
+f652 int,
+f653 int,
+f654 int,
+f655 int,
+f656 int,
+f657 int,
+f658 int,
+f659 int,
+f660 int,
+f661 int,
+f662 int,
+f663 int,
+f664 int,
+f665 int,
+f666 int,
+f667 int,
+f668 int,
+f669 int,
+f670 int,
+f671 int,
+f672 int,
+f673 int,
+f674 int,
+f675 int,
+f676 int,
+f677 int,
+f678 int,
+f679 int,
+f680 int,
+f681 int,
+f682 int,
+f683 int,
+f684 int,
+f685 int,
+f686 int,
+f687 int,
+f688 int,
+f689 int,
+f690 int,
+f691 int,
+f692 int,
+f693 int,
+f694 int,
+f695 int,
+f696 int,
+f697 int,
+f698 int,
+f699 int,
+f700 int,
+f701 int,
+f702 int,
+f703 int,
+f704 int,
+f705 int,
+f706 int,
+f707 int,
+f708 int,
+f709 int,
+f710 int,
+f711 int,
+f712 int,
+f713 int,
+f714 int,
+f715 int,
+f716 int,
+f717 int,
+f718 int,
+f719 int,
+f720 int,
+f721 int,
+f722 int,
+f723 int,
+f724 int,
+f725 int,
+f726 int,
+f727 int,
+f728 int,
+f729 int,
+f730 int,
+f731 int,
+f732 int,
+f733 int,
+f734 int,
+f735 int,
+f736 int,
+f737 int,
+f738 int,
+f739 int,
+f740 int,
+f741 int,
+f742 int,
+f743 int,
+f744 int,
+f745 int,
+f746 int,
+f747 int,
+f748 int,
+f749 int,
+f750 int,
+f751 int,
+f752 int,
+f753 int,
+f754 int,
+f755 int,
+f756 int,
+f757 int,
+f758 int,
+f759 int,
+f760 int,
+f761 int,
+f762 int,
+f763 int,
+f764 int,
+f765 int,
+f766 int,
+f767 int,
+f768 int,
+f769 int,
+f770 int,
+f771 int,
+f772 int,
+f773 int,
+f774 int,
+f775 int,
+f776 int,
+f777 int,
+f778 int,
+f779 int,
+f780 int,
+f781 int,
+f782 int,
+f783 int,
+f784 int,
+f785 int,
+f786 int,
+f787 int,
+f788 int,
+f789 int,
+f790 int,
+f791 int,
+f792 int,
+f793 int,
+f794 int,
+f795 int,
+f796 int,
+f797 int,
+f798 int,
+f799 int,
+f800 int,
+f801 int,
+f802 int,
+f803 int,
+f804 int,
+f805 int,
+f806 int,
+f807 int,
+f808 int,
+f809 int,
+f810 int,
+f811 int,
+f812 int,
+f813 int,
+f814 int,
+f815 int,
+f816 int,
+f817 int,
+f818 int,
+f819 int,
+f820 int,
+f821 int,
+f822 int,
+f823 int,
+f824 int,
+f825 int,
+f826 int,
+f827 int,
+f828 int,
+f829 int,
+f830 int,
+f831 int,
+f832 int,
+f833 int,
+f834 int,
+f835 int,
+f836 int,
+f837 int,
+f838 int,
+f839 int,
+f840 int,
+f841 int,
+f842 int,
+f843 int,
+f844 int,
+f845 int,
+f846 int,
+f847 int,
+f848 int,
+f849 int,
+f850 int,
+f851 int,
+f852 int,
+f853 int,
+f854 int,
+f855 int,
+f856 int,
+f857 int,
+f858 int,
+f859 int,
+f860 int,
+f861 int,
+f862 int,
+f863 int,
+f864 int,
+f865 int,
+f866 int,
+f867 int,
+f868 int,
+f869 int,
+f870 int,
+f871 int,
+f872 int,
+f873 int,
+f874 int,
+f875 int,
+f876 int,
+f877 int,
+f878 int,
+f879 int,
+f880 int,
+f881 int,
+f882 int,
+f883 int,
+f884 int,
+f885 int,
+f886 int,
+f887 int,
+f888 int,
+f889 int,
+f890 int,
+f891 int,
+f892 int,
+f893 int,
+f894 int,
+f895 int,
+f896 int,
+f897 int,
+f898 int,
+f899 int,
+f900 int,
+f901 int,
+f902 int,
+f903 int,
+f904 int,
+f905 int,
+f906 int,
+f907 int,
+f908 int,
+f909 int,
+f910 int,
+f911 int,
+f912 int,
+f913 int,
+f914 int,
+f915 int,
+f916 int,
+f917 int,
+f918 int,
+f919 int,
+f920 int,
+f921 int,
+f922 int,
+f923 int,
+f924 int,
+f925 int,
+f926 int,
+f927 int,
+f928 int,
+f929 int,
+f930 int,
+f931 int,
+f932 int,
+f933 int,
+f934 int,
+f935 int,
+f936 int,
+f937 int,
+f938 int,
+f939 int,
+f940 int,
+f941 int,
+f942 int,
+f943 int,
+f944 int,
+f945 int,
+f946 int,
+f947 int,
+f948 int,
+f949 int,
+f950 int,
+f951 int,
+f952 int,
+f953 int,
+f954 int,
+f955 int,
+f956 int,
+f957 int,
+f958 int,
+f959 int,
+f960 int,
+f961 int,
+f962 int,
+f963 int,
+f964 int,
+f965 int,
+f966 int,
+f967 int,
+f968 int,
+f969 int,
+f970 int,
+f971 int,
+f972 int,
+f973 int,
+f974 int,
+f975 int,
+f976 int,
+f977 int,
+f978 int,
+f979 int,
+f980 int,
+f981 int,
+f982 int,
+f983 int,
+f984 int,
+f985 int,
+f986 int,
+f987 int,
+f988 int,
+f989 int,
+f990 int,
+f991 int,
+f992 int,
+f993 int,
+f994 int,
+f995 int,
+f996 int,
+f997 int,
+f998 int,
+f999 int,
+f1000 int,
+f1001 int,
+f1002 int,
+f1003 int,
+f1004 int,
+f1005 int,
+f1006 int,
+f1007 int,
+f1008 int,
+f1009 int,
+f1010 int,
+f1011 int,
+f1012 int,
+f1013 int,
+f1014 int,
+f1015 int,
+f1016 int,
+f1017 int,
+f1018 int,
+f1019 int,
+f1020 int,
+f1021 int,
+f1022 int,
+f1023 int,
+f1024 int,
+f1025 int,
+f1026 int,
+f1027 int,
+f1028 int,
+f1029 int,
+f1030 int,
+f1031 int,
+f1032 int,
+f1033 int,
+f1034 int,
+f1035 int,
+f1036 int,
+f1037 int,
+f1038 int,
+f1039 int,
+f1040 int,
+f1041 int,
+f1042 int,
+f1043 int,
+f1044 int,
+f1045 int,
+f1046 int,
+f1047 int,
+f1048 int,
+f1049 int,
+f1050 int,
+f1051 int,
+f1052 int,
+f1053 int,
+f1054 int,
+f1055 int,
+f1056 int,
+f1057 int,
+f1058 int,
+f1059 int,
+f1060 int,
+f1061 int,
+f1062 int,
+f1063 int,
+f1064 int,
+f1065 int,
+f1066 int,
+f1067 int,
+f1068 int,
+f1069 int,
+f1070 int,
+f1071 int,
+f1072 int,
+f1073 int,
+f1074 int,
+f1075 int,
+f1076 int,
+f1077 int,
+f1078 int,
+f1079 int,
+f1080 int,
+f1081 int,
+f1082 int,
+f1083 int,
+f1084 int,
+f1085 int,
+f1086 int,
+f1087 int,
+f1088 int,
+f1089 int,
+f1090 int,
+f1091 int,
+f1092 int,
+f1093 int,
+f1094 int,
+f1095 int,
+f1096 int,
+f1097 int,
+f1098 int,
+f1099 int,
+f1100 int,
+f1101 int,
+f1102 int,
+f1103 int,
+f1104 int,
+f1105 int,
+f1106 int,
+f1107 int,
+f1108 int,
+f1109 int,
+f1110 int,
+f1111 int,
+f1112 int,
+f1113 int,
+f1114 int,
+f1115 int,
+f1116 int,
+f1117 int,
+f1118 int,
+f1119 int,
+f1120 int,
+f1121 int,
+f1122 int,
+f1123 int,
+f1124 int,
+f1125 int,
+f1126 int,
+f1127 int,
+f1128 int,
+f1129 int,
+f1130 int,
+f1131 int,
+f1132 int,
+f1133 int,
+f1134 int,
+f1135 int,
+f1136 int,
+f1137 int,
+f1138 int,
+f1139 int,
+f1140 int,
+f1141 int,
+f1142 int,
+f1143 int,
+f1144 int,
+f1145 int,
+f1146 int,
+f1147 int,
+f1148 int,
+f1149 int,
+f1150 int,
+f1151 int,
+f1152 int,
+f1153 int,
+f1154 int,
+f1155 int,
+f1156 int,
+f1157 int,
+f1158 int,
+f1159 int,
+f1160 int,
+f1161 int,
+f1162 int,
+f1163 int,
+f1164 int,
+f1165 int,
+f1166 int,
+f1167 int,
+f1168 int,
+f1169 int,
+f1170 int,
+f1171 int,
+f1172 int,
+f1173 int,
+f1174 int,
+f1175 int,
+f1176 int,
+f1177 int,
+f1178 int,
+f1179 int,
+f1180 int,
+f1181 int,
+f1182 int,
+f1183 int,
+f1184 int,
+f1185 int,
+f1186 int,
+f1187 int,
+f1188 int,
+f1189 int,
+f1190 int,
+f1191 int,
+f1192 int,
+f1193 int,
+f1194 int,
+f1195 int,
+f1196 int,
+f1197 int,
+f1198 int,
+f1199 int,
+f1200 int,
+f1201 int,
+f1202 int,
+f1203 int,
+f1204 int,
+f1205 int,
+f1206 int,
+f1207 int,
+f1208 int,
+f1209 int,
+f1210 int,
+f1211 int,
+f1212 int,
+f1213 int,
+f1214 int,
+f1215 int,
+f1216 int,
+f1217 int,
+f1218 int,
+f1219 int,
+f1220 int,
+f1221 int,
+f1222 int,
+f1223 int,
+f1224 int,
+f1225 int,
+f1226 int,
+f1227 int,
+f1228 int,
+f1229 int,
+f1230 int,
+f1231 int,
+f1232 int,
+f1233 int,
+f1234 int,
+f1235 int,
+f1236 int,
+f1237 int,
+f1238 int,
+f1239 int,
+f1240 int,
+f1241 int,
+f1242 int,
+f1243 int,
+f1244 int,
+f1245 int,
+f1246 int,
+f1247 int,
+f1248 int,
+f1249 int,
+f1250 int,
+f1251 int,
+f1252 int,
+f1253 int,
+f1254 int,
+f1255 int,
+f1256 int,
+f1257 int,
+f1258 int,
+f1259 int,
+f1260 int,
+f1261 int,
+f1262 int,
+f1263 int,
+f1264 int,
+f1265 int,
+f1266 int,
+f1267 int,
+f1268 int,
+f1269 int,
+f1270 int,
+f1271 int,
+f1272 int,
+f1273 int,
+f1274 int,
+f1275 int,
+f1276 int,
+f1277 int,
+f1278 int,
+f1279 int,
+f1280 int,
+f1281 int,
+f1282 int,
+f1283 int,
+f1284 int,
+f1285 int,
+f1286 int,
+f1287 int,
+f1288 int,
+f1289 int,
+f1290 int,
+f1291 int,
+f1292 int,
+f1293 int,
+f1294 int,
+f1295 int,
+f1296 int,
+f1297 int,
+f1298 int,
+f1299 int,
+f1300 int,
+f1301 int,
+f1302 int,
+f1303 int,
+f1304 int,
+f1305 int,
+f1306 int,
+f1307 int,
+f1308 int,
+f1309 int,
+f1310 int,
+f1311 int,
+f1312 int,
+f1313 int,
+f1314 int,
+f1315 int,
+f1316 int,
+f1317 int,
+f1318 int,
+f1319 int,
+f1320 int,
+f1321 int,
+f1322 int,
+f1323 int,
+f1324 int,
+f1325 int,
+f1326 int,
+f1327 int,
+f1328 int,
+f1329 int,
+f1330 int,
+f1331 int,
+f1332 int,
+f1333 int,
+f1334 int,
+f1335 int,
+f1336 int,
+f1337 int,
+f1338 int,
+f1339 int,
+f1340 int,
+f1341 int,
+f1342 int,
+f1343 int,
+f1344 int,
+f1345 int,
+f1346 int,
+f1347 int,
+f1348 int,
+f1349 int,
+f1350 int,
+f1351 int,
+f1352 int,
+f1353 int,
+f1354 int,
+f1355 int,
+f1356 int,
+f1357 int,
+f1358 int,
+f1359 int,
+f1360 int,
+f1361 int,
+f1362 int,
+f1363 int,
+f1364 int,
+f1365 int,
+f1366 int,
+f1367 int,
+f1368 int,
+f1369 int,
+f1370 int,
+f1371 int,
+f1372 int,
+f1373 int,
+f1374 int,
+f1375 int,
+f1376 int,
+f1377 int,
+f1378 int,
+f1379 int,
+f1380 int,
+f1381 int,
+f1382 int,
+f1383 int,
+f1384 int,
+f1385 int,
+f1386 int,
+f1387 int,
+f1388 int,
+f1389 int,
+f1390 int,
+f1391 int,
+f1392 int,
+f1393 int,
+f1394 int,
+f1395 int,
+f1396 int,
+f1397 int,
+f1398 int,
+f1399 int,
+f1400 int,
+f1401 int,
+f1402 int,
+f1403 int,
+f1404 int,
+f1405 int,
+f1406 int,
+f1407 int,
+f1408 int,
+f1409 int,
+f1410 int,
+f1411 int,
+f1412 int,
+f1413 int,
+f1414 int,
+f1415 int,
+f1416 int,
+f1417 int,
+f1418 int,
+f1419 int,
+f1420 int,
+f1421 int,
+f1422 int,
+f1423 int,
+f1424 int,
+f1425 int,
+f1426 int,
+f1427 int,
+f1428 int,
+f1429 int,
+f1430 int,
+f1431 int,
+f1432 int,
+f1433 int,
+f1434 int,
+f1435 int,
+f1436 int,
+f1437 int,
+f1438 int,
+f1439 int,
+f1440 int,
+f1441 int,
+f1442 int,
+f1443 int,
+f1444 int,
+f1445 int,
+f1446 int,
+f1447 int,
+f1448 int,
+f1449 int,
+f1450 int,
+f1451 int,
+f1452 int,
+f1453 int,
+f1454 int,
+f1455 int,
+f1456 int,
+f1457 int,
+f1458 int,
+f1459 int,
+f1460 int,
+f1461 int,
+f1462 int,
+f1463 int,
+f1464 int,
+f1465 int,
+f1466 int,
+f1467 int,
+f1468 int,
+f1469 int,
+f1470 int,
+f1471 int,
+f1472 int,
+f1473 int,
+f1474 int,
+f1475 int,
+f1476 int,
+f1477 int,
+f1478 int,
+f1479 int,
+f1480 int,
+f1481 int,
+f1482 int,
+f1483 int,
+f1484 int,
+f1485 int,
+f1486 int,
+f1487 int,
+f1488 int,
+f1489 int,
+f1490 int,
+f1491 int,
+f1492 int,
+f1493 int,
+f1494 int,
+f1495 int,
+f1496 int,
+f1497 int,
+f1498 int,
+f1499 int,
+f1500 int,
+f1501 int,
+f1502 int,
+f1503 int,
+f1504 int,
+f1505 int,
+f1506 int,
+f1507 int,
+f1508 int,
+f1509 int,
+f1510 int,
+f1511 int,
+f1512 int,
+f1513 int,
+f1514 int,
+f1515 int,
+f1516 int,
+f1517 int,
+f1518 int,
+f1519 int,
+f1520 int,
+f1521 int,
+f1522 int,
+f1523 int,
+f1524 int,
+f1525 int,
+f1526 int,
+f1527 int,
+f1528 int,
+f1529 int,
+f1530 int,
+f1531 int,
+f1532 int,
+f1533 int,
+f1534 int,
+f1535 int,
+f1536 int,
+f1537 int,
+f1538 int,
+f1539 int,
+f1540 int,
+f1541 int,
+f1542 int,
+f1543 int,
+f1544 int,
+f1545 int,
+f1546 int,
+f1547 int,
+f1548 int,
+f1549 int,
+f1550 int,
+f1551 int,
+f1552 int,
+f1553 int,
+f1554 int,
+f1555 int,
+f1556 int,
+f1557 int,
+f1558 int,
+f1559 int,
+f1560 int,
+f1561 int,
+f1562 int,
+f1563 int,
+f1564 int,
+f1565 int,
+f1566 int,
+f1567 int,
+f1568 int,
+f1569 int,
+f1570 int,
+f1571 int,
+f1572 int,
+f1573 int,
+f1574 int,
+f1575 int,
+f1576 int,
+f1577 int,
+f1578 int,
+f1579 int,
+f1580 int,
+f1581 int,
+f1582 int,
+f1583 int,
+f1584 int,
+f1585 int,
+f1586 int,
+f1587 int,
+f1588 int,
+f1589 int,
+f1590 int,
+f1591 int,
+f1592 int,
+f1593 int,
+f1594 int,
+f1595 int,
+f1596 int,
+f1597 int,
+f1598 int,
+f1599 int,
+f1600 int,
+f1601 int,
+f1602 int,
+f1603 int,
+f1604 int,
+f1605 int,
+f1606 int,
+f1607 int,
+f1608 int,
+f1609 int,
+f1610 int,
+f1611 int,
+f1612 int,
+f1613 int,
+f1614 int,
+f1615 int,
+f1616 int,
+f1617 int,
+f1618 int,
+f1619 int,
+f1620 int,
+f1621 int,
+f1622 int,
+f1623 int,
+f1624 int,
+f1625 int,
+f1626 int,
+f1627 int,
+f1628 int,
+f1629 int,
+f1630 int,
+f1631 int,
+f1632 int,
+f1633 int,
+f1634 int,
+f1635 int,
+f1636 int,
+f1637 int,
+f1638 int,
+f1639 int,
+f1640 int,
+f1641 int,
+f1642 int,
+f1643 int,
+f1644 int,
+f1645 int,
+f1646 int,
+f1647 int,
+f1648 int,
+f1649 int,
+f1650 int,
+f1651 int,
+f1652 int,
+f1653 int,
+f1654 int,
+f1655 int,
+f1656 int,
+f1657 int,
+f1658 int,
+f1659 int,
+f1660 int,
+f1661 int,
+f1662 int,
+f1663 int,
+f1664 int,
+f1665 int,
+f1666 int,
+f1667 int,
+f1668 int,
+f1669 int,
+f1670 int,
+f1671 int,
+f1672 int,
+f1673 int,
+f1674 int,
+f1675 int,
+f1676 int,
+f1677 int,
+f1678 int,
+f1679 int,
+f1680 int,
+f1681 int,
+f1682 int,
+f1683 int,
+f1684 int,
+f1685 int,
+f1686 int,
+f1687 int,
+f1688 int,
+f1689 int,
+f1690 int,
+f1691 int,
+f1692 int,
+f1693 int,
+f1694 int,
+f1695 int,
+f1696 int,
+f1697 int,
+f1698 int,
+f1699 int,
+f1700 int,
+f1701 int,
+f1702 int,
+f1703 int,
+f1704 int,
+f1705 int,
+f1706 int,
+f1707 int,
+f1708 int,
+f1709 int,
+f1710 int,
+f1711 int,
+f1712 int,
+f1713 int,
+f1714 int,
+f1715 int,
+f1716 int,
+f1717 int,
+f1718 int,
+f1719 int,
+f1720 int,
+f1721 int,
+f1722 int,
+f1723 int,
+f1724 int,
+f1725 int,
+f1726 int,
+f1727 int,
+f1728 int,
+f1729 int,
+f1730 int,
+f1731 int,
+f1732 int,
+f1733 int,
+f1734 int,
+f1735 int,
+f1736 int,
+f1737 int,
+f1738 int,
+f1739 int,
+f1740 int,
+f1741 int,
+f1742 int,
+f1743 int,
+f1744 int,
+f1745 int,
+f1746 int,
+f1747 int,
+f1748 int,
+f1749 int,
+f1750 int,
+f1751 int,
+f1752 int,
+f1753 int,
+f1754 int,
+f1755 int,
+f1756 int,
+f1757 int,
+f1758 int,
+f1759 int,
+f1760 int,
+f1761 int,
+f1762 int,
+f1763 int,
+f1764 int,
+f1765 int,
+f1766 int,
+f1767 int,
+f1768 int,
+f1769 int,
+f1770 int,
+f1771 int,
+f1772 int,
+f1773 int,
+f1774 int,
+f1775 int,
+f1776 int,
+f1777 int,
+f1778 int,
+f1779 int,
+f1780 int,
+f1781 int,
+f1782 int,
+f1783 int,
+f1784 int,
+f1785 int,
+f1786 int,
+f1787 int,
+f1788 int,
+f1789 int,
+f1790 int,
+f1791 int,
+f1792 int,
+f1793 int,
+f1794 int,
+f1795 int,
+f1796 int,
+f1797 int,
+f1798 int,
+f1799 int,
+f1800 int,
+f1801 int,
+f1802 int,
+f1803 int,
+f1804 int,
+f1805 int,
+f1806 int,
+f1807 int,
+f1808 int,
+f1809 int,
+f1810 int,
+f1811 int,
+f1812 int,
+f1813 int,
+f1814 int,
+f1815 int,
+f1816 int,
+f1817 int,
+f1818 int,
+f1819 int,
+f1820 int,
+f1821 int,
+f1822 int,
+f1823 int,
+f1824 int,
+f1825 int,
+f1826 int,
+f1827 int,
+f1828 int,
+f1829 int,
+f1830 int,
+f1831 int,
+f1832 int,
+f1833 int,
+f1834 int,
+f1835 int,
+f1836 int,
+f1837 int,
+f1838 int,
+f1839 int,
+f1840 int,
+f1841 int,
+f1842 int,
+f1843 int,
+f1844 int,
+f1845 int,
+f1846 int,
+f1847 int,
+f1848 int,
+f1849 int,
+f1850 int,
+f1851 int,
+f1852 int,
+f1853 int,
+f1854 int,
+f1855 int,
+f1856 int,
+f1857 int,
+f1858 int,
+f1859 int,
+f1860 int,
+f1861 int,
+f1862 int,
+f1863 int,
+f1864 int,
+f1865 int,
+f1866 int,
+f1867 int,
+f1868 int,
+f1869 int,
+f1870 int,
+f1871 int,
+f1872 int,
+f1873 int,
+f1874 int,
+f1875 int,
+f1876 int,
+f1877 int,
+f1878 int,
+f1879 int,
+f1880 int,
+f1881 int,
+f1882 int,
+f1883 int,
+f1884 int,
+f1885 int,
+f1886 int,
+f1887 int,
+f1888 int,
+f1889 int,
+f1890 int,
+f1891 int,
+f1892 int,
+f1893 int,
+f1894 int,
+f1895 int,
+f1896 int,
+f1897 int,
+f1898 int,
+f1899 int,
+f1900 int,
+f1901 int,
+f1902 int,
+f1903 int,
+f1904 int,
+f1905 int,
+f1906 int,
+f1907 int,
+f1908 int,
+f1909 int,
+f1910 int,
+f1911 int,
+f1912 int,
+f1913 int,
+f1914 int,
+f1915 int,
+f1916 int,
+f1917 int,
+f1918 int,
+f1919 int,
+f1920 int,
+f1921 int,
+f1922 int,
+f1923 int,
+f1924 int,
+f1925 int,
+f1926 int,
+f1927 int,
+f1928 int,
+f1929 int,
+f1930 int,
+f1931 int,
+f1932 int,
+f1933 int,
+f1934 int,
+f1935 int,
+f1936 int,
+f1937 int,
+f1938 int,
+f1939 int,
+f1940 int,
+f1941 int,
+f1942 int,
+f1943 int,
+f1944 int,
+f1945 int,
+f1946 int,
+f1947 int,
+f1948 int,
+f1949 int,
+f1950 int,
+f1951 int,
+f1952 int,
+f1953 int,
+f1954 int,
+f1955 int,
+f1956 int,
+f1957 int,
+f1958 int,
+f1959 int,
+f1960 int,
+f1961 int,
+f1962 int,
+f1963 int,
+f1964 int,
+f1965 int,
+f1966 int,
+f1967 int,
+f1968 int,
+f1969 int,
+f1970 int,
+f1971 int,
+f1972 int,
+f1973 int,
+f1974 int,
+f1975 int,
+f1976 int,
+f1977 int,
+f1978 int,
+f1979 int,
+f1980 int,
+f1981 int,
+f1982 int,
+f1983 int,
+f1984 int,
+f1985 int,
+f1986 int,
+f1987 int,
+f1988 int,
+f1989 int,
+f1990 int,
+f1991 int,
+f1992 int,
+f1993 int,
+f1994 int,
+f1995 int,
+f1996 int,
+f1997 int,
+f1998 int,
+f1999 int,
+f2000 int,
+f2001 int,
+f2002 int,
+f2003 int,
+f2004 int,
+f2005 int,
+f2006 int,
+f2007 int,
+f2008 int,
+f2009 int,
+f2010 int,
+f2011 int,
+f2012 int,
+f2013 int,
+f2014 int,
+f2015 int,
+f2016 int,
+f2017 int,
+f2018 int,
+f2019 int,
+f2020 int,
+f2021 int,
+f2022 int,
+f2023 int,
+f2024 int,
+f2025 int,
+f2026 int,
+f2027 int,
+f2028 int,
+f2029 int,
+f2030 int,
+f2031 int,
+f2032 int,
+f2033 int,
+f2034 int,
+f2035 int,
+f2036 int,
+f2037 int,
+f2038 int,
+f2039 int,
+f2040 int,
+f2041 int,
+f2042 int,
+f2043 int,
+f2044 int,
+f2045 int,
+f2046 int,
+f2047 int,
+f2048 int,
+f2049 int,
+f2050 int,
+f2051 int,
+f2052 int,
+f2053 int,
+f2054 int,
+f2055 int,
+f2056 int,
+f2057 int,
+f2058 int,
+f2059 int,
+f2060 int,
+f2061 int,
+f2062 int,
+f2063 int,
+f2064 int,
+f2065 int,
+f2066 int,
+f2067 int,
+f2068 int,
+f2069 int,
+f2070 int,
+f2071 int,
+f2072 int,
+f2073 int,
+f2074 int,
+f2075 int,
+f2076 int,
+f2077 int,
+f2078 int,
+f2079 int,
+f2080 int,
+f2081 int,
+f2082 int,
+f2083 int,
+f2084 int,
+f2085 int,
+f2086 int,
+f2087 int,
+f2088 int,
+f2089 int,
+f2090 int,
+f2091 int,
+f2092 int,
+f2093 int,
+f2094 int,
+f2095 int,
+f2096 int,
+f2097 int,
+f2098 int,
+f2099 int,
+f2100 int,
+f2101 int,
+f2102 int,
+f2103 int,
+f2104 int,
+f2105 int,
+f2106 int,
+f2107 int,
+f2108 int,
+f2109 int,
+f2110 int,
+f2111 int,
+f2112 int,
+f2113 int,
+f2114 int,
+f2115 int,
+f2116 int,
+f2117 int,
+f2118 int,
+f2119 int,
+f2120 int,
+f2121 int,
+f2122 int,
+f2123 int,
+f2124 int,
+f2125 int,
+f2126 int,
+f2127 int,
+f2128 int,
+f2129 int,
+f2130 int,
+f2131 int,
+f2132 int,
+f2133 int,
+f2134 int,
+f2135 int,
+f2136 int,
+f2137 int,
+f2138 int,
+f2139 int,
+f2140 int,
+f2141 int,
+f2142 int,
+f2143 int,
+f2144 int,
+f2145 int,
+f2146 int,
+f2147 int,
+f2148 int,
+f2149 int,
+f2150 int,
+f2151 int,
+f2152 int,
+f2153 int,
+f2154 int,
+f2155 int,
+f2156 int,
+f2157 int,
+f2158 int,
+f2159 int,
+f2160 int,
+f2161 int,
+f2162 int,
+f2163 int,
+f2164 int,
+f2165 int,
+f2166 int,
+f2167 int,
+f2168 int,
+f2169 int,
+f2170 int,
+f2171 int,
+f2172 int,
+f2173 int,
+f2174 int,
+f2175 int,
+f2176 int,
+f2177 int,
+f2178 int,
+f2179 int,
+f2180 int,
+f2181 int,
+f2182 int,
+f2183 int,
+f2184 int,
+f2185 int,
+f2186 int,
+f2187 int,
+f2188 int,
+f2189 int,
+f2190 int,
+f2191 int,
+f2192 int,
+f2193 int,
+f2194 int,
+f2195 int,
+f2196 int,
+f2197 int,
+f2198 int,
+f2199 int,
+f2200 int,
+f2201 int,
+f2202 int,
+f2203 int,
+f2204 int,
+f2205 int,
+f2206 int,
+f2207 int,
+f2208 int,
+f2209 int,
+f2210 int,
+f2211 int,
+f2212 int,
+f2213 int,
+f2214 int,
+f2215 int,
+f2216 int,
+f2217 int,
+f2218 int,
+f2219 int,
+f2220 int,
+f2221 int,
+f2222 int,
+f2223 int,
+f2224 int,
+f2225 int,
+f2226 int,
+f2227 int,
+f2228 int,
+f2229 int,
+f2230 int,
+f2231 int,
+f2232 int,
+f2233 int,
+f2234 int,
+f2235 int,
+f2236 int,
+f2237 int,
+f2238 int,
+f2239 int,
+f2240 int,
+f2241 int,
+f2242 int,
+f2243 int,
+f2244 int,
+f2245 int,
+f2246 int,
+f2247 int,
+f2248 int,
+f2249 int,
+f2250 int,
+f2251 int,
+f2252 int,
+f2253 int,
+f2254 int,
+f2255 int,
+f2256 int,
+f2257 int,
+f2258 int,
+f2259 int,
+f2260 int,
+f2261 int,
+f2262 int,
+f2263 int,
+f2264 int,
+f2265 int,
+f2266 int,
+f2267 int,
+f2268 int,
+f2269 int,
+f2270 int,
+f2271 int,
+f2272 int,
+f2273 int,
+f2274 int,
+f2275 int,
+f2276 int,
+f2277 int,
+f2278 int,
+f2279 int,
+f2280 int,
+f2281 int,
+f2282 int,
+f2283 int,
+f2284 int,
+f2285 int,
+f2286 int,
+f2287 int,
+f2288 int,
+f2289 int,
+f2290 int,
+f2291 int,
+f2292 int,
+f2293 int,
+f2294 int,
+f2295 int,
+f2296 int,
+f2297 int,
+f2298 int,
+f2299 int,
+f2300 int,
+f2301 int,
+f2302 int,
+f2303 int,
+f2304 int,
+f2305 int,
+f2306 int,
+f2307 int,
+f2308 int,
+f2309 int,
+f2310 int,
+f2311 int,
+f2312 int,
+f2313 int,
+f2314 int,
+f2315 int,
+f2316 int,
+f2317 int,
+f2318 int,
+f2319 int,
+f2320 int,
+f2321 int,
+f2322 int,
+f2323 int,
+f2324 int,
+f2325 int,
+f2326 int,
+f2327 int,
+f2328 int,
+f2329 int,
+f2330 int,
+f2331 int,
+f2332 int,
+f2333 int,
+f2334 int,
+f2335 int,
+f2336 int,
+f2337 int,
+f2338 int,
+f2339 int,
+f2340 int,
+f2341 int,
+f2342 int,
+f2343 int,
+f2344 int,
+f2345 int,
+f2346 int,
+f2347 int,
+f2348 int,
+f2349 int,
+f2350 int,
+f2351 int,
+f2352 int,
+f2353 int,
+f2354 int,
+f2355 int,
+f2356 int,
+f2357 int,
+f2358 int,
+f2359 int,
+f2360 int,
+f2361 int,
+f2362 int,
+f2363 int,
+f2364 int,
+f2365 int,
+f2366 int,
+f2367 int,
+f2368 int,
+f2369 int,
+f2370 int,
+f2371 int,
+f2372 int,
+f2373 int,
+f2374 int,
+f2375 int,
+f2376 int,
+f2377 int,
+f2378 int,
+f2379 int,
+f2380 int,
+f2381 int,
+f2382 int,
+f2383 int,
+f2384 int,
+f2385 int,
+f2386 int,
+f2387 int,
+f2388 int,
+f2389 int,
+f2390 int,
+f2391 int,
+f2392 int,
+f2393 int,
+f2394 int,
+f2395 int,
+f2396 int,
+f2397 int,
+f2398 int,
+f2399 int,
+f2400 int,
+f2401 int,
+f2402 int,
+f2403 int,
+f2404 int,
+f2405 int,
+f2406 int,
+f2407 int,
+f2408 int,
+f2409 int,
+f2410 int,
+f2411 int,
+f2412 int,
+f2413 int,
+f2414 int,
+f2415 int,
+f2416 int,
+f2417 int,
+f2418 int,
+f2419 int,
+f2420 int,
+f2421 int,
+f2422 int,
+f2423 int,
+f2424 int,
+f2425 int,
+f2426 int,
+f2427 int,
+f2428 int,
+f2429 int,
+f2430 int,
+f2431 int,
+f2432 int,
+f2433 int,
+f2434 int,
+f2435 int,
+f2436 int,
+f2437 int,
+f2438 int,
+f2439 int,
+f2440 int,
+f2441 int,
+f2442 int,
+f2443 int,
+f2444 int,
+f2445 int,
+f2446 int,
+f2447 int,
+f2448 int,
+f2449 int,
+f2450 int,
+f2451 int,
+f2452 int,
+f2453 int,
+f2454 int,
+f2455 int,
+f2456 int,
+f2457 int,
+f2458 int,
+f2459 int,
+f2460 int,
+f2461 int,
+f2462 int,
+f2463 int,
+f2464 int,
+f2465 int,
+f2466 int,
+f2467 int,
+f2468 int,
+f2469 int,
+f2470 int,
+f2471 int,
+f2472 int,
+f2473 int,
+f2474 int,
+f2475 int,
+f2476 int,
+f2477 int,
+f2478 int,
+f2479 int,
+f2480 int,
+f2481 int,
+f2482 int,
+f2483 int,
+f2484 int,
+f2485 int,
+f2486 int,
+f2487 int,
+f2488 int,
+f2489 int,
+f2490 int,
+f2491 int,
+f2492 int,
+f2493 int,
+f2494 int,
+f2495 int,
+f2496 int,
+f2497 int,
+f2498 int,
+f2499 int,
+f2500 int,
+f2501 int,
+f2502 int,
+f2503 int,
+f2504 int,
+f2505 int,
+f2506 int,
+f2507 int,
+f2508 int,
+f2509 int,
+f2510 int,
+f2511 int,
+f2512 int,
+f2513 int,
+f2514 int,
+f2515 int,
+f2516 int,
+f2517 int,
+f2518 int,
+f2519 int,
+f2520 int,
+f2521 int,
+f2522 int,
+f2523 int,
+f2524 int,
+f2525 int,
+f2526 int,
+f2527 int,
+f2528 int,
+f2529 int,
+f2530 int,
+f2531 int,
+f2532 int,
+f2533 int,
+f2534 int,
+f2535 int,
+f2536 int,
+f2537 int,
+f2538 int,
+f2539 int,
+f2540 int,
+f2541 int,
+f2542 int,
+f2543 int,
+f2544 int,
+f2545 int,
+f2546 int,
+f2547 int,
+f2548 int,
+f2549 int,
+f2550 int,
+f2551 int,
+f2552 int,
+f2553 int,
+f2554 int,
+f2555 int,
+f2556 int,
+f2557 int,
+f2558 int,
+f2559 int,
+f2560 int,
+f2561 int,
+f2562 int,
+f2563 int,
+f2564 int,
+f2565 int,
+f2566 int,
+f2567 int,
+f2568 int,
+f2569 int,
+f2570 int,
+f2571 int,
+f2572 int,
+f2573 int,
+f2574 int,
+f2575 int,
+f2576 int,
+f2577 int,
+f2578 int,
+f2579 int,
+f2580 int,
+f2581 int,
+f2582 int,
+f2583 int,
+f2584 int,
+f2585 int,
+f2586 int,
+f2587 int,
+f2588 int,
+f2589 int,
+f2590 int,
+f2591 int,
+f2592 int,
+f2593 int,
+f2594 int,
+f2595 int,
+f2596 int,
+f2597 int,
+f2598 int,
+f2599 int,
+f2600 int,
+f2601 int,
+f2602 int,
+f2603 int,
+f2604 int,
+f2605 int,
+f2606 int,
+f2607 int,
+f2608 int,
+f2609 int,
+f2610 int,
+f2611 int,
+f2612 int,
+f2613 int,
+f2614 int,
+f2615 int,
+f2616 int,
+f2617 int,
+f2618 int,
+f2619 int,
+f2620 int,
+f2621 int,
+f2622 int,
+f2623 int,
+f2624 int,
+f2625 int,
+f2626 int,
+f2627 int,
+f2628 int,
+f2629 int,
+f2630 int,
+f2631 int,
+f2632 int,
+f2633 int,
+f2634 int,
+f2635 int,
+f2636 int,
+f2637 int,
+f2638 int,
+f2639 int,
+f2640 int,
+f2641 int,
+f2642 int,
+f2643 int,
+f2644 int,
+f2645 int,
+f2646 int,
+f2647 int,
+f2648 int,
+f2649 int,
+f2650 int,
+f2651 int,
+f2652 int,
+f2653 int,
+f2654 int,
+f2655 int,
+f2656 int,
+f2657 int,
+f2658 int,
+f2659 int,
+f2660 int,
+f2661 int,
+f2662 int,
+f2663 int,
+f2664 int,
+f2665 int,
+f2666 int,
+f2667 int,
+f2668 int,
+f2669 int,
+f2670 int,
+f2671 int,
+f2672 int,
+f2673 int,
+f2674 int,
+f2675 int,
+f2676 int,
+f2677 int,
+f2678 int,
+f2679 int,
+f2680 int,
+f2681 int,
+f2682 int,
+f2683 int,
+f2684 int,
+f2685 int,
+f2686 int,
+f2687 int,
+f2688 int,
+f2689 int,
+f2690 int,
+f2691 int,
+f2692 int,
+f2693 int,
+f2694 int,
+f2695 int,
+f2696 int,
+f2697 int,
+f2698 int,
+f2699 int,
+f2700 int,
+f2701 int,
+f2702 int,
+f2703 int,
+f2704 int,
+f2705 int,
+f2706 int,
+f2707 int,
+f2708 int,
+f2709 int,
+f2710 int,
+f2711 int,
+f2712 int,
+f2713 int,
+f2714 int,
+f2715 int,
+f2716 int,
+f2717 int,
+f2718 int,
+f2719 int,
+f2720 int,
+f2721 int,
+f2722 int,
+f2723 int,
+f2724 int,
+f2725 int,
+f2726 int,
+f2727 int,
+f2728 int,
+f2729 int,
+f2730 int,
+f2731 int,
+f2732 int,
+f2733 int,
+f2734 int,
+f2735 int,
+f2736 int,
+f2737 int,
+f2738 int,
+f2739 int,
+f2740 int,
+f2741 int,
+f2742 int,
+f2743 int,
+f2744 int,
+f2745 int,
+f2746 int,
+f2747 int,
+f2748 int,
+f2749 int,
+f2750 int,
+f2751 int,
+f2752 int,
+f2753 int,
+f2754 int,
+f2755 int,
+f2756 int,
+f2757 int,
+f2758 int,
+f2759 int,
+f2760 int,
+f2761 int,
+f2762 int,
+f2763 int,
+f2764 int,
+f2765 int,
+f2766 int,
+f2767 int,
+f2768 int,
+f2769 int,
+f2770 int,
+f2771 int,
+f2772 int,
+f2773 int,
+f2774 int,
+f2775 int,
+f2776 int,
+f2777 int,
+f2778 int,
+f2779 int,
+f2780 int,
+f2781 int,
+f2782 int,
+f2783 int,
+f2784 int,
+f2785 int,
+f2786 int,
+f2787 int,
+f2788 int,
+f2789 int,
+f2790 int,
+f2791 int,
+f2792 int,
+f2793 int,
+f2794 int,
+f2795 int,
+f2796 int,
+f2797 int,
+f2798 int,
+f2799 int,
+f2800 int,
+f2801 int,
+f2802 int,
+f2803 int,
+f2804 int,
+f2805 int,
+f2806 int,
+f2807 int,
+f2808 int,
+f2809 int,
+f2810 int,
+f2811 int,
+f2812 int,
+f2813 int,
+f2814 int,
+f2815 int,
+f2816 int,
+f2817 int,
+f2818 int,
+f2819 int,
+f2820 int,
+f2821 int,
+f2822 int,
+f2823 int,
+f2824 int,
+f2825 int,
+f2826 int,
+f2827 int,
+f2828 int,
+f2829 int,
+f2830 int,
+f2831 int,
+f2832 int,
+f2833 int,
+f2834 int,
+f2835 int,
+f2836 int,
+f2837 int,
+f2838 int,
+f2839 int,
+f2840 int,
+f2841 int,
+f2842 int,
+f2843 int,
+f2844 int,
+f2845 int,
+f2846 int,
+f2847 int,
+f2848 int,
+f2849 int,
+f2850 int,
+f2851 int,
+f2852 int,
+f2853 int,
+f2854 int,
+f2855 int,
+f2856 int,
+f2857 int,
+f2858 int,
+f2859 int,
+f2860 int,
+f2861 int,
+f2862 int,
+f2863 int,
+f2864 int,
+f2865 int,
+f2866 int,
+f2867 int,
+f2868 int,
+f2869 int,
+f2870 int,
+f2871 int,
+f2872 int,
+f2873 int,
+f2874 int,
+f2875 int,
+f2876 int,
+f2877 int,
+f2878 int,
+f2879 int,
+f2880 int,
+f2881 int,
+f2882 int,
+f2883 int,
+f2884 int,
+f2885 int,
+f2886 int,
+f2887 int,
+f2888 int,
+f2889 int,
+f2890 int,
+f2891 int,
+f2892 int,
+f2893 int,
+f2894 int,
+f2895 int,
+f2896 int,
+f2897 int,
+f2898 int,
+f2899 int,
+f2900 int,
+f2901 int,
+f2902 int,
+f2903 int,
+f2904 int,
+f2905 int,
+f2906 int,
+f2907 int,
+f2908 int,
+f2909 int,
+f2910 int,
+f2911 int,
+f2912 int,
+f2913 int,
+f2914 int,
+f2915 int,
+f2916 int,
+f2917 int,
+f2918 int,
+f2919 int,
+f2920 int,
+f2921 int,
+f2922 int,
+f2923 int,
+f2924 int,
+f2925 int,
+f2926 int,
+f2927 int,
+f2928 int,
+f2929 int,
+f2930 int,
+f2931 int,
+f2932 int,
+f2933 int,
+f2934 int,
+f2935 int,
+f2936 int,
+f2937 int,
+f2938 int,
+f2939 int,
+f2940 int,
+f2941 int,
+f2942 int,
+f2943 int,
+f2944 int,
+f2945 int,
+f2946 int,
+f2947 int,
+f2948 int,
+f2949 int,
+f2950 int,
+f2951 int,
+f2952 int,
+f2953 int,
+f2954 int,
+f2955 int,
+f2956 int,
+f2957 int,
+f2958 int,
+f2959 int,
+f2960 int,
+f2961 int,
+f2962 int,
+f2963 int,
+f2964 int,
+f2965 int,
+f2966 int,
+f2967 int,
+f2968 int,
+f2969 int,
+f2970 int,
+f2971 int,
+f2972 int,
+f2973 int,
+f2974 int,
+f2975 int,
+f2976 int,
+f2977 int,
+f2978 int,
+f2979 int,
+f2980 int,
+f2981 int,
+f2982 int,
+f2983 int,
+f2984 int,
+f2985 int,
+f2986 int,
+f2987 int,
+f2988 int,
+f2989 int,
+f2990 int,
+f2991 int,
+f2992 int,
+f2993 int,
+f2994 int,
+f2995 int,
+f2996 int,
+f2997 int,
+f2998 int,
+f2999 int,
+f3000 int,
+f3001 int,
+f3002 int,
+f3003 int,
+f3004 int,
+f3005 int,
+f3006 int,
+f3007 int,
+f3008 int,
+f3009 int,
+f3010 int,
+f3011 int,
+f3012 int,
+f3013 int,
+f3014 int,
+f3015 int,
+f3016 int,
+f3017 int,
+f3018 int,
+f3019 int,
+f3020 int,
+f3021 int,
+f3022 int,
+f3023 int,
+f3024 int,
+f3025 int,
+f3026 int,
+f3027 int,
+f3028 int,
+f3029 int,
+f3030 int,
+f3031 int,
+f3032 int,
+f3033 int,
+f3034 int,
+f3035 int,
+f3036 int,
+f3037 int,
+f3038 int,
+f3039 int,
+f3040 int,
+f3041 int,
+f3042 int,
+f3043 int,
+f3044 int,
+f3045 int,
+f3046 int,
+f3047 int,
+f3048 int,
+f3049 int,
+f3050 int,
+f3051 int,
+f3052 int,
+f3053 int,
+f3054 int,
+f3055 int,
+f3056 int,
+f3057 int,
+f3058 int,
+f3059 int,
+f3060 int,
+f3061 int,
+f3062 int,
+f3063 int,
+f3064 int,
+f3065 int,
+f3066 int,
+f3067 int,
+f3068 int,
+f3069 int,
+f3070 int,
+f3071 int,
+f3072 int,
+f3073 int,
+f3074 int,
+f3075 int,
+f3076 int,
+f3077 int,
+f3078 int,
+f3079 int,
+f3080 int,
+f3081 int,
+f3082 int,
+f3083 int,
+f3084 int,
+f3085 int,
+f3086 int,
+f3087 int,
+f3088 int,
+f3089 int,
+f3090 int,
+f3091 int,
+f3092 int,
+f3093 int,
+f3094 int,
+f3095 int,
+f3096 int,
+f3097 int,
+f3098 int,
+f3099 int,
+f3100 int,
+f3101 int,
+f3102 int,
+f3103 int,
+f3104 int,
+f3105 int,
+f3106 int,
+f3107 int,
+f3108 int,
+f3109 int,
+f3110 int,
+f3111 int,
+f3112 int,
+f3113 int,
+f3114 int,
+f3115 int,
+f3116 int,
+f3117 int,
+f3118 int,
+f3119 int,
+f3120 int,
+f3121 int,
+f3122 int,
+f3123 int,
+f3124 int,
+f3125 int,
+f3126 int,
+f3127 int,
+f3128 int,
+f3129 int,
+f3130 int,
+f3131 int,
+f3132 int,
+f3133 int,
+f3134 int,
+f3135 int,
+f3136 int,
+f3137 int,
+f3138 int,
+f3139 int,
+f3140 int,
+f3141 int,
+f3142 int,
+f3143 int,
+f3144 int,
+f3145 int,
+f3146 int,
+f3147 int,
+f3148 int,
+f3149 int,
+f3150 int,
+f3151 int,
+f3152 int,
+f3153 int,
+f3154 int,
+f3155 int,
+f3156 int,
+f3157 int,
+f3158 int,
+f3159 int,
+f3160 int,
+f3161 int,
+f3162 int,
+f3163 int,
+f3164 int,
+f3165 int,
+f3166 int,
+f3167 int,
+f3168 int,
+f3169 int,
+f3170 int,
+f3171 int,
+f3172 int,
+f3173 int,
+f3174 int,
+f3175 int,
+f3176 int,
+f3177 int,
+f3178 int,
+f3179 int,
+f3180 int,
+f3181 int,
+f3182 int,
+f3183 int,
+f3184 int,
+f3185 int,
+f3186 int,
+f3187 int,
+f3188 int,
+f3189 int,
+f3190 int,
+f3191 int,
+f3192 int,
+f3193 int,
+f3194 int,
+f3195 int,
+f3196 int,
+f3197 int,
+f3198 int,
+f3199 int,
+f3200 int,
+f3201 int,
+f3202 int,
+f3203 int,
+f3204 int,
+f3205 int,
+f3206 int,
+f3207 int,
+f3208 int,
+f3209 int,
+f3210 int,
+f3211 int,
+f3212 int,
+f3213 int,
+f3214 int,
+f3215 int,
+f3216 int,
+f3217 int,
+f3218 int,
+f3219 int,
+f3220 int,
+f3221 int,
+f3222 int,
+f3223 int,
+f3224 int,
+f3225 int,
+f3226 int,
+f3227 int,
+f3228 int,
+f3229 int,
+f3230 int,
+f3231 int,
+f3232 int,
+f3233 int,
+f3234 int,
+f3235 int,
+f3236 int,
+f3237 int,
+f3238 int,
+f3239 int,
+f3240 int,
+f3241 int,
+f3242 int,
+f3243 int,
+f3244 int,
+f3245 int,
+f3246 int,
+f3247 int,
+f3248 int,
+f3249 int,
+f3250 int,
+f3251 int,
+f3252 int,
+f3253 int,
+f3254 int,
+f3255 int,
+f3256 int,
+f3257 int,
+f3258 int,
+f3259 int,
+f3260 int,
+f3261 int,
+f3262 int,
+f3263 int,
+f3264 int,
+f3265 int,
+f3266 int,
+f3267 int,
+f3268 int,
+f3269 int,
+f3270 int,
+f3271 int,
+f3272 int,
+f3273 int,
+f3274 int,
+f3275 int,
+f3276 int,
+f3277 int,
+f3278 int,
+f3279 int,
+f3280 int,
+f3281 int,
+f3282 int,
+f3283 int,
+f3284 int,
+f3285 int,
+f3286 int,
+f3287 int,
+f3288 int,
+f3289 int,
+f3290 int,
+f3291 int,
+f3292 int,
+f3293 int,
+f3294 int,
+f3295 int,
+f3296 int,
+f3297 int,
+f3298 int,
+f3299 int,
+f3300 int,
+f3301 int,
+f3302 int,
+f3303 int,
+f3304 int,
+f3305 int,
+f3306 int,
+f3307 int,
+f3308 int,
+f3309 int,
+f3310 int,
+f3311 int,
+f3312 int,
+f3313 int,
+f3314 int,
+f3315 int,
+f3316 int,
+f3317 int,
+f3318 int,
+f3319 int,
+f3320 int,
+f3321 int,
+f3322 int,
+f3323 int,
+f3324 int,
+f3325 int,
+f3326 int,
+f3327 int,
+f3328 int,
+f3329 int,
+f3330 int,
+f3331 int,
+f3332 int,
+f3333 int,
+f3334 int,
+f3335 int,
+f3336 int,
+f3337 int,
+f3338 int,
+f3339 int,
+f3340 int,
+f3341 int,
+f3342 int,
+f3343 int,
+f3344 int,
+f3345 int,
+f3346 int,
+f3347 int,
+f3348 int,
+f3349 int,
+f3350 int,
+f3351 int,
+f3352 int,
+f3353 int,
+f3354 int,
+f3355 int,
+f3356 int,
+f3357 int,
+f3358 int,
+f3359 int,
+f3360 int,
+f3361 int,
+f3362 int,
+f3363 int,
+f3364 int,
+f3365 int,
+f3366 int,
+f3367 int,
+f3368 int,
+f3369 int,
+f3370 int,
+f3371 int,
+f3372 int,
+f3373 int,
+f3374 int,
+f3375 int,
+f3376 int,
+f3377 int,
+f3378 int,
+f3379 int,
+f3380 int,
+f3381 int,
+f3382 int,
+f3383 int,
+f3384 int,
+f3385 int,
+f3386 int,
+f3387 int,
+f3388 int,
+f3389 int,
+f3390 int,
+f3391 int,
+f3392 int,
+f3393 int,
+f3394 int,
+f3395 int,
+f3396 int,
+f3397 int,
+f3398 int,
+f3399 int,
+f3400 int,
+f3401 int,
+f3402 int,
+f3403 int,
+f3404 int,
+f3405 int,
+f3406 int,
+f3407 int,
+f3408 int,
+f3409 int,
+f3410 int,
+f3411 int,
+f3412 int,
+f3413 int,
+f3414 int,
+f3415 int,
+f3416 int,
+f3417 int,
+f3418 int,
+f3419 int,
+f3420 int,
+f3421 int,
+f3422 int,
+f3423 int,
+f3424 int,
+f3425 int,
+f3426 int,
+f3427 int,
+f3428 int,
+f3429 int,
+f3430 int,
+f3431 int,
+f3432 int,
+f3433 int,
+f3434 int,
+f3435 int,
+f3436 int,
+f3437 int,
+f3438 int,
+f3439 int,
+f3440 int,
+f3441 int,
+f3442 int,
+f3443 int,
+f3444 int,
+f3445 int,
+f3446 int,
+f3447 int,
+f3448 int,
+f3449 int,
+f3450 int,
+f3451 int,
+f3452 int,
+f3453 int,
+f3454 int,
+f3455 int,
+f3456 int,
+f3457 int,
+f3458 int,
+f3459 int,
+f3460 int,
+f3461 int,
+f3462 int,
+f3463 int,
+f3464 int,
+f3465 int,
+f3466 int,
+f3467 int,
+f3468 int,
+f3469 int,
+f3470 int,
+f3471 int,
+f3472 int,
+f3473 int,
+f3474 int,
+f3475 int,
+f3476 int,
+f3477 int,
+f3478 int,
+f3479 int,
+f3480 int,
+f3481 int,
+f3482 int,
+f3483 int,
+f3484 int,
+f3485 int,
+f3486 int,
+f3487 int,
+f3488 int,
+f3489 int,
+f3490 int,
+f3491 int,
+f3492 int,
+f3493 int,
+f3494 int,
+f3495 int,
+f3496 int,
+f3497 int,
+f3498 int,
+f3499 int,
+f3500 int,
+f3501 int,
+f3502 int,
+f3503 int,
+f3504 int,
+f3505 int,
+f3506 int,
+f3507 int,
+f3508 int,
+f3509 int,
+f3510 int,
+f3511 int,
+f3512 int,
+f3513 int,
+f3514 int,
+f3515 int,
+f3516 int,
+f3517 int,
+f3518 int,
+f3519 int,
+f3520 int,
+f3521 int,
+f3522 int,
+f3523 int,
+f3524 int,
+f3525 int,
+f3526 int,
+f3527 int,
+f3528 int,
+f3529 int,
+f3530 int,
+f3531 int,
+f3532 int,
+f3533 int,
+f3534 int,
+f3535 int,
+f3536 int,
+f3537 int,
+f3538 int,
+f3539 int,
+f3540 int,
+f3541 int,
+f3542 int,
+f3543 int,
+f3544 int,
+f3545 int,
+f3546 int,
+f3547 int,
+f3548 int,
+f3549 int,
+f3550 int,
+f3551 int,
+f3552 int,
+f3553 int,
+f3554 int,
+f3555 int,
+f3556 int,
+f3557 int,
+f3558 int,
+f3559 int,
+f3560 int,
+f3561 int,
+f3562 int,
+f3563 int,
+f3564 int,
+f3565 int,
+f3566 int,
+f3567 int,
+f3568 int,
+f3569 int,
+f3570 int,
+f3571 int,
+f3572 int,
+f3573 int,
+f3574 int,
+f3575 int,
+f3576 int,
+f3577 int,
+f3578 int,
+f3579 int,
+f3580 int,
+f3581 int,
+f3582 int,
+f3583 int,
+f3584 int,
+f3585 int,
+f3586 int,
+f3587 int,
+f3588 int,
+f3589 int,
+f3590 int,
+f3591 int,
+f3592 int,
+f3593 int,
+f3594 int,
+f3595 int,
+f3596 int,
+f3597 int,
+f3598 int,
+f3599 int,
+f3600 int,
+f3601 int,
+f3602 int,
+f3603 int,
+f3604 int,
+f3605 int,
+f3606 int,
+f3607 int,
+f3608 int,
+f3609 int,
+f3610 int,
+f3611 int,
+f3612 int,
+f3613 int,
+f3614 int,
+f3615 int,
+f3616 int,
+f3617 int,
+f3618 int,
+f3619 int,
+f3620 int,
+f3621 int,
+f3622 int,
+f3623 int,
+f3624 int,
+f3625 int,
+f3626 int,
+f3627 int,
+f3628 int,
+f3629 int,
+f3630 int,
+f3631 int,
+f3632 int,
+f3633 int,
+f3634 int,
+f3635 int,
+f3636 int,
+f3637 int,
+f3638 int,
+f3639 int,
+f3640 int,
+f3641 int,
+f3642 int,
+f3643 int,
+f3644 int,
+f3645 int,
+f3646 int,
+f3647 int,
+f3648 int,
+f3649 int,
+f3650 int,
+f3651 int,
+f3652 int,
+f3653 int,
+f3654 int,
+f3655 int,
+f3656 int,
+f3657 int,
+f3658 int,
+f3659 int,
+f3660 int,
+f3661 int,
+f3662 int,
+f3663 int,
+f3664 int,
+f3665 int,
+f3666 int,
+f3667 int,
+f3668 int,
+f3669 int,
+f3670 int,
+f3671 int,
+f3672 int,
+f3673 int,
+f3674 int,
+f3675 int,
+f3676 int,
+f3677 int,
+f3678 int,
+f3679 int,
+f3680 int,
+f3681 int,
+f3682 int,
+f3683 int,
+f3684 int,
+f3685 int,
+f3686 int,
+f3687 int,
+f3688 int,
+f3689 int,
+f3690 int,
+f3691 int,
+f3692 int,
+f3693 int,
+f3694 int,
+f3695 int,
+f3696 int,
+f3697 int,
+f3698 int,
+f3699 int,
+f3700 int,
+f3701 int,
+f3702 int,
+f3703 int,
+f3704 int,
+f3705 int,
+f3706 int,
+f3707 int,
+f3708 int,
+f3709 int,
+f3710 int,
+f3711 int,
+f3712 int,
+f3713 int,
+f3714 int,
+f3715 int,
+f3716 int,
+f3717 int,
+f3718 int,
+f3719 int,
+f3720 int,
+f3721 int,
+f3722 int,
+f3723 int,
+f3724 int,
+f3725 int,
+f3726 int,
+f3727 int,
+f3728 int,
+f3729 int,
+f3730 int,
+f3731 int,
+f3732 int,
+f3733 int,
+f3734 int,
+f3735 int,
+f3736 int,
+f3737 int,
+f3738 int,
+f3739 int,
+f3740 int,
+f3741 int,
+f3742 int,
+f3743 int,
+f3744 int,
+f3745 int,
+f3746 int,
+f3747 int,
+f3748 int,
+f3749 int,
+f3750 int,
+f3751 int,
+f3752 int,
+f3753 int,
+f3754 int,
+f3755 int,
+f3756 int,
+f3757 int,
+f3758 int,
+f3759 int,
+f3760 int,
+f3761 int,
+f3762 int,
+f3763 int,
+f3764 int,
+f3765 int,
+f3766 int,
+f3767 int,
+f3768 int,
+f3769 int,
+f3770 int,
+f3771 int,
+f3772 int,
+f3773 int,
+f3774 int,
+f3775 int,
+f3776 int,
+f3777 int,
+f3778 int,
+f3779 int,
+f3780 int,
+f3781 int,
+f3782 int,
+f3783 int,
+f3784 int,
+f3785 int,
+f3786 int,
+f3787 int,
+f3788 int,
+f3789 int,
+f3790 int,
+f3791 int,
+f3792 int,
+f3793 int,
+f3794 int,
+f3795 int,
+f3796 int,
+f3797 int,
+f3798 int,
+f3799 int,
+f3800 int,
+f3801 int,
+f3802 int,
+f3803 int,
+f3804 int,
+f3805 int,
+f3806 int,
+f3807 int,
+f3808 int,
+f3809 int,
+f3810 int,
+f3811 int,
+f3812 int,
+f3813 int,
+f3814 int,
+f3815 int,
+f3816 int,
+f3817 int,
+f3818 int,
+f3819 int,
+f3820 int,
+f3821 int,
+f3822 int,
+f3823 int,
+f3824 int,
+f3825 int,
+f3826 int,
+f3827 int,
+f3828 int,
+f3829 int,
+f3830 int,
+f3831 int,
+f3832 int,
+f3833 int,
+f3834 int,
+f3835 int,
+f3836 int,
+f3837 int,
+f3838 int,
+f3839 int,
+f3840 int,
+f3841 int,
+f3842 int,
+f3843 int,
+f3844 int,
+f3845 int,
+f3846 int,
+f3847 int,
+f3848 int,
+f3849 int,
+f3850 int,
+f3851 int,
+f3852 int,
+f3853 int,
+f3854 int,
+f3855 int,
+f3856 int,
+f3857 int,
+f3858 int,
+f3859 int,
+f3860 int,
+f3861 int,
+f3862 int,
+f3863 int,
+f3864 int,
+f3865 int,
+f3866 int,
+f3867 int,
+f3868 int,
+f3869 int,
+f3870 int,
+f3871 int,
+f3872 int,
+f3873 int,
+f3874 int,
+f3875 int,
+f3876 int,
+f3877 int,
+f3878 int,
+f3879 int,
+f3880 int,
+f3881 int,
+f3882 int,
+f3883 int,
+f3884 int,
+f3885 int,
+f3886 int,
+f3887 int,
+f3888 int,
+f3889 int,
+f3890 int,
+f3891 int,
+f3892 int,
+f3893 int,
+f3894 int,
+f3895 int,
+f3896 int,
+f3897 int,
+f3898 int,
+f3899 int,
+f3900 int,
+f3901 int,
+f3902 int,
+f3903 int,
+f3904 int,
+f3905 int,
+f3906 int,
+f3907 int,
+f3908 int,
+f3909 int,
+f3910 int,
+f3911 int,
+f3912 int,
+f3913 int,
+f3914 int,
+f3915 int,
+f3916 int,
+f3917 int,
+f3918 int,
+f3919 int,
+f3920 int,
+f3921 int,
+f3922 int,
+f3923 int,
+f3924 int,
+f3925 int,
+f3926 int,
+f3927 int,
+f3928 int,
+f3929 int,
+f3930 int,
+f3931 int,
+f3932 int,
+f3933 int,
+f3934 int,
+f3935 int,
+f3936 int,
+f3937 int,
+f3938 int,
+f3939 int,
+f3940 int,
+f3941 int,
+f3942 int,
+f3943 int,
+f3944 int,
+f3945 int,
+f3946 int,
+f3947 int,
+f3948 int,
+f3949 int,
+f3950 int,
+f3951 int,
+f3952 int,
+f3953 int,
+f3954 int,
+f3955 int,
+f3956 int,
+f3957 int,
+f3958 int,
+f3959 int,
+f3960 int,
+f3961 int,
+f3962 int,
+f3963 int,
+f3964 int,
+f3965 int,
+f3966 int,
+f3967 int,
+f3968 int,
+f3969 int,
+f3970 int,
+f3971 int,
+f3972 int,
+f3973 int,
+f3974 int,
+f3975 int,
+f3976 int,
+f3977 int,
+f3978 int,
+f3979 int,
+f3980 int,
+f3981 int,
+f3982 int,
+f3983 int,
+f3984 int,
+f3985 int,
+f3986 int,
+f3987 int,
+f3988 int,
+f3989 int,
+f3990 int,
+f3991 int,
+f3992 int,
+f3993 int,
+f3994 int,
+f3995 int,
+f3996 int,
+f3997 int,
+f3998 int,
+f3999 int,
+f4000 int,
+f4001 int,
+f4002 int,
+f4003 int,
+f4004 int,
+f4005 int,
+f4006 int,
+f4007 int,
+f4008 int,
+f4009 int,
+f4010 int,
+f4011 int,
+f4012 int,
+f4013 int,
+f4014 int,
+f4015 int,
+f4016 int,
+f4017 int,
+f4018 int,
+f4019 int,
+f4020 int,
+f4021 int,
+f4022 int,
+f4023 int,
+f4024 int,
+f4025 int,
+f4026 int,
+f4027 int,
+f4028 int,
+f4029 int,
+f4030 int,
+f4031 int,
+f4032 int,
+f4033 int,
+f4034 int,
+f4035 int,
+f4036 int,
+f4037 int,
+f4038 int,
+f4039 int,
+f4040 int,
+f4041 int,
+f4042 int,
+f4043 int,
+f4044 int,
+f4045 int,
+f4046 int,
+f4047 int,
+f4048 int,
+f4049 int,
+f4050 int,
+f4051 int,
+f4052 int,
+f4053 int,
+f4054 int,
+f4055 int,
+f4056 int,
+f4057 int,
+f4058 int,
+f4059 int,
+f4060 int,
+f4061 int,
+f4062 int,
+f4063 int,
+f4064 int,
+f4065 int,
+f4066 int,
+f4067 int,
+f4068 int,
+f4069 int,
+f4070 int,
+f4071 int,
+f4072 int,
+f4073 int,
+f4074 int,
+f4075 int,
+f4076 int,
+f4077 int,
+f4078 int,
+f4079 int,
+f4080 int,
+f4081 int,
+f4082 int,
+f4083 int,
+f4084 int,
+f4085 int,
+f4086 int,
+f4087 int,
+f4088 int,
+f4089 int,
+f4090 int,
+f4091 int,
+f4092 int,
+f4093 int,
+f4094 int,
+f4095 int,
+f4096 int
+) ENGINE=BLACKHOLE;
+SHOW CREATE TABLE t1;
+DROP TABLE t1;
+--echo Now, MAX_FIELDS. This fails.
+--error 1117
+CREATE TABLE t1 (
+f0 int,
+f1 int,
+f2 int,
+f3 int,
+f4 int,
+f5 int,
+f6 int,
+f7 int,
+f8 int,
+f9 int,
+f10 int,
+f11 int,
+f12 int,
+f13 int,
+f14 int,
+f15 int,
+f16 int,
+f17 int,
+f18 int,
+f19 int,
+f20 int,
+f21 int,
+f22 int,
+f23 int,
+f24 int,
+f25 int,
+f26 int,
+f27 int,
+f28 int,
+f29 int,
+f30 int,
+f31 int,
+f32 int,
+f33 int,
+f34 int,
+f35 int,
+f36 int,
+f37 int,
+f38 int,
+f39 int,
+f40 int,
+f41 int,
+f42 int,
+f43 int,
+f44 int,
+f45 int,
+f46 int,
+f47 int,
+f48 int,
+f49 int,
+f50 int,
+f51 int,
+f52 int,
+f53 int,
+f54 int,
+f55 int,
+f56 int,
+f57 int,
+f58 int,
+f59 int,
+f60 int,
+f61 int,
+f62 int,
+f63 int,
+f64 int,
+f65 int,
+f66 int,
+f67 int,
+f68 int,
+f69 int,
+f70 int,
+f71 int,
+f72 int,
+f73 int,
+f74 int,
+f75 int,
+f76 int,
+f77 int,
+f78 int,
+f79 int,
+f80 int,
+f81 int,
+f82 int,
+f83 int,
+f84 int,
+f85 int,
+f86 int,
+f87 int,
+f88 int,
+f89 int,
+f90 int,
+f91 int,
+f92 int,
+f93 int,
+f94 int,
+f95 int,
+f96 int,
+f97 int,
+f98 int,
+f99 int,
+f100 int,
+f101 int,
+f102 int,
+f103 int,
+f104 int,
+f105 int,
+f106 int,
+f107 int,
+f108 int,
+f109 int,
+f110 int,
+f111 int,
+f112 int,
+f113 int,
+f114 int,
+f115 int,
+f116 int,
+f117 int,
+f118 int,
+f119 int,
+f120 int,
+f121 int,
+f122 int,
+f123 int,
+f124 int,
+f125 int,
+f126 int,
+f127 int,
+f128 int,
+f129 int,
+f130 int,
+f131 int,
+f132 int,
+f133 int,
+f134 int,
+f135 int,
+f136 int,
+f137 int,
+f138 int,
+f139 int,
+f140 int,
+f141 int,
+f142 int,
+f143 int,
+f144 int,
+f145 int,
+f146 int,
+f147 int,
+f148 int,
+f149 int,
+f150 int,
+f151 int,
+f152 int,
+f153 int,
+f154 int,
+f155 int,
+f156 int,
+f157 int,
+f158 int,
+f159 int,
+f160 int,
+f161 int,
+f162 int,
+f163 int,
+f164 int,
+f165 int,
+f166 int,
+f167 int,
+f168 int,
+f169 int,
+f170 int,
+f171 int,
+f172 int,
+f173 int,
+f174 int,
+f175 int,
+f176 int,
+f177 int,
+f178 int,
+f179 int,
+f180 int,
+f181 int,
+f182 int,
+f183 int,
+f184 int,
+f185 int,
+f186 int,
+f187 int,
+f188 int,
+f189 int,
+f190 int,
+f191 int,
+f192 int,
+f193 int,
+f194 int,
+f195 int,
+f196 int,
+f197 int,
+f198 int,
+f199 int,
+f200 int,
+f201 int,
+f202 int,
+f203 int,
+f204 int,
+f205 int,
+f206 int,
+f207 int,
+f208 int,
+f209 int,
+f210 int,
+f211 int,
+f212 int,
+f213 int,
+f214 int,
+f215 int,
+f216 int,
+f217 int,
+f218 int,
+f219 int,
+f220 int,
+f221 int,
+f222 int,
+f223 int,
+f224 int,
+f225 int,
+f226 int,
+f227 int,
+f228 int,
+f229 int,
+f230 int,
+f231 int,
+f232 int,
+f233 int,
+f234 int,
+f235 int,
+f236 int,
+f237 int,
+f238 int,
+f239 int,
+f240 int,
+f241 int,
+f242 int,
+f243 int,
+f244 int,
+f245 int,
+f246 int,
+f247 int,
+f248 int,
+f249 int,
+f250 int,
+f251 int,
+f252 int,
+f253 int,
+f254 int,
+f255 int,
+f256 int,
+f257 int,
+f258 int,
+f259 int,
+f260 int,
+f261 int,
+f262 int,
+f263 int,
+f264 int,
+f265 int,
+f266 int,
+f267 int,
+f268 int,
+f269 int,
+f270 int,
+f271 int,
+f272 int,
+f273 int,
+f274 int,
+f275 int,
+f276 int,
+f277 int,
+f278 int,
+f279 int,
+f280 int,
+f281 int,
+f282 int,
+f283 int,
+f284 int,
+f285 int,
+f286 int,
+f287 int,
+f288 int,
+f289 int,
+f290 int,
+f291 int,
+f292 int,
+f293 int,
+f294 int,
+f295 int,
+f296 int,
+f297 int,
+f298 int,
+f299 int,
+f300 int,
+f301 int,
+f302 int,
+f303 int,
+f304 int,
+f305 int,
+f306 int,
+f307 int,
+f308 int,
+f309 int,
+f310 int,
+f311 int,
+f312 int,
+f313 int,
+f314 int,
+f315 int,
+f316 int,
+f317 int,
+f318 int,
+f319 int,
+f320 int,
+f321 int,
+f322 int,
+f323 int,
+f324 int,
+f325 int,
+f326 int,
+f327 int,
+f328 int,
+f329 int,
+f330 int,
+f331 int,
+f332 int,
+f333 int,
+f334 int,
+f335 int,
+f336 int,
+f337 int,
+f338 int,
+f339 int,
+f340 int,
+f341 int,
+f342 int,
+f343 int,
+f344 int,
+f345 int,
+f346 int,
+f347 int,
+f348 int,
+f349 int,
+f350 int,
+f351 int,
+f352 int,
+f353 int,
+f354 int,
+f355 int,
+f356 int,
+f357 int,
+f358 int,
+f359 int,
+f360 int,
+f361 int,
+f362 int,
+f363 int,
+f364 int,
+f365 int,
+f366 int,
+f367 int,
+f368 int,
+f369 int,
+f370 int,
+f371 int,
+f372 int,
+f373 int,
+f374 int,
+f375 int,
+f376 int,
+f377 int,
+f378 int,
+f379 int,
+f380 int,
+f381 int,
+f382 int,
+f383 int,
+f384 int,
+f385 int,
+f386 int,
+f387 int,
+f388 int,
+f389 int,
+f390 int,
+f391 int,
+f392 int,
+f393 int,
+f394 int,
+f395 int,
+f396 int,
+f397 int,
+f398 int,
+f399 int,
+f400 int,
+f401 int,
+f402 int,
+f403 int,
+f404 int,
+f405 int,
+f406 int,
+f407 int,
+f408 int,
+f409 int,
+f410 int,
+f411 int,
+f412 int,
+f413 int,
+f414 int,
+f415 int,
+f416 int,
+f417 int,
+f418 int,
+f419 int,
+f420 int,
+f421 int,
+f422 int,
+f423 int,
+f424 int,
+f425 int,
+f426 int,
+f427 int,
+f428 int,
+f429 int,
+f430 int,
+f431 int,
+f432 int,
+f433 int,
+f434 int,
+f435 int,
+f436 int,
+f437 int,
+f438 int,
+f439 int,
+f440 int,
+f441 int,
+f442 int,
+f443 int,
+f444 int,
+f445 int,
+f446 int,
+f447 int,
+f448 int,
+f449 int,
+f450 int,
+f451 int,
+f452 int,
+f453 int,
+f454 int,
+f455 int,
+f456 int,
+f457 int,
+f458 int,
+f459 int,
+f460 int,
+f461 int,
+f462 int,
+f463 int,
+f464 int,
+f465 int,
+f466 int,
+f467 int,
+f468 int,
+f469 int,
+f470 int,
+f471 int,
+f472 int,
+f473 int,
+f474 int,
+f475 int,
+f476 int,
+f477 int,
+f478 int,
+f479 int,
+f480 int,
+f481 int,
+f482 int,
+f483 int,
+f484 int,
+f485 int,
+f486 int,
+f487 int,
+f488 int,
+f489 int,
+f490 int,
+f491 int,
+f492 int,
+f493 int,
+f494 int,
+f495 int,
+f496 int,
+f497 int,
+f498 int,
+f499 int,
+f500 int,
+f501 int,
+f502 int,
+f503 int,
+f504 int,
+f505 int,
+f506 int,
+f507 int,
+f508 int,
+f509 int,
+f510 int,
+f511 int,
+f512 int,
+f513 int,
+f514 int,
+f515 int,
+f516 int,
+f517 int,
+f518 int,
+f519 int,
+f520 int,
+f521 int,
+f522 int,
+f523 int,
+f524 int,
+f525 int,
+f526 int,
+f527 int,
+f528 int,
+f529 int,
+f530 int,
+f531 int,
+f532 int,
+f533 int,
+f534 int,
+f535 int,
+f536 int,
+f537 int,
+f538 int,
+f539 int,
+f540 int,
+f541 int,
+f542 int,
+f543 int,
+f544 int,
+f545 int,
+f546 int,
+f547 int,
+f548 int,
+f549 int,
+f550 int,
+f551 int,
+f552 int,
+f553 int,
+f554 int,
+f555 int,
+f556 int,
+f557 int,
+f558 int,
+f559 int,
+f560 int,
+f561 int,
+f562 int,
+f563 int,
+f564 int,
+f565 int,
+f566 int,
+f567 int,
+f568 int,
+f569 int,
+f570 int,
+f571 int,
+f572 int,
+f573 int,
+f574 int,
+f575 int,
+f576 int,
+f577 int,
+f578 int,
+f579 int,
+f580 int,
+f581 int,
+f582 int,
+f583 int,
+f584 int,
+f585 int,
+f586 int,
+f587 int,
+f588 int,
+f589 int,
+f590 int,
+f591 int,
+f592 int,
+f593 int,
+f594 int,
+f595 int,
+f596 int,
+f597 int,
+f598 int,
+f599 int,
+f600 int,
+f601 int,
+f602 int,
+f603 int,
+f604 int,
+f605 int,
+f606 int,
+f607 int,
+f608 int,
+f609 int,
+f610 int,
+f611 int,
+f612 int,
+f613 int,
+f614 int,
+f615 int,
+f616 int,
+f617 int,
+f618 int,
+f619 int,
+f620 int,
+f621 int,
+f622 int,
+f623 int,
+f624 int,
+f625 int,
+f626 int,
+f627 int,
+f628 int,
+f629 int,
+f630 int,
+f631 int,
+f632 int,
+f633 int,
+f634 int,
+f635 int,
+f636 int,
+f637 int,
+f638 int,
+f639 int,
+f640 int,
+f641 int,
+f642 int,
+f643 int,
+f644 int,
+f645 int,
+f646 int,
+f647 int,
+f648 int,
+f649 int,
+f650 int,
+f651 int,
+f652 int,
+f653 int,
+f654 int,
+f655 int,
+f656 int,
+f657 int,
+f658 int,
+f659 int,
+f660 int,
+f661 int,
+f662 int,
+f663 int,
+f664 int,
+f665 int,
+f666 int,
+f667 int,
+f668 int,
+f669 int,
+f670 int,
+f671 int,
+f672 int,
+f673 int,
+f674 int,
+f675 int,
+f676 int,
+f677 int,
+f678 int,
+f679 int,
+f680 int,
+f681 int,
+f682 int,
+f683 int,
+f684 int,
+f685 int,
+f686 int,
+f687 int,
+f688 int,
+f689 int,
+f690 int,
+f691 int,
+f692 int,
+f693 int,
+f694 int,
+f695 int,
+f696 int,
+f697 int,
+f698 int,
+f699 int,
+f700 int,
+f701 int,
+f702 int,
+f703 int,
+f704 int,
+f705 int,
+f706 int,
+f707 int,
+f708 int,
+f709 int,
+f710 int,
+f711 int,
+f712 int,
+f713 int,
+f714 int,
+f715 int,
+f716 int,
+f717 int,
+f718 int,
+f719 int,
+f720 int,
+f721 int,
+f722 int,
+f723 int,
+f724 int,
+f725 int,
+f726 int,
+f727 int,
+f728 int,
+f729 int,
+f730 int,
+f731 int,
+f732 int,
+f733 int,
+f734 int,
+f735 int,
+f736 int,
+f737 int,
+f738 int,
+f739 int,
+f740 int,
+f741 int,
+f742 int,
+f743 int,
+f744 int,
+f745 int,
+f746 int,
+f747 int,
+f748 int,
+f749 int,
+f750 int,
+f751 int,
+f752 int,
+f753 int,
+f754 int,
+f755 int,
+f756 int,
+f757 int,
+f758 int,
+f759 int,
+f760 int,
+f761 int,
+f762 int,
+f763 int,
+f764 int,
+f765 int,
+f766 int,
+f767 int,
+f768 int,
+f769 int,
+f770 int,
+f771 int,
+f772 int,
+f773 int,
+f774 int,
+f775 int,
+f776 int,
+f777 int,
+f778 int,
+f779 int,
+f780 int,
+f781 int,
+f782 int,
+f783 int,
+f784 int,
+f785 int,
+f786 int,
+f787 int,
+f788 int,
+f789 int,
+f790 int,
+f791 int,
+f792 int,
+f793 int,
+f794 int,
+f795 int,
+f796 int,
+f797 int,
+f798 int,
+f799 int,
+f800 int,
+f801 int,
+f802 int,
+f803 int,
+f804 int,
+f805 int,
+f806 int,
+f807 int,
+f808 int,
+f809 int,
+f810 int,
+f811 int,
+f812 int,
+f813 int,
+f814 int,
+f815 int,
+f816 int,
+f817 int,
+f818 int,
+f819 int,
+f820 int,
+f821 int,
+f822 int,
+f823 int,
+f824 int,
+f825 int,
+f826 int,
+f827 int,
+f828 int,
+f829 int,
+f830 int,
+f831 int,
+f832 int,
+f833 int,
+f834 int,
+f835 int,
+f836 int,
+f837 int,
+f838 int,
+f839 int,
+f840 int,
+f841 int,
+f842 int,
+f843 int,
+f844 int,
+f845 int,
+f846 int,
+f847 int,
+f848 int,
+f849 int,
+f850 int,
+f851 int,
+f852 int,
+f853 int,
+f854 int,
+f855 int,
+f856 int,
+f857 int,
+f858 int,
+f859 int,
+f860 int,
+f861 int,
+f862 int,
+f863 int,
+f864 int,
+f865 int,
+f866 int,
+f867 int,
+f868 int,
+f869 int,
+f870 int,
+f871 int,
+f872 int,
+f873 int,
+f874 int,
+f875 int,
+f876 int,
+f877 int,
+f878 int,
+f879 int,
+f880 int,
+f881 int,
+f882 int,
+f883 int,
+f884 int,
+f885 int,
+f886 int,
+f887 int,
+f888 int,
+f889 int,
+f890 int,
+f891 int,
+f892 int,
+f893 int,
+f894 int,
+f895 int,
+f896 int,
+f897 int,
+f898 int,
+f899 int,
+f900 int,
+f901 int,
+f902 int,
+f903 int,
+f904 int,
+f905 int,
+f906 int,
+f907 int,
+f908 int,
+f909 int,
+f910 int,
+f911 int,
+f912 int,
+f913 int,
+f914 int,
+f915 int,
+f916 int,
+f917 int,
+f918 int,
+f919 int,
+f920 int,
+f921 int,
+f922 int,
+f923 int,
+f924 int,
+f925 int,
+f926 int,
+f927 int,
+f928 int,
+f929 int,
+f930 int,
+f931 int,
+f932 int,
+f933 int,
+f934 int,
+f935 int,
+f936 int,
+f937 int,
+f938 int,
+f939 int,
+f940 int,
+f941 int,
+f942 int,
+f943 int,
+f944 int,
+f945 int,
+f946 int,
+f947 int,
+f948 int,
+f949 int,
+f950 int,
+f951 int,
+f952 int,
+f953 int,
+f954 int,
+f955 int,
+f956 int,
+f957 int,
+f958 int,
+f959 int,
+f960 int,
+f961 int,
+f962 int,
+f963 int,
+f964 int,
+f965 int,
+f966 int,
+f967 int,
+f968 int,
+f969 int,
+f970 int,
+f971 int,
+f972 int,
+f973 int,
+f974 int,
+f975 int,
+f976 int,
+f977 int,
+f978 int,
+f979 int,
+f980 int,
+f981 int,
+f982 int,
+f983 int,
+f984 int,
+f985 int,
+f986 int,
+f987 int,
+f988 int,
+f989 int,
+f990 int,
+f991 int,
+f992 int,
+f993 int,
+f994 int,
+f995 int,
+f996 int,
+f997 int,
+f998 int,
+f999 int,
+f1000 int,
+f1001 int,
+f1002 int,
+f1003 int,
+f1004 int,
+f1005 int,
+f1006 int,
+f1007 int,
+f1008 int,
+f1009 int,
+f1010 int,
+f1011 int,
+f1012 int,
+f1013 int,
+f1014 int,
+f1015 int,
+f1016 int,
+f1017 int,
+f1018 int,
+f1019 int,
+f1020 int,
+f1021 int,
+f1022 int,
+f1023 int,
+f1024 int,
+f1025 int,
+f1026 int,
+f1027 int,
+f1028 int,
+f1029 int,
+f1030 int,
+f1031 int,
+f1032 int,
+f1033 int,
+f1034 int,
+f1035 int,
+f1036 int,
+f1037 int,
+f1038 int,
+f1039 int,
+f1040 int,
+f1041 int,
+f1042 int,
+f1043 int,
+f1044 int,
+f1045 int,
+f1046 int,
+f1047 int,
+f1048 int,
+f1049 int,
+f1050 int,
+f1051 int,
+f1052 int,
+f1053 int,
+f1054 int,
+f1055 int,
+f1056 int,
+f1057 int,
+f1058 int,
+f1059 int,
+f1060 int,
+f1061 int,
+f1062 int,
+f1063 int,
+f1064 int,
+f1065 int,
+f1066 int,
+f1067 int,
+f1068 int,
+f1069 int,
+f1070 int,
+f1071 int,
+f1072 int,
+f1073 int,
+f1074 int,
+f1075 int,
+f1076 int,
+f1077 int,
+f1078 int,
+f1079 int,
+f1080 int,
+f1081 int,
+f1082 int,
+f1083 int,
+f1084 int,
+f1085 int,
+f1086 int,
+f1087 int,
+f1088 int,
+f1089 int,
+f1090 int,
+f1091 int,
+f1092 int,
+f1093 int,
+f1094 int,
+f1095 int,
+f1096 int,
+f1097 int,
+f1098 int,
+f1099 int,
+f1100 int,
+f1101 int,
+f1102 int,
+f1103 int,
+f1104 int,
+f1105 int,
+f1106 int,
+f1107 int,
+f1108 int,
+f1109 int,
+f1110 int,
+f1111 int,
+f1112 int,
+f1113 int,
+f1114 int,
+f1115 int,
+f1116 int,
+f1117 int,
+f1118 int,
+f1119 int,
+f1120 int,
+f1121 int,
+f1122 int,
+f1123 int,
+f1124 int,
+f1125 int,
+f1126 int,
+f1127 int,
+f1128 int,
+f1129 int,
+f1130 int,
+f1131 int,
+f1132 int,
+f1133 int,
+f1134 int,
+f1135 int,
+f1136 int,
+f1137 int,
+f1138 int,
+f1139 int,
+f1140 int,
+f1141 int,
+f1142 int,
+f1143 int,
+f1144 int,
+f1145 int,
+f1146 int,
+f1147 int,
+f1148 int,
+f1149 int,
+f1150 int,
+f1151 int,
+f1152 int,
+f1153 int,
+f1154 int,
+f1155 int,
+f1156 int,
+f1157 int,
+f1158 int,
+f1159 int,
+f1160 int,
+f1161 int,
+f1162 int,
+f1163 int,
+f1164 int,
+f1165 int,
+f1166 int,
+f1167 int,
+f1168 int,
+f1169 int,
+f1170 int,
+f1171 int,
+f1172 int,
+f1173 int,
+f1174 int,
+f1175 int,
+f1176 int,
+f1177 int,
+f1178 int,
+f1179 int,
+f1180 int,
+f1181 int,
+f1182 int,
+f1183 int,
+f1184 int,
+f1185 int,
+f1186 int,
+f1187 int,
+f1188 int,
+f1189 int,
+f1190 int,
+f1191 int,
+f1192 int,
+f1193 int,
+f1194 int,
+f1195 int,
+f1196 int,
+f1197 int,
+f1198 int,
+f1199 int,
+f1200 int,
+f1201 int,
+f1202 int,
+f1203 int,
+f1204 int,
+f1205 int,
+f1206 int,
+f1207 int,
+f1208 int,
+f1209 int,
+f1210 int,
+f1211 int,
+f1212 int,
+f1213 int,
+f1214 int,
+f1215 int,
+f1216 int,
+f1217 int,
+f1218 int,
+f1219 int,
+f1220 int,
+f1221 int,
+f1222 int,
+f1223 int,
+f1224 int,
+f1225 int,
+f1226 int,
+f1227 int,
+f1228 int,
+f1229 int,
+f1230 int,
+f1231 int,
+f1232 int,
+f1233 int,
+f1234 int,
+f1235 int,
+f1236 int,
+f1237 int,
+f1238 int,
+f1239 int,
+f1240 int,
+f1241 int,
+f1242 int,
+f1243 int,
+f1244 int,
+f1245 int,
+f1246 int,
+f1247 int,
+f1248 int,
+f1249 int,
+f1250 int,
+f1251 int,
+f1252 int,
+f1253 int,
+f1254 int,
+f1255 int,
+f1256 int,
+f1257 int,
+f1258 int,
+f1259 int,
+f1260 int,
+f1261 int,
+f1262 int,
+f1263 int,
+f1264 int,
+f1265 int,
+f1266 int,
+f1267 int,
+f1268 int,
+f1269 int,
+f1270 int,
+f1271 int,
+f1272 int,
+f1273 int,
+f1274 int,
+f1275 int,
+f1276 int,
+f1277 int,
+f1278 int,
+f1279 int,
+f1280 int,
+f1281 int,
+f1282 int,
+f1283 int,
+f1284 int,
+f1285 int,
+f1286 int,
+f1287 int,
+f1288 int,
+f1289 int,
+f1290 int,
+f1291 int,
+f1292 int,
+f1293 int,
+f1294 int,
+f1295 int,
+f1296 int,
+f1297 int,
+f1298 int,
+f1299 int,
+f1300 int,
+f1301 int,
+f1302 int,
+f1303 int,
+f1304 int,
+f1305 int,
+f1306 int,
+f1307 int,
+f1308 int,
+f1309 int,
+f1310 int,
+f1311 int,
+f1312 int,
+f1313 int,
+f1314 int,
+f1315 int,
+f1316 int,
+f1317 int,
+f1318 int,
+f1319 int,
+f1320 int,
+f1321 int,
+f1322 int,
+f1323 int,
+f1324 int,
+f1325 int,
+f1326 int,
+f1327 int,
+f1328 int,
+f1329 int,
+f1330 int,
+f1331 int,
+f1332 int,
+f1333 int,
+f1334 int,
+f1335 int,
+f1336 int,
+f1337 int,
+f1338 int,
+f1339 int,
+f1340 int,
+f1341 int,
+f1342 int,
+f1343 int,
+f1344 int,
+f1345 int,
+f1346 int,
+f1347 int,
+f1348 int,
+f1349 int,
+f1350 int,
+f1351 int,
+f1352 int,
+f1353 int,
+f1354 int,
+f1355 int,
+f1356 int,
+f1357 int,
+f1358 int,
+f1359 int,
+f1360 int,
+f1361 int,
+f1362 int,
+f1363 int,
+f1364 int,
+f1365 int,
+f1366 int,
+f1367 int,
+f1368 int,
+f1369 int,
+f1370 int,
+f1371 int,
+f1372 int,
+f1373 int,
+f1374 int,
+f1375 int,
+f1376 int,
+f1377 int,
+f1378 int,
+f1379 int,
+f1380 int,
+f1381 int,
+f1382 int,
+f1383 int,
+f1384 int,
+f1385 int,
+f1386 int,
+f1387 int,
+f1388 int,
+f1389 int,
+f1390 int,
+f1391 int,
+f1392 int,
+f1393 int,
+f1394 int,
+f1395 int,
+f1396 int,
+f1397 int,
+f1398 int,
+f1399 int,
+f1400 int,
+f1401 int,
+f1402 int,
+f1403 int,
+f1404 int,
+f1405 int,
+f1406 int,
+f1407 int,
+f1408 int,
+f1409 int,
+f1410 int,
+f1411 int,
+f1412 int,
+f1413 int,
+f1414 int,
+f1415 int,
+f1416 int,
+f1417 int,
+f1418 int,
+f1419 int,
+f1420 int,
+f1421 int,
+f1422 int,
+f1423 int,
+f1424 int,
+f1425 int,
+f1426 int,
+f1427 int,
+f1428 int,
+f1429 int,
+f1430 int,
+f1431 int,
+f1432 int,
+f1433 int,
+f1434 int,
+f1435 int,
+f1436 int,
+f1437 int,
+f1438 int,
+f1439 int,
+f1440 int,
+f1441 int,
+f1442 int,
+f1443 int,
+f1444 int,
+f1445 int,
+f1446 int,
+f1447 int,
+f1448 int,
+f1449 int,
+f1450 int,
+f1451 int,
+f1452 int,
+f1453 int,
+f1454 int,
+f1455 int,
+f1456 int,
+f1457 int,
+f1458 int,
+f1459 int,
+f1460 int,
+f1461 int,
+f1462 int,
+f1463 int,
+f1464 int,
+f1465 int,
+f1466 int,
+f1467 int,
+f1468 int,
+f1469 int,
+f1470 int,
+f1471 int,
+f1472 int,
+f1473 int,
+f1474 int,
+f1475 int,
+f1476 int,
+f1477 int,
+f1478 int,
+f1479 int,
+f1480 int,
+f1481 int,
+f1482 int,
+f1483 int,
+f1484 int,
+f1485 int,
+f1486 int,
+f1487 int,
+f1488 int,
+f1489 int,
+f1490 int,
+f1491 int,
+f1492 int,
+f1493 int,
+f1494 int,
+f1495 int,
+f1496 int,
+f1497 int,
+f1498 int,
+f1499 int,
+f1500 int,
+f1501 int,
+f1502 int,
+f1503 int,
+f1504 int,
+f1505 int,
+f1506 int,
+f1507 int,
+f1508 int,
+f1509 int,
+f1510 int,
+f1511 int,
+f1512 int,
+f1513 int,
+f1514 int,
+f1515 int,
+f1516 int,
+f1517 int,
+f1518 int,
+f1519 int,
+f1520 int,
+f1521 int,
+f1522 int,
+f1523 int,
+f1524 int,
+f1525 int,
+f1526 int,
+f1527 int,
+f1528 int,
+f1529 int,
+f1530 int,
+f1531 int,
+f1532 int,
+f1533 int,
+f1534 int,
+f1535 int,
+f1536 int,
+f1537 int,
+f1538 int,
+f1539 int,
+f1540 int,
+f1541 int,
+f1542 int,
+f1543 int,
+f1544 int,
+f1545 int,
+f1546 int,
+f1547 int,
+f1548 int,
+f1549 int,
+f1550 int,
+f1551 int,
+f1552 int,
+f1553 int,
+f1554 int,
+f1555 int,
+f1556 int,
+f1557 int,
+f1558 int,
+f1559 int,
+f1560 int,
+f1561 int,
+f1562 int,
+f1563 int,
+f1564 int,
+f1565 int,
+f1566 int,
+f1567 int,
+f1568 int,
+f1569 int,
+f1570 int,
+f1571 int,
+f1572 int,
+f1573 int,
+f1574 int,
+f1575 int,
+f1576 int,
+f1577 int,
+f1578 int,
+f1579 int,
+f1580 int,
+f1581 int,
+f1582 int,
+f1583 int,
+f1584 int,
+f1585 int,
+f1586 int,
+f1587 int,
+f1588 int,
+f1589 int,
+f1590 int,
+f1591 int,
+f1592 int,
+f1593 int,
+f1594 int,
+f1595 int,
+f1596 int,
+f1597 int,
+f1598 int,
+f1599 int,
+f1600 int,
+f1601 int,
+f1602 int,
+f1603 int,
+f1604 int,
+f1605 int,
+f1606 int,
+f1607 int,
+f1608 int,
+f1609 int,
+f1610 int,
+f1611 int,
+f1612 int,
+f1613 int,
+f1614 int,
+f1615 int,
+f1616 int,
+f1617 int,
+f1618 int,
+f1619 int,
+f1620 int,
+f1621 int,
+f1622 int,
+f1623 int,
+f1624 int,
+f1625 int,
+f1626 int,
+f1627 int,
+f1628 int,
+f1629 int,
+f1630 int,
+f1631 int,
+f1632 int,
+f1633 int,
+f1634 int,
+f1635 int,
+f1636 int,
+f1637 int,
+f1638 int,
+f1639 int,
+f1640 int,
+f1641 int,
+f1642 int,
+f1643 int,
+f1644 int,
+f1645 int,
+f1646 int,
+f1647 int,
+f1648 int,
+f1649 int,
+f1650 int,
+f1651 int,
+f1652 int,
+f1653 int,
+f1654 int,
+f1655 int,
+f1656 int,
+f1657 int,
+f1658 int,
+f1659 int,
+f1660 int,
+f1661 int,
+f1662 int,
+f1663 int,
+f1664 int,
+f1665 int,
+f1666 int,
+f1667 int,
+f1668 int,
+f1669 int,
+f1670 int,
+f1671 int,
+f1672 int,
+f1673 int,
+f1674 int,
+f1675 int,
+f1676 int,
+f1677 int,
+f1678 int,
+f1679 int,
+f1680 int,
+f1681 int,
+f1682 int,
+f1683 int,
+f1684 int,
+f1685 int,
+f1686 int,
+f1687 int,
+f1688 int,
+f1689 int,
+f1690 int,
+f1691 int,
+f1692 int,
+f1693 int,
+f1694 int,
+f1695 int,
+f1696 int,
+f1697 int,
+f1698 int,
+f1699 int,
+f1700 int,
+f1701 int,
+f1702 int,
+f1703 int,
+f1704 int,
+f1705 int,
+f1706 int,
+f1707 int,
+f1708 int,
+f1709 int,
+f1710 int,
+f1711 int,
+f1712 int,
+f1713 int,
+f1714 int,
+f1715 int,
+f1716 int,
+f1717 int,
+f1718 int,
+f1719 int,
+f1720 int,
+f1721 int,
+f1722 int,
+f1723 int,
+f1724 int,
+f1725 int,
+f1726 int,
+f1727 int,
+f1728 int,
+f1729 int,
+f1730 int,
+f1731 int,
+f1732 int,
+f1733 int,
+f1734 int,
+f1735 int,
+f1736 int,
+f1737 int,
+f1738 int,
+f1739 int,
+f1740 int,
+f1741 int,
+f1742 int,
+f1743 int,
+f1744 int,
+f1745 int,
+f1746 int,
+f1747 int,
+f1748 int,
+f1749 int,
+f1750 int,
+f1751 int,
+f1752 int,
+f1753 int,
+f1754 int,
+f1755 int,
+f1756 int,
+f1757 int,
+f1758 int,
+f1759 int,
+f1760 int,
+f1761 int,
+f1762 int,
+f1763 int,
+f1764 int,
+f1765 int,
+f1766 int,
+f1767 int,
+f1768 int,
+f1769 int,
+f1770 int,
+f1771 int,
+f1772 int,
+f1773 int,
+f1774 int,
+f1775 int,
+f1776 int,
+f1777 int,
+f1778 int,
+f1779 int,
+f1780 int,
+f1781 int,
+f1782 int,
+f1783 int,
+f1784 int,
+f1785 int,
+f1786 int,
+f1787 int,
+f1788 int,
+f1789 int,
+f1790 int,
+f1791 int,
+f1792 int,
+f1793 int,
+f1794 int,
+f1795 int,
+f1796 int,
+f1797 int,
+f1798 int,
+f1799 int,
+f1800 int,
+f1801 int,
+f1802 int,
+f1803 int,
+f1804 int,
+f1805 int,
+f1806 int,
+f1807 int,
+f1808 int,
+f1809 int,
+f1810 int,
+f1811 int,
+f1812 int,
+f1813 int,
+f1814 int,
+f1815 int,
+f1816 int,
+f1817 int,
+f1818 int,
+f1819 int,
+f1820 int,
+f1821 int,
+f1822 int,
+f1823 int,
+f1824 int,
+f1825 int,
+f1826 int,
+f1827 int,
+f1828 int,
+f1829 int,
+f1830 int,
+f1831 int,
+f1832 int,
+f1833 int,
+f1834 int,
+f1835 int,
+f1836 int,
+f1837 int,
+f1838 int,
+f1839 int,
+f1840 int,
+f1841 int,
+f1842 int,
+f1843 int,
+f1844 int,
+f1845 int,
+f1846 int,
+f1847 int,
+f1848 int,
+f1849 int,
+f1850 int,
+f1851 int,
+f1852 int,
+f1853 int,
+f1854 int,
+f1855 int,
+f1856 int,
+f1857 int,
+f1858 int,
+f1859 int,
+f1860 int,
+f1861 int,
+f1862 int,
+f1863 int,
+f1864 int,
+f1865 int,
+f1866 int,
+f1867 int,
+f1868 int,
+f1869 int,
+f1870 int,
+f1871 int,
+f1872 int,
+f1873 int,
+f1874 int,
+f1875 int,
+f1876 int,
+f1877 int,
+f1878 int,
+f1879 int,
+f1880 int,
+f1881 int,
+f1882 int,
+f1883 int,
+f1884 int,
+f1885 int,
+f1886 int,
+f1887 int,
+f1888 int,
+f1889 int,
+f1890 int,
+f1891 int,
+f1892 int,
+f1893 int,
+f1894 int,
+f1895 int,
+f1896 int,
+f1897 int,
+f1898 int,
+f1899 int,
+f1900 int,
+f1901 int,
+f1902 int,
+f1903 int,
+f1904 int,
+f1905 int,
+f1906 int,
+f1907 int,
+f1908 int,
+f1909 int,
+f1910 int,
+f1911 int,
+f1912 int,
+f1913 int,
+f1914 int,
+f1915 int,
+f1916 int,
+f1917 int,
+f1918 int,
+f1919 int,
+f1920 int,
+f1921 int,
+f1922 int,
+f1923 int,
+f1924 int,
+f1925 int,
+f1926 int,
+f1927 int,
+f1928 int,
+f1929 int,
+f1930 int,
+f1931 int,
+f1932 int,
+f1933 int,
+f1934 int,
+f1935 int,
+f1936 int,
+f1937 int,
+f1938 int,
+f1939 int,
+f1940 int,
+f1941 int,
+f1942 int,
+f1943 int,
+f1944 int,
+f1945 int,
+f1946 int,
+f1947 int,
+f1948 int,
+f1949 int,
+f1950 int,
+f1951 int,
+f1952 int,
+f1953 int,
+f1954 int,
+f1955 int,
+f1956 int,
+f1957 int,
+f1958 int,
+f1959 int,
+f1960 int,
+f1961 int,
+f1962 int,
+f1963 int,
+f1964 int,
+f1965 int,
+f1966 int,
+f1967 int,
+f1968 int,
+f1969 int,
+f1970 int,
+f1971 int,
+f1972 int,
+f1973 int,
+f1974 int,
+f1975 int,
+f1976 int,
+f1977 int,
+f1978 int,
+f1979 int,
+f1980 int,
+f1981 int,
+f1982 int,
+f1983 int,
+f1984 int,
+f1985 int,
+f1986 int,
+f1987 int,
+f1988 int,
+f1989 int,
+f1990 int,
+f1991 int,
+f1992 int,
+f1993 int,
+f1994 int,
+f1995 int,
+f1996 int,
+f1997 int,
+f1998 int,
+f1999 int,
+f2000 int,
+f2001 int,
+f2002 int,
+f2003 int,
+f2004 int,
+f2005 int,
+f2006 int,
+f2007 int,
+f2008 int,
+f2009 int,
+f2010 int,
+f2011 int,
+f2012 int,
+f2013 int,
+f2014 int,
+f2015 int,
+f2016 int,
+f2017 int,
+f2018 int,
+f2019 int,
+f2020 int,
+f2021 int,
+f2022 int,
+f2023 int,
+f2024 int,
+f2025 int,
+f2026 int,
+f2027 int,
+f2028 int,
+f2029 int,
+f2030 int,
+f2031 int,
+f2032 int,
+f2033 int,
+f2034 int,
+f2035 int,
+f2036 int,
+f2037 int,
+f2038 int,
+f2039 int,
+f2040 int,
+f2041 int,
+f2042 int,
+f2043 int,
+f2044 int,
+f2045 int,
+f2046 int,
+f2047 int,
+f2048 int,
+f2049 int,
+f2050 int,
+f2051 int,
+f2052 int,
+f2053 int,
+f2054 int,
+f2055 int,
+f2056 int,
+f2057 int,
+f2058 int,
+f2059 int,
+f2060 int,
+f2061 int,
+f2062 int,
+f2063 int,
+f2064 int,
+f2065 int,
+f2066 int,
+f2067 int,
+f2068 int,
+f2069 int,
+f2070 int,
+f2071 int,
+f2072 int,
+f2073 int,
+f2074 int,
+f2075 int,
+f2076 int,
+f2077 int,
+f2078 int,
+f2079 int,
+f2080 int,
+f2081 int,
+f2082 int,
+f2083 int,
+f2084 int,
+f2085 int,
+f2086 int,
+f2087 int,
+f2088 int,
+f2089 int,
+f2090 int,
+f2091 int,
+f2092 int,
+f2093 int,
+f2094 int,
+f2095 int,
+f2096 int,
+f2097 int,
+f2098 int,
+f2099 int,
+f2100 int,
+f2101 int,
+f2102 int,
+f2103 int,
+f2104 int,
+f2105 int,
+f2106 int,
+f2107 int,
+f2108 int,
+f2109 int,
+f2110 int,
+f2111 int,
+f2112 int,
+f2113 int,
+f2114 int,
+f2115 int,
+f2116 int,
+f2117 int,
+f2118 int,
+f2119 int,
+f2120 int,
+f2121 int,
+f2122 int,
+f2123 int,
+f2124 int,
+f2125 int,
+f2126 int,
+f2127 int,
+f2128 int,
+f2129 int,
+f2130 int,
+f2131 int,
+f2132 int,
+f2133 int,
+f2134 int,
+f2135 int,
+f2136 int,
+f2137 int,
+f2138 int,
+f2139 int,
+f2140 int,
+f2141 int,
+f2142 int,
+f2143 int,
+f2144 int,
+f2145 int,
+f2146 int,
+f2147 int,
+f2148 int,
+f2149 int,
+f2150 int,
+f2151 int,
+f2152 int,
+f2153 int,
+f2154 int,
+f2155 int,
+f2156 int,
+f2157 int,
+f2158 int,
+f2159 int,
+f2160 int,
+f2161 int,
+f2162 int,
+f2163 int,
+f2164 int,
+f2165 int,
+f2166 int,
+f2167 int,
+f2168 int,
+f2169 int,
+f2170 int,
+f2171 int,
+f2172 int,
+f2173 int,
+f2174 int,
+f2175 int,
+f2176 int,
+f2177 int,
+f2178 int,
+f2179 int,
+f2180 int,
+f2181 int,
+f2182 int,
+f2183 int,
+f2184 int,
+f2185 int,
+f2186 int,
+f2187 int,
+f2188 int,
+f2189 int,
+f2190 int,
+f2191 int,
+f2192 int,
+f2193 int,
+f2194 int,
+f2195 int,
+f2196 int,
+f2197 int,
+f2198 int,
+f2199 int,
+f2200 int,
+f2201 int,
+f2202 int,
+f2203 int,
+f2204 int,
+f2205 int,
+f2206 int,
+f2207 int,
+f2208 int,
+f2209 int,
+f2210 int,
+f2211 int,
+f2212 int,
+f2213 int,
+f2214 int,
+f2215 int,
+f2216 int,
+f2217 int,
+f2218 int,
+f2219 int,
+f2220 int,
+f2221 int,
+f2222 int,
+f2223 int,
+f2224 int,
+f2225 int,
+f2226 int,
+f2227 int,
+f2228 int,
+f2229 int,
+f2230 int,
+f2231 int,
+f2232 int,
+f2233 int,
+f2234 int,
+f2235 int,
+f2236 int,
+f2237 int,
+f2238 int,
+f2239 int,
+f2240 int,
+f2241 int,
+f2242 int,
+f2243 int,
+f2244 int,
+f2245 int,
+f2246 int,
+f2247 int,
+f2248 int,
+f2249 int,
+f2250 int,
+f2251 int,
+f2252 int,
+f2253 int,
+f2254 int,
+f2255 int,
+f2256 int,
+f2257 int,
+f2258 int,
+f2259 int,
+f2260 int,
+f2261 int,
+f2262 int,
+f2263 int,
+f2264 int,
+f2265 int,
+f2266 int,
+f2267 int,
+f2268 int,
+f2269 int,
+f2270 int,
+f2271 int,
+f2272 int,
+f2273 int,
+f2274 int,
+f2275 int,
+f2276 int,
+f2277 int,
+f2278 int,
+f2279 int,
+f2280 int,
+f2281 int,
+f2282 int,
+f2283 int,
+f2284 int,
+f2285 int,
+f2286 int,
+f2287 int,
+f2288 int,
+f2289 int,
+f2290 int,
+f2291 int,
+f2292 int,
+f2293 int,
+f2294 int,
+f2295 int,
+f2296 int,
+f2297 int,
+f2298 int,
+f2299 int,
+f2300 int,
+f2301 int,
+f2302 int,
+f2303 int,
+f2304 int,
+f2305 int,
+f2306 int,
+f2307 int,
+f2308 int,
+f2309 int,
+f2310 int,
+f2311 int,
+f2312 int,
+f2313 int,
+f2314 int,
+f2315 int,
+f2316 int,
+f2317 int,
+f2318 int,
+f2319 int,
+f2320 int,
+f2321 int,
+f2322 int,
+f2323 int,
+f2324 int,
+f2325 int,
+f2326 int,
+f2327 int,
+f2328 int,
+f2329 int,
+f2330 int,
+f2331 int,
+f2332 int,
+f2333 int,
+f2334 int,
+f2335 int,
+f2336 int,
+f2337 int,
+f2338 int,
+f2339 int,
+f2340 int,
+f2341 int,
+f2342 int,
+f2343 int,
+f2344 int,
+f2345 int,
+f2346 int,
+f2347 int,
+f2348 int,
+f2349 int,
+f2350 int,
+f2351 int,
+f2352 int,
+f2353 int,
+f2354 int,
+f2355 int,
+f2356 int,
+f2357 int,
+f2358 int,
+f2359 int,
+f2360 int,
+f2361 int,
+f2362 int,
+f2363 int,
+f2364 int,
+f2365 int,
+f2366 int,
+f2367 int,
+f2368 int,
+f2369 int,
+f2370 int,
+f2371 int,
+f2372 int,
+f2373 int,
+f2374 int,
+f2375 int,
+f2376 int,
+f2377 int,
+f2378 int,
+f2379 int,
+f2380 int,
+f2381 int,
+f2382 int,
+f2383 int,
+f2384 int,
+f2385 int,
+f2386 int,
+f2387 int,
+f2388 int,
+f2389 int,
+f2390 int,
+f2391 int,
+f2392 int,
+f2393 int,
+f2394 int,
+f2395 int,
+f2396 int,
+f2397 int,
+f2398 int,
+f2399 int,
+f2400 int,
+f2401 int,
+f2402 int,
+f2403 int,
+f2404 int,
+f2405 int,
+f2406 int,
+f2407 int,
+f2408 int,
+f2409 int,
+f2410 int,
+f2411 int,
+f2412 int,
+f2413 int,
+f2414 int,
+f2415 int,
+f2416 int,
+f2417 int,
+f2418 int,
+f2419 int,
+f2420 int,
+f2421 int,
+f2422 int,
+f2423 int,
+f2424 int,
+f2425 int,
+f2426 int,
+f2427 int,
+f2428 int,
+f2429 int,
+f2430 int,
+f2431 int,
+f2432 int,
+f2433 int,
+f2434 int,
+f2435 int,
+f2436 int,
+f2437 int,
+f2438 int,
+f2439 int,
+f2440 int,
+f2441 int,
+f2442 int,
+f2443 int,
+f2444 int,
+f2445 int,
+f2446 int,
+f2447 int,
+f2448 int,
+f2449 int,
+f2450 int,
+f2451 int,
+f2452 int,
+f2453 int,
+f2454 int,
+f2455 int,
+f2456 int,
+f2457 int,
+f2458 int,
+f2459 int,
+f2460 int,
+f2461 int,
+f2462 int,
+f2463 int,
+f2464 int,
+f2465 int,
+f2466 int,
+f2467 int,
+f2468 int,
+f2469 int,
+f2470 int,
+f2471 int,
+f2472 int,
+f2473 int,
+f2474 int,
+f2475 int,
+f2476 int,
+f2477 int,
+f2478 int,
+f2479 int,
+f2480 int,
+f2481 int,
+f2482 int,
+f2483 int,
+f2484 int,
+f2485 int,
+f2486 int,
+f2487 int,
+f2488 int,
+f2489 int,
+f2490 int,
+f2491 int,
+f2492 int,
+f2493 int,
+f2494 int,
+f2495 int,
+f2496 int,
+f2497 int,
+f2498 int,
+f2499 int,
+f2500 int,
+f2501 int,
+f2502 int,
+f2503 int,
+f2504 int,
+f2505 int,
+f2506 int,
+f2507 int,
+f2508 int,
+f2509 int,
+f2510 int,
+f2511 int,
+f2512 int,
+f2513 int,
+f2514 int,
+f2515 int,
+f2516 int,
+f2517 int,
+f2518 int,
+f2519 int,
+f2520 int,
+f2521 int,
+f2522 int,
+f2523 int,
+f2524 int,
+f2525 int,
+f2526 int,
+f2527 int,
+f2528 int,
+f2529 int,
+f2530 int,
+f2531 int,
+f2532 int,
+f2533 int,
+f2534 int,
+f2535 int,
+f2536 int,
+f2537 int,
+f2538 int,
+f2539 int,
+f2540 int,
+f2541 int,
+f2542 int,
+f2543 int,
+f2544 int,
+f2545 int,
+f2546 int,
+f2547 int,
+f2548 int,
+f2549 int,
+f2550 int,
+f2551 int,
+f2552 int,
+f2553 int,
+f2554 int,
+f2555 int,
+f2556 int,
+f2557 int,
+f2558 int,
+f2559 int,
+f2560 int,
+f2561 int,
+f2562 int,
+f2563 int,
+f2564 int,
+f2565 int,
+f2566 int,
+f2567 int,
+f2568 int,
+f2569 int,
+f2570 int,
+f2571 int,
+f2572 int,
+f2573 int,
+f2574 int,
+f2575 int,
+f2576 int,
+f2577 int,
+f2578 int,
+f2579 int,
+f2580 int,
+f2581 int,
+f2582 int,
+f2583 int,
+f2584 int,
+f2585 int,
+f2586 int,
+f2587 int,
+f2588 int,
+f2589 int,
+f2590 int,
+f2591 int,
+f2592 int,
+f2593 int,
+f2594 int,
+f2595 int,
+f2596 int,
+f2597 int,
+f2598 int,
+f2599 int,
+f2600 int,
+f2601 int,
+f2602 int,
+f2603 int,
+f2604 int,
+f2605 int,
+f2606 int,
+f2607 int,
+f2608 int,
+f2609 int,
+f2610 int,
+f2611 int,
+f2612 int,
+f2613 int,
+f2614 int,
+f2615 int,
+f2616 int,
+f2617 int,
+f2618 int,
+f2619 int,
+f2620 int,
+f2621 int,
+f2622 int,
+f2623 int,
+f2624 int,
+f2625 int,
+f2626 int,
+f2627 int,
+f2628 int,
+f2629 int,
+f2630 int,
+f2631 int,
+f2632 int,
+f2633 int,
+f2634 int,
+f2635 int,
+f2636 int,
+f2637 int,
+f2638 int,
+f2639 int,
+f2640 int,
+f2641 int,
+f2642 int,
+f2643 int,
+f2644 int,
+f2645 int,
+f2646 int,
+f2647 int,
+f2648 int,
+f2649 int,
+f2650 int,
+f2651 int,
+f2652 int,
+f2653 int,
+f2654 int,
+f2655 int,
+f2656 int,
+f2657 int,
+f2658 int,
+f2659 int,
+f2660 int,
+f2661 int,
+f2662 int,
+f2663 int,
+f2664 int,
+f2665 int,
+f2666 int,
+f2667 int,
+f2668 int,
+f2669 int,
+f2670 int,
+f2671 int,
+f2672 int,
+f2673 int,
+f2674 int,
+f2675 int,
+f2676 int,
+f2677 int,
+f2678 int,
+f2679 int,
+f2680 int,
+f2681 int,
+f2682 int,
+f2683 int,
+f2684 int,
+f2685 int,
+f2686 int,
+f2687 int,
+f2688 int,
+f2689 int,
+f2690 int,
+f2691 int,
+f2692 int,
+f2693 int,
+f2694 int,
+f2695 int,
+f2696 int,
+f2697 int,
+f2698 int,
+f2699 int,
+f2700 int,
+f2701 int,
+f2702 int,
+f2703 int,
+f2704 int,
+f2705 int,
+f2706 int,
+f2707 int,
+f2708 int,
+f2709 int,
+f2710 int,
+f2711 int,
+f2712 int,
+f2713 int,
+f2714 int,
+f2715 int,
+f2716 int,
+f2717 int,
+f2718 int,
+f2719 int,
+f2720 int,
+f2721 int,
+f2722 int,
+f2723 int,
+f2724 int,
+f2725 int,
+f2726 int,
+f2727 int,
+f2728 int,
+f2729 int,
+f2730 int,
+f2731 int,
+f2732 int,
+f2733 int,
+f2734 int,
+f2735 int,
+f2736 int,
+f2737 int,
+f2738 int,
+f2739 int,
+f2740 int,
+f2741 int,
+f2742 int,
+f2743 int,
+f2744 int,
+f2745 int,
+f2746 int,
+f2747 int,
+f2748 int,
+f2749 int,
+f2750 int,
+f2751 int,
+f2752 int,
+f2753 int,
+f2754 int,
+f2755 int,
+f2756 int,
+f2757 int,
+f2758 int,
+f2759 int,
+f2760 int,
+f2761 int,
+f2762 int,
+f2763 int,
+f2764 int,
+f2765 int,
+f2766 int,
+f2767 int,
+f2768 int,
+f2769 int,
+f2770 int,
+f2771 int,
+f2772 int,
+f2773 int,
+f2774 int,
+f2775 int,
+f2776 int,
+f2777 int,
+f2778 int,
+f2779 int,
+f2780 int,
+f2781 int,
+f2782 int,
+f2783 int,
+f2784 int,
+f2785 int,
+f2786 int,
+f2787 int,
+f2788 int,
+f2789 int,
+f2790 int,
+f2791 int,
+f2792 int,
+f2793 int,
+f2794 int,
+f2795 int,
+f2796 int,
+f2797 int,
+f2798 int,
+f2799 int,
+f2800 int,
+f2801 int,
+f2802 int,
+f2803 int,
+f2804 int,
+f2805 int,
+f2806 int,
+f2807 int,
+f2808 int,
+f2809 int,
+f2810 int,
+f2811 int,
+f2812 int,
+f2813 int,
+f2814 int,
+f2815 int,
+f2816 int,
+f2817 int,
+f2818 int,
+f2819 int,
+f2820 int,
+f2821 int,
+f2822 int,
+f2823 int,
+f2824 int,
+f2825 int,
+f2826 int,
+f2827 int,
+f2828 int,
+f2829 int,
+f2830 int,
+f2831 int,
+f2832 int,
+f2833 int,
+f2834 int,
+f2835 int,
+f2836 int,
+f2837 int,
+f2838 int,
+f2839 int,
+f2840 int,
+f2841 int,
+f2842 int,
+f2843 int,
+f2844 int,
+f2845 int,
+f2846 int,
+f2847 int,
+f2848 int,
+f2849 int,
+f2850 int,
+f2851 int,
+f2852 int,
+f2853 int,
+f2854 int,
+f2855 int,
+f2856 int,
+f2857 int,
+f2858 int,
+f2859 int,
+f2860 int,
+f2861 int,
+f2862 int,
+f2863 int,
+f2864 int,
+f2865 int,
+f2866 int,
+f2867 int,
+f2868 int,
+f2869 int,
+f2870 int,
+f2871 int,
+f2872 int,
+f2873 int,
+f2874 int,
+f2875 int,
+f2876 int,
+f2877 int,
+f2878 int,
+f2879 int,
+f2880 int,
+f2881 int,
+f2882 int,
+f2883 int,
+f2884 int,
+f2885 int,
+f2886 int,
+f2887 int,
+f2888 int,
+f2889 int,
+f2890 int,
+f2891 int,
+f2892 int,
+f2893 int,
+f2894 int,
+f2895 int,
+f2896 int,
+f2897 int,
+f2898 int,
+f2899 int,
+f2900 int,
+f2901 int,
+f2902 int,
+f2903 int,
+f2904 int,
+f2905 int,
+f2906 int,
+f2907 int,
+f2908 int,
+f2909 int,
+f2910 int,
+f2911 int,
+f2912 int,
+f2913 int,
+f2914 int,
+f2915 int,
+f2916 int,
+f2917 int,
+f2918 int,
+f2919 int,
+f2920 int,
+f2921 int,
+f2922 int,
+f2923 int,
+f2924 int,
+f2925 int,
+f2926 int,
+f2927 int,
+f2928 int,
+f2929 int,
+f2930 int,
+f2931 int,
+f2932 int,
+f2933 int,
+f2934 int,
+f2935 int,
+f2936 int,
+f2937 int,
+f2938 int,
+f2939 int,
+f2940 int,
+f2941 int,
+f2942 int,
+f2943 int,
+f2944 int,
+f2945 int,
+f2946 int,
+f2947 int,
+f2948 int,
+f2949 int,
+f2950 int,
+f2951 int,
+f2952 int,
+f2953 int,
+f2954 int,
+f2955 int,
+f2956 int,
+f2957 int,
+f2958 int,
+f2959 int,
+f2960 int,
+f2961 int,
+f2962 int,
+f2963 int,
+f2964 int,
+f2965 int,
+f2966 int,
+f2967 int,
+f2968 int,
+f2969 int,
+f2970 int,
+f2971 int,
+f2972 int,
+f2973 int,
+f2974 int,
+f2975 int,
+f2976 int,
+f2977 int,
+f2978 int,
+f2979 int,
+f2980 int,
+f2981 int,
+f2982 int,
+f2983 int,
+f2984 int,
+f2985 int,
+f2986 int,
+f2987 int,
+f2988 int,
+f2989 int,
+f2990 int,
+f2991 int,
+f2992 int,
+f2993 int,
+f2994 int,
+f2995 int,
+f2996 int,
+f2997 int,
+f2998 int,
+f2999 int,
+f3000 int,
+f3001 int,
+f3002 int,
+f3003 int,
+f3004 int,
+f3005 int,
+f3006 int,
+f3007 int,
+f3008 int,
+f3009 int,
+f3010 int,
+f3011 int,
+f3012 int,
+f3013 int,
+f3014 int,
+f3015 int,
+f3016 int,
+f3017 int,
+f3018 int,
+f3019 int,
+f3020 int,
+f3021 int,
+f3022 int,
+f3023 int,
+f3024 int,
+f3025 int,
+f3026 int,
+f3027 int,
+f3028 int,
+f3029 int,
+f3030 int,
+f3031 int,
+f3032 int,
+f3033 int,
+f3034 int,
+f3035 int,
+f3036 int,
+f3037 int,
+f3038 int,
+f3039 int,
+f3040 int,
+f3041 int,
+f3042 int,
+f3043 int,
+f3044 int,
+f3045 int,
+f3046 int,
+f3047 int,
+f3048 int,
+f3049 int,
+f3050 int,
+f3051 int,
+f3052 int,
+f3053 int,
+f3054 int,
+f3055 int,
+f3056 int,
+f3057 int,
+f3058 int,
+f3059 int,
+f3060 int,
+f3061 int,
+f3062 int,
+f3063 int,
+f3064 int,
+f3065 int,
+f3066 int,
+f3067 int,
+f3068 int,
+f3069 int,
+f3070 int,
+f3071 int,
+f3072 int,
+f3073 int,
+f3074 int,
+f3075 int,
+f3076 int,
+f3077 int,
+f3078 int,
+f3079 int,
+f3080 int,
+f3081 int,
+f3082 int,
+f3083 int,
+f3084 int,
+f3085 int,
+f3086 int,
+f3087 int,
+f3088 int,
+f3089 int,
+f3090 int,
+f3091 int,
+f3092 int,
+f3093 int,
+f3094 int,
+f3095 int,
+f3096 int,
+f3097 int,
+f3098 int,
+f3099 int,
+f3100 int,
+f3101 int,
+f3102 int,
+f3103 int,
+f3104 int,
+f3105 int,
+f3106 int,
+f3107 int,
+f3108 int,
+f3109 int,
+f3110 int,
+f3111 int,
+f3112 int,
+f3113 int,
+f3114 int,
+f3115 int,
+f3116 int,
+f3117 int,
+f3118 int,
+f3119 int,
+f3120 int,
+f3121 int,
+f3122 int,
+f3123 int,
+f3124 int,
+f3125 int,
+f3126 int,
+f3127 int,
+f3128 int,
+f3129 int,
+f3130 int,
+f3131 int,
+f3132 int,
+f3133 int,
+f3134 int,
+f3135 int,
+f3136 int,
+f3137 int,
+f3138 int,
+f3139 int,
+f3140 int,
+f3141 int,
+f3142 int,
+f3143 int,
+f3144 int,
+f3145 int,
+f3146 int,
+f3147 int,
+f3148 int,
+f3149 int,
+f3150 int,
+f3151 int,
+f3152 int,
+f3153 int,
+f3154 int,
+f3155 int,
+f3156 int,
+f3157 int,
+f3158 int,
+f3159 int,
+f3160 int,
+f3161 int,
+f3162 int,
+f3163 int,
+f3164 int,
+f3165 int,
+f3166 int,
+f3167 int,
+f3168 int,
+f3169 int,
+f3170 int,
+f3171 int,
+f3172 int,
+f3173 int,
+f3174 int,
+f3175 int,
+f3176 int,
+f3177 int,
+f3178 int,
+f3179 int,
+f3180 int,
+f3181 int,
+f3182 int,
+f3183 int,
+f3184 int,
+f3185 int,
+f3186 int,
+f3187 int,
+f3188 int,
+f3189 int,
+f3190 int,
+f3191 int,
+f3192 int,
+f3193 int,
+f3194 int,
+f3195 int,
+f3196 int,
+f3197 int,
+f3198 int,
+f3199 int,
+f3200 int,
+f3201 int,
+f3202 int,
+f3203 int,
+f3204 int,
+f3205 int,
+f3206 int,
+f3207 int,
+f3208 int,
+f3209 int,
+f3210 int,
+f3211 int,
+f3212 int,
+f3213 int,
+f3214 int,
+f3215 int,
+f3216 int,
+f3217 int,
+f3218 int,
+f3219 int,
+f3220 int,
+f3221 int,
+f3222 int,
+f3223 int,
+f3224 int,
+f3225 int,
+f3226 int,
+f3227 int,
+f3228 int,
+f3229 int,
+f3230 int,
+f3231 int,
+f3232 int,
+f3233 int,
+f3234 int,
+f3235 int,
+f3236 int,
+f3237 int,
+f3238 int,
+f3239 int,
+f3240 int,
+f3241 int,
+f3242 int,
+f3243 int,
+f3244 int,
+f3245 int,
+f3246 int,
+f3247 int,
+f3248 int,
+f3249 int,
+f3250 int,
+f3251 int,
+f3252 int,
+f3253 int,
+f3254 int,
+f3255 int,
+f3256 int,
+f3257 int,
+f3258 int,
+f3259 int,
+f3260 int,
+f3261 int,
+f3262 int,
+f3263 int,
+f3264 int,
+f3265 int,
+f3266 int,
+f3267 int,
+f3268 int,
+f3269 int,
+f3270 int,
+f3271 int,
+f3272 int,
+f3273 int,
+f3274 int,
+f3275 int,
+f3276 int,
+f3277 int,
+f3278 int,
+f3279 int,
+f3280 int,
+f3281 int,
+f3282 int,
+f3283 int,
+f3284 int,
+f3285 int,
+f3286 int,
+f3287 int,
+f3288 int,
+f3289 int,
+f3290 int,
+f3291 int,
+f3292 int,
+f3293 int,
+f3294 int,
+f3295 int,
+f3296 int,
+f3297 int,
+f3298 int,
+f3299 int,
+f3300 int,
+f3301 int,
+f3302 int,
+f3303 int,
+f3304 int,
+f3305 int,
+f3306 int,
+f3307 int,
+f3308 int,
+f3309 int,
+f3310 int,
+f3311 int,
+f3312 int,
+f3313 int,
+f3314 int,
+f3315 int,
+f3316 int,
+f3317 int,
+f3318 int,
+f3319 int,
+f3320 int,
+f3321 int,
+f3322 int,
+f3323 int,
+f3324 int,
+f3325 int,
+f3326 int,
+f3327 int,
+f3328 int,
+f3329 int,
+f3330 int,
+f3331 int,
+f3332 int,
+f3333 int,
+f3334 int,
+f3335 int,
+f3336 int,
+f3337 int,
+f3338 int,
+f3339 int,
+f3340 int,
+f3341 int,
+f3342 int,
+f3343 int,
+f3344 int,
+f3345 int,
+f3346 int,
+f3347 int,
+f3348 int,
+f3349 int,
+f3350 int,
+f3351 int,
+f3352 int,
+f3353 int,
+f3354 int,
+f3355 int,
+f3356 int,
+f3357 int,
+f3358 int,
+f3359 int,
+f3360 int,
+f3361 int,
+f3362 int,
+f3363 int,
+f3364 int,
+f3365 int,
+f3366 int,
+f3367 int,
+f3368 int,
+f3369 int,
+f3370 int,
+f3371 int,
+f3372 int,
+f3373 int,
+f3374 int,
+f3375 int,
+f3376 int,
+f3377 int,
+f3378 int,
+f3379 int,
+f3380 int,
+f3381 int,
+f3382 int,
+f3383 int,
+f3384 int,
+f3385 int,
+f3386 int,
+f3387 int,
+f3388 int,
+f3389 int,
+f3390 int,
+f3391 int,
+f3392 int,
+f3393 int,
+f3394 int,
+f3395 int,
+f3396 int,
+f3397 int,
+f3398 int,
+f3399 int,
+f3400 int,
+f3401 int,
+f3402 int,
+f3403 int,
+f3404 int,
+f3405 int,
+f3406 int,
+f3407 int,
+f3408 int,
+f3409 int,
+f3410 int,
+f3411 int,
+f3412 int,
+f3413 int,
+f3414 int,
+f3415 int,
+f3416 int,
+f3417 int,
+f3418 int,
+f3419 int,
+f3420 int,
+f3421 int,
+f3422 int,
+f3423 int,
+f3424 int,
+f3425 int,
+f3426 int,
+f3427 int,
+f3428 int,
+f3429 int,
+f3430 int,
+f3431 int,
+f3432 int,
+f3433 int,
+f3434 int,
+f3435 int,
+f3436 int,
+f3437 int,
+f3438 int,
+f3439 int,
+f3440 int,
+f3441 int,
+f3442 int,
+f3443 int,
+f3444 int,
+f3445 int,
+f3446 int,
+f3447 int,
+f3448 int,
+f3449 int,
+f3450 int,
+f3451 int,
+f3452 int,
+f3453 int,
+f3454 int,
+f3455 int,
+f3456 int,
+f3457 int,
+f3458 int,
+f3459 int,
+f3460 int,
+f3461 int,
+f3462 int,
+f3463 int,
+f3464 int,
+f3465 int,
+f3466 int,
+f3467 int,
+f3468 int,
+f3469 int,
+f3470 int,
+f3471 int,
+f3472 int,
+f3473 int,
+f3474 int,
+f3475 int,
+f3476 int,
+f3477 int,
+f3478 int,
+f3479 int,
+f3480 int,
+f3481 int,
+f3482 int,
+f3483 int,
+f3484 int,
+f3485 int,
+f3486 int,
+f3487 int,
+f3488 int,
+f3489 int,
+f3490 int,
+f3491 int,
+f3492 int,
+f3493 int,
+f3494 int,
+f3495 int,
+f3496 int,
+f3497 int,
+f3498 int,
+f3499 int,
+f3500 int,
+f3501 int,
+f3502 int,
+f3503 int,
+f3504 int,
+f3505 int,
+f3506 int,
+f3507 int,
+f3508 int,
+f3509 int,
+f3510 int,
+f3511 int,
+f3512 int,
+f3513 int,
+f3514 int,
+f3515 int,
+f3516 int,
+f3517 int,
+f3518 int,
+f3519 int,
+f3520 int,
+f3521 int,
+f3522 int,
+f3523 int,
+f3524 int,
+f3525 int,
+f3526 int,
+f3527 int,
+f3528 int,
+f3529 int,
+f3530 int,
+f3531 int,
+f3532 int,
+f3533 int,
+f3534 int,
+f3535 int,
+f3536 int,
+f3537 int,
+f3538 int,
+f3539 int,
+f3540 int,
+f3541 int,
+f3542 int,
+f3543 int,
+f3544 int,
+f3545 int,
+f3546 int,
+f3547 int,
+f3548 int,
+f3549 int,
+f3550 int,
+f3551 int,
+f3552 int,
+f3553 int,
+f3554 int,
+f3555 int,
+f3556 int,
+f3557 int,
+f3558 int,
+f3559 int,
+f3560 int,
+f3561 int,
+f3562 int,
+f3563 int,
+f3564 int,
+f3565 int,
+f3566 int,
+f3567 int,
+f3568 int,
+f3569 int,
+f3570 int,
+f3571 int,
+f3572 int,
+f3573 int,
+f3574 int,
+f3575 int,
+f3576 int,
+f3577 int,
+f3578 int,
+f3579 int,
+f3580 int,
+f3581 int,
+f3582 int,
+f3583 int,
+f3584 int,
+f3585 int,
+f3586 int,
+f3587 int,
+f3588 int,
+f3589 int,
+f3590 int,
+f3591 int,
+f3592 int,
+f3593 int,
+f3594 int,
+f3595 int,
+f3596 int,
+f3597 int,
+f3598 int,
+f3599 int,
+f3600 int,
+f3601 int,
+f3602 int,
+f3603 int,
+f3604 int,
+f3605 int,
+f3606 int,
+f3607 int,
+f3608 int,
+f3609 int,
+f3610 int,
+f3611 int,
+f3612 int,
+f3613 int,
+f3614 int,
+f3615 int,
+f3616 int,
+f3617 int,
+f3618 int,
+f3619 int,
+f3620 int,
+f3621 int,
+f3622 int,
+f3623 int,
+f3624 int,
+f3625 int,
+f3626 int,
+f3627 int,
+f3628 int,
+f3629 int,
+f3630 int,
+f3631 int,
+f3632 int,
+f3633 int,
+f3634 int,
+f3635 int,
+f3636 int,
+f3637 int,
+f3638 int,
+f3639 int,
+f3640 int,
+f3641 int,
+f3642 int,
+f3643 int,
+f3644 int,
+f3645 int,
+f3646 int,
+f3647 int,
+f3648 int,
+f3649 int,
+f3650 int,
+f3651 int,
+f3652 int,
+f3653 int,
+f3654 int,
+f3655 int,
+f3656 int,
+f3657 int,
+f3658 int,
+f3659 int,
+f3660 int,
+f3661 int,
+f3662 int,
+f3663 int,
+f3664 int,
+f3665 int,
+f3666 int,
+f3667 int,
+f3668 int,
+f3669 int,
+f3670 int,
+f3671 int,
+f3672 int,
+f3673 int,
+f3674 int,
+f3675 int,
+f3676 int,
+f3677 int,
+f3678 int,
+f3679 int,
+f3680 int,
+f3681 int,
+f3682 int,
+f3683 int,
+f3684 int,
+f3685 int,
+f3686 int,
+f3687 int,
+f3688 int,
+f3689 int,
+f3690 int,
+f3691 int,
+f3692 int,
+f3693 int,
+f3694 int,
+f3695 int,
+f3696 int,
+f3697 int,
+f3698 int,
+f3699 int,
+f3700 int,
+f3701 int,
+f3702 int,
+f3703 int,
+f3704 int,
+f3705 int,
+f3706 int,
+f3707 int,
+f3708 int,
+f3709 int,
+f3710 int,
+f3711 int,
+f3712 int,
+f3713 int,
+f3714 int,
+f3715 int,
+f3716 int,
+f3717 int,
+f3718 int,
+f3719 int,
+f3720 int,
+f3721 int,
+f3722 int,
+f3723 int,
+f3724 int,
+f3725 int,
+f3726 int,
+f3727 int,
+f3728 int,
+f3729 int,
+f3730 int,
+f3731 int,
+f3732 int,
+f3733 int,
+f3734 int,
+f3735 int,
+f3736 int,
+f3737 int,
+f3738 int,
+f3739 int,
+f3740 int,
+f3741 int,
+f3742 int,
+f3743 int,
+f3744 int,
+f3745 int,
+f3746 int,
+f3747 int,
+f3748 int,
+f3749 int,
+f3750 int,
+f3751 int,
+f3752 int,
+f3753 int,
+f3754 int,
+f3755 int,
+f3756 int,
+f3757 int,
+f3758 int,
+f3759 int,
+f3760 int,
+f3761 int,
+f3762 int,
+f3763 int,
+f3764 int,
+f3765 int,
+f3766 int,
+f3767 int,
+f3768 int,
+f3769 int,
+f3770 int,
+f3771 int,
+f3772 int,
+f3773 int,
+f3774 int,
+f3775 int,
+f3776 int,
+f3777 int,
+f3778 int,
+f3779 int,
+f3780 int,
+f3781 int,
+f3782 int,
+f3783 int,
+f3784 int,
+f3785 int,
+f3786 int,
+f3787 int,
+f3788 int,
+f3789 int,
+f3790 int,
+f3791 int,
+f3792 int,
+f3793 int,
+f3794 int,
+f3795 int,
+f3796 int,
+f3797 int,
+f3798 int,
+f3799 int,
+f3800 int,
+f3801 int,
+f3802 int,
+f3803 int,
+f3804 int,
+f3805 int,
+f3806 int,
+f3807 int,
+f3808 int,
+f3809 int,
+f3810 int,
+f3811 int,
+f3812 int,
+f3813 int,
+f3814 int,
+f3815 int,
+f3816 int,
+f3817 int,
+f3818 int,
+f3819 int,
+f3820 int,
+f3821 int,
+f3822 int,
+f3823 int,
+f3824 int,
+f3825 int,
+f3826 int,
+f3827 int,
+f3828 int,
+f3829 int,
+f3830 int,
+f3831 int,
+f3832 int,
+f3833 int,
+f3834 int,
+f3835 int,
+f3836 int,
+f3837 int,
+f3838 int,
+f3839 int,
+f3840 int,
+f3841 int,
+f3842 int,
+f3843 int,
+f3844 int,
+f3845 int,
+f3846 int,
+f3847 int,
+f3848 int,
+f3849 int,
+f3850 int,
+f3851 int,
+f3852 int,
+f3853 int,
+f3854 int,
+f3855 int,
+f3856 int,
+f3857 int,
+f3858 int,
+f3859 int,
+f3860 int,
+f3861 int,
+f3862 int,
+f3863 int,
+f3864 int,
+f3865 int,
+f3866 int,
+f3867 int,
+f3868 int,
+f3869 int,
+f3870 int,
+f3871 int,
+f3872 int,
+f3873 int,
+f3874 int,
+f3875 int,
+f3876 int,
+f3877 int,
+f3878 int,
+f3879 int,
+f3880 int,
+f3881 int,
+f3882 int,
+f3883 int,
+f3884 int,
+f3885 int,
+f3886 int,
+f3887 int,
+f3888 int,
+f3889 int,
+f3890 int,
+f3891 int,
+f3892 int,
+f3893 int,
+f3894 int,
+f3895 int,
+f3896 int,
+f3897 int,
+f3898 int,
+f3899 int,
+f3900 int,
+f3901 int,
+f3902 int,
+f3903 int,
+f3904 int,
+f3905 int,
+f3906 int,
+f3907 int,
+f3908 int,
+f3909 int,
+f3910 int,
+f3911 int,
+f3912 int,
+f3913 int,
+f3914 int,
+f3915 int,
+f3916 int,
+f3917 int,
+f3918 int,
+f3919 int,
+f3920 int,
+f3921 int,
+f3922 int,
+f3923 int,
+f3924 int,
+f3925 int,
+f3926 int,
+f3927 int,
+f3928 int,
+f3929 int,
+f3930 int,
+f3931 int,
+f3932 int,
+f3933 int,
+f3934 int,
+f3935 int,
+f3936 int,
+f3937 int,
+f3938 int,
+f3939 int,
+f3940 int,
+f3941 int,
+f3942 int,
+f3943 int,
+f3944 int,
+f3945 int,
+f3946 int,
+f3947 int,
+f3948 int,
+f3949 int,
+f3950 int,
+f3951 int,
+f3952 int,
+f3953 int,
+f3954 int,
+f3955 int,
+f3956 int,
+f3957 int,
+f3958 int,
+f3959 int,
+f3960 int,
+f3961 int,
+f3962 int,
+f3963 int,
+f3964 int,
+f3965 int,
+f3966 int,
+f3967 int,
+f3968 int,
+f3969 int,
+f3970 int,
+f3971 int,
+f3972 int,
+f3973 int,
+f3974 int,
+f3975 int,
+f3976 int,
+f3977 int,
+f3978 int,
+f3979 int,
+f3980 int,
+f3981 int,
+f3982 int,
+f3983 int,
+f3984 int,
+f3985 int,
+f3986 int,
+f3987 int,
+f3988 int,
+f3989 int,
+f3990 int,
+f3991 int,
+f3992 int,
+f3993 int,
+f3994 int,
+f3995 int,
+f3996 int,
+f3997 int,
+f3998 int,
+f3999 int,
+f4000 int,
+f4001 int,
+f4002 int,
+f4003 int,
+f4004 int,
+f4005 int,
+f4006 int,
+f4007 int,
+f4008 int,
+f4009 int,
+f4010 int,
+f4011 int,
+f4012 int,
+f4013 int,
+f4014 int,
+f4015 int,
+f4016 int,
+f4017 int,
+f4018 int,
+f4019 int,
+f4020 int,
+f4021 int,
+f4022 int,
+f4023 int,
+f4024 int,
+f4025 int,
+f4026 int,
+f4027 int,
+f4028 int,
+f4029 int,
+f4030 int,
+f4031 int,
+f4032 int,
+f4033 int,
+f4034 int,
+f4035 int,
+f4036 int,
+f4037 int,
+f4038 int,
+f4039 int,
+f4040 int,
+f4041 int,
+f4042 int,
+f4043 int,
+f4044 int,
+f4045 int,
+f4046 int,
+f4047 int,
+f4048 int,
+f4049 int,
+f4050 int,
+f4051 int,
+f4052 int,
+f4053 int,
+f4054 int,
+f4055 int,
+f4056 int,
+f4057 int,
+f4058 int,
+f4059 int,
+f4060 int,
+f4061 int,
+f4062 int,
+f4063 int,
+f4064 int,
+f4065 int,
+f4066 int,
+f4067 int,
+f4068 int,
+f4069 int,
+f4070 int,
+f4071 int,
+f4072 int,
+f4073 int,
+f4074 int,
+f4075 int,
+f4076 int,
+f4077 int,
+f4078 int,
+f4079 int,
+f4080 int,
+f4081 int,
+f4082 int,
+f4083 int,
+f4084 int,
+f4085 int,
+f4086 int,
+f4087 int,
+f4088 int,
+f4089 int,
+f4090 int,
+f4091 int,
+f4092 int,
+f4093 int,
+f4094 int,
+f4095 int,
+f4096 int
+) ENGINE=BLACKHOLE;

=== modified file 'tests/t/default_collation.test'
--- tests/t/default_collation.test	2009-07-09 01:35:10 +0000
+++ tests/t/default_collation.test	2009-12-07 04:59:40 +0000
@@ -9,6 +9,10 @@
 
 create database mysqltest collate = utf8_bin;
 show create database mysqltest;
+alter database mysqltest collate = utf8_general_ci;
+show create database mysqltest;
+alter database mysqltest collate = utf8_bin;
+show create database mysqltest;
 
 use mysqltest;
 

